#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
/*-------------------------------------------------------------------------*/
void Residue_bond::create(Maps* maps, Residue* res1, Residue* res2)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm, jatm;
  int ibranch1, ibranch2;
  int iibranch1, iibranch2;

  int valid;
  int ibond, ibend, itors, iimpr, ionfo;
  int inter_on_3bond_conn = maps->inter_on_3bond_conn;
  int inter_on_4bond_conn = maps->inter_on_4bond_conn;

  int size, iatm_next, iatm_prev, bonding;
  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* I) Fill in variables                                                    */

  strcpy(residue1_name,res1->residue_name);     
  strcpy(residue2_name,res2->residue_name);
  residue1_index = res1->residue_index;                
  residue2_index = res2->residue_index;                

#define DEBUG
#undef DEBUG
#ifdef DEBUG
  printf("Residue_bond::create\n"); 
  printf("residue1: %s\n",residue1_name);
  printf("residue2: %s\n",residue2_name);
#endif
#undef DEBUG

  nbond = 1;
  bond_atom[nbond][1] = res1->branch_root_s[0][0];
  bond_atom[nbond][2] = res2->branch_root_f[0][0];

  nbend = 0;
  for (ibranch1=1;ibranch1<=res1->nbranch_s1;ibranch1++) {
      nbend++;
      if (nbend>NMEM_MIN) {
	sprintf(err_msg,"Allocate more bend in Residue_bond:setup %s");
	errorMessage(err_msg);
      }
      bend_atom[nbend][1] =  res2->branch_root_f[0][0];
      bend_atom[nbend][2] =  res1->branch_root_s[0][0];
      bend_atom[nbend][3] =  res1->branch_root_s[ibranch1][0];
  }/*end for*/
  for (ibranch2=1;ibranch2<=res2->nbranch_f1;ibranch2++) {
      nbend++;
      if (nbend>NMEM_MIN) {
	sprintf(err_msg,"Allocate more bend in Residue_bond:setup %s");
	errorMessage(err_msg);
      }
      bend_atom[nbend][1] =  res1->branch_root_s[0][0];
      bend_atom[nbend][2] =  res2->branch_root_f[0][0];
      bend_atom[nbend][3] =  res2->branch_root_f[ibranch2][0];
  }/*end for*/

  ntors = 0;
  for (ibranch1=1;ibranch1<=res1->nbranch_s1;ibranch1++) {
      for (ibranch2=1;ibranch2<=res2->nbranch_f1;ibranch2++) {
          ntors++;
          if (ntors>NMEM_MIN) {
	     sprintf(err_msg,"Allocate more tors in Residue_bond:setup %s");
	     errorMessage(err_msg);
          }
          tors_atom[ntors][1] =  res1->branch_root_s[ibranch1][0];
          tors_atom[ntors][2] =  res1->branch_root_s[0][0];
          tors_atom[ntors][3] =  res2->branch_root_f[0][0];
          tors_atom[ntors][4] =  res2->branch_root_f[ibranch2][0];   
      }/*end for*/
  }/*end for*/
  for (ibranch1=1;ibranch1<=res1->nbranch_s1;ibranch1++) {
      for (iibranch1=1;iibranch1<=res1->nbranch_s2[ibranch1];iibranch1++) {
          ntors++;
          if (ntors>NMEM_MIN) {
	     sprintf(err_msg,"Allocate more tors in Residue_bond:setup %s");
	     errorMessage(err_msg);
          }
          tors_atom[ntors][1] = res2->branch_root_f[0][0];
          tors_atom[ntors][2] = res1->branch_root_s[0][0]; 
          tors_atom[ntors][3] = res1->branch_root_s[ibranch1][0];
          tors_atom[ntors][4] = res1->branch_root_s[ibranch1][iibranch1];
      }/*end for*/
  }/*end for*/
  for (ibranch2=1;ibranch2<=res2->nbranch_f1;ibranch2++) {
      for (iibranch2=1;iibranch2<=res2->nbranch_f2[ibranch2];iibranch2++) {
          ntors++;
          if (ntors>NMEM_MIN) {
	     sprintf(err_msg,"Allocate more tors in Residue_bond:setup %s");
	     errorMessage(err_msg);
          }
          tors_atom[ntors][1] = res1->branch_root_s[0][0]; 
          tors_atom[ntors][2] = res2->branch_root_f[0][0];
          tors_atom[ntors][3] = res2->branch_root_f[ibranch2][0];
          tors_atom[ntors][4] = res2->branch_root_f[ibranch2][iibranch2];
      }/*end for*/
  }/*end for*/

  nonfo = 0;
  for (itors=1;itors<=ntors;itors++) {
      nonfo++;
      if (nonfo>NMEM_MIN) {
	 sprintf(err_msg,"Allocate more onfo in Residue_bond:setup %s");
	 errorMessage(err_msg);
      }
      onfo_atom[nonfo][1] =  tors_atom[itors][1];
      onfo_atom[nonfo][2] =  tors_atom[itors][4];
  }/*end for*/

  nexclude = 0;
  for (iatm=1;iatm<=res1->natom;iatm++) {
      for (jatm=1;jatm<=res2->natom;jatm++) {
	   valid = 1;
           for (ibond=1; ibond<=nbond; ibond++) {
	        if ((res1->atom[iatm]==bond_atom[ibond][1] ||
	             res1->atom[iatm]==bond_atom[ibond][2] ) &&
	            (res2->atom[jatm]==bond_atom[ibond][1] ||
	             res2->atom[jatm]==bond_atom[ibond][2] )  ) { valid = 0;}
           }/*end for*/
           if ((valid==1)&&(inter_on_3bond_conn==0)) {
              for (ibend=1; ibend<=nbend; ibend++) {
	        if ((res1->atom[iatm]==bend_atom[ibend][1] ||
	             res1->atom[iatm]==bend_atom[ibend][2] ||
	             res1->atom[iatm]==bend_atom[ibend][3] ) &&
	            (res2->atom[jatm]==bend_atom[ibend][1] ||
	             res2->atom[jatm]==bend_atom[ibend][2] ||
	             res2->atom[jatm]==bend_atom[ibend][3] )  ) { valid = 0;}
              }/*end for*/
           }/*end if*/
           if ((valid==1)&&(inter_on_3bond_conn==0)&&(inter_on_4bond_conn==0)) {
              for (ionfo=1; ionfo<=nonfo; ionfo++) {
	        if ((res1->atom[iatm]==onfo_atom[ionfo][1] ||
	             res1->atom[iatm]==onfo_atom[ionfo][2] ) &&
                    (res2->atom[jatm]==onfo_atom[ionfo][1] ||
	             res2->atom[jatm]==onfo_atom[ionfo][2] )  ) { valid = 0;}
              }/*end for*/
              for (itors=1; itors<=ntors; itors++) {
	        if ((res1->atom[iatm]==tors_atom[itors][1] ||
	             res1->atom[iatm]==tors_atom[itors][2] ||
	             res1->atom[iatm]==tors_atom[itors][3] ||
	             res1->atom[iatm]==tors_atom[itors][4] ) &&
	            (res2->atom[jatm]==tors_atom[itors][1] ||
	             res2->atom[jatm]==tors_atom[itors][2] ||
	             res2->atom[jatm]==tors_atom[itors][3] ||
	             res2->atom[jatm]==tors_atom[itors][4] )  ) { valid = 0;}
              }/*end for*/
           }/*end if*/
           if (valid==0) {
              nexclude++;
              if (nexclude>NMEM_MIN) {
  	          sprintf(err_msg,"Allocate more exclude in Residue_bond:setup %s");
	          errorMessage(err_msg);
              }
              exclude_atom[nexclude][1] = res1->atom[iatm];
              exclude_atom[nexclude][2] = res2->atom[jatm];
           }/*end if*/
      }/*end for*/
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* II) Define improper torsions                                            */
/*     convention used 0 : root atom in other residue                      */
/*     res1_tail(bimpr_atom_s) (=) res2_head(bimpr_atom_f)                 */
/*     planed for nucleic acids and proteins                               */
/*     cannot be applied for PRES terminal groups                          */

  size = 4;
  nimpr = 0;
  if (res1->nbimpr==2) {/* res1 has 2nd bimpr */
     nimpr++;
     if (nimpr>NMEM_MIN) {
	sprintf(err_msg,"Allocate more impr in Residue_bond:setup %s");
	errorMessage(err_msg);
     }
     for (iatm=1;iatm<=size;iatm++) {
         if (res1->bimpr_atom_s[iatm]==0) {/* assume 1st root in res2 */
             impr_atom[nimpr][iatm] = res2->branch_root_f[0][0];
         }
         else {
             impr_atom[nimpr][iatm] = res1->bimpr_atom_s[iatm];  
         }/* end if */
     }/* end for */
  }/* end if */

  if (res2->nbimpr!=0) {/* res2 has 1st bimpr */
     nimpr++;
     if (nimpr>NMEM_MIN) {
	sprintf(err_msg,"Allocate more impr in Residue_bond:setup %s");
	errorMessage(err_msg);
     }/* end if */
     for (iatm=1;iatm<=size;iatm++) {
         if (res2->bimpr_atom_f[iatm]==0) {/* assume 1st root in res2 */
             impr_atom[nimpr][iatm] = res1->branch_root_s[0][0];
         }
         else {
             impr_atom[nimpr][iatm] = res2->bimpr_atom_f[iatm];  
         }/* end if */
     }/* end for */
  }/* end if */

/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/
