#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Design::allocate()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  resAf.bond          =  new int[NMEM_MIN] - 1;
  resAf.bend          =  new int[NMEM_MIN] - 1;
  resAf.tors          =  new int[NMEM_MIN] - 1;
  resAf.impr          =  new int[NMEM_MIN] - 1;
  resAf.onfo          =  new int[NMEM_MIN] - 1;
  resAf.exclude       =  new int[NMEM_MIN] - 1;
  resAf.bond_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resAf.bend_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 3);
  resAf.tors_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resAf.impr_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resAf.onfo_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resAf.exclude_atom  =  cmall_int_mat(1, NMEM_MIN, 1, 2);

  resAs.bond          =  new int[NMEM_MIN] - 1;
  resAs.bend          =  new int[NMEM_MIN] - 1;
  resAs.tors          =  new int[NMEM_MIN] - 1;
  resAs.impr          =  new int[NMEM_MIN] - 1;
  resAs.onfo          =  new int[NMEM_MIN] - 1;
  resAs.exclude       =  new int[NMEM_MIN] - 1;
  resAs.bond_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resAs.bend_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 3);
  resAs.tors_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resAs.impr_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resAs.onfo_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resAs.exclude_atom  =  cmall_int_mat(1, NMEM_MIN, 1, 2);

  resBf.bond          =  new int[NMEM_MIN] - 1;
  resBf.bend          =  new int[NMEM_MIN] - 1;
  resBf.tors          =  new int[NMEM_MIN] - 1;
  resBf.impr          =  new int[NMEM_MIN] - 1;
  resBf.onfo          =  new int[NMEM_MIN] - 1;
  resBf.exclude       =  new int[NMEM_MIN] - 1;
  resBf.bond_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resBf.bend_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 3);
  resBf.tors_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resBf.impr_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resBf.onfo_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resBf.exclude_atom  =  cmall_int_mat(1, NMEM_MIN, 1, 2);

  resBs.bond          =  new int[NMEM_MIN] - 1;
  resBs.bend          =  new int[NMEM_MIN] - 1;
  resBs.tors          =  new int[NMEM_MIN] - 1;
  resBs.impr          =  new int[NMEM_MIN] - 1;
  resBs.onfo          =  new int[NMEM_MIN] - 1;
  resBs.exclude       =  new int[NMEM_MIN] - 1;
  resBs.bond_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resBs.bend_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 3);
  resBs.tors_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resBs.impr_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 4);
  resBs.onfo_atom     =  cmall_int_mat(1, NMEM_MIN, 1, 2);
  resBs.exclude_atom  =  cmall_int_mat(1, NMEM_MIN, 1, 2);

/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/


/*=========================================================================*/
void Design::setup( Pos *pos, Maps *maps )
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int    nres                = pos->nres;
  int    nresbond            = pos->nresbond;
  Residue      *residue      = pos->residue;
  Residue_bond *residue_bond = pos->residue_bond;

/*-------------------------------------------------------------------------*/
/* 0) Assume that resA_ind and resB_ind is initialized                     */

/*-------------------------------------------------------------------------*/
/* I) Find the corresponding residue_bond and residue indices              */
/*         resbnd_resAf_ind : index of the residue bond, which involves    */
/*                            branch_root_f[][] of resA                    */ 
/*         resbnd_resAs_ind : index of the residue bond, which involves    */
/*                            branch_root_s[][] of resA                    */
/*         resbnd_resBf_ind : index of the residue bond, which involves    */
/*                            branch_root_f[][] of resB                    */ 
/*         resbnd_resBs_ind : index of the residue bond, which involves    */
/*                            branch_root_s[][] of resB                    */
/*         res_resAf_ind    : index of the residue, which bonds to         */
/*                            branch_root_f[][] of resA                    */
/*         res_resAs_ind    : index of the residue, which bonds to         */
/*                            branch_root_s[][] of resA                    */
/*         res_resBf_ind    : index of the residue, which bonds to         */
/*                            branch_root_f[][] of resB                    */
/*         res_resBs_ind    : index of the residue, which bonds to         */
/*                            branch_root_s[][] of resB                    */
/*         resAf, resAs     : proposal residue bonds for resA              */
/*         resBf, resBs     : proposal residue bonds for resB              */
/*         if non existent  resbnd_res**_ind = 0                           */
/*-------------------------------------------------------------------------*/ 
  
  int iresbond;
  for (iresbond=1;iresbond<=nresbond;iresbond++) {
       if (residue_bond[iresbond].residue2_index==resA_ind) {
	   resbnd_resAf_ind = iresbond;
           res_resAf_ind    = residue_bond[iresbond].residue1_index;
       }/*end if*/
       if (residue_bond[iresbond].residue1_index==resA_ind) {
	   resbnd_resAs_ind = iresbond;
           res_resAs_ind    = residue_bond[iresbond].residue2_index;
       }/*end if*/
       if (residue_bond[iresbond].residue2_index==resB_ind) {
	   resbnd_resBf_ind = iresbond;
           res_resBf_ind    = residue_bond[iresbond].residue1_index;
       }/*end if*/
       if (residue_bond[iresbond].residue1_index==resB_ind) {
	   resbnd_resBs_ind = iresbond;
           res_resBs_ind    = residue_bond[iresbond].residue2_index;
       }/*end if*/
  }/*end for*/
                  
/*-------------------------------------------------------------------------*/
/* II) Create new residue bonds                                            */
/*   Residue_bond resAf, resAs, resBf, resBs;                              */

/*-------------------------------------------------------------------------*/
/* II.a) resA is inserted btw res_resBf and res_resBs                      */

      ires1 = res_resBf_ind;
      ires2 = resA_ind;
      resAf.create(maps,&(residue[ires1]),&(residue[ires2]));

      ires1 = resA_ind;
      ires2 = res_resBs_ind;
      resAs.create(maps,&(residue[ires1]),&(residue[ires2]));

/*-------------------------------------------------------------------------*/
/* II.b) resB is inserted btw res_resAf and res_resAs                      */

      ires1 = res_resAf_ind;
      ires2 = resB_ind;
      resBf.create(maps,&(residue[ires1]),&(residue[ires2]));

      ires1 = resB_ind;
      ires2 = res_resAs_ind;
      resBs.create(maps,&(residue[ires1]),&(residue[ires2]));

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
