#include "../../include/cryo_em/Sim_Score_Matrix.h"

Sim_Score_Matrix::Sim_Score_Matrix()
{
	score_matrix = NULL;
}

Sim_Score_Matrix::~Sim_Score_Matrix()
{
}

float Sim_Score_Matrix::compare_with_em2d(EMData * proj, EMData * image_fft_amp)
{
	float sim_score;

	EMData *proj_fft=new EMData();
	EMData *proj_fft_amp = new EMData();
	

	proj_fft = proj->do_fft();
	proj_fft->ri2ap();
	proj_fft_amp=proj_fft->amplitude();
	sim_score = image_fft_amp->cmp("ccc", proj_fft_amp);
	
	delete proj_fft;
	delete proj_fft_amp;
	
	return sim_score;
}

void Sim_Score_Matrix::calc_sim_score_matrix(PointArray *new_pa, const char * em_image_file,  float old_az, float old_alt, float old_phi, float apix, float res)
{
	int pt;
	int image_xsize;
	
	PointArray *pa=new PointArray(); // a copy of point array of the PDB to compare
	pa=new_pa->copy();
	pa->center_to_zero();
	
    EMData *proj = new EMData(); // to store the 2D projection of the PDB
    EMData *new_mrc = new EMData(); // to store the 3D map blurred from PDB

    float current_az;
    float current_alt;
    float current_phi;
    
	EMData *image = new EMData(); // to store the 2D EM image
	EMData *image_fft =new EMData(); // to store the fft of the 2D EM image
	EMData *image_fft_amp = new EMData(); // to store the amplitude of the 2D EM fft


    
	image->read_image(em_image_file);
	image_xsize=image->get_xsize();
	image_fft=image->do_fft();
	image_fft->ri2ap();
	image_fft_amp=image_fft->amplitude();


    for (int i=0;i<=score_matrix_dim-1;i++) { 
    	for (int j=0;j<=score_matrix_dim-1;j++) {
    		for (int k=0;k<=score_matrix_dim-1;k++) {    			    			
    		    //fluctuate around the original euler angles and project along these angles

    		        current_az=old_az-euler_range+i*euler_interval;
    			current_alt=old_alt-euler_range+j*euler_interval;
    			current_phi=old_phi-euler_range+k*euler_interval;		
    			Dict d("type","eman");
    			d["az"]=current_az;
    			d["alt"]=current_alt;
    			d["phi"]=current_phi;
    			Transform t(d);   		    

    		    new_mrc=pa->pdb2mrc_by_summation(image_xsize, apix, res);	    
    		    proj=new_mrc->project("standard", t);


    	 	    pt=i*score_matrix_dim*score_matrix_dim+j*score_matrix_dim+k;
    		    score_matrix[pt].set_score(compare_with_em2d(proj, image_fft_amp));  

    		    score_matrix[pt].set_az(current_az);
    		    score_matrix[pt].set_alt(current_alt);
    		    score_matrix[pt].set_phi(current_phi);  		    
    		}
    	}   	
    }

	delete proj;
	delete new_mrc;
	delete image;
	delete image_fft;
	delete image_fft_amp;
	delete pa;
}

void Sim_Score_Matrix::dump_sim_score_matrix() {
	int pt;
    for (int i=0;i<=score_matrix_dim-1;i++) { 
    	for (int j=0;j<=score_matrix_dim-1;j++) {
    		for (int k=0;k<=score_matrix_dim-1;k++) { 
    			pt=i*score_matrix_dim*score_matrix_dim+j*score_matrix_dim+k;
    		}
//    		cout<<endl;
    	}
//    	cout<<endl;
    }
}

void Sim_Score_Matrix::order_sim_score_matrix() {
	Orientation_Score temp;
	for(int i=0;i<score_matrix_dim*score_matrix_dim*score_matrix_dim-1;i++) {
		for(int j=i+1;j<score_matrix_dim*score_matrix_dim*score_matrix_dim;j++) {
			if(score_matrix[i].get_score()>score_matrix[j].get_score()) { 
				temp=score_matrix[i]; 
				score_matrix[i]=score_matrix[j]; 
				score_matrix[j]=temp;
			}
		}
	}
}
