#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Pos::debug_force(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

 int  i,j;
 int trial     =   0;
 int iatm1, iatm2, iatm3, three, ndatom;
 int itors1, itors2, itors3, ndtors;
 int itbend1, itbend2, itbend3, ndtbend;
 int int_arg, debug_all, debug_three;

 double eps, V0, V1; 
 double   fx_num, fx_an, fy_num, fy_an, fz_num, fz_an;
 double ft_num, ft_an, fb_num, fb_an;

 int ntors_tr         = system.tors_defs.num_trans;
 int ntbend_tr        = system.tors_defs.num_tbend;
 int *natm_trans      = system.tors_defs.natm_trans;

/*-------------------------------------------------------------------------*/
/* I) Energies                                                             */

 print_doubleline();
 printf(">>Debug energies and forces<<\n");
 print_line();

 force(system);

     LINE
     printf("\tEnergies in kcal/mol\n\n");
     printf("\t-Bond energy                 %.16g\n",KCAL*ebond);
     printf("\t-Bend energy                 %.16g\n",KCAL*ebend);
     printf("\t   Bond-Bend energy          %.16g\n",KCAL*system.bend.vbond);
     printf("\t   Bend-Bend energy          %.16g\n",KCAL*system.bend.vbend);
     printf("\t-Torsion energy              %.16g\n",KCAL*etors);
     printf("\t   Tors-Tors energy          %.16g\n",KCAL*system.tors.vtors);
     printf("\t   Tors-Improper energy      %.16g\n",KCAL*system.tors.vimpr);
     printf("\t-Onefour energy              %.16g\n",KCAL*eonfo);
     printf("\t   Onfo-Coulomb energy       %.16g\n",KCAL*system.onfo.vcoul);
     printf("\t   Onfo-Lennard-Jones energy %.16g\n",KCAL*system.onfo.vlj);
     printf("\t-Inter energy                %.16g\n",KCAL*einter);
     printf("\t   Lennard-Jones energy      %.16g\n",KCAL*system.inter.elj);
     printf("\t   Coulomb energy            %.16g\n",KCAL*system.inter.ecoul);
     printf("\t   SCP energy                %.16g\n",KCAL*system.inter.escp);
     printf("\t   Knowledge based energy    %.16g\n",KCAL*system.inter.ekb);
     printf("\t-Total Lennard-Jones energy  %.16g\n",
              KCAL*(system.inter.elj+system.onfo.vlj));
     printf("\t-Total Coulomb energy        %.16g\n",
              KCAL*(system.inter.ecoul+system.onfo.vcoul));
     printf("\t-Constraint energy           %.16g\n",KCAL*econstraint);


     LINE

     printf("\tEnergies in Ha\n\n");
     printf("\t-Bond energy                 %.16g\n",ebond);
     printf("\t-Bend energy                 %.16g\n",ebend);
     printf("\t   Bond-Bend energy          %.16g\n",system.bend.vbond);
     printf("\t   Bend-Bend energy          %.16g\n",system.bend.vbend);
     printf("\t-Torsion energy              %.16g\n",etors);
     printf("\t   Tors-Tors energy          %.16g\n",system.tors.vtors);
     printf("\t   Tors-Improper energy      %.16g\n",system.tors.vimpr);
     printf("\t-Onefour energy              %.16g\n",eonfo);
     printf("\t   Onfo-Coulomb energy       %.16g\n",system.onfo.vcoul);
     printf("\t   Onfo-Lennard-Jones energy %.16g\n",system.onfo.vlj);
     printf("\t-Inter energy                %.16g\n",einter);
     printf("\t   Lennard-Jones energy      %.16g\n",system.inter.elj);
     printf("\t   Coulomb energy            %.16g\n",system.inter.ecoul);
     printf("\t   SCP energy                %.16g\n",system.inter.escp);
     printf("\t   Knowledge based energy    %.16g\n",system.inter.ekb);
     printf("\t-Total Lennard-Jones energy  %.16g\n",
              (system.inter.elj+system.onfo.vlj));
     printf("\t-Total Coulomb energy        %.16g\n",
              (system.inter.ecoul+system.onfo.vcoul));
     printf("\t-Constraint energy           %.16g\n",econstraint);

/*-------------------------------------------------------------------------*/
/* II) Cartesian Forces                                                    */

 LINE 
 LINE
 printf("Analytical vs. numerical forces\n");

 if (system.general.prop_type==1) {/*cartesian propagation*/
    debug_all   = 0;
    debug_three = 0;
    LINE
    printf("Test could be performed either on all \"atoms\" \n");
    printf("or a selected set (three) of \"atoms\". The former \n");
    printf("choice will take more time! \n");
    printf("Please type \"1\" if interested in all \"atoms\"\n");
    printf("Please type \"2\" if interested in a selected set of \"atoms\"\n");
    scanf("%d", &int_arg); 
    if (int_arg==1) {debug_all   = 1;}
    if (int_arg==2) {debug_three = 1;}
    if (debug_three) {
        LINE
        printf("Please give the indices of three \"atoms\" \n");
        three = 3;
        scanf("%d %d %d", &iatm1, &iatm2, &iatm3); 
        ndatom = three;
    }/*debug_three*/
    if (debug_all)   {ndatom = natom;}
        init_propose(system);
        potential(system,trial);
        V0 = epotp;
        eps = 1.e-6;
        for (i=1;i<=ndatom;i++) {
           if (debug_three) {
              if (i==1) {j = iatm1;}    
              if (i==2) {j = iatm2;}
              if (i==3) {j = iatm3;}
           }/*debug_three*/
           if (debug_all) {j=i;}
   
           init_propose(system);
           potential(system,trial);
           fx_an  = fx[j];
           xi[j] += eps;
           potential(system,trial);
           V1 = epotp;
           fx_num = -(V1-V0)/eps;
                   
           init_propose(system);
           potential(system,trial);
           fy_an  = fy[j];
           yi[j] += eps;
           potential(system,trial);
           V1 = epotp;
           fy_num = -(V1-V0)/eps;
          
           init_propose(system);
           potential(system,trial);
           fz_an  = fz[j];
           zi[j] += eps;
           potential(system,trial);
           V1 = epotp;
           fz_num = -(V1-V0)/eps;
          
           printf("\n\t atom %d fx_num %g fx_an %g  \n",j,fx_num,fx_an);
           printf("\t atom %d fy_num %g fy_an %g  \n",j,fy_num,fy_an);
           printf("\t atom %d fz_num %g fz_an %g  \n",j,fz_num,fz_an);
        }/*endfor*/
 }/*cartesian propagation*/


/*-------------------------------------------------------------------------*/
/* III) Torsional Forces                                                   */

 if (system.general.prop_type==2) {/*torsional propagation*/
    debug_all   = 0;
    debug_three = 0;
    LINE
    printf("Test could be performed either on all \"torsions\" \n");
    printf("or a selected set (three) of \"torsions\". The former \n");
    printf("choice will take more time! \n");
    printf("Please type \"1\" if interested in all \"torsions\"\n");
    printf("Please type \"2\" if interested in a set of \"torsions\"\n");
    scanf("%d", &int_arg); 
    if (int_arg==1) {debug_all   = 1;}
    if (int_arg==2) {debug_three = 1;}
    if (debug_three) {
        LINE
        printf("The number of primary dihedrals: %d\n",ntors_tr); 
        printf("Please give the indices of three \"dihedrals\" \n");
        three = 3;
        scanf("%d %d %d", &itors1, &itors2, &itors3);
        ndtors = three;
    }/*debug_three*/
    if (debug_all)   {ndtors = ntors_tr;}
    init_propose(system);
    potential(system,trial);
    V0 = epotp;
    eps = 1.e-7;
    for (i=1;i<=ndtors;i++) {
      if (debug_three) {
         if (i==1) {j = itors1;}
         if (i==2) {j = itors2;}
         if (i==3) {j = itors3;}
      }
      if (debug_all) {j=i;}
      init_propose(system);
      potential(system,trial);
      tors_transformXFXtoTFT();
      ft_an = ft[j];
      t[j] += eps;
      tors_transformTtoX();
      potential(system,trial);
      V1 = epotp;
      ft_num = -(V1-V0)/eps;
         
      printf("\n\t dihedral %d fx_num %g fx_an %g  \n",j,ft_num,ft_an);
    }/*endfor*/
 }/*torsional propagation*/


/*-------------------------------------------------------------------------*/
/* IV) Bend Forces                                                         */

 if (system.general.prop_type==3) {/*bend propagation*/
    debug_all   = 0;
    debug_three = 0;
    LINE
    printf("Test could be performed either on all transformed \"bends\" \n");
    printf("or a selected set (three) of these \"bends\". The former \n");
    printf("choice will take more time! \n");
    printf("Please type \"1\" if interested in all \"bends\"\n");
    printf("Please type \"2\" if interested in a set of \"bends\"\n");
    scanf("%d", &int_arg); 
    if (int_arg==1) {debug_all   = 1;}
    if (int_arg==2) {debug_three = 1;}
    if (debug_three) {
        LINE
        printf("The number of transformed bends: %d\n",ntbend_tr); 
        printf("Please give the indices of three \"bends\" \n");
        three = 3;
        scanf("%d %d %d", &itbend1, &itbend2, &itbend3);
        ndtbend = three;
    }/*debug_three*/
    if (debug_all)   {ndtbend = ntbend_tr;}
    init_propose(system);
    potential(system,trial);
    V0 = epotp;
    eps = 1.e-7;
    for (i=1;i<=ndtbend;i++) {
      if (debug_three) {
         if (i==1) {j = itbend1;}
         if (i==2) {j = itbend2;}
         if (i==3) {j = itbend3;}
      }
      if (debug_all) {j=i;}
      init_propose(system);
      potential(system,trial);
      tors_transformXFXtoBFB();
      fb_an = fb[j];
      b[j] += eps;
      tors_transformBtoX();
      potential(system,trial);
      V1 = epotp;
      fb_num = -(V1-V0)/eps;
         
      printf("\n\t bend %d fb_num %g fb_an %g  \n",j,fb_num,fb_an);
    }/*endfor*/
 }/*bend propagation*/


/*-------------------------------------------------------------------------*/
/* V) Testing different energy routines                                   */

    
 printf("\n\n");
 printf("system.bond.energy(this) %g system.bond.getPot(this) %g\n",
                   system.bond.energy(this),system.bond.getPot(this));

 printf("system.bend.energy(this) %g system.bend.getPot(this) %g\n",
                   system.bend.energy(this),system.bend.getPot(this));

 printf("system.tors.energy(this) %g system.tors.getPot(this) %g\n",
                   system.tors.energy(this),system.tors.getPot(this));

 printf("system.onfo.energy(this) %g system.onfo.getPot(this) %g\n",
                   system.onfo.energy(this),system.onfo.getPot(this));

 printf("system.inter.energy(this) %g system.inter.getPot(this) %g\n",
                   system.inter.energy(this),system.inter.getPot(this));

 printf("system.constraint.energy(this) %g system.constraint.getPot(this) %g\n",
                   system.constraint.energy(this),system.constraint.getPot(this));

 LINE

 print_line();
 printf("Completed >>Debug energies and forces<<\n");
 print_doubleline();


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
