#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void dpotential_cart(double *xyz, double *dxyz)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
  int pp_min, pp_e;
  double epot, ebond, ebend, etors, eonfo, einter, ecnstr;
  double zero;

  extern  System  *global_system;
  System *system = global_system;

  pp_min = system->general.postprop_minimize;
  pp_e   = system->general.postprop_minimize_energy;

  Pos *pos = &(system->pos);
  pos->xyz = xyz;
  pos->update_x_y_z();

  if (pp_min==0) {
    pos->force_zero();
    ebond  = system->bond.energy(pos);
    ebend  = system->bend.energy(pos);
    etors  = system->tors.energy(pos);
    eonfo  = system->onfo.energy(pos);
    einter = system->inter.energy(pos);
    ecnstr = system->constraint.energy(pos);
    epot   = ebond + ebend + etors + eonfo + einter + ecnstr;
  }/*endif*/

  if (pp_min==1) {
    zero = 0.0;
    pos->force_zero();
    if (pp_e>=1) {ebond  = system->bond.energy(pos);}
    else         {ebond  = zero;}
    if (pp_e>=2) {ebend  = system->bend.energy(pos);}
    else         {ebend  = zero;}
    if (pp_e>=3) {etors  = system->tors.energy(pos);}
    else         {etors  = zero;}
    if (pp_e>=4) {eonfo  = system->onfo.energy(pos);}
    else         {eonfo  = zero;}
    if (pp_e>=5) {einter = system->inter.energy(pos);}
    else         {einter = zero;}
    epot   = ebond + ebend + etors + eonfo + einter;
  }/*endif*/

  pos->dxyz = dxyz;
  pos->update_dxyz();

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void dpotential_tors(double *t, double *dt)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
  int i, ntors_tr; 
  int pp_min, pp_e;  
  double  pot;
  double epot, ebond, ebend, etors, eonfo, einter, ecnstr; 
  double *xt, *yt, *zt, *pt;
  double zero;
  
  double mone = -1.0;

  extern  System  *global_system;
  System *system = global_system;

  pp_min = system->general.postprop_minimize;
  pp_e   = system->general.postprop_minimize_energy;

  ntors_tr = system->tors_defs.num_trans;

  pbc_0_2Pi(t,ntors_tr);


  Pos *pos = &(system->pos);
  pos->system_ptr = system;

  pos->tors_transformXtoT();
  pt = pos->t;
  pos->t = t;
  pos->tors_transformTtoX();
  pos->t = pt;

      xt = pos->x; 
      yt = pos->y; 
      zt = pos->z;
  pos->x = pos->xi; 
  pos->y = pos->yi; 
  pos->z = pos->zi;

  if (pp_min==0) {
    pos->force_zero();
    ebond  = system->bond.energy(pos);
    ebend  = system->bend.energy(pos);
    etors  = system->tors.energy(pos);
    eonfo  = system->onfo.energy(pos);
    einter = system->inter.energy(pos);
    ecnstr = system->constraint.energy(pos);
    epot   = ebond + ebend + etors + eonfo + einter + ecnstr;
  }/*endif*/

  if (pp_min==1) {
    zero = 0.0;
    pos->force_zero();
    if (pp_e>=1) {ebond  = system->bond.energy(pos);}
    else         {ebond  = zero;}
    if (pp_e>=2) {ebend  = system->bend.energy(pos);}
    else         {ebend  = zero;}
    if (pp_e>=3) {etors  = system->tors.energy(pos);}
    else         {etors  = zero;}
    if (pp_e>=4) {eonfo  = system->onfo.energy(pos);}
    else         {eonfo  = zero;}
    if (pp_e>=5) {einter = system->inter.energy(pos);}
    else         {einter = zero;}
    epot   = ebond + ebend + etors + eonfo + einter;
  }/*endif*/

  pot = epot;   

  pos->x = xt;
  pos->y = yt;
  pos->z = zt;

  pos->ft = dt;                                              /* ! dt = -ft */
  pos->tors_transformXFXtoTFT();
  pos->tors_transformTtoX();
  scal_vec(mone,dt,ntors_tr);                                /* ! dt = -ft */

  pos->init_propose(*system);                        /* restore xi, yi, zi */


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void dpotential_bend(double *b, double *db)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
  int i, ntbend_tr;  
  int pp_min, pp_e;   
  double  pot;
  double epot, ebond, ebend, etors, eonfo, einter, ecnstr; 
  double *xt, *yt, *zt, *pb;
  double zero;  

  double mone = -1.0;

  extern  System  *global_system;
  System *system = global_system;

  pp_min = system->general.postprop_minimize;
  pp_e   = system->general.postprop_minimize_energy;

  ntbend_tr = system->tors_defs.num_tbend;

  Pos *pos = &(system->pos);
  pos->system_ptr = system;

  pos->tors_transformXtoB();
  pb = pos->b;
  pos->b = b;
  pos->tors_transformBtoX();
  pos->b = pb;

      xt = pos->x; 
      yt = pos->y; 
      zt = pos->z;
  pos->x = pos->xi; 
  pos->y = pos->yi; 
  pos->z = pos->zi;

  if (pp_min==0) {
    pos->force_zero();
    ebond  = system->bond.energy(pos);
    ebend  = system->bend.energy(pos);
    etors  = system->tors.energy(pos);
    eonfo  = system->onfo.energy(pos);
    einter = system->inter.energy(pos);
    ecnstr = system->constraint.energy(pos);
    epot   = ebond + ebend + etors + eonfo + einter + ecnstr;
  }/*endif*/

  if (pp_min==1) {
    zero = 0.0;
    pos->force_zero();
    if (pp_e>=1) {ebond  = system->bond.energy(pos);}
    else         {ebond  = zero;}
    if (pp_e>=2) {ebend  = system->bend.energy(pos);}
    else         {ebend  = zero;}
    if (pp_e>=3) {etors  = system->tors.energy(pos);}
    else         {etors  = zero;}
    if (pp_e>=4) {eonfo  = system->onfo.energy(pos);}
    else         {eonfo  = zero;}
    if (pp_e>=5) {einter = system->inter.energy(pos);}
    else         {einter = zero;}
    epot   = ebond + ebend + etors + eonfo + einter;
  }/*endif*/

  pot = epot;   

  pos->x = xt;
  pos->y = yt;
  pos->z = zt;

  pos->fb = db;                                              /* ! db = -fb */
  pos->tors_transformXFXtoBFB();
  pos->tors_transformBtoX();
  scal_vec(mone,db,ntbend_tr);                               /* ! db = -fb */

  pos->init_propose(*system);                        /* restore xi, yi, zi */


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

