#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"


/*=========================================================================*/
double Pos::energy(System &system, int type)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  double total;      
    
  int action_type = system.general.action_type;
  int prop_type   = system.general.prop_type;

  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     if (prop_type==1) {/*cartesian propagation, in paticl. HMC*/   
         total = potential(system,type) + kinetic();
     }
     else if (prop_type==2) {/*torsional propagation*/
         total = potential(system,type);
     }/*endif*/
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
       total =  potential(system,type);
  }/*endif*/

  return total;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
double Pos::potential(System &system, int type)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
       
  double pot = 0.0;
  double *xt,*yt,*zt; 
  int *permute_seqt;

  int trial = 0;
  int real = 1;

  if (type == real) {
     if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
         pot = epot; 
     }
     else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
        pot += system.inter.getPotPermute(this);
     }
  } else if (type == trial) {
     if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
         xt = x; yt = y; zt = z;
         x = xi; y = yi; z = zi;
         force_zero(); 
          ebondp    = system.bond.energy(this);
          ebendp    = system.bend.energy(this);
          etorsp    = system.tors.energy(this);
          eonfop    = system.onfo.energy(this);
          einterp   = system.inter.energy(this);
          ecryo_emp = system.cryo_em.energy(this);
          econstraintp = system.constraint.energy(this);
          epotp = ebondp + ebendp + etorsp + eonfop + einterp + ecryo_emp
                         + econstraintp;
          pot   = epotp;
         x = xt; y = yt; z = zt;
         ebendp_bond  = system.bend.vbond;
         ebendp_bend  = system.bend.vbend;
         etorsp_tors  = system.tors.vtors;
         etorsp_imp   = system.tors.vimpr;
         eonfop_lj    = system.onfo.vlj;
         eonfop_coul  = system.onfo.vcoul;
         einterp_lj   = system.inter.elj;
         einterp_coul = system.inter.ecoul;
         einterp_scp  = system.inter.escp;
         einterp_kb   = system.inter.ekb;
     }
     else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
         permute_seqt = permute_seq;
         permute_seq = permute_seq_prop;
         pot += system.inter.getPotPermute(this);
         permute_seq = permute_seqt;
     }
  } else {
     cout << "type is not the right number" <<endl;
     exit(1);
  }

  return pot;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
double Chain::potential(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
  return pos.potential(system);

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
double Pos::potential(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
     
  int real = 1;
 
  return potential(system,real); 

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
double Pos::potential()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
     
  double pot;
  System *system = system_ptr;

    ebond    = system->bond.getPot(this);
    ebend    = system->bend.getPot(this);
    etors    = system->tors.getPot(this);
    eonfo    = system->onfo.getPot(this);
    einter   = system->inter.getPot(this);
    ecryo_em = system->cryo_em.getPot(this);
    econstraint = system->constraint.getPot(this);
    epot   = ebond + ebend + etors + eonfo + einter + ecryo_em
                   + econstraint;
  pot = epot;   

  return pot;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
double Pos::kinetic()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i;
  double kin = 0.0;

  for (i=1;i<=natom;i++) {
      kin += 1.0/2.0*mass[i]*vx[i]*vx[i];
      kin += 1.0/2.0*mass[i]*vy[i]*vy[i];
      kin += 1.0/2.0*mass[i]*vz[i]*vz[i];
  }
  return kin;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
