#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
double potential_cart(double *xyz)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
     
  int pp_min, pp_e;
  double  pot;
  double epot, ebond, ebend, etors, eonfo, einter, ecnstr; 
  double zero;

  extern  System  *global_system;
  System *system = global_system;

  pp_min = system->general.postprop_minimize;
  pp_e   = system->general.postprop_minimize_energy;

  Pos *pos = &(system->pos);
  pos->xyz = xyz;
  pos->update_x_y_z();

  if (pp_min==0) {
    ebond  = system->bond.getPot(pos);
    ebend  = system->bend.getPot(pos);
    etors  = system->tors.getPot(pos);
    eonfo  = system->onfo.getPot(pos);
    einter = system->inter.getPot(pos);
    ecnstr = system->constraint.getPot(pos);
    epot   = ebond + ebend + etors + eonfo + einter + ecnstr;
  }/*endif*/

  zero = 0.0;
  if (pp_min==1) {
    if (pp_e>=1) {ebond  = system->bond.getPot(pos);}
    else         {ebond  = zero;}
    if (pp_e>=2) {ebend  = system->bend.getPot(pos);}
    else         {ebend  = zero;}
    if (pp_e>=3) {etors  = system->tors.getPot(pos);}
    else         {etors  = zero;}
    if (pp_e>=4) {eonfo  = system->onfo.getPot(pos);}
    else         {eonfo  = zero;}
    if (pp_e>=5) {einter = system->inter.getPot(pos);}
    else         {einter = zero;}
    epot   = ebond + ebend + etors + eonfo + einter;
  }/*endif*/


  pos->einter = einter;  
  pos->epot   = epot;

  pot = epot;   

  return pot;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
double potential_tors(double *t)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
  int ntors_tr;
  int pp_min, pp_e;
     
  double  pot;
  double epot, ebond, ebend, etors, eonfo, einter, ecnstr;
  double *xt, *yt, *zt, *pt;
  double zero;

  extern  System  *global_system;
  System *system = global_system;

  pp_min = system->general.postprop_minimize;
  pp_e   = system->general.postprop_minimize_energy;

  ntors_tr = system->tors_defs.num_trans;

  pbc_0_2Pi(t,ntors_tr);

  Pos *pos = &(system->pos);
  pos->system_ptr = system;

  pos->tors_transformXtoT();
  pt = pos->t;
  pos->t = t;
  pos->tors_transformTtoX();
  pos->t = pt;

      xt = pos->x;
      yt = pos->y;
      zt = pos->z;
  pos->x = pos->xi;
  pos->y = pos->yi;
  pos->z = pos->zi;
   
  if (pp_min==0) {
    ebond  = system->bond.getPot(pos);
    ebend  = system->bend.getPot(pos);
    etors  = system->tors.getPot(pos);
    eonfo  = system->onfo.getPot(pos);
    einter = system->inter.getPot(pos);
    ecnstr = system->constraint.getPot(pos);
    epot   = ebond + ebend + etors + eonfo + einter + ecnstr;
  }/*endif*/

  zero = 0.0;
  if (pp_min==1) {
    if (pp_e>=1) {ebond  = system->bond.getPot(pos);}
    else         {ebond  = zero;}
    if (pp_e>=2) {ebend  = system->bend.getPot(pos);}
    else         {ebend  = zero;}
    if (pp_e>=3) {etors  = system->tors.getPot(pos);}
    else         {etors  = zero;}
    if (pp_e>=4) {eonfo  = system->onfo.getPot(pos);}
    else         {eonfo  = zero;}
    if (pp_e>=5) {einter = system->inter.getPot(pos);}
    else         {einter = zero;}
    epot   = ebond + ebend + etors + eonfo + einter;
  }/*endif*/

  pos->einter = einter;  
  pos->epot   = epot;

  pot = epot;   

  pos->x = xt;
  pos->y = yt;
  pos->z = zt;

  return pot;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
double potential_bend(double *b)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
  int ntbend_tr;
  int pp_min, pp_e;
     
  double  pot;
  double epot, ebond, ebend, etors, eonfo, einter, ecnstr;
  double *xt, *yt, *zt, *pb;
  double zero;

  extern  System  *global_system;
  System *system = global_system;

  pp_min = system->general.postprop_minimize;
  pp_e   = system->general.postprop_minimize_energy;

  ntbend_tr = system->tors_defs.num_tbend;

  Pos *pos = &(system->pos);
  pos->system_ptr = system;

  pos->tors_transformXtoB();
  pb = pos->b;
  pos->b = b;
  pos->tors_transformBtoX();
  pos->b = pb;

      xt = pos->x;
      yt = pos->y;
      zt = pos->z;
  pos->x = pos->xi;
  pos->y = pos->yi;
  pos->z = pos->zi;

  if (pp_min==0) {   
    ebond  = system->bond.getPot(pos);
    ebend  = system->bend.getPot(pos);
    etors  = system->tors.getPot(pos);
    eonfo  = system->onfo.getPot(pos);
    einter = system->inter.getPot(pos);
    ecnstr = system->constraint.getPot(pos);
    epot   = ebond + ebend + etors + eonfo + einter + ecnstr;
  }/*endif*/

  zero = 0.0;
  if (pp_min==1) {
    if (pp_e>=1) {ebond  = system->bond.getPot(pos);}
    else         {ebond  = zero;}
    if (pp_e>=2) {ebend  = system->bend.getPot(pos);}
    else         {ebend  = zero;}
    if (pp_e>=3) {etors  = system->tors.getPot(pos);}
    else         {etors  = zero;}
    if (pp_e>=4) {eonfo  = system->onfo.getPot(pos);}
    else         {eonfo  = zero;}
    if (pp_e>=5) {einter = system->inter.getPot(pos);}
    else         {einter = zero;}
    epot   = ebond + ebend + etors + eonfo + einter;
  }/*endif*/


  pos->einter = einter;  
  pos->epot   = epot;

  pot = epot;   

  pos->x = xt;
  pos->y = yt;
  pos->z = zt;

  return pot;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
