#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_HESSIAN_OFF
#define DEBUG_DIAGON_OFF

/*=========================================================================*/
void Pos::get_hessian(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int  i,ii,j,jj, inm,jnm;
  int   trial  =  0;
  int   hdim;
  double eps, eps0, dxyz_eps, dt_eps;
  double bohr2_ikcal;
  double hconv;

  int prop_type = system.general.prop_type;
  int ntors_tr  = system.tors_defs.num_trans;
  char *hessian_filename = system.filenames.hessian_filename;
  FILE *output_file;


/*-------------------------------------------------------------------------*/
/* 0) Output                                                               */

  LINE
  LINE
  print_doubleline();
  if (prop_type==1)
      printf("Construct cartesian hessian\n");
  if (prop_type==2)
      printf("Construct torsional hessian\n");
  print_line(); 


/*-------------------------------------------------------------------------*/
/* I) Numerical approximation to the lower trangular part of the Hessian   */

  if (prop_type==1) {/*cartesian*/
  eps = 1.e-6;
  update_xyz();
  update_xi_yi_zi();
  potential(system,trial);  
  update_dxyz();
  for (i=1;i<=3*natom;i++) {
       if ((i % 3)==0) {ii = i/3;} else {ii = i/3 + 1;}      
       update_xyz();
       xyz[i] += eps;
       update_xi_yi_zi();
       potential(system,trial);
       update_fxyz();
       for (j=1;j<=i;j++) {
            if ((j % 3)==0) {jj = j/3;} else {jj = j/3 + 1;}
   	    dxyz_eps = -fxyz[j];
            hessian[i][j] = (dxyz_eps-dxyz[j])/eps;
	    hessian[i][j] /= sqrt(mass[ii]*mass[jj]);
       }/*-j-endfor*/
  }/*-i-endfor*/
  }/*cartesian*/


  if (prop_type==2) {/*torsional*/
    //  if ((system.inter.implicit_solvent)==0) {eps = 1.e-6;}
    //  if ((system.inter.implicit_solvent)==1) {eps = 1.e-7;}
  eps = 1.e-6;
  update_xyz();
  update_xi_yi_zi();
  potential(system,trial);
  tors_transformXFXtoTFT();
  update_dt();
  for (i=1;i<=ntors_tr;i++) {
       update_xyz();
       update_xi_yi_zi();
       tors_transformXtoT(); 
       t[i] += eps; 
       tors_transformTtoX();
       potential(system,trial);
       tors_transformXFXtoTFT();
       for (j=1;j<=i;j++) {
   	    dt_eps = -ft[j];
            hessian[i][j] = (dt_eps-dt[j])/eps;
       }/*-j-endfor*/
  }/*-i-endfor*/
  }/*torsional*/

/*-------------------------------------------------------------------------*/
/* II) Get the upper trangular part of the Hessian                         */

  if (prop_type==1) {hdim = 3*natom;}
  if (prop_type==2) {hdim = ntors_tr;}

  for (i=1;i<=hdim;i++) {
       for (j=1;j<i;j++) {
    	    hessian[j][i] = hessian[i][j];
       }
  }  

/*-------------------------------------------------------------------------*/
/* III) Write out the Hessian matrix in kcal/(Angstrom*Angstrom) dimensions*/

  if (prop_type==1) {
      bohr2_ikcal = BOHR*BOHR/KCAL; 
      hconv = 1.0/bohr2_ikcal;
  }
  if (prop_type==2) {hconv = KCAL;}

  if (system.general.write_hessian==1) {
      if (prop_type==1) 
         printf("hessian in kcal/mol/A^2 -> file \"%s\"\n",hessian_filename);
      if (prop_type==2) 
         printf("hessian in kcal/mol/rad^2 -> file \"%s\"\n",hessian_filename);
      output_file = fopen(hessian_filename,"w");
      for (i=1;i<=hdim;i++) {
          for (j=1;j<=hdim;j++) {
     	       fprintf(output_file," %d %d %20.16g\n",i,j,hessian[i][j]*hconv);
          }
      } 
      fclose(output_file);
  }

/*-------------------------------------------------------------------------*/
/* IV) Get Normal Mode Center coordinate                                   */
  
  if (prop_type==1) {/*cartesian*/
  for (i=1;i<=natom;i++) {
       xc[i] = x[i];
       yc[i] = y[i];
       zc[i] = z[i];
  }
  update_xyzc();
  }/*cartesian*/

  if (prop_type==2) {/*torsional*/
      update_xyz();
      update_xi_yi_zi();
      tors_transformXtoT();
      for (i=1;i<=ntors_tr;i++) {tc[i] = t[i];}
  }/*torsional*/



/*-------------------------------------------------------------------------*/
/* V) Output                                                               */

  print_line(); 
  if (prop_type==1)
      printf("Completed catesian hessian construction  \n");
  if (prop_type==2)
      printf("Completed torsional hessian construction \n");
  print_doubleline();


/*-------------------------------------------------------------------------*/
/* VI) Debug Hessian                                                       */

#ifdef DEBUG_HESSIAN
  if (prop_type==1) {
  double V0, V1, Vm1, dV, dV_s, ddV, iter, fx_an;
  // convergence test with changing epsilon
  eps = 1.e-2;
  for (iter=1;iter<=100;iter++) {

  init_propose(system);
  potential(system,trial);  
  V0 = epotp;
  fx_an = fx[1];
  
  xi[1] += eps;
  potential(system,trial);
  V1 = epotp;

  init_propose(system);
  xi[1] -= eps;
  potential(system,trial);
  Vm1 = epotp;

  ddV = (V1 + Vm1 - 2.0*V0)/(eps*eps);
  dV = (V1 - V0)/(eps);
  dV_s = (V1 - Vm1)/(2.0*eps);

    printf("%.16g %.16g %.16g %.16g %.16g\n",eps,dV,dV_s,-fx_an,ddV);
    eps *= pow(0.999,iter);

  }  
  // second order approximation to the first diagnal element of the hessian
  bohr2_ikcal = BOHR*BOHR/KCAL;
  printf("second order approx. to the first diagnal element of the hessian\n");
  printf("no analytical derivatives are used, only the function value\n");
  printf("eps %.16g ddV %.16g\n",eps,ddV/bohr2_ikcal);
  }/*prop_type==1*/
#endif


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::diagon_hessian(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i,ii, j, ndim; 
  double econv, hconv;
  int prop_type = system.general.prop_type;
  int ntors_tr  = system.tors_defs.num_trans;
  int nmode_stg = system.general.nmode_stg;

  char *eighess_filename = system.filenames.eighess_filename;
  char *eigvechess_filename = system.filenames.eigvechess_filename;
  FILE *output_file;


/*-------------------------------------------------------------------------*/
/* 0) Output                                                               */

  LINE
  LINE
  print_doubleline();
  if (prop_type==1) {
      printf("Diagonalize cartesian hessian\n");
  }/*endif*/
  if (prop_type==2)
      printf("Diagonalize torsional hessian\n");
  print_line(); 


/*-------------------------------------------------------------------------*/
/* I) Diagonalize Hessian                                                  */

  if (prop_type==1) {
      if (nmode_stg==1) {ndim = 3*natm_bb;} 
      else {ndim = 3*natom;}
  }
  if (prop_type==2) {ndim = ntors_tr;}


  tred2(hessian,ndim,lam_hess,llam_hess);
  tqli(lam_hess,llam_hess,ndim,hessian);  
  eigsrt(lam_hess,hessian,ndim);

/*-------------------------------------------------------------------------*/
/* II) Write out Eigenvalues                                               */

  if (system.general.write_eighess == 1) {
      if (system.general.write_energy_unit==1) {
          econv=KCAL;
          printf("eigenvalues in kcal/mol -> file \"%s\"\n",eighess_filename);
      }
      if (system.general.write_energy_unit==2) {
          econv=1.0;
          printf("eigenvalues in Ha -> file \"%s\"\n",eighess_filename);
      }
      output_file = fopen(eighess_filename,"w");
      for (i=1;i<=ndim;i++) {
           fprintf(output_file,"lam_hess[%d]: %20.16g\n",i,lam_hess[i]*econv);
      }
      fclose(output_file);
  }/*endif*/


/*-------------------------------------------------------------------------*/
/* III) Write out Eigenvectors                                             */
  
  hconv = 1.0;
  if (system.general.write_eigvechess==1) {
      if (prop_type==1) 
         printf("hessian eigenvectors -> file \"%s\"\n",eigvechess_filename);
      if (prop_type==2) 
         printf("hessian eigenvectors -> file \"%s\"\n",eigvechess_filename);
      output_file = fopen(eigvechess_filename,"w");
      for (i=1;i<=ndim;i++) {
          for (j=1;j<=ndim;j++) {
     	       fprintf(output_file," %d %d %20.16g\n",i,j,hessian[i][j]*hconv);
          }
      } 
      fclose(output_file);
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* IV) Close Module                                                        */

  print_line(); 
  if (prop_type==1)
      printf("Solved catesian hessian eigenvalue problem\n");
  if (prop_type==2)
      printf("Solved torsional hessian eigenvalue problem\n");
  print_doubleline();

/*-------------------------------------------------------------------------*/
/* V) Debug Eigenvectors                                                   */

#ifdef DEBUG_DIAGON
  printf("first eigenvector\n");
  for (i=1;i<=ndim;i++) {
       printf("hessian[%d][1]: %g\n",i,hessian[i][1]);
  }
  printf("last eigenvector\n");
  for (i=1;i<=ndim;i++) {
       printf("hessian[%d][%d]: %g\n",i,ndim,hessian[i][ndim]);
  }
  getchar();
#endif

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

