#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_HESSIAN_OFF
#define MASSWEIGHT

/*=========================================================================*/
void Pos::get_hessian_enm(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int  i,j,k,inm,jnm;
  int   trial  =  0;
  int   hdim;
  double eps, dxyz_eps, dt_eps;
  double bohr2_ikcal;
  double hconv;

  int prop_type = system.general.prop_type;
  int ntors_tr  = system.tors_defs.num_trans;
  char *hessian_filename = system.filenames.hessian_filename;
  FILE *output_file;


/*-------------------------------------------------------------------------*/
/* 0) Output                                                               */

  LINE
  LINE
  print_doubleline();
  if (prop_type==1)
      printf("Construct cartesian hessian based on Elastic Network Model\n");
  if (prop_type==2)
      printf("Construct torsional hessian based on Elastic Network Model\n");
  print_line(); 


/*-------------------------------------------------------------------------*/
/* I) Numerical approximation to the lower trangular part of the Hessian   */

  if (prop_type==1) {/*cartesian*/
  eps = 1.e-6;
  update_xyz();
  force_enm();
  update_dxyz();
  update_xi_yi_zi();
  for (i=1;i<=3*natom;i++) {
       copy_3vec_to_3vec(xi,yi,zi,x,y,z,natom);
       update_xyz();
       xyz[i] += eps;
       update_x_y_z();
       force_enm();
       update_fxyz();
       for (j=1;j<=i;j++) {
   	    dxyz_eps = -fxyz[j];
            hessian[i][j] = (dxyz_eps-dxyz[j])/eps;
       }/*-j-endfor*/
  }/*-i-endfor*/
  }/*cartesian*/


  if (prop_type==2) {/*torsional*/
  eps = 1.e-6;
  update_xyz();
  update_xi_yi_zi();
  force_enm();
  tors_transformXFXtoTFT();
  update_dt();
  for (i=1;i<=ntors_tr;i++) {
       update_xi_yi_zi();
       tors_transformXtoT();
       t[i] += eps;
       tors_transformTtoX();
       copy_3vec_to_3vec(xi,yi,zi,x,y,z,natom);
       force_enm();
       tors_transformXFXtoTFT();
       for (j=1;j<=i;j++) {
   	    dt_eps = -ft[j];
            hessian[i][j] = (dt_eps-dt[j])/eps;
       }/*-j-endfor*/
  }/*-i-endfor*/
  }/*torsional*/

/*-------------------------------------------------------------------------*/
/* II) Get the upper trangular part of the Hessian                         */

  if (prop_type==1) {hdim = 3*natom;}
  if (prop_type==2) {hdim = ntors_tr;}

  for (i=1;i<=hdim;i++) {
       for (j=1;j<i;j++) {
    	    hessian[j][i] = hessian[i][j];
       }
  }  

/*-------------------------------------------------------------------------*/
/* III) Write out the Hessian matrix in kcal/(Angstrom*Angstrom) dimensions*/

  if (prop_type==1) {
      bohr2_ikcal = BOHR*BOHR/KCAL; 
      hconv = 1.0/bohr2_ikcal;
  }
  if (prop_type==2) {hconv = KCAL;}

  if (system.general.write_hessian==1) {
      if (prop_type==1) 
         printf("hessian in kcal/mol/A^2 -> file \"%s\"\n",hessian_filename);
      if (prop_type==2) 
         printf("hessian in kcal/mol/rad^2 -> file \"%s\"\n",hessian_filename);
      output_file = fopen(hessian_filename,"w");
      for (i=1;i<=hdim;i++) {
          for (j=1;j<=hdim;j++) {
     	       fprintf(output_file," %d %d %20.16g\n",i,j,hessian[i][j]*hconv);
          }
      } 
      fclose(output_file);
  }

/*-------------------------------------------------------------------------*/
/* IV) Get Normal Mode Center coordinate                                   */
  
  if (prop_type==1) {/*cartesian*/
  for (i=1;i<=natom;i++) {
       xc[i] = x[i];
       yc[i] = y[i];
       zc[i] = z[i];
  }
  update_xyzc();
  }/*cartesian*/

  if (prop_type==2) {/*torsional*/
      update_xyz();
      update_xi_yi_zi();
      tors_transformXtoT();
      for (i=1;i<=ntors_tr;i++) {tc[i] = t[i];}
  }/*torsional*/

/*-------------------------------------------------------------------------*/
/* V) Mass weighting                                                       */
/*    the hessian and cartesian center coords xc, yc, zc                   */

#ifdef MASSWEIGHT  

  if (prop_type==1) {/*cartesian*/
//       for (i=1;i<=natom;i++) {
//            xc[i] = x[i]*sqrt(mass[i]);
//            yc[i] = y[i]*sqrt(mass[i]);
//            zc[i] = z[i]*sqrt(mass[i]);
//       }
//      update_xyzc();
     
      k = 1;
      for (j=1;j<=hdim;j++) {	  
	// printf("k %d\n",k);
	// getchar();
           for (i=1;i<=hdim;i++) {
	     // printf("hessian %g\n",hessian[i][j]);
     	    hessian[i][j] *= 1.0/sqrt(mass[k]);
	    // printf("hessian weighted%g\n",hessian[i][j]);
	    // getchar();
           }
           if ((j % 3)==0) k++;
      }
     
      k = 1;
      for (i=1;i<=hdim;i++) {
           for (j=1;j<=hdim;j++) {
     	    hessian[i][j] *= 1.0/sqrt(mass[k]);
           }
           if ((i % 3)==0) k++;
      }
  }/*cartesian*/
#endif

/*-------------------------------------------------------------------------*/
/* VI) Output                                                              */

  print_line(); 
  if (prop_type==1)
      printf("Completed catesian hessian construction  \n");
  if (prop_type==2)
      printf("Completed torsional hessian construction \n");
  print_doubleline();


/*-------------------------------------------------------------------------*/
/* VII) Debug Hessian                                                      */

#ifdef DEBUG_HESSIAN
  if (prop_type==1) {
  double V0, V1, Vm1, dV, dV_s, ddV, iter, fx_an;
  // convergence test with changing epsilon
  eps = 1.e-2;
  for (iter=1;iter<=100;iter++) {

  copy_3vec_to_3vec(xi,yi,zi,x,y,z,natom);
  force_enm();
  V0 = ebond_enm;
  fx_an = fx[1];
  
  x[1] += eps;
  potential_enm();
  V1 = ebond_enm;

  copy_3vec_to_3vec(xi,yi,zi,x,y,z,natom);
  xi[1] -= eps;
  potential_enm();
  Vm1 = ebond_enm;

  ddV = (V1 + Vm1 - 2.0*V0)/(eps*eps);
  dV = (V1 - V0)/(eps);
  dV_s = (V1 - Vm1)/(2.0*eps);

    printf("%.16g %.16g %.16g %.16g %.16g\n",eps,dV,dV_s,-fx_an,ddV);
    eps *= pow(0.999,iter);

  }
  // second order approximation to the first diagnal element of the hessian
  bohr2_ikcal = BOHR*BOHR/KCAL;
  printf("second order approx. to the first diagnal element of the hessian\n");
  printf("no analytical derivatives are used, only the function value\n");
  printf("eps %.16g ddV %.16g\n",eps,ddV/bohr2_ikcal);
  }/*prop_type==1*/
#endif

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

