#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_HESSIAN_OFF

/*=========================================================================*/
void Pos::get_hessian_stg(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int  i,ii,iii,j,jj,jjj,inm,jnm;
  int   trial  =  0;
  int   hdim;
  double eps, dxyz_eps, duxyzbb_eps, dt_eps;
  double bohr2_ikcal;
  double hconv;

  int prop_type   = system.general.prop_type;
  int ntors_tr    = system.tors_defs.num_trans;
  int *ind_atm_bb = system.maps.ind_atm_bb;
  char *hessian_filename = system.filenames.hessian_filename;
  FILE *output_file;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Output                                                               */

  LINE
  LINE
  print_doubleline();
  if (prop_type==1)
      printf("Construct cartesian staging hessian\n");
  if (prop_type==2) {
      sprintf(err_msg,"torsional staging hessian is not supported");
      errorMessage(err_msg);
  }/*endif*/
  print_line(); 


/*-------------------------------------------------------------------------*/
/* I) Numerical approximation to the lower trangular part of the backbone  */
/*    cartesian staging hessian                                            */

  if (prop_type==1) {/*cartesian*/
      eps = 1.e-6;
      update_xyz();
      update_xi_yi_zi();
      potential(system,trial);
      stage_transformXFXtoUFU();
      update_duxyzbb(system);
      for (i=1;i<=3*natm_bb;i++) {
           if ((i % 3)==0) {ii = i/3;} else {ii = i/3 + 1;}      
	   iii = ind_atm_bb[ii];
           update_xyz();
           update_xi_yi_zi();
           stage_transformXtoU();
           update_uxyzbb(system);
           uxyzbb[i] += eps;
           update_uxbb_uybb_uzbb(system);
           stage_transformUtoX();
           potential(system,trial);
           stage_transformXFXtoUFU();
           update_fuxyzbb(system);
           for (j=1;j<=i;j++) {
                if ((j % 3)==0) {jj = j/3;} else {jj = j/3 + 1;}      
   	        jjj = ind_atm_bb[jj];
       	        duxyzbb_eps   = -fuxyzbb[j];
                hessian[i][j] = (duxyzbb_eps-duxyzbb[j])/eps;
		//   	        hessian[i][j] /= sqrt(mass[iii]*mass[jjj]);
           }/*-j-endfor*/
      }/*-i-endfor*/
  }/*cartesian*/

/*-------------------------------------------------------------------------*/
/* II) Get the upper trangular part of the Hessian                         */

  if (prop_type==1) {hdim = 3*natm_bb;}

  for (i=1;i<=hdim;i++) {
       for (j=1;j<i;j++) {
    	    hessian[j][i] = hessian[i][j];
       }
  }  

/*-------------------------------------------------------------------------*/
/* III) Write out the Hessian matrix in kcal/(Angstrom*Angstrom) dimensions*/

  if (prop_type==1) { 
      bohr2_ikcal = BOHR*BOHR/KCAL; 
      hconv = 1.0/bohr2_ikcal; 
  }
  if (prop_type==2) {hconv = KCAL;}

  if (system.general.write_hessian==1) {
      if (prop_type==1) 
         printf("hessian in kcal/mol/A^2 -> file \"%s\"\n",hessian_filename);
      if (prop_type==2) 
         printf("hessian in kcal/mol/rad^2 -> file \"%s\"\n",hessian_filename);
      output_file = fopen(hessian_filename,"w");
      for (i=1;i<=hdim;i++) {
          for (j=1;j<=hdim;j++) {
     	       fprintf(output_file," %d %d %20.16g\n",i,j,hessian[i][j]*hconv);
          }
      } 
      fclose(output_file);
  }

/*-------------------------------------------------------------------------*/
/* IV) Get Normal Mode Center coordinate                                   */

  if (prop_type==1) {/*cartesian*/
      update_xyz();
      update_xi_yi_zi();
      stage_transformXtoU();
      update_uxyzcbb(system);
  }/*cartesian*/

/*-------------------------------------------------------------------------*/
/* V) Output                                                               */

  print_line(); 
  if (prop_type==1)
      printf("Completed catesian hessian construction  \n");
  if (prop_type==2)
      printf("Completed torsional hessian construction \n");
  print_doubleline();


/*-------------------------------------------------------------------------*/
/* VI) Debug Hessian                                                       */

#ifdef DEBUG_HESSIAN
  getchar();
  if (prop_type==1) {
  double V0, V1, Vm1, dV, dV_s, ddV, iter, fx_an;
  // convergence test with changing epsilon
  eps = 1.e-2;
  for (iter=1;iter<=100;iter++) {

  init_propose(system);
  potential(system,trial);  
  V0 = epotp;
  fx_an = fx[1];
  
  xi[1] += eps;
  potential(system,trial);
  V1 = epotp;

  init_propose(system);
  xi[1] -= eps;
  potential(system,trial);
  Vm1 = epotp;

  ddV = (V1 + Vm1 - 2.0*V0)/(eps*eps);
  dV = (V1 - V0)/(eps);
  dV_s = (V1 - Vm1)/(2.0*eps);

    printf("%.16g %.16g %.16g %.16g %.16g\n",eps,dV,dV_s,-fx_an,ddV);
    eps *= pow(0.999,iter);

  }  
  // second order approximation to the first diagnal element of the hessian
  bohr2_ikcal = BOHR*BOHR/KCAL;
  printf("second order approx. to the first diagnal element of the hessian\n");
  printf("no analytical derivatives are used, only the function value\n");
  printf("eps %.16g ddV %.16g\n",eps,ddV/bohr2_ikcal);
  }/*prop_type==1*/
#endif


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

