#include "libscpenergy/scpenergy.h"
#include "libscpenergy/Report.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"


//_____________________________________________________________________Setup
struct Energy_scp_setup {
  Real *positions; 
  Real *charges;   
  int *rs;        
  string *atomTypes; 

  SCPBond *bonds;    
  SCPAngle *angles;   
  SCPDihedral *dihedrals;
};//Energy_scp_setup//
//_____________________________________________________________________Setup




#define DEBUG
#undef DEBUG

using namespace ProtoMol::Report;


double energy_lscp(const System *system, const Pos *pos) {


  int iatm,jatm, slen;
  int ibond, ibend, itors;
  double *x, *y, *z, *fx, *fy, *fz, *q;  
  double vscp, bohr_ikcal;
  const Maps *maps;
  
  char err_msg[LINE_NCHARMAX];


  int numatoms     = pos->natom;
  int numbonds     = system->bond.nbond;
  int numangles    = system->bend.nbend;
  int numdihedrals = (system->tors.ntors)-(system->tors.nimproper);


  Real *positions, *charges;
  string *atomTypes;
  int *rs;


  SCPBond *bonds;
  SCPAngle *angles;
  SCPDihedral *dihedrals;

  static Energy_scp_setup energy_scp_setup;
  static int setup = 0; setup++;
  
//_____________________________________________________________________Setup
  if (setup==1) {
     //_________________________Group_I
     positions  =  new Real[numatoms*3];
     charges    =  new Real[numatoms];
     rs         =  new int[numatoms];
     atomTypes  =  new string[numatoms];
    
     energy_scp_setup.positions = positions;    
     energy_scp_setup.charges   = charges;    
     energy_scp_setup.rs        = rs;
     energy_scp_setup.atomTypes = atomTypes;        

     //_________________________Group_II
     bonds     = new SCPBond[numbonds];
     angles    = new SCPAngle[numangles];
     dihedrals = new SCPDihedral[numdihedrals];
   
     energy_scp_setup.bonds     = bonds;
     energy_scp_setup.angles    = angles;
     energy_scp_setup.dihedrals = dihedrals;    
  }/*setup*/
//_____________________________________________________________________Setup

    
  positions   =  energy_scp_setup.positions;    
  charges     =  energy_scp_setup.charges;    
  rs          =  energy_scp_setup.rs;
  atomTypes   =  energy_scp_setup.atomTypes;        

  bonds       =  energy_scp_setup.bonds;
  angles      =  energy_scp_setup.angles;
  dihedrals   =  energy_scp_setup.dihedrals;    


  maps = &(system->maps);
  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;
  q  = pos->chrg;


  //___________________________________________________________Assign Group_I
  for (iatm = 0; iatm < numatoms; iatm++) {

      //__________________________BOHR->A
      positions[iatm*3]   = x[iatm+1]*BOHR;   
      positions[iatm*3+1] = y[iatm+1]*BOHR;
      positions[iatm*3+2] = z[iatm+1]*BOHR;

      atomTypes[iatm].assign(maps->atm_label[iatm+1]);
      charges[iatm] = q[iatm+1];
      rs[iatm]      = 1;

#ifdef DEBUG
      printf("charges[%d] %g\n",iatm,charges[iatm]);
      printf("maps->atm_label[%d] #%s#\n",iatm+1,maps->atm_label[iatm+1]);/*C*/
      cout << "atomType is" <<"#"<<atomTypes[iatm]<<"#"<<endl;//C++//     
      getchar();
#endif

  }/*end for*/

#ifdef DEBUG
  printf(" positions \n");
  for (iatm = 0; iatm < numatoms; iatm++) {
   printf("%g %g %g\n",positions[iatm*3],positions[iatm*3+1],positions[iatm*3+2]);
  }
#endif

  //___________________________________________________________Assign Group_II
  for (ibond=1;ibond<=numbonds;ibond++) {
       bonds[ibond-1].atom1 = system->bond.atom1[ibond];
       bonds[ibond-1].atom2 = system->bond.atom2[ibond];
#ifdef DEBUG
        printf("bonds[%d] (%d,%d)\n",ibond,bonds[ibond-1].atom1,
                                         bonds[ibond-1].atom2);
#endif
  }

  for (ibend=1;ibend<=numangles;ibend++) {
       angles[ibend-1].atom1 = system->bend.atom1[ibend];
       angles[ibend-1].atom2 = system->bend.atom2[ibend];
       angles[ibend-1].atom3 = system->bend.atom3[ibend];
#ifdef DEBUG
        printf("angles[%d] (%d,%d,%d)\n",ibend,angles[ibend-1].atom1,
	                                       angles[ibend-1].atom2,
                                               angles[ibend-1].atom3);
#endif
  }

  for (itors=1;itors<=numdihedrals;itors++) {
       dihedrals[itors-1].atom1 = system->tors.atom1[itors];
       dihedrals[itors-1].atom2 = system->tors.atom2[itors];
       dihedrals[itors-1].atom3 = system->tors.atom3[itors];
       dihedrals[itors-1].atom4 = system->tors.atom4[itors];
#ifdef DEBUG
        printf("dihedrals[%d] (%d,%d,%d,%d)\n",itors,dihedrals[itors-1].atom1,
	                                             dihedrals[itors-1].atom2,
                                                     dihedrals[itors-1].atom3,
                                                     dihedrals[itors-1].atom4);
#endif

       if (system->tors.improper[itors]==1) {
	  sprintf(err_msg,"From energy_lscp::%d. torsion is improper",itors);
          errorMessage(err_msg);
       }
  }

  // choices are: "none", "1-2", "1-3", "1-4", "scaled1-4"
  string exclusion = "1-3"; // "1-3";

  SCPEnergy myValues = scpenergy(numatoms, positions, charges, bonds, numbonds, angles, numangles, dihedrals, numdihedrals, atomTypes, rs,exclusion);

#ifdef DEBUG
  report << plain << "ENERGY: " << myValues.energy << endr;
  report << plain << "FORCES: " << endr;
#endif

  bohr_ikcal = BOHR/KCAL;
  for (iatm = 0; iatm < numatoms; iatm++) {

     //__________________________________KCAL/A->Ha/Bohr
     fx[iatm+1] += (myValues.force[iatm*3])*bohr_ikcal;
     fy[iatm+1] += (myValues.force[iatm*3+1])*bohr_ikcal;
     fz[iatm+1] += (myValues.force[iatm*3+2])*bohr_ikcal;
  }


#ifdef DEBUG
  for (unsigned int i = 0; i < numatoms; i++)
    report << plain <<atomTypes[i]<<" "<< myValues.force[i*3] << " " << myValues.force[i*3+1] << " " << myValues.force[i*3+2] << endr;
#endif

//  delete myValues.force;

  //___________________KCAL->Ha
  vscp = (myValues.energy)/KCAL;

#ifdef DEBUG   
  report <<"myValues.energy "<< myValues.energy << endr;
  report <<"vscp "<< vscp << endr;
#endif

  return vscp;
}
