#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
double Inter::energy_atomic(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, iinter, itype, num_size;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz, *q;
  double *num_x, *num_V_x, *num_d2V_x;
  double dx, dy, dz, r, r2;
  double qqs, ri, rs, xr, xxx;
  double pre, vinter;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double r6, r8, r10;
  double a, b, c6, c8, c10;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double eps_ddd, deps_ddd;

  double asw, bsw, switch_length, rswt;
  int    switch_type;
  switch_type   = 4;
  switch_length = rswitch_length; 

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;
  q  = pos->chrg;
  
  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
              for (i=1;i<=excl_num[jatm];i++) 
                  {if (excl_list[i][jatm]==iatm) {valid=0;}}
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];    
                 itype = type_index[iinter];
                 if (r2 <= r_max2[itype]) {
                   if (hash_table[itype]!=0) {
                     if (hash_table[itype]==1)
                        {r       = sqrt(r2);
                         hash_r  = r-r_min[itype];  }
                     if (hash_table[itype]==2)
                        {hash_r  = r2-r_min2[itype];}
                     ihash   = int(hash_r/hash_delta[itype]) + 1;
                     ihash   = MIN(ihash,hash_size[itype]);
                     vnow    = hash_numeric_V_x[itype][ihash];
                     dvnow   = hash_numeric_dV_x[itype][ihash];
                     hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
                     vnow   += dvnow*hash_m;
                     pre     = hash_numeric_pre_x[itype][ihash];
                     vnow   *= scale_iinter[iinter];
                     pre    *= scale_iinter[iinter];
		     vinter +=  vnow;                    
                   }
                   else {
                     r  = sqrt(r2);
                     ri = 1.0/r;
		     if ( implicit_solvent==0 ) {
                      qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
                      qqs    *= scale_iinter[iinter];

                      switch_on = 0;
                      if (r >= (r_max[itype]-switch_length)) {
                          switch_on=1;
                          asw = r_max[itype]-switch_length;
                          bsw = r_max[itype];
                      }
                      if (switch_on==1) {
                          vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type);
                          pre  = qqs*switch_right(r,asw,bsw,switch_type)/(r*r2);
                          pre += -qqs*dx_switch_right(r,asw,bsw,switch_type)/r2;
                      }
                      else {
                          vnow    = qqs*ri;
                          pre     = qqs/(r*r2);
                      }

                      if (ddd>0) {
                        eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
                        deps_ddd = ddd_deps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
                        pre   = pre/eps_ddd 
		              + vnow*deps_ddd/(r*eps_ddd*eps_ddd);
                        vnow /= eps_ddd; 
                      } 

                      vinter += vnow;
                      vcoul  += vnow;                        

                     } else {
                          pre=0.0;
                     }/*endif-implicit*/
                     if (type[itype]==0) { 
                         vinter += 0;
                         pre    += 0;
                     } else if (type[itype]==1 || type[itype]==2) {
                         rs      = r+(r0[itype]-sig[itype]);
                         xr      = r0[itype]/rs;
                         xxx     = xr*xr*xr*xr*xr*xr;
                         vnow    = feps[itype]*xxx*(xxx-beta[itype]);
                         dvnow   = 6.0*feps[itype]*xxx
                                 *(2.0*xxx-beta[itype])/(r*rs);
                         vnow   *= scale_iinter[iinter];
                         dvnow  *= scale_iinter[iinter];
                         vinter += vnow;
                         vlj    += vnow;
                         pre    += dvnow;
                     } else if (type[itype]==3) {
                         a   = awill[itype];
                         b   = bwill[itype];
                         c6  = c6m[itype];
                         c8  = c8m[itype];
                         c10 = c10m[itype];
                         r6  = r2 * r2 * r2;
                         r8  = r2 * r2 * r2 * r2;
                         r10 = r8 * r2;
                         vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
                         dvnow = a*b*exp(-b*r)/r 
                               - (c6 * 6.0/r6 + c8*8.0/r8 + c10*10.0/r10)/r2;
                         vnow   *= scale_iinter[iinter];
                         dvnow  *= scale_iinter[iinter];
                         vinter += vnow;
                         pre    += dvnow;
                     } else if (type[itype]==4) {
                         num_size  = numeric_size[itype];
                         num_x     = numeric_x[itype];
                         num_V_x   = numeric_V_x[itype];
                         num_d2V_x = numeric_d2V_x[itype];
                         splint(&(num_x[0]),&(num_V_x[0]),
                                &(num_d2V_x[0]),num_size,r,&vnow);
                         dsplint(&(num_x[0]),&(num_V_x[0]),
                                 &(num_d2V_x[0]),num_size,r,&dvnow);

                         vnow   *= scale_iinter[iinter];
                         dvnow  *= scale_iinter[iinter];

                         if (switch_on==1) {
                           vnowtmp = vnow*switch_right(r,asw,bsw,switch_type);
                           vinter += vnowtmp;
                           vkb    += vnowtmp;
                           pre += -dvnow*switch_right(r,asw,bsw,switch_type)/r;
                           pre += -vnow*dx_switch_right(r,asw,bsw,switch_type)/r;
                         } else {                         
                           vinter +=  vnow;
                           vkb    +=  vnow;
                           pre    += -dvnow/r;
                         }     
		     
                     }/*end if type[itype]==*/
		   }/*endif-hash-no-hash*/
                     fx[atom1_ind] += pre*dx;
                     fy[atom1_ind] += pre*dy;
                     fz[atom1_ind] += pre*dz;
                     fx[atom2_ind] -= pre*dx;
                     fy[atom2_ind] -= pre*dy;
                     fz[atom2_ind] -= pre*dz;
                 }/*end if r <= r_max[itype]*/
              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/
      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif*/

  if (neighbor_list == 1) {
      neighbor_list_check(pos);
      if (neighbor_list_update==1) {
	  conf_save(pos);
      }/*endif*/
  }/*endif*/

  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0;
  for (iinter=1; iinter<=ninter; iinter++) {

    if (inter_off[iinter]) {continue;}
    if ((neighbor_list==1) && (neighbor_list_update==0)) {
        if (neighbors[iinter] == 0) {continue;}
    }/*endif*/

    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
      
    if ((neighbor_list==1) && (neighbor_list_update==1)) {
        if (r2 < neighbor_list_cutoff2) {neighbors[iinter] = 1;}
        else                            {neighbors[iinter] = 0;}
    }/*endif*/

    itype = type_index[iinter];

    if (r2 > r_max2[itype]) {continue;}

    if (hash_table[itype]!=0) {
        if (hash_table[itype]==1) 
           {r       = sqrt(r2);
            hash_r  = r-r_min[itype]; }
        if (hash_table[itype]==2) 
           {hash_r  = r2-r_min2[itype];}
        ihash   = int(hash_r/hash_delta[itype]) + 1;
        ihash   = MIN(ihash,hash_size[itype]);
        vnow    = hash_numeric_V_x[itype][ihash];
        dvnow   = hash_numeric_dV_x[itype][ihash];
        hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
        vnow   += dvnow*hash_m;
        pre     = hash_numeric_pre_x[itype][ihash];
        vnow   *= scale_iinter[iinter];
        pre    *= scale_iinter[iinter];
        vinter += vnow;                    

        fx[atom1_ind] += pre*dx;
        fy[atom1_ind] += pre*dy;
        fz[atom1_ind] += pre*dz;
        fx[atom2_ind] -= pre*dx;
        fy[atom2_ind] -= pre*dy;
        fz[atom2_ind] -= pre*dz;

        continue;
    }/*endif*/

    r  = sqrt(r2);
    ri = 1.0/r;    
    if ( implicit_solvent==0 ) {
      qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
      qqs    *= scale_iinter[iinter];

      switch_on = 0;

      if (r >= (r_max[itype]-switch_length)) {
          switch_on=1;
          asw = r_max[itype]-switch_length;
          bsw = r_max[itype];
      }

      if (switch_on==1) {
          vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type);
          pre =  qqs*switch_right(r,asw,bsw,switch_type)/(r*r2);
          pre += -qqs*dx_switch_right(r,asw,bsw,switch_type)/r2;
      }
      else {
          vnow    = qqs*ri;
          pre     = qqs/(r*r2);
      }/*endif-switch_on*/ 

      if (ddd>0) {
          eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd);
          deps_ddd = ddd_deps(r,ddd_p1,ddd_p2,ddd_p3,ddd);    
          pre   = pre/eps_ddd
	            + vnow*deps_ddd/(r*eps_ddd*eps_ddd);
          vnow /= eps_ddd;
      }/*endif-ddd*/

      vinter += vnow;
      vcoul  += vnow;

    } else {
          pre=0.0;
    }/*endif-implicit*/

    if (type[itype]==0) { 
      vinter += 0;
      pre    += 0;
    } else if (type[itype]==1 || type[itype]==2) {
      rs      = r+(r0[itype]-sig[itype]);
      xr      = r0[itype]/rs;
      xxx     = xr*xr*xr*xr*xr*xr;
      vnow    = feps[itype]*xxx*(xxx-beta[itype]);
      dvnow   = 6.0*feps[itype]*xxx*(2.0*xxx-beta[itype])/(r*rs);
      vnow   *= scale_iinter[iinter];
      dvnow  *= scale_iinter[iinter];
      vinter += vnow;
      vlj    += vnow;
      pre    += dvnow;
    } else if ( type[itype]==3) {
      a   = awill[itype];
      b   = bwill[itype];
      c6  = c6m[itype];
      c8  = c8m[itype];
      c10 = c10m[itype];
      r6  = r2 * r2 * r2;
      r8  = r2 * r2 * r2 * r2;
      r10 = r8 * r2;
      vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
      dvnow = a*b*exp(-b*r)/r - (c6 * 6.0/r6 + c8*8.0/r8 + c10*10.0/r10)/r2;
      vnow  *= scale_iinter[iinter];
      dvnow *= scale_iinter[iinter];
      vinter += vnow;
      pre    += dvnow;
    } else if ( type[itype]==4) {
      num_size  = numeric_size[itype];
      num_x     = numeric_x[itype];
      num_V_x   = numeric_V_x[itype];
      num_d2V_x = numeric_d2V_x[itype];
      splint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
      dsplint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&dvnow);
      vnow   *= scale_iinter[iinter];
      dvnow  *= scale_iinter[iinter];

      if (switch_on==1) {
        vnowtmp =  vnow*switch_right(r,asw,bsw,switch_type);
        vinter +=  vnowtmp;
        vkb    +=  vnowtmp;
        pre += -dvnow*switch_right(r,asw,bsw,switch_type)/r;
        pre += -vnow*dx_switch_right(r,asw,bsw,switch_type)/r;
      } else {                         
        vinter +=  vnow;
        vkb    +=  vnow;
	pre    += -dvnow/r;
      }/*endif-switch_on*/     
    }/*endif-type[itype]*/
    fx[atom1_ind] += pre*dx;
    fy[atom1_ind] += pre*dy;
    fz[atom1_ind] += pre*dz;
    fx[atom2_ind] -= pre*dx;
    fy[atom2_ind] -= pre*dy;
    fz[atom2_ind] -= pre*dz;
  }/*endfor-iinter*/
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
    
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getPot_atomic(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, iinter, itype, num_size;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz, *q;
  double dx, dy, dz, r, r2;
  double *num_x, *num_V_x, *num_d2V_x;
  double qqs, ri, rs, xr, xxx;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double vinter;
  double r6, r8, r10;
  double a, b, c6, c8, c10;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double eps_ddd;

  double asw, bsw, switch_length;
  int    switch_type;
  switch_type   = 4;

  switch_length = rswitch_length; 

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  q  = pos->chrg;

  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;
               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
 
              for (i=1;i<=excl_num[jatm];i++) {                   
                  if (excl_list[i][jatm]==iatm) {valid=0;}
              }
 
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];
                 itype = type_index[iinter];
    
                if (r2 <= r_max2[itype]) {
                 if (hash_table[itype]!=0) {
                     if (hash_table[itype]==1) 
                        {r       = sqrt(r2);
                         hash_r  = r-r_min[itype]; }
                     if (hash_table[itype]==2) 
                        {hash_r  = r2-r_min2[itype];}
                   ihash   = int(hash_r/hash_delta[itype]) + 1;
                   ihash   = MIN(ihash,hash_size[itype]);
                   vnow    = hash_numeric_V_x[itype][ihash];
                   dvnow   = hash_numeric_dV_x[itype][ihash];
                   hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
                   vnow   += dvnow*hash_m;
                   vnow   *= scale_iinter[iinter];
 		   vinter +=  vnow;                    
                 }
                 else {
                  r  = sqrt(r2);
                  ri = 1.0/r;

                  if ( implicit_solvent==0 ) {
                    qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
                    qqs    *= scale_iinter[iinter];
      			
                    switch_on = 0;
                    if (r >= (r_max[itype]-switch_length)) {
                        switch_on=1;
                        asw = r_max[itype]-switch_length;
                        bsw = r_max[itype];
                    }
                    if (switch_on==1) {
                        vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type);
                    }
                    else {
                        vnow    =  qqs*ri;
                    }            

                    if (ddd>0) {
                        eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
                        vnow /= eps_ddd; 
                    }

                    vinter += vnow;
                    vcoul  += vnow; 

                  }/*end-implicit_solvent==0*/

                  if (type[itype]==0) { 
                     vinter += 0;
                  } else if (type[itype]==1 || type[itype]==2) {
                     rs      = r+(r0[itype]-sig[itype]);
                     xr      = r0[itype]/rs;
                     xxx     = xr*xr*xr*xr*xr*xr;
                     vnow    = feps[itype]*xxx*(xxx-beta[itype]);
                     vnow   *= scale_iinter[iinter];
                     vinter += vnow;
                     vlj    += vnow;
                  } else if ( type[itype]==3) {
                     a   = awill[itype];
                     b   = bwill[itype];
                     c6  = c6m[itype];
                     c8  = c8m[itype];
                     c10 = c10m[itype];
                     r6  = r2 * r2 * r2;
                     r8  = r2 * r2 * r2 * r2;
                     r10 = r8 * r2;
                     vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
                     vnow *= scale_iinter[iinter]; 
                     vinter += vnow;
                  } else if ( type[itype]==4) {
                     num_size  = numeric_size[itype];
                     num_x     = numeric_x[itype];
                     num_V_x   = numeric_V_x[itype];
                     num_d2V_x = numeric_d2V_x[itype];
                     splint(&(num_x[1]),&(num_V_x[1]),
                            &(num_d2V_x[1]),num_size,r,&vnow);
                     vnow   *= scale_iinter[iinter];

                     if (switch_on==1) {
                       vnowtmp =  vnow*switch_right(r,asw,bsw,switch_type);
                       vinter +=  vnowtmp;
                       vkb    +=  vnowtmp;
                     } else {                         
                       vinter += vnow;
                       vkb    += vnow;
                     }     

                  }/*end if type[itype]==*/
                 }/*endif-hash-no-hash*/
                }/*end r <= r_max[itype]*/
              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/
      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif*/


  if (neighbor_list == 1) {
      neighbor_list_check(pos);
      if (neighbor_list_update==1) {
	  conf_save(pos);
      }/*endif*/
  }/*endif*/


  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;     
  vinter = 0;

  for (iinter=1; iinter<=ninter; iinter++) {

    if (inter_off[iinter]) {continue;}
    if ((neighbor_list==1) && (neighbor_list_update==0)) {
        if (neighbors[iinter] == 0) {continue;}
    }/*endif*/

    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;

    if ((neighbor_list==1) && (neighbor_list_update==1)) {
        if (r2 < neighbor_list_cutoff2) {neighbors[iinter] = 1;}
        else                            {neighbors[iinter] = 0;}
    }/*endif*/
    
    itype = type_index[iinter];
    if (r2 > r_max2[itype]) {continue;}

    if (hash_table[itype]!=0) {  
        if (hash_table[itype]==1) 
           {r       = sqrt(r2);
            hash_r  = r-r_min[itype]; }
        if (hash_table[itype]==2) 
           {hash_r  = r2-r_min2[itype];}
        ihash   = int(hash_r/hash_delta[itype]) + 1;
        ihash   = MIN(ihash,hash_size[itype]);
        vnow    = hash_numeric_V_x[itype][ihash];
        dvnow   = hash_numeric_dV_x[itype][ihash];
        hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
        vnow   += dvnow*hash_m;
        vnow   *= scale_iinter[iinter];
        vinter += vnow;

        continue;
    }/*endif*/

    r  = sqrt(r2);
    ri = 1.0/r;
    if ( implicit_solvent==0 ) {
      qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
      qqs    *= scale_iinter[iinter];
  		
      switch_on = 0;
      if (r >= (r_max[itype]-switch_length)) {
          switch_on=1;
          asw = r_max[itype]-switch_length;
          bsw = r_max[itype];
      }
      if (switch_on==1) {
          vnow    =   qqs*ri*switch_right(r,asw,bsw,switch_type);
      }
      else {
          vnow    =  qqs*ri; 
      } 

      if (ddd>0) {
         eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
         vnow /= eps_ddd; 
      }

      vinter += vnow;
      vcoul  += vnow; 

    }/*end-implicit_solvent==0*/

    if (type[itype]==0) { 
      vinter += 0;
    } else if (type[itype]==1 || type[itype]==2) {
      rs      = r+(r0[itype]-sig[itype]);
      xr      = r0[itype]/rs;
      xxx     = xr*xr*xr*xr*xr*xr;
      vnow    = feps[itype]*xxx*(xxx-beta[itype]);
      vnow   *= scale_iinter[iinter];
      vinter += vnow;
      vlj    += vnow;
    } else if ( type[itype]==3) {
      a   = awill[itype];
      b   = bwill[itype];
      c6  = c6m[itype];
      c8  = c8m[itype];
      c10 = c10m[itype];
      r6  = r2 * r2 * r2;
      r8  = r2 * r2 * r2 * r2;
      r10 = r8 * r2;
      vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
      vnow *= scale_iinter[iinter];
      vinter += vnow;
    } else if ( type[itype]==4) {
      num_size  = numeric_size[itype];
      num_x     = numeric_x[itype];
      num_V_x   = numeric_V_x[itype];
      num_d2V_x = numeric_d2V_x[itype];
      splint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
      vnow     *= scale_iinter[iinter];

      if (switch_on==1) {
        vnowtmp = vnow*switch_right(r,asw,bsw,switch_type);
        vinter += vnowtmp;
        vkb    += vnowtmp;
      } else {                         
        vinter += vnow;
        vkb    += vnowtmp;
      }     

    }
  }
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getInterPot_atomic(const Pos *pos, int iinter)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, num_size, itype;
  int atom1_ind, atom2_ind;
  int switch_on;  

  double dx, dy, dz, r, r2;
  double qqs, ri, rs, xr, xxx;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double vinter;
  double r6, r8, r10;
  double a, b, c6, c8, c10;

  double *num_x, *num_V_x, *num_d2V_x;

  double eps_ddd;

// SWITCH
  double asw, bsw, switch_length;
  int    switch_type;
  switch_type   = 4;
//  switch_length = 1.0/0.529; // 1A in Bohr //
  switch_length = rswitch_length; 
// SWITCH

  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0;
  atom1_ind = atom1[iinter];
  atom2_ind = atom2[iinter];
  dx = pos->x[atom1_ind] - pos->x[atom2_ind];
  dy = pos->y[atom1_ind] - pos->y[atom2_ind];
  dz = pos->z[atom1_ind] - pos->z[atom2_ind];
  r2 = dx*dx+dy*dy+dz*dz;
  r  = sqrt(r2);
  ri = 1.0/r;

  if (inter_off[iinter]) {return 0.0;}

  itype = type_index[iinter];

  if (r > r_max[itype]) {return 0.0;}
  
  if ( implicit_solvent==0 ) {
      qqs     = pos->chrg[atom1_ind]*pos->chrg[atom2_ind]*scale[itype]; 		
      qqs    *= scale_iinter[iinter];
// SWITCH
      switch_on = 0;
      if (r >= (r_max[itype]-switch_length)) {
          switch_on=1;
          asw = r_max[itype]-switch_length;
          bsw = r_max[itype];
      }
      if (switch_on==1) {
          vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type);
      }
      else {
          vnow    =  qqs*ri;
      } 
// SWITCH

      if (ddd>0) {
         eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
         vnow /= eps_ddd; 
      }

      vinter += vnow;
      vcoul  += vnow; 
  }/*end-implicit_solvent==0*/

  if (type[itype]==0) { 
    vinter += 0;
  } else if (type[itype]==1 || type[itype]==2) {
    rs      = r+(r0[itype]-sig[itype]);
    xr      = r0[itype]/rs;
    xxx     = xr*xr*xr*xr*xr*xr;
    vnow    = feps[itype]*xxx*(xxx-beta[itype]);
    vnow   *= scale_iinter[iinter];
    vinter += vnow;
    vlj    += vnow;
  } else if ( type[itype]==3) {
    a   = awill[itype];
    b   = bwill[itype];
    c6  = c6m[itype];
    c8  = c8m[itype];
    c10 = c10m[itype];
    r6  = r2 * r2 * r2;
    r8  = r2 * r2 * r2 * r2;
    r10 = r8 * r2;
    vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
    vnow *= scale_iinter[iinter];
    vinter += vnow;
  } else if ( type[itype]==4) {
    num_size  = numeric_size[itype];
    num_x     = numeric_x[itype];
    num_V_x   = numeric_V_x[itype];
    num_d2V_x = numeric_d2V_x[itype];
    splint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
    vnow     *= scale_iinter[iinter];
// SWITCH
    if (switch_on==1) {
      vnowtmp =  vnow*switch_right(r,asw,bsw,switch_type);
      vinter +=  vnowtmp;
      vkb    +=  vnowtmp; 
    } else {                         
      vinter += vnow;
      vkb    += vnow;
    }     
// SWITCH
  }
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
    
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getPotPermute_atomic(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, iinter, itype, jinter, kinter, num_size;
  int atom1_ind, atom2_ind, atom1_pind, atom2_pind;
  int *permute;  
  int switch_on;

  double *x, *y, *z, *fx, *fy, *fz, *q;
  double dx, dy, dz, r, r2;
  double *num_x, *num_V_x, *num_d2V_x;
  double qqs, ri, rs, xr, xxx;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double vinter;
  double r6, r8, r10;
  double a, b, c6, c8, c10;

  double eps_ddd;

// SWITCH
  double asw, bsw, switch_length;
  int    switch_type;
  switch_type   = 4;
//  switch_length = 1.0/0.529; // 1A in Bohr //
  switch_length = rswitch_length; 
// SWITCH

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  q  = pos->chrg;
  permute = pos->permute_seq;

#define DEBUG_OFF
#ifdef DEBUG
  printf(" From Inter::getPotPermute \n");
  printf(" Checkpoint I\n");
  for (i=1;i<=pos->natom;i++) {
    printf(" permute[%d] = %d\n",i,permute[i]);
  }
  getchar();
#endif
  
  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0;
  for (kinter=1; kinter<=ninter; kinter++) {
 
    if (inter_off[kinter]) {continue;}

    atom1_ind = atom1[kinter];
    atom2_ind = atom2[kinter];
    atom1_pind = permute[atom1_ind];
    atom2_pind = permute[atom2_ind];

#ifdef DEBUG
  printf(" From Inter::getPotPermute \n");
  printf(" Checkpoint II\n");
  for (i=1;i<=pos->natom;i++) {
    printf(" permute[%d] = %d\n",i,permute[i]);
  }
  printf("atom1_ind %d atom2_ind %d\n",atom1_ind,atom2_ind);
  getchar();
#endif

  if (inter_index[atom1_pind][atom2_pind]==0) {
     iinter = inter_index[atom2_pind][atom1_pind];
  } else {
     iinter = inter_index[atom1_pind][atom2_pind];
  }

#ifdef DEBUG
  printf(" From Inter::getPotPermute \n");
  printf(" Checkpoint III\n");
  for (i=1;i<=pos->natom;i++) {
    printf(" permute[%d] = %d\n",i,permute[i]);
  }
  printf("atom1_ind %d atom2_ind %d\n",atom1_ind,atom2_ind); 
  printf("iinter %d \n",iinter); getchar();
  printf("iinter %d scale[%d] %g\n",iinter,iinter,scale[iinter]);
  getchar();
#endif

    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    r  = sqrt(r2);
    ri = 1.0/r;    

    itype = type_index[iinter];
    if (r > r_max[itype]) {continue;}

    if ( implicit_solvent==0 ) {
      qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];  		
      qqs    *= scale_iinter[iinter];
// SWITCH
      switch_on = 0;
      if (r >= (r_max[itype]-switch_length)) {
          switch_on=1;
          asw = r_max[itype]-switch_length;
          bsw = r_max[itype];
      }
      if (switch_on==1) {
          vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type); 
      }
      else {
          vnow    = qqs*ri;
      } 
// SWITCH

      if (ddd>0) {
         eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
         vnow /= eps_ddd; 
      }

      vinter += vnow;
      vcoul  += vnow; 

    }/*end-implicit_solvent==0*/
    if (type[itype]==0) { 
      vinter += 0;
    } else if (type[itype]==1 || type[itype]==2) {
      rs      = r+(r0[itype]-sig[itype]);
      xr      = r0[itype]/rs;
      xxx     = xr*xr*xr*xr*xr*xr;
      vnow    = feps[itype]*xxx*(xxx-beta[itype]);
      vnow   *= scale_iinter[iinter];
      vinter += vnow;
    } else if ( type[itype]==3) {
      a   = awill[itype];
      b   = bwill[itype];
      c6  = c6m[itype];
      c8  = c8m[itype];
      c10 = c10m[itype];
      r6  = r2 * r2 * r2;
      r8  = r2 * r2 * r2 * r2;
      r10 = r8 * r2;
      vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
      vnow *= scale_iinter[iinter];
      vinter += vnow;
      vlj    += vnow;
    } else if ( type[itype]==4) {
      num_size  = numeric_size[itype];
      num_x     = numeric_x[itype];
      num_V_x   = numeric_V_x[itype];
      num_d2V_x = numeric_d2V_x[itype];
      splint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
      vnow   *= scale_iinter[iinter];
// SWITCH
      if (switch_on==1) {
        vnowtmp = vnow*switch_right(r,asw,bsw,switch_type); 
        vinter += vnowtmp;
        vkb    += vnowtmp;
      } else {                         
        vinter += vnow;
        vkb    += vnow;
      }     
// SWITCH
    }
  }
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*=========================================================================*/
void Inter::neighbor_list_check(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm;
  double dispmx;
  
  double *x, *y, *z;
  double *x0, *y0, *z0;

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;

  x0  = pos->x0;
  y0  = pos->y0;
  z0  = pos->z0;

  dispmx = 0.0;
  for (iatm=1;iatm<=pos->natom;iatm++) {
       dispmx = MAX(fabs(x[iatm]-x0[iatm]),dispmx); 
       dispmx = MAX(fabs(y[iatm]-y0[iatm]),dispmx); 
       dispmx = MAX(fabs(z[iatm]-z0[iatm]),dispmx); 
  }/*endfor*/

  dispmx = 2.0*sqrt(3.0*dispmx*dispmx);

  if (dispmx > neighbor_list_skin) {neighbor_list_update = 1;}
  else                             {neighbor_list_update = 0;} 
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*=========================================================================*/
void Inter::conf_save(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm;
  double dispmx;
  
  double *x, *y, *z;
  double *x0, *y0, *z0;

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;

  x0  = pos->x0;
  y0  = pos->y0;
  z0  = pos->z0;

  for (iatm=1;iatm<=pos->natom;iatm++) {
       x0[iatm] = x[iatm]; 
       y0[iatm] = y[iatm]; 
       z0[iatm] = z[iatm]; 
  }/*endfor*/

  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

