#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
/*=========================================================================*/
double Inter::energy(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
  double vinter;

  vinter = 0.0;

/*-------------------------------------------------------------------------*/
/* 0) Return if no inter term                                              */

  if (energy_term != 1) {
      ecoul  = 0.0;
      ekb    = 0.0;
      elj    = 0.0;
      escp   = 0.0;
      einter = 0.0;
      return vinter;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Evaluate energy                                                      */

  if (nprw == 1) {
     if ((hash_nprw_all == 1) && (hash_all == 1)) {
       vinter += energy_atomic_nonpair_hash(pos);
     } else {
       vinter += energy_atomic_nonpair(pos);
     }/*endif*/
  } else {
     if (hash_all == 1) {
       vinter += energy_atomic_hash(pos);
     } else {
       vinter += energy_atomic(pos);
     }/*endif*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* II) Evaluate implicit solvent energy                                    */

  if ( implicit_solvent == 1 ) {
       vinter += energy_scp(pos);
  }

  return vinter;
    
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getPot(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  double vinter;

  vinter = 0.0;

/*-------------------------------------------------------------------------*/
/* 0) Return if no inter term                                              */

  if (energy_term != 1) {
      ecoul  = 0.0;
      ekb    = 0.0;
      elj    = 0.0;
      escp   = 0.0;
      einter = 0.0;
      return vinter;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Evaluate energy                                                      */

  if (nprw == 1) {
    if ((hash_nprw_all == 1) && (hash_all == 1)) {
      vinter += getPot_atomic_nonpair_hash(pos);
    } else {
      vinter += getPot_atomic_nonpair(pos);
    }/*endif*/
  } else {
    if (hash_all == 1) {
      vinter += getPot_atomic_hash(pos);
    } else {
      vinter += getPot_atomic(pos);
    }/*endif*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* II) Evaluate implicit solvent energy                                    */

  if ( implicit_solvent == 1 ) {
       vinter += getPot_scp(pos);
  }

  return vinter;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getInterPot(const Pos *pos, int iinter)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  double vinter;

  char err_msg[LINE_NCHARMAX];

  vinter = 0.0;

  vinter += getInterPot_atomic(pos,iinter);

  if ( implicit_solvent == 1 ) {  
   sprintf(err_msg,"Inter::getInterPot->implicit_solvent not supported");
   errorMessage(err_msg);
  }

  return vinter;
    
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getPotPermute(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  double vinter;

  char err_msg[LINE_NCHARMAX];

  vinter = 0.0;

  vinter += getPotPermute_atomic(pos);

  if ( implicit_solvent == 1 ) {  
   sprintf(err_msg,"Inter::getPotPermute->implicit_solvent not supported");
   errorMessage(err_msg);
  }

  return vinter;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
