#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
double Inter::energy_atomic_hash(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, itype, num_size;
  int iinter, iinter_neighbors;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz;
  double *num_x, *num_V_x, *num_d2V_x;
  double dx, dy, dz, r, r2;
  double pre, vinter;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double asw, bsw, switch_length, rswt;
  int    switch_type;
  switch_type   = 4;
  switch_length = rswitch_length; 

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;

/*-------------------------------------------------------------------------*/
/* I) Linked list inter evaluation                                         */
  
  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
              for (i=1;i<=excl_num[jatm];i++) 
                  {if (excl_list[i][jatm]==iatm) {valid=0;}}
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];    
                 itype = type_index[iinter];
                 if (r2 <= r_max2[itype]) {

                     if (hash_table[itype]==1)
                        {r       = sqrt(r2);
                         hash_r  = r-r_min[itype];  }
                     if (hash_table[itype]==2)
                        {hash_r  = r2-r_min2[itype];}
                     ihash   = int(hash_r/hash_delta[itype]) + 1;
                     ihash   = MIN(ihash,hash_size[itype]);
                     vnow    = hash_numeric_V_x[itype][ihash];
                     dvnow   = hash_numeric_dV_x[itype][ihash];
                     hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
                     vnow   += dvnow*hash_m;
                     pre     = hash_numeric_pre_x[itype][ihash];
                     vnow   *= scale_iinter[iinter];
                     pre    *= scale_iinter[iinter];
		     vinter +=  vnow;                    

                     fx[atom1_ind] += pre*dx;
                     fy[atom1_ind] += pre*dy;
                     fz[atom1_ind] += pre*dz;
                     fx[atom2_ind] -= pre*dx;
                     fy[atom2_ind] -= pre*dy;
                     fz[atom2_ind] -= pre*dz;
                 }/*end if r <= r_max[itype]*/
              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/
      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif-lnk_list*/


/*-------------------------------------------------------------------------*/
/* II) Neighbor list inter evaluation                                      */

 if (neighbor_list == 1) {
  neighbor_list_check(pos);
  if (neighbor_list_update==1) {
      conf_save(pos);
  }/*endif*/
  vcoul = 0.0;
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0.0;
  if (neighbor_list_update==1) {
     ninter_neighbors = 0;
     for (iinter=1; iinter<=ninter; iinter++) {
       if (inter_off[iinter]) {continue;}
       atom1_ind = atom1[iinter];
       atom2_ind = atom2[iinter];
       dx = x[atom1_ind] - x[atom2_ind];
       dy = y[atom1_ind] - y[atom2_ind];
       dz = z[atom1_ind] - z[atom2_ind];
       r2 = dx*dx+dy*dy+dz*dz;      
       if (r2 < neighbor_list_cutoff2) 
	   {ninter_neighbors++;
	    neighbors[ninter_neighbors] = iinter;}
       itype = type_index[iinter];
       if (r2 > r_max2[itype]) {continue;}
           if (hash_table[itype]==1) 
              {r       = sqrt(r2);
               hash_r  = r-r_min[itype]; }
           if (hash_table[itype]==2) 
              {hash_r  = r2-r_min2[itype];}
           ihash   = int(hash_r/hash_delta[itype]) + 1;
           ihash   = MIN(ihash,hash_size[itype]);
           vnow    = hash_numeric_V_x[itype][ihash];
           dvnow   = hash_numeric_dV_x[itype][ihash];
           hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
           vnow   += dvnow*hash_m;
           pre     = hash_numeric_pre_x[itype][ihash];
           vnow   *= scale_iinter[iinter];
           pre    *= scale_iinter[iinter];
           vinter += vnow;                    
   
           fx[atom1_ind] += pre*dx;
           fy[atom1_ind] += pre*dy;
           fz[atom1_ind] += pre*dz;
           fx[atom2_ind] -= pre*dx;
           fy[atom2_ind] -= pre*dy;
           fz[atom2_ind] -= pre*dz;
     }/*endfor-iinter*/
  }/*endif-neighbor_list_update==1*/

  if (neighbor_list_update==0) {
     for (iinter_neighbors=1;iinter_neighbors<=ninter_neighbors;
                                               iinter_neighbors++) {
       iinter = neighbors[iinter_neighbors];
       if (inter_off[iinter]) {continue;}
       atom1_ind = atom1[iinter];
       atom2_ind = atom2[iinter];
       dx = x[atom1_ind] - x[atom2_ind];
       dy = y[atom1_ind] - y[atom2_ind];
       dz = z[atom1_ind] - z[atom2_ind];
       r2 = dx*dx+dy*dy+dz*dz;      

       itype = type_index[iinter];
       if (r2 > r_max2[itype]) {continue;}
           if (hash_table[itype]==1) 
              {r       = sqrt(r2);
               hash_r  = r-r_min[itype]; }
           if (hash_table[itype]==2) 
              {hash_r  = r2-r_min2[itype];}
           ihash   = int(hash_r/hash_delta[itype]) + 1;
           ihash   = MIN(ihash,hash_size[itype]);
           vnow    = hash_numeric_V_x[itype][ihash];
           dvnow   = hash_numeric_dV_x[itype][ihash];
           hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
           vnow   += dvnow*hash_m;
           pre     = hash_numeric_pre_x[itype][ihash];
           vnow   *= scale_iinter[iinter];
           pre    *= scale_iinter[iinter];
           vinter += vnow;                    
    
           fx[atom1_ind] += pre*dx;
           fy[atom1_ind] += pre*dy;
           fz[atom1_ind] += pre*dz;
           fx[atom2_ind] -= pre*dx;
           fy[atom2_ind] -= pre*dy;
           fz[atom2_ind] -= pre*dz;
     }/*endfor-iinter*/
  }/*endif-neighbor_list_update==0*/
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
 }/*endif-neighbor_list*/    


/*-------------------------------------------------------------------------*/
/* III) No list inter evaluation                                           */

  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0.0;
  for (iinter=1; iinter<=ninter; iinter++) {
    if (inter_off[iinter]) {continue;}
    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    itype = type_index[iinter];
    if (r2 > r_max2[itype]) {continue;}
        if (hash_table[itype]==1) 
           {r       = sqrt(r2);
            hash_r  = r-r_min[itype]; }
        if (hash_table[itype]==2) 
           {hash_r  = r2-r_min2[itype];}
        ihash   = int(hash_r/hash_delta[itype]) + 1;
        ihash   = MIN(ihash,hash_size[itype]);
        vnow    = hash_numeric_V_x[itype][ihash];
        dvnow   = hash_numeric_dV_x[itype][ihash];
        hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
        vnow   += dvnow*hash_m;
        pre     = hash_numeric_pre_x[itype][ihash];
        vnow   *= scale_iinter[iinter];
        pre    *= scale_iinter[iinter];
        vinter += vnow;                    

        fx[atom1_ind] += pre*dx;
        fy[atom1_ind] += pre*dy;
        fz[atom1_ind] += pre*dz;
        fx[atom2_ind] -= pre*dx;
        fy[atom2_ind] -= pre*dy;
        fz[atom2_ind] -= pre*dz;
  }/*endfor-iinter*/
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getPot_atomic_hash(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, itype, num_size;
  int iinter, iinter_neighbors;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz;
  double *num_x, *num_V_x, *num_d2V_x;
  double dx, dy, dz, r, r2;
  double pre, vinter;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double asw, bsw, switch_length, rswt;
  int    switch_type;
  switch_type   = 4;
  switch_length = rswitch_length; 

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;

/*-------------------------------------------------------------------------*/
/* I) Linked list inter evaluation                                         */
  
  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
              for (i=1;i<=excl_num[jatm];i++) 
                  {if (excl_list[i][jatm]==iatm) {valid=0;}}
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];    
                 itype = type_index[iinter];
                 if (r2 <= r_max2[itype]) {

                     if (hash_table[itype]==1)
                        {r       = sqrt(r2);
                         hash_r  = r-r_min[itype];  }
                     if (hash_table[itype]==2)
                        {hash_r  = r2-r_min2[itype];}
                     ihash   = int(hash_r/hash_delta[itype]) + 1;
                     ihash   = MIN(ihash,hash_size[itype]);
                     vnow    = hash_numeric_V_x[itype][ihash];
                     dvnow   = hash_numeric_dV_x[itype][ihash];
                     hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
                     vnow   += dvnow*hash_m;
                     vnow   *= scale_iinter[iinter];
		     vinter +=  vnow;                    

                 }/*end if r <= r_max[itype]*/
              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/
      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif-lnk_list*/


/*-------------------------------------------------------------------------*/
/* II) Neighbor list inter evaluation                                      */

 if (neighbor_list == 1) {
  neighbor_list_check(pos);
  if (neighbor_list_update==1) {
      conf_save(pos);
  }/*endif*/
  vcoul = 0.0;
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0.0;
  if (neighbor_list_update==1) {
     ninter_neighbors = 0;
     for (iinter=1; iinter<=ninter; iinter++) {
       if (inter_off[iinter]) {continue;}
       atom1_ind = atom1[iinter];
       atom2_ind = atom2[iinter];
       dx = x[atom1_ind] - x[atom2_ind];
       dy = y[atom1_ind] - y[atom2_ind];
       dz = z[atom1_ind] - z[atom2_ind];
       r2 = dx*dx+dy*dy+dz*dz;      
       if (r2 < neighbor_list_cutoff2) 
	   {ninter_neighbors++;
	    neighbors[ninter_neighbors] = iinter;}
       itype = type_index[iinter];
       if (r2 > r_max2[itype]) {continue;}
           if (hash_table[itype]==1) 
              {r       = sqrt(r2);
               hash_r  = r-r_min[itype]; }
           if (hash_table[itype]==2) 
              {hash_r  = r2-r_min2[itype];}
           ihash   = int(hash_r/hash_delta[itype]) + 1;
           ihash   = MIN(ihash,hash_size[itype]);
           vnow    = hash_numeric_V_x[itype][ihash];
           dvnow   = hash_numeric_dV_x[itype][ihash];
           hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
           vnow   += dvnow*hash_m;
           vnow   *= scale_iinter[iinter];
           vinter += vnow;
     }/*endfor-iinter*/
  }/*endif-neighbor_list_update==1*/

  if (neighbor_list_update==0) {
     for (iinter_neighbors=1;iinter_neighbors<=ninter_neighbors;
                                               iinter_neighbors++) {
       iinter = neighbors[iinter_neighbors];
       if (inter_off[iinter]) {continue;}
       atom1_ind = atom1[iinter];
       atom2_ind = atom2[iinter];
       dx = x[atom1_ind] - x[atom2_ind];
       dy = y[atom1_ind] - y[atom2_ind];
       dz = z[atom1_ind] - z[atom2_ind];
       r2 = dx*dx+dy*dy+dz*dz;      

       itype = type_index[iinter];
       if (r2 > r_max2[itype]) {continue;}
           if (hash_table[itype]==1) 
              {r       = sqrt(r2);
               hash_r  = r-r_min[itype]; }
           if (hash_table[itype]==2) 
              {hash_r  = r2-r_min2[itype];}
           ihash   = int(hash_r/hash_delta[itype]) + 1;
           ihash   = MIN(ihash,hash_size[itype]);
           vnow    = hash_numeric_V_x[itype][ihash];
           dvnow   = hash_numeric_dV_x[itype][ihash];
           hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
           vnow   += dvnow*hash_m;
           vnow   *= scale_iinter[iinter];
           vinter += vnow;
     }/*endfor-iinter*/
  }/*endif-neighbor_list_update==0*/
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
 }/*endif-neighbor_list*/    


/*-------------------------------------------------------------------------*/
/* III) No list inter evaluation                                           */

  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0.0;
  for (iinter=1; iinter<=ninter; iinter++) {
    if (inter_off[iinter]) {continue;}
    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    itype = type_index[iinter];
    if (r2 > r_max2[itype]) {continue;}
        if (hash_table[itype]==1) 
           {r       = sqrt(r2);
            hash_r  = r-r_min[itype]; }
        if (hash_table[itype]==2) 
           {hash_r  = r2-r_min2[itype];}
        ihash   = int(hash_r/hash_delta[itype]) + 1;
        ihash   = MIN(ihash,hash_size[itype]);
        vnow    = hash_numeric_V_x[itype][ihash];
        dvnow   = hash_numeric_dV_x[itype][ihash];
        hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
        vnow   += dvnow*hash_m;
        vnow   *= scale_iinter[iinter];
        vinter += vnow;
  }/*endfor-iinter*/
  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

