#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/* This files contains inter energy related routines that are called if    */
/* the total energy of the system has any non pairwise energy components   */
/* Written and maintained by Sam Nakagawa                      07/29/09    */

/*=========================================================================*/
/*=========================================================================*/
double Inter::energy_atomic_nonpair(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, iinter, itype, num_size;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz, *q;
  double *num_x, *num_V_x, *num_d2V_x;
  double dx, dy, dz, r, r2;
  double qqs, ri, rs, xr, xxx;
  double pre, vinter;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double r6, r8, r10;
  double a, b, c6, c8, c10;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double eps_ddd, deps_ddd;

  double asw, bsw, switch_length, rswt;
  int    switch_type;
  switch_type   = 4;
  switch_length = rswitch_length; 

  for (int i = 1; i <= natom; i++) {
    sumRho[i] = 0;
    for (int j = 1; j <= natom; j++) {
      TX[i][j] = 0.0;
      TY[i][j] = 0.0;
      TZ[i][j] = 0.0;
    }
  }

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;
  q  = pos->chrg;
  
  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
              for (i=1;i<=excl_num[jatm];i++) 
                  {if (excl_list[i][jatm]==iatm) {valid=0;}}
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];    
                 itype = type_index[iinter];
                 if (r2 <= r_max2[itype]) {
                   if (hash_table[itype]!=0) {
                     if (hash_table[itype]==1)
                        {r       = sqrt(r2);
                         hash_r  = r-r_min[itype];  }
                     if (hash_table[itype]==2)
                        {hash_r  = r2-r_min2[itype];}
                     ihash   = int(hash_r/hash_delta[itype]) + 1;
                     ihash   = MIN(ihash,hash_size[itype]);
                     vnow    = hash_numeric_V_x[itype][ihash];
                     dvnow   = hash_numeric_dV_x[itype][ihash];
                     hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
                     vnow   += dvnow*hash_m;
                     pre     = hash_numeric_pre_x[itype][ihash];
                     vnow   *= scale_iinter[iinter];
                     pre    *= scale_iinter[iinter];
		     vinter +=  vnow;                    
                   }
                   else {
                     r  = sqrt(r2);
                     ri = 1.0/r;
		     if ( implicit_solvent==0 ) {
                      qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
                      qqs    *= scale_iinter[iinter];

                      switch_on = 0;
                      if (r >= (r_max[itype]-switch_length)) {
                          switch_on=1;
                          asw = r_max[itype]-switch_length;
                          bsw = r_max[itype];
                      }
                      if (switch_on==1) {
                          vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type);
                          pre  = qqs*switch_right(r,asw,bsw,switch_type)/(r*r2);
                          pre += -qqs*dx_switch_right(r,asw,bsw,switch_type)/r2;
                      }
                      else {
                          vnow    = qqs*ri;
                          pre     = qqs/(r*r2);
                      }

                      if (ddd>0) {
                        eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
                        deps_ddd = ddd_deps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
                        pre   = pre/eps_ddd 
		              + vnow*deps_ddd/(r*eps_ddd*eps_ddd);
                        vnow /= eps_ddd; 
                      } 

                      vinter += vnow;
                      vcoul  += vnow;                        

                     } else {
                          pre=0.0;
                     }/*endif-implicit*/
                     if (type[itype]==0) { 
                         vinter += 0;
                         pre    += 0;
                     } else if (type[itype]==1 || type[itype]==2) {
                         rs      = r+(r0[itype]-sig[itype]);
                         xr      = r0[itype]/rs;
                         xxx     = xr*xr*xr*xr*xr*xr;
                         vnow    = feps[itype]*xxx*(xxx-beta[itype]);
                         dvnow   = 6.0*feps[itype]*xxx
                                 *(2.0*xxx-beta[itype])/(r*rs);
                         vnow   *= scale_iinter[iinter];
                         dvnow  *= scale_iinter[iinter];
                         vinter += vnow;
                         vlj    += vnow;
                         pre    += dvnow;
                     } else if (type[itype]==3) {
                         a   = awill[itype];
                         b   = bwill[itype];
                         c6  = c6m[itype];
                         c8  = c8m[itype];
                         c10 = c10m[itype];
                         r6  = r2 * r2 * r2;
                         r8  = r2 * r2 * r2 * r2;
                         r10 = r8 * r2;
                         vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
                         dvnow = a*b*exp(-b*r)/r 
                               - (c6 * 6.0/r6 + c8*8.0/r8 + c10*10.0/r10)/r2;
                         vnow   *= scale_iinter[iinter];
                         dvnow  *= scale_iinter[iinter];
                         vinter += vnow;
                         pre    += dvnow;
                     } else if (type[itype]==4) {
                         num_size  = numeric_size[itype];
                         num_x     = numeric_x[itype];
                         num_V_x   = numeric_V_x[itype];
                         num_d2V_x = numeric_d2V_x[itype];

                         int kf;
                         nsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
                                 _cache_phi, kf, r, &vnow);
                         _cache_phi = kf;
                         ndsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
                                  kf, kf, r, &dvnow);
                         vnow   *= scale_iinter[iinter];
                         dvnow  *= scale_iinter[iinter];

                         // Computes additional non-pairwise term
                         int ifunc = 1; // This is for rho.
                         int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[iinter]];
                         double uConversion = BOLTZ/BOHR;
                         _populate_density(ifunc_nprw, r,atom1_ind,atom2_ind, uConversion);
                         _populate_dvdensity(ifunc_nprw, r, dx, dy, dz,
                           atom1_ind, atom2_ind, uConversion);

                         if (switch_on==1) {
                           vnowtmp = vnow*switch_right(r,asw,bsw,switch_type);
                           vinter += vnowtmp;
                           vkb    += vnowtmp;
                           pre += -dvnow*switch_right(r,asw,bsw,switch_type)/r;
                           pre += -vnow*dx_switch_right(r,asw,bsw,switch_type)/r;
                         } else {                         
                           vinter +=  vnow;
                           vkb    +=  vnow;
                           pre    += -dvnow/r;
                         }     
		     
                     }/*end if type[itype]==*/
		   }/*endif-hash-no-hash*/
                     fx[atom1_ind] += pre*dx;
                     fy[atom1_ind] += pre*dy;
                     fz[atom1_ind] += pre*dz;
                     fx[atom2_ind] -= pre*dx;
                     fy[atom2_ind] -= pre*dy;
                     fz[atom2_ind] -= pre*dz;
                 }/*end if r <= r_max[itype]*/
              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/

      // Computes additional non-pairwise term
      double sumU = _get_sum_of_glue();
      vinter += sumU;
      _add_force_from_glue(pos);

      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif*/

  if (neighbor_list == 1) {
      neighbor_list_check(pos);
      if (neighbor_list_update==1) {
	  conf_save(pos);
      }/*endif*/
  }/*endif*/

  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0;
  for (iinter=1; iinter<=ninter; iinter++) {

    if (inter_off[iinter]) {continue;}
    if ((neighbor_list==1) && (neighbor_list_update==0)) {
        if (neighbors[iinter] == 0) {continue;}
    }/*endif*/

    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    if ((neighbor_list==1) && (neighbor_list_update==1)) {
        if (r2 < neighbor_list_cutoff2) {neighbors[iinter] = 1;}
        else                            {neighbors[iinter] = 0;}
    }/*endif*/

    itype = type_index[iinter];
    if (r2 > r_max2[itype]) {continue;}

    if (hash_table[itype]!=0) {
        if (hash_table[itype]==1) 
           {r       = sqrt(r2);
            hash_r  = r-r_min[itype]; }
        if (hash_table[itype]==2) 
           {hash_r  = r2-r_min2[itype];}
        ihash   = int(hash_r/hash_delta[itype]) + 1;
        ihash   = MIN(ihash,hash_size[itype]);
        vnow    = hash_numeric_V_x[itype][ihash];
        dvnow   = hash_numeric_dV_x[itype][ihash];
        hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
        vnow   += dvnow*hash_m;
        pre     = hash_numeric_pre_x[itype][ihash];
        vnow   *= scale_iinter[iinter];
        pre    *= scale_iinter[iinter];
        vinter += vnow;                    

        fx[atom1_ind] += pre*dx;
        fy[atom1_ind] += pre*dy;
        fz[atom1_ind] += pre*dz;
        fx[atom2_ind] -= pre*dx;
        fy[atom2_ind] -= pre*dy;
        fz[atom2_ind] -= pre*dz;

        continue;
    }/*endif*/

    r  = sqrt(r2);
    ri = 1.0/r;    
    if ( implicit_solvent==0 ) {
      qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
      qqs    *= scale_iinter[iinter];

      switch_on = 0;
      if (r >= (r_max[itype]-switch_length)) {
          switch_on=1;
          asw = r_max[itype]-switch_length;
          bsw = r_max[itype];
      }
      if (switch_on==1) {
          vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type);
          pre =  qqs*switch_right(r,asw,bsw,switch_type)/(r*r2);
          pre += -qqs*dx_switch_right(r,asw,bsw,switch_type)/r2;
      }
      else {
          vnow    = qqs*ri;
          pre     = qqs/(r*r2);
      }/*endif-switch_on*/ 

      if (ddd>0) {
          eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
          deps_ddd = ddd_deps(r,ddd_p1,ddd_p2,ddd_p3,ddd);    
          pre   = pre/eps_ddd 
	            + vnow*deps_ddd/(r*eps_ddd*eps_ddd);
          vnow /= eps_ddd;
      }/*endif-ddd*/

      vinter += vnow;
      vcoul  += vnow; 

    } else {
          pre=0.0;
    }/*endif-implicit*/

    if (type[itype]==0) { 
      vinter += 0;
      pre    += 0;
    } else if (type[itype]==1 || type[itype]==2) {
      rs      = r+(r0[itype]-sig[itype]);
      xr      = r0[itype]/rs;
      xxx     = xr*xr*xr*xr*xr*xr;
      vnow    = feps[itype]*xxx*(xxx-beta[itype]);
      dvnow   = 6.0*feps[itype]*xxx*(2.0*xxx-beta[itype])/(r*rs);
      vnow   *= scale_iinter[iinter];
      dvnow  *= scale_iinter[iinter];
      vinter += vnow;
      vlj    += vnow;
      pre    += dvnow;
    } else if ( type[itype]==3) {
      a   = awill[itype];
      b   = bwill[itype];
      c6  = c6m[itype];
      c8  = c8m[itype];
      c10 = c10m[itype];
      r6  = r2 * r2 * r2;
      r8  = r2 * r2 * r2 * r2;
      r10 = r8 * r2;
      vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
      dvnow = a*b*exp(-b*r)/r - (c6 * 6.0/r6 + c8*8.0/r8 + c10*10.0/r10)/r2;
      vnow  *= scale_iinter[iinter];
      dvnow *= scale_iinter[iinter];
      vinter += vnow;
      pre    += dvnow;
    } else if ( type[itype]==4) {
      num_size  = numeric_size[itype];
      num_x     = numeric_x[itype];
      num_V_x   = numeric_V_x[itype];
      num_d2V_x = numeric_d2V_x[itype];

      int kf;
      nsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
              _cache_phi, kf, r, &vnow);
      _cache_phi = kf;
      ndsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
              kf, kf, r, &dvnow);
      vnow   *= scale_iinter[iinter];
      dvnow  *= scale_iinter[iinter];

      // Computes additional non-pairwise term
      int ifunc = 1; // This is for rho.
      int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[iinter]];
      double uConversion = BOLTZ/BOHR;
      _populate_density(ifunc_nprw, r,atom1_ind,atom2_ind,uConversion);
      _populate_dvdensity(ifunc_nprw, r, dx, dy, dz,
                          atom1_ind, atom2_ind, uConversion);

      if (switch_on==1) {
        vnowtmp =  vnow*switch_right(r,asw,bsw,switch_type);
        vinter +=  vnowtmp;
        vkb    +=  vnowtmp;
        pre += -dvnow*switch_right(r,asw,bsw,switch_type)/r;
        pre += -vnow*dx_switch_right(r,asw,bsw,switch_type)/r;
      } else {                         
        vinter +=  vnow;
        vkb    +=  vnow;
	pre    += -dvnow/r;
      }/*endif-switch_on*/     
    }/*endif-type[itype]*/
    fx[atom1_ind] += pre*dx;
    fy[atom1_ind] += pre*dy;
    fz[atom1_ind] += pre*dz;
    fx[atom2_ind] -= pre*dx;
    fy[atom2_ind] -= pre*dy;
    fz[atom2_ind] -= pre*dz;
  }/*endfor-iinter*/

  // Computes additional non-pairwise term
  double sumU = _get_sum_of_glue();
  vinter += sumU;
  _add_force_from_glue(pos);

  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
    
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getPot_atomic_nonpair(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, iinter, itype, num_size;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz, *q;
  double dx, dy, dz, r, r2;
  double *num_x, *num_V_x, *num_d2V_x;
  double qqs, ri, rs, xr, xxx;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double vinter;
  double r6, r8, r10;
  double a, b, c6, c8, c10;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double eps_ddd;

  double asw, bsw, switch_length;
  int    switch_type;
  switch_type   = 4;

  switch_length = rswitch_length; 

  double         rhoNow = 0.0;            // temp accumulator for rho
  double         uNow = 0.0;              // temp accumulator for hNow
  for (int i = 1; i <= natom; i++) {
    sumRho[i] = 0;
  }

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  q  = pos->chrg;

  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;
               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
 
              for (i=1;i<=excl_num[jatm];i++) {                   
                  if (excl_list[i][jatm]==iatm) {valid=0;}
              }
 
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];
                 itype = type_index[iinter];
    
                if (r2 <= r_max2[itype]) {
                 if (hash_table[itype]!=0) {
                     if (hash_table[itype]==1) 
                        {r       = sqrt(r2);
                         hash_r  = r-r_min[itype]; }
                     if (hash_table[itype]==2) 
                        {hash_r  = r2-r_min2[itype];}
                   ihash   = int(hash_r/hash_delta[itype]) + 1;
                   ihash   = MIN(ihash,hash_size[itype]);
                   vnow    = hash_numeric_V_x[itype][ihash];
                   dvnow   = hash_numeric_dV_x[itype][ihash];
                   hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
                   vnow   += dvnow*hash_m;
                   vnow   *= scale_iinter[iinter];
 		   vinter +=  vnow;                    
                 }
                 else {
                  r  = sqrt(r2);
                  ri = 1.0/r;

                  if ( implicit_solvent==0 ) {
                    qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
                    qqs    *= scale_iinter[iinter];
      			
                    switch_on = 0;
                    if (r >= (r_max[itype]-switch_length)) {
                        switch_on=1;
                        asw = r_max[itype]-switch_length;
                        bsw = r_max[itype];
                    }
                    if (switch_on==1) {
                        vnow    =  qqs*ri*switch_right(r,asw,bsw,switch_type);
                    }
                    else {
                        vnow    =  qqs*ri;
                    }            

                    if (ddd>0) {
                        eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
                        vnow /= eps_ddd; 
                    }

                    vinter += vnow;
                    vcoul  += vnow; 

                  }/*end-implicit_solvent==0*/

                  if (type[itype]==0) { 
                     vinter += 0;
                  } else if (type[itype]==1 || type[itype]==2) {
                     rs      = r+(r0[itype]-sig[itype]);
                     xr      = r0[itype]/rs;
                     xxx     = xr*xr*xr*xr*xr*xr;
                     vnow    = feps[itype]*xxx*(xxx-beta[itype]);
                     vnow   *= scale_iinter[iinter];
                     vinter += vnow;
                     vlj    += vnow;
                  } else if ( type[itype]==3) {
                     a   = awill[itype];
                     b   = bwill[itype];
                     c6  = c6m[itype];
                     c8  = c8m[itype];
                     c10 = c10m[itype];
                     r6  = r2 * r2 * r2;
                     r8  = r2 * r2 * r2 * r2;
                     r10 = r8 * r2;
                     vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
                     vnow *= scale_iinter[iinter]; 
                     vinter += vnow;
                  } else if ( type[itype]==4) {
                     num_size  = numeric_size[itype];
                     num_x     = numeric_x[itype];
                     num_V_x   = numeric_V_x[itype];
                     num_d2V_x = numeric_d2V_x[itype];
                     int kf;
                     nsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
                          _cache_phi, kf, r, &vnow);
                     vnow   *= scale_iinter[iinter];
                     int ifunc = 1; // This is for rho.
                     int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[iinter]];
                     double uConversion = BOLTZ/BOHR;
                     _populate_density(ifunc_nprw, r,atom1_ind,atom2_ind, uConversion);

                     if (switch_on==1) {
                       vnowtmp =  vnow*switch_right(r,asw,bsw,switch_type);
                       vinter +=  vnowtmp;
                       vkb    +=  vnowtmp;
                     } else {                         
                       vinter += vnow;
                       vkb    += vnow;
                     }     

                  }/*end if type[itype]==*/
                 }/*endif-hash-no-hash*/
                }/*end r <= r_max[itype]*/
              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/

      // Computes additional non-pairwise term
      double sumU = _get_sum_of_glue();
      vinter += sumU;

      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif*/


  if (neighbor_list == 1) {
      neighbor_list_check(pos);
      if (neighbor_list_update==1) {
	  conf_save(pos);
      }/*endif*/
  }/*endif*/


  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;     
  vinter = 0;
  for (iinter=1; iinter<=ninter; iinter++) {

    if (inter_off[iinter]) {continue;}
    if ((neighbor_list==1) && (neighbor_list_update==0)) {
        if (neighbors[iinter] == 0) {continue;}
    }/*endif*/

    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    if ((neighbor_list==1) && (neighbor_list_update==1)) {
        if (r2 < neighbor_list_cutoff2) {neighbors[iinter] = 1;}
        else                            {neighbors[iinter] = 0;}
    }/*endif*/
    
    itype = type_index[iinter];
    if (r2 > r_max2[itype]) {continue;}

    if (hash_table[itype]!=0) {  
        if (hash_table[itype]==1) 
           {r       = sqrt(r2);
            hash_r  = r-r_min[itype]; }
        if (hash_table[itype]==2) 
           {hash_r  = r2-r_min2[itype];}
        ihash   = int(hash_r/hash_delta[itype]) + 1;
        ihash   = MIN(ihash,hash_size[itype]);
        vnow    = hash_numeric_V_x[itype][ihash];
        dvnow   = hash_numeric_dV_x[itype][ihash];
        hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
        vnow   += dvnow*hash_m;
        vnow   *= scale_iinter[iinter];
        vinter += vnow;

        continue;
    }/*endif*/

    r  = sqrt(r2);
    ri = 1.0/r;
    if ( implicit_solvent==0 ) {
      qqs     = q[atom1_ind]*q[atom2_ind]*scale[itype];
      qqs    *= scale_iinter[iinter];
  		
      switch_on = 0;
      if (r >= (r_max[itype]-switch_length)) {
          switch_on=1;
          asw = r_max[itype]-switch_length;
          bsw = r_max[itype];
      }
      if (switch_on==1) {
          vnow    =   qqs*ri*switch_right(r,asw,bsw,switch_type);
      }
      else {
          vnow    =  qqs*ri; 
      } 

      if (ddd>0) {
         eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
         vnow /= eps_ddd; 
      }

      vinter += vnow;
      vcoul  += vnow; 

    }/*end-implicit_solvent==0*/

    if (type[itype]==0) { 
      vinter += 0;
    } else if (type[itype]==1 || type[itype]==2) {
      rs      = r+(r0[itype]-sig[itype]);
      xr      = r0[itype]/rs;
      xxx     = xr*xr*xr*xr*xr*xr;
      vnow    = feps[itype]*xxx*(xxx-beta[itype]);
      vnow   *= scale_iinter[iinter];
      vinter += vnow;
      vlj    += vnow;
    } else if ( type[itype]==3) {
      a   = awill[itype];
      b   = bwill[itype];
      c6  = c6m[itype];
      c8  = c8m[itype];
      c10 = c10m[itype];
      r6  = r2 * r2 * r2;
      r8  = r2 * r2 * r2 * r2;
      r10 = r8 * r2;
      vnow  = a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
      vnow *= scale_iinter[iinter];
      vinter += vnow;
    } else if ( type[itype]==4) {
      num_size  = numeric_size[itype];
      num_x     = numeric_x[itype];
      num_V_x   = numeric_V_x[itype];
      num_d2V_x = numeric_d2V_x[itype];

      nsplint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
      vnow     *= scale_iinter[iinter];

      // Computes additional non-pairwise term
      int ifunc = 1; // This is for rho.
      int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[iinter]];
      double uConversion = BOLTZ/BOHR;
      _populate_density(ifunc_nprw, r,atom1_ind,atom2_ind,uConversion);

      if (switch_on==1) {
        vnowtmp = vnow*switch_right(r,asw,bsw,switch_type);
        vinter += vnowtmp;
        vkb    += vnowtmp;
      } else {                         
        vinter += vnow;
        vkb    += vnowtmp;
      }     

    }
  }

  // Computes additional non-pairwise term
  double sumU = _get_sum_of_glue();
  vinter += sumU;

  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::_get_sum_of_glue()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Given a vector of density function for atoms 1..natom,
  // Evaluate the glue function over all atoms, and return the sum.

  double sumU = 0.0;
  int ifunc = 2; // This is for U.
  int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[1]];
  int num_size = nprw_numeric_size[ifunc_nprw];
  double* num_x = nprw_numeric_x[ifunc_nprw];
  double* num_V_x = nprw_numeric_V_x[ifunc_nprw];
  double* num_d2V_x = nprw_numeric_d2V_x[ifunc_nprw];

  for (int i = 1; i <= natom; i++) {
    double uNow = 0.0;
    int kf;
    nsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
            _cache_u, kf, sumRho[i], &uNow);
    _cache_u = kf;
    sumU += uNow;
  }
  return sumU;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Inter::_populate_density(const int       ifunc_nprw,
                              const double    r,
                              const int       atom1_ind,
                              const int       atom2_ind,
                              const double    uConversion)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Given a pair of atoms represented by atom1_ind and atom2_ind,
  // evaluate their density function values and perform summationi.

  int num_size = nprw_numeric_size[ifunc_nprw];
  double* num_x = nprw_numeric_x[ifunc_nprw];
  double* num_V_x = nprw_numeric_V_x[ifunc_nprw];
  double* num_d2V_x = nprw_numeric_d2V_x[ifunc_nprw];
  double rhoNow = 0.0;
  int kf;
  nsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
          _cache_rho, kf, r, &rhoNow);
  _cache_rho = kf;
  sumRho[atom1_ind] += (rhoNow*uConversion); // add rho to atom1
  sumRho[atom2_ind] += (rhoNow*uConversion); // add rho to atom2
  return;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/ 


/*=========================================================================*/
/*=========================================================================*/
void Inter::_populate_dvdensity(const int       ifunc_nprw,
                                const double    r,
                                const double    x,
                                const double    y,
                                const double    z,
                                const int       atom1_ind,
                                const int       atom2_ind,
                                const double    uConversion)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Given a pair of atoms represented by atom1_ind and atom2_ind,
  // compute the derivative values of their density function
  // and store the result TX, TY, and TZ matrices.

  int num_size = nprw_numeric_size[ifunc_nprw];
  double* num_x = nprw_numeric_x[ifunc_nprw];
  double* num_V_x = nprw_numeric_V_x[ifunc_nprw];
  double* num_d2V_x = nprw_numeric_d2V_x[ifunc_nprw];
  double dRho_dr_now = 0.0;
  int kf;
  ndsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
           _cache_rho, kf, r, &dRho_dr_now);
  _cache_rho = kf;
  TX[atom1_ind][atom2_ind] = -dRho_dr_now*uConversion*x/r;
  TX[atom2_ind][atom1_ind] = dRho_dr_now*uConversion*x/r;
  TY[atom1_ind][atom2_ind] = -dRho_dr_now*uConversion*y/r;
  TY[atom2_ind][atom1_ind] = dRho_dr_now*uConversion*y/r;
  TZ[atom1_ind][atom2_ind] = -dRho_dr_now*uConversion*z/r;
  TZ[atom2_ind][atom1_ind] = dRho_dr_now*uConversion*z/r;
  return;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Inter::_add_force_from_glue(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Evaluate force from the glue function part for all atoms.

  int ifunc = 2; // This is for U.
  int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[1]];
  int num_size = nprw_numeric_size[ifunc_nprw];
  double* num_x = nprw_numeric_x[ifunc_nprw];
  double* num_V_x = nprw_numeric_V_x[ifunc_nprw];
  double* num_d2V_x = nprw_numeric_d2V_x[ifunc_nprw];

  // Compute dU/drho for all atoms.
  // Note consider sorting sumRho here.
//  vector<double> dU_drho(natom+1, 0);
//  int ks = 1;
//  int kf;
//  for (int i = 1; i <= natom; i++) {

//    double duNow;
//    ndsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
//             ks, kf, sumRho[i], &duNow);
//    ks = kf;
//    dU_drho[i] = duNow;
//  }

  // Compute fx, fy, and fz for all atoms.
  double duNow = 0.0;
  double* fx = pos->fx;  // get pointers to force
  double* fy = pos->fy;  //
  double* fz = pos->fz;  //
  int ks, kf; 
  ks = 1;
  for (int i = 1; i <= natom; i++) {

    double uix = 0.0;
    double uiy = 0.0;
    double uiz = 0.0;
    double ujx = 0.0;
    double ujy = 0.0;
    double ujz = 0.0;
    for (int j = 1; j <= natom; j++) {

      if (i != j) {

        // For the ith atom, compute the sum of derivatives over all j's except j=i.
        uix += TX[i][j];
        uiy += TY[i][j];
        uiz += TZ[i][j];

        // For the ith aotm, compute the sum of (dU/drho)(drho/dr) for all j's except j=i.
        ndsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
                 ks, kf, sumRho[j], &duNow);
        ks = kf;
        ujx += duNow*TX[i][j];
        ujy += duNow*TY[i][j];
        ujz += duNow*TZ[i][j];
      }
    }

    // For the ith atom, compute the sum of (dU/drho)(drho/dr) for j=i.
    ndsplint(&(num_x[0]), &(num_V_x[0]), &(num_d2V_x[0]), num_size,
             ks, kf, sumRho[i], &duNow);
    ks = kf;
    fx[i] += (duNow*uix + ujx);
    fy[i] += (duNow*uiy + ujy);
    fz[i] += (duNow*uiz + ujz);
  }
  return;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

 
