#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/* This files contains inter energy related routines that are called if    */
/* the total energy of the system has any non pairwise energy components   */
/* and all energy components (pairwise and non-pairwise) are evaluated     */
/* based on a priory calculated numerical tables                           */
/* Written and maintained by Sam Nakagawa                      07/29/09    */


/*=========================================================================*/
/*=========================================================================*/
double Inter::energy_atomic_nonpair_hash(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, iinter, itype, num_size;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz;
  double *num_x, *num_V_x, *num_d2V_x;
  double dx, dy, dz, r, r2;
  double pre, vinter;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double asw, bsw, switch_length, rswt;
  int    switch_type;
  switch_type   = 4;
  switch_length = rswitch_length; 

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;

  // Initialize temporary accumulators  
  for (int i = 1; i <= natom; i++) {
    sumRho[i] = 0;
    for (int j = 1; j <= natom; j++) {
      TX[i][j] = 0.0;
      TY[i][j] = 0.0;
      TZ[i][j] = 0.0;
    }
  }
  double uConversion = BOLTZ/BOHR;
 
  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
              for (i=1;i<=excl_num[jatm];i++) 
                  {if (excl_list[i][jatm]==iatm) {valid=0;}}
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];    
                 itype = type_index[iinter];

                 // First, compute force from pairwise interaction
                 //  if r2 is less than or equal to r_max2
                 if (r2 <= r_max2[itype]) {

                     if (hash_table[itype]==1) {  
                        r       = sqrt(r2);
                        hash_r  = r - r_min[itype];  
                     } else {  
                        hash_r  = r2 - r_min2[itype];
                     }
                     _compute_pairwise_force(hash_r, itype, vinter, atom1_ind, 
                        atom2_ind, iinter, dx, dy, dz, fx, fy, fz); 
                 } // end of pairwise  

                 // Second, compute force from glue function
                 // if r2 <= nprw_r_max2 
                 int ifunc_nprw = ind_func_nprw_type[1][type_index[iinter]]; 
                                                     // This is for rho.
                 if (r2 <= nprw_r_max2[ifunc_nprw]) {  

                     if (hash_nprw_table[ifunc_nprw]==1) { 
                        r       = sqrt(r2);
                        hash_r  = r - nprw_r_min[ifunc_nprw]; 
                     } else {
                        hash_r  = r2 - nprw_r_min2[ifunc_nprw];
                     }  
                     _populate_dvdensity_hash(ifunc_nprw, hash_r, dx, dy, dz,
                        atom1_ind, atom2_ind, uConversion);
                 } // end of glue function  

              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/

      double sumU = _add_force_from_glue_hash(fx, fy, fz);
      vinter += sumU;

      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif-lnk_list*/


  if (neighbor_list == 1) {
      neighbor_list_check(pos);
      if (neighbor_list_update==1) {
	  conf_save(pos);
      }/*endif*/
  }/*endif*/

  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;   
  vinter = 0.0;
  for (iinter=1; iinter<=ninter; iinter++) {
    if (inter_off[iinter]) {continue;}
    if ((neighbor_list==1) && (neighbor_list_update==0)) {
        if (neighbors[iinter] == 0) {continue;}
    }/*endif*/

    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
      
    if ((neighbor_list==1) && (neighbor_list_update==1)) {
        if (r2 < neighbor_list_cutoff2) {neighbors[iinter] = 1;}
        else                            {neighbors[iinter] = 0;}
    }/*endif*/

    itype = type_index[iinter];

    // First, compute force from pairwise interaction
    //  if r2 is less than or equal to r_max2
    if (r2 <= r_max2[itype]) {

       if (hash_table[itype]==1) {
          r       = sqrt(r2);
          hash_r  = r - r_min[itype];
       } else {
          hash_r  = r2 - r_min2[itype];
       }
       _compute_pairwise_force(hash_r, itype, vinter, atom1_ind,
          atom2_ind, iinter, dx, dy, dz, fx, fy, fz);
     } // end of pairwise

     // Second, compute force from glue function
     // if r2 <= nprw_r_max2
     int ifunc_nprw = ind_func_nprw_type[1][type_index[iinter]];
                                         // This is for rho.
     if (r2 <= nprw_r_max2[ifunc_nprw]) {

        if (hash_nprw_table[ifunc_nprw]==1) {
           r       = sqrt(r2);
           hash_r  = r - nprw_r_min[ifunc_nprw];
        } else {
           hash_r  = r2 - nprw_r_min2[ifunc_nprw];
        }
        _populate_dvdensity_hash(ifunc_nprw, hash_r, dx, dy, dz,
           atom1_ind, atom2_ind, uConversion);
     } // end of glue function

  }/*endfor-iinter*/

  double sumU = _add_force_from_glue_hash(fx, fy, fz);
  vinter += sumU;

  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
    
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::getPot_atomic_nonpair_hash(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, iinter, itype, num_size;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz, *q;
  double dx, dy, dz, r, r2;
  double *num_x, *num_V_x, *num_d2V_x;
  double vcoul, vkb, vlj;
  double vnow, vnowtmp, dvnow;
  double vinter;
  double hash_m, hash_r;

  int ndim;
  int iatm, jatm, icell, ihash; 
  int valid;
  int expand_ncell;
  int c_ll, c1_ll;
  int switch_on;
  double xmin, ymin, zmin, xmax, ymax, zmax;

  char err_msg[LINE_NCHARMAX];

  double asw, bsw, switch_length;
  int    switch_type;
  switch_type   = 4;

  switch_length = rswitch_length; 

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;

  // Initialize temporary accumulators
  for (int i = 1; i <= natom; i++) {
    sumRho[i] = 0;
  }
  double uConversion = BOLTZ/BOHR;

  if (lnk_list) {
      ndim  = 3;
      for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
      xmin = x[1]; ymin = y[1]; zmin = z[1];
      xmax = x[1]; ymax = y[1]; zmax = z[1];
      for (iatm=2;iatm<=natom;iatm++) {
           xmin = MIN(x[iatm],xmin); xmax = MAX(x[iatm],xmax);
           ymin = MIN(y[iatm],ymin); ymax = MAX(y[iatm],ymax);
           zmin = MIN(z[iatm],zmin); zmax = MAX(z[iatm],zmax);
      }/*end for*/
          vcell_ll[1] =  (int)(floor((xmax-xmin)/rcell_ll[1])+1);
          vcell_ll[2] =  (int)(floor((ymax-ymin)/rcell_ll[2])+1);
          vcell_ll[3] =  (int)(floor((zmax-zmin)/rcell_ll[3])+1);
          expand_ncell = 0;
          if (vcell_ll[1]>ncell_ll[1]){
              ncell_ll[1]  =  vcell_ll[1];
              expand_ncell =  1;                     
          }/*end if*/
          if (vcell_ll[2]>ncell_ll[2]) {
              ncell_ll[2]  =  vcell_ll[2];
              expand_ncell =  1;
          }/*end if*/
          if (vcell_ll[3]>ncell_ll[3]) {
              ncell_ll[3]  =  vcell_ll[3];
              expand_ncell =  1;
          }/*end if*/
	  if (expand_ncell == 1) {
              ncell_tot = ncell_ll[1]*ncell_ll[2]*ncell_ll[3];
              delete[] &(head_ll[1]); head_ll = new int[ncell_tot] - 1;
              for (icell=1;icell<=ncell_tot;icell++) {head_ll[icell]=-1;}
          }/*end if*/
      for (iatm=1;iatm<=natom;iatm++) {
           vcell_ll[1] =  (int)(floor((x[iatm]-xmin)/rcell_ll[1])+1);
           vcell_ll[2] =  (int)(floor((y[iatm]-ymin)/rcell_ll[2])+1);
           vcell_ll[3] =  (int)(floor((z[iatm]-zmin)/rcell_ll[3])+1);
           if ( (vcell_ll[1]>ncell_ll[1]) ||
                (vcell_ll[2]>ncell_ll[2]) ||
                (vcell_ll[3]>ncell_ll[3]) ) {
                 sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
                 sprintf(strchr(err_msg,0),"atom %d is out of the box",iatm);
                 errorMessage(err_msg);
           }/*end if*/
           c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                 +(vcell_ll[2]-1)*ncell_ll[3]
	         +(vcell_ll[3]-1) + 1;
           if ( c_ll > ncell_tot ) {
              sprintf(err_msg,"lnk_list handling in Inter::getPot\n");
              sprintf(strchr(err_msg,0),"c_ll %d > ncell_tot %d\n",c_ll,ncell_tot);
              errorMessage(err_msg);
           }/*end if*/
           lscl_ll[iatm] = head_ll[c_ll];
           head_ll[c_ll] = iatm;
      }/*end for*/     

      vcoul = 0.0; 
      vkb   = 0.0;   
      vlj   = 0.0;   
      vinter = 0.0; 
      for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++) 
      for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++) 
      for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {
        c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
              +(vcell_ll[2]-1)*ncell_ll[3]
	      +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++) 
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++) 
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {
           c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3]
                  +(vcell1_ll[2]-1)*ncell_ll[3]
	          +(vcell1_ll[3]-1) + 1;
               
           /*scan atom i in cell c_ll*/
           iatm = head_ll[c_ll];
           while (iatm!=-1) {
             /*scan atom j in cell c1_ll*/
	     jatm = head_ll[c1_ll];
             while (jatm!=-1) {
              valid = 1;
 
              for (i=1;i<=excl_num[jatm];i++) {                   
                  if (excl_list[i][jatm]==iatm) {valid=0;}
              }
 
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;
                 dx = x[atom1_ind] - x[atom2_ind];
                 dy = y[atom1_ind] - y[atom2_ind];
                 dz = z[atom1_ind] - z[atom2_ind];
                 r2 = dx*dx+dy*dy+dz*dz;
                 iinter = inter_index[iatm][jatm];
                 itype = type_index[iinter];

                 // First, compute force from pairwise interaction
                 //  if r2 is less than or equal to r_max2
                 if (r2 <= r_max2[itype]) {
                     if (hash_table[itype]==1) {
                        r       = sqrt(r2);
                        hash_r  = r - r_min[itype];
                     } else {
                        hash_r  = r2 - r_min2[itype];
                     }
                     _compute_pairwise_pot(hash_r, itype, vinter, atom1_ind,
                        atom2_ind, iinter);
                 } // end of pairwise

                 // Second, compute force from glue function
                 // if r2 <= nprw_r_max2
                 int ifunc_nprw = ind_func_nprw_type[1][type_index[iinter]];
                                                     // This is for rho.
                 if (r2 <= nprw_r_max2[ifunc_nprw]) {

                     if (hash_nprw_table[ifunc_nprw]==1) {
                        r       = sqrt(r2);
                        hash_r  = r - nprw_r_min[ifunc_nprw];
                     } else {
                        hash_r  = r2 - nprw_r_min2[ifunc_nprw];
                     }
                     _populate_density_hash(ifunc_nprw, hash_r, atom1_ind, 
                        atom2_ind, uConversion);
                 } // end of glue function

              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
      }/*end for c_ll cell*/
      double sumU = _get_sum_of_glue_hash();
      vinter += sumU;

      ecoul  = vcoul;   
      ekb    = vkb;     
      elj    = vlj;     
      escp   = 0.0;
      einter = vinter;  
      return vinter;
  }/*endif*/


  if (neighbor_list == 1) {
      neighbor_list_check(pos);
      if (neighbor_list_update==1) {
	  conf_save(pos);
      }/*endif*/
  }/*endif*/


  vcoul = 0.0; 
  vkb   = 0.0;   
  vlj   = 0.0;     
  vinter = 0;
  for (iinter=1; iinter<=ninter; iinter++) {

    if (inter_off[iinter]) {continue;}
    if ((neighbor_list==1) && (neighbor_list_update==0)) {
        if (neighbors[iinter] == 0) {continue;}
    }/*endif*/

    atom1_ind = atom1[iinter];
    atom2_ind = atom2[iinter];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    if ((neighbor_list==1) && (neighbor_list_update==1)) {
        if (r2 < neighbor_list_cutoff2) {neighbors[iinter] = 1;}
        else                            {neighbors[iinter] = 0;}
    }/*endif*/
    
    itype = type_index[iinter];

    // First, compute force from pairwise interaction
    //  if r2 is less than or equal to r_max2
    if (r2 <= r_max2[itype]) {

       if (hash_table[itype]==1) {
          r       = sqrt(r2);
          hash_r  = r - r_min[itype];
       } else {
          hash_r  = r2 - r_min2[itype];
       }
       _compute_pairwise_pot(hash_r, itype, vinter, atom1_ind,
          atom2_ind, iinter);
     } // end of pairwise

     // Second, compute force from glue function
     // if r2 <= nprw_r_max2
     int ifunc_nprw = ind_func_nprw_type[1][type_index[iinter]];
                                         // This is for rho.
     if (r2 <= nprw_r_max2[ifunc_nprw]) {

        if (hash_nprw_table[ifunc_nprw]==1) {
           r       = sqrt(r2);
           hash_r  = r - nprw_r_min[ifunc_nprw];
        } else {
           hash_r  = r2 - nprw_r_min2[ifunc_nprw];
        }
        _populate_density_hash(ifunc_nprw, hash_r, atom1_ind, atom2_ind, 
           uConversion);
     } // end of glue function

  }/*endfor-iinter*/

  double sumU = _get_sum_of_glue_hash();
  vinter += sumU;

  ecoul  = vcoul;   
  ekb    = vkb;     
  elj    = vlj;     
  escp   = 0.0;
  einter = vinter;  
  return vinter;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
/*=========================================================================*/
void Inter::_compute_pairwise_force(const double hash_r, const int itype, 
                                    double& vinter, const int atom1_ind, 
                                    const int atom2_ind, const int iinter, 
                                    const double dx, const double dy, 
                                    const double dz, double*& fx, 
                                    double*& fy, double*& fz)  
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int ihash      = int(hash_r/hash_delta[itype]) + 1;
  ihash          = MIN(ihash,hash_size[itype]);
  double vnow    = hash_numeric_V_x[itype][ihash];
  double dvnow   = hash_numeric_dV_x[itype][ihash];
  double hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
  vnow   += dvnow*hash_m;
  double pre     = hash_numeric_pre_x[itype][ihash];
  vnow          *= scale_iinter[iinter];
  pre           *= scale_iinter[iinter];
  vinter        +=  vnow;
  fx[atom1_ind] += pre*dx;
  fx[atom2_ind] -= pre*dx;
  fy[atom1_ind] += pre*dy;
  fy[atom2_ind] -= pre*dy;
  fz[atom1_ind] += pre*dz;
  fz[atom2_ind] -= pre*dz;

  return; 
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Inter::_compute_pairwise_pot(const double hash_r, const int itype, 
                                  double& vinter, const int atom1_ind, 
                                  const int atom2_ind, const int iinter)  
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int ihash      = int(hash_r/hash_delta[itype]) + 1;
  ihash          = MIN(ihash,hash_size[itype]);
  double vnow    = hash_numeric_V_x[itype][ihash];
  double dvnow   = hash_numeric_dV_x[itype][ihash];
  double hash_m  = (hash_r-(double(ihash-1))*hash_delta[itype]);
  vnow   += dvnow*hash_m;
  double pre     = hash_numeric_pre_x[itype][ihash];
  vnow          *= scale_iinter[iinter];
  pre           *= scale_iinter[iinter];
  vinter        +=  vnow;

  return; 
/*-------------------------------------------------------------------------*/
} /*end routine */

/*=========================================================================*/


/*=========================================================================*/
void Inter::_populate_density_hash(const int ifunc_nprw, const double hash_r,
                                   const int atom1_ind,  const int atom2_ind, 
                                   const double uConversion)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Given a pair of atoms represented by atom1_ind and atom2_ind,
  // evaluate their density function values and perform summation.
  int ihash      = int(hash_r/hash_nprw_delta[ifunc_nprw]) + 1;
  ihash          = MIN(ihash,hash_nprw_size[ifunc_nprw]);
  double rhoNow  = hash_nprw_numeric_V_x[ifunc_nprw][ihash];  
  double drhoNow = hash_nprw_numeric_dV_x[ifunc_nprw][ihash]; 
  double hash_m  = hash_r - (double(ihash - 1)) * hash_nprw_delta[ifunc_nprw];  
  rhoNow += drhoNow * hash_m; 
  sumRho[atom1_ind] += (rhoNow*uConversion); // add rho to atom1
  sumRho[atom2_ind] += (rhoNow*uConversion); // add rho to atom2
 
  return; 
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Inter::_populate_dvdensity_hash(const int ifunc_nprw, const double hash_r,
                                     const double dx, const double dy,
                                     const double dz, const int atom1_ind,
                                     const int atom2_ind, const double uConversion)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Given a pair of atoms represented by atom1_ind and atom2_ind,
  // evaluate their density function values and perform summation.
  int ihash      = int(hash_r/hash_nprw_delta[ifunc_nprw]) + 1;
  ihash          = MIN(ihash,hash_nprw_size[ifunc_nprw]);
  double rhoNow  = hash_nprw_numeric_V_x[ifunc_nprw][ihash];
  double drhoNow = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
  double hash_m  = hash_r - (double(ihash - 1)) * hash_nprw_delta[ifunc_nprw];
  rhoNow += drhoNow * hash_m;
  sumRho[atom1_ind] += (rhoNow*uConversion); // add rho to atom1
  sumRho[atom2_ind] += (rhoNow*uConversion); // add rho to atom2

  // Also, store the result TX, TY, and TZ matrices.
  double pre = hash_nprw_numeric_pre_x[ifunc_nprw][ihash]; 
  TX[atom1_ind][atom2_ind] = -pre * uConversion * dx;
  TX[atom2_ind][atom1_ind] =  pre * uConversion * dx;
  TY[atom1_ind][atom2_ind] = -pre * uConversion * dy;
  TY[atom2_ind][atom1_ind] =  pre * uConversion * dy;
  TZ[atom1_ind][atom2_ind] = -pre * uConversion * dz;
  TZ[atom2_ind][atom1_ind] =  pre * uConversion * dz;

  return;
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::_get_sum_of_glue_hash() 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Given a vector of density function for atoms 1..natom,
  // Evaluate the glue function over all atoms, and return the sum.

  double sumU = 0.0;
  int ifunc = 2; // This is for U.
  int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[1]];

  for (int i = 1; i <= natom; i++) {
    double hash_rho  = sumRho[i] - nprw_r_min[ifunc_nprw];
    int ihash        = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
    ihash            = MIN(ihash,hash_nprw_size[ifunc_nprw]);
    double uNow      = hash_nprw_numeric_V_x[ifunc_nprw][ihash];
    double duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
    double hash_m    = hash_rho - (double(ihash - 1)) * hash_nprw_delta[ifunc_nprw];
    uNow += duNow * hash_m;
    sumU += uNow;
  }
  return sumU;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Inter::_add_force_from_glue_hash(double*& fx, double*& fy, double*& fz) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  // Note:
  // In addition to computing force, this function also computes potential 
  // identically to _get_sum_of_glue_hash().

  int ifunc = 2; // This is for U.
  int ifunc_nprw = ind_func_nprw_type[ifunc][type_index[1]];

  /********************************************************/ 
  /* If using a neighbor list ... 
  /********************************************************/ 
  if (neighbor_list) {

    // Initialize ui , uiy, uiz, ujz, ujy, ujz 
    for (int i = 1; i <= natom; i++) {
      array_uix[i] = 0.0;
      array_uiy[i] = 0.0;
      array_uiz[i] = 0.0;
      array_ujx[i] = 0.0;
      array_ujy[i] = 0.0;
      array_ujz[i] = 0.0;
    }  
 
    int atom1_ind, atom2_ind, ihash; 
    double hash_rho, hash_m, uNow, duNow;

    for (int iinter=1; iinter<=ninter; iinter++) {
      if (inter_off[iinter]) {continue;}
      if ((neighbor_list==1) && (neighbor_list_update==0)) {
          if (neighbors[iinter] == 0) {continue;}
      }/*endif*/
  
      atom1_ind = atom1[iinter];
      atom2_ind = atom2[iinter];

      // Accumulate the sum of derivatives
      array_uix[atom1_ind] += TX[atom1_ind][atom2_ind];
      array_uiy[atom1_ind] += TY[atom1_ind][atom2_ind];
      array_uiz[atom1_ind] += TZ[atom1_ind][atom2_ind];
      array_uix[atom2_ind] += TX[atom2_ind][atom1_ind];
      array_uiy[atom2_ind] += TY[atom2_ind][atom1_ind];
      array_uiz[atom2_ind] += TZ[atom2_ind][atom1_ind];

      // Accumulate the sum of (dU/drho)(drho/dr) for each atom. 
      hash_rho  = sumRho[atom2_ind] - nprw_r_min[ifunc_nprw];
      ihash     = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
      ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
      duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
      array_ujx[atom1_ind] += duNow * TX[atom1_ind][atom2_ind]; 
      array_ujy[atom1_ind] += duNow * TX[atom1_ind][atom2_ind]; 
      array_ujz[atom1_ind] += duNow * TX[atom1_ind][atom2_ind]; 
      hash_rho  = sumRho[atom1_ind] - nprw_r_min[ifunc_nprw];
      ihash     = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
      ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
      duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
      array_ujx[atom2_ind] += duNow * TX[atom2_ind][atom1_ind]; 
      array_ujy[atom2_ind] += duNow * TX[atom2_ind][atom1_ind]; 
      array_ujz[atom2_ind] += duNow * TX[atom2_ind][atom1_ind]; 

    } 

    // Evaluate force and potential  
    double sumU = 0.0; 
    for (int i = 1; i <= natom; i++) {

      // Compute force  
      hash_rho  = sumRho[i] - nprw_r_min[ifunc_nprw];
      ihash     = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
      ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
      uNow      = hash_nprw_numeric_V_x[ifunc_nprw][ihash];
      duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
      hash_m    = hash_rho - (double(ihash - 1)) * hash_nprw_delta[ifunc_nprw];
      uNow     += duNow * hash_m;
      sumU     += uNow; 
      fx[i]    += (duNow * array_uix[i] + array_ujx[i]);
      fy[i]    += (duNow * array_uiy[i] + array_ujy[i]);
      fz[i]    += (duNow * array_uiz[i] + array_ujz[i]);
    }

    return sumU;

  /********************************************************/ 
  /* If using a linked list ... 
  /********************************************************/ 
  } else if (lnk_list) {

    // Initialize ui , uiy, uiz, ujz, ujy, ujz
    for (int i = 1; i <= natom; i++) {
      array_uix[i] = 0.0;
      array_uiy[i] = 0.0;
      array_uiz[i] = 0.0;
      array_ujx[i] = 0.0;
      array_ujy[i] = 0.0;
      array_ujz[i] = 0.0;
    }

    int atom1_ind, atom2_ind, ihash, iatm, jatm;
    int valid, c_ll, c1_ll;
    double hash_rho, hash_m, uNow, duNow;

    for (vcell_ll[1]=1;vcell_ll[1]<=ncell_ll[1];vcell_ll[1]++)
    for (vcell_ll[2]=1;vcell_ll[2]<=ncell_ll[2];vcell_ll[2]++)
    for (vcell_ll[3]=1;vcell_ll[3]<=ncell_ll[3];vcell_ll[3]++) {

       c_ll = (vcell_ll[1]-1)*ncell_ll[2]*ncell_ll[3] + (vcell_ll[2]-1)*ncell_ll[3]
            +(vcell_ll[3]-1) + 1;
       for (vcell1_ll[1] =MAX(vcell_ll[1]-1,1); /*no pbc, vacuum*/
            vcell1_ll[1]<=MIN(vcell_ll[1]+1,ncell_ll[1]);vcell1_ll[1]++)
       for (vcell1_ll[2] =MAX(vcell_ll[2]-1,1); /*no pbc, vacuum*/
            vcell1_ll[2]<=MIN(vcell_ll[2]+1,ncell_ll[2]);vcell1_ll[2]++)
       for (vcell1_ll[3] =MAX(vcell_ll[3]-1,1); /*no pbc, vacuum*/
            vcell1_ll[3]<=MIN(vcell_ll[3]+1,ncell_ll[3]);vcell1_ll[3]++) {

          c1_ll = (vcell1_ll[1]-1)*ncell_ll[2]*ncell_ll[3] + (vcell1_ll[2]-1)*ncell_ll[3]
                 +(vcell1_ll[3]-1) + 1;

          /*scan atom i in cell c_ll*/
          iatm = head_ll[c_ll];
          while (iatm!=-1) {

             /*scan atom j in cell c1_ll*/
             jatm = head_ll[c1_ll];
             while (jatm!=-1) {

              valid = 1;
              for (int i=1;i<=excl_num[jatm];i++) {if (excl_list[i][jatm]==iatm) {valid=0;}}
              if ((iatm<jatm) && (valid==1)) {
                 atom1_ind = iatm;
                 atom2_ind = jatm;

                 // Accumulate the sum of derivatives
                 array_uix[atom1_ind] += TX[atom1_ind][atom2_ind];
                 array_uiy[atom1_ind] += TY[atom1_ind][atom2_ind];
                 array_uiz[atom1_ind] += TZ[atom1_ind][atom2_ind];
                 array_uix[atom2_ind] += TX[atom2_ind][atom1_ind];
                 array_uiy[atom2_ind] += TY[atom2_ind][atom1_ind];
                 array_uiz[atom2_ind] += TZ[atom2_ind][atom1_ind];

                 // Accumulate the sum of (dU/drho)(drho/dr) for each atom.
                 hash_rho  = sumRho[atom2_ind] - nprw_r_min[ifunc_nprw];
                 ihash     = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
                 ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
                 duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
                 array_ujx[atom1_ind] += duNow * TX[atom1_ind][atom2_ind];
                 array_ujy[atom1_ind] += duNow * TX[atom1_ind][atom2_ind];
                 array_ujz[atom1_ind] += duNow * TX[atom1_ind][atom2_ind];
                 hash_rho  = sumRho[atom1_ind] - nprw_r_min[ifunc_nprw];
                 ihash     = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
                 ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
                 duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
                 array_ujx[atom2_ind] += duNow * TX[atom2_ind][atom1_ind];
                 array_ujy[atom2_ind] += duNow * TX[atom2_ind][atom1_ind];
                 array_ujz[atom2_ind] += duNow * TX[atom2_ind][atom1_ind];

              }/*end if i<j*/
              jatm = lscl_ll[jatm];
             }/*end while jatm!=-1*/
             iatm = lscl_ll[iatm];
           }/*end while iatm!=-1*/
       }/*end for c1_ll cell*/
    }/*end for c_ll cell*/

    // Evaluate force and potential
    double sumU = 0.0;
    for (int i = 1; i <= natom; i++) {

      // Compute force
      hash_rho  = sumRho[i] - nprw_r_min[ifunc_nprw];
      ihash     = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
      ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
      uNow      = hash_nprw_numeric_V_x[ifunc_nprw][ihash];
      duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
      hash_m    = hash_rho - (double(ihash - 1)) * hash_nprw_delta[ifunc_nprw];
      uNow     += duNow * hash_m;
      sumU     += uNow;
      fx[i]    += (duNow * array_uix[i] + array_ujx[i]);
      fy[i]    += (duNow * array_uiy[i] + array_ujy[i]);
      fz[i]    += (duNow * array_uiz[i] + array_ujz[i]);
    }

    return sumU;

  /********************************************************/ 
  /* If NOT using any lists ... 
  /********************************************************/ 
  } else {

    int ihash; 
    double hash_rho, hash_m, uNow, duNow, uix, uiy, uiz, ujx, ujy, ujz;
    for (int i = 1; i <= natom; i++) {

      uix = 0.0;
      uiy = 0.0;
      uiz = 0.0;
      ujx = 0.0;
      ujy = 0.0;
      ujz = 0.0;
      for (int j = 1; j <= natom; j++) {

        if (i != j) {

          // For the ith atom, compute the sum of derivatives over all j's except j=i.
          uix += TX[i][j];
          uiy += TY[i][j];
          uiz += TZ[i][j];

          // For the ith aotm, compute the sum of (dU/drho)(drho/dr) for all j's except j=i.
          hash_rho  = sumRho[j] - nprw_r_min[ifunc_nprw];
          ihash     = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
          ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
          duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
          ujx += duNow*TX[i][j];
          ujy += duNow*TY[i][j];
          ujz += duNow*TZ[i][j];
        }
      }

      // For the ith atom, compute the sum of (dU/drho)(drho/dr) for j=i.
      hash_rho  = sumRho[i] - nprw_r_min[ifunc_nprw];
      ihash      = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
      ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
      duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
      fx[i] += (duNow*uix + ujx);
      fy[i] += (duNow*uiy + ujy);
      fz[i] += (duNow*uiz + ujz);
    }

    // Compute potential for all atoms.
    double sumU = 0.0; 
    for (int i = 1; i <= natom; i++) {

      hash_rho  = sumRho[i] - nprw_r_min[ifunc_nprw];
      ihash      = int(hash_rho/hash_nprw_delta[ifunc_nprw]) + 1;
      ihash     = MIN(ihash,hash_nprw_size[ifunc_nprw]);
      uNow      = hash_nprw_numeric_V_x[ifunc_nprw][ihash];
      duNow     = hash_nprw_numeric_dV_x[ifunc_nprw][ihash];
      hash_m    = hash_rho - (double(ihash - 1)) * hash_nprw_delta[ifunc_nprw];
      uNow += duNow * hash_m;
      sumU += uNow;
    }

    return sumU;

  }
 
/*-------------------------------------------------------------------------*/
} /*end routine */

 
