#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
void Inter::nonpair_setup(const Pot_data_entry *database, const Maps *maps, 
               		                          const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iii, iinter, itype;
  int natm_tot, inter_check;

  int ifunc, ifunc_nprw;
  int *nprw_type;

  int nhash;
  int local_size;
  int switch_on, switch_type;
  double asw,bsw;

  int    num_size;          
  double r, r2, vnow, dvnow;
  double *num_x, *num_V_x, *num_d2V_x;

  int atoms[3];

  char **atm_label;
  char atm1_label[NNAMECHARMAX];
  char atm2_label[NNAMECHARMAX];

  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;
  const Pot_data_entry **database_entry_map;
  const Pot_data_entry **nprw_database_entry_map;
  char err_msg[LINE_NCHARMAX];

  printf("----------------------------------------------\n");
  printf("Setting up nonpair inter-molecular interactions\n...\n");
  
  natom       = pos->natm_tot;  /* class*/
  natm_tot    = maps->natm_tot; /* local*/
  atm_label   = maps->atm_label;


/*-------------------------------------------------------------------------*/
/* 0) Return if no inter term                                              */
  
  if (energy_term != 1) {return;}

/*-------------------------------------------------------------------------*/
/* I) Setup non pairwise function parameters, allocate variables           */

  if (nprw==1) {/*ercolessi-adams*/
      nfunc_nprw_type   =  new int[ntype]-1;
      for (itype=1;itype<=ntype;itype++) {
	   nfunc_nprw_type[itype] = 2;           
      }/*endfor*/
  }/*endif*/ 
  if (nprw!=0) {
      max_nfunc_nprw_type = 0;
      for (itype=1;itype<=ntype;itype++) {
           if (nfunc_nprw_type[itype]> max_nfunc_nprw_type) {
	       max_nfunc_nprw_type = nfunc_nprw_type[itype];
           }/*endif*/
      }/*endfor*/
      ind_func_nprw_type = imatrix(1,max_nfunc_nprw_type,1,ntype);
      nfunc_nprw = 0;
      for (itype=1;itype<=ntype;itype++) {
	for (ifunc=1;ifunc<=nfunc_nprw_type[itype];ifunc++) {
	     nfunc_nprw++;
             ind_func_nprw_type[ifunc][itype] = nfunc_nprw;
        }/*endfor*/   
      }/*endfor*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* II) Find non pairwise functions in database                             */

  if (nprw != 0) {
     nprw_database_entry_map = new const Pot_data_entry*[nfunc_nprw]-1;
     for (itype=1;itype<=ntype;itype++) {
	for (ifunc=1;ifunc<=nfunc_nprw_type[itype];ifunc++) {
	  atoms[1] = atm1_type[itype];                                  
	  atoms[2] = atm2_type[itype];
          strcpy(atm1_label,atm_label[atoms[1]]);
          strcpy(atm2_label,atm_label[atoms[2]]);
          if ((nprw==1) && (ifunc==1)) {strcat(atm1_label,"ro");
                                        strcat(atm2_label,"ro");}
          if ((nprw==1) && (ifunc==2)) {strcat(atm1_label,"U");
                                        strcat(atm2_label,"U");}
	  exact_match_entry = NULL;
	  wild_match_entry  = NULL;
	  database_entry    = database;
	  inter_check = 0;
	  while (database_entry!=NULL && inter_check!=1) {
	     inter_check = database_entry->match(atm1_label, atm2_label, 0);
	     if (inter_check==1) { exact_match_entry = database_entry; }
	     if (inter_check==2) { wild_match_entry  = database_entry; }
	     database_entry = database_entry->next;
	  }/*end while*/
	  if (exact_match_entry==NULL && wild_match_entry==NULL) {
           sprintf(err_msg, "Inter pair is not found in database.\n"
                            "indices %d %d\nlabels %s %s",
	   atoms[1], atoms[2], atm1_label, atm2_label);
           errorMessage(err_msg);
	  }/*end if*/
	  if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
	  else                         { database_entry = wild_match_entry; }
          ifunc_nprw = ind_func_nprw_type[ifunc][itype];
          nprw_database_entry_map[ifunc_nprw] = database_entry;
        }/*end for*/
     }/*end for*/
  }/*end if*/


/*-------------------------------------------------------------------------*/
/* III) Find all null inters                                               */
/*      Will be implemented if needed                                      */

/*-------------------------------------------------------------------------*/
/* IV) Allocate and fill in variables                                      */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.1) From database                                                     */  

  nprw_type          = new int[nfunc_nprw]-1;
  nprw_numeric_size  = new int[nfunc_nprw]-1;
  hash_nprw_table    = new int[nfunc_nprw]-1;
  hash_nprw_size     = new int[nfunc_nprw]-1;
  nprw_r_min         = new double[nfunc_nprw]-1;
  nprw_r_max         = new double[nfunc_nprw]-1;
  nprw_numeric_V_x   = new double*[nfunc_nprw]-1;
  nprw_numeric_d2V_x = new double*[nfunc_nprw]-1;
  nprw_numeric_x     = new double*[nfunc_nprw]-1;

  for (ifunc_nprw=1; ifunc_nprw<=nfunc_nprw; ifunc_nprw++) {
      database_entry              = nprw_database_entry_map[ifunc_nprw];
      nprw_type[ifunc_nprw]       = database_entry->pot_type;
      hash_nprw_table[ifunc_nprw] = database_entry->hash_table;
      hash_nprw_size[ifunc_nprw]  = database_entry->hash_size;
      nprw_r_min[ifunc_nprw]      = database_entry->r_min;
      nprw_r_max[ifunc_nprw]      = database_entry->r_max;
      if (nprw_type[ifunc_nprw] == 4) {/*numeric type*/
          nprw_numeric_size[ifunc_nprw]  = database_entry->numeric_size;  
          nprw_numeric_V_x[ifunc_nprw]   = 
                                  new double[nprw_numeric_size[ifunc_nprw]]-1;
          nprw_numeric_d2V_x[ifunc_nprw] = 
                                  new double[3*nprw_numeric_size[ifunc_nprw]]-1;
          nprw_numeric_x[ifunc_nprw]     = 
                                  new double[nprw_numeric_size[ifunc_nprw]]-1; 
          for (iii=1;iii<=nprw_numeric_size[ifunc_nprw];iii++) {
           nprw_numeric_x[ifunc_nprw][iii]     
                                         = database_entry->numeric_x[iii];
           nprw_numeric_V_x[ifunc_nprw][iii]   
                                         = database_entry->numeric_V_x[iii];
          }/*endfor*/
          for (iii=1;iii<=3*nprw_numeric_size[ifunc_nprw];iii++) {
           nprw_numeric_d2V_x[ifunc_nprw][iii] 
                                         = database_entry->numeric_d2V_x[iii];
          }/*endfor*/
   } else {
        sprintf(err_msg,"From Inter::nonpair_setup()"
                        "For non pairwise functions only"
		        "{numeric} type is supported....");
        errorMessage(err_msg);
   }/*endif*/
  }/*endfor*/


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.2) Derived from database                                             */

  nprw_r_min2 = new double[nfunc_nprw]-1;
  nprw_r_max2 = new double[nfunc_nprw]-1;
  hash_nprw_delta    = new double[nfunc_nprw]-1;
  hash_nprw_numeric_V_x   = new double*[nfunc_nprw]-1;
  hash_nprw_numeric_dV_x  = new double*[nfunc_nprw]-1;
  hash_nprw_numeric_pre_x = new double*[nfunc_nprw]-1;

  for (ifunc_nprw=1; ifunc_nprw<=nfunc_nprw; ifunc_nprw++) {
       nprw_r_min2[ifunc_nprw]= nprw_r_min[ifunc_nprw]*nprw_r_min[ifunc_nprw];
       nprw_r_max2[ifunc_nprw]= nprw_r_max[ifunc_nprw]*nprw_r_max[ifunc_nprw];
       if (hash_nprw_table[ifunc_nprw] != 0) {
         local_size = hash_nprw_size[ifunc_nprw];
         hash_nprw_numeric_V_x[ifunc_nprw]  = new double[local_size] - 1;
         hash_nprw_numeric_dV_x[ifunc_nprw] = new double[local_size] - 1;
         hash_nprw_numeric_pre_x[ifunc_nprw]= new double[local_size] - 1;
       } else {
           hash_nprw_numeric_V_x[ifunc_nprw]   = NULL;
           hash_nprw_numeric_dV_x[ifunc_nprw]  = NULL;
           hash_nprw_numeric_pre_x[ifunc_nprw] = NULL;
       }/*endfi*/
  }/*endfor*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.3) Intermediate arrays                                               */

  sumRho = new double[natom] - 1;
  TX = new double*[natom] - 1;
  TY = new double*[natom] - 1;
  TZ = new double*[natom] - 1;
  if (neighbor_list || lnk_list) { 
    array_uix = new double[natom] - 1;
    array_uiy = new double[natom] - 1;
    array_uiz = new double[natom] - 1;
    array_ujx = new double[natom] - 1;
    array_ujy = new double[natom] - 1;
    array_ujz = new double[natom] - 1;
  } 
  for (int i = 1; i <= natom ; i++) {

    TX[i] = new double[natom] - 1;
    TY[i] = new double[natom] - 1;
    TZ[i] = new double[natom] - 1;
  }

/*-------------------------------------------------------------------------*/
/* V) Inter list                                                           */
/*    To be implemented if needed                                          */

/*-------------------------------------------------------------------------*/
/* VI) Setup Distance dependent dielectric                                 */
/*    To be implemented if needed                                          */

/*-------------------------------------------------------------------------*/
/* VII) Allocate initialize scale_iinter                                   */
/*    To be implemented if needed                                          */

/*-------------------------------------------------------------------------*/
/* VIII) Fill hash tables and related variables                            */

  nhash = 0;
  for (ifunc_nprw=1; ifunc_nprw<=nfunc_nprw; ifunc_nprw++) {
    
    if (hash_nprw_table[ifunc_nprw] != 0) {   
        nhash++;
        if (hash_nprw_table[ifunc_nprw] == 1) {
            hash_nprw_delta[ifunc_nprw]  = nprw_r_max[ifunc_nprw]
                                          -nprw_r_min[ifunc_nprw];
        }
        else if (hash_nprw_table[ifunc_nprw] == 2) {
            hash_nprw_delta[ifunc_nprw]  = nprw_r_max2[ifunc_nprw]
                                          -nprw_r_min2[ifunc_nprw];
        }/*end if*/
        hash_nprw_delta[ifunc_nprw] /= ((double)hash_nprw_size[ifunc_nprw]);
    }/*endif*/

    if ((hash_nprw_table[ifunc_nprw]!=0) && (nprw_type[ifunc_nprw] == 4)) {/*num*/
        for (iii=1;iii<=hash_nprw_size[ifunc_nprw];iii++) {
          if (hash_nprw_table[ifunc_nprw] == 1) 
	     { r = nprw_r_min[ifunc_nprw] 
                 + ((double)(iii-1))*hash_nprw_delta[ifunc_nprw]; }/*endif*/
          if (hash_nprw_table[ifunc_nprw] == 2) 
	     { r2 = nprw_r_min2[ifunc_nprw] 
                 + ((double)(iii-1))*hash_nprw_delta[ifunc_nprw];
	       r=sqrt(r2);                                        }/*endif*/
          num_size  = nprw_numeric_size[ifunc_nprw];
          num_x     = nprw_numeric_x[ifunc_nprw];
          num_V_x   = nprw_numeric_V_x[ifunc_nprw];
          num_d2V_x = nprw_numeric_d2V_x[ifunc_nprw];
          nsplint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
          ndsplint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&dvnow);
          switch_on = 0;
          if (r >= (nprw_r_max[ifunc_nprw]-rswitch_length)) {
              switch_on = 1;
              switch_type = 4;
              asw = nprw_r_max[ifunc_nprw]-rswitch_length;
              bsw = nprw_r_max[ifunc_nprw];
          }/*end if*/
          if (switch_on==1) {
            dvnow   =  dvnow*switch_right(r,asw,bsw,switch_type)
                      +vnow*dx_switch_right(r,asw,bsw,switch_type);
            vnow    =  vnow*switch_right(r,asw,bsw,switch_type);
          }/*end if*/
          hash_nprw_numeric_V_x[ifunc_nprw][iii]   =  vnow;
          if (hash_nprw_table[ifunc_nprw] == 1) 
	     {hash_nprw_numeric_dV_x[ifunc_nprw][iii]  =  dvnow;}
          if (hash_nprw_table[ifunc_nprw] == 2) 
	     {hash_nprw_numeric_dV_x[ifunc_nprw][iii]  =  dvnow/(2.0*r);}
	  hash_nprw_numeric_pre_x[ifunc_nprw][iii] = -dvnow/r;
        }/*end for*/
    }/*endif*/

  }/*end for*/

  if (nhash==nfunc_nprw) {hash_nprw_all  = 1;}/* hash all interaction      */
  else                   {hash_nprw_all  = 0;}/* not for all               */

/*-------------------------------------------------------------------------*/
/* IX) DEBUG                                                               */

#ifdef DEBUG
  for (iinter=1; iinter<=ninter; iinter++) {
    printf("Inter pair %2d: %d(%s)--%d(%s)\n",iinter,
	   atom1[iinter], maps->atm_label[atom1[iinter]],
	   atom2[iinter], maps->atm_label[atom2[iinter]]);
    if (nprw==1) {
      for (ifunc=1;ifunc<=2;ifunc++) {
        if (ifunc==1) {printf("ro (ercolessi-adams): \n");}
        if (ifunc==2) {printf("U (ercolessi-adams): \n");}
        printf("numeric_x: \n");
        ifunc_nprw = ind_func_nprw_type[ifunc][type_index[iinter]];
        for (iii=1;iii<=nprw_numeric_size[ifunc_nprw];iii++) {
	    printf("%g ",nprw_numeric_x[ifunc_nprw][iii]);
        }/*endfor*/
        printf("\n");
        printf("numeric_V_x: \n");
        for (iii=1;iii<=nprw_numeric_size[ifunc_nprw];iii++) {
	    printf("%g ",nprw_numeric_V_x[ifunc_nprw][iii]);
        }/*endfor*/
        printf("\n");
        printf("numeric_d2V_x: \n");
        for (iii=1;iii<=nprw_numeric_size[ifunc_nprw];iii++) {
	    printf("%g ",nprw_numeric_d2V_x[ifunc_nprw][iii]);
        }/*endfor*/
        printf("\n");
      }/*endfor*/
    }/*endif*/

      }/*endfor*/
#endif

/*-------------------------------------------------------------------------*/
/* VI) Deallocate memory                                                   */
  
  delete (nprw_database_entry_map+1);
  
  printf("Inter-mol setting up done.\n");
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
