#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_REPLACE_ATM_LABEL_OFF
#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
void Inter::replace_setup(const Pot_data_entry *database, const Maps *maps,
                          const System *system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int atoms[3]; 
  int i, iii, iatm, natm_tot;
  int ires, nres, nres_type, ind_res_type, res_found;

  int res_id, res_id1, res_id2;
  int his, ade, cyt, gua, thy, ura; 
  int inter_check; 
  int iinter, iinter_replace, ninter_replace;

  int KB_UA, KB_5pt;

  int cancel_res_inter_local, cancel_res_inter_neighbor;  
  int valid_atom1, valid_atom2, valid_atoms, valid_res, valid;
  
  int itype, ntype_mall, ntype_tot;
  int ind_atm1_replace, ind_atm2_replace;
  int itype_replace, ntype_replace, type_found;  
  int *ratm1_type, *ratm2_type;

  int *atm_res_id, *res_mol_id;
  int *atom1_replace, *atom2_replace, *index_replace;
  int *type_index_replace;

  char **mol_label;
  char **atm_label, **upac_atm_label, **replace_atm_label;
  char *atm1_label, *atm2_label;
  char *ratm1_type_label,*ratm2_type_label; 
  char **aa1_code, **aa3_code; 

  Residue *residue;

  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;
  const Pot_data_entry **database_entry_map;
  char err_msg[LINE_NCHARMAX];

  residue                   = system->pos.residue; 
  cancel_res_inter_local    = system->general.cancel_res_inter_replace_local;
  cancel_res_inter_neighbor = system->general.cancel_res_inter_replace_neighbor;
  KB_UA                     = system->general.inter_replace_KB_UA;
  KB_5pt                    = system->general.inter_replace_KB_5pt;
 
  atm_res_id     =  maps->atm_res_id;
  res_mol_id     =  maps->res_mol_id;
  atm_label      =  maps->atm_label;
  mol_label      =  maps->mol_label;
  upac_atm_label =  maps->upac_atm_label;
  aa1_code       =  maps->aa1_code;
  aa3_code       =  maps->aa3_code;

  natm_tot       =  maps->natm_tot; /* local*/
  nres           =  maps->nres;

  printf("----------------------------------------------\n");
  printf("Setting up replace inter-molecular interactions\n...\n");


/*-------------------------------------------------------------------------*/
/* 0.A) Return if no inter term                                            */
  
  if (energy_term != 1) {return;}

/*-------------------------------------------------------------------------*/
/* 0.B) Allocate variables                                                 */

  atom1_replace     = new int[ninter] - 1;
  atom2_replace     = new int[ninter] - 1;
  index_replace     = new int[ninter] - 1;
  replace_atm_label = (char **)cmall_int_mat(1, natm_tot, 0, NNAMECHARMAX);

/*-------------------------------------------------------------------------*/
/* 0.C) Get replace_atm_label                                              */

  nres_type = 25;
  for (iatm=1;iatm<=natm_tot;iatm++) {
       res_id = atm_res_id[iatm];
       res_found = 0;
       his = 0; 
       if ( (strcasecmp(residue[res_id].residue_name,"HSE")==0) ||
            (strcasecmp(residue[res_id].residue_name,"HSD")==0) ||
            (strcasecmp(residue[res_id].residue_name,"HSP")==0) ) {his = 1;}
       ade = 0; cyt = 0; gua = 0; thy = 0; ura = 0; 
       if (strcasecmp(residue[res_id].residue_name,"ADE")==0) {ade = 1;}
       if (strcasecmp(residue[res_id].residue_name,"CYT")==0) {cyt = 1;}
       if (strcasecmp(residue[res_id].residue_name,"GUA")==0) {gua = 1;}
       if (strcasecmp(residue[res_id].residue_name,"THY")==0) {thy = 1;}
       if (strcasecmp(residue[res_id].residue_name,"URA")==0) {ura = 1;}
             
       for (ires=1;ires<=nres_type;ires++) {
	 if (his == 1) {
	  if (strcasecmp("HIS",aa3_code[ires])==0)
              {ind_res_type = ires; res_found = 1; break;}
         } 
	 else if (ade == 1) {
	  if (strcasecmp("A",aa3_code[ires])==0)
              {ind_res_type = ires; res_found = 1; break;}
         } 
	 else if (cyt == 1) {
	  if (strcasecmp("C",aa3_code[ires])==0)
              {ind_res_type = ires; res_found = 1; break;}
         } 
	 else if (gua == 1) {
	  if (strcasecmp("G",aa3_code[ires])==0)
              {ind_res_type = ires; res_found = 1; break;}
         } 
	 else if (thy == 1) {
	  if (strcasecmp("T",aa3_code[ires])==0)
              {ind_res_type = ires; res_found = 1; break;}
         }
	 else if (ura == 1) {
	  if (strcasecmp("U",aa3_code[ires])==0)
              {ind_res_type = ires; res_found = 1; break;}
         } 
         else {
	    if (strcasecmp(residue[res_id].residue_name,aa3_code[ires])==0) {
                ind_res_type = ires; 
                res_found = 1;
                break;
            }/*endif*/
         }/*endif*/
       }/*endfor*/ 
       if (res_found == 0) {
         sprintf(err_msg, "Inter::replace_setup : Cannot process non regular"
                          " residue: res_id %d res_label %s",
                           res_id,residue[res_id].residue_name);
         errorMessage(err_msg);
       }/*endif*/
       strcpy(replace_atm_label[iatm],aa1_code[ind_res_type]);
       if (strcasecmp(upac_atm_label[iatm],"C4'")==0) {/*exceptions*/
	   strcat(replace_atm_label[iatm],"C4*");
       } else {
           strcat(replace_atm_label[iatm],upac_atm_label[iatm]);
       }/*endif*/
       /*------------------------ corrections --------------------------*/ 
       if (strcasecmp(replace_atm_label[iatm],"ICD")==0) {
	   strcat(replace_atm_label[iatm],"1");
       }/*endfor*/

#ifdef DEBUG_REPLACE_ATM_LABEL
     printf("aa1_code[%d]  #%s#\n",ind_res_type,aa1_code[ind_res_type]);
     printf("upac_atm_label[%d]  #%s#\n",iatm,upac_atm_label[iatm]);
     printf("replace_atm_label[%d]  #%s#\n",iatm,replace_atm_label[iatm]);
     getchar();
#endif
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* I) Get the interactive pairs to replace                                 */

  ninter_replace = 0;
  for (iinter=1;iinter<=ninter;iinter++) {
       res_id1    = atm_res_id[atom1[iinter]];
       res_id2    = atm_res_id[atom2[iinter]];
       atm1_label = upac_atm_label[atom1[iinter]];
       atm2_label = upac_atm_label[atom2[iinter]];
       valid_atom1 = 0; valid_atom2 = 0;
       valid_atoms = 0; valid_res = 0; valid = 0;

       if (KB_UA == 1) {
           if ( (atm1_label[0] != 'H') && 
                (atm2_label[0] != 'H') ) {valid_atoms = 1;}
       }/*endif*/
       if (KB_5pt == 1) {       
         if ( (strcasecmp(mol_label[res_mol_id[res_id1]],"protein")==0) ) {
               sprintf(err_msg, "Inter::replace_setup:\n"
                          " \\inter_replace_type{KB_5pt} is not supported\n"
                          " for protein residue: res_id %d res_label %s",
                            res_id1,residue[res_id1].residue_name);
               errorMessage(err_msg);               
         }/*endif*/
         if ( (strcasecmp(mol_label[res_mol_id[res_id2]],"protein")==0) ) {
               sprintf(err_msg, "Inter::replace_setup:\n"
                          " \\inter_replace_type{KB_5pt} is not supported\n"
                          " for protein residue: res_id %d res_label %s",
                            res_id1,residue[res_id2].residue_name);
               errorMessage(err_msg);               
         }/*endif*/
	 if  ( (strcasecmp(atm1_label,"P")==0)   ||
               (strcasecmp(atm1_label,"C4'")==0) ||
               (strcasecmp(atm1_label,"C6")==0)  ||
               (strcasecmp(atm1_label,"C2")==0)  ||
	       (strcasecmp(atm1_label,"C4")==0)  ) {valid_atom1 = 1;}
	 if  ( (strcasecmp(atm2_label,"P")==0)   ||
               (strcasecmp(atm2_label,"C4'")==0) ||
               (strcasecmp(atm2_label,"C6")==0)  ||
               (strcasecmp(atm2_label,"C2")==0)  ||
	       (strcasecmp(atm2_label,"C4")==0)  ) {valid_atom2 = 1;}
         if ((valid_atom1==1)&&(valid_atom2==1)) {valid_atoms = 1;}
       }/*endif*/       

       if (cancel_res_inter_neighbor==1) {
   	      if (abs(res_id1-res_id2)>1) {valid_res = 1;}           
       } 
       else {
	   if (cancel_res_inter_local==1) {
              if (abs(res_id1-res_id2)>0) {valid_res = 1;}           
           }
           else {
	      valid_res = 1;
           }/*endif*/
       }/*endif*/
       if ((valid_res == 1) && (valid_atoms == 1)) {valid = 1;}
       if (valid==1) {
           ninter_replace++;
           atom1_replace[ninter_replace] = atom1[iinter];
           atom2_replace[ninter_replace] = atom2[iinter];
           index_replace[ninter_replace] = iinter;
       }/*endif*/
  }/*end for*/

  ntype_mall = 100*NMEM_MIN;
  ratm1_type = new int[ntype_mall] - 1;
  ratm2_type = new int[ntype_mall] - 1;
  
  type_index_replace = new int[ninter_replace] - 1;
  
  itype_replace = 0;
  for (iinter_replace=1;iinter_replace<=ninter_replace;iinter_replace++) {
       type_found = 0;
       ind_atm1_replace = atom1_replace[iinter_replace];
       ind_atm2_replace = atom2_replace[iinter_replace];
       for (i=1;i<=itype_replace;i++) {
	    ratm1_type_label = replace_atm_label[ratm1_type[i]]; 
	    ratm2_type_label = replace_atm_label[ratm2_type[i]];
            atm1_label = replace_atm_label[ind_atm1_replace];
            atm2_label = replace_atm_label[ind_atm2_replace];
            if ( ( strcasecmp(ratm1_type_label,atm1_label)==0 &&
                   strcasecmp(ratm2_type_label,atm2_label)==0 ) ||
                 ( strcasecmp(ratm1_type_label,atm2_label)==0 &&
                   strcasecmp(ratm2_type_label,atm1_label)==0 ) ) { 
	           type_found = 1;
                   type_index_replace[iinter_replace] =  i;
                   break;
            }/*endif*/
       }/*endfor*/
       if (type_found == 0) {
	   itype_replace++;
           if (itype_replace>ntype_mall) {
	     ntype_mall += NMEM_MIN;
             ratm1_type = (int *)realloc(ratm1_type+1, ntype_mall*sizeof(int))-1;
             ratm2_type = (int *)realloc(ratm2_type+1, ntype_mall*sizeof(int))-1;
           }/*endif*/
           type_index_replace[iinter_replace] = itype_replace;
           ratm1_type[itype_replace] = ind_atm1_replace;
           ratm2_type[itype_replace] = ind_atm2_replace;
       }/*endif*/
  }/*endfor*/
  ntype_replace = itype_replace;

/*-------------------------------------------------------------------------*/
/* II) Find interactions in database                                       */

  database_entry_map = new const Pot_data_entry*[ntype_replace]-1;
  for (itype_replace=1;itype_replace<=ntype_replace;itype_replace++) {
	atoms[1] = ratm1_type[itype_replace];
	atoms[2] = ratm2_type[itype_replace];
	exact_match_entry = NULL;
	wild_match_entry  = NULL;
	database_entry    = database;
	inter_check = 0;
	while (database_entry!=NULL && inter_check!=1) {
	  inter_check =database_entry->match(replace_atm_label,atoms,0);
	  if (inter_check==1) { exact_match_entry = database_entry; }
	  if (inter_check==2) { wild_match_entry  = database_entry; }
	  database_entry = database_entry->next;
	}/*end while*/
	if (exact_match_entry==NULL && wild_match_entry==NULL) {
           sprintf(err_msg, "Inter pair is not found in database.\nindices %d %d\nlabels %s %s",
	   atoms[1], atoms[2], replace_atm_label[atoms[1]], replace_atm_label[atoms[2]]);
           errorMessage(err_msg);
	}/*end if*/
	if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
	else                         { database_entry = wild_match_entry; }
        database_entry_map[itype_replace] = database_entry;
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* III) Find all null inters                                               */

  for (iinter_replace=1;iinter_replace<=ninter_replace;iinter_replace++) {
      itype_replace  = type_index_replace[iinter_replace];
      database_entry =  database_entry_map[itype_replace];
      if (database_entry->pot_type==0) {
          inter_off[index_replace[iinter_replace]] = 1;
      }/*endif*/
  }/*end for*/

/*------------------------------------------------------------------------*/
/* IV) Allocate and fill in variables                                     */

  ntype_tot    =  ntype + ntype_replace;
  
  type         = (int *)realloc(type+1, ntype_tot*sizeof(int)) - 1;
  numeric_size = (int *)realloc(numeric_size+1, ntype_tot*sizeof(int)) - 1;
  hash_table   = (int *)realloc(hash_table+1, ntype_tot*sizeof(int)) - 1;
  hash_size    = (int *)realloc(hash_size+1, ntype_tot*sizeof(int)) - 1;
  eps          = (double *)realloc(eps+1, ntype_tot*sizeof(double)) - 1;  
  feps  = (double *)realloc(feps+1, ntype_tot*sizeof(double)) - 1;  
  sig   = (double *)realloc(sig+1, ntype_tot*sizeof(double)) - 1;
  beta  = (double *)realloc(beta+1, ntype_tot*sizeof(double)) - 1;   
  scale = (double *)realloc(scale+1, ntype_tot*sizeof(double)) - 1;    
  r0    = (double *)realloc(r0+1, ntype_tot*sizeof(double)) - 1; 
  awill = (double *)realloc(awill+1, ntype_tot*sizeof(double)) - 1;   
  bwill = (double *)realloc(bwill+1, ntype_tot*sizeof(double)) - 1;   
  c6m   = (double *)realloc(c6m+1, ntype_tot*sizeof(double)) - 1;
  c8m   = (double *)realloc(c8m+1, ntype_tot*sizeof(double)) - 1;
  c10m  = (double *)realloc(c10m+1, ntype_tot*sizeof(double)) - 1;
  r_min = (double *)realloc(r_min+1, ntype_tot*sizeof(double)) - 1;
  r_max = (double *)realloc(r_max+1, ntype_tot*sizeof(double)) - 1;
  r_res = (double *)realloc(r_res+1, ntype_tot*sizeof(double)) - 1;
  r_min2 = (double *)realloc(r_min2+1, ntype_tot*sizeof(double)) - 1;
  r_max2 = (double *)realloc(r_max2+1, ntype_tot*sizeof(double)) - 1;


  numeric_x   = (double **)realloc(numeric_x+1, ntype_tot*sizeof(double*))-1;
  numeric_V_x = (double **)realloc(numeric_V_x+1,ntype_tot*sizeof(double*))-1;
  numeric_d2V_x = (double **)realloc(numeric_d2V_x+1, 
                                                  ntype_tot*sizeof(double*))-1;

  for (itype=ntype+1; itype<=ntype_tot; itype++) {
    database_entry = database_entry_map[itype-ntype];
    type[itype] = database_entry->pot_type;
    hash_table[itype] = database_entry->hash_table;
    hash_size[itype]  = database_entry->hash_size;
    eps[itype]  = database_entry->eps;
    feps[itype] = 4.0*database_entry->eps;
    sig[itype]  = database_entry->sig;
    beta[itype] = database_entry->beta;
    scale[itype]= database_entry->scale;
    r0[itype]   = database_entry->r0;
    awill[itype]= database_entry->awill;
    bwill[itype]= database_entry->bwill;
    c6m[itype]  = database_entry->c6m;
    c8m[itype]  = database_entry->c8m;
    c10m[itype] = database_entry->c10m;
    r_min[itype]= database_entry->r_min;
    r_max[itype]= database_entry->r_max;
    r_res[itype]= database_entry->r_res;
    r_min2[itype] = r_min[itype]*r_min[itype];
    r_max2[itype] = r_max[itype]*r_max[itype];


    if (type[itype] == 4) {/*numeric type*/
       numeric_size[itype] = database_entry->numeric_size;  
       numeric_V_x[itype]   = new double[numeric_size[itype]] - 1;
       numeric_d2V_x[itype] = new double[numeric_size[itype]] - 1;
       numeric_x[itype]     = new double[numeric_size[itype]] - 1;
       for (iii=1;iii<=numeric_size[itype];iii++) {
           numeric_x[itype][iii]     = database_entry->numeric_x[iii];
           numeric_V_x[itype][iii]   = database_entry->numeric_V_x[iii];
           numeric_d2V_x[itype][iii] = database_entry->numeric_d2V_x[iii];
       }
    }
  }


  for (iinter_replace=1;iinter_replace<=ninter_replace;iinter_replace++) {
      itype_replace  = type_index_replace[iinter_replace];
      type_index[index_replace[iinter_replace]] = ntype + itype_replace;
  }/*end for*/

  ntype = ntype_tot;

/*-------------------------------------------------------------------------*/
/* V) Hashing is not supported                                             */

  for (itype=ntype-ntype_replace+1; itype<=ntype_tot; itype++) {
       if (hash_table[itype] != 0) {
               sprintf(err_msg, "Inter::replace_setup:\n"
		       " hashing is not supported in %s\n",
                        system->filenames.inter_replace_database_filename);
               errorMessage(err_msg);               

       }/*endif*/
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* VI) DEBUG                                                               */

#ifdef DEBUG
  for (iinter=1; iinter<=ninter; iinter++) {
    printf("Inter pair %2d: %d(%s,%s)--%d(%s,%s)\n",iinter,
	   atom1[iinter], 
           maps->upac_atm_label[atom1[iinter]],
           maps->atm_label[atom1[iinter]],
	   atom2[iinter],
           maps->upac_atm_label[atom2[iinter]], 
           maps->atm_label[atom2[iinter]]);
    printf("type_index[%d] %d\n",iinter,type_index[iinter]);
    if (type[type_index[iinter]]==1 || type[type_index[iinter]]==2) {
      printf("\t\t{  eps=%.8g, sig=%.8g, beta=%.8g}\n",
	     eps[type_index[iinter]],sig[type_index[iinter]],beta[type_index[iinter]]);
      printf("\t\t{scale=%.8g,  r0=%.8g}\n",
	     scale[type_index[iinter]], r0[type_index[iinter]]);
    }
    if (type[type_index[iinter]]==3) {
      printf("\t\t{awill=%.8g, bwill=%.8g }\n", awill[type_index[iinter]], bwill[type_index[iinter]]);
      printf("\t\t{   c6=%.8g,    c8=%.8g, c10=%.8g}\n", 
	     c6m[type_index[iinter]], c8m[type_index[iinter]], c10m[type_index[iinter]]);
    }
    if (type[type_index[iinter]]==4) {
        printf("hash_table: %d\n",hash_table[type_index[iinter]]);
        printf("numeric_x: \n");
        printf("type_index[%d] %d\n",iinter,type_index[iinter]); getchar();
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
  	    printf("%g ",numeric_x[type_index[iinter]][iii]);
        }
        printf("\n");
	getchar();
        printf("numeric_V_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_V_x[type_index[iinter]][iii]);
        }
        printf("\n");
        getchar();
        printf("numeric_d2V_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_d2V_x[type_index[iinter]][iii]);
        }
        printf("\n");
    }
  }
#endif

/*-------------------------------------------------------------------------*/
/* VII) Deallocate memory                                                  */
  
  delete (database_entry_map+1);
  
  printf("Replace Inter-mol setting up done.\n");
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
