#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
void Inter::setup(const Pot_data_entry *database, const Maps *maps, 
               		                          const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int ires, iresbond, iexclude, iexclude_global;
  int i, ii, jj, iii, iatm, jatm, iinter, itype;
  int inter_check;
  int atoms[3];
          
  int jatm_found;
  int natm_tot, ndim;
  int atom1_ind, atom2_ind;
  int ninter_mall, ntype_mall; 
  int ntype_iatm, ntype_jatm, nexclude_iatm;
  int ntype_iatm_mall, ntype_jatm_mall;
  int valid, action_type, type_found;

  double edx, edy, edz, er2, rexclude_length2;

  int nhash;
  int switch_on, switch_type;
  double vnowtmp, asw,bsw;

  int    num_size;          
  double r, r2, vnow, dvnow;
  double *num_x, *num_V_x, *num_d2V_x;

  char *atm1_type_label,*atm2_type_label;
  char **atm_label,*atm1_label, *atm2_label, *iatm_label, *jatm_label;
  char* jatm_type_label; 

  int *exclude_iatm, *exclude_iatm_order;
  int *type_iatm, *type_jatm, *type_iatm_order, *type_jatm_order;

  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;
  const Pot_data_entry **database_entry_map;
  char err_msg[LINE_NCHARMAX];

  printf("----------------------------------------------\n");
  printf("Setting up inter-molecular interactions\n...\n");
  
  natom       = pos->natm_tot;  /* class*/
  natm_tot    = maps->natm_tot; /* local*/
  atm_label   = maps->atm_label;
  action_type = pos->action_type;
  lnk_list    = maps->lnk_list;
  neighbor_list = maps->neighbor_list;


/*-------------------------------------------------------------------------*/
/* I) Setup topology: Get the interactive pairs                            */

  nexclude        = maps->nexclude;
  ninter_mall     = 100*NMEM_MIN;
  ntype_mall      = 100*NMEM_MIN;
  ntype_iatm_mall = ntype_mall;
  ntype_jatm_mall = ntype_mall;

  atom1         = new int[ninter_mall]-1;
  atom2         = new int[ninter_mall]-1;
  exclude_atom1 = new int[nexclude]-1;
  exclude_atom2 = new int[nexclude]-1;
  inter_off     = new int[ninter_mall]-1;
  type_index    = new int[ninter_mall]-1;
  atm1_type     = new int[ntype_mall]-1;
  atm2_type     = new int[ntype_mall]-1;
  exclude_iatm       = new int[nexclude] - 1;
  exclude_iatm_order = new int[nexclude] - 1;
  type_iatm          = new int[ntype_iatm_mall] - 1;
  type_iatm_order    = new int[ntype_iatm_mall] - 1;
  type_jatm          = new int[ntype_jatm_mall] - 1;
  type_jatm_order    = new int[ntype_jatm_mall] - 1;

  inter_index = imatrix(1,natm_tot,1,natm_tot);

  for (ires=1;ires<=pos->nres;ires++) {
      for (iexclude=1;iexclude<=pos->residue[ires].nexclude;iexclude++) {
   	  iexclude_global = pos->residue[ires].exclude[iexclude];
          if (iexclude_global>nexclude) {
             sprintf(err_msg,"Allocation problem in exclude setup topology!");
             sprintf(err_msg,"iexclude_global:%d>nexclude %d",
                                                    iexclude_global,nexclude);
             errorMessage(err_msg);
          } 
   	  exclude_atom1[iexclude_global] = 
                                 pos->residue[ires].exclude_atom[iexclude][1];
   	  exclude_atom2[iexclude_global] = 
                                 pos->residue[ires].exclude_atom[iexclude][2];
      }/*end for*/       
  }/*end for*/
  for (iresbond=1;iresbond<=maps->nresbond;iresbond++) {
      for (iexclude=1;iexclude<=pos->residue_bond[iresbond].nexclude;iexclude++) {
   	  iexclude_global = pos->residue_bond[iresbond].exclude[iexclude];
          if (iexclude_global>nexclude) {
             sprintf(err_msg,"Allocation problem in exclude setup topology!");
             sprintf(err_msg,"iexclude_global:%d>nexclude %d",
                                                    iexclude_global,nexclude);
             errorMessage(err_msg);
          } 
   	  exclude_atom1[iexclude_global] = 
                         pos->residue_bond[iresbond].exclude_atom[iexclude][1];
   	  exclude_atom2[iexclude_global] = 
                         pos->residue_bond[iresbond].exclude_atom[iexclude][2];
      }/*end for*/
  }/*end for*/


  iinter = 0;
  itype = 0;
  nexclude_iatm = 0;

  for (iatm=1; iatm<=natm_tot; iatm++) {

    for (iexclude=1;iexclude<=nexclude;iexclude++) {
         if (iatm==exclude_atom1[iexclude]) {
             nexclude_iatm++;
             exclude_iatm[nexclude_iatm] = iexclude;
             exclude_iatm_order[nexclude_iatm] = 1;
         }/*endif*/
         if (iatm==exclude_atom2[iexclude]) {
             nexclude_iatm++;
             exclude_iatm[nexclude_iatm] = iexclude;
             exclude_iatm_order[nexclude_iatm] = 2;
         }/*endif*/
    }/*end for*/


    ntype_iatm =  0;
    ntype_jatm =  0;
    iatm_label =  maps->atm_label[iatm];
    for (i=1;i<=itype;i++) { /* Look for existing type */
         atm1_type_label = maps->atm_label[atm1_type[i]];
         atm2_type_label = maps->atm_label[atm2_type[i]];
         if  ( strcasecmp(atm1_type_label,iatm_label)==0 ) {
	       ntype_iatm++;
               if (ntype_iatm>ntype_iatm_mall) {
    	          ntype_iatm_mall += NMEM_MIN;
    	          type_iatm   = (int *)realloc(type_iatm+1, ntype_iatm_mall*sizeof(int))-1;
               }/*end if*/
		     type_iatm[ntype_iatm] = i;
               type_iatm_order[ntype_iatm] = 1;
	      }/*end if*/
         if  ( strcasecmp(atm2_type_label,iatm_label)==0 ) {
	       ntype_iatm++;
               if (ntype_iatm>ntype_iatm_mall) {
    	          ntype_iatm_mall += NMEM_MIN;
    	          type_iatm   = (int *)realloc(type_iatm+1, ntype_iatm_mall*sizeof(int))-1;
               }/*end if*/
	       type_iatm[ntype_iatm] = i;
               type_iatm_order[ntype_iatm] = 2;
	      }/*end if*/
    }/*end for*/

    
    for (jatm=iatm+1; jatm<=natm_tot; jatm++) {
      valid = 1;
      for (iexclude=1;iexclude<=nexclude_iatm;iexclude++) {
	      if (exclude_iatm_order[iexclude]==1) {
		  if (jatm==exclude_atom2[exclude_iatm[iexclude]]) { valid = 0; break; }
           }/*endif*/
	      if (exclude_iatm_order[iexclude]==2) {
		  if (jatm==exclude_atom1[exclude_iatm[iexclude]]) { valid = 0; break; }
           }/*endif*/
      }/*end for*/

      if (rexclude_length_defined == 1) {
          rexclude_length2 = rexclude_length*rexclude_length;
          edx = pos->x[iatm] - pos->x[jatm];
          edy = pos->y[iatm] - pos->y[jatm];
          edz = pos->z[iatm] - pos->z[jatm];
          er2 = edx*edx + edy*edy + edz*edz;
          if (er2 > rexclude_length2) {valid = 0;}
      }/*end if*/

      if (valid==1) {
  	  iinter++;
	  if (iinter>ninter_mall) {
	    ninter_mall += NMEM_MIN;
	    atom1      = (int *)realloc(atom1+1, ninter_mall*sizeof(int))-1;
	    atom2      = (int *)realloc(atom2+1, ninter_mall*sizeof(int))-1;
            inter_off  = (int *)realloc(inter_off+1, ninter_mall*sizeof(int))-1;
            type_index = (int *)realloc(type_index+1, ninter_mall*sizeof(int))-1;
	  }/*end if*/
          inter_off[iinter] = 0;
	  atom1[iinter] = iatm;
	  atom2[iinter] = jatm;
          inter_index[iatm][jatm] = iinter;

          type_found = 0;
          jatm_found =  0;
          jatm_label =  maps->atm_label[jatm];


          for (jj=1;jj<=ntype_jatm;jj++) {
               if (type_jatm_order[jj] == 1) {
                   jatm_type_label = maps->atm_label[atm1_type[type_jatm[jj]]];
               }/*endif*/
               if (type_jatm_order[jj] == 2) {
                   jatm_type_label = maps->atm_label[atm2_type[type_jatm[jj]]];
               }/*endif*/
	       if (strcasecmp(jatm_label,jatm_type_label)==0) { 
		   jatm_found = 1; 
                   type_index[iinter] = type_jatm[jj];
                   type_found = 1; 
                   break;
               }/*endif*/
          }/*endfor*/
          if (jatm_found == 0) {
	      ntype_jatm++;
              if (ntype_jatm>ntype_jatm_mall) {
   	          ntype_jatm_mall += NMEM_MIN;
   	          type_jatm   = (int *)realloc(type_jatm+1, ntype_jatm_mall*sizeof(int))-1;
              }/*end if*/              
          }/*endfor*/

          if (type_found == 0) {
             for (ii =1;ii<=ntype_iatm;ii++) { /* Look for existing type */
    	       i = type_iatm[ii];
    	       if ( type_iatm_order[ii] == 1 ) {
                       atm2_type_label = maps->atm_label[atm2_type[i]];
                       if  (strcasecmp(atm2_type_label,jatm_label)==0) {
    	 	            type_index[iinter] = i;
                            type_jatm[ntype_jatm] = i;
                            type_jatm_order[ntype_jatm] = 2;
                            type_found = 1;
                            break;
    		    }/*endif*/
                  }/*endif*/
    	       if ( type_iatm_order[ii] == 2 ) {
                       atm1_type_label = maps->atm_label[atm1_type[i]];
                       if  (strcasecmp(atm1_type_label,jatm_label)==0) {
    		            type_index[iinter] = i;
                            type_jatm[ntype_jatm] = i;
                            type_jatm_order[ntype_jatm] = 1;
                            type_found = 1;
                            break;
    		    }/*endif*/
                  }/*endif*/
             }/*end for*/
          }/*endif*/

          if (type_found==0) {/* Adding new type */
             itype++;
             if (itype>ntype_mall) {
	        ntype_mall += NMEM_MIN;
	        atm1_type = (int *)realloc(atm1_type+1, ntype_mall*sizeof(int))-1;
	        atm2_type = (int *)realloc(atm2_type+1, ntype_mall*sizeof(int))-1;
             }/*end if*/
             atm1_type[itype] = iatm;
             atm2_type[itype] = jatm;
             type_index[iinter] = itype;
             type_jatm[ntype_jatm] = itype;
             type_jatm_order[ntype_jatm] = 2;
	  }/*end if*/
      }/*end if: valid==1*/
    }/*end for: jatm */
    nexclude_iatm = 0;
  }/*end for: iatm */
  ninter = iinter;
  ntype  = itype;


/*-------------------------------------------------------------------------*/
/* II) Find interactions in database                                       */

  database_entry_map = new const Pot_data_entry*[ntype]-1;
  for (itype=1;itype<=ntype;itype++) {
	atoms[1] = atm1_type[itype];                                  
	atoms[2] = atm2_type[itype];
	exact_match_entry = NULL;
	wild_match_entry  = NULL;
	database_entry    = database;
	inter_check = 0;
	while (database_entry!=NULL && inter_check!=1) {
	  inter_check = database_entry->match(maps, atoms, 0);
	  if (inter_check==1) { exact_match_entry = database_entry; }
	  if (inter_check==2) { wild_match_entry  = database_entry; }
	  database_entry = database_entry->next;
	}/*end while*/
	if (exact_match_entry==NULL && wild_match_entry==NULL) {
           sprintf(err_msg, "Inter pair is not found in database.\nindices %d %d\nlabels %s %s",
	   atoms[1], atoms[2], maps->atm_label[atoms[1]], maps->atm_label[atoms[2]]);
           errorMessage(err_msg);
	}/*end if*/
	if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
	else                         { database_entry = wild_match_entry; }
        database_entry_map[itype] = database_entry;
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* III) Find all null inters                                               */

  for (iinter=1;iinter<=ninter;iinter++) {
      itype = type_index[iinter];
      database_entry =  database_entry_map[itype];
      if (database_entry->pot_type==0) {inter_off[iinter] = 1;}
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* IV) Allocate and fill in variables                                      */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.1) From database                                                     */  

  type         = new int[ntype]-1;
  numeric_size = new int[ntype]-1;
  hash_table   = new int[ntype]-1;
  hash_size    = new int[ntype]-1;
  eps   = new double[ntype]-1;
  feps  = new double[ntype]-1;
  sig   = new double[ntype]-1;
  beta  = new double[ntype]-1;
  scale = new double[ntype]-1;
  r0    = new double[ntype]-1;
  awill = new double[ntype]-1;
  bwill = new double[ntype]-1;
  c6m   = new double[ntype]-1;
  c8m   = new double[ntype]-1;
  c10m  = new double[ntype]-1;
  r_min = new double[ntype]-1;
  r_max = new double[ntype]-1;
  r_res = new double[ntype]-1;
  numeric_V_x   = new double*[ntype]-1;
  numeric_d2V_x = new double*[ntype]-1;
  numeric_x     = new double*[ntype]-1;


  for (itype=1; itype<=ntype; itype++) {
    database_entry = database_entry_map[itype];
    type[itype] = database_entry->pot_type;
    hash_table[itype] = database_entry->hash_table;
    hash_size[itype]  = database_entry->hash_size;
    eps[itype]  = database_entry->eps;
    feps[itype] = 4.0*database_entry->eps;
    sig[itype]  = database_entry->sig;
    beta[itype] = database_entry->beta;
    scale[itype]= database_entry->scale;
    r0[itype]   = database_entry->r0;
    awill[itype]= database_entry->awill;
    bwill[itype]= database_entry->bwill;
    c6m[itype]  = database_entry->c6m;
    c8m[itype]  = database_entry->c8m;
    c10m[itype] = database_entry->c10m;
    r_min[itype]= database_entry->r_min;
    r_max[itype]= database_entry->r_max;
    r_res[itype]= database_entry->r_res;
    if (type[itype] == 4) {/*numeric type*/
       if (nprw == 0) { 
         numeric_size[itype] = database_entry->numeric_size;  
         numeric_V_x[itype]   = new double[numeric_size[itype]] - 1;
         numeric_d2V_x[itype] = new double[numeric_size[itype]] - 1;
         numeric_x[itype]     = new double[numeric_size[itype]] - 1;     
         for (iii=1;iii<=numeric_size[itype];iii++) {
            numeric_x[itype][iii]     = database_entry->numeric_x[iii];    
            numeric_V_x[itype][iii]   = database_entry->numeric_V_x[iii];
            numeric_d2V_x[itype][iii] = database_entry->numeric_d2V_x[iii];
         }
       } else if (nprw == 1) {
         numeric_size[itype] = database_entry->numeric_size;  
         numeric_V_x[itype]   = new double[numeric_size[itype]] - 1;
         numeric_x[itype]     = new double[numeric_size[itype]] - 1;     
         for (iii=1;iii<=numeric_size[itype];iii++) {
            numeric_x[itype][iii]     = database_entry->numeric_x[iii];    
            numeric_V_x[itype][iii]   = database_entry->numeric_V_x[iii];
         }
         numeric_d2V_x[itype] = new double[3*numeric_size[itype]] - 1;
         for (iii=1;iii<=3*numeric_size[itype];iii++) {
            numeric_d2V_x[itype][iii] = database_entry->numeric_d2V_x[iii];
         }
       }/*endif*/ 
    } else {
       numeric_V_x[itype]   = NULL;
       numeric_d2V_x[itype] = NULL;
       numeric_x[itype]     = NULL;
    }
  }

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.2) Derived from database                                             */  

  r_min2 = new double[ntype]-1;
  r_max2 = new double[ntype]-1;

  hash_delta    = new double[ntype]-1;
  hash_numeric_V_x   = new double*[ntype]-1;
  hash_numeric_dV_x  = new double*[ntype]-1;
  hash_numeric_pre_x = new double*[ntype]-1;

  for (itype=1; itype<=ntype; itype++) {
       r_min2[itype]= r_min[itype]*r_min[itype];
       r_max2[itype]= r_max[itype]*r_max[itype];
       if (hash_table[itype] != 0) {
           hash_numeric_V_x[itype]   = new double[hash_size[itype]] - 1;
           hash_numeric_dV_x[itype]  = new double[hash_size[itype]] - 1;
           hash_numeric_pre_x[itype] = new double[hash_size[itype]] - 1;     
       } else {
           hash_numeric_V_x[itype]   = NULL;
           hash_numeric_dV_x[itype]  = NULL;
           hash_numeric_pre_x[itype] = NULL;
       }
  }

/*-------------------------------------------------------------------------*/
/* V) Inter lists                                                          */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* V.1) Linked list                                                        */  

  if (lnk_list) {/*if linked list is used*/
      excl_num_mall  = NMEM_MIN;

      ndim = 3;
      excl_num    = new int[natm_tot]-1;
      lbox_ll     = new double[ndim]-1;
      rcell_ll    = new double[ndim]-1;
      ncell_ll    = new int[ndim]-1;
      vcell_ll    = new int[ndim]-1;
      vcell1_ll   = new int[ndim]-1;
      lscl_ll     = new int[natm_tot]-1;
      excl_list   = imatrix(1,excl_num_mall,1,natm_tot);
     
      rcut_ll = r_max[1];
      for (itype=2; itype<=ntype; itype++) {
	   rcut_ll = MAX(r_max[itype],rcut_ll);
      }/*end for*/
      lbox_ll[1]  = pos->boxl; // + rcut_ll; /* x */
      lbox_ll[2]  = pos->boxw; // + rcut_ll; /* y */
      lbox_ll[3]  = pos->boxh; // + rcut_ll; /* z */
      ncell_ll[1] = (int)floor(lbox_ll[1]/rcut_ll); /* x */
      ncell_ll[2] = (int)floor(lbox_ll[2]/rcut_ll); /* y */
      ncell_ll[3] = (int)floor(lbox_ll[3]/rcut_ll); /* z */
      rcell_ll[1] =  lbox_ll[1]/((double)ncell_ll[1]); /* x */
      rcell_ll[2] =  lbox_ll[2]/((double)ncell_ll[2]); /* y */
      rcell_ll[3] =  lbox_ll[3]/((double)ncell_ll[3]); /* z */
      ncell_tot   =  ncell_ll[1]*ncell_ll[2]*ncell_ll[3];  
      head_ll     =  new int[ncell_tot]-1;

      for (jatm=1;jatm<=natm_tot;jatm++) {
	 excl_num[jatm] = 0;  
         for (iexclude=1;iexclude<=nexclude;iexclude++) {
	      if ( (jatm==exclude_atom1[iexclude]) &&
	           (jatm >exclude_atom2[iexclude]) ) {
   		    excl_num[jatm]++;
                    if (excl_num[jatm]>excl_num_mall) {
                       sprintf(err_msg,"Allocate more for excl_num_mall!");
                       sprintf(err_msg,"excl_num[%d]=%d>excl_num_mall=%d",
                                        jatm,excl_num[jatm],excl_num_mall);
                       errorMessage(err_msg);
                    }/*end if*/
                    excl_list[excl_num[jatm]][jatm]=exclude_atom2[iexclude];
              }/*end if*/               
              if  ( (jatm==exclude_atom2[iexclude]) &&
	            (jatm >exclude_atom1[iexclude]) )  {
   		    excl_num[jatm]++;
                    if (excl_num[jatm]>excl_num_mall) {
                       sprintf(err_msg,"Allocate more for excl_num_mall!");
                       sprintf(err_msg,"excl_num[%d]=%d>excl_num_mall=%d",
                                        jatm,excl_num[jatm],excl_num_mall);
                       errorMessage(err_msg);
                    }/*end if*/
                    excl_list[excl_num[jatm]][jatm]=exclude_atom1[iexclude];
              }/*end if*/            
         }/*end for*/
      }/*end for*/
  }/*end if*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* V.2) Neighbor list                                                      */  

  if (neighbor_list) { 
      neighbor_list_update = 1;
      neighbors = new int[ninter] - 1;  
      neighbor_list_cutoff = r_max[1];
      for (itype=2; itype<=ntype; itype++) {
	   neighbor_list_cutoff = MAX(r_max[itype],neighbor_list_cutoff);
      }/*end for*/
      neighbor_list_cutoff += neighbor_list_skin;
      neighbor_list_cutoff2 = neighbor_list_cutoff*neighbor_list_cutoff;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* VI) Setup Distance dependent dielectric                                 */
   
  if (ddd==1) {
      ddd_p1 = ddd_D;
      ddd_p2 = ddd_D0;
      ddd_p3 = ddd_S;
  } else if (ddd==2) {
      ddd_p1 = ddd_c;
      ddd_p2 = ddd_e;
  }/* end if*/

/*-------------------------------------------------------------------------*/
/* VII) Allocate initialize scale_iinter                                   */

  scale_iinter = new double[ninter] - 1;
  for (iinter=1;iinter<=ninter;iinter++) {scale_iinter[iinter] = 1.0;}  

/*-------------------------------------------------------------------------*/
/* VIII) Fill hash tables and related variables                            */

  nhash = 0;
  for (itype=1; itype<=ntype; itype++) {
    
    if (hash_table[itype] != 0) {   
        nhash++;
        if (hash_table[itype] == 1) {
            hash_delta[itype]  = (r_max[itype]-r_min[itype]);
        }
        else if (hash_table[itype] == 2) {
            hash_delta[itype]  = (r_max2[itype]-r_min2[itype]);
        }/*end if*/
        hash_delta[itype] /= ((double)hash_size[itype]);
    }/*endif*/

    if ((hash_table[itype]!=0) && (type[itype] == 4)) {/*numeric type */
        for (iii=1;iii<=hash_size[itype];iii++) {
          if (hash_table[itype] == 1) 
	    {r = r_min[itype] + ((double)(iii-1))*hash_delta[itype]; }
          if (hash_table[itype] == 2) 
	    {r2 = r_min2[itype] + ((double)(iii-1))*hash_delta[itype];
             r=sqrt(r2);                                         }
          num_size  = numeric_size[itype];
          num_x     = numeric_x[itype];
          num_V_x   = numeric_V_x[itype];
          num_d2V_x = numeric_d2V_x[itype];
          splint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
          dsplint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&dvnow);
          switch_on = 0;
          if (r >= (r_max[itype]-rswitch_length)) {
              switch_on = 1;
              switch_type = 4;
              asw = r_max[itype]-rswitch_length;
              bsw = r_max[itype];
          }/*end if*/
          if (switch_on==1) {
            dvnow   =  dvnow*switch_right(r,asw,bsw,switch_type)
                      +vnow*dx_switch_right(r,asw,bsw,switch_type);
            vnow    =  vnow*switch_right(r,asw,bsw,switch_type);
          }/*end if*/
          hash_numeric_V_x[itype][iii]   =  vnow;
          if (hash_table[itype] == 1) 
	     {hash_numeric_dV_x[itype][iii]  =  dvnow;}
          if (hash_table[itype] == 2) 
	     {hash_numeric_dV_x[itype][iii]  =  dvnow/(2.0*r);}
	  hash_numeric_pre_x[itype][iii] = -dvnow/r;
        }/*end for*/
    }/*endif*/

  }/*end for*/

  if (nhash==ntype) {hash_all  = 1;}    /* hash table for each interaction */
  else              {hash_all  = 0;}    /* not for all                     */


/*-------------------------------------------------------------------------*/
/* IX) DEBUG                                                               */

#ifdef DEBUG
  for (iinter=1; iinter<=ninter; iinter++) {
    printf("Inter pair %2d: %d(%s)--%d(%s)\n",iinter,
	   atom1[iinter], maps->atm_label[atom1[iinter]],
	   atom2[iinter], maps->atm_label[atom2[iinter]]);
    if (type[type_index[iinter]]==1 || type[type_index[iinter]]==2) {
      printf("\t\t{  eps=%.8g, sig=%.8g, beta=%.8g}\n",
	     eps[type_index[iinter]],sig[type_index[iinter]],beta[type_index[iinter]]);
      printf("\t\t{scale=%.8g,  r0=%.8g}\n",
	     scale[type_index[iinter]], r0[type_index[iinter]]);
    }
    if (type[type_index[iinter]]==3) {
      printf("\t\t{awill=%.8g, bwill=%.8g }\n", awill[type_index[iinter]], bwill[type_index[iinter]]);
      printf("\t\t{   c6=%.8g,    c8=%.8g, c10=%.8g}\n", 
	     c6m[type_index[iinter]], c8m[type_index[iinter]], c10m[type_index[iinter]]);
    }
    if (type[type_index[iinter]]==4) {
        printf("numeric_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_x[type_index[iinter]][iii]);
        }
        printf("\n");
        printf("numeric_V_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_V_x[type_index[iinter]][iii]);
        }
        printf("\n");
        printf("numeric_d2V_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_d2V_x[type_index[iinter]][iii]);
        }
        printf("\n");
    }
  }
#endif

/*-------------------------------------------------------------------------*/
/* VII) Deallocate memory                                                  */

#define DEALLOCATE_OFF
#ifdef DEALLOCATE  
  delete (database_entry_map+1);
#endif  

  printf("Inter-mol setting up done.\n");
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Inter::setup(const Pot_data_entry *database, const Maps *maps, 
               		                          const System *system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, iii, iatm, jatm, katm, latm, iinter, jinter, itype, n;
  int ibond, ibond_atm, jbond_atm, kbond_atm, lbond_atm;
  int ibend, ionfo, itors;
  int natm_tot, ndim;
  int atom1_ind, atom2_ind;
  int ninter_mall, ntype_mall;
  int valid, action_type, type_found, inter_check;
  int inter_on_3bond_conn, inter_on_4bond_conn;
  int *nbond_atm, **bond_atm_map;
  int atoms[3];
  int *atm1_type, *atm2_type;

  int nhash;
  int switch_on, switch_type;
  double vnowtmp, asw,bsw;

  int    num_size;          
  double r, r2, vnow, dvnow;
  double *num_x, *num_V_x, *num_d2V_x;

  char *atm1_type_label,*atm2_type_label;
  char **atm_label, *atm1_label, *atm2_label;
  char err_msg[LINE_NCHARMAX];

  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;
  const Pot_data_entry **database_entry_map;

  printf("----------------------------------------------\n");
  printf("Setting up inter-molecular interactions\n...\n");
  
  action_type = system->general.action_type;
  natom       = system->pos.natm_tot; /* class*/
  natm_tot    = maps->natm_tot;       /* local*/
  atm_label   = maps->atm_label;
  lnk_list    = system->general.lnk_list;
  neighbor_list = system->general.neighbor_list;

/*-------------------------------------------------------------------------*/
/* I) Get the interacting pairs */

  ninter_mall = NMEM_MIN;
  ntype_mall  = NMEM_MIN;
  excl_num_mall  = NMEM_MIN;

  atom1       = new int[ninter_mall]-1;
  atom2       = new int[ninter_mall]-1;
  inter_off   = new int[ninter_mall]-1;
  type_index  = new int[ninter_mall]-1;
  atm1_type   = new int[ntype_mall]-1;
  atm2_type   = new int[ntype_mall]-1;
  excl_num    = new int[natm_tot]-1;

  excl_list   = imatrix(1,excl_num_mall,1,natm_tot);
  inter_index = imatrix(1,natm_tot,1,natm_tot);
  
  nbond_atm    = system->bond.nbond_atm;
  bond_atm_map = system->bond.bond_atm_map;
 
  inter_on_3bond_conn = system->general.inter_on_3bond_conn;
  inter_on_4bond_conn = system->general.inter_on_4bond_conn;
 
  for (iatm=1; iatm<=natm_tot; iatm++) {
    for (jatm=1; jatm<=natm_tot; jatm++) {
        inter_index[iatm][jatm] = 0;
    }/*end for*/
  }/*end for*/

  iinter = 0;
  itype = 0;
  for (jatm=1; jatm<=natm_tot; jatm++) {excl_num[jatm] = 0;}
  for (iatm=1; iatm<=natm_tot; iatm++) {
    for (jatm=iatm+1; jatm<=natm_tot; jatm++) {
      valid = 1;
      /*Check direct bond connection: iatm--jatm*/
      for (ibond_atm=1; ibond_atm<=nbond_atm[iatm]; ibond_atm++) {
	if (jatm==bond_atm_map[iatm][ibond_atm]) { 
	  valid = 0;
	  break;  /*Not necessary to continue. */
	}/*end if*/
      }/*end for*/
      /*Check three-atom bond connection: iatm--katm--jatm */
      if ((valid==1)&&(inter_on_3bond_conn==0)) {
	for (ibond_atm=1; ibond_atm<=nbond_atm[iatm]; ibond_atm++) {
	  katm=bond_atm_map[iatm][ibond_atm];
	  for (kbond_atm=1; kbond_atm<=nbond_atm[katm]; kbond_atm++) {
	    if(jatm== bond_atm_map[katm][kbond_atm]) {
	      valid = 0;
	      break;  /*Not necessary to continue. */
	    }/*end if*/
	  }/*end for*/ 
	}/*end for*/
        for (ibend=1; ibend<=system->bend.nbend; ibend++) {
	  if ((iatm==system->bend.atom1[ibend] ||
	       iatm==system->bend.atom2[ibend] ||
	       iatm==system->bend.atom3[ibend]   ) &&
	      (jatm==system->bend.atom1[ibend] ||
	       jatm==system->bend.atom2[ibend] ||
	       jatm==system->bend.atom3[ibend]   )  ) { valid = 0;}
        }/*end for*/
      }/*end if*/
      /*Check four-atom bond connection: iatm--katm--latm--jatm */
      if ((valid==1)&&(inter_on_3bond_conn==0)&&(inter_on_4bond_conn==0)) {
	for (ibond_atm=1; ibond_atm<=nbond_atm[iatm]; ibond_atm++) {
	  katm=bond_atm_map[iatm][ibond_atm];
	  for (kbond_atm=1; kbond_atm<=nbond_atm[katm]; kbond_atm++) {
	    latm=bond_atm_map[katm][kbond_atm];
	    for (lbond_atm=1; lbond_atm<=nbond_atm[latm]; lbond_atm++) {
	      if (jatm==bond_atm_map[latm][lbond_atm]) {
		valid = 0;
		break;  /*Not necessary to continue. */
	      }/*end if*/
	    }/*end for*/ 
	  }/*end for*/
	}/*end for*/
        for (ionfo=1; ionfo<=system->onfo.nonfo; ionfo++) {
	  if ((iatm==system->onfo.atom1[ionfo] ||
	       iatm==system->onfo.atom2[ionfo]   ) &&
              (jatm==system->onfo.atom1[ionfo] ||
	       jatm==system->onfo.atom2[ionfo]   )  ) { valid = 0;}
        }/*end for*/
        for (itors=1; itors<=system->tors.ntors; itors++) {
	  if ((iatm==system->tors.atom1[itors] ||
	       iatm==system->tors.atom2[itors] ||
	       iatm==system->tors.atom3[itors] ||
	       iatm==system->tors.atom4[itors]   ) &&
	      (jatm==system->tors.atom1[itors] ||
	       jatm==system->tors.atom2[itors] ||
	       jatm==system->tors.atom3[itors] ||
	       jatm==system->tors.atom4[itors]   )  ) { valid = 0;}
        }/*end for*/
      }/*end if*/

      if (valid==0) {
          excl_num[jatm]++;
          if (excl_num[jatm]>excl_num_mall) {
              sprintf(err_msg,"Allocate more for excl_num_mall!");
              sprintf(strchr(err_msg,0),"excl_num[%d]=%d>excl_num_mall=%d",
                                        jatm,excl_num[jatm],excl_num_mall);
              errorMessage(err_msg);
          }/*end if*/
          excl_list[excl_num[jatm]][jatm]=iatm;
      }/*end if*/

      if ((valid==1)||(action_type==3)||(action_type==4)) {
  	  iinter++;                                        
	  if (iinter>ninter_mall) {
	    ninter_mall += NMEM_MIN;
	    atom1      = (int *)realloc(atom1+1, ninter_mall*sizeof(int))-1;
	    atom2      = (int *)realloc(atom2+1, ninter_mall*sizeof(int))-1;
            inter_off  = (int *)realloc(inter_off+1, ninter_mall*sizeof(int))-1;
            type_index = (int *)realloc(type_index+1, ninter_mall*sizeof(int))-1;
	  }/*end if*/
          inter_off[iinter] = 0;
          if (valid == 0) {inter_off[iinter] = 1;}
	  atom1[iinter] = iatm;
	  atom2[iinter] = jatm;
          inter_index[iatm][jatm] = iinter;
          type_found = 0;
          for (i=1;i<=itype;i++) { /* Look for existing type */
	      atm1_type_label = maps->atm_label[atm1_type[i]];
              atm2_type_label = maps->atm_label[atm2_type[i]];
              atm1_label =  maps->atm_label[iatm];                            
              atm2_label =  maps->atm_label[jatm];
              if ( ( strcasecmp(atm1_type_label,atm1_label)==0 &&
		     strcasecmp(atm2_type_label,atm2_label)==0 ) || 
                   ( strcasecmp(atm1_type_label,atm2_label)==0 &&
		     strcasecmp(atm2_type_label,atm1_label)==0 )  ) {
		    type_index[iinter] = i;
                    type_found = 1;
                    break;
	      }/*end if*/              
          }/*end for*/
          if (type_found==0) {/* Adding new type */
             itype++;
             if (itype>ntype_mall) {
                ntype_mall += NMEM_MIN; 
	        atm1_type = (int *)realloc(atm1_type+1, ntype_mall*sizeof(int))-1;
	        atm2_type = (int *)realloc(atm2_type+1, ntype_mall*sizeof(int))-1;
             }/*end if*/
             atm1_type[itype] = iatm;
             atm2_type[itype] = jatm;
             type_index[iinter] = itype;
	  }/*end if*/
      }/*end if: valid==1*/
    }/*end for: jatm */
  }/*end for: iatm */
  ninter = iinter;
  ntype  = itype;

/*-------------------------------------------------------------------------*/
/* II) Find interactions in database                                       */

  database_entry_map = new const Pot_data_entry*[ntype]-1;
  for (itype=1;itype<=ntype;itype++) {
	atoms[1] = atm1_type[itype];                                  
	atoms[2] = atm2_type[itype];
	exact_match_entry = NULL;
	wild_match_entry  = NULL;
	database_entry    = database;
	inter_check = 0;
	while (database_entry!=NULL && inter_check!=1) {
	  inter_check = database_entry->match(maps, atoms, 0);
	  if (inter_check==1) { exact_match_entry = database_entry; }
	  if (inter_check==2) { wild_match_entry  = database_entry; }
	  database_entry = database_entry->next;
	}/*end while*/
	if (exact_match_entry==NULL && wild_match_entry==NULL) {
	   sprintf(err_msg,"Inter pair is not found in database.\n");
           sprintf(strchr(err_msg,0),"indices %d %d\nlabels %s %s",
	               atoms[1],atoms[2],maps->atm_label[atoms[1]], 
                                         maps->atm_label[atoms[2]]);
           errorMessage(err_msg);
	}/*end if*/
	if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
	else                         { database_entry = wild_match_entry; }
        database_entry_map[itype] = database_entry;
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* III) Find all null inters                                               */

  for (iinter=1;iinter<=ninter;iinter++) {
      itype = type_index[iinter];
      database_entry =  database_entry_map[itype];
      if (database_entry->pot_type==0) {inter_off[iinter] = 1;}
  }/*end for*/


/*-------------------------------------------------------------------------*/
/* IV) Allocate and fill in variables                                      */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.1) From database                                                     */  

  type         = new int[ntype]-1;
  numeric_size = new int[ntype]-1;
  hash_table   = new int[ntype]-1;
  hash_size    = new int[ntype]-1;
  eps   = new double[ntype]-1;
  feps  = new double[ntype]-1;
  sig   = new double[ntype]-1;
  beta  = new double[ntype]-1;
  scale = new double[ntype]-1;
  r0    = new double[ntype]-1;
  awill = new double[ntype]-1;
  bwill = new double[ntype]-1;
  c6m   = new double[ntype]-1;
  c8m   = new double[ntype]-1;
  c10m  = new double[ntype]-1;
  r_min = new double[ntype]-1;
  r_max = new double[ntype]-1;
  r_res = new double[ntype]-1;
  numeric_V_x   = new double*[ntype]-1;
  numeric_d2V_x = new double*[ntype]-1;
  numeric_x     = new double*[ntype]-1;

  for (itype=1; itype<=ntype; itype++) {
    database_entry = database_entry_map[itype];
    type[itype] = database_entry->pot_type;
    hash_table[itype] = database_entry->hash_table;
    hash_size[itype]  = database_entry->hash_size;
    eps[itype]  = database_entry->eps;
    feps[itype] = 4.0*database_entry->eps;
    sig[itype]  = database_entry->sig;
    beta[itype] = database_entry->beta;
    scale[itype]= database_entry->scale;
    r0[itype]   = database_entry->r0;
    awill[itype]= database_entry->awill;
    bwill[itype]= database_entry->bwill;
    c6m[itype]  = database_entry->c6m;
    c8m[itype]  = database_entry->c8m;
    c10m[itype] = database_entry->c10m;
    r_min[itype]= database_entry->r_min;
    r_max[itype]= database_entry->r_max;
    r_res[itype]= database_entry->r_res;
    if (type[itype] == 4) {/*numeric type*/
       if (nprw == 0) { 
         numeric_size[itype] = database_entry->numeric_size;  
         numeric_V_x[itype]   = new double[numeric_size[itype]] - 1;
         numeric_d2V_x[itype] = new double[numeric_size[itype]] - 1;
         numeric_x[itype]     = new double[numeric_size[itype]] - 1;     
         for (iii=1;iii<=numeric_size[itype];iii++) {
            numeric_x[itype][iii]     = database_entry->numeric_x[iii];    
            numeric_V_x[itype][iii]   = database_entry->numeric_V_x[iii];
            numeric_d2V_x[itype][iii] = database_entry->numeric_d2V_x[iii];
         }
       } else if (nprw == 1) {
         numeric_size[itype] = database_entry->numeric_size;  
         numeric_V_x[itype]   = new double[numeric_size[itype]] - 1;
         numeric_x[itype]     = new double[numeric_size[itype]] - 1;     
         for (iii=1;iii<=numeric_size[itype];iii++) {
            numeric_x[itype][iii]     = database_entry->numeric_x[iii];    
            numeric_V_x[itype][iii]   = database_entry->numeric_V_x[iii];
         }
         numeric_d2V_x[itype] = new double[3*numeric_size[itype]] - 1;
         for (iii=1;iii<=3*numeric_size[itype];iii++) {
            numeric_d2V_x[itype][iii] = database_entry->numeric_d2V_x[iii];
         }
       }/*endif*/
    } else {
       numeric_V_x[itype]   = NULL;
       numeric_d2V_x[itype] = NULL;
       numeric_x[itype]     = NULL;
    }
  }

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.2) Derived from database                                             */

  r_min2 = new double[ntype]-1;
  r_max2 = new double[ntype]-1;
  hash_delta         = new double[ntype]-1;
  hash_numeric_V_x   = new double*[ntype]-1;
  hash_numeric_dV_x  = new double*[ntype]-1;
  hash_numeric_pre_x = new double*[ntype]-1;

  for (itype=1; itype<=ntype; itype++) {
       r_min2[itype]= r_min[itype]*r_min[itype];
       r_max2[itype]= r_max[itype]*r_max[itype];
       if (hash_table[itype] != 0) {
           hash_numeric_V_x[itype]   = new double[hash_size[itype]] - 1;
           hash_numeric_dV_x[itype]  = new double[hash_size[itype]] - 1;
           hash_numeric_pre_x[itype] = new double[hash_size[itype]] - 1;
       } else {
           hash_numeric_V_x[itype]   = NULL;
           hash_numeric_dV_x[itype]  = NULL;
           hash_numeric_pre_x[itype] = NULL;
       }
  }

/*-------------------------------------------------------------------------*/
/* V) Inter list                                                           */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* V.1) Linked list                                                        */  

  if (lnk_list) {/*if linked list is used*/

      ndim = 3;
      lbox_ll     = new double[ndim]-1;
      rcell_ll    = new double[ndim]-1;
      ncell_ll    = new int[ndim]-1;
      vcell_ll    = new int[ndim]-1;
      vcell1_ll   = new int[ndim]-1;
      lscl_ll     = new int[natm_tot]-1;
     
      rcut_ll = r_max[1];
      for (itype=2; itype<=ntype; itype++) {
	   rcut_ll = MAX(r_max[itype],rcut_ll);
      }/*end for*/
      lbox_ll[1]  = system->pos.boxl + rcut_ll; /* x */
      lbox_ll[2]  = system->pos.boxw + rcut_ll; /* y */
      lbox_ll[3]  = system->pos.boxh + rcut_ll; /* z */
      ncell_ll[1] = (int)floor(lbox_ll[1]/rcut_ll); /* x */
      ncell_ll[2] = (int)floor(lbox_ll[2]/rcut_ll); /* y */
      ncell_ll[3] = (int)floor(lbox_ll[3]/rcut_ll); /* z */
      rcell_ll[1] =  lbox_ll[1]/((double)ncell_ll[1]); /* x */
      rcell_ll[2] =  lbox_ll[2]/((double)ncell_ll[2]); /* y */
      rcell_ll[3] =  lbox_ll[3]/((double)ncell_ll[3]); /* z */
      ncell_tot   =  ncell_ll[1]*ncell_ll[2]*ncell_ll[3];  
      head_ll     =  new int[ncell_tot]-1;

  }/*end if*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* V.2) Neighbor list                                                      */  

  if (neighbor_list) { 
      neighbor_list_update = 1;
      neighbors = new int[ninter] - 1;  
      neighbor_list_cutoff = r_max[1];
      for (itype=2; itype<=ntype; itype++) {
	   neighbor_list_cutoff = MAX(r_max[itype],neighbor_list_cutoff);
      }/*end for*/
      neighbor_list_cutoff += neighbor_list_skin;
      neighbor_list_cutoff2 = neighbor_list_cutoff*neighbor_list_cutoff;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* VI) Setup Distance dependent dielectric                                 */
   
  if (ddd==1) {
      ddd_p1 = ddd_D;
      ddd_p2 = ddd_D0;
      ddd_p3 = ddd_S;
  } else if (ddd==2) {
      ddd_p1 = ddd_c;
      ddd_p2 = ddd_e;
  }/* end if*/

/*-------------------------------------------------------------------------*/
/* VII) Allocate initialize scale_iinter                                   */

  scale_iinter = new double[ninter] - 1;
  for (iinter=1;iinter<=ninter;iinter++) {scale_iinter[iinter] = 1.0;}  


/*-------------------------------------------------------------------------*/
/* VIII) Fill hash tables and related variables                            */

  nhash = 0;
  for (itype=1; itype<=ntype; itype++) {
    
    if (hash_table[itype] != 0) {   
        nhash++;
        if (hash_table[itype] == 1) {
            hash_delta[itype]  = (r_max[itype]-r_min[itype]);
        }
        else if (hash_table[itype] == 2) {
            hash_delta[itype]  = (r_max2[itype]-r_min2[itype]);
        }/*end if*/
        hash_delta[itype] /= ((double)hash_size[itype]);
    }/*endif*/

    if ((hash_table[itype]!=0) && (type[itype] == 4)) {/*numeric type */
        for (iii=1;iii<=hash_size[itype];iii++) {
          if (hash_table[itype] == 1) 
	    {r = r_min[itype] + ((double)(iii-1))*hash_delta[itype]; }
          if (hash_table[itype] == 2) 
	    {r2 = r_min2[itype] + ((double)(iii-1))*hash_delta[itype];
             r=sqrt(r2);                                         }
          num_size  = numeric_size[itype];
          num_x     = numeric_x[itype];
          num_V_x   = numeric_V_x[itype];
          num_d2V_x = numeric_d2V_x[itype];
          splint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&vnow);
          dsplint(&(num_x[0]),&(num_V_x[0]),&(num_d2V_x[0]),num_size,r,&dvnow);
          switch_on = 0;
          if (r >= (r_max[itype]-rswitch_length)) {
              switch_on = 1;
              switch_type = 4;
              asw = r_max[itype]-rswitch_length;
              bsw = r_max[itype];
          }/*end if*/
          if (switch_on==1) {
            dvnow   =  dvnow*switch_right(r,asw,bsw,switch_type)
                      +vnow*dx_switch_right(r,asw,bsw,switch_type);
            vnow    =  vnow*switch_right(r,asw,bsw,switch_type);
          }/*end if*/
          hash_numeric_V_x[itype][iii]   =  vnow;
          if (hash_table[itype] == 1) 
	     {hash_numeric_dV_x[itype][iii]  =  dvnow;}
          if (hash_table[itype] == 2) 
	     {hash_numeric_dV_x[itype][iii]  =  dvnow/(2.0*r);}
	  hash_numeric_pre_x[itype][iii] = -dvnow/r;
        }/*end for*/
    }/*endif*/

  }/*end for*/

  if (nhash==ntype) {hash_all  = 1;}    /* hash table for each interaction */
  else              {hash_all  = 0;}    /* not for all                     */

/*-------------------------------------------------------------------------*/
/* IX) DEBUG                                                               */

#ifdef DEBUG
  for (iinter=1; iinter<=ninter; iinter++) {
    printf("Inter pair %2d: %d(%s)--%d(%s)\n",iinter,
	   atom1[iinter], maps->atm_label[atom1[iinter]],
	   atom2[iinter], maps->atm_label[atom2[iinter]]);
    if (type[type_index[iinter]]==1 || type[type_index[iinter]]==2) {
      printf("\t\t{  eps=%.8g, sig=%.8g, beta=%.8g}\n",
	     eps[type_index[iinter]],sig[type_index[iinter]],beta[type_index[iinter]]);
      printf("\t\t{scale=%.8g,  r0=%.8g}\n",
	     scale[type_index[iinter]], r0[type_index[iinter]]);
    }
    if (type[type_index[iinter]]==3) {
      printf("\t\t{awill=%.8g, bwill=%.8g }\n", awill[type_index[iinter]], bwill[type_index[iinter]]);
      printf("\t\t{   c6=%.8g,    c8=%.8g, c10=%.8g}\n", 
	     c6m[type_index[iinter]], c8m[type_index[iinter]], c10m[type_index[iinter]]);
    }
    if (type[type_index[iinter]]==4) {
        printf("numeric_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_x[type_index[iinter]][iii]);
        }
        printf("numeric_V_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_V_x[type_index[iinter]][iii]);
        }
        printf("numeric_d2V_x: \n");
        for (iii=1;iii<=numeric_size[type_index[iinter]];iii++) {
	    printf("%g ",numeric_d2V_x[type_index[iinter]][iii]);
        }
    }
  }
#endif

/*-------------------------------------------------------------------------*/
/* VII) Deallocate memory                                                  */

#define DEALLOCATE_OFF
#ifdef DEALLOCATE    
  delete (database_entry_map+1);
#endif
  
  printf("Inter-mol setting up done.\n");
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

