#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG


/*========================================================================*/
/*========================================================================*/
void Onfo::setup(Pot_data_entry *database, Maps *maps, Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int ires, iresbond, ionfo_global;
  int i, iii, ionfo, jonfo, n;
  int onfo_check;
  int atoms[3];
  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;
  char   err_msg[LINE_NCHARMAX];

  printf("----------------------------------------------\n");
  printf("Setting up onefour\n...\n");


/*-------------------------------------------------------------------------*/
/* I) Setup topology                                                       */
 
  nonfo = maps->nonfo;

  atom1 = new int[nonfo]-1;
  atom2 = new int[nonfo]-1;

  type = new int[nonfo]-1;
  eps  = new double[nonfo]-1;
  feps = new double[nonfo]-1;
  sig  = new double[nonfo]-1;
  beta = new double[nonfo]-1;
  scale= new double[nonfo]-1;
  r0   = new double[nonfo]-1;
  awill= new double[nonfo]-1;
  bwill= new double[nonfo]-1;
  c6m  = new double[nonfo]-1;
  c8m  = new double[nonfo]-1;
  c10m = new double[nonfo]-1;
  r_min= new double[nonfo]-1;
  r_max= new double[nonfo]-1;
  r_res= new double[nonfo]-1;

  for (ires=1;ires<=pos->nres;ires++) {
      for (ionfo=1;ionfo<=pos->residue[ires].nonfo;ionfo++) {
   	  ionfo_global = pos->residue[ires].onfo[ionfo];
          if (ionfo_global>nonfo) {
             sprintf(err_msg,"Allocation problem in onfo setup topology!");
             sprintf(err_msg,"ionfo_global:%d>nonfo %d",ionfo_global,nonfo);
             errorMessage(err_msg);
          } 
   	  atom1[ionfo_global] = pos->residue[ires].onfo_atom[ionfo][1];
   	  atom2[ionfo_global] = pos->residue[ires].onfo_atom[ionfo][2];
      }/*end for*/       
  }/*end for*/
  for (iresbond=1;iresbond<=maps->nresbond;iresbond++) {
      for (ionfo=1;ionfo<=pos->residue_bond[iresbond].nonfo;ionfo++) {
   	  ionfo_global = pos->residue_bond[iresbond].onfo[ionfo];
          if (ionfo_global>nonfo) {
             sprintf(err_msg,"Allocation problem in onfo setup topology!");
             sprintf(err_msg,"ionfo_global:%d>nonfo %d",ionfo_global,nonfo);
             errorMessage(err_msg);
          } 
   	  atom1[ionfo_global] = pos->residue_bond[iresbond].onfo_atom[ionfo][1];
   	  atom2[ionfo_global] = pos->residue_bond[iresbond].onfo_atom[ionfo][2];
      }/*end for*/
  }/*end for*/
  
  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    if (atom1[ionfo]==atom2[ionfo]) {
      sprintf(err_msg, "Ilegal onfo #%d: same atom index.", ionfo);
      errorMessage(err_msg);
    }
  }


  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    for (jonfo=ionfo+1; jonfo<=nonfo; jonfo++) {
      if (atom1[ionfo]==atom1[jonfo] &&
	  atom2[ionfo]==atom2[jonfo] ) {
	sprintf(err_msg, "Onefour %d and %d repeat.", ionfo, jonfo);
	errorMessage(err_msg);
      }
      if (atom1[ionfo]==atom2[jonfo] &&
	  atom2[ionfo]==atom1[jonfo] ) {
	sprintf(err_msg, "Onefour %d and %d repeat.", ionfo, jonfo);
	errorMessage(err_msg);
      }
    }
  }

/*-------------------------------------------------------------------------*/
/* II) Setup potential parameters                                          */

  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    atoms[1] = atom1[ionfo];
    atoms[2] = atom2[ionfo];
    exact_match_entry = NULL;
    wild_match_entry  = NULL;
    database_entry    = database;
    onfo_check = 0;
    while (database_entry!=NULL && onfo_check!=1) {
      onfo_check = database_entry->match(maps, atoms, 3);
      if (onfo_check==1) { exact_match_entry = database_entry; }
      if (onfo_check==2) { wild_match_entry  = database_entry; }
      database_entry = database_entry->next;
    }/*end while */
    if (exact_match_entry==NULL && wild_match_entry==NULL) {
      sprintf(err_msg, "Onefour is not found in database.\nindices %d %d\nlabels %s %s",
	      atoms[1], atoms[2], maps->atm_label[atoms[1]], maps->atm_label[atoms[2]]);
      errorMessage(err_msg);
    }
    if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
    else                         { database_entry = wild_match_entry; }
    
    type[ionfo] = database_entry->pot_type;
    eps[ionfo]  = database_entry->eps;
    feps[ionfo] = 4.0*database_entry->eps;
    sig[ionfo]  = database_entry->sig;
    beta[ionfo] = database_entry->beta;
    scale[ionfo]= database_entry->scale;
    r0[ionfo]   = database_entry->r0;
    r0[ionfo]   = database_entry->r0;
    awill[ionfo]= database_entry->awill;
    bwill[ionfo]= database_entry->bwill;
    c6m[ionfo]  = database_entry->c6m;
    c8m[ionfo]  = database_entry->c8m;
    c10m[ionfo] = database_entry->c10m;
    r_min[ionfo]= database_entry->r_min;
    r_max[ionfo]= database_entry->r_max;
    r_res[ionfo]= database_entry->r_res;
  }/*end for: ionfo */


/*-------------------------------------------------------------------------*/
/* III) Setup Distance dependent dielectric                                */
   
  if (ddd==1) {
      ddd_p1 = ddd_D;
      ddd_p2 = ddd_D0;
      ddd_p3 = ddd_S;
  } else if (ddd==2) {
      ddd_p1 = ddd_c;
      ddd_p2 = ddd_e;
  }/* end if*/

/*-------------------------------------------------------------------------*/
/* IV) DEBUG                                                               */
  
#ifdef DEBUG      
  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    printf("One-four %2d: %d(%s)--%d(%s)\n",ionfo,
	   atom1[ionfo], maps->atm_label[atom1[ionfo]],
	   atom2[ionfo], maps->atm_label[atom2[ionfo]]);
    if (type[ionfo]==1 || type[ionfo]==2) {
      printf("\t\t{  eps=%.8g, sig=%.8g, beta=%.8g}\n",
	     eps[ionfo],sig[ionfo],beta[ionfo]);
      printf("\t\t{scale=%.8g,  r0=%.8g}\n",
	     scale[ionfo], r0[ionfo]);
    }
    if (type[ionfo]==3) {
      printf("\t\t{awill=%.8g, bwill=%.8g }\n", awill[ionfo], bwill[ionfo]);
      printf("\t\t{   c6=%.8g,    c8=%.8g, c10=%.8g}\n", 
	     c6m[ionfo], c8m[ionfo], c10m[ionfo]);
    }
  }
#endif

  printf("Onefour setting up done.\n");
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
void Onfo::setup(Pot_data_entry *database, Maps *maps, char *parm_filename)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, iii, ionfo, jonfo, n;
  int natm_tot, nkey, key_defined, line_num;
  int nonfo_now, nonfo_mall, atom1_ind, atom2_ind;
  int onfo_check, key_found[4];
  int atoms[3];
  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;
  
  char   err_msg[LINE_NCHARMAX];
  char entry_line[LINE_NCHARMAX];
  char entry_name[NCHARMAX], key[NCHARMAX], arg[NCHARMAX];
  char *def;
  FILE *parm_file;
  
  char *onfo_def_keys[] = {"", "atom1", "atom2"};

  printf("----------------------------------------------\n");
  printf("Setting up onefour\n...\n");
  
  if ((parm_file=fopen(parm_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open parm file %s.", parm_filename);
    errorMessage(err_msg);
  }
  
  nonfo_mall= NMEM_MIN;
  nonfo_now = 0;
  natm_tot  = maps->natm_tot;
  line_num  = 0;
  
  atom1 = new int[nonfo_mall]-1;
  atom2 = new int[nonfo_mall]-1;
  
  while((n=readOneEntry(parm_file,'~','[',']',entry_line, line_num))!=0) {
    if (n!=1) {
      sprintf(err_msg, "%s at line %d while reading file %s.",
	      (n==-1) ? "Syntax error" : "Internal error", 
	      line_num, parm_filename);
      errorMessage(err_msg);
    }
    n=getEntryName(entry_line, '~', '[', entry_name, def);
    if (n!=1) {
      sprintf(err_msg, "Syntax error at parm file line:\n%s", entry_line);
      errorMessage(err_msg);
    }
    if (strcasecmp(entry_name, "onefour_def")==0) {
      nkey = 2;
      for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
      while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
	key_defined = 0;
	for ( i=1; i<=nkey; i++) {                  /*loop for key words.*/
	  if (strcasecmp(key, onfo_def_keys[i])==0) {/*it's a keyword.  */
	    key_defined  = 1;
	    key_found[i] = 1;
	    switch (i) {
	    case 1:
	      sscanf(arg, "%d", &atom1_ind);
	      break;
	    case 2:
	      sscanf(arg, "%d", &atom2_ind);
	      break;
	    }/*switch */
	  }
	}/*end for*/
	if (key_defined==0) {
	  sprintf(err_msg, "Key word %s unknown in parmfile line:\n%s",
		  key, entry_line);
	  errorMessage(err_msg);
	}
      }/*end while: getKeyArgPair*/
      if (n!=0) {
	sprintf(err_msg, "Syntax error in parm file line:\n%s",entry_line);
	errorMessage(err_msg);
      }
      for ( i=1; i<=nkey; i++) { 
	if (key_found[i]==0) {
	  sprintf(err_msg, "Key word %s missed in parm file line:\n%s",
		  onfo_def_keys[i], entry_line);
	  errorMessage(err_msg);
	}
      }
      if (atom1_ind<1 || atom1_ind>natm_tot || 
	  atom2_ind<1 || atom2_ind>natm_tot) {
	sprintf(err_msg, "ONFO::setup(): Illegal atom index in parm file %s.",
		parm_filename);
	errorMessage(err_msg);
      }
      nonfo_now++;
      if (nonfo_now>nonfo_mall) {
	nonfo_mall += NMEM_MIN;
	atom1       = (int *) crealloc(atom1+1, nonfo_mall*sizeof(int))-1;
	atom2       = (int *) crealloc(atom2+1, nonfo_mall*sizeof(int))-1;
      }
      atom1[nonfo_now] = atom1_ind;
      atom2[nonfo_now] = atom2_ind;
    }/*end if: onefour_def */
  }/*end while: readOneEntry*/
  fclose(parm_file);
  
  nonfo= nonfo_now;
  type = new int[nonfo]-1;
  eps  = new double[nonfo]-1;
  feps = new double[nonfo]-1;
  sig  = new double[nonfo]-1;
  beta = new double[nonfo]-1;
  scale= new double[nonfo]-1;
  r0   = new double[nonfo]-1;
  awill= new double[nonfo]-1;
  bwill= new double[nonfo]-1;
  c6m  = new double[nonfo]-1;
  c8m  = new double[nonfo]-1;
  c10m = new double[nonfo]-1;
  r_min= new double[nonfo]-1;
  r_max= new double[nonfo]-1;
  r_res= new double[nonfo]-1;
  
  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    if (atom1[ionfo]==atom2[ionfo]) {
      sprintf(err_msg, "Ilegal onfo #%d: same atom index.", ionfo);
      errorMessage(err_msg);
    }
  }

  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    atoms[1] = atom1[ionfo];
    atoms[2] = atom2[ionfo];
    exact_match_entry = NULL;
    wild_match_entry  = NULL;
    database_entry    = database;
    onfo_check = 0;
    while (database_entry!=NULL && onfo_check!=1) {
      onfo_check = database_entry->match(maps, atoms, 3);
      if (onfo_check==1) { exact_match_entry = database_entry; }
      if (onfo_check==2) { wild_match_entry  = database_entry; }
      database_entry = database_entry->next;
    }/*end while */
    if (exact_match_entry==NULL && wild_match_entry==NULL) {
      sprintf(err_msg, "Onefour is not found in database.\nindices %d %d\nlabels %s %s",
	      atoms[1], atoms[2], maps->atm_label[atoms[1]], maps->atm_label[atoms[2]]);
      errorMessage(err_msg);
    }
    if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
    else                         { database_entry = wild_match_entry; }
    
    type[ionfo] = database_entry->pot_type;
    eps[ionfo]  = database_entry->eps;
    feps[ionfo] = 4.0*database_entry->eps;
    sig[ionfo]  = database_entry->sig;
    beta[ionfo] = database_entry->beta;
    scale[ionfo]= database_entry->scale;
    r0[ionfo]   = database_entry->r0;
    r0[ionfo]   = database_entry->r0;
    awill[ionfo]= database_entry->awill;
    bwill[ionfo]= database_entry->bwill;
    c6m[ionfo]  = database_entry->c6m;
    c8m[ionfo]  = database_entry->c8m;
    c10m[ionfo] = database_entry->c10m;
    r_min[ionfo]= database_entry->r_min;
    r_max[ionfo]= database_entry->r_max;
    r_res[ionfo]= database_entry->r_res;
  }/*end for: ionfo */
  
  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    for (jonfo=ionfo+1; jonfo<=nonfo; jonfo++) {
      if (atom1[ionfo]==atom1[jonfo] &&
	  atom2[ionfo]==atom2[jonfo] ) {
	sprintf(err_msg, "Onefour %d and %d repeat.", ionfo, jonfo);
	errorMessage(err_msg);
      }
      if (atom1[ionfo]==atom2[jonfo] &&
	  atom2[ionfo]==atom1[jonfo] ) {
	sprintf(err_msg, "Onefour %d and %d repeat.", ionfo, jonfo);
	errorMessage(err_msg);
      }
    }
  }


/*-------------------------------------------------------------------------*/
/* III) Setup Distance dependent dielectric                                */
   
  if (ddd==1) {
      ddd_p1 = ddd_D;
      ddd_p2 = ddd_D0;
      ddd_p3 = ddd_S;
  } else if (ddd==2) {
      ddd_p1 = ddd_c;
      ddd_p2 = ddd_e;
  }/* end if*/

/*-------------------------------------------------------------------------*/
/* IV) DEBUG                                                               */

#ifdef DEBUG      
  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    printf("One-four %2d: %d(%s)--%d(%s)\n",ionfo,
	   atom1[ionfo], maps->atm_label[atom1[ionfo]],
	   atom2[ionfo], maps->atm_label[atom2[ionfo]]);
    if (type[ionfo]==1 || type[ionfo]==2) {
      printf("\t\t{  eps=%.8g, sig=%.8g, beta=%.8g}\n",
	     eps[ionfo],sig[ionfo],beta[ionfo]);
      printf("\t\t{scale=%.8g,  r0=%.8g}\n",
	     scale[ionfo], r0[ionfo]);
    }
    if (type[ionfo]==3) {
      printf("\t\t{awill=%.8g, bwill=%.8g }\n", awill[ionfo], bwill[ionfo]);
      printf("\t\t{   c6=%.8g,    c8=%.8g, c10=%.8g}\n", 
	     c6m[ionfo], c8m[ionfo], c10m[ionfo]);
    }
  }
#endif

  printf("Onefour setting up done.\n");
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/




/*========================================================================*/
/*========================================================================*/
double Onfo::energy(Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, ionfo;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *fx, *fy, *fz, *q;
  double dx, dy, dz, r, r2, dr;
  double qqs, ri, rs, xx, xxx;
  double pre, vonfo, vnow;
  double r6, r8, r10;
  double a, b, c6, c8, c10;
  double eps_ddd, deps_ddd;  

  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;
  q  = pos->chrg;

/*---------------------------------------------------------------------------*/
/* 0) Return if no onfo term                                                 */

  if (energy_term != 1) { 
      vcoul = 0.0;
      vlj   = 0.0;
      vonfo = 0.0;
      return vonfo;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Evaluate onfo interactions and forces                                */

  vcoul = 0;
  vlj   = 0;  
  vonfo = 0;
  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    atom1_ind = atom1[ionfo];
    atom2_ind = atom2[ionfo];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    r  = sqrt(r2);
    ri = 1.0/r;
    
    if (implicit_solvent==0) 
         {qqs = q[atom1_ind]*q[atom2_ind]*scale[ionfo];}
    else {qqs = 0.0;}
    vnow   = qqs*ri;
    pre    = qqs/(r*r2);

    if (ddd>0) {
        eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
        deps_ddd = ddd_deps(r,ddd_p1,ddd_p2,ddd_p3,ddd);    
        pre   = pre/eps_ddd 
	            + vnow*deps_ddd/(r*eps_ddd*eps_ddd);
        vnow /= eps_ddd;
    }

    vonfo += vnow;
    vcoul += vnow;
    
    if (type[ionfo]==1 || type[ionfo]==2) {
      rs  = r+(r0[ionfo]-sig[ionfo]);
      xx  = r0[ionfo]/rs;
      xxx   = xx*xx*xx*xx*xx*xx;
      vnow  = feps[ionfo]*xxx*(xxx-beta[ionfo]);
      vonfo+= vnow;
      vlj  += vnow;
      pre  += 6.0*feps[ionfo]*xxx*(2.0*xxx-beta[ionfo])/(r*rs);
    }
    if ( type[ionfo]==3) {
      a   = awill[ionfo];
      b   = bwill[ionfo];
      c6  = c6m[ionfo];
      c8  = c8m[ionfo];
      c10 = c10m[ionfo];
      r6  = r2 * r2 * r2;
      r8  = r2 * r2 * r2 * r2;
      r10 = r8 * r2;
      vonfo += a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
      pre   += a*b*exp(-b*r)/r - (c6 * 6.0/r6 + c8*8.0/r8 + c10*10.0/r10)/r2;
    }
    fx[atom1_ind] += pre*dx;
    fy[atom1_ind] += pre*dy;
    fz[atom1_ind] += pre*dz;
    fx[atom2_ind] -= pre*dx;
    fy[atom2_ind] -= pre*dy;
    fz[atom2_ind] -= pre*dz;
  }

/*-------------------------------------------------------------------------*/
/* II) Debug distance dependent dielectric                                 */

#define DEBUG_DDD_OFF
#ifdef DEBUG_DDD
  for (i=1;i<=250;i++) {
    r = i*0.1;
    printf("r %g eps %g deps %g\n",r*BOHR,ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd),
                                          ddd_deps(r,ddd_p1,ddd_p2,ddd_p3,ddd));
  }
  getchar();
#endif

/*-------------------------------------------------------------------------*/
/* III) Return the energy value                                            */
  return vonfo;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/




/*========================================================================*/
/*========================================================================*/
double Onfo::getPot(Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, ionfo;
  int atom1_ind, atom2_ind;
  
  double *x, *y, *z, *q;
  double dx, dy, dz, r, r2, dr;
  double qqs, ri, rs, xx, xxx;
  double vonfo, vnow;
  double r6, r8, r10;
  double a, b, c6, c8, c10;
  double eps_ddd;
  
  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  q  = pos->chrg;


/*---------------------------------------------------------------------------*/
/* 0) Return if no onfo term                                                 */

  if (energy_term != 1) { 
      vcoul = 0.0;
      vlj   = 0.0;
      vonfo = 0.0;
      return vonfo;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Evaluate onfo interactions                                           */
  
  vcoul = 0;
  vlj   = 0;
  vonfo = 0;
  for (ionfo=1; ionfo<=nonfo; ionfo++) {
    atom1_ind = atom1[ionfo];
    atom2_ind = atom2[ionfo];
    dx = x[atom1_ind] - x[atom2_ind];
    dy = y[atom1_ind] - y[atom2_ind];
    dz = z[atom1_ind] - z[atom2_ind];
    r2 = dx*dx+dy*dy+dz*dz;
    r  = sqrt(r2);
    ri = 1.0/r;
    
    if (implicit_solvent==0) 
         {qqs = q[atom1_ind]*q[atom2_ind]*scale[ionfo];}
    else {qqs = 0.0;}

    vnow   = qqs*ri;

    if (ddd>0) {
        eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
        vnow /= eps_ddd;
    }

    vonfo += vnow;
    vcoul += vnow;
    
    if (type[ionfo]==1 || type[ionfo]==2) {
      rs  = r+(r0[ionfo]-sig[ionfo]);
      xx  = r0[ionfo]/rs;
      xxx    = xx*xx*xx*xx*xx*xx;
      vnow   = feps[ionfo]*xxx*(xxx-beta[ionfo]);
      vonfo += vnow;
      vlj   += vnow;
    }
    if ( type[ionfo]==3) {
      a   = awill[ionfo];
      b   = bwill[ionfo];
      c6  = c6m[ionfo];
      c8  = c8m[ionfo];
      c10 = c10m[ionfo];
      r6  = r2 * r2 * r2;
      r8  = r2 * r2 * r2 * r2;
      r10 = r8 * r2;
      vonfo += a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
    }
  }
  return vonfo;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/




/*========================================================================*/
/*========================================================================*/
double Onfo::getOnfoPot(Pos *pos, int jtyp, double r)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int atom1_ind, atom2_ind;
  double qqs, vonfo, vnow, ri, r2, rs, xx, xxx;
  double r6, r8, r10;
  double a, b, c6, c8, c10;
  double eps_ddd;
  double *q = pos->chrg;  

  ri = 1.0/r;
  r2 = r*r;
  atom1_ind = atom1[jtyp];
  atom2_ind = atom2[jtyp];
  qqs   = q[atom1_ind]*q[atom2_ind]*scale[jtyp];
  vnow  = qqs*ri;

  if (ddd>0) {
      eps_ddd  = ddd_eps(r,ddd_p1,ddd_p2,ddd_p3,ddd); 
      vnow /= eps_ddd;
  }

  vonfo = vnow;
  vcoul = vnow;
  
  if (type[jtyp]==1 || type[jtyp]==2) {
    rs  = r+(r0[jtyp]-sig[jtyp]);
    xx  = r0[jtyp]/rs;
    xxx    = xx*xx*xx*xx*xx*xx;
    vnow   = feps[jtyp]*xxx*(xxx-beta[jtyp]);
    vonfo += vnow;
    vlj    = vnow;
  }
  if ( type[jtyp]==3) {
    a   = awill[jtyp];
    b   = bwill[jtyp];
    c6  = c6m[jtyp];
    c8  = c8m[jtyp];
    c10 = c10m[jtyp];
    r6  = r2 * r2 * r2;
    r8  = r2 * r2 * r2 * r2;
    r10 = r8 * r2;
    vonfo += a*exp(-b*r) - (c6/r6 + c8/r8 + c10/r10);
  }
  return vonfo;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/
  
