#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Pos::report_energy(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

 int report_level = system.general.energy_report;

/*-------------------------------------------------------------------------*/
/* I) Decide if report is needed                                           */

 if (report_level==0) {return;}

 print_doubleline();
 printf(">>Report energies at step %d<<\n",system.general.istep);
 print_line();


/*-------------------------------------------------------------------------*/
/* II) Report major energy terms                                           */

 if (report_level==1) {
   if (system.general.write_energy_unit==1) {/*kcal/mol*/  
       LINE
       printf("\tEnergies in kcal/mol\n\n");
       printf("\t-Bond energy                 %.16g\n",KCAL*ebond);
       printf("\t-Bend energy                 %.16g\n",KCAL*ebend);
       printf("\t-Torsion energy              %.16g\n",KCAL*etors);
       printf("\t-Onefour energy              %.16g\n",KCAL*eonfo);
       printf("\t-Inter energy                %.16g\n",KCAL*einter);
       printf("\t-Total Lennard-Jones energy  %.16g\n",
                KCAL*(einter_lj+eonfo_lj));
       printf("\t-Total Coulomb energy        %.16g\n",
                KCAL*(einter_coul+eonfo_coul));

       if (system.general.energy_term_cryo_em==1)         
	  {printf("\t-Cryo-em energy              %.16g\n",KCAL*ecryo_em);}

       printf("\t-Constraint energy           %.16g\n",KCAL*econstraint);

       printf("\t-Total energy                %.16g\n",KCAL*epot);
   }/*kcal/mol*/
   
   if (system.general.write_energy_unit==2) {/*Ha*/  
       LINE
       printf("\tEnergies in Ha\n\n");
       printf("\t-Bond energy                 %.16g\n",ebond);	
       printf("\t-Bend energy                 %.16g\n",ebend);	
       printf("\t-Torsion energy              %.16g\n",etors);	
       printf("\t-Onefour energy              %.16g\n",eonfo);	
       printf("\t-Inter energy                %.16g\n",einter);
       printf("\t-Total Lennard-Jones energy  %.16g\n",
                           (einter_lj+eonfo_lj));
       printf("\t-Total Coulomb energy        %.16g\n",
                       (einter_coul+eonfo_coul));

       if (system.general.energy_term_cryo_em==1)         
	  {printf("\t-Cryo-em energy              %.16g\n",ecryo_em);}

       printf("\t-Constraint energy           %.16g\n",econstraint);	

       printf("\t-Total energy                %.16g\n",epot);
   }/*Ha*/
 }/*endif*/

/*-------------------------------------------------------------------------*/
/* III) Report all energy terms                                            */

 if (report_level==2) {
   if (system.general.write_energy_unit==1) {/*kcal/mol*/  
       LINE
       printf("\tEnergies in kcal/mol\n\n");
       printf("\t-Bond energy                 %.16g\n",KCAL*ebond);
       printf("\t-Bend energy                 %.16g\n",KCAL*ebend);
       printf("\t   Bond-Bend energy          %.16g\n",KCAL*ebend_bond);
       printf("\t   Bend-Bend energy          %.16g\n",KCAL*ebend_bend);
       printf("\t-Torsion energy              %.16g\n",KCAL*etors);
       printf("\t   Tors-Tors energy          %.16g\n",KCAL*etors_tors);
       printf("\t   Tors-Improper energy      %.16g\n",KCAL*etors_imp);
       printf("\t-Onefour energy              %.16g\n",KCAL*eonfo);
       printf("\t   Onfo-Coulomb energy       %.16g\n",KCAL*eonfo_coul);
       printf("\t   Onfo-Lennard-Jones energy %.16g\n",KCAL*eonfo_lj);
       printf("\t-Inter energy                %.16g\n",KCAL*einter);
       printf("\t   Lennard-Jones energy      %.16g\n",KCAL*einter_lj);
       printf("\t   Coulomb energy            %.16g\n",KCAL*einter_coul);
       printf("\t   SCP energy                %.16g\n",KCAL*einter_scp);
       printf("\t   Knowledge based energy    %.16g\n",KCAL*einter_kb);  
       printf("\t-Total Lennard-Jones energy  %.16g\n",
                KCAL*(einter_lj+eonfo_lj));
       printf("\t-Total Coulomb energy        %.16g\n",
                KCAL*(einter_coul+eonfo_coul));

       if (system.general.energy_term_cryo_em==1)         
          {printf("\t-Cryo-em energy              %.16g\n",KCAL*ecryo_em);}

       printf("\t-Constraint energy           %.16g\n",KCAL*econstraint);

       printf("\t-Total energy                %.16g\n",KCAL*epot);
   }/*kcal/mol*/
   
   if (system.general.write_energy_unit==2) {/*Ha*/  
       LINE
       printf("\tEnergies in Ha\n\n");
       printf("\t-Bond energy                 %.16g\n",ebond);	
       printf("\t-Bend energy                 %.16g\n",ebend);	
       printf("\t   Bond-Bend energy          %.16g\n",ebend_bond);	
       printf("\t   Bend-Bend energy          %.16g\n",ebend_bend);	
       printf("\t-Torsion energy              %.16g\n",etors);	
       printf("\t   Tors-Tors energy          %.16g\n",etors_tors);	
       printf("\t   Tors-Improper energy      %.16g\n",etors_imp);	
       printf("\t-Onefour energy              %.16g\n",eonfo);	
       printf("\t   Onfo-Coulomb energy       %.16g\n",eonfo_coul);	
       printf("\t   Onfo-Lennard-Jones energy %.16g\n",eonfo_lj);	
       printf("\t-Inter energy                %.16g\n",einter);	
       printf("\t   Lennard-Jones energy      %.16g\n",einter_lj);	
       printf("\t   Coulomb energy            %.16g\n",einter_coul);	
       printf("\t   SCP energy                %.16g\n",einter_scp);	
       printf("\t   Knowledge based energy    %.16g\n",einter_kb);  
       printf("\t-Total Lennard-Jones energy  %.16g\n",
                           (einter_lj+eonfo_lj));
       printf("\t-Total Coulomb energy        %.16g\n",
                       (einter_coul+eonfo_coul));

       if (system.general.energy_term_cryo_em==1)         
          {printf("\t-Cryo-em energy              %.16g\n",ecryo_em);}

       printf("\t-Constraint energy           %.16g\n",econstraint);	

       printf("\t-Total energy                %.16g\n",epot);
   }/*Ha*/
 }/*endif*/

/*-------------------------------------------------------------------------*/
/* IV) Report Temperature                                                  */
/*     In case of stochastic minimization                                  */

 if (system.general.minimize==1) {
     if (system.general.minimize_type>=2) {
         printf("\t-Temperature                 %.16g\n",T*BOLTZ);
     }/*endif*/
 }/*endif*/

/*-------------------------------------------------------------------------*/
 LINE 
 print_line();
 print_doubleline();
 LINE

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
