#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"


/*=========================================================================*/
void Pos::respa_intra(System &system)
/*-------------------------------------------------------------------------*/
/* Calculates intramolecular forces, energy and adds to "constant"         */
/* intermolecular energy, einterp so that the proposed energy, epotp is    */
/* updated                                                                 */
/* Forces are also updated by adding new intramolecular forces to the      */
/* constant intermolecular forces,fxri, fyri, fzri                         */
/* forces, energy based on xi, yi, zi                                      */
{ /*begin routine */
/*-------------------------------------------------------------------------*/
       
  double *xt,*yt,*zt;

         xt = x; yt = y; zt = z;
         x = xi; y = yi; z = zi;
         force_zero();
          ebondp  = system.bond.energy(this);
          ebendp  = system.bend.energy(this);
          etorsp  = system.tors.energy(this);
          eonfop  = system.onfo.energy(this);
          epotp   = ebondp + ebendp + etorsp + eonfop;
         x = xt; y = yt; z = zt;
          ebendp_bond  = system.bend.vbond;
          ebendp_bend  = system.bend.vbend;
          etorsp_tors  = system.tors.vtors;
          etorsp_imp   = system.tors.vimpr;
          eonfop_lj    = system.onfo.vlj;
          eonfop_coul  = system.onfo.vcoul;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::respa_inter(System &system)
/*-------------------------------------------------------------------------*/
/* Updates proposed intermolecular energy, einterp and forces              */
/* on fxri, fyri, fzri                                                     */ 
/* based on xi, yi, zi                                                     */
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm;
  double *xt,*yt,*zt;
  double *fxt,*fyt,*fzt;

  int intra_respa_step = system.general.intra_respa_step;

  
/*-------------------------------------------------------------------------*/
/* I) Add the intermolecular energy to epotp                               */
/*    Calculate intermolecular forces and store in fxri, fyri, fzri        */

         xt = x; yt = y; zt = z;
         x = xi; y = yi; z = zi;
         fxt = fx; fyt = fy; fzt = fz;
         fx = fxri; fy = fyri; fz = fzri;

          force_zero(); 
          einterp = system.inter.energy(this);
          einterp_lj   = system.inter.elj;
          einterp_coul = system.inter.ecoul;
          einterp_scp  = system.inter.escp;
          einterp_kb   = system.inter.ekb;
	  epotp += einterp;

         x = xt; y = yt; z = zt;
         fx = fxt; fy = fyt; fz = fzt;
  
/*-------------------------------------------------------------------------*/
/* II) Add the intermolecular force                                        */

  for (iatm=1;iatm<=natom;iatm++) {
       fx[iatm] += ((double)intra_respa_step)*fxri[iatm];
       fy[iatm] += ((double)intra_respa_step)*fyri[iatm];
       fz[iatm] += ((double)intra_respa_step)*fzri[iatm];           
  }/*endfor*/

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


