#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF


/*========================================================================*/
/*========================================================================*/
void Tors::setup(const Pot_data_entry *database, const Maps *maps, 
		                                 const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int ires, iresbond, itors_global;
  int i, iii, itors, jtors, iimpr, n; 
  int tors_check;
  int  atoms[5];
  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;
  char   err_msg[LINE_NCHARMAX];  
  
  double fk_tors, theta_0;
  double c2th0,c4th0,c6th0;                 /* Cos(n*theta_0)          */
  double s2th0,s4th0,s6th0;                 /* Sin(n*theta_0)          */

  static double cn1=49.0/72.0,cn2=3.0/4.0,cn3=3.0/40.0,cn4=1.0/180.0;
  static double cn5=3.0/2.0,cn6=3.0/5.0,cn7=1.0/10.0,cn8=4.0/15.0;
  static double cn9=8.0/45.0,cn10=3.0/10.0,cn11=1.0/30.0;
  static double small=1.0e-8;

  printf("----------------------------------------------\n");
  printf("Setting up tors\n...\n");

/*-------------------------------------------------------------------------*/
/* I) Setup topology                                                       */
  
  ntors = maps->ntors + maps->nimpr;              /* ntors: <- tors + impr */

  atom1    =  new int[ntors]-1;
  atom2    =  new int[ntors]-1;
  atom3    =  new int[ntors]-1;
  atom4    =  new int[ntors]-1;
  improper =  new int[ntors]-1;
  type     =  new int[ntors]-1;

  p0    = new double[ntors] -1;
  p1    = new double[ntors] -1;
  p2    = new double[ntors] -1;
  p3    = new double[ntors] -1;
  p4    = new double[ntors] -1;
  p5    = new double[ntors] -1;
  p6    = new double[ntors] -1;
  s0    = new double[ntors] -1;
  s1    = new double[ntors] -1;
  s2    = new double[ntors] -1;
  s3    = new double[ntors] -1;
  s4    = new double[ntors] -1;
  s5    = new double[ntors] -1;
  s6    = new double[ntors] -1;
  sp2   = new double[ntors] -1;
  sp3   = new double[ntors] -1;
  sp4   = new double[ntors] -1;
  sp5   = new double[ntors] -1;
  sp6   = new double[ntors] -1;
  eq    = new double[ntors] -1;

  for (ires=1;ires<=pos->nres;ires++) {
      for (itors=1;itors<=pos->residue[ires].ntors;itors++) {
   	  itors_global = pos->residue[ires].tors[itors];
          if (itors_global>ntors) {
             sprintf(err_msg,"Allocation problem in tors setup topology!");
             sprintf(err_msg,"itors_global:%d>ntors %d",itors_global,ntors);
             errorMessage(err_msg);
          } 
   	  atom1[itors_global] = pos->residue[ires].tors_atom[itors][1];
   	  atom2[itors_global] = pos->residue[ires].tors_atom[itors][2];
   	  atom3[itors_global] = pos->residue[ires].tors_atom[itors][3];
   	  atom4[itors_global] = pos->residue[ires].tors_atom[itors][4];
          improper[itors_global] = 0;
      }/*end for*/       
  }/*end for*/
  for (iresbond=1;iresbond<=maps->nresbond;iresbond++) {
      for (itors=1;itors<=pos->residue_bond[iresbond].ntors;itors++) {
   	  itors_global = pos->residue_bond[iresbond].tors[itors];
          if (itors_global>ntors) {
             sprintf(err_msg,"Allocation problem in tors setup topology!");
             sprintf(err_msg,"itors_global:%d>ntors %d",itors_global,ntors);
             errorMessage(err_msg);
          } 
   	  atom1[itors_global] = pos->residue_bond[iresbond].tors_atom[itors][1];
   	  atom2[itors_global] = pos->residue_bond[iresbond].tors_atom[itors][2];
   	  atom3[itors_global] = pos->residue_bond[iresbond].tors_atom[itors][3];
   	  atom4[itors_global] = pos->residue_bond[iresbond].tors_atom[itors][4];
          improper[itors_global] = 0;
      }/*end for*/
  }/*end for*/

  for (ires=1;ires<=pos->nres;ires++) {
      for (iimpr=1;iimpr<=pos->residue[ires].nimpr;iimpr++) {
   	  itors_global = maps->ntors + pos->residue[ires].impr[iimpr];
          if (itors_global>ntors) {
             sprintf(err_msg,"Allocation problem in impr tors setup topology!");
             sprintf(err_msg,"itors_global:%d>ntors %d",itors_global,ntors);
             errorMessage(err_msg);
          } 
   	  atom1[itors_global] = pos->residue[ires].impr_atom[iimpr][1];
   	  atom2[itors_global] = pos->residue[ires].impr_atom[iimpr][2];
   	  atom3[itors_global] = pos->residue[ires].impr_atom[iimpr][3];
   	  atom4[itors_global] = pos->residue[ires].impr_atom[iimpr][4];
          improper[itors_global] = 1;
      }/*end for*/       
  }/*end for*/
  for (iresbond=1;iresbond<=maps->nresbond;iresbond++) {
      for (iimpr=1;iimpr<=pos->residue_bond[iresbond].nimpr;iimpr++) {
   	  itors_global = maps->ntors + pos->residue_bond[iresbond].impr[iimpr];
          if (itors_global>ntors) {
             sprintf(err_msg,"Allocation problem in impr tors setup topology!");
             sprintf(err_msg,"itors_global:%d>ntors %d",itors_global,ntors);
             errorMessage(err_msg);
          } 
   	  atom1[itors_global] = pos->residue_bond[iresbond].impr_atom[iimpr][1];
   	  atom2[itors_global] = pos->residue_bond[iresbond].impr_atom[iimpr][2];
   	  atom3[itors_global] = pos->residue_bond[iresbond].impr_atom[iimpr][3];
   	  atom4[itors_global] = pos->residue_bond[iresbond].impr_atom[iimpr][4];
          improper[itors_global] = 1;
      }/*end for*/
  }/*end for*/
 
  for (itors=1; itors<=ntors; itors++) {
    if (atom1[itors]==atom2[itors] ||
	atom1[itors]==atom3[itors] ||
	atom1[itors]==atom4[itors] ||
	atom2[itors]==atom3[itors] ||
	atom2[itors]==atom4[itors] ||
	atom3[itors]==atom4[itors] ) {
      sprintf(err_msg, "Ilegal tors #%d: same atom index.", itors);
      errorMessage(err_msg);
    }
  }
  
  for (itors=1; itors<=ntors; itors++) {
    for (jtors=itors+1; jtors<=ntors; jtors++) {
      if (atom1[itors]==atom1[jtors] &&
	  atom2[itors]==atom2[jtors] &&
	  atom3[itors]==atom3[jtors] &&
	  atom4[itors]==atom4[jtors]   ){
	sprintf(err_msg, "Tors %d and %d repeat.", itors, jtors);
	errorMessage(err_msg);
      }
      if (atom1[itors]==atom4[jtors] &&
	  atom2[itors]==atom3[jtors] &&
	  atom3[itors]==atom2[jtors] &&
	  atom4[itors]==atom1[jtors]   ){
	sprintf(err_msg, "Tors %d and %d repeat.", itors, jtors);
	errorMessage(err_msg);
      }
    }
  }

/*-------------------------------------------------------------------------*/
/* II) Setup potential parameters                                          */
  
  nimproper = 0;  
  for (itors=1; itors<=ntors; itors++) {
    atoms[1] = atom1[itors];
    atoms[2] = atom2[itors];
    atoms[3] = atom3[itors];
    atoms[4] = atom4[itors];
    exact_match_entry = NULL;
    wild_match_entry  = NULL;
    database_entry    = database;
    tors_check = 0;
    while (database_entry!=NULL && tors_check!=1) {
      tors_check = database_entry->match(maps, atoms, 4);
      if (tors_check==1) {
	 if (improper[itors]==database_entry->imp_tors) {
            exact_match_entry = database_entry;
         }
         else {tors_check=0;}
      }
      if (tors_check==2) { 
	 if (improper[itors]==database_entry->imp_tors) {
             wild_match_entry = database_entry; 
         }
         else {tors_check=0;}
      }      
      database_entry = database_entry->next;
    }/*end while */
    if (exact_match_entry==NULL && wild_match_entry==NULL) {
      sprintf(err_msg, "Torsion is not found in database.\nindices %d %d %d %d\nlabels %s %s %s %s",
	      atoms[1], atoms[2], atoms[3], atoms[4],
              maps->atm_label[atoms[1]],maps->atm_label[atoms[2]],
              maps->atm_label[atoms[3]],maps->atm_label[atoms[4]]);
      errorMessage(err_msg);
    }
    if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
    else                         { database_entry = wild_match_entry; }
    
    type[itors]     = database_entry->pot_type;
    nimproper      += improper[itors];
    switch(type[itors]) {
    case 1:
      p0[itors] = database_entry->p0;
      p1[itors] = database_entry->p1;
      p2[itors] = database_entry->p2;
      p3[itors] = database_entry->p3;
      p4[itors] = database_entry->p4;
      p5[itors] = database_entry->p5;
      p6[itors] = database_entry->p6;

      s0[itors] = 0.0;
      s1[itors] = database_entry->s1;
      s2[itors] = database_entry->s2;
      s3[itors] = database_entry->s3;
      s4[itors] = database_entry->s4;
      s5[itors] = database_entry->s5;
      s6[itors] = database_entry->s6;

      sp2[itors] = database_entry->sp2;
      sp3[itors] = database_entry->sp3;
      sp4[itors] = database_entry->sp4;
      sp5[itors] = database_entry->sp5;
      sp6[itors] = database_entry->sp6;
      break;
    case 2: /*Not implemented */
      break;
    case 3:
      theta_0  = database_entry->eq_tors;
      fk_tors  = database_entry->fk_tors;
      eq[itors]= theta_0;
      
      c2th0 = cos(2.0*theta_0);
      c4th0 = cos(4.0*theta_0);
      c6th0 = cos(6.0*theta_0);
      p0[itors] = 0.5*fk_tors*(cn1 + cn2*c2th0 + cn3*c4th0 + cn4*c6th0);
      p1[itors] = 0.0;
      p2[itors] = 0.5*fk_tors*(-cn5*c2th0 - cn6*c4th0 - cn7*c6th0);
      p3[itors] = 0.0;
      p4[itors] = 0.5*fk_tors*(cn6*c4th0 + cn8*c6th0);
      p5[itors] = 0.0;
      p6[itors] = 0.5*fk_tors*(-cn9*c6th0);
      
      s2th0 = sin(2.0*theta_0);
      s4th0 = sin(4.0*theta_0);
      s6th0 = sin(6.0*theta_0);
      s0[itors] = 0.0;
      s1[itors] = 0.0;
      s2[itors] = 0.5*fk_tors*(-cn5*s2th0 + cn10*s4th0 - cn11*s6th0);
      s3[itors] = 0.0;
      s4[itors] = 0.5*fk_tors*(-cn6*s4th0 + cn9*s6th0);
      s5[itors] = 0.0;
      s6[itors] = 0.5*fk_tors*(-cn9*s6th0);

      sp2[itors] = 0.0;
      sp3[itors] = 0.0;
      sp4[itors] = 0.0;
      sp5[itors] = 0.0;
      sp6[itors] = 0.0;
      break;
    }
  }/*end for: itors */
      

/*-------------------------------------------------------------------------*/
/* III) DEBUG                                                              */


#ifdef DEBUG      
  for (itors=1; itors<=ntors; itors++) {
    printf("Tors %2d:  %d(%s)--%d(%s)--%d(%s)--%d(%s)\n", itors,
   	   atom1[itors], maps->atm_label[atom1[itors]],
   	   atom2[itors], maps->atm_label[atom2[itors]],
   	   atom3[itors], maps->atm_label[atom3[itors]],
   	   atom4[itors], maps->atm_label[atom4[itors]]);
    printf("           {p0=%.8g, p1=%.8g, p2=%.8g, p3=%.8g}\n",
   	   p0[itors], p1[itors], p2[itors], p3[itors]);
    printf("           {p4=%.8g, p5=%.8g, p6=%.8g}\n",
   	   p4[itors], p5[itors], p6[itors]);
    printf("           {s0=%.8g, s1=%.8g, s2=%.8g, s3=%.8g}\n",
   	   s0[itors], s1[itors], s2[itors], s3[itors]);
    printf("           {s4=%.8g, s5=%.8g, s6=%.8g}\n",
   	   s4[itors], s5[itors], s6[itors]);
    printf("           {sp2=%.8g, sp3=%.8g}\n",
   	   sp2[itors], sp3[itors]);
    printf("           {sp4=%.8g, sp5=%.8g, sp6=%.8g}\n",
   	   sp4[itors], sp5[itors], sp6[itors]);
  }
#endif

  printf("Tors setting up done.\n");
  printf("----------------------------------------------\n");

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
void Tors::setup(const Pot_data_entry *database, const Maps *maps, 
		 const char *parm_filename)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, iii, itors, jtors, n; 
  int natm_tot, nkey, key_defined, line_num;
  int ntors_now, ntors_mall;
  int imp_tors, atom1_ind, atom2_ind, atom3_ind, atom4_ind;
  int tors_check, key_found[6];
  double fk_tors, theta_0;
  double c2th0,c4th0,c6th0;                 /* Cos(n*theta_0)          */
  double s2th0,s4th0,s6th0;                 /* Sin(n*theta_0)          */
  /* Useful constants        */
  static double cn1=49.0/72.0,cn2=3.0/4.0,cn3=3.0/40.0,cn4=1.0/180.0;
  static double cn5=3.0/2.0,cn6=3.0/5.0,cn7=1.0/10.0,cn8=4.0/15.0;
  static double cn9=8.0/45.0,cn10=3.0/10.0,cn11=1.0/30.0;
  static double small=1.0e-8;
  int  atoms[5];
  const Pot_data_entry *database_entry, *exact_match_entry, *wild_match_entry;

  char   err_msg[LINE_NCHARMAX];  
  char entry_line[LINE_NCHARMAX];
  char entry_name[NCHARMAX], key[NCHARMAX], arg[NCHARMAX], str_arg[NCHARMAX];;
  char *def;
  FILE *parm_file;  
  
  char *tors_def_keys[] = {"", "atom1", "atom2", "atom3", "atom4", "label"};

  printf("----------------------------------------------\n");
  printf("Setting up tors\n...\n");

  if ((parm_file=fopen(parm_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open parm file %s.", parm_filename);
    errorMessage(err_msg);
  }
  
  ntors_mall= NMEM_MIN;
  ntors_now = 0;
  natm_tot  = maps->natm_tot;
  line_num  = 0;
  
  atom1    =  new int[ntors_mall]-1;
  atom2    =  new int[ntors_mall]-1;
  atom3    =  new int[ntors_mall]-1;
  atom4    =  new int[ntors_mall]-1;
  improper =  new int[ntors_mall]-1;
  
  while((n=readOneEntry(parm_file,'~','[',']',entry_line, line_num))!=0) {
    if (n!=1) {
      sprintf(err_msg, "%s at line %d while reading file %s.",
	      (n==-1) ? "Syntax error" : "Internal error", 
	      line_num, parm_filename);
      errorMessage(err_msg);
    }
    n=getEntryName(entry_line, '~', '[', entry_name, def);
    if (n!=1) {
      sprintf(err_msg, "Syntax error at parm file line:\n%s", entry_line);
      errorMessage(err_msg);
    }
    if (strcasecmp(entry_name, "torsion_def")==0) {
      nkey = 5;
      for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
      key_found[5] = 1;  imp_tors = 0;  /*Default not to be improper */
      while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
	key_defined = 0;
	for ( i=1; i<=nkey; i++) {                  /*loop for key words.*/
	  if (strcasecmp(key, tors_def_keys[i])==0) {/*it's a keyword.  */
	    key_defined  = 1;
	    key_found[i] = 1;
	    switch (i) {
	    case 1:
	      sscanf(arg, "%d", &atom1_ind);
	      break;
	    case 2:
	      sscanf(arg, "%d", &atom2_ind);
	      break;
	    case 3:
	      sscanf(arg, "%d", &atom3_ind);
	      break;
	    case 4:
	      sscanf(arg, "%d", &atom4_ind);
	      break;
	    case 5:
	      strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "improper")==0) {
	         imp_tors = 1;
	      }
	    break;
	    }/*switch */
	  }
	}/*end for*/
	if (key_defined==0) {
	  sprintf(err_msg, "Key word %s unknown in parmfile line:\n%s",
		  key, entry_line);
	  errorMessage(err_msg);
	}
      }/*end while: getKeyArgPair*/
      if (n!=0) {
	sprintf(err_msg, "Syntax error in parm file line:\n%s",entry_line);
	errorMessage(err_msg);
      }
      for ( i=1; i<=nkey; i++) { 
	if (key_found[i]==0) {
	  sprintf(err_msg, "Key word %s missed in parm file line:\n%s",
		  tors_def_keys[i], entry_line);
	  errorMessage(err_msg);
	}
      }
      if (atom1_ind<1 || atom1_ind>natm_tot || 
	  atom2_ind<1 || atom2_ind>natm_tot || 
	  atom3_ind<1 || atom3_ind>natm_tot ||
	  atom4_ind<1 || atom4_ind>natm_tot    ) {
	sprintf(err_msg, "TORS::setup(): Illegal atom index in parm file %s.",
		parm_filename);
	errorMessage(err_msg);
      }
      ntors_now++;
      if (ntors_now>ntors_mall) {
	ntors_mall += NMEM_MIN;
	atom1       = (int *) crealloc(atom1+1, ntors_mall*sizeof(int))-1;
	atom2       = (int *) crealloc(atom2+1, ntors_mall*sizeof(int))-1;
	atom3       = (int *) crealloc(atom3+1, ntors_mall*sizeof(int))-1;
	atom4       = (int *) crealloc(atom4+1, ntors_mall*sizeof(int))-1;
	improper    = (int *) crealloc(improper+1, ntors_mall*sizeof(int))-1;
      }
      atom1[ntors_now]    = atom1_ind;
      atom2[ntors_now]    = atom2_ind;
      atom3[ntors_now]    = atom3_ind;
      atom4[ntors_now]    = atom4_ind;
      improper[ntors_now] = imp_tors;
    }/*end if: tors_def */
  }/*end while: readOneEntry*/
  fclose(parm_file);
  
  ntors = ntors_now;
  type     = new int[ntors] -1;
  p0    = new double[ntors] -1;
  p1    = new double[ntors] -1;
  p2    = new double[ntors] -1;
  p3    = new double[ntors] -1;
  p4    = new double[ntors] -1;
  p5    = new double[ntors] -1;
  p6    = new double[ntors] -1;
  s0    = new double[ntors] -1;
  s1    = new double[ntors] -1;
  s2    = new double[ntors] -1;
  s3    = new double[ntors] -1;
  s4    = new double[ntors] -1;
  s5    = new double[ntors] -1;
  s6    = new double[ntors] -1;
  sp2   = new double[ntors] -1;
  sp3   = new double[ntors] -1;
  sp4   = new double[ntors] -1;
  sp5   = new double[ntors] -1;
  sp6   = new double[ntors] -1;
  eq    = new double[ntors] -1;

  for (itors=1; itors<=ntors; itors++) {
    if (atom1[itors]==atom2[itors] ||
	atom1[itors]==atom3[itors] ||
	atom1[itors]==atom4[itors] ||
	atom2[itors]==atom3[itors] ||
	atom2[itors]==atom4[itors] ||
	atom3[itors]==atom4[itors] ) {
      sprintf(err_msg, "Ilegal tors #%d: same atom index.", itors);
      errorMessage(err_msg);
    }
  }
  
  nimproper = 0;  
  for (itors=1; itors<=ntors; itors++) {
    atoms[1] = atom1[itors];
    atoms[2] = atom2[itors];
    atoms[3] = atom3[itors];
    atoms[4] = atom4[itors];
    exact_match_entry = NULL;
    wild_match_entry  = NULL;
    database_entry    = database;
    tors_check = 0;
    while (database_entry!=NULL && tors_check!=1) {
      tors_check = database_entry->match(maps, atoms, 4);
      if (tors_check==1) {
	 if (improper[itors]==database_entry->imp_tors) {
            exact_match_entry = database_entry;
         }
         else {tors_check=0;}
      }
      if (tors_check==2) { 
	 if (improper[itors]==database_entry->imp_tors) {
             wild_match_entry = database_entry; 
         }
         else {tors_check=0;}
      }      
      database_entry = database_entry->next;
    }/*end while */
#define DEBUG_OFF
#ifdef DEBUG
    printf(" parm atoms ");
    printf("%s ",maps->atm_label[atoms[1]]);
    printf("%s ",maps->atm_label[atoms[2]]);
    printf("%s ",maps->atm_label[atoms[3]]);
    printf("%s\n",maps->atm_label[atoms[4]]);
#endif
#undef DEBUG
    if (exact_match_entry==NULL && wild_match_entry==NULL) {
      sprintf(err_msg, "Torsion is not found in database.\nindices %d %d %d %d\nlabels %s %s %s %s",
	      atoms[1], atoms[2], atoms[3], atoms[4],
              maps->atm_label[atoms[1]],maps->atm_label[atoms[2]],
              maps->atm_label[atoms[3]],maps->atm_label[atoms[4]]);
      errorMessage(err_msg);
    }
    if (exact_match_entry!=NULL) { database_entry = exact_match_entry;}
    else                         { database_entry = wild_match_entry; }
    
    type[itors]     = database_entry->pot_type;
    /* improper[itors] = database_entry->imp_tors;  no need for it */ 
    nimproper      += improper[itors];
    switch(type[itors]) {
    case 1:
      p0[itors] = database_entry->p0;
      p1[itors] = database_entry->p1;
      p2[itors] = database_entry->p2;
      p3[itors] = database_entry->p3;
      p4[itors] = database_entry->p4;
      p5[itors] = database_entry->p5;
      p6[itors] = database_entry->p6;

      s0[itors] = 0.0;
      s1[itors] = database_entry->s1;
      s2[itors] = database_entry->s2;
      s3[itors] = database_entry->s3;
      s4[itors] = database_entry->s4;
      s5[itors] = database_entry->s5;
      s6[itors] = database_entry->s6;

      sp2[itors] = database_entry->sp2;
      sp3[itors] = database_entry->sp3;
      sp4[itors] = database_entry->sp4;
      sp5[itors] = database_entry->sp5;
      sp6[itors] = database_entry->sp6;
      break;
    case 2: /*Not implemented */
      break;
    case 3:
      theta_0  = database_entry->eq_tors;
      fk_tors  = database_entry->fk_tors;
      eq[itors]= theta_0;
      
      c2th0 = cos(2.0*theta_0);
      c4th0 = cos(4.0*theta_0);
      c6th0 = cos(6.0*theta_0);
      p0[itors] = 0.5*fk_tors*(cn1 + cn2*c2th0 + cn3*c4th0 + cn4*c6th0);
      p1[itors] = 0.0;
      p2[itors] = 0.5*fk_tors*(-cn5*c2th0 - cn6*c4th0 - cn7*c6th0);
      p3[itors] = 0.0;
      p4[itors] = 0.5*fk_tors*(cn6*c4th0 + cn8*c6th0);
      p5[itors] = 0.0;
      p6[itors] = 0.5*fk_tors*(-cn9*c6th0);
      
      s2th0 = sin(2.0*theta_0);
      s4th0 = sin(4.0*theta_0);
      s6th0 = sin(6.0*theta_0);
      s0[itors] = 0.0;
      s1[itors] = 0.0;
      s2[itors] = 0.5*fk_tors*(-cn5*s2th0 + cn10*s4th0 - cn11*s6th0);
      s3[itors] = 0.0;
      s4[itors] = 0.5*fk_tors*(-cn6*s4th0 + cn9*s6th0);
      s5[itors] = 0.0;
      s6[itors] = 0.5*fk_tors*(-cn9*s6th0);

      sp2[itors] = 0.0;
      sp3[itors] = 0.0;
      sp4[itors] = 0.0;
      sp5[itors] = 0.0;
      sp6[itors] = 0.0;
      break;
    }
  }/*end for: itors */
  
  for (itors=1; itors<=ntors; itors++) {
    for (jtors=itors+1; jtors<=ntors; jtors++) {
      if (atom1[itors]==atom1[jtors] &&
	  atom2[itors]==atom2[jtors] &&
	  atom3[itors]==atom3[jtors] &&
	  atom4[itors]==atom4[jtors]   ){
	sprintf(err_msg, "Tors %d and %d repeat.", itors, jtors);
	errorMessage(err_msg);
      }
      if (atom1[itors]==atom4[jtors] &&
	  atom2[itors]==atom3[jtors] &&
	  atom3[itors]==atom2[jtors] &&
	  atom4[itors]==atom1[jtors]   ){
	sprintf(err_msg, "Tors %d and %d repeat.", itors, jtors);
	errorMessage(err_msg);
      }
    }
  }
      
#ifdef DEBUG      
  for (itors=1; itors<=ntors; itors++) {
    printf("Tors %2d:  %d(%s)--%d(%s)--%d(%s)--%d(%s)\n", itors,
   	   atom1[itors], maps->atm_label[atom1[itors]],
   	   atom2[itors], maps->atm_label[atom2[itors]],
   	   atom3[itors], maps->atm_label[atom3[itors]],
   	   atom4[itors], maps->atm_label[atom4[itors]]);
    printf("           {p0=%.8g, p1=%.8g, p2=%.8g, p3=%.8g}\n",
   	   p0[itors], p1[itors], p2[itors], p3[itors]);
    printf("           {p4=%.8g, p5=%.8g, p6=%.8g}\n",
   	   p4[itors], p5[itors], p6[itors]);
    printf("           {s0=%.8g, s1=%.8g, s2=%.8g, s3=%.8g}\n",
   	   s0[itors], s1[itors], s2[itors], s3[itors]);
    printf("           {s4=%.8g, s5=%.8g, s6=%.8g}\n",
   	   s4[itors], s5[itors], s6[itors]);
    printf("           {sp2=%.8g, sp3=%.8g}\n",
   	   sp2[itors], sp3[itors]);
    printf("           {sp4=%.8g, sp5=%.8g, sp6=%.8g}\n",
   	   sp4[itors], sp5[itors], sp6[itors]);

  }
#endif

  printf("Tors setting up done.\n");
  printf("----------------------------------------------\n");

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/



/*========================================================================*/
/*========================================================================*/
double Tors::energy(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, itors;
  int atom1_ind, atom2_ind, atom3_ind, atom4_ind;
  
  double *x, *y, *z, *fx, *fy, *fz;
  double dx21, dy21, dz21;
  double dx31, dy31, dz31;
  double dx23, dy23, dz23;
  double dx24, dy24, dz24;
  double dx34, dy34, dz34;
  double n123[3], n234[3], l123, l234;
  double rq1x, rq1y, rq1z, rq2x, rq2y, rq2z;
  double cost, sint, seps, phi, pre, vtors_vimpr;
  double vtorsc, vtorss, sisum, vtorssp, dvtorsc, dvtorss, dvtorssp;     
  
  seps = 1.0e-8;
  
  x  = pos->x;
  y  = pos->y;
  z  = pos->z;
  fx = pos->fx;
  fy = pos->fy;
  fz = pos->fz;


/*---------------------------------------------------------------------------*/
/* 0) Return if no tors term                                                 */

  if (energy_term != 1) { 
      vtors = 0.0;
      vimpr = 0.0;
      return (vtors+vimpr);
  }/*endif*/


/*---------------------------------------------------------------------------*/
/* I) Evaluate energy                                                        */

  vtors=0;
  vimpr=0;
  for (itors=1; itors<=ntors; itors++) {
    atom1_ind = atom1[itors];
    atom2_ind = atom2[itors];
    atom3_ind = atom3[itors];
    atom4_ind = atom4[itors];
    dx21 = x[atom1_ind] - x[atom2_ind];
    dy21 = y[atom1_ind] - y[atom2_ind];
    dz21 = z[atom1_ind] - z[atom2_ind];

    dx31 = x[atom1_ind] - x[atom3_ind];
    dy31 = y[atom1_ind] - y[atom3_ind];
    dz31 = z[atom1_ind] - z[atom3_ind];

    dx23 = x[atom3_ind] - x[atom2_ind];
    dy23 = y[atom3_ind] - y[atom2_ind];
    dz23 = z[atom3_ind] - z[atom2_ind];

    dx24 = x[atom4_ind] - x[atom2_ind];
    dy24 = y[atom4_ind] - y[atom2_ind];
    dz24 = z[atom4_ind] - z[atom2_ind];

    dx34 = x[atom4_ind] - x[atom3_ind];
    dy34 = y[atom4_ind] - y[atom3_ind];
    dz34 = z[atom4_ind] - z[atom3_ind];
    
    n123[0] = dy23*dz21 - dz23*dy21;
    n123[1] = dz23*dx21 - dx23*dz21;
    n123[2] = dx23*dy21 - dy23*dx21;
    
    n234[0] = dy23*dz34 - dz23*dy34;
    n234[1] = dz23*dx34 - dx23*dz34;
    n234[2] = dx23*dy34 - dy23*dx34;
    
    l123 = sqrt(n123[0]*n123[0]+n123[1]*n123[1]+n123[2]*n123[2]);
    l234 = sqrt(n234[0]*n234[0]+n234[1]*n234[1]+n234[2]*n234[2]);

    cost  = n123[0]*n234[0] + n123[1]*n234[1] + n123[2]*n234[2];
    cost /= (l123*l234); 
    if (cost> 1) cost = 1;
    if (cost<-1) cost =-1;
    sint = sqrt(1.0 - cost*cost);
    sint = (sint > seps ? sint:seps);
    
    vtorsc = ((((((p6[itors]
		   *cost + p5[itors])
		  *cost + p4[itors])
		 *cost + p3[itors])
		*cost + p2[itors])
	       *cost + p1[itors])
	      *cost + p0[itors]);
    sisum = (((((s6[itors]
		 *sint + s5[itors])
		*sint + s4[itors])
	       *sint + s3[itors])
	      *sint + s2[itors])
	     *sint);
    vtorss = sisum*cost + s1[itors]*sint;
    vtorssp = (((((sp6[itors]
		 *sint + sp5[itors])
		*sint + sp4[itors])
	       *sint + sp3[itors])
	      *sint + sp2[itors])
	     *sint
            *sint);

    if (improper[itors]==0) {vtors += vtorsc + vtorss + vtorssp;}
    if (improper[itors]==1) {vimpr += vtorsc + vtorss + vtorssp;}
    
    rq1x = n234[0] - cost*l234/l123*n123[0];
    rq1y = n234[1] - cost*l234/l123*n123[1];
    rq1z = n234[2] - cost*l234/l123*n123[2];
    rq2x = n123[0] - cost*l123/l234*n234[0];
    rq2y = n123[1] - cost*l123/l234*n234[1];
    rq2z = n123[2] - cost*l123/l234*n234[2];
    dvtorsc  = (((((6.0*p6[itors]
		   *cost + 5.0*p5[itors])
		  *cost + 4.0*p4[itors])
		 *cost + 3.0*p3[itors])
		*cost + 2.0*p2[itors])
	       *cost + 1.0*p1[itors]);
    dvtorss  = ((((5.0*s6[itors]
		  *sint + 4.0*s5[itors])
		 *sint + 3.0*s4[itors])
		*sint + 2.0*s3[itors])
	       *sint + 1.0*s2[itors]);
    dvtorss  = dvtorss*cost + s1[itors];
    dvtorssp = (((((6.0*sp6[itors]
		   *sint + 5.0*sp5[itors])
		  *sint + 4.0*sp4[itors])
		 *sint + 3.0*sp3[itors])
		*sint + 2.0*sp2[itors])
	       *sint);

    pre =-(dvtorsc + sisum - (dvtorss+dvtorssp)*cost/sint);    
    pre /= (l123*l234);
    fx[atom1_ind] += (rq1y*dz23-rq1z*dy23)*pre;
    fy[atom1_ind] += (rq1z*dx23-rq1x*dz23)*pre;
    fz[atom1_ind] += (rq1x*dy23-rq1y*dx23)*pre;
    
    fx[atom2_ind] += (rq1y*dz31-rq1z*dy31)*pre;
    fy[atom2_ind] += (rq1z*dx31-rq1x*dz31)*pre;
    fz[atom2_ind] += (rq1x*dy31-rq1y*dx31)*pre;

    fx[atom3_ind] -= (rq1y*dz21-rq1z*dy21)*pre;
    fy[atom3_ind] -= (rq1z*dx21-rq1x*dz21)*pre;
    fz[atom3_ind] -= (rq1x*dy21-rq1y*dx21)*pre;

    fx[atom2_ind] += (rq2y*dz34-rq2z*dy34)*pre;
    fy[atom2_ind] += (rq2z*dx34-rq2x*dz34)*pre;
    fz[atom2_ind] += (rq2x*dy34-rq2y*dx34)*pre;
    
    fx[atom3_ind] -= (rq2y*dz24-rq2z*dy24)*pre;
    fy[atom3_ind] -= (rq2z*dx24-rq2x*dz24)*pre;
    fz[atom3_ind] -= (rq2x*dy24-rq2y*dx24)*pre;

    fx[atom4_ind] += (rq2y*dz23-rq2z*dy23)*pre;
    fy[atom4_ind] += (rq2z*dx23-rq2x*dz23)*pre;
    fz[atom4_ind] += (rq2x*dy23-rq2y*dx23)*pre;
  }/*end for*/  
  vtors_vimpr = vtors + vimpr;
  return vtors_vimpr;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
double Tors::getPot(const Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, itors;
  int atom1_ind, atom2_ind, atom3_ind, atom4_ind;
  
  double *x, *y, *z;
  double dx21, dy21, dz21;
  double dx23, dy23, dz23;
  double dx34, dy34, dz34;
  double n123[3], n234[3], l123, l234;
  double cost, sint, seps, phi, vtors_vimpr;
  double vtorsc, vtorss, sisum, vtorssp;

  seps = 1.0e-8;
  
  x  = pos->x;
  y  = pos->y;
  z  = pos->z;


/*---------------------------------------------------------------------------*/
/* 0) Return if no tors term                                                 */

  if (energy_term != 1) { 
      vtors = 0.0;
      vimpr = 0.0;
      return (vtors+vimpr);
  }/*endif*/

/*---------------------------------------------------------------------------*/
/* I) Evaluate energy                                                        */

  vtors=0;
  vimpr=0;
  for (itors=1; itors<=ntors; itors++) {
    atom1_ind = atom1[itors];
    atom2_ind = atom2[itors];
    atom3_ind = atom3[itors];
    atom4_ind = atom4[itors];
    dx21 = x[atom1_ind] - x[atom2_ind];
    dy21 = y[atom1_ind] - y[atom2_ind];
    dz21 = z[atom1_ind] - z[atom2_ind];

    dx23 = x[atom3_ind] - x[atom2_ind];
    dy23 = y[atom3_ind] - y[atom2_ind];
    dz23 = z[atom3_ind] - z[atom2_ind];

    dx34 = x[atom4_ind] - x[atom3_ind];
    dy34 = y[atom4_ind] - y[atom3_ind];
    dz34 = z[atom4_ind] - z[atom3_ind];
    
    n123[0] = dy23*dz21 - dz23*dy21;
    n123[1] = dz23*dx21 - dx23*dz21;
    n123[2] = dx23*dy21 - dy23*dx21;
    
    n234[0] = dy23*dz34 - dz23*dy34;
    n234[1] = dz23*dx34 - dx23*dz34;
    n234[2] = dx23*dy34 - dy23*dx34;
    
    l123 = sqrt(n123[0]*n123[0]+n123[1]*n123[1]+n123[2]*n123[2]);
    l234 = sqrt(n234[0]*n234[0]+n234[1]*n234[1]+n234[2]*n234[2]);

    cost  = n123[0]*n234[0] + n123[1]*n234[1] + n123[2]*n234[2];
    cost /= (l123*l234); 
    if (cost> 1) cost = 1;
    if (cost<-1) cost =-1;
    sint = sqrt(1.0 - cost*cost);
    sint = (sint > seps ? sint:seps);
    
    vtorsc = ((((((p6[itors]
		   *cost + p5[itors])
		  *cost + p4[itors])
		 *cost + p3[itors])
		*cost + p2[itors])
	       *cost + p1[itors])
	      *cost + p0[itors]);
    sisum = (((((s6[itors]
		 *sint + s5[itors])
		*sint + s4[itors])
	       *sint + s3[itors])
	      *sint + s2[itors])
	     *sint);
    vtorssp = (((((sp6[itors]
		 *sint + sp5[itors])
		*sint + sp4[itors])
	       *sint + sp3[itors])
	      *sint + sp2[itors])
	     *sint
            *sint);
    vtorss = sisum*cost + s1[itors]*sint;
    if (improper[itors]==0) {vtors += vtorsc + vtorss + vtorssp;}
    if (improper[itors]==1) {vimpr += vtorsc + vtorss + vtorssp;}
  }/*end for*/  
  vtors_vimpr = vtors + vimpr;
  return vtors_vimpr;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
double Tors::getTorsPot(int itors, double cost)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  double sint, seps, vtors_vimpr;
  double vtorsc, vtorss, sisum, vtorssp;
  
  seps = 1.0e-8;
  sint = sqrt(1.0 - cost*cost);
  sint = (sint > seps ? sint:seps);
  
  vtorsc = ((((((p6[itors]
		 *cost + p5[itors])
		*cost + p4[itors])
	       *cost + p3[itors])
	      *cost + p2[itors])
	     *cost + p1[itors])
	    *cost + p0[itors]);
  sisum = (((((s6[itors]
	       *sint + s5[itors])
	      *sint + s4[itors])
	     *sint + s3[itors])
	    *sint + s2[itors])
	   *sint);
  vtorss = sisum*cost + s1[itors]*sint;
  vtorssp = (((((sp6[itors]
		 *sint + sp5[itors])
		*sint + sp4[itors])
	       *sint + sp3[itors])
	      *sint + sp2[itors])
	     *sint
            *sint);

  if (improper[itors]==0) {vtors  = vtorsc + vtorss + vtorssp;}
  if (improper[itors]==1) {vimpr  = vtorsc + vtorss + vtorssp;}
  vtors_vimpr = vtors + vimpr;
  return vtors_vimpr;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/
