#define CRYO_EM_OFF

#ifdef CRYO_EM
#include "../include/cryo_em/Sim_Score_Matrix.h"
#endif

#include "definitions.h"

#ifdef CRYO_EM
using namespace EMAN;
#endif

class General;
class Energy_part;
class System;
class Coord;
class Chain;
class Pos;
class Maps;
class Disk;
class Filenames;
class Restop_data;
class Regions;
class REgion;
class Region_data_entry;
class Rotate_utils;
class Tors_defs;


/*-------------------------------------------------------------------------*/
class Pdb_data_entry {
 public: 
  int atom_number;
  int residue_number;
  char chain_id[3], altloc[3];
  char field_name[10];
  char atom_name[10];
  char residue_name[10];
  double coord_x, coord_y, coord_z;
  Pdb_data_entry *next;
 public:
  void setup(const char *);
};//Pdb_data_entry//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Rtf_data_entry {
 public:
  int nfield;
  char **field;
  Rtf_data_entry *next;
 public:
  int setup(char *);
};//Rtf_data_entry//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Res_data_entry {
 public:
  int natom;
  int residue_number_old;
  int residue_number;
  int atom1_global_number;           
  int   *atom_number;   
  char  *residue_label;
  char  *chain_id;
  char **atom_label;
  double *coord_x,*coord_y,*coord_z;  
 public:
  void setup(Pdb_data_entry*, General*, int, int, int);
  void write_alldata();
};//Res_data_entry//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Residue {
 public:
  char residue_name[10];              /* 3 letter rtf name of residue      */
  int residue_index;                  /* residue index, e.g. 3 if 3rd res. */
  int natom;                          /* number of atoms in residue        */
  int nbond, ndbond;                  /* number of bonds, double bonds     */
  int nbend;                          /* number of bends                   */
  int ntors, nimpr;                   /* number of torsions, improper      */
  int nonfo;                          /* number of onefour                 */
  int nbimpr;                         /* number of bondimproper, [0,1,2]   */
  int nexclude;                       /* number of exclusions in residue   */
  int nchigrp;                        /* number of side chain chi groups   */
  int multiple_type;                  /* 1: multiple type, 0: regular      */
  int ntype;                          /* number of types                   */
  int natom_shared;                   /* number of shared atoms among types*/
  int max_natom_type;                 /* max{natom_type[1..ntype]}         */
  int *natm_chigrp;                   /* # of atms in chigps, Lth: nchigrp */
  int *atom;                          /* global atom indeces, Lth: natom   */
  int *bond;                          /* global bond indeces,  Lth: nbond  */ 
  int *bend;                          /* global bend indeces,  Lth: nbend  */ 
  int *tors;                          /* global tors indeces,  Lth: ntors  */ 
  int *impr;                          /* global impr indeces,  Lth: nimpr  */ 
  int *onfo;                          /* global onfo indeces,  Lth: nonfo  */
  int *exclude;                       /* gl.exclude indeces, Lth: nexclude */
  int **bond_atom;                    /* bond atm indeces,  Lth: nbond x 2 */ 
  int **dbond_atom;                   /* dbond atm indeces, Lth: ndbond x 2*/ 
  int **bend_atom;                    /* bend atm indeces, Lth: nbend x 3  */ 
  int **tors_atom;                    /* tors atm indeces, Lth: ntors x 4  */
  int **impr_atom;                    /* impr atm indeces, Lth: nimpr x 4  */
  int **onfo_atom;                    /* onfo atm indeces, Lth: nonfo x 2  */
  int **exclude_atom;                 /* excl. atm inds, Lth: nexclude x 2 */ 
  int **chigrp_atom;                  /* ind. Lth: nchigrp x natm_chigrp[] */
  int **chiprim_atom;                 /* prim tors ind. Lth: nchigrp x 4   */
  int nbranch_root;                   /* 0: no root, 1: one root, ...      */
  int **branch_root_f;                /* branching from root first         */  
  int **branch_root_s;                /* branching from root second        */  
  int   nbranch_f1;                   /* Number of branches at depth 1     */
  int  *nbranch_f2;                   /* Lth:  nbranch_f1                  */ 
  int   nbranch_s1;                   /* Number of branches at depth 1     */ 
  int  *nbranch_s2;                   /* Lth:  nbranch_s1                  */
  int  *bimpr_atom_f;                 /* 1st bond impr indeces, Lth: 1 x 4 */
  int  *bimpr_atom_s;                 /* 2nd bond impr indeces, Lth: 1 x 4 */
  int  *ind_atom_shared;              /* index of shared atoms among types */
                                      /*                 Lth: natom_shared */
  int  *natom_type;                   /* number of unique atoms for each   */
                                      /* type,                  Lth: ntype */
  int **ind_atom_type;                /*      Lth: ntype x max{natom_type} */
  char  **type_label;                  /*                        Lth: ntype */
  double *type_weight;                /* the weight of a type,  Lth: ntype */
 public: 
  Residue();
 ~Residue();
  void setup( Maps*, Restop_data*, Res_data_entry*, Pos*);
  void write_alldata( Maps* , Pos*);
  void clean();
};//Residue//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Residue_bond {
 public:
  char residue1_name[10];             /* 3 letter rtf name of residue      */
  char residue2_name[10];             /* 3 letter rtf name of residue      */
  int residue1_index;                 /* residue1 index, e.g. 3 if 3rd res.*/
  int residue2_index;                 /* residue2 index, e.g. 4 if 4th res.*/
  int nbond;                          /* number of bonds, always 1         */
  int nbend;                          /* number of bends                   */
  int ntors, nimpr;                   /* number of torsions, improper      */
  int nonfo;                          /* number of onefour                 */
  int nexclude;                       /* number of exclusions in residue   */
  int *bond;                          /* bond indeces,  Lth: nbond         */ 
  int *bend;                          /* bend indeces,  Lth: nbend         */ 
  int *tors;                          /* tors indeces,  Lth: ntors         */ 
  int *impr;                          /* impr indeces,  Lth: nimpr         */ 
  int *onfo;                          /* onfo indeces,  Lth: nonfo         */ 
  int *exclude;                       /* gl.exclude indeces, Lth: nexclude */
  int **bond_atom;                    /* bond indeces,  Lth: nbond x 2     */ 
  int **bend_atom;                    /* bend indeces, Lth: nbend x 3      */ 
  int **tors_atom;                    /* tors indeces, Lth: ntors x 4      */
  int **impr_atom;                    /* tors indeces, Lth: nimpr x 4      */
  int **onfo_atom;                    /* onfo indeces, Lth: nonfo x 2      */ 
  int **exclude_atom;                 /* excl. atm inds, Lth: nexclude x 2 */ 
 public: 
  Residue_bond();
 ~Residue_bond();
  void setup_1pt( Maps*, Residue*, int ** );
  void setup( Maps*, Residue*, Residue*, Pos* );
  void create( Maps*, Residue*, Residue* );
  void write_alldata( Maps* );
};//Residue_bond//
/*-------------------------------------------------------------------------*/
  
/*-------------------------------------------------------------------------*/
class Restop_data {
 public:
  char residue_type[10];              /* RESI or PRES                      */
  char residue_name[10];              /* 3 letter rtf name of residue      */
  double residue_chrg;                /* net charge of residue             */
  int unique_atom_list;               /* 1: unique atom list, 0 none       */
  int ngroup;                         /* number of groups in residue       */
  int natom;                          /* number of atoms in residue        */
  int nbond, ndbond;                  /* number of bonds, double bonds     */
  int nbend;                          /* number of bends                   */
  int ntors, nimpr;                   /* number of torsions, improper      */
  int nonfo;                          /* number of onefour                 */
  int nbimpr;                         /* number of bondimproper, [0,1,2]   */
  int nexclude;                       /* number of exclusions in residue   */
  int nchigrp;                        /* number of side chain chi groups   */
  int *natm_chigrp;                   /* # of atms in chigps, Lth: nchigrp */
  int *atom_gindex;                   /* atom group index, Lth: natom      */
  char **atom_label;                  /* atom label, Lth: natom x NCHARMAX */
  char **upac_atom_label;             /* upac  ,Lth: [0,natom+1]x NCHARMAX */
  double *mass;                       /* atom masses, Lth: natom           */
  double *charge;                     /* partial charges, Lth: natom       */
  int *group_begin, *group_end;       /* group boundaries, Lth: ngroup     */
  int **bond_atom;                    /* bond indeces,  Lth: nbond x 2     */ 
  int **dbond_atom;                   /* dbond indeces, Lth: ndbond x 2    */ 
  int **bend_atom;                    /* bend indeces, Lth: nbend x 3      */ 
  int **tors_atom;                    /* tors indeces, Lth: ntors x 4      */
  int **impr_atom;                    /* tors indeces, Lth: nimpr x 4      */
  int **onfo_atom;                    /* onfo indeces, Lth: nonfo x 2      */ 
  int **exclude_atom;                 /* excl. atm inds, Lth: nexclude x 2 */
  int **chigrp_atom;                  /* ind. Lth: nchigrp x natm_chigrp[] */
  int **chiprim_atom;                 /* prim tors ind. Lth: nchigrp x 4   */
  int nbranch_root;                   /* 0: no root, 1: one root, ...      */
  int **branch_root_f;                /* branching from root first         */  
  int **branch_root_s;                /* branching from root second        */  
  int   nbranch_f1;                   /* Number of branches at depth 1     */
  int  *nbranch_f2;                   /* Lth:  nbranch_f1                  */
  int   nbranch_s1;                   /* Number of branches at depth 1     */ 
  int  *nbranch_s2;                   /* Lth:  nbranch_s1                  */
  int  *bimpr_atom_f;                 /* 1st bond impr indeces, Lth: 1 x 4 */
  int  *bimpr_atom_s;                 /* 2nd bond impr indeces, Lth: 1 x 4 */
  int *nbond_atm;                     /* Lth: natom, # of bonds for atoms  */
  int **bond_atm_map;                 /* bond_atom_map,Lth: natom x nabond */ 
 public:
  Restop_data();
 ~Restop_data(); 
  void setup(General, Rtf_data_entry*, int, Rtf_data_entry*, int);
  void write_alldata();
};//Restop_data//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/*                            class Design                                 */
/*-------------------------------------------------------------------------*/
/*         resA_ind         : index of residue to exchange                 */ 
/*         resB_ind         : index of residue to exchange                 */ 
/*         resbnd_resAf_ind : index of the residue bond, which involves    */
/*                            branch_root_f[][] of resA                    */ 
/*         resbnd_resAs_ind : index of the residue bond, which involves    */
/*                            branch_root_s[][] of resA                    */
/*         resbnd_resBf_ind : index of the residue bond, which involves    */
/*                            branch_root_f[][] of resB                    */ 
/*         resbnd_resBs_ind : index of the residue bond, which involves    */
/*                            branch_root_s[][] of resB                    */
/*         res_resAf_ind    : index of the residue, which bonds to         */
/*                            branch_root_f[][] of resA                    */
/*         res_resAs_ind    : index of the residue, which bonds to         */
/*                            branch_root_s[][] of resA                    */
/*         res_resBf_ind    : index of the residue, which bonds to         */
/*                            branch_root_f[][] of resB                    */
/*         res_resBs_ind    : index of the residue, which bonds to         */
/*                            branch_root_s[][] of resB                    */
/*         resAf, resAs     : proposal residue bonds for resA              */
/*         resBf, resBs     : proposal residue bonds for resB              */
/*         if non existent  resbnd_res**_ind = 0                           */
/*-------------------------------------------------------------------------*/ 
class Design {
 public:
  int  resA_ind, resB_ind;
  int  resbnd_resAf_ind, resbnd_resAs_ind;
  int  resbnd_resBf_ind, resbnd_resBs_ind;
  int  res_resAf_ind, res_resAs_ind;
  int  res_resBf_ind, res_resBs_ind;
  Residue_bond resAf, resAs, resBf, resBs;    
 public:
  void allocate();
  void setup(Pos *, Maps *);
};//Design//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Maps {
 public:
  int  natm_tot;                /* number of atoms                         */
  int  natm_bb;                 /* number of backbone atoms                */
  int  nbond, nbend;            /* number of bonds, bends                  */ 
  int  ntors, nimpr, nonfo;     /* number of torsions, impropers, onefours */
  int  nexclude;                /* number of exclusions                    */
  int  nres;                    /* number of residues                      */
  int  nresbond;                /* number of residue bonds                 */
  int  inter_on_3bond_conn, inter_on_4bond_conn;/* turn on 3 and 4 bond    */
                                                /*      connected inter    */ 
  int  cancel_res_inter_local;     /* 1: cancel inter within the residue   */
  int  cancel_res_inter_neighbor;  /* 1: ... between neighboring residues  */
  int  lnk_list;                   /* 1: linked list is used in inter      */
  int  neighbor_list;              /* 1: neighbor list is used in inter    */

  int postprop_succeed;         /* 1: post propagation is successful       */
                                /* 0: post propagation is not successful   */
  int  natm_freeze;
  int  naatype;

  char **aa3_code;              /*  3 letter amino-acid code, Lth: 20 x 3   */
  char **aa1_code;              /*  1 letter amino-acid code, Lth: 20 x 1   */
  char **aa_label;              /*  amino-acid label code, Lth: 20 x 3 or 4 */

  char **res_label;             /*  res type label, Lth: nres x NCHRMAX     */
  char **atm_label;             /*  atom type label, Lth: natm_tot x NCHRMAX*/
  char **upac_atm_label;        /*  atom type label, Lth: natm_tot x NCHRMAX*/
  char **replace_atm_label;     /*  replace atom type label,                */ 
                                /*                   Lth: natm_tot x NCHRMAX*/
  char **atm_dna_bb_label;      /*  dna backbone atom type label,           */
				/*                          Lth:18 x 3 or 4 */
  char **atm_rna_bb_label;      /*  dna backbone atom type label,           */
				/*                          Lth:19 x 3 or 4 */
  char **atm_ade_base_label;    /*  ADE base atom type label,               */
				/*                          Lth:14 x 3 or 4 */
  char **atm_gua_base_label;    /*  GUA base atom type label,               */
				/*                          Lth:15 x 3 or 4 */
  char **atm_cyt_base_label;    /*  CYT base atom type label,               */
				/*                          Lth:12 x 3 or 4 */
  char **atm_thy_base_label;    /*  THY base atom type label,               */
				/*                          Lth:14 x 3 or 4 */
  char **atm_ura_base_label;    /*  URA base atom type label,               */
				/*                          Lth:11 x 3 or 4 */

  int  *atm_freeze;             /*  Map: atom freezing opt, Lth: natm_tot   */
  int  *atm_res_id;             /*  res ind of host residue, Lth: natm_tot  */

  int  *aa_type;                /*  refers to aa*_code, Lth: natm_tot       */ 
  int  *design_field;           /*  specifies which residues to design      */
                                /*  Lth: 5                                  */
  int  *ind_atm_bb;             /*  indices of backbone atoms, Lth: natm_bb */

  int    nmol;                  /* # of molecules                           */
  int   *nres_mol;              /* # of residues in a molecule, Lth: nmol   */
  int   *npro_mol;              /* # of proline residues in a molecule      */
  int   *res_mol_id;            /*  mol ind of host molecule, Lth: nres     */
  int  **ind_res_mol;           /* residue index, Lth: nmol x mol_nres[imol]*/
  char **mol_label;             /* molecule type label, Lth: nmol x NCHRMAX */
  char **mol_chainid;           /*   molecule chain id, Lth: nmol x NCHRMAX */

  int  *nrseg_mol;              /* # of rigid sgmnts per molecule, Lth: nmol*/
  int **ind_rseg_begin;         /*   residue index, Lth: nmol  x nrseg_imol */
  int **ind_rseg_central;       /*   residue index, Lth: nmol  x nrseg_imol */
  int **ind_rseg_end;           /*   residue index, Lth: nmol  x nrseg_imol */
  int **rseg_length;            /*   segment length, Lth: nmol x nrseg_imol */

  int  *ncseg_mol;              /* # of flxb sgmnts per molecule, Lth: nmol */
  int **ind_cseg_begin;         /*   residue index, Lth: nmol  x ncseg_imol */
  int **ind_cseg_end;           /*   residue index, Lth: nmol  x ncseg_imol */
  int **cseg_length;            /*   segment length, Lth: nmol x ncseg_imol */

  char mol_name[NCHARMAX];

  int  *order_X;                /*  order of Chains : used in mpi           */
 public:
  void setup(Filenames*, General*);
  Maps() {
    nmol = 1;                   /*  currently treating one molecule */      
  }
};//Maps//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Filenames {
 public: 
  char pos_out_filename[NCHARMAX];
  char mol_parm_filename[NCHARMAX];
  char bond_database_filename[NCHARMAX];
  char bend_database_filename[NCHARMAX];
  char tors_database_filename[NCHARMAX];
  char onfo_database_filename[NCHARMAX];
  char inter_database_filename[NCHARMAX];
  char inter_replace_database_filename[NCHARMAX];
  char cryo_em_database_filename[NCHARMAX];
  char cryo_em_expermnt_filename[NCHARMAX];
  char region_database_filename[NCHARMAX];
  char constraint_database_filename[NCHARMAX];
  char pos_init_filename[NCHARMAX];
  char atom_pos_filename[NCHARMAX];
  char tors_pos_filename[NCHARMAX];
  char epot_filename[NCHARMAX];
  char einter_filename[NCHARMAX];
  char design_filename[NCHARMAX];
  char hessian_filename[NCHARMAX];
  char eighess_filename[NCHARMAX];
  char eigvechess_filename[NCHARMAX];
};//Filenames//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Pot_data_entry {
 public:
  int isWild;
  int type;    /*types:INTER(0),BOND(1),BEND(2),ONFO(3),TORS(4),CRYO_EM(5)*/
  int pot_type;           /*For each interaction type's individual type   */
  char atm_lbl1[10], atm_lbl2[10], atm_lbl3[10], atm_lbl4[10], atm_lbl5[10];
  int imp_tors;
  double fk, fk_bend, fk_bond, r0, theta0, fk_tors, eq_tors;
  double eps, sig, beta, scale, awill, bwill, c6m, c8m, c10m;
  double r_min, r_max, r_res;
  double p0, p1, p2, p3, p4, p5, p6;
  double     s1, s2, s3, s4, s5, s6;
  double     sp2, sp3, sp4, sp5, sp6;
  int hash_table;
  int hash_size;
  int numeric_size;
  double *numeric_V_x, *numeric_d2V_x, *numeric_x;
  char *name;

  double pixel_size, resol_blur;                               /* cryo_em */
  double ea_az, ea_alt, ea_phi;                                /* cryo_em */
  double ea_range, ea_interval;                                /* cryo_em */
  double energy_scale;
  char expermnt_filename[NCHARMAX];                            /* cryo_em */ 

  Pot_data_entry *next;
 public:
  void setup(char *);
  char *get_name(void) const;
  bool repeat(const Pot_data_entry &) const;
  bool operator==(const Pot_data_entry &) const;
  int  match(const Maps *, const int *, int) const;
  int  match(char **, const int *, int) const;
  int  match(char *, char *, int) const;
};//Pot_data_entry//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Region_data_entry {
 public:
  int  element_top_type;           /*    1: atom, 2: residue               */
  int  dependency_type;            /*    1: independent, 2: dependent      */    

  int  natom;                      /* number of atoms                      */

  int  nres;                       /* number of residues                   */
  int  nseg;                       /* number of segments                   */
  int  nrespair;                   /* number of paired residues            */
  int  ncenteres;                  /* number of alternating center residues*/
  int *ind_centeres;               /* index of center residue              */
  int *ind_res;                    /* index of residues,     Lth: nres     */
  int *ind_seg_firstres;           /* index of first res in segs, Lth: nseg*/
  int *ind_seg_lastres;            /* index of last res in segs, Lth: nseg */
  int *ind_seg_baseres;            /* index of base res in segs, Lth: nseg */ 
  int *ind_pairedres1;             /* index of pairedres1,   Lth: nrespair */
  int *ind_pairedres2;             /* index of pairedres2,   Lth: nrespair */
  char *name;                      /* name of region                       */
  char **centeres_chainid;   /* center res chain id,Lth:ncenteres x NCHRMAX*/
  char **res_chainid;        /*    residue chain id,Lth: nres     x NCHRMAX*/
  char **seg_firstres_chainid;/* seg first res chain id,Lth: nseg x NCHRMAX*/
  char **seg_lastres_chainid; /* seg last res chain id, Lth: nseg x NCHRMAX*/
  char **seg_baseres_chainid;/* seg base res chain id,  Lth: nseg x NCHRMAX*/
  char **pairedres1_chainid; /*pairedres1 chain id, Lth: nrespair x NCHRMAX*/
  char **pairedres2_chainid; /*pairedres2 chain id, Lth: nrespair x NCHRMAX*/
  double prop_trans_sig;
  double prop_rot_sig;
  double prop_trans_sig_freeres;                        
  double prop_rot_sig_freeres;                          
  double prop_trans_sig_respair;                        
  double prop_rot_sig_respair;                          
  double prop_trans_sig_respair_internal;
  double prop_rot_sig_respair_internal;                          

  Region_data_entry *next;
 public:
  void setup(char *, General *);
};//Region_data_entry//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Constraint_data_entry {
 public:
 
  int    atom1, atom2; 
  double fk, r0;

  Constraint_data_entry *next;
 public:
  void setup(char *, General *);
};//Constraint_data_entry//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class General {
 public:
  int system_definition;                  /* 1: primitive, 2: residue      */
  int allatm_model;                       /* 1: all atom residues          */
  int cgres_model;                        /* 1: coarse grained residues    */
  int cgres_model_KB;                     /* 1: KB model                   */ 
                                          /*    can be 1pt,3pt,5pt,UA,UAPH */
  int cgres_model_1pt;                    /* 1: 1pt model,(KB,physical:Go.)*/
  int cgres_model_3pt;                    /* 1: 3pt model (KB,physical)    */
  int cgres_model_5pt;                    /* 1: 5pt model (KB,physical)    */
  int cgres_model_UA;                     /* 1: UA model (KB,physical)     */
  int cgres_model_UAPH;                   /* 1: UA model with Polar Hyd.   */
  int cgres_model_KB_UA;                  /* 1: UA coarse KB residue       */
  int cgres_model_KB_UAPH;                /* 1: UAPH coarse KB residue     */
  int cgres_model_KB_5pt;                 /* 1: 5pt coarse KB residue      */
  int cgres_model_KB_3pt;                 /* 1: 3pt coarse KB residue      */
  int cgres_model_KB_1pt;                 /* 1: 1pt coarse KB residue      */
  int cgres_model_Go;                     /* 1: Go-type model, only CA     */
  int cgres_model_BNL;                    /* 1: BNL model, only CA         */

  int stride_defined;                     /* 1: STRIDE defined in PDB      */
  int torcompl_defined;                   /* 1: TORCOMPL defined in PDB    */
  int closcompl_defined;                  /* 1: CLOSCOMPL defined in PDB   */
  int cblc_defined;                       /* 1: CBLC defined in PDB        */
  int clos_defined;                       /* 1: if any clos is defined     */
                                          /*    e.g. CBLC                  */
  int region_topology_defined;            /* 1:  region topology defined   */
  int region_database_defined;            /* 1:  region_database_file defnd*/
  int cryo_em_database_defined;           /* 1: cryo_em_database_file defnd*/
  int cryo_em_expermnt_defined;           /* 1: cryo_em_expermnt_file defnd*/
  int constraint_topology_defined;        /* 1: constraint_topology defined*/
  int constraint_database_defined;        /* 1: constraint_database defined*/

  int write_energy_unit;                  /* 1: kcal/mol, 0: Ha            */
  int write_hessian;                      /* 1: write , 0: silent          */
  int write_eighess;                      /* 1: write , 0: silent          */
  int write_eigvechess;                   /* 1: write , 0: silent          */

  int action_type;                        /* highest level action_type     */
                                          /* 1: PT,NM(EN),NMPT,DBFR(EN),MIN*/
                                          /* 2: EEMC, EEMC_PT */
                                          /* 3: SEQ_PT,       */
                                          /* 4: SEQ_EEMC,     */

  int pt;                                 /* if 1 : Parallel Tempering MC  */
  int ptemp;                              /* if 1 : Parallel Tempering on  */
  int ptrepswa;                           /* if 1 : Parallel Tempering     */
                                          /*        Repswa MC              */
  int repswa;                             /* if 1 : Repswa transformation  */
                                          /*        is present             */
  int ptstage;                            /* if 1 : Parallel Tempering     */
                                          /*        Staging MC             */
  int stage;                              /* if 1 : Staging transformation */
                                          /*        is present             */
  int nmode;                              /* if 1 : Normal Mode Calc. (NM) */
  int nmode_enm;                          /* if 1 : Normal Mode ENM (NMEN) */
  int nmode_stg;                          /* if 1 : Normal Mode STG (NMSTG)*/
  int nmpt;                               /* if 1 : Normal Mode PT (NMPT)  */
  int debug_force;                        /* if 1 : Debug Force (DBFR)     */
  int debug_force_enm;                    /* if 1 : DBFR on ENM (DBFREN)   */
  int debug_force_stage;                  /* if 1 : DBFR on STG (DBFRSTG)  */
  int debug_tors_defs;                    /* if 1 : Debug tors defs (DBTD) */
  int md;                                 /* if 1 : MD (Molecular Dynamics)*/
  int minimize;                           /* if 1 : Minimization (MIN)     */

  int postprop_minimize;                  /* if 1 : every propagation      */
                                          /* is followed by minimization   */
                                          /* e.g: Monte Carlo Minimization */

  int postprop_minimize_type;             /* if 1 : cartesian              */
                                          /*    2 : torsional              */
                                          /*    3 : bend                   */
                                          /*    4 : bend + torsional       */
                                          /*    5 : torsional + bend       */
                                          /*    6 : closure                */
                                          /*    7 : gradient closure       */
                                          /*        gradient:              */
                                          /*        bend + torsional       */
                                          /*    8 : sidechain              */
                                          /*        only tors              */


  int postprop_minimize_energy;           /* if 1 : bond                   */
                                          /*    2 : bond+bend              */
                                          /*    3 : bond+bend+torsion      */
                                          /*    4 : bond+bend+torsion+onfo */
                                          /*    5 : all (4+inter)          */


  int postprop_minimize_itmax;            /* maximum number of postprop    */
                                          /* minimization steps/iterations */

  int postprop_minimize_gradclos_itmax;   /* maximum number of postprop    */
                                          /* (gradient_min-closure) super- */
                                          /* steps, the number of steps in */
                                          /* gradient_min is controlled by */
                                          /* the previous option           */


  int    minimize_type;                   /* 1: cg, 2: bfgs, 3: samc       */
  double minimize_tol;                    /*minimization gradient tolerance*/

  int    minimize_report;                 /* 0: none, 1: screen, 2: scr+out*/
  int    energy_report;                   /* 0: none, 1: major,  2:detailed*/

  int enm;                                /*    1 : If any enm is on       */
  int debug;                              /*    1 : If any debug is on     */
  int prop_type;                          /* 1: cartesian, 2: torsional    */
                                          /* 3: bend                       */
  int eemc_type;                          /* 1: eemc, 2: eemc_pt           */
  int lnk_list;                           /* 1: linked_list inter          */
  int neighbor_list;                      /* 1: neighbor list inter        */
  int num_procs;                          /* # of processors to be used    */
  int control_design;
  int nres, nrtfres;                      
  int natom,natm_tot,mdisk;
  int n_chains_mc;
  int nstep_md,nstep_mc,nstat;
  int intra_respa_step;                   /* # of respa steps for intra-   */
                                          /* molecular interaction         */

  double dt,pee,T,text; 

  double sa_temp_first;                 /* initial temperature in simulated*/
                                        /* annealing                       */
  double sa_temp_last;                  /* final temperature in simulated  */
                                        /* annealing                       */
  double sa_rate;                       /* rate of change in simulated     */
                                        /* annealing                       */
  int    stsamc_type;                   /* 1: trigonimetric, 2: linear     */
  int    stsamc_period;                 /* stsamc period in # of steps     */
  double stsamc_ampl;                   /* stsamc amplitude in K           */
  double stsamc_shift;                  /* stsamc shift in K               */  

  double prop_clos_sig;                   /*  closure (type=5) mc proposal */
                                          /*                        width  */
  double prop_tors_sig;                   /*  torsional mc proposal width  */
  double prop_rot_sig;                    /*  rotational ..proposal width  */
  double prop_trans_sig;                  /*  translation .proposal width  */
  double prop_notors_sig_scale;           /*  non torsional sigma scaling  */
                                          /*  of all proposal widths       */
  int prop_tors_type;                     /*  1: full, 2: side_chain_only  */
  int prop_regions_type;                  /*  1: superimpose, 2: onebyone  */
  int prop_closcompl_type;                /*  1: translation, 2:rotation   */
                                          /*  3: translation + rotation    */
  int prop_DNA_closcompl_type;            /*  1: intralevel, 2: interlevel */
                                          /*  3: intralevel + interlevel   */
  int deg_free;
  int B,N;                                     /* burn in periods          */
  int inter_on_3bond_conn, inter_on_4bond_conn;/* turn on 3-4bnd inter     */ 
  int cancel_res_inter_local;     /* 1: cancel inter within the residue    */
  int cancel_res_inter_neighbor;  /* 1: ... between neighboring residues   */
  int inter_replace;              /* 1: replace some intermolecular inters */
  int inter_replace_KB_UA;        /* 1: KB_UA is the replacing potential   */
  int inter_replace_KB_5pt;       /* 1: KB_5pt is the replacing potential  */
  int inter_nprw;                 /* adds non pairwise inter potential     */
                                  /* 0: no addition                        */
                                  /* 1: ercolessi-adams                    */
  int cancel_res_inter_replace_local; /* 1: cancel inter replace within res*/
  int cancel_res_inter_replace_neighbor;/* 1:   ... between neighboring res*/
  int neutralize;                 /* 0: no effect, 1: neutralize nucl acids*/
                                  /* 2: neutralize proteins, 3: neutralize */
                                  /*    both nucleic acids and proteins    */

  int scale_mass;                 /* 0:none, 1: all, 2: backbone only      */
  double scale_mass_value;        /* scale mass value                      */
  double scale_mass_value_H;      /* scale mass value for H in the system  */

  int    force_const_cutoff;      /* 0: off, 1: bond, 2: bond_bend         */
  double force_const_cutoff_bond; /* max force const cutoff for bonds      */
  double force_const_cutoff_bend; /* max force const cutoff for bends      */

  int energy_term_bond;           /* 1/0:         bond energy is on/off    */
  int energy_term_bend;           /* 1/0:         bend energy is on/off    */
  int energy_term_tors;           /* 1/0:         tors energy is on/off    */
  int energy_term_onfo;           /* 1/0:         onfo energy is on/off    */
  int energy_term_inter;          /* 1/0: inter atomic energy is on/off    */
  int energy_term_cryo_em;        /* 1/0: cryo_em      energy is on/off    */
  int energy_term_constraint;     /* 1/0: constraint   energy is on/off    */

  long seed;                                   /* random seed              */ 
  int iseed[1], iseed2[1];
  double qseed[1];
  int istep;

  General() {
    seed     = -2039480938l;
    iseed[0] = 19541;
    iseed2[0]= 92571;
    qseed[0] = (double)19541;
    stride_defined = 0;
    istep    = 0;
  }
};//General//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Energy_part {
 public:
  Energy_part() {}
  void setup(General&);
  void setup_silent(General&);
  void partition(General&);
  void partition_silent(General&);
 public:
  double H_lower,H_upper;   /* energy bounds      */
  double energy_gap;        /* E_i = (E_gap)^i    */   
  double *H_ladder;         /* energy ladder      */
  double *T;                /* temperature ladder */
};//Energy_part//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Bond {
 public:
  int energy_term;                   /* 1/0: bond energy is on/off        */
  int nbond, nbond_enm;
  int natm_tot, nbond_atm_max;
  int *atom1, *atom2, *atom1_enm, *atom2_enm;
  int *nbond_atm, **bond_atm_map;
  double *fk, *r0, *fk_enm, *r0_enm;
 public:
  Bond();
 ~Bond();
  void setup(const Pot_data_entry *, const Maps *, const char *);
  void setup(const Pot_data_entry *, const Maps *, const Pos *);
  void setup_enm(const General *, const Maps *, const Pos *);
  double energy(const Pos *);
  double getPot(const Pos *);
  double energy_enm(const Pos *);
  double getPot_enm(const Pos *);
};//Bond//
/*-------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------*/
class Bend{
 public:
  int energy_term;                   /* 1/0: bend energy is on/off        */
  int nbend;
  int *atom1, *atom2, *atom3;
  double *fk_bend, *theta0;
  double *fk_bond, *r0;
  double vbond;                      /*   1-3 bond potential contribution */
  double vbend;                      /* 1-2-3 bend potential contribution */
 public:
  Bend();
 ~Bend();
  void setup(const Pot_data_entry *, const Maps *, const char *);
  void setup(const Pot_data_entry *, const Maps *, const Pos *);
  double energy(const Pos *);
  double getPot(const Pos *);
};//Bend//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Tors {
 public:
  int energy_term;                   /* 1/0: tors energy is on/off        */
  int ntors;
  int nimproper;
  int *atom1, *atom2, *atom3, *atom4, *type, *improper;
  double *s0;
  double *p0, *p1, *p2, *p3, *p4, *p5, *p6; /*pi: cfs.of cos^i            */
  double      *s1, *s2, *s3, *s4, *s5, *s6; /*si: cfs.of cos^{1-d(i,1)}   */
                                            /*          *sin^{i-(1-d(i,1)}*/
  double      *sp2, *sp3, *sp4, *sp5, *sp6; /*pi: cfs.of sin^i            */

  double *eq;
  double vtors;                       /* torsional potential contribution */
  double vimpr;                       /*  improper potential contribution */
 public:
  Tors();
 ~Tors();
  void setup(const Pot_data_entry *, const Maps *, const char *);
  void setup(const Pot_data_entry *, const Maps *, const Pos *);
  double energy(const Pos *);
  double getPot(const Pos *);
  double getTorsPot(int, double);
};//Tors//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Onfo {
 public:
  int energy_term;                   /* 1/0: onfo energy is on/off         */
  int nonfo;
  int *atom1, *atom2, *type;
  int implicit_solvent;              /*    1: implicit solvent presence    */
  int implicit_solvent_scp;          /*    1: scp implicit solvent         */
  int ddd;                           /*    1: DD0S type distance dependent */ 
                                     /*       dielectric                   */
                                     /*    2,3..: to be implemented        */
                                     /*    0: off                          */
  double ddd_p1,ddd_p2, ddd_p3;      /*  parameters for all ddd type       */
  double ddd_D, ddd_D0, ddd_S;       /*  parameters for ddd type 1 (DD0S)  */
  double ddd_c, ddd_e;               /*  parameters for ddd type 2 (ce)    */
  double *eps, *feps, *sig, *beta, *scale, *r0;
  double *awill, *bwill, *c6m, *c8m, *c10m;
  double *r_min, *r_max, *r_res;
  double vlj;                     /* lennard-jones potential contribution */
  double vcoul;                   /*       coulomb potential contribution */ 
 public:
  Onfo();
 ~Onfo();
  void setup(Pot_data_entry *, Maps *, char *);
  void setup(Pot_data_entry *, Maps *, Pos *);
  double energy(Pos *);
  double getPot(Pos *);
  double getOnfoPot(Pos *, int, double);
};//Onfo//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Inter {
 public:
  int energy_term;                   /* 1/0: inter energy is on/off        */
  int natom;                         /* number of atoms                    */
  int ninter;                        /* number of inter interactions       */
  int ninter_neighbors;              /* number of neighbor inter interact. */
  int ntype;                         /* number of interaction types        */
  int nexclude;                      /* number of exclusions               */
  int perd;                          /* system periodicity                 */
  int *atom1, *atom2;                /* index of atm1&2, Lth: ninter       */
  int *exclude_atom1, *exclude_atom2;/* index of exclusions, Lth: nexclude */ 
  int *type;                         /* interaction type, Lth: ntype       */
  int *atm1_type, *atm2_type;        /* atm type of i.t., Lth: ntype       */
  int *numeric_size;                 /* size of the num. grid, Lth: ntype  */
  int *inter_off;                    /* inter switch, Lth: ninter          */
  int *type_index;                   /* lookup index for inter, Lth: ninter*/
  double *scale_iinter;              /* scale interacrions, Lth: ninter    */
  double *eps, *feps, *sig, *beta, *scale, *r0; /* inter parms, Lth: ntype */
  double *awill, *bwill, *c6m, *c8m, *c10m;     /* inter parms, Lth: ntype */
  double *r_min, *r_max, *r_res;                /* inter parms, Lth: ntype */
  double *r_min2, *r_max2;                      /* inter parms, Lth: ntype */
  double **numeric_V_x;              /* numpot., Lth: ntype x numeric_size */
  double **numeric_d2V_x;            /* numpot., Lth: ntype x numeric_size */
  double **numeric_x;                /* numpot., Lth: ntype x numeric_size */

  int  hash_all;                     /* 1: hashing all interactions        */ 
  int *hash_table;                   /* 0: no hashing           Lth: ntype */
                                     /* 1: hashing inter in x,             */
                                     /* 2: hashing inter in x^2            */
  int *hash_size;                    /*    size of hash table,  Lth: ntype */
  double *hash_delta;                /*       hash grid width,  Lth: ntype */
  double **hash_numeric_V_x;         /* V(x),       Lth: ntype x hash_size */
  double **hash_numeric_dV_x;        /* dV/dx,      Lth: ntype x hash_size */
  double **hash_numeric_pre_x;       /* pre(x),     Lth: ntype x hash_size */
                                     /*     prefactor for force evaluation */

  int nprw;                          /* adds non pairwise inter potential  */
                                     /* 0: no addition                     */
                                     /* 1: ercolessi-adams                 */
  int nfunc_nprw;                    /* number of non pairwise interaction */
                                     /* functions: the non pairwise inter- */
				     /* actions are based on them          */
  int max_nfunc_nprw_type;           /* max{nfunc_nprw_type[itype]}        */
  int *nfunc_nprw_type;              /* number of functions assigned to    */
                                     /* each pairwise type, Lth: ntype     */
  int **ind_func_nprw_type;          /* Lth: nfunc_nprw_type[itype] x ntype*/
                                     /* global indices of func_nprw        */
  double *nprw_r_min, *nprw_r_max;   /* inter parms, Lth: nfunc_nprw       */
  double *nprw_r_min2, *nprw_r_max2; /* inter parms, Lth: nfunc_nprw       */ 
  int  *nprw_numeric_size;           /* size of the grid, Lth: nfunc_nprw  */
  double **nprw_numeric_V_x;         /* Lth: nfunc_nprw x numeric_nprw_size*/
  double **nprw_numeric_d2V_x;       /* Lth: nfunc_nprw x numeric_nprw_size*/
  double **nprw_numeric_x;           /* Lth: nfunc_nprw x numeric_nprw_size*/

  int  hash_nprw_all;                /* 1: hashing all nprw interactions   */ 
  int *hash_nprw_table;              /* 0: no hashing    Lth:  nfunc_nprw  */
                                     /* 1: hashing inter in x,             */
                                     /* 2: hashing inter in x^2            */
  int *hash_nprw_size;               /* size of hash table,Lth: nfunc_nprw */
  double *hash_nprw_delta;           /* hash grid width,Lth:    nfunc_nprw */
  double **hash_nprw_numeric_V_x;  /* V(x), Lth:nfunc_nprw x hash_nprw_size*/
  double **hash_nprw_numeric_dV_x; /* dV/dx,Lth:nfunc_nprw x hash_nprw_size*/
  double **hash_nprw_numeric_pre_x;/*pre(x),Lth:nfunc_nprw x hash_nprw_size*/
                                     /*  ---- Added by Sam Nakagawa ----   */
  double *sumRho;                    /*     sum of rho's for each atom     */
  double **TX;                       /*     dU/drho for all  interactions  */
  double **TY;                       /*     Tx, Ty, and Tz for the x-, y-  */
  double **TZ;                       /*     and z-directions               */
  double *array_uix;                 /*                                    */ 
  double *array_uiy;                 /*                                    */ 
  double *array_uiz;                 /*                                    */ 
  double *array_ujx;                 /*                                    */ 
  double *array_ujy;                 /*                                    */ 
  double *array_ujz;                 /*                                    */ 
                                     /*  ----    End of Addition    ----   */


  int implicit_solvent;              /*    1: implicit solvent presence    */
  int implicit_solvent_scp;          /*    1: scp implicit solvent         */
  int ddd;                           /*    1: DD0S type distance dependent */ 
                                     /*       dielectric                   */
                                     /*    2,3..: to be implemented        */
                                     /*    0: off                          */
  double ddd_p1,ddd_p2, ddd_p3;      /*  parameters for all ddd type       */
  double ddd_D, ddd_D0, ddd_S;       /*  parameters for ddd type 1 (DD0S)  */
  double ddd_c, ddd_e;               /*  parameters for ddd type 2 (ce)    */

  int lnk_list;                      /* 1: lnk_list option is turned on    */
  int ncell_tot;                     /*  total number of cells in lnk_list */
  int excl_num_mall;                 /*  allocation par for excl_list      */
  int **inter_index;  /*i_i[i][j]:the ind of j>i inter, Lth: natom x natom */
  int *excl_num;      /*      # of i exclusions for atom j: i<j, Lth:natom */
  int **excl_list;    /*    list of exclusions, Lth: excl_num_mall x natom */
  double rcut_ll;     /*cutoff for linked list <- some average from *r_max */  
  double lnk_list_skin;  /*               linked list skin                 */ 
  double *lbox_ll;    /*   box length for linked list, Lth: 1 x 3 -> x,y,z */
  double *rcell_ll;   /*    cell size for linked list, Lth: 1 x 3 -> x,y,z */
  int    *ncell_ll;   /*# of cells in xyz linked list, Lth: 1 x 3 -> x,y,z */
  int    *vcell_ll;   /*   cell index for linked list, Lth: 1 x 3 -> x,y,z */
  int    *vcell1_ll;  /*   cell index for linked list, Lth: 1 x 3 -> x,y,z */
  int    *lscl_ll;    /*atom index to which the ith atom points, Lth: natom*/
  int    *head_ll;    /* the first atm in each cell/-1 if empty: Lth: ncell*/

  int neighbor_list;                 /* 1: neighbor_list option is turned  */
  int neighbor_list_update;          /* 1: neighbor list needs update      */
  int *neighbors;                    /* 1: atoms are neighbors, Lth: ninter*/
  double neighbor_list_skin;         /*    neighbor list skin              */
  double neighbor_list_cutoff;       /*    neighbor list cutoff            */
  double neighbor_list_cutoff2;      /*    neighbor list cutoff^2          */

  int    rexclude_length_defined;       /* defined flag for rexclude_length*/
  double rexclude_length;               /* initial length to exclude inter */
  double rswitch_length;                /* the length of real inter switch */
  double ecoul;                /* coulomb contribution to the inter energy */
  double ekb;          /* knowledge based contribution to the inter energy */ 
  double elj;             /* lennard-jone contribution to the inter energy */ 
  double escp;              /* scp energy contribution to the inter energy */ 
  double einter;            /* all energy contribution to the inter energy */ 
 public:
  Inter();
 ~Inter();
  void setup(const Pot_data_entry *, const Maps *, const System *);
  void setup(const Pot_data_entry *, const Maps *, const Pos *);
  void nonpair_setup(const Pot_data_entry *, const Maps *, const Pos *);
  void replace_setup(const Pot_data_entry *,const Maps *, const System *);
  void neighbor_list_check(const Pos *);
  void conf_save(const Pos *);
  double energy_atomic(const Pos *);
  double energy_atomic_nonpair(const Pos *);
  double energy_atomic_hash(const Pos *);
  double energy_atomic_nonpair_hash(const Pos *);
  double getPot_atomic(const Pos *);
  double getPot_atomic_nonpair(const Pos *);
  double getPot_atomic_hash(const Pos *);
  double getPot_atomic_nonpair_hash(const Pos *);
  double getPotPermute_atomic(const Pos *);
  double getInterPot_atomic(const Pos *, int);
  double energy_scp(const Pos *);                  
  double getPot_scp(const Pos *);
  double getPotPermute_scp(const Pos *);
  double getInterPot_scp(const Pos *, int);
  double energy(const Pos *);
  double getPot(const Pos *);
  double getPotPermute(const Pos *);
  double getInterPot(const Pos *, int);
 
 private:
  /* ---- Added by Sam Nakagawa ---- */
  int _cache_phi;
  int _cache_rho;
  int _cache_u;
  void _compute_pairwise_force(const double hash_r, 
                               const int    itype,
                               double&      vinter, 
                               const int    atom1_ind, 
                               const int    atom2_ind, 
                               const int    iinter,
                               const double dx, 
                               const double dy, 
                               const double dz,  
                               double*&     fx, 
                               double*&     fy, 
                               double*&     fz);
  void _compute_pairwise_pot(const double hash_r, 
                             const int    itype,
                             double&      vinter, 
                             const int    atom1_ind, 
                             const int    atom2_ind, 
                             const int    iinter); 
  void _populate_density(const int    ifunc_nprw,
                         const double r,
                         const int    atom1_ind,
                         const int    atom2_ind,
                         const double uConversion);    
  void _populate_density_hash(const int    ifunc_nprw, 
                              const double hash_r,
                              const int    atom1_ind,  
                              const int    atom2_ind,
                              const double uConversion);
  void _populate_dvdensity(const int    ifunc_nprw,
                           const double r,
                           const double x,
                           const double y,
                           const double z,
                           const int    atom1_ind,
                           const int    atom2_ind,
                           const double uConversion);  
  void _populate_dvdensity_hash(const int    ifunc_nprw, 
                                const double hash_r,
                                const double x, 
                                const double y,
                                const double z, 
                                const int    atom1_ind,
                                const int    atom2_ind, 
                                const double uConversion);
  double _get_sum_of_glue();
  double _get_sum_of_glue_hash();
  void   _add_force_from_glue(const Pos *pos); 
  double _add_force_from_glue_hash(double*& fx, 
                                   double*& fy, 
                                   double*& fz);   

  /* ---- End of Addition ---- */
};//Inter//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Cryo_em {
 public:
  int energy_term;                   /*      1/0: cryo_em energy is on/off */
  int norient;
  int ncalpha;
  int *ind_calpha;
  double *pixel_size, *resol_blur;                         /* Lth: norient */
  double *ea_az, *ea_alt, *ea_phi;                         /* Lth: norient */
  double *ea_range, *ea_interval;                          /* Lth: norient */
  double *energy_scale;                                    /* Lth: norient */
  char **expermnt_filename;                    /*   Lth: norient x NCHARMAX*/
#ifdef CRYO_EM
  PointArray *eman2_pa;                        /*       EMAN2 point array  */
  Sim_Score_Matrix ssm;                        /*       Sim_Score_Matrix   */
#endif
 public:
  Cryo_em();
 ~Cryo_em();
  void setup(const Pot_data_entry *, const System *);
  double energy(const Pos *);
  double getPot(const Pos *);
};//Cryo_em//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Constraint {
 public:
  int energy_term;                   /* 1/0: bond energy is on/off        */
  int nbond;
  int natm_tot, nbond_atm_max;
  int *atom1, *atom2;
  int *nbond_atm, **bond_atm_map;
  double *fk, *r0;
  double vbond;
 public:
  Constraint();
 ~Constraint();
  void setup(const Constraint_data_entry *, const Maps *, const Pos *);
  double energy(const Pos *);
  double getPot(const Pos *);
};//Constraint//
/*-------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------*/
class Rotate_utils {
   public:
     double *a,*b;                                    /* scratch variables */ 
     double *r1,*r2,*r3,*r4,*r4_;                     /* scratch variables */ 
     double *f1,*f2,*f3,*f4;                          /* scratch variables */ 
     double **R,**RT,**I,**A,**B,**C;                 /* scratch variables */ 
     double *vec1scr_3x1,*vec2scr_3x1;                /* scratch variables */
     double **mat1scr_3x3,**mat2scr_3x3;              /* scratch variables */ 

     double **dr4_dr1,**dr4_dr2,**dr4_dr3;            /* dr4_dri matrices  */ 
     double *ex,*ey,*ez;                              /* reference frame   */
     double *e0,*e1,*e2,*e3;                          /* e0,e1,e2,e3       */
     double *e1_r,*e2_r,*e3_r;                        /* e1_r, e2_r, e3_r  */
     double *e1_Ar,*e2_Ar,*e3_Ar;                     /* e1_Ar,e2_Ar,e3_Ar */
     double **de1_dr2,**de1_dr3,**de2_dr1,**de2_dr2;  /* e1, e2 dependence */ 
     double **dey_de1,**dey_de2,**dex_dey,**dex_dez;  /* e1, e2, basis dep.*/ 

     double **dex_dr1,**dex_dr2,**dex_dr3;    /*  basis dep. on (r1,r2,r3) */ 
     double **dey_dr1,**dey_dr2,**dey_dr3;    /*  basis dep. on (r1,r2,r3) */ 
     double **dez_dr1,**dez_dr2,**dez_dr3;    /*  basis dep. on (r1,r2,r3) */
     double **dR_dr1x,**dR_dr1y,**dR_dr1z;    /*  dR/dr1 tensor in matrices*/
     double **dR_dr2x,**dR_dr2y,**dR_dr2z;    /*  dR/dr2 tensor in matrices*/ 
     double **dR_dr3x,**dR_dr3y,**dR_dr3z;    /*  dR/dr3 tensor in matrices*/ 

     double **dRT_dr1x,**dRT_dr1y,**dRT_dr1z; /* dRT/dr1 tensor in matrices*/ 
     double **dRT_dr2x,**dRT_dr2y,**dRT_dr2z; /* dRT/dr2 tensor in matrices*/ 
     double **dRT_dr3x,**dRT_dr3y,**dRT_dr3z; /* dRT/dr3 tensor in matrices*/
   public:
     void setup();    
};//Rotate_utils//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Stage_defs {
 public:
  int num_trans;                    /* num : Number of transformations     */
  int natm_trans_max;               /* num : Max # of atms in a transform  */
  int *res_id;                      /* Lth : num_trans,each grp is in a res*/
  int *ityp_trans;                  /* Lth : num_trans                     */
			            /* type of transformation              */
                                    /*     : 0 identical                   */
                                    /*     : 1 closed (circular)           */
                                    /*     : 2 open                        */ 
  int *natm_trans;                  /* Lth : num_trans                     */
                                    /*       Number of atms in each trans  */
                                    /*     Eg. for trans 1                 */
                                    /*         natm_trans[1] = 3           */
  int **ind_trans;                  /* Lth : num_trans x natm_trans_max    */
				    /* Ind. of atoms in each trans         */
                                    /*     Eg. for trans 1                 */
                                    /*         ind_trans[1][1] = 1         */
                                    /*         ind_trans[1][2] = 3         */
                                    /*         ind_trans[1][3] = 15        */
 public:
  Stage_defs();
 ~Stage_defs();
  void setup(const Maps *, const Pos *, const General *); 
  void setup_model_AA(const Maps *, const Pos *, const General *); 
  void setup_AA_protein(const Maps *,const Pos *,const General *,int,int*);
  void setup_AA_DNA(const Maps *,const Pos *,const General *,int,int*);
  void write_alldata(const Maps *maps);
};//Stage_defs//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Tors_defs {
 public:
  int num_trans;                    /* num : Number of transformations     */
  int num_tbend;                    /* num : Number of theta bends         */
  int natm_trans_max;               /* num : Max of atms in any grp        */
  int nsec_tors_max;                /* num : Max # of sec torsions         */
  int nstride;                      /* # of STRIDE defined in PDB          */

  int ntorcompl;                    /* # of TORCOMPL defined in PDB        */
  int ncloscompl;                   /* # of CLOSCOMPL defined in PDB       */
  int max_nmol_torcompl;            /* max # of molecules in any torcomplex*/

  int num_clos;                     /* # of closures need to be healed     */ 
  int num_clbase;                   /* # of closure bases to build on      */
  int nmol_cblc;                    /* # of molecules treated by CBLC clos */

  int nmol_free;                    /* # of molecules  not in TORCOMPL/CBLC*/
  int num_clbase_free;              /* # of closure bases not present in   */
                                    /*   any CLOSCOMPL                     */
  int nmol_notors;                  /* # of molecules with no torsional    */
                                    /*   degrees of freedom, molecules that*/
                                    /*   can only be propagated by transl. */
                                    /*   and/or rotation                   */
                                    /*  e.g. "water" or "atom/ion"         */ 

  int *res_id;                      /* Lth : num_trans, each grp is in a res*/
  int *ityp_trans;                  /* Lth : num_trans                     */
			            /* type of transformation              */
                                    /*     : 0 identical                   */
                                    /*     : 1 prop (Gaussian kernel)      */
                                    /*     : 2 prot (.. ) and closure      */ 
                                    /*       ind_clos_trans[itrans]        */
                                    /*       gives the index for transform */
                                    /*       healed by closure             */
                                    /*--------------REPSWA-----------------*/
                                    /*     : 3 intramolecular-static       */
                                    /*     : 4 intramolecular-dyn          */
                                    /*     : 5 intermolecular-static-intra */
                                    /*     : 6 intermolecular-dynamic-intra*/
                                    /*-------------------------------------*/
  int *secondary_only;              /* Lth : num_trans                     */
                                    /*   only secondary 'dihedral' members */
                                    /*   of the group are transformed      */
                                    /*   0 : everybody is tranformed       */
                                    /*                (not activated)      */
                                    /*   k : activated and k indicates the */
                                    /*       index of the primary dihedral */
  int *off_trans;                   /* turns off transformation, equivalant*/
                                    /* to identical, no change in torsion  */
                                    /* this option is usually used for     */
                                    /* support STRIDE                      */
  int *natm_trans;                  /* Lth : num_trans                     */
                                    /*       Number of atms in each trans  */
                                    /*     Eg. for trans 1                 */
                                    /*         natm_trans[1] = 3           */
  int *ntbend_trans;                /* Lth : num_trans                     */
                                    /*       Number of tbends in each trans*/
                                    /*     Eg. for trans 1                 */
                                    /*         ntbend_trans[1] = 3         */
  int **ind_trans;                  /* Lth : num_trans x natm_trans_max    */
				    /* Ind. of atoms in each trans         */
                                    /*     Eg. for trans 1                 */
                                    /*         ind_trans[1][1] = 1         */
                                    /*         ind_trans[1][2] = 3         */
                                    /*         ind_trans[1][3] = 15        */
  int **ind_tbend_trans;            /* Lth : num_trans x natm_trans_max    */
                                    /* Ind of tbend in each trans          */
                                    /*     Eg. for trans 4                 */
                                    /*         ind_tbend[4][1] = 11        */
                                    /*         ind_tbend[4][2] = 12        */
                                    /*         ind_tbend[4][3] = 13        */
  int **ind_prim_tors;              /* Lth :  num_trans x 4                */
				    /*  Torsion angle to be transformed    */
                                    /*     Eg. for trans 1                 */
                                    /*         prim_tors_def[1][1] = 11    */
                                    /*         prim_tors_def[1][2] = 9     */
                                    /*         prim_tors_def[1][3] = 5     */
                                    /*         prim_tors_def[1][4] = 1     */
                                    /*    where atom 1 is the group and    */
                                    /*    5,9,11 are not                   */
  int *ityp_prim_tors;              /* Lth : num_trans : pot type tors     */
  int *ityp_prim_onfo;              /* Lth : num_trans : pot type onfo     */
  int *nsec_tors;                   /* Lth : num_trans                     */ 
                                    /*     # sec tors angles in each tran  */
  int ***ind_sec_tors;              /* Lth : num_trans x nsec_tors_max x 4 */
                                    /*       ind_sec_tors[1][1][1] = 3     */
                                    /*       ind_sec_tors[1][1][2] = 9     */
                                    /*       ind_sec_tors[1][1][3] = 5     */
                                    /*       ind_sec_tors[1][1][4] = 1     */
  int **ityp_sec_tors;              /* Lth : num_trans x nsec_tors_max     */
  int **ityp_sec_onfo;              /*       pot type of sec tors/onfo     */
  double *prim_upper_lim;           /* Lth : num_trans (upper limit)       */
  double *prim_lower_lim;           /* Lth : num_trans (lower limit)       */
  double *prop_sig_scale;           /* Lth : num_trans                     */
  char **stride;                    /* Lth : NSTRIDEMAX x NRESMAX          */
  int   *ind_stride_mol;            /* Lth : nstride, 0 or the stride index*/
  char **torcompl;                  /* Lth : NTORCOMPLMAX x NMOLMAX        */
  char **torcompl_label;            /* Lth : ntorcompl x NCHRMAX           */
  int  *ityp_torcompl;              /* Lth : ntorcompl,  1 : DNA_duplex    */
                                    /*                   2 : protein_D6    */
  int  *nmol_torcompl;              /* Lth : ntorcompl : nmol in a torcompl*/
  int **ind_mol_torcompl;           /* Lth : ntorcompl x max_nmol_torcompl */
                                    /*  indices of mols in tors complexes  */
                                    /*  E.g. for first torsion complex     */
                                    /*       with three molecules (2,3,4)  */
                                    /*       ind_mol_torcompl[1][1] = 2    */
                                    /*       ind_mol_torcompl[1][2] = 3    */
                                    /*       ind_mol_torcompl[1][3] = 4    */
  char **closcompl;                 /* Lth : NCLOSCOMPLMAX x NMOLMAX       */
  char **closcompl_label;           /* Lth : ncloscompl x NCHRMAX          */
  int  *nmol_closcompl;             /* Lth : ncloscompl:nmol in a closcompl*/
  int **ind_mol_closcompl;          /* Lth : ncloscompl x max_nmol_closcompl*/
                                    /*  indices of mols in clos complexes  */
                                    /*  E.g. for first clos complex        */
                                    /*       with three molecules (2,3,4)  */
                                    /*       ind_mol_closcompl[1][1] = 2   */
                                    /*       ind_mol_closcompl[1][2] = 3   */
                                    /*       ind_mol_closcompl[1][3] = 4   */
  int   *nlevel_closcompl;          /* Lth : ncloscompl:nlevel in closcompl*/
  int ***ind_clbase_level_closcompl;/* Lth :    ncloscompl                 */
                                    /*        x max_nlevel_closcompl       */
                                    /*        x MAXCLBPL                   */
  int  *nclbase_mol;                /* Lth : nmol, 0 if no clbase present  */
  int **ind_clbase_mol;             /* Lth : nmol x num_clbase             */
  int  *ind_clbase_free;            /* Lth : num_clbase_free               */
                                    /*       indices of clbase not present */
                                    /*       in any CLOSCOMPL              */
  int  *ind_mol_notors;             /* Lth : nmol_notors                   */
                                    /*       index of molecules with no    */ 
                                    /*       torsional degrees of freedom  */
  int  *ind_clos_trans;             /* Lth : num_trans                     */
                                    /*       the index (1<.<num_clos) of   */
                                    /*       closure healed by transform   */
                                    /*       if ityp_trans != 2, index = 0 */
  int  *natm_clos;                  /* Lth : num_clos, # of atms in closure*/
  int  *ityp_clos;                  /* Lth : num_clos, # 0:cblc, ...       */
  int  *ityp_clos_postprop;         /* Lth : num_clos, # 0:cblc, ...       */
  double **const_clos;              /* Lth : num_clos x NATM_CLOS_MAX      */
                                    /*       closure constraints           */
                                    /*       e.g. for sixth cblc           */
                                    /*       const_clos[6][1] = |r45|      */
  int **ind_atm_clos;               /* Lth : num_clos x NATM_CLOS_MAX      */
                                    /*  indices of atoms involved in clos  */
                                    /*  E.g. for fourth closure with five  */
                                    /*  atoms (7,8,10,12,13)               */
                                    /*      ind_atm_cblc[4][1] = 7         */
                                    /*      ind_atm_cblc[4][2] = 8         */
                                    /*      ind_atm_cblc[4][3] = 10        */
                                    /*      ind_atm_cblc[4][4] = 12        */
                                    /*      ind_atm_cblc[4][5] = 13        */
  int  *natm_clbase;                /* Lth : num_clbase                    */
  int  *res_id_clbase;              /* Lth : num_clbase, res_id for clbase */
  int  *freeze_clbase;              /* Lth : num_clbase, 1: fixed clbase   */
                                    /*          default  0: regular clbase */
  int **ind_atm_clbase;             /* Lth : num_clbase x NATM_CLBASE_MAX  */
  char *cblc;                       /* Lth : NMOLMAX                       */
  int  *ind_mol_cblc;               /* Lth : nmol_cblc :indices of cblc mol*/

  int  *natm_torcomplbase;          /* Lth : ntorcompl, number of atoms in */
                                    /*       in each torcomplbase          */
  int **ind_atm_torcomplbase;       /* Lth :   ntorcompl                   */ 
                                    /*       x NATM_TORCOMPLBASE_MAX       */

  int  *ind_mol_free;               /* Lth : nmol_free :indices of mols    */
                                    /*       not in a torsional complex    */
                                    /*       and not treated by cblc       */
  Rotate_utils rotate_utils;        /*  Rotational utility variables       */
 public:
  Tors_defs();
 ~Tors_defs();
  void setup( System& );
  void setup_sidechains(const Maps *, const Pos *, const General *);
  void setup_model_AA(const Maps *, const Pos *, const General *); 
  void setup_AA_protein(const Maps *,const Pos *,const General *,int,int*);
  void setup_AA_protein_D6(const Maps *,const Pos *,const General *,int,int*);
  void setup_AA_DNA_strand(const Maps *,const Pos *,const General *,int,int*);
  void setup_AA_RNA_strand(const Maps *,const Pos *,const General *,int,int*);
  void setup_AA_DNA_duplex(const Maps *,const Pos *,const General *,int,int*);
  void setup_AA_DNA_cblc(const Maps*,const Pos*,const General*,int, int*,
                                                               int*,int*);
  void setup_AA_DNA_scblc(const Maps*,const Pos*,const General*,int, int*,
			                                       int*,int*);
  void setup_AA_RNA_cblc(const Maps*,const Pos*,const General*,int, int*,
                                                               int*,int*);
  void setup_AA_RNA_scblc(const Maps*,const Pos*,const General*,int, int*,
                                                                int*,int*);
  void setup_AA_DNA_cblc_top_scblc(const Maps*,const Pos*,const General*,
                                                                int, int*,
                                                                int*,int*);
  void setup_AA_protein_cblc(const Maps*,const Pos*,const General*,int, int*,
                                                                   int*,int*);
  void setup_cgresmodel_UAPH(const Maps *, const Pos *, const General *);  
  void setup_cgresmodel_UA(const Maps *, const Pos *, const General *);  
  void setup_cgresmodel_5pt(const Maps *, const Pos *, const General *);  
  void setup_cgresmodel_5pt_cblc(const Maps *, const Pos *, const General *);  
  void setup_cgresmodel_3pt(const Maps *, const Pos *, const General *);  
  void setup_3pt(const Maps *, const Pos *, const General *);  
  void setup_3pt_cblc(const Maps *, const Pos *, const General *);  
  void setup_3pt_rb_cblc(const Maps *, const Pos *, const General *);  
  void setup_3pt_rb_scblc(const Maps *, const Pos *, const General *);  
  void setup_3pt_rb_scblc_mulchn(const Maps *, const Pos *, const General *);  
  void setup_3pt_region(const Maps *, const Pos *, const General *);  
  void setup_3pt_region_gen(const Maps *, const Pos *, const General *);  
  void setup_cgresmodel_1pt(const Maps *, const Pos *, const General *);  
  void write_alldata(const Maps *maps);
  void setup_closcompl(const Maps *, const Pos *, const General *);
  void setup_closcompl_DNA_duplex(const Maps*,const Pos*,const General*,int);
};//Tors_defs//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Pos {
  public:
   Pos() {}
   /*--------- Pos in System----------*/
   void setup(char*, Maps*, General*);
   void setup( Maps*, Restop_data*, Res_data_entry*, General*);
   void sampleVelocity(General*, Maps*);
   void readCoord(char*, General*);
   void write_res_arrangement(Maps *);
   double getEkin(General*);
   /*--------- Pos in Chain----------*/
   void setup( System&, double );
   void report_energy( System& );
   double energy( System&, int );
   double potential( System&, int );
   double potential( System& );
   double potential();
   double potential_enm();
   double kinetic();
   void respa_intra(System &);
   void respa_inter(System &);
   void move_md( System& );
   void move_md( System&, int );
   void move_tors( System& );
   void move_clbase(System & );
   void move_clbase_regular(System & );
   void move_clbase_regular(System &, int, double, double );
   void move_clbase_regions(System & );
   void move_clbase_region(System &, int);
   void move_clbase_respair(System &, REgion *, int );
   void move_torcomplbase(System &);
   void move_torcomplbase(System &, int, double, double);
   void move_closcompl(System & );
   void move_closcompl_DNA_duplex(System &, int);
   void move_notors(System & );
   void move_tomin( System& );
   void move_entry(System &, int, double);
   void move_entry(System & , int, int, int, double, double);
   void rotate_entry(System &, double*,double*,double*,double*, 
		               double, double, double,int,int,int);
   void translate_entry(System &, double, double, double,
			                   int, int, int);
   void postprop(System &);
   void postprop_minimize_cart(System &);
   void postprop_minimize_tors(System &);
   void postprop_minimize_bend(System &);
   void postprop_minimize_clos(System &);
   void postprop_minimize_sidechain(System &);
   void postprop_move_clos(System &);
   void vel_sampl( System& );
   void force_zero();
   void force_save();
   void force_restore();
   void force( System& );
   void force_enm();
   void debug_force( System& );
   void debug_force_enm( System& );
   void debug_force_u( System& );
   void init_propose( System& );
   void acc_propose();
   void get_hessian( System& );
   void get_hessian_enm( System& );
   void get_hessian_stg( System& );
   void diagon_hessian( System& );
   void update_xyz();
   void update_xyzc();
   void update_fxyz();
   void update_dxyz();
   void update_x_y_z();
   void update_xi_yi_zi();
   void update_fx_fy_fz();
   void update_dt();
   void update_uxyzbb( System& );
   void update_fuxyzbb( System& );   
   void update_duxyzbb( System& );   
   void update_uxyzcbb( System& );
   void update_uxbb_uybb_uzbb( System& );   
   void nm_transformXtoCNM();
   void nm_transformCNMtoX();
   void nm_transformTtoTNM();
   void nm_transformTNMtoT();
   void nm_transformUBBNMtoUBB();
   void tors_transformXtoT();
   void tors_transformTtoX();
   void tors_transformXFXtoTFT();
   void tors_transformTFTtoXFX();
   void tors_transformXtoB();
   void tors_transformBtoX();
   void tors_transformXFXtoBFB();
   void tors_transformBFBtoXFX();
   void tors_backward_half();
   void stage_transformXtoU();
   void stage_transformXFXtoUFU();
   void stage_transformUtoX();
   void write();
   void write_tors_defs(Tors_defs&, int );
   void operator=( Coord& );
  public:
   int allatm_model;                       /* 1: all atom residues          */
   int cgres_model;                        /* 1: coarse grained residues    */
   int cgres_model_KB;                     /* 1: KB model (1pt,3pt)         */
   int cgres_model_1pt;                    /* 1: 1pt model,(KB,physical:Go.)*/
   int cgres_model_3pt;                    /* 1: 3pt model (KB,physical)    */
   int cgres_model_5pt;                    /* 1: 5pt model (KB,physical)    */
   int cgres_model_UA;                     /* 1: UA model (KB,physical)     */
   int cgres_model_KB_UA;                  /* 1: UA  coarse KB residue      */
   int cgres_model_KB_5pt;                 /* 1: 5pt coarse KB residue      */
   int cgres_model_KB_3pt;                 /* 1: 3pt coarse KB residue      */
   int cgres_model_KB_1pt;                 /* 1: 1pt coarse KB residue      */
   int cgres_model_Go;                     /* 1: Go-type model, only CA     */
   int cgres_model_BNL;                    /* 1: BNL model, only CA         */
   int    action_type;                     /* <- system.general.action_type */
   int    natom,natm_tot;                  /*       both are the # of atoms */ 
   int    natm_bb;                         /* # of backbone atoms           */
   int    nres,nresbond,nrtfres;           /* # of res-, resbond-,restopres */
   int    nmol;                            /* # of molecules                */

   double boxl,boxw,boxh; /* if primtop, read in, oth. calc.min dist. in xyz*/
   double T;                               /*                   Temperature */
   double *tors_X;            /* X-space dihedral, Lth: Tors_defs.num_trans */
   double *t;                 /* X-space dihedral, Lth: Tors_defs.num_trans */
   double *ft;                /* X-sp. dih. force, Lth: Tors_defs.num_trans */
   double *dt;                /* X-sp. dih. deriv, Lth: Tors_defs.num_trans */
   double *t_scr;             /* dihedral scrmem., Lth: Tors_defs.num_trans */
   double *b;                 /* bend,      Lth:\sum_{i=1}^n natm_trans[i]  */
   double *fb;                /* bend force,Lth:\sum_{i=1}^n natm_trans[i]  */
   double *db;                /* bend deriv,Lth:\sum_{i=1}^n natm_trans[i]  */
   double *b_scr;             /* bend scrm, Lth:\sum_{i=1}^n natm_trans[i]  */
   double *x,*y,*z;           /*            atom/bead positions, Lth: natom */
   double *x0,*y0,*z0;        /*            atom/bead positions, Lth: natom */
   double *xyz;               /*            atom/bead positions, Lth:3*natom*/
   double *xi,*yi,*zi;        /*   proposed atom/bead positions, Lth: natom */
   double *vx,*vy,*vz;        /*            atom/bead velocities,Lth: natom */ 
   double *fx,*fy,*fz;        /*            atom/bead forces,    Lth: natom */
   double *fxsa,*fysa,*fzsa;  /*   saved    atom/bead forces,    Lth: natom */ 
   double *fxyz;              /*            atom/bead forces,    Lth:3*natom*/ 
   double *dxyz;              /*           -atom/bead forces,    Lth:3*natom*/ 
   double *ux,*uy,*uz;        /*transformed atom/bead positions, Lth: natom */
   double *fux,*fuy,*fuz;     /*transformed atom/bead forces,    Lth: natom */
   double *uxi,*uyi,*uzi;     /* init trans atom/bead positions, Lth: natom */
   double *fuxi,*fuyi,*fuzi;  /* init trans atom/bead forces,    Lth: natom */

   double *fxri,*fyri,*fzri;  /* respa inter atom/bead forces,   Lth: natom */

   double *uxyzbb;            /* transf. backb. a/b positions,Lth: 3*natm_bb*/
   double *fuxyzbb;           /* transf.backb.a/b  forces,    Lth: 3*natm_bb*/
   double *duxyzbb;           /* transf.backb.a/b -forces,    Lth: 3*natm_bb*/
   

   double *mass;              /*            atom/bead massess,   Lth: natom */ 
   double *chrg;              /*            atom/bead charges,   Lth: natom */ 

   double *cnm;               /* normal mode,NM coordinates, Lth: 3 x natom */
   double *xc,*yc,*zc;        /*  NM center atom/bead positions, Lth: natom */
   double *xyzc;              /*  NM center atom/bead positions, Lth:3*natom*/
   double *dxc,*dyc,*dzc;     /*  NM center atom/bead gradient,  Lth: natom */

   double *tc;                /* tors. NM center,  Lth: Tors_defs.num_trans */
   double *tnm;               /* tors. normal mode,Lth: Tors_defs.num_trans */

   double *uxyzcbb;          /*  bb based NM center,       Lth: 3 x natm_bb */
   double *ubbnm;            /*transformed bb normal mode, Lth: 3 x natm_bb */

   double **hessian;          /*    hessian, Lth: (3 x natom) x (3 x natom) */
                              /*              or Lth: num_trans x num_trans */
   double **eig_hess;         /*eig hessian, Lth: (3 x natom) x (3 x natom) */
                              /*              or Lth: num_trans x num_trans */
   double *lam_hess;          /*eigenvalues of the hessian, Lth: 3 x natom  */
                              /*                          or Lth: num_trans */
   double *llam_hess;         /* lower diagonal of the tridiagonal form     */
                              /* Lth: 3x natom, llam_hess[1] is ignored or  */
                              /* Lth: num_trans,llam_hess[1] is ignored     */ 

   double ebond,ebend,etors,eonfo,einter,epot;        /* energies for x y z */
   double ebend_bond, ebend_bend, etors_tors, etors_imp; /*energy fractions */
   double eonfo_lj, eonfo_coul;                          /*energy fractions */
   double einter_lj, einter_coul, einter_scp, einter_kb; /*energy fractions */
   double econstraint;                                   /*energy fractions */
   double ebondp,ebendp,etorsp,eonfop,einterp,epotp; /*energies for xi yi zi*/ 
   double ebendp_bond,ebendp_bend,etorsp_tors,etorsp_imp;/*energy fractions */
   double eonfop_lj, eonfop_coul;                        /*energy fractions */
   double einterp_lj,einterp_coul,einterp_scp,einterp_kb;/*energy fractions */
   double econstraintp;                                  /*energy fractions */

   double ebond_enm, ecryo_em;        /* enm, cryo_em energies for x y z    */
   double ebond_enmp, ecryo_emp;      /* enm, cryo_em energies for xi yi zi */

   int clos_succeed;  /*    indicated the presence of unclosed cloures     */
                      /*    1: there is an unclosed closure                */
                      /*    active if only general.clos_defined == 1       */ 

   int *permute_seq;  /* permutation of atomic indices(primitive topology) */ 
                      /* permutation of residue indices (residue topology) */
                      /* only used in \simulation_typ{SEQ_EEMC/SEQ_PT}     */ 
   int *permute_seq_prop;            /* MC proposal kernel for permute_seq */
   
   Residue *residue;
   Residue_bond *residue_bond;
   System *system_ptr;
};//Pos//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class REgion {
 public:
  int region_index;                /*  region index, e.g. 3 if 3rd region  */
  int element_top_type;            /*    1: atom, 2: residue               */
  int dependency_type;             /*    1: independent, 2: dependent      */    

  int natom;                       /* number of atoms                      */

  int nres;                    /* number of residues                       */
  int nseg;                    /* number of segments                       */
  int nfreeres;                /* number of free residues                  */
  int nrespair;                /* number of paired residues                */
  int ncenteres;               /* number of alternating center residues    */
  int *ind_res;                /* index of residues,      Lth:nres         */
  int *ind_seg_firstres;       /* index of first res in segs,  Lth: nseg   */
  int *ind_seg_lastres;        /* index of last res in segs,   Lth: nseg   */
  int *ind_seg_baseres;        /* index of base res in segs, Lth: nseg     */ 
  int *ind_freeres;            /* index of free residues, Lth: nfreeres    */
  int *ind_pairedres1;         /* index of pairedres1, Lth: nrespair       */
  int *ind_pairedres2;         /* index of pairedres2, Lth: nrespair       */
  int *ind_centeres;           /* index of center residues, Lth: ncenteres */

  int *ind_clbase_res;         /* index of clbase in residues, Lth:nres    */
  int *ind_clbase_seg_baseres; /* index of clbase in seg base res,Lth: nseg*/
  int *ind_clbase_freeres;     /* ind clbase in free residues, Lth:nfreeres*/
  int *ind_clbase_pairedres1;  /* ind clbase in pairedres1, Lth: nrespair  */
  int *ind_clbase_pairedres2;  /* ind clbase in pairedres2, Lth: nrespair  */
  int *ind_clbase_centeres;    /* ind clbase in cnt residues,Lth: ncenteres*/          
  double prop_trans_sig;
  double prop_rot_sig;
  double prop_trans_sig_freeres;           
  double prop_rot_sig_freeres;                  
  double prop_trans_sig_respair;      
  double prop_rot_sig_respair;
  double prop_trans_sig_respair_internal;                   
  double prop_rot_sig_respair_internal;   
  Tors_defs *tors_defs;        /* torsional definitions */
 public:
  void setup( const Region_data_entry*, System*, int );                     
  void write_alldata(System *);
};//Region//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Regions {
 public:
  int nregion;
  REgion *region;
 public:
  void setup( const Region_data_entry*, System* );
};//Regions//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class System {
 public:
  System() {}
  void checks();
  void setup(char*);
  void setup_silent(char*);
  void parse_input(Filenames*,char*);
  void vomit_input(Filenames*);
  void set_pot_data(const char*,const char*);  
  void set_region_data(const char*,const char*);  
  void set_constraint_data(const char*,const char*);
  void set_pdb_data(const char *);
  void set_res_data();
  void set_rtf_data(const char *);
 public:
  Pos pos;
  General general;
  Energy_part energy_part;
  Bond  bond;
  Bend  bend;
  Tors  tors;
  Onfo  onfo;
  Inter inter;
  Cryo_em cryo_em;
  Constraint constraint;
  Maps  maps;                         /* atom maps                */
  Regions regions;
  Tors_defs  tors_defs;
  Tors_defs  tors_defs_sidechain;
  Tors_defs *tors_defs_ptr;
  Stage_defs stage_defs;
  Filenames filenames;
  Pot_data_entry *bondprm_database;   /* bond potential database  */
  Pot_data_entry *bendprm_database;   /* bend potential database  */
  Pot_data_entry *torsprm_database;   /* tors potential database  */
  Pot_data_entry *onfoprm_database;   /* onfo potential database  */
  Pot_data_entry *interprm_database;  /* inter potential database */
  Pot_data_entry *inter_replaceprm_database; /* inter_replace pot. database */
  Pot_data_entry *cryo_emprm_database;  /* cryo_em potential database */
  Region_data_entry *regionprm_database;   /*         region database */
  Constraint_data_entry *constraintprm_database; /* constraint database */
  Pdb_data_entry *pdb_database;
  Res_data_entry *res_database;
  Restop_data *restop_database;
};//System//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Chain {
 public:
  Chain() {}
  void setup(int,System&);
  void setup_scalar(int,System&);
  void prop(System&);
  void get_I(System&);    
  void ee_jump(double,Coord,long*);
  double potential(System&);
  void operator=( Coord& );
  void operator=( Chain& );
 public:
   Pos pos;
   int nprop;          /* # of total propagation attempts WITHIN the Chain */
   int nacc;        /* # of accepted propagation attempts WITHIN the Chain */
   int nstat,I;
   int acc;       /* 1/0 a prop. att. WITHIN the Chain is accept/reject(ed)*/
   int eeacc;     /* 1/0 an equi-energy jump is accept/reject(ed)          */
   int ntprop;    /* total # of propagation attempts of any kind           */
   int neeprop;   /* total # of equi-energy jump attempts                  */
   int neeacc;    /* # of accepted equi-energy jump attempts               */
   int nptprop;   /* total # of parallel-tempering exchange attempts       */
   int nptacc;    /* # of accepted parallel-tempering exchange attempts    */
   int index;     /* natural indices for MC Chains 0, 1, 2, 3, ..., K      */
   int order;     /* ordering of MC Chains for identity exchange in mpi    */
   int action_type; /* simulation_typ basic action, inherited from general */
   double T;        /* temperature of the Chain, !for mpi it gets dynamic! */
};//Chain//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Coord {
  public:
   Coord();
  ~Coord();
   void setup( System& );
   void operator=( Coord& );
   void operator=( Chain& );
   double potential( System& );
  public:
   int natom,action_type;
   double *x,*y,*z;
   double ebond,ebend,etors,eonfo,einter,epot;       /* energies for x y z */
   /* for sequence sampling */
   int *permute_seq;           /* this refers to the permutation of atomic */
                               /* idices with respect to the initial setup */
};//Coord//
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
class Disk {
 public:
    Disk() {length = 0;}
    void add(Chain, long*);
    void setup(System &);
 public:
    int length;
    int max_size;
    Coord* stock;
};//Disk//
/*-------------------------------------------------------------------------*/ 
