#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"


#define DEBUG
#undef DEBUG

/*=========================================================================*/
/*=========================================================================*/
void System::checks() 
/*  This routine defines incompatibilities for certain options that has    */
/*  limited generality of operation                                        */ 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  

  int support_model, support_sim_typ, support_min_typ;  
  int support_prop_typ;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* I) \prop_type{bend}                                                     */

  if ( (general.prop_type == 3) ) {
        support_model    =  0;
        support_sim_typ  =  0;
        support_min_typ  =  0;
        
        support_model = general.allatm_model;
        if (support_model==0) {
            sprintf(err_msg, "\\prop_type{bend} is not supported with\n"
                             "the current model                      \n"
                             "Supported setups: all_atom_model only  \n"
                             "E.g.\\cgres_model{off} or do not use   \n"
                             "\\cgres_model, default is all_atom       ");
            errorMessage(err_msg);
        }/*endif*/

        
        support_sim_typ = general.debug_force + general.minimize;
        if (support_sim_typ==0) {
            sprintf(err_msg, "\\prop_type{bend} is not supported with \n"
                             "the current simulation type             \n"
                             "Supported setups:                       \n"
                             "\\simulation_typ{MIN,DBFR}                ");
            errorMessage(err_msg);
        }/*endif*/

        
        if (general.minimize == 1) {
            support_min_typ = general.minimize_type;
            if (support_min_typ > 2) {
                sprintf(err_msg, "\\prop_type{bend} is not supported with \n"
                                 "current type of minimization            \n"
                                 "Supported setups:                       \n"
                                 "\\minimize_type{cg,bfgs}                  ");
                errorMessage(err_msg);
            }/*endif*/
        }/*endif*/


        if (general.prop_tors_type != 1) {
            sprintf(err_msg, "\\prop_type{bend} is only supported with    \n"
                             "\\prop_tors_type{full} option, which is the \n"
                             "  default, so it need not be defined.         ");
            errorMessage(err_msg);
        }/*endif*/


  }/*endif*/    


/*-------------------------------------------------------------------------*/
/* II) \sym_type{PTSTG/PTSTGMC/NMSTG}                                      */

  if ( (general.stage == 1) ) {
        support_model     =  0;
        support_prop_typ  =  0;
        
        support_model = general.allatm_model;
        if (support_model==0) {
            sprintf(err_msg, "\\sym_type{PTSTG/PTSTGMC/NMSTG} is not\n"
                             "supported with the current model      \n"
                             "supported setups: all_atom_model only \n"
                             "E.g.\\cgres_model{off} or do not use  \n"
                             "\\cgres_model, default is all_atom       ");
            errorMessage(err_msg);
        }/*endif*/

        support_prop_typ = general.prop_type;
        if (support_prop_typ!=1) {
            sprintf(err_msg, "\\sym_type{PTSTG/PTSTGMC/NMSTG} is only  \n"
                             "supported with cartesian propagation     \n"
                             "E.g.\\prop_type{cart}                        ");
            errorMessage(err_msg);
        }/*endif*/


  }/*endif*/


/*-------------------------------------------------------------------------*/
/* III) \energy_term{cryo_em}                                              */

  if (general.energy_term_cryo_em == 1) {
        support_sim_typ  =  0;
        support_prop_typ =  0;
        support_min_typ  =  0;

        support_sim_typ = general.pt + general.minimize;                
        if (support_sim_typ==0) {
            sprintf(err_msg, "\\energy_term{cryo_em} is not supported \n"
                             "with the current simulation type        \n"
                             "Supported setups:                       \n"
                             "\\simulation_typ{PT,MIN}                  ");
            errorMessage(err_msg);
        }/*endif*/

        support_prop_typ = general.prop_type;
        if (support_prop_typ!=2) {
            sprintf(err_msg, "\\energy_term{cryo_em} is only       \n"
                             "supported with torsional propagation \n"
                             "E.g.\\prop_type{tors}                  ");
            errorMessage(err_msg);
        }/*endif*/
        
        if (general.minimize == 1) {
            support_min_typ = general.minimize_type;
            if (support_min_typ <= 2) {
                sprintf(err_msg, "\\energy_term{cryo_em} is not supported \n"
                                 "with current type of minimization       \n"
                                 "Supported types:                        \n"
                                 "\\minimize_type{samc,stsamc}              ");
                errorMessage(err_msg);
            }/*endif*/
        }/*endif*/

        if (general.cryo_em_database_defined != 1) {
                sprintf(err_msg, "\\energy_term{cryo_em} requires         \n"
			         "\\cryo_em_database_file{}                 ");
                errorMessage(err_msg);
        }/*endif*/

        if (general.cryo_em_expermnt_defined != 1) {
                sprintf(err_msg, "\\energy_term{cryo_em} requires         \n"
			         "\\cryo_em_expermnt_file{}                 ");
                errorMessage(err_msg);
        }/*endif*/

  }/*endif*/


/*-------------------------------------------------------------------------*/
/* IV) \region_database_file{}                                             */

  if (general.region_topology_defined == 1) {
        support_model    =  0;
        support_model = general.allatm_model + general.cgres_model_3pt;
        if (support_model==0) {
            sprintf(err_msg, "\\region_database_file{} is not supported with\n"
                             "the current model                      \n"
                             "Supported setups: all_atom_model only  \n"
                             "E.g.\\cgres_model{off} or do not use   \n"
                             "\\cgres_model, default is all_atom       ");
            errorMessage(err_msg);
        }/*endif*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
}/*end routine */
/*=========================================================================*/

