#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"


#define DEBUG_OFF


/*========================================================================*/
/*========================================================================*/
void System::set_pot_data(const char* ptype,const char *database_filename) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int bond_type, bend_type, tors_type, onfo_type;
  int inter_type, inter_replace_type;
  int cryo_em_type;
  int ientry, jentry, n, n_entry_mall, n_entry_now;
  int line_num;
  double *p2double,zero,inf;  

  char   err_msg[LINE_NCHARMAX];
  char   wrn_msg[LINE_NCHARMAX];
  char entry_line[LINE_NCHARMAX];
  FILE *database_file;
  Pot_data_entry *pot_database;

/*-------------------------------------------------------------------------*/
/*  Finding database type                                                  */

  bond_type = 0;
  bend_type = 0;
  tors_type = 0;
  onfo_type = 0;
  inter_type = 0;
  inter_replace_type = 0;
  cryo_em_type       = 0;

  if (strcasecmp(ptype,"bond")==0) {
     bond_type = 1;
  }
  else if (strcasecmp(ptype,"bend")==0) {
     bend_type = 1;
  }
  else if (strcasecmp(ptype,"tors")==0) {
     tors_type = 1;
  }
  else if (strcasecmp(ptype,"onfo")==0) {
     onfo_type = 1;
  }
  else if (strcasecmp(ptype,"inter")==0) {
     inter_type = 1;
  }
  else if (strcasecmp(ptype,"inter_replace")==0) {
     inter_replace_type = 1;
  }
  else if (strcasecmp(ptype,"cryo_em")==0) {
     cryo_em_type = 1;
  }/*end if*/

/*-------------------------------------------------------------------------*/
/*  Reading pot_database                                                   */

  printf("----------------------------------------------\n");
  printf("Reading in database from file %s\n...\n", database_filename);
  
  if ((database_file=fopen(database_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open database file %s.", database_filename);
    errorMessage(err_msg);
  }
  n_entry_mall = NMEM_MIN;
  pot_database = new Pot_data_entry[n_entry_mall]-1;
  
  line_num=0;
  n_entry_now=0;
  while((n=readOneEntry(database_file,'~','[',']',entry_line, line_num))!=0) {
    if (n!=1) {
      sprintf(err_msg, "%s at line %d while reading file %s.",
	      (n==-1) ? "Syntax error" : "Internal error", 
	      line_num, database_filename);
      errorMessage(err_msg);
    }
    n_entry_now++;
    if (n_entry_now>n_entry_mall) {
      n_entry_mall += NMEM_MIN;
      pot_database  = (Pot_data_entry *)
	realloc(pot_database+1, (n_entry_mall)*sizeof(Pot_data_entry))-1;
      for (ientry=1; ientry<n_entry_now-1; ientry++) {
	pot_database[ientry].next = &(pot_database[ientry+1]);
      }
    }
    pot_database[n_entry_now].setup(entry_line);
    if (n_entry_now>1) {
      pot_database[n_entry_now-1].next = &(pot_database[n_entry_now]);
    }
  }
  pot_database[n_entry_now].next = NULL;
  fclose(database_file);
  
//   for (ientry=1; ientry<=n_entry_now; ientry++) {
//     for (jentry=ientry+1; jentry<=n_entry_now; jentry++) {
//       if (pot_database[jentry].repeat(pot_database[ientry])) {
// 	if (pot_database[jentry]==pot_database[ientry]) {
// 	  sprintf(wrn_msg, "Database entry repeats. Occurences \n%s\n%s",
// 		  pot_database[ientry].get_name(),
//                   pot_database[jentry].get_name());
// 	  warnMessage(wrn_msg);
// 	}/*end if:equal*/
// 	else {
// 	  sprintf(err_msg, "Database entry repeats inconsistently. Occurences \n%s\n%s",
// 		  pot_database[ientry].get_name(),
//                   pot_database[jentry].get_name());
// 	  errorMessage(err_msg);
// 	}/*end else*/
//       }/*end if: entries repeat */
//     }/*end for: jentry */
//   }/*end for: ientry */

/*-------------------------------------------------------------------------*/
/* Create the second derivative of the numeric potential for splining      */

  if ((inter_type==1) || (inter_replace_type==1)) {
    if (inter.nprw == 0) {
      zero = 0.0;
      inf = 10.0;
      for (ientry=1; ientry<=n_entry_now; ientry++) {
        if ( (pot_database[ientry].type == 0) && 
             (pot_database[ientry].pot_type == 4) ) {
  	     p2double = dvector(1,pot_database[ientry].numeric_size);  
             spline(&(pot_database[ientry].numeric_x[0]),
                    &(pot_database[ientry].numeric_V_x[0]),
          	      pot_database[ientry].numeric_size,
		      inf,zero,p2double);
             pot_database[ientry].numeric_d2V_x = p2double; 
        }/*end if*/
      }/*end for*/
    } else if (inter.nprw == 1) {
      for (ientry=1; ientry<=n_entry_now; ientry++) {
        if ( (pot_database[ientry].type == 0) && 
             (pot_database[ientry].pot_type == 4) ) {
             p2double = dvector(1, 3*pot_database[ientry].numeric_size);
             nspline(&(pot_database[ientry].numeric_x[0]),
                     &(pot_database[ientry].numeric_V_x[0]),
                       pot_database[ientry].numeric_size,
                       p2double);
             pot_database[ientry].numeric_d2V_x = p2double;
        }/*end if*/
      }/*end for*/
    }/*end inter.nprw*/
  }/*end if*/

/*-------------------------------------------------------------------------*/
/* Initialize the cooresponding prm_database                               */

  if (bond_type==1) {
     bondprm_database = &(pot_database[1]);
  }
  else if (bend_type==1) {
     bendprm_database = &(pot_database[1]);
  }
  else if (tors_type==1) {
     torsprm_database = &(pot_database[1]);
  }
  else if (onfo_type==1) {
     onfoprm_database = &(pot_database[1]);
  }
  else if (inter_type==1) {
     interprm_database = &(pot_database[1]);
  }
  else if (inter_replace_type==1) {
     inter_replaceprm_database = &(pot_database[1]);
  }
  else if (cryo_em_type==1) {
     cryo_emprm_database = &(pot_database[1]);
  }/*end if*/

/*-------------------------------------------------------------------------*/
/* write out the splined numerial inter potentials in gnuplot format       */

#define DEBUG_OFF  
#ifdef DEBUG
 if (inter_type==1) {
  int size, ngrid, numeric;
  double x_sp, y_sp, length;
  int from_0;

  from_0 = 0;
  if (from_0) {/*write out spline from 0 in kcal/mol */
     ngrid = 100;
     length = 20.0; /* in Angstrom*/
     for (n=1;n<=ngrid;n++) {
         x_sp = ((double)n)*length/ngrid;  
         printf(" %g ", x_sp*BOHR);
         for (ientry=1; ientry<=n_entry_now; ientry++) {
             if ( (pot_database[ientry].type == 0) && 
                  (pot_database[ientry].pot_type == 4) ) {
	          splint(&(pot_database[ientry].numeric_x[1]),
                         &(pot_database[ientry].numeric_V_x[1]),
                	 &(pot_database[ientry].numeric_d2V_x[1]), 
                      pot_database[ientry].numeric_size,x_sp,&y_sp);
                  printf(" %g ", KCAL*y_sp);
             }
         }
         printf("\n");
     }
  }
  else {/*write out spline on the last entry's grid in kcal/mol*/
        /*do not write out last entry, ! -1 */
     if ( !(pot_database[n_entry_now].type == 0) && 
           (pot_database[n_entry_now].pot_type == 4) ) {
          sprintf(err_msg, " The last entry is not numeric inter type");
	  errorMessage(err_msg);
     }
     ngrid = pot_database[n_entry_now].numeric_size;
     for (n=1;n<=ngrid;n++) {
         x_sp = pot_database[n_entry_now].numeric_x[n];
         printf(" %g ", x_sp*BOHR);
         for (ientry=1; ientry<=n_entry_now-1; ientry++) {/* ! -1 */
             if ( (pot_database[ientry].type == 0) && 
                  (pot_database[ientry].pot_type == 4) ) {
	          splint(&(pot_database[ientry].numeric_x[1]),
                         &(pot_database[ientry].numeric_V_x[1]),
        	         &(pot_database[ientry].numeric_d2V_x[1]), 
                         pot_database[ientry].numeric_size,x_sp,&y_sp);
                  printf(" %g ", KCAL*y_sp);
             }
         }
         printf("\n");
     } 
  }
 }/*end if- inter_type*/
#endif

/*-------------------------------------------------------------------------*/

  printf("Database reading done, total %d entries read.\n", n_entry_now);
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
void Pot_data_entry::setup(char *database_line)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int i, j, n, nkey, key_defined, entry_defined, numeric_size_defined;
  int key_found[27];
  double bohr2,bohr6,bohr8,bohr9,bohr10;
  char err_msg[LINE_NCHARMAX];
  char wrn_msg[LINE_NCHARMAX];
  char line_save[LINE_NCHARMAX];
  char str_arg[NCHARMAX];
  char entry_name[NCHARMAX], key[NCHARMAX], arg[LINE_NCHARMAX];
  char *def;
  char *arg_token;
  char *space = " ";
  
  char *inter_keys[]= {"", "atom1", "atom2", "pot_type", 
		       "eps", "sig", "beta", "scale", "r0",
		       "awill", "bwill", "c6", "c8", "c10",
		       "min_dist", "max_dist", "res_dist", 
                       "numeric_size", "numeric_V_x", "numeric_x",
                       "label", "hash_size"};
  char *bond_keys[] = {"", "atom1", "atom2", "pot_type", "fk", "eq"};
  char *bend_keys[] = {"", "atom1", "atom2", "atom3", 
		       "pot_type_bend", "fk_bend", "eq_bend",
		       "pot_type_bond", "fk_bond", "eq_bond"};
  char *onfo_keys[] = {"", "atom1", "atom2", "pot_type", 
		       "eps", "sig", "beta", "scale", "r0",
		       "awill", "bwill", "c6", "c8", "c10",
		       "min_dist", "max_dist", "res_dist"};
  char *tors_keys[] = {"", "atom1", "atom2", "atom3", "atom4",
		       "pot_type", "p0", "p1", "p2", "p3",
		       "p4", "p5", "p6", "fk", "eq", "label", 
                       "s1", "s2", "s3", "s4", "s5", "s6",
                       "sp2", "sp3", "sp4", "sp5", "sp6"};
  char *cryo_em_keys[] = {"", "pot_type", "ea_az", "ea_alt", "ea_phi",  
                          "ea_range", "ea_interval", "pixel_size",
                          "resol_blur", "expermnt_file", 
                          "energy_scale"};
  
  strncpy(line_save, database_line, LINE_NCHARMAX);
  n=getEntryName(database_line, '~', '[', entry_name, def);
  if (n!=1) {
    sprintf(err_msg, "Syntax error at database line:\n%s", database_line);
    errorMessage(err_msg);
  }

  bohr2  = BOHR*BOHR;
  bohr6  = bohr2*bohr2*bohr2;
  bohr8  = bohr6*bohr2;
  bohr9  = bohr8*BOHR;
  bohr10 = bohr8*bohr2;
  
  entry_defined = 0;
  numeric_size_defined = 0;  
  
/*--------------------------------------------------------------------------*/
/* 0) Inter parm def    */
  if (strcasecmp(entry_name, "inter_parm")==0) {
    entry_defined = 1;
    type = 0;                                       /*set type to be inter */
    nkey = 21;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
    beta = 1.0;  key_found[6] = 1;             /*default value supported. */
    scale= 1.0;  key_found[7] = 1;
    c6m  = 0.0;  key_found[11]= 1;
    c8m  = 0.0;  key_found[12]= 1;
    c10m = 0.0;  key_found[13]= 1;
    hash_table  = 0;  key_found[20]= 1;     
    hash_size = 100;  key_found[21]= 1; 
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, inter_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
	  case 1:
	    strcpy(atm_lbl1, arg);
	    break;
	  case 2:
	    strcpy(atm_lbl2, arg);
	    break;
	  case 3:
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "null")==0) {
	      pot_type = 0;      /*null inter */
	      key_found[4] = 1; key_found[5] = 1; /*Need not specified */
	      key_found[6] = 1; key_found[7] = 1; 
	      key_found[8] = 1; key_found[9] = 1;
	      key_found[10]= 1; key_found[11]= 1;
	      key_found[12]= 1; key_found[13]= 1;
	      key_found[14]= 1; key_found[15]= 1;
	      key_found[16]= 1; key_found[17]= 1; 
              key_found[18]= 1; key_found[19]= 1;
	    }
	    else if (strcasecmp(str_arg, "lennard-jones")==0) {
	      pot_type = 1; /*lennard-jones inter */
	      key_found[8] = 1;  /*Need not specified */
	      key_found[9] = 1; key_found[10] = 1;
	      key_found[11]= 1; key_found[12] = 1;
	      key_found[13]= 1; key_found[17]= 1; 
              key_found[18]= 1; key_found[19]= 1;
	    }
	    else if (strcasecmp(str_arg, "lennard-jones-kihara")==0) {
	      pot_type = 2; /*lennard-jones-kihara inter */
	      key_found[9] = 1; /*Need not specified */
	      key_found[10]= 1; key_found[11]= 1; 
	      key_found[12]= 1; key_found[13]= 1;
              key_found[17]= 1; key_found[18]=1;
              key_found[19]= 1;
	    }
	    else if (strcasecmp(str_arg, "williams")==0) {
	      pot_type = 3; /*williams inter */
	      key_found[4] = 1; /*Need not specified */
	      key_found[5] = 1; key_found[6]= 1; 
	      key_found[7] = 1; key_found[8]= 1;
              key_found[17]= 1; key_found[18]=1;
              key_found[19]= 1;
	    }
            else if (strcasecmp(str_arg, "numeric")==0) {
	      pot_type = 4;      /*null inter */
	      key_found[4] = 1; key_found[5] = 1; /*Need not specified */
	      key_found[6] = 1; key_found[7] = 1; 
	      key_found[8] = 1; key_found[9] = 1;
	      key_found[10]= 1; key_found[11]= 1;
	      key_found[12]= 1; key_found[13]= 1;
	      key_found[14]= 1; r_min = 0.1;  
              key_found[15]= 1; r_max = 10.0;
	      key_found[16]= 1; r_res = 1.0;
	    }
	    else {
	      sprintf(err_msg, "Inter type %s not supported.",str_arg);
	      errorMessage(err_msg);
	    }
	    break;
	  case 4:
	    sscanf(arg, "%lf", &eps);
	    eps /= BOLTZ;
	    break;
	  case 5:
	    sscanf(arg, "%lf", &sig);
	    sig /= BOHR;
	    break;
	  case 6:
	    sscanf(arg, "%lf", &beta);
	    break;
	  case 7:
	    sscanf(arg, "%lf", &scale);
	    break;
	  case 8:
	    sscanf(arg, "%lf", &r0);
	    r0 /= BOHR;
	    break;
	  case 9:
	    sscanf(arg, "%lf", &awill);
	    awill /= BOLTZ;
	    break;
	  case 10:
	    sscanf(arg, "%lf", &bwill);
	    bwill *= BOHR;
	    break;
	  case 11:
	    sscanf(arg, "%lf", &c6m);
	    c6m /= (BOLTZ*bohr6);
	    break;
	  case 12:
	    sscanf(arg, "%lf", &c8m);
	    c8m /= (BOLTZ*bohr8);
	    break;
	  case 13:
	    sscanf(arg, "%lf", &c10m);
	    c10m /= (BOLTZ*bohr10);
	    break;
	  case 14:
	    sscanf(arg, "%lf", &r_min);
	    r_min /= BOHR;
	    break;
	  case 15:
	    sscanf(arg, "%lf", &r_max);
	    r_max /= BOHR;
	    break;
	  case 16:
	    sscanf(arg, "%lf", &r_res);
	    r_res /= BOHR;
	    break;
          case 17:
	    sscanf(arg, "%d", &numeric_size);
	    numeric_size_defined = 1;
	    break;  
          case 18:
            if (numeric_size_defined==0) {
	      sprintf(err_msg, "Numeric size undefined, atom1 %d and atom2 %d in database line:\n%s",
               atm_lbl1, atm_lbl2, database_line);
              errorMessage(err_msg);
            }  
            numeric_V_x = dvector(1,numeric_size);	
            for (j=1;j<=numeric_size;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,space);
                } 
                else {
	         arg_token = strtok(NULL,space);
                }
                sscanf(arg_token,"%lf",&(numeric_V_x[j]));
                numeric_V_x[j] /= BOLTZ;
            }
	    break;
          case 19:
            if (numeric_size_defined==0) {
	      sprintf(err_msg, "Numeric size undefined, atom1 %d and atom2 %d in database line:\n%s",
               atm_lbl1, atm_lbl2, database_line);
              errorMessage(err_msg);
            }  
            numeric_x = dvector(1,numeric_size);	
            for (j=1;j<=numeric_size;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,space);
                } 
                else {
	         arg_token = strtok(NULL,space);
                }
                sscanf(arg_token,"%lf",&(numeric_x[j]));
                numeric_x[j] /= BOHR;
            }
	    break;  
	  case 20:
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "hash_x")==0) {
	        hash_table = 1;
	    }
            else if (strcasecmp(str_arg, "hash_x2")==0) {
	        hash_table = 2;
	    }
	    break;
          case 21:
	    sscanf(arg, "%d", &hash_size);
	    break;
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
    for ( i=1; i<=nkey; i++) { 
      if (key_found[i]==0) {
	sprintf(err_msg, "Key word %s missed in database line:\n%s",
		inter_keys[i], database_line);
	errorMessage(err_msg);
      }
    }
    if (pot_type==0) {
      eps  = 0.0; sig   = 1.0; 
      beta = 1.0; scale = 1.0;  r0   = 1.0; 
      awill= 0.0; bwill = 0.0;
      c6m  = 0.0; c8m   = 0.0;  c10m = 0.0;
      r_min= 0.1; r_max = 10.0; r_res= 1.0;      
    }
    if (pot_type==1) {
      r0 = sig; /*For normal LJ potential */
      awill= 0.0; bwill = 0.0;
      c6m  = 0.0; c8m   = 0.0;  c10m = 0.0;
    }
    if (pot_type==2) {
      awill= 0.0; bwill = 0.0;
      c6m  = 0.0; c8m   = 0.0;  c10m = 0.0;
    }
    if (pot_type==3) {
      eps  = 0.0; sig   = 1.0; 
      beta = 1.0; scale = 1.0;  r0   = 1.0; 
    }
    if (pot_type==4) {
      eps  = 0.0; sig   = 0.0; 
      beta = 0.0; scale = 0.0;  r0   = 1.0; 
      awill= 0.0; bwill = 0.0;
      c6m  = 0.0; c8m   = 0.0;  c10m = 0.0;
    }
#ifdef DEBUG
    printf("eps %g, sig %g, r0 %g, awill %g, bwill %g, c6m %g\n",
                                       eps,sig,r0,awill,bwill,c6m);
    printf("c8m %g, c10m %g, r_min %g, r_max %g, r_res %g\n",
                                       c8m,c10m,r_min,r_max,r_res);
    getchar();
#endif

    if (eps  <0 || sig  <0 || r0   <0 ||
	awill<0 || bwill<0 || c6m  <0 ||
	c8m  <0 || c10m <0 || r_min<0 ||
	r_max<0 || r_res<0 ) { barf(line_save); }

    if ((hash_table!=0) && (pot_type!=4)) {
	sprintf(err_msg, "\\label{hash_x} or \\label{hash_x2}\n"
                         "is only supported by \\pot_type{numeric}");
	errorMessage(err_msg);
    }

  }/*endif: inter parm def */
  
/*--------------------------------------------------------------------------*/
/* 1) Bond parm def    */
  if (strcasecmp(entry_name, "bond_parm")==0) {
    entry_defined = 1;
    type = 1;                                       /*set type to be bond */
    nkey = 5;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, bond_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
	  case 1:
	    strcpy(atm_lbl1, arg);
	    break;
	  case 2:
	    strcpy(atm_lbl2, arg);
	    break;
	  case 3:	
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "harm")!=0) {
	      sprintf(err_msg, "Non-harmonic bond type not supported.");
	      errorMessage(err_msg);
	    } else { pot_type = 1; }
	    break;
	  case 4:
	    sscanf(arg, "%lf", &fk);
	    fk *= (BOHR*BOHR/BOLTZ);
	    break;
	  case 5:
	    sscanf(arg, "%lf", &r0);
	    r0 /= BOHR;
	    break;
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
    for ( i=1; i<=nkey; i++) { 
      if (key_found[i]==0) {
	sprintf(err_msg, "Key word %s missed in database line:\n%s",
		bond_keys[i], database_line);
	errorMessage(err_msg);
      }
    }
    if (fk<0 || r0<0) { barf(line_save); }
  }/*endif: bond parm def */
  
/*--------------------------------------------------------------------------*/
/* 2) Bend parm def    */
  if (strcasecmp(entry_name, "bend_parm")==0) {
    entry_defined = 1;
    type = 2;                                       /*set type to be bend */
    nkey = 9;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
    key_found[7]=1; key_found[8]=1; key_found[9]=1; /*Default values available */
    fk_bond = 0; r0 = 1.0;                          /*Default values           */
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, bend_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
	  case 1:
	    strcpy(atm_lbl1, arg);
	    break;
	  case 2:
	    strcpy(atm_lbl2, arg);
	    break;
	  case 3:
	    strcpy(atm_lbl3, arg);
	    break;
	  case 4:
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "harm")!=0) {
	      sprintf(err_msg, "Non-harmonic bend type not supported.");
	      errorMessage(err_msg);
	    } else {pot_type = 1;}
	    break;
	  case 5:
	    sscanf(arg, "%lf", &fk_bend);
	    fk_bend /= BOLTZ;
	    break;
	  case 6:
	    sscanf(arg, "%lf", &theta0);
	    theta0 *= M_PI/180.0;
	    break;
	  case 7:
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "harm")!=0) {
	      sprintf(err_msg, "Non harmonic bend-bond type not supported.");
	      errorMessage(err_msg);
	    }
	    break;
	  case 8:
	    sscanf(arg, "%lf", &fk_bond);
	    fk_bond *= (BOHR*BOHR/BOLTZ);
	    break;
	  case 9:
	    sscanf(arg, "%lf", &r0);
	    r0 /= BOHR;
	    break;
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
    for ( i=1; i<=nkey; i++) { 
      if (key_found[i]==0) {
	sprintf(err_msg, "Key word %s missed in database line:\n%s",
		bend_keys[i], database_line);
	errorMessage(err_msg);
      }
    }
    if (fk_bend<0 || theta0<0 || fk_bond<0 || r0<0) { barf(line_save); }
  }/*endif: bend parm def */
  
/*--------------------------------------------------------------------------*/
/* 3) Onfo parm def    */
  if (strcasecmp(entry_name, "onefour_parm")==0) {
    entry_defined = 1;
    type = 3;                                       /*set type to be onfo */
    nkey = 16;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
    beta = 1.0;  key_found[6] = 1;             /*default value supported. */
    scale= 1.0;  key_found[7] = 1;
    c6m  = 0.0;  key_found[11]= 1;
    c8m  = 0.0;  key_found[12]= 1;
    c10m = 0.0;  key_found[13]= 1;
    r_min= 0.1;  key_found[14]= 1;
    r_max=10.0;  key_found[15]= 1;
    r_res= 1.0;  key_found[16]= 1;
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, onfo_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
	  case 1:
	    strcpy(atm_lbl1, arg);
	    break;
	  case 2:
	    strcpy(atm_lbl2, arg);
	    break;
	  case 3:
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "null")==0) {
	      pot_type = 0;      /*null onfo */
	      key_found[4] = 1; key_found[5] = 1; /*Need not specified */
	      key_found[6] = 1; key_found[7] = 1; 
	      key_found[8] = 1; key_found[9] = 1;
	      key_found[10]= 1; key_found[11]= 1;
	      key_found[12]= 1; key_found[13]= 1;
	      key_found[14]= 1; key_found[15]= 1;
	      key_found[16]= 1;
	    }
	    else if (strcasecmp(str_arg, "lennard-jones")==0) {
	      pot_type = 1; /*lennard-jones onfo */
	      key_found[8] = 1;  /*Need not specified */
	      key_found[9] = 1; key_found[10] = 1;
	      key_found[11]= 1; key_found[12] = 1;
	      key_found[13]= 1;
	    }
	    else if (strcasecmp(str_arg, "lennard-jones-kihara")==0) {
	      pot_type = 2; /*lennard-jones-kihara onfo */
	      key_found[9] = 1; /*Need not specified */
	      key_found[10]= 1; key_found[11]= 1; 
	      key_found[12]= 1; key_found[13]= 1;
	    }
	    else if (strcasecmp(str_arg, "williams")==0) {
	      pot_type = 3; /*williams onfo */
	      key_found[4] = 1; /*Need not specified */
	      key_found[5]= 1; key_found[6]= 1; 
	      key_found[7]= 1; key_found[8]= 1;
	    }
	    else {
	      sprintf(err_msg, "Onfo type %s not supported.",str_arg);
	      errorMessage(err_msg);
	    }
	    break;
	  case 4:
	    sscanf(arg, "%lf", &eps);
	    eps /= BOLTZ;
	    break;
	  case 5:
	    sscanf(arg, "%lf", &sig);
	    sig /= BOHR;
	    break;
	  case 6:
	    sscanf(arg, "%lf", &beta);
	    break;
	  case 7:
	    sscanf(arg, "%lf", &scale);
	    break;
	  case 8:
	    sscanf(arg, "%lf", &r0);
	    r0 /= BOHR;
	    break;
	  case 9:
	    sscanf(arg, "%lf", &awill);
	    awill /= BOLTZ;
	    break;
	  case 10:
	    sscanf(arg, "%lf", &bwill);
	    bwill *= BOHR;
	    break;
	  case 11:
	    sscanf(arg, "%lf", &c6m);
	    c6m /= (BOLTZ*bohr6);
	    break;
	  case 12:
	    sscanf(arg, "%lf", &c8m);
	    c8m /= (BOLTZ*bohr8);
	    break;
	  case 13:
	    sscanf(arg, "%lf", &c10m);
	    c10m /= (BOLTZ*bohr10);
	    break;
	  case 14:
	    sscanf(arg, "%lf", &r_min);
	    r_min /= BOHR;
	    break;
	  case 15:
	    sscanf(arg, "%lf", &r_max);
	    r_max /= BOHR;
	    break;
	  case 16:
	    sscanf(arg, "%lf", &r_res);
	    r_res /= BOHR;
	    break;
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
    for ( i=1; i<=nkey; i++) { 
      if (key_found[i]==0) {
	sprintf(err_msg, "Key word %s missed in database line:\n%s",
		onfo_keys[i], database_line);
	errorMessage(err_msg);
      }
    }
    if (pot_type==0) {
      eps  = 0.0; sig   = 1.0; 
      beta = 1.0; scale = 1.0;  r0   = 1.0; 
      awill= 0.0; bwill = 0.0;
      c6m  = 0.0; c8m   = 0.0;  c10m = 0.0;
      r_min= 0.1; r_max = 10.0; r_res= 1.0;      
    }
    if (pot_type==1) {
      r0 = sig; /*For normal LJ potential */
      awill= 0.0; bwill = 0.0;
      c6m  = 0.0; c8m   = 0.0;  c10m = 0.0;
    }
    if (pot_type==2) {
      awill= 0.0; bwill = 0.0;
      c6m  = 0.0; c8m   = 0.0;  c10m = 0.0;
    }
    if (pot_type==3) {
      eps  = 0.0; sig   = 1.0; 
      beta = 1.0; scale = 1.0;  r0   = 1.0; 
    }
    if (eps  <0 || sig  <0 || r0   <0 ||
	awill<0 || bwill<0 || c6m  <0 ||
	c8m  <0 || c10m <0 || r_min<0 ||
	r_max<0 || r_res<0 ) { barf(line_save); }
  }/*endif: onfo parm def */
  
/*--------------------------------------------------------------------------*/
/* 4) Tors parm def    */
  if (strcasecmp(entry_name, "torsion_parm")==0) {
    entry_defined = 1;
    type = 4;                                       /*set type to be tors */
    nkey = 26;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
    /*Default value supported */
    key_found[15] = 1;  imp_tors = 0;  /*Default not to be improper */
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, tors_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
	  case 1:
	    strcpy(atm_lbl1, arg);
	    break;
	  case 2:
	    strcpy(atm_lbl2, arg);
	    break;
	  case 3:
	    strcpy(atm_lbl3, arg);
	    break;
	  case 4:
	    strcpy(atm_lbl4, arg);
	    break;
	  case 5:
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "power")==0) {
	      pot_type = 1;
	      /*Not necessary to specify */
	      key_found[13] = 1; key_found[14] = 1;
              /*Default value supported */
              key_found[16] = 1;  key_found[17] = 1;
              key_found[18] = 1;  key_found[19] = 1;
              key_found[20] = 1;  key_found[21] = 1;
              s1 = 0.0; s2 = 0.0; s3 = 0.0; 
              s4 = 0.0; s5 = 0.0; s6 = 0.0;    
              key_found[22] = 1;  key_found[23] = 1;
              key_found[24] = 1;  key_found[25] = 1;
              key_found[26] = 1;
              sp2 = 0.0; sp3 = 0.0; sp4 = 0.0;
              sp5 = 0.0; sp6 = 0.0;
	    } else if (strcasecmp(str_arg, "harm")==0) {
	      pot_type = 3;
	      /*Not necessary to specify */
	      key_found[6]  = 1; key_found[7]  = 1; 
	      key_found[8]  = 1; key_found[9]  = 1; 
	      key_found[10] = 1; key_found[11] = 1; 
	      key_found[12] = 1; 	      
              key_found[16] = 1;  key_found[17] = 1;
              key_found[18] = 1;  key_found[19] = 1;
              key_found[20] = 1;  key_found[21] = 1;
              key_found[22] = 1;  key_found[23] = 1;
              key_found[24] = 1;  key_found[25] = 1;
              key_found[26] = 1;
	    } else {
	      sprintf(err_msg, "Tors type not supported.");
	      errorMessage(err_msg);
	    }
	    break;
	  case 6:
	    sscanf(arg, "%lf", &p0);
	    p0 /= BOLTZ;
	    break;
	  case 7:
	    sscanf(arg, "%lf", &p1);
	    p1 /= BOLTZ;
	    break;
	  case 8:
	    sscanf(arg, "%lf", &p2);
	    p2 /= BOLTZ;
	    break;
	  case 9:
	    sscanf(arg, "%lf", &p3);
	    p3 /= BOLTZ;
	    break;
	  case 10:
	    sscanf(arg, "%lf", &p4);
	    p4 /= BOLTZ;
	    break;
	  case 11:
	    sscanf(arg, "%lf", &p5);
	    p5 /= BOLTZ;
	    break;
	  case 12:
	    sscanf(arg, "%lf", &p6);
	    p6 /= BOLTZ;
	    break;
	  case 13:
	    sscanf(arg, "%lf", &fk_tors);
	    fk_tors /= BOLTZ;
	    break;
	  case 14:
	    sscanf(arg, "%lf", &eq_tors);
	    eq_tors *= M_PI/180.0;
	    break;
	  case 15:
	    strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "improper")==0) {
	      imp_tors = 1;
	    }
	    break;
	  case 16:
	    sscanf(arg, "%lf", &s1);
	    s1 /= BOLTZ;
	    break;
	  case 17:
	    sscanf(arg, "%lf", &s2);
	    s2 /= BOLTZ;
	    break;
	  case 18:
	    sscanf(arg, "%lf", &s3);
	    s3 /= BOLTZ;
	    break;
	  case 19:
	    sscanf(arg, "%lf", &s4);
	    s4 /= BOLTZ;
	    break;
	  case 20:
	    sscanf(arg, "%lf", &s5);
	    s5 /= BOLTZ;
	    break;
	  case 21:
	    sscanf(arg, "%lf", &s6);
	    s6 /= BOLTZ;
	    break;
	  case 22:
	    sscanf(arg, "%lf", &sp2);
	    sp2 /= BOLTZ;
	    break;
	  case 23:
	    sscanf(arg, "%lf", &sp3);
	    sp3 /= BOLTZ;
	    break;
	  case 24:
	    sscanf(arg, "%lf", &sp4);
	    sp4 /= BOLTZ;
	    break;
	  case 25:
	    sscanf(arg, "%lf", &sp5);
	    sp5 /= BOLTZ;
	    break;
	  case 26:
	    sscanf(arg, "%lf", &sp6);
	    sp6 /= BOLTZ;
	    break;
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
    for ( i=1; i<=nkey; i++) { 
      if (key_found[i]==0) {
	sprintf(err_msg, "Key word %s missed in database line:\n%s",
		tors_keys[i], database_line);
	errorMessage(err_msg);
      }
    }
  }/*endif: tors parm def */
  
  
/*--------------------------------------------------------------------------*/
/* 5) Cryo_em parm def    */

  if (strcasecmp(entry_name, "cryo_em_parm")==0) {
    entry_defined = 1;
    type = 5;                                       /*set type to be bond */
    nkey = 10;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, cryo_em_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
	  case 1:
            strcpy(str_arg, arg);
	    if (strcasecmp(str_arg, "normal")!=0) {
	      sprintf(err_msg, "Non-normal cryo_em type not supported.");
	      errorMessage(err_msg);
	    } else { pot_type = 1; }
	    break;
	  case 2:
	    sscanf(arg, "%lf", &ea_az);
	    break;
	  case 3:	
	    sscanf(arg, "%lf", &ea_alt);
	    break;
	  case 4:
	    sscanf(arg, "%lf", &ea_phi);
	    break;
	  case 5:
	    sscanf(arg, "%lf", &ea_range);
	    break;
	  case 6:
	    sscanf(arg, "%lf", &ea_interval);
	    break;
          case 7:
	    sscanf(arg, "%lf", &pixel_size);
	    break;  
          case 8:
	    sscanf(arg, "%lf", &resol_blur);
	    break;  
	  case 9:
	    strcpy(expermnt_filename, arg);
	    break;
          case 10:
	    sscanf(arg, "%lf", &energy_scale);
	    break;  
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
    for ( i=1; i<=nkey; i++) { 
      if (key_found[i]==0) {
	sprintf(err_msg, "Key word %s missed in database line:\n%s",
		cryo_em_keys[i], database_line);
	errorMessage(err_msg);
      }
    }
#define DEBUG_CRYO_EM_OFF
#ifdef DEBUG_CRYO_EM
  getchar();
    printf("pixel_size %g, resol_blur %g\n",pixel_size,resol_blur);
    printf("ea_az %g, ea_alt %g, ea_phi %g\n",ea_az,ea_alt,ea_phi);
    printf("ea_range %g, ea_interval %g\n",ea_range,ea_interval);
    printf("expermnt_file %s\n",expermnt_filename);
    getchar();
#endif
  }/*endif: cryo_em parm def */

  
/*--------------------------------------------------------------------------*/
/* 6) Further processing    */


  if (entry_defined==0) {
    sprintf(err_msg, "Unknown database parm entry %s", entry_name);
    errorMessage(err_msg);
  }
  isWild = 0;
  if (strcasecmp(atm_lbl1, "X")==0 ||
      strcasecmp(atm_lbl2, "X")==0   )         { isWild = 1; }
  if (type==2 && strcasecmp(atm_lbl3, "X")==0) { isWild = 1; }
  if (type==4 && strcasecmp(atm_lbl4, "X")==0) { isWild = 1; }
  
  switch(type) {
  case 0:
    name = new char[50];
    sprintf(name, "Inter parm for %s--%s", atm_lbl1, atm_lbl2);
    break;
  case 1:    
    name = new char[50];
    sprintf(name, "Bond parm for %s--%s", atm_lbl1, atm_lbl2);
    break;
  case 2:
    name = new char[60];
    sprintf(name, "Bend parm for %s--%s--%s", 
	    atm_lbl1, atm_lbl2, atm_lbl3);
    break;
  case 3:
    name = new char[50];
    sprintf(name, "One-four parm for %s--%s", atm_lbl1, atm_lbl2);
    break;
  case 4:
    name = new char[70];
    if (imp_tors==0) {
       sprintf(name, "Torsion parm for %s--%s--%s--%s", 
	      atm_lbl1, atm_lbl2, atm_lbl3, atm_lbl4);
    }
    else if (imp_tors==1) {
       sprintf(name, "Torsion parm for %s--%s--%s--%s improper", 
              atm_lbl1, atm_lbl2, atm_lbl3, atm_lbl4);
    }
    break;
  case 5:
    name = new char[50];
    sprintf(name, "Orientation parm for Cryo_em energy score");
    break;
  default:
    name = new char[30];
    sprintf(name, "Unknown parm entry");
    break;
  }
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/



/*========================================================================*/
/*========================================================================*/
char* Pot_data_entry::get_name(void) const
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  return name;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/



/*========================================================================*/
/*========================================================================*/
bool Pot_data_entry::repeat(const Pot_data_entry &right) const
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  bool repeat=false;
  int Xmatch;
  
  if (type!=right.type) {return false;}
  if (isWild!=right.isWild) { return false;}
  if (strcasecmp(name, "Unknown parm entry")==0) 
    { return false;}                /*Unknown entries are all nonrepeating */
  if (strcasecmp(name, right.get_name())==0) { return true;}
  
  switch (type) {
  case 0:
  case 1:
  case 3:
    if (wild_strcasecmp(atm_lbl1, right.atm_lbl1)==0 &&
	wild_strcasecmp(atm_lbl2, right.atm_lbl2)==0 ) { 
        Xmatch = 0;
        if (strcasecmp(atm_lbl1,"X")==0 || strcasecmp(right.atm_lbl1,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl2,"X")==0 || strcasecmp(right.atm_lbl2,"X")==0) 
           {Xmatch++;}
        if (Xmatch!=2)
	   {repeat=true;} 
    }
    else if (wild_strcasecmp(atm_lbl1, right.atm_lbl2)==0 &&
	     wild_strcasecmp(atm_lbl2, right.atm_lbl1)==0 ) { 
        Xmatch = 0;
        if (strcasecmp(atm_lbl1,"X")==0 || strcasecmp(right.atm_lbl2,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl2,"X")==0 || strcasecmp(right.atm_lbl1,"X")==0) 
           {Xmatch++;}
        if (Xmatch!=2)
	   {repeat=true;}
    }
    break;
  case 2:
    if (wild_strcasecmp(atm_lbl1, right.atm_lbl1)==0 &&
	wild_strcasecmp(atm_lbl2, right.atm_lbl2)==0 &&
	wild_strcasecmp(atm_lbl3, right.atm_lbl3)==0 ) { 
        Xmatch = 0;
        if (strcasecmp(atm_lbl1,"X")==0 || strcasecmp(right.atm_lbl1,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl2,"X")==0 || strcasecmp(right.atm_lbl2,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl3,"X")==0 || strcasecmp(right.atm_lbl3,"X")==0) 
           {Xmatch++;}
        if (Xmatch!=3)
	   {repeat=true;}
    }
    else if (wild_strcasecmp(atm_lbl1, right.atm_lbl3)==0 &&
	     wild_strcasecmp(atm_lbl2, right.atm_lbl2)==0 &&
	     wild_strcasecmp(atm_lbl3, right.atm_lbl1)==0 ) { 
        Xmatch = 0;
        if (strcasecmp(atm_lbl1,"X")==0 || strcasecmp(right.atm_lbl3,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl2,"X")==0 || strcasecmp(right.atm_lbl2,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl3,"X")==0 || strcasecmp(right.atm_lbl1,"X")==0) 
           {Xmatch++;}
        if (Xmatch!=3)
	   {repeat=true;}
    }
    break;
  case 4:
    if (wild_strcasecmp(atm_lbl1, right.atm_lbl1)==0 &&
	wild_strcasecmp(atm_lbl2, right.atm_lbl2)==0 &&
	wild_strcasecmp(atm_lbl3, right.atm_lbl3)==0 &&
	wild_strcasecmp(atm_lbl4, right.atm_lbl4)==0 )	{
        Xmatch = 0;
        if (strcasecmp(atm_lbl1,"X")==0 || strcasecmp(right.atm_lbl1,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl2,"X")==0 || strcasecmp(right.atm_lbl2,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl3,"X")==0 || strcasecmp(right.atm_lbl3,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl4,"X")==0 || strcasecmp(right.atm_lbl4,"X")==0) 
           {Xmatch++;}
        if ((Xmatch!=4)&&(imp_tors==right.imp_tors))
	   {repeat=true;}
    }
    else if (wild_strcasecmp(atm_lbl1, right.atm_lbl4)==0 &&
	     wild_strcasecmp(atm_lbl2, right.atm_lbl3)==0 &&
	     wild_strcasecmp(atm_lbl3, right.atm_lbl2)==0 &&
	     wild_strcasecmp(atm_lbl4, right.atm_lbl1)==0 ) { 
        Xmatch = 0;
        if (strcasecmp(atm_lbl1,"X")==0 || strcasecmp(right.atm_lbl4,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl2,"X")==0 || strcasecmp(right.atm_lbl3,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl3,"X")==0 || strcasecmp(right.atm_lbl2,"X")==0) 
           {Xmatch++;}
        if (strcasecmp(atm_lbl4,"X")==0 || strcasecmp(right.atm_lbl1,"X")==0) 
           {Xmatch++;}
        if ((Xmatch!=4)&&(imp_tors==right.imp_tors))
	   {repeat=true;} 
    }
    break;
  default:
    repeat = false; /*Unknown entries are all nonrepeating */
    break;
  } /*end switch */

  return repeat;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/
  


/*========================================================================*/
/*========================================================================*/
bool Pot_data_entry::operator==(const Pot_data_entry &right) const
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  bool equal;
  if (type!=right.type)   {return false;}
  else if (pot_type!=right.pot_type) {return false;}
  
  equal = false;
  switch (type) {
  case 0:
    if (pot_type==0) {equal = true;}
    else if (scale!=right.scale || r_min!=right.r_min || 
	     r_max!=right.r_max || r_res!=right.r_res   ) {equal = false;} 
    else if (pot_type==1) {
      if (eps !=right.eps  || sig!=right.sig || beta!=right.beta) 
	{equal = false;}
      else { equal = true;}
    }
    else if (pot_type==2) {
      if (eps !=right.eps  || sig!=right.sig || 
	  beta!=right.beta || r0 !=right.r0    ) 
	{equal = false;}
      else { equal = true;}
    }
    else if (pot_type==3) {
      if (awill!=right.awill || bwill!=right.bwill ||
	  c6m  !=right.c6m   || c8m  != right.c8m  || c10m !=right.c10m) 
	{equal = false;}
      else { equal = true;}
    }
    break;
  case 1:
    if (fk!=right.fk || r0!=right.r0) {equal = false;}
    else { equal = true;}
    break;
  case 2:
    if (fk_bend!=right.fk_bend || theta0!=right.theta0 ||
	fk_bond!=right.fk_bond || r0!=right.r0) 
      {equal = false;}
    else { equal = true;}
    break;
  case 3:
    if (pot_type==0) {equal = true;}
    else if (scale!=right.scale || r_min!=right.r_min || 
	     r_max!=right.r_max || r_res!=right.r_res   ) {equal = false;} 
    else if (pot_type==1) {
      if (eps !=right.eps  || sig!=right.sig || beta!=right.beta) 
	{equal = false;}
      else { equal = true;}
    }
    else if (pot_type==2) {
      if (eps !=right.eps  || sig!=right.sig || 
	  beta!=right.beta || r0 !=right.r0    ) 
	{equal = false;}
      else { equal = true;}
    }
    else if (pot_type==3) {
      if (awill!=right.awill || bwill!=right.bwill ||
	  c6m  !=right.c6m   || c8m  != right.c8m  || c10m !=right.c10m) 
	{equal = false;}
      else { equal = true;}
    }
    break;
  case 4:
    if (p0!=right.p0 || p1!=right.p1 || p2!=right.p2 || 
	p3!=right.p3 || p4!=right.p4 || p5!=right.p5 || p6!=right.p6) 
      {equal = false;}
    else {equal = true;}
    break;
  default:
    equal = false; /*All unknown parm entries are unequal */
  }
  
  return equal;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/



/*========================================================================*/
/*========================================================================*/
int Pot_data_entry::match(const Maps *maps, const int *atoms, int type_in) const
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int exact_match=0, wild_match=0;
  int atom1_ind, atom2_ind, atom3_ind, atom4_ind;
  if (type!=type_in) { return 0; } //not matching
  switch(type_in) {
  case 0:
  case 1:
  case 3:
    atom1_ind = atoms[1]; 
    atom2_ind = atoms[2];
    if ((strcasecmp(maps->atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0)    
      {exact_match=1;}
    else if((strcasecmp(maps->atm_label[atom1_ind],atm_lbl2)) ==0 && 
	    (strcasecmp(maps->atm_label[atom2_ind],atm_lbl1)) ==0)
      {exact_match=1;}
    if ((wild_strcasecmp(maps->atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(wild_strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0)    
      {wild_match=1;}
    else if((wild_strcasecmp(maps->atm_label[atom1_ind],atm_lbl2)) ==0 && 
	    (wild_strcasecmp(maps->atm_label[atom2_ind],atm_lbl1)) ==0)
      {wild_match=1;}
    break;
  case 2:
    atom1_ind = atoms[1]; 
    atom2_ind = atoms[2];
    atom3_ind = atoms[3];
    if ((strcasecmp(maps->atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(strcasecmp(maps->atm_label[atom3_ind],atm_lbl3)) ==0 )    
      {exact_match=1;}
    else if ((strcasecmp(maps->atm_label[atom1_ind],atm_lbl3)) ==0 && 
	     (strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0 &&
	     (strcasecmp(maps->atm_label[atom3_ind],atm_lbl1)) ==0 )    
      {exact_match=1;}
    if ((wild_strcasecmp(maps->atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(wild_strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(wild_strcasecmp(maps->atm_label[atom3_ind],atm_lbl3)) ==0 )    
      {wild_match=1;}
    else if ((wild_strcasecmp(maps->atm_label[atom1_ind],atm_lbl3)) ==0 && 
	     (wild_strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0 &&
	     (wild_strcasecmp(maps->atm_label[atom3_ind],atm_lbl1)) ==0 )    
      {wild_match=1;}
    break;
  case 4:
    atom1_ind = atoms[1]; 
    atom2_ind = atoms[2];
    atom3_ind = atoms[3];
    atom4_ind = atoms[4];
    if ((strcasecmp(maps->atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(strcasecmp(maps->atm_label[atom3_ind],atm_lbl3)) ==0 && 
	(strcasecmp(maps->atm_label[atom4_ind],atm_lbl4)) ==0   )    
      {exact_match=1;}
    else if((strcasecmp(maps->atm_label[atom1_ind],atm_lbl4)) ==0 && 
	    (strcasecmp(maps->atm_label[atom2_ind],atm_lbl3)) ==0 &&
	    (strcasecmp(maps->atm_label[atom3_ind],atm_lbl2)) ==0 &&
	    (strcasecmp(maps->atm_label[atom4_ind],atm_lbl1)) ==0   )
      {exact_match=1;}
    if ((wild_strcasecmp(maps->atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(wild_strcasecmp(maps->atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(wild_strcasecmp(maps->atm_label[atom3_ind],atm_lbl3)) ==0 && 
	(wild_strcasecmp(maps->atm_label[atom4_ind],atm_lbl4)) ==0   )    
      {wild_match=1;}
    else if((wild_strcasecmp(maps->atm_label[atom1_ind],atm_lbl4)) ==0 && 
	    (wild_strcasecmp(maps->atm_label[atom2_ind],atm_lbl3)) ==0 &&
	    (wild_strcasecmp(maps->atm_label[atom3_ind],atm_lbl2)) ==0 &&
	    (wild_strcasecmp(maps->atm_label[atom4_ind],atm_lbl1)) ==0   )
      {wild_match=1;}
    break;
  default:
    exact_match = 0; /*Unknown type*/
    wild_match  = 0;
    break;
  } /*end switch */
  
  if (exact_match==1) { return 1;}
  if (wild_match ==1) { return 2;}
  return 0;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/



/*========================================================================*/
/*========================================================================*/
int Pot_data_entry::match(char **atm_label, const int *atoms, 
                                                 int type_in) const
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
 int exact_match=0, wild_match=0;
 int atom1_ind, atom2_ind, atom3_ind, atom4_ind;
 if (type!=type_in) { return 0; } //not matching
 switch(type_in) {
 case 0:
 case 1:
 case 3:
  atom1_ind = atoms[1]; 
  atom2_ind = atoms[2];
  if ((strcasecmp(atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(strcasecmp(atm_label[atom2_ind],atm_lbl2)) ==0)    
    {exact_match=1;}
  else if((strcasecmp(atm_label[atom1_ind],atm_lbl2)) ==0 && 
	    (strcasecmp(atm_label[atom2_ind],atm_lbl1)) ==0)
    {exact_match=1;}
  if ((wild_strcasecmp(atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(wild_strcasecmp(atm_label[atom2_ind],atm_lbl2)) ==0)    
    {wild_match=1;}
  else if((wild_strcasecmp(atm_label[atom1_ind],atm_lbl2)) ==0 &&
          (wild_strcasecmp(atm_label[atom2_ind],atm_lbl1)) ==0)
    {wild_match=1;}
  break;
 case 2:
  atom1_ind = atoms[1]; 
  atom2_ind = atoms[2];
  atom3_ind = atoms[3];
  if ((strcasecmp(atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(strcasecmp(atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(strcasecmp(atm_label[atom3_ind],atm_lbl3)) ==0 )    
    {exact_match=1;}
  else if ((strcasecmp(atm_label[atom1_ind],atm_lbl3)) ==0 && 
	     (strcasecmp(atm_label[atom2_ind],atm_lbl2)) ==0 &&
	     (strcasecmp(atm_label[atom3_ind],atm_lbl1)) ==0 )    
    {exact_match=1;}
  if ((wild_strcasecmp(atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(wild_strcasecmp(atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(wild_strcasecmp(atm_label[atom3_ind],atm_lbl3)) ==0 )    
    {wild_match=1;}
  else if ((wild_strcasecmp(atm_label[atom1_ind],atm_lbl3))==0 &&
	   (wild_strcasecmp(atm_label[atom2_ind],atm_lbl2))==0 &&
	   (wild_strcasecmp(atm_label[atom3_ind],atm_lbl1))==0 )
    {wild_match=1;}
  break;
 case 4:
  atom1_ind = atoms[1]; 
  atom2_ind = atoms[2];
  atom3_ind = atoms[3];
  atom4_ind = atoms[4];
  if ((strcasecmp(atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(strcasecmp(atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(strcasecmp(atm_label[atom3_ind],atm_lbl3)) ==0 && 
	(strcasecmp(atm_label[atom4_ind],atm_lbl4)) ==0   )    
    {exact_match=1;}
  else if((strcasecmp(atm_label[atom1_ind],atm_lbl4)) ==0 && 
	    (strcasecmp(atm_label[atom2_ind],atm_lbl3)) ==0 &&
	    (strcasecmp(atm_label[atom3_ind],atm_lbl2)) ==0 &&
	    (strcasecmp(atm_label[atom4_ind],atm_lbl1)) ==0   )
    {exact_match=1;}
  if ((wild_strcasecmp(atm_label[atom1_ind],atm_lbl1)) ==0 && 
	(wild_strcasecmp(atm_label[atom2_ind],atm_lbl2)) ==0 &&
	(wild_strcasecmp(atm_label[atom3_ind],atm_lbl3)) ==0 && 
	(wild_strcasecmp(atm_label[atom4_ind],atm_lbl4)) ==0   )    
    {wild_match=1;}
  else if((wild_strcasecmp(atm_label[atom1_ind],atm_lbl4)) ==0 &&
	  (wild_strcasecmp(atm_label[atom2_ind],atm_lbl3)) ==0 &&
	  (wild_strcasecmp(atm_label[atom3_ind],atm_lbl2)) ==0 &&
	  (wild_strcasecmp(atm_label[atom4_ind],atm_lbl1)) ==0  )
    {wild_match=1;}
  break;
 default:
   exact_match = 0; /*Unknown type*/
   wild_match  = 0;
   break;
 } /*end switch */
 
 if (exact_match==1) { return 1;}
 if (wild_match ==1) { return 2;}
 return 0;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
int Pot_data_entry::match(char *atm1_label, char *atm2_label, 
                                                 int type_in) const
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
 int exact_match=0, wild_match=0;
 int atom1_ind, atom2_ind, atom3_ind, atom4_ind;
 if (type!=type_in) { return 0; } //not matching
 switch(type_in) {
 case 0:
 case 1:
 case 3:
  if ((strcasecmp(atm1_label,atm_lbl1)) ==0 && 
	(strcasecmp(atm2_label,atm_lbl2)) ==0)    
    {exact_match=1;}
  else if((strcasecmp(atm1_label,atm_lbl2)) ==0 && 
	    (strcasecmp(atm2_label,atm_lbl1)) ==0)
    {exact_match=1;}
  if ((wild_strcasecmp(atm1_label,atm_lbl1)) ==0 && 
	(wild_strcasecmp(atm2_label,atm_lbl2)) ==0)    
    {wild_match=1;}
  else if((wild_strcasecmp(atm1_label,atm_lbl2)) ==0 &&
          (wild_strcasecmp(atm2_label,atm_lbl1)) ==0)
    {wild_match=1;}
  break;
 default:
   exact_match = 0; /*Unknown type*/
   wild_match  = 0;
   break;
 } /*end switch */
 
 if (exact_match==1) { return 1;}
 if (wild_match ==1) { return 2;}
 return 0;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/

