#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF


/*========================================================================*/
/*========================================================================*/
void System::set_constraint_data(const char* ptype,
                                 const char *database_filename) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int ientry, jentry, n, n_entry_mall, n_entry_now;
  int line_num;

  char   err_msg[LINE_NCHARMAX];
  char   wrn_msg[LINE_NCHARMAX];
  char entry_line[LINE_NCHARMAX];
  FILE *database_file;
  Constraint_data_entry *constraint_database;

/*-------------------------------------------------------------------------*/
/*  Reading constraint_database                                            */

  printf("----------------------------------------------\n");
  printf("Reading in database from file %s\n...\n", database_filename);
  
  if ((database_file=fopen(database_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open database file %s.", database_filename);
    errorMessage(err_msg);
  }
  n_entry_mall = NMEM_MIN;
  constraint_database = new Constraint_data_entry[n_entry_mall]-1;
  
  line_num=0;
  n_entry_now=0;
  while((n=readOneEntry(database_file,'~','[',']',entry_line, line_num))!=0) {
    if (n!=1) {
      sprintf(err_msg, "%s at line %d while reading file %s.",
	      (n==-1) ? "Syntax error" : "Internal error", 
	      line_num, database_filename);
      errorMessage(err_msg);
    }
    n_entry_now++;
    if (n_entry_now>n_entry_mall) {
      n_entry_mall += NMEM_MIN;
      constraint_database  = (Constraint_data_entry *)
	realloc(constraint_database+1, (n_entry_mall)*sizeof(Constraint_data_entry))-1;
      for (ientry=1; ientry<n_entry_now-1; ientry++) {
	constraint_database[ientry].next = &(constraint_database[ientry+1]);
      }
    }
    constraint_database[n_entry_now].setup(entry_line,&general);
    if (n_entry_now>1) {
      constraint_database[n_entry_now-1].next = &(constraint_database[n_entry_now]);
    }
  }
  constraint_database[n_entry_now].next = NULL;
  fclose(database_file);

/*-------------------------------------------------------------------------*/
/* Initialize the cooresponding prm_database                               */

  constraintprm_database = &(constraint_database[1]);

/*-------------------------------------------------------------------------*/

  printf("Database reading done, total %d entries read.\n", n_entry_now);
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
void Constraint_data_entry::setup(char *database_line, General *general)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, j, n, nkey, key_defined, entry_defined;

  int key_found[5];
  char err_msg[LINE_NCHARMAX];
  char wrn_msg[LINE_NCHARMAX];
  char line_save[LINE_NCHARMAX];
  char str_arg[NCHARMAX];
  char entry_name[NCHARMAX], key[NCHARMAX], arg[LINE_NCHARMAX];
  char arg_save[LINE_NCHARMAX];
  char *def;
  
  char *constraint_keys[]= {"", "atom1", "atom2", "fk", "r0" };

  
  strncpy(line_save, database_line, LINE_NCHARMAX);
  n=getEntryName(database_line, '~', '[', entry_name, def);
  if (n!=1) {
    sprintf(err_msg, "Syntax error at database line:\n%s", database_line);
    errorMessage(err_msg);
  }
  
  entry_defined    = 0;

/*--------------------------------------------------------------------------*/
/* 0) Read in constraint into database                                      */
  if (strcasecmp(entry_name, "constraint")==0) {
    entry_defined = 1;
    nkey = 4;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, constraint_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
            case 1:
              sscanf(arg, "%d", &atom1);
              break;
            case 2:
              sscanf(arg, "%d", &atom2);
              break;
            case 3:
              sscanf(arg, "%lf", &fk);
              fk *= (BOHR*BOHR/BOLTZ);
              break;
            case 4:
              sscanf(arg, "%lf", &r0);
              r0 /= BOHR;
              break;
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    /*-----------------------check database line reading--------------------*/
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
  }/*endif: entry_name "constraint"*/
/*--------------------------------------------------------------------------*/
/* II) Unknown database line                                                */

  if (entry_defined==0) {
    sprintf(err_msg, "Unknown database parm entry %s", entry_name);
    errorMessage(err_msg);
  }
  
/*--------------------------------------------------------------------------*/
/* III) DEBUG                                                               */

#ifdef DEBUG
    printf("~~~~~~~~~~~~~~~~~~~~~~~~~~Constraint~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
    printf("atom1_ind: %d -- atom2_ind %d, {fk=%.8g, r0=%.8g}\n",atom1,atom2,fk,r0);
    getchar();
#endif
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

