#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF


/*========================================================================*/
/*========================================================================*/
void System::set_region_data(const char* ptype,const char *database_filename) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int ientry, jentry, n, n_entry_mall, n_entry_now;
  int line_num;

  char   err_msg[LINE_NCHARMAX];
  char   wrn_msg[LINE_NCHARMAX];
  char entry_line[LINE_NCHARMAX];
  FILE *database_file;
  Region_data_entry *region_database;

/*-------------------------------------------------------------------------*/
/*  Reading region_database                                                   */

  printf("----------------------------------------------\n");
  printf("Reading in database from file %s\n...\n", database_filename);
  
  if ((database_file=fopen(database_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open database file %s.", database_filename);
    errorMessage(err_msg);
  }
  n_entry_mall = NMEM_MIN;
  region_database = new Region_data_entry[n_entry_mall]-1;
  
  line_num=0;
  n_entry_now=0;
  while((n=readOneEntry(database_file,'~','[',']',entry_line, line_num))!=0) {
    if (n!=1) {
      sprintf(err_msg, "%s at line %d while reading file %s.",
	      (n==-1) ? "Syntax error" : "Internal error", 
	      line_num, database_filename);
      errorMessage(err_msg);
    }
    n_entry_now++;
    if (n_entry_now>n_entry_mall) {
      n_entry_mall += NMEM_MIN;
      region_database  = (Region_data_entry *)
	realloc(region_database+1, (n_entry_mall)*sizeof(Region_data_entry))-1;
      for (ientry=1; ientry<n_entry_now-1; ientry++) {
	region_database[ientry].next = &(region_database[ientry+1]);
      }
    }
    region_database[n_entry_now].setup(entry_line,&general);
    if (n_entry_now>1) {
      region_database[n_entry_now-1].next = &(region_database[n_entry_now]);
    }
  }
  region_database[n_entry_now].next = NULL;
  fclose(database_file);

/*-------------------------------------------------------------------------*/
/* Initialize the cooresponding prm_database                               */

  regionprm_database = &(region_database[1]);

/*-------------------------------------------------------------------------*/
/* write out the splined numerial inter potentials in gnuplot format       */

#define DEBUG_OFF  
#ifdef DEBUG

#endif

/*-------------------------------------------------------------------------*/

  printf("Database reading done, total %d entries read.\n", n_entry_now);
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*========================================================================*/


/*========================================================================*/
/*========================================================================*/
void Region_data_entry::setup(char *database_line, General *general)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int ires, iseg, irespair, two;
  int i, j, n, nkey, key_defined, entry_defined;
  int natom_defined, nres_defined, nrespair_defined, ncenter_defined;
  int nseg_defined, centeres_found, pairedres1_found, pairedres2_found;


  int key_found[22];
  char err_msg[LINE_NCHARMAX];
  char wrn_msg[LINE_NCHARMAX];
  char line_save[LINE_NCHARMAX];
  char str_arg[NCHARMAX];
  char entry_name[NCHARMAX], key[NCHARMAX], arg[LINE_NCHARMAX];
  char arg_save[LINE_NCHARMAX];
  char *def;
  char *arg_token;
  char *space = " ";
  char *comma = ",";
  char *colon = ":";
  
  char *region_keys[]= {"", "element_top_type", "dependency_type", "natom",
		       "nres", "nrespair", "ncenter", "residues", "residue_pair",
		       "prop_trans_sig", "prop_rot_sig",
                       "prop_trans_sig_freeres", "prop_rot_sig_freeres",
		       "prop_trans_sig_respair", "prop_rot_sig_respair",
		       "prop_trans_sig_respair_internal",
		       "prop_rot_sig_respair_internal","centers", "nseg",
		       "segments_firstres", "segments_lastres",
                       "segments_baseres" };

  
  strncpy(line_save, database_line, LINE_NCHARMAX);
  n=getEntryName(database_line, '~', '[', entry_name, def);
  if (n!=1) {
    sprintf(err_msg, "Syntax error at database line:\n%s", database_line);
    errorMessage(err_msg);
  }
  
  entry_defined    = 0;
  natom_defined    = 0;
  nres_defined     = 0;
  nseg_defined     = 0;
  nrespair_defined = 0;
  ncenter_defined  = 0;

/*--------------------------------------------------------------------------*/
/* 0) Read in region into database                                          */
  if (strcasecmp(entry_name, "region")==0) {
    entry_defined = 1;
    nkey = 21;
    for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
     key_found[1] = 1;  element_top_type = 2;/* default value: {residue}     */ 
     key_found[2] = 1;  dependency_type  = 1;/* default value: {independent} */ 
     key_found[5] = 1;  nrespair         = 0;/* default value: {0}           */ 
     key_found[8] = 1;

     key_found[9]  = 1;  prop_trans_sig         = general->prop_trans_sig;
     key_found[10] = 1;  prop_rot_sig           = general->prop_rot_sig;
     key_found[11] = 1;  prop_trans_sig_freeres = general->prop_trans_sig;
     key_found[12] = 1;  prop_rot_sig_freeres   = general->prop_rot_sig;
     key_found[13] = 1;  prop_trans_sig_respair = general->prop_trans_sig;
     key_found[14] = 1;  prop_rot_sig_respair   = general->prop_rot_sig;
     key_found[15] = 1;  prop_trans_sig_respair_internal = general->prop_trans_sig;
     key_found[16] = 1;  prop_rot_sig_respair_internal   = general->prop_rot_sig;

    irespair = 0;
    two      = 2;
    while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
      key_defined = 0;
      for ( i=1; i<=nkey; i++) {                    /*loop for key words. */
	if (strcasecmp(key, region_keys[i])==0) { /*it's a keyword.     */
	  key_defined  = 1;
	  key_found[i] = 1;
	  switch (i) {
          case 1:
            strcpy(str_arg, arg);
            if (strcasecmp(str_arg, "atom")==0) {
	        element_top_type = 1;
            }
            else if (strcasecmp(str_arg, "residue")==0) {
	        element_top_type = 2;
            }
            else if (strcasecmp(str_arg, "segment")==0) {
	        element_top_type = 3;
            }
	    break;
          case 2:
            strcpy(str_arg, arg);
            if (strcasecmp(str_arg, "independent")==0) {
	        dependency_type = 1;
            }
            else if (strcasecmp(str_arg, "dependent")==0) {
	        dependency_type = 2;
            }
	    break;
          case 3:
	    sscanf(arg, "%d", &natom);
            natom_defined    = 1;
	    break;
          case 4:
	    sscanf(arg, "%d", &nres);
            nres_defined     = 1;
	    break;
          case 5:
	    sscanf(arg, "%d", &nrespair);
            nrespair_defined = 1;
	    break;
          case 6:
	    sscanf(arg, "%d", &ncenteres);
            ncenter_defined = 1;
	    break;
          case 7:
            if (nres_defined==0) {
	      sprintf(err_msg, "\\nres has not been defined" 
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }  
            if (nres==0) {
	      sprintf(err_msg, "\\nres{>0} has to be defined" 
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }  
            ind_res     = new int[nres] - 1;
            res_chainid = (char **)cmall_int_mat(1,nres,0,NCHARMAX);
            strcpy(arg_save,arg);
            ires = 0;
            arg_token = strtok(arg_save,comma); ires++;
            while (arg_token!=NULL) {
                   arg_token = strtok(NULL,comma);
                   if (arg_token!=NULL) {ires++;}
            }
            for (j=1;j<=nres;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,colon);
                } 
                else {
	         arg_token = strtok(NULL,colon);
                }
                if (arg_token==NULL) {
		    sprintf(err_msg, "the number (%d) of elements defined"
                                     " in \\residues{}"
                                     "\nis smaller than \\nres{%d}" 
			             " in database line:\n%s",ires,
                                                              nres,
                                                              database_line);
                    errorMessage(err_msg);
                }
                strcpy(res_chainid[j],arg_token);
	        arg_token = strtok(NULL,comma);
                sscanf(arg_token,"%d",&(ind_res[j]));
            }
            if (nres<ires) {
                sprintf(err_msg, "the number (%d) of elements defined"
                                 " in \\residues{}"
                                 "\nis larger than \\nres{%d}" 
			         " in database line:\n%s",ires,
                                                          nres,
                                                          database_line);
                errorMessage(err_msg);
            }
	    break;
          case 8:
            irespair++;
            if (irespair == 1) {
                if (nrespair_defined==0) {
       	            sprintf(err_msg, "\\nrespair has not been defined" 
                                     " in database line:\n%s",database_line);
                    errorMessage(err_msg);
                }  
                if (nrespair==0) {
       	            sprintf(err_msg, "\\nrespair{>0} has to be defined" 
                                     " in database line:\n%s",database_line);
                    errorMessage(err_msg);
                }                  
                ind_pairedres1     = new int[nrespair] - 1;
                ind_pairedres2     = new int[nrespair] - 1;
                pairedres1_chainid = (char **)cmall_int_mat(1,nrespair,0,NCHARMAX);      
                pairedres2_chainid = (char **)cmall_int_mat(1,nrespair,0,NCHARMAX);      
            }
            if (nrespair<irespair) {
              sprintf(err_msg, "the number (%d) of \\residue_pair"
                               "\nis larger than \\nrespair{%d}"
		               "\nin database line:\n%s",irespair,
                                                         nrespair,
                                                         database_line);
              errorMessage(err_msg);
            }
            for (j=1;j<=two;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,colon);
                } 
                else {
	         arg_token = strtok(NULL,colon);
                }
                if (j==1) {strcpy(pairedres1_chainid[irespair],arg_token);}
                if (j==2) {strcpy(pairedres2_chainid[irespair],arg_token);}
	        arg_token = strtok(NULL,comma);
                if (j==1) {sscanf(arg_token,"%d",&(ind_pairedres1[irespair]));}
                if (j==2) {sscanf(arg_token,"%d",&(ind_pairedres2[irespair]));}
            }
	    break;
	  case 9:
	    sscanf(arg, "%lf", &prop_trans_sig);
	    prop_trans_sig /= BOHR;
	    break;
	  case 10:
	    sscanf(arg, "%lf", &prop_rot_sig);
	    break;
	  case 11:
	    sscanf(arg, "%lf", &prop_trans_sig_freeres);
	    prop_trans_sig_freeres /= BOHR;
	    break;
	  case 12:
	    sscanf(arg, "%lf", &prop_rot_sig_freeres);
	    break;
	  case 13:
	    sscanf(arg, "%lf", &prop_trans_sig_respair);
	    prop_trans_sig_respair /= BOHR;
	    break;
	  case 14:
	    sscanf(arg, "%lf", &prop_rot_sig_respair);
	    break;
	  case 15:
	    sscanf(arg, "%lf", &prop_trans_sig_respair_internal);
	    prop_trans_sig_respair_internal /= BOHR;
	    break;
	  case 16:
	    sscanf(arg, "%lf", &prop_rot_sig_respair_internal);
	    break;
          case 17:
            if (ncenter_defined==0) {
	      sprintf(err_msg, "\\ncenter has not been defined" 
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }  
            if (ncenteres==0) {
	      sprintf(err_msg, "\\ncenter{>0} has to be defined" 
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }  
            ind_centeres     = new int[ncenteres] - 1;
            centeres_chainid = (char **)cmall_int_mat(1,ncenteres,0,NCHARMAX);
            strcpy(arg_save,arg);
            ires = 0;
            arg_token = strtok(arg_save,comma); ires++;
            while (arg_token!=NULL) {
                   arg_token = strtok(NULL,comma);
                   if (arg_token!=NULL) {ires++;}
            }
            for (j=1;j<=ncenteres;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,colon);
                } 
                else {
	         arg_token = strtok(NULL,colon);
                }
                if (arg_token==NULL) {
		    sprintf(err_msg, "the number (%d) of elements defined"
                                     " in \\centers{}"
                                     "\nis smaller than \\ncenter{%d}" 
			             " in database line:\n%s",ires,
                                                              ncenteres,
                                                              database_line);
                    errorMessage(err_msg);
                }
                strcpy(centeres_chainid[j],arg_token);
	        arg_token = strtok(NULL,comma);
                sscanf(arg_token,"%d",&(ind_centeres[j]));
            }
            if (ncenteres<ires) {
                sprintf(err_msg, "the number (%d) of elements defined"
                                 " in \\centers{}"
                                 "\nis larger than \\ncenteres{%d}" 
			         " in database line:\n%s",ires,
                                                          ncenteres,
                                                          database_line);
                errorMessage(err_msg);
            }
	    break;
          case 18:
	    sscanf(arg, "%d", &nseg);
            nseg_defined     = 1;
	    break;
          case 19:
            if (nseg_defined==0) {
	      sprintf(err_msg, "\\nseg has not been defined"
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }
            if (nseg==0) {
	      sprintf(err_msg, "\\nseg{>0} has to be defined"
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }
            ind_seg_firstres  = new int[nseg] - 1;
            seg_firstres_chainid = (char **)cmall_int_mat(1,nseg,0,NCHARMAX);
            strcpy(arg_save,arg);
            iseg = 0;
            arg_token = strtok(arg_save,comma); iseg++;
            while (arg_token!=NULL) {
                   arg_token = strtok(NULL,comma);
                   if (arg_token!=NULL) {iseg++;}
            }
            for (j=1;j<=nseg;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,colon);
                } 
                else {
	         arg_token = strtok(NULL,colon);
                }
                if (arg_token==NULL) {
		    sprintf(err_msg, "the number (%d) of elements defined"
                                     " in \\segments_firstres{}"
                                     "\nis smaller than \\nseg{%d}" 
			             " in database line:\n%s",iseg,
                                                              nseg,
                                                              database_line);
                    errorMessage(err_msg);
                }
                strcpy(seg_firstres_chainid[j],arg_token);
	        arg_token = strtok(NULL,comma);
                sscanf(arg_token,"%d",&(ind_seg_firstres[j]));
            }
            if (nseg<iseg) {
                sprintf(err_msg, "the number (%d) of elements defined"
                                 " in \\segments_firstres{}"
                                 "\nis larger than \\nseg{%d}" 
			         " in database line:\n%s",iseg,
                                                          nseg,
                                                          database_line);
                errorMessage(err_msg);
            }
	    break;
          case 20:
            if (nseg_defined==0) {
	      sprintf(err_msg, "\\nseg has not been defined"
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }
            if (nseg==0) {
	      sprintf(err_msg, "\\nseg{>0} has to be defined"
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }
            ind_seg_lastres  = new int[nseg] - 1;
            seg_lastres_chainid = (char **)cmall_int_mat(1,nseg,0,NCHARMAX);
            strcpy(arg_save,arg);
            iseg = 0;
            arg_token = strtok(arg_save,comma); iseg++;
            while (arg_token!=NULL) {
                   arg_token = strtok(NULL,comma);
                   if (arg_token!=NULL) {iseg++;}
            }
            for (j=1;j<=nseg;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,colon);
                } 
                else {
	         arg_token = strtok(NULL,colon);
                }
                if (arg_token==NULL) {
		    sprintf(err_msg, "the number (%d) of elements defined"
                                     " in \\segments_lastres{}"
                                     "\nis smaller than \\nseg{%d}" 
			             " in database line:\n%s",iseg,
                                                              nseg,
                                                              database_line);
                    errorMessage(err_msg);
                }
                strcpy(seg_lastres_chainid[j],arg_token);
	        arg_token = strtok(NULL,comma);
                sscanf(arg_token,"%d",&(ind_seg_lastres[j]));
            }
            if (nseg<iseg) {
                sprintf(err_msg, "the number (%d) of elements defined"
                                 " in \\segments_lastres{}"
                                 "\nis larger than \\nseg{%d}" 
			         " in database line:\n%s",iseg,
                                                          nseg,
                                                          database_line);
                errorMessage(err_msg);
            }
	    break;
          case 21:
            if (nseg_defined==0) {
	      sprintf(err_msg, "\\nseg has not been defined"
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }
            if (nseg==0) {
	      sprintf(err_msg, "\\nseg{>0} has to be defined"
                               " in database line:\n%s",database_line);
              errorMessage(err_msg);
            }
            ind_seg_baseres  = new int[nseg] - 1;
            seg_baseres_chainid = (char **)cmall_int_mat(1,nseg,0,NCHARMAX);
            strcpy(arg_save,arg);
            iseg = 0;
            arg_token = strtok(arg_save,comma); iseg++;
            while (arg_token!=NULL) {
                   arg_token = strtok(NULL,comma);
                   if (arg_token!=NULL) {iseg++;}
            }
            for (j=1;j<=nseg;j++) {
	        if (j==1) { 
                 arg_token = strtok(arg,colon);
                } 
                else {
	         arg_token = strtok(NULL,colon);
                }
                if (arg_token==NULL) {
		    sprintf(err_msg, "the number (%d) of elements defined"
                                     " in \\segments_baseres{}"
                                     "\nis smaller than \\nseg{%d}" 
			             " in database line:\n%s",iseg,
                                                              nseg,
                                                              database_line);
                    errorMessage(err_msg);
                }
                strcpy(seg_baseres_chainid[j],arg_token);
	        arg_token = strtok(NULL,comma);
                sscanf(arg_token,"%d",&(ind_seg_baseres[j]));
            }
            if (nseg<iseg) {
                sprintf(err_msg, "the number (%d) of elements defined"
                                 " in \\segments_baseres{}"
                                 "\nis larger than \\nseg{%d}" 
			         " in database line:\n%s",iseg,
                                                          nseg,
                                                          database_line);
                errorMessage(err_msg);
            }
	    break;
	  }/*end switch*/
	} /*end if */
      }/*end for */
      if (key_defined==0) {
	sprintf(err_msg, "Key word %s unknown in database line:\n%s",
		key, database_line);
	errorMessage(err_msg);
      }
    }/*end while: getKeyArgPair */
    /*-----------------------check database line reading--------------------*/
    if (n!=0) {
      sprintf(err_msg, "Syntax error in database line:\n%s",database_line);
      errorMessage(err_msg);
    }
    /*---------------------------check value--------------------------------*/
    if (element_top_type == 1) {
        if (element_top_type <0 || dependency_type <0 || natom <0 ||
	    prop_trans_sig <0   || prop_rot_sig <0 )
            { barf(line_save); }
    }/*endif*/
    if (element_top_type == 2) {
        if (element_top_type <0 || dependency_type <0 || nres <0 || 
            nrespair <0         || ncenteres <0       ||
            prop_trans_sig <0         || prop_rot_sig <0         ||
	    prop_trans_sig_freeres <0 || prop_rot_sig_freeres <0 ||
	    prop_trans_sig_respair <0 || prop_rot_sig_respair <0 ||
	    prop_trans_sig_respair_internal <0                   || 
            prop_rot_sig_respair_internal   <0 )
            { barf(line_save); }
    }/*endif*/
    if (element_top_type == 3) {
        if (element_top_type <0 || dependency_type <0 || nseg <0 ||
	    prop_trans_sig <0   || prop_rot_sig <0 )
            { barf(line_save); }
    }/*endif*/
    /*-------------------------check irespair<nrespair----------------------*/
    if (nrespair>irespair) {
        sprintf(err_msg, "the number (%d) of \\residue_pair"
                         "\nis smaller than \\nrespair{%d}"
   	                 "\nin database line:\n%s",irespair,
                                                   nrespair,
                                                   database_line);
        errorMessage(err_msg);
    }
    /*--------------------check missed necessary element--------------------*/
    if (element_top_type == 1) {
        for (i=1; i<=nkey; i++) { 
          if (strcasecmp("nres", region_keys[i])==0) {continue;}    /* res */
          if (strcasecmp("nrespair", region_keys[i])==0) {continue;}/* res */
          if (strcasecmp("ncenter", region_keys[i])==0)  {continue;}/* res & seg*/
          if (strcasecmp("residues", region_keys[i])==0) {continue;}/* res */
          if (strcasecmp("residue_pair", region_keys[i])==0) {continue;}/*res*/
          if (strcasecmp("centers", region_keys[i])==0) {continue;}/* res & seg*/
          if (strcasecmp("nseg", region_keys[i])==0) {continue;}/*seg*/
          if (strcasecmp("segments_firstres", region_keys[i])==0) {continue;}/*seg*/
          if (strcasecmp("segments_lastres", region_keys[i])==0) {continue;}/*seg*/
          if (strcasecmp("segments_baseres", region_keys[i])==0) {continue;}/*seg*/
          if (key_found[i]==0) {
   	        sprintf(err_msg, "Key word %s missed in database line:\n%s",
   		region_keys[i], database_line);
   	        errorMessage(err_msg);
	  }/*endif*/
	}/*endfor*/
    }/*endif*/
    if (element_top_type == 2) {
        for (i=1; i<=nkey; i++) {
          if (strcasecmp("natom", region_keys[i])==0) {continue;} /* atom */
          if (strcasecmp("nseg", region_keys[i])==0) {continue;}/*seg*/
          if (strcasecmp("segments_firstres", region_keys[i])==0) {continue;}/*seg*/
          if (strcasecmp("segments_lastres", region_keys[i])==0) {continue;}/*seg*/
          if (strcasecmp("segments_baseres", region_keys[i])==0) {continue;}/*seg*/
          if (key_found[i]==0) {
   	        sprintf(err_msg, "Key word %s missed in database line:\n%s",
   		region_keys[i], database_line);
   	        errorMessage(err_msg);
	  }/*endif*/
	}/*endfor*/
    }/*endif*/
    if (element_top_type == 3) {
        for (i=1; i<=nkey; i++) {
          if (strcasecmp("natom", region_keys[i])==0) {continue;} /* atom */
          if (strcasecmp("nres", region_keys[i])==0) {continue;}    /* res */
          if (strcasecmp("nrespair", region_keys[i])==0) {continue;}/* res */
          if (strcasecmp("residues", region_keys[i])==0) {continue;}/* res */
          if (strcasecmp("residue_pair", region_keys[i])==0) {continue;}/*res*/
          if (key_found[i]==0) {
   	        sprintf(err_msg, "Key word %s missed in database line:\n%s",
   		region_keys[i], database_line);
   	        errorMessage(err_msg);
	  }/*endif*/
	}/*endfor*/
    }/*endif*/
  }/*endif: entry_name "region"*/
/*--------------------------------------------------------------------------*/
/* II) Unknown database line                                                */

  if (entry_defined==0) {
    sprintf(err_msg, "Unknown database parm entry %s", entry_name);
    errorMessage(err_msg);
  }
  
  switch(element_top_type) {
  case 1:    
    name = new char[NCHARMAX];
    sprintf(name, "Atomic region with %d atoms", natom);
    break;
  case 2:
    name = new char[NCHARMAX];
    sprintf(name, "Residue region with %d residues", nres);
    break;
  }

/*--------------------------------------------------------------------------*/
/* III) Further checks                                                      */

    if (element_top_type==2) {
        if (2*nrespair>nres) {
	    sprintf(err_msg, "2 x \\nrespair{%d} > \\nres{%d}",
		                                      nrespair,
		                                         nres);
   	    errorMessage(err_msg);
        }/*endif*/

        for (j=1;j<=ncenteres;j++) {
          centeres_found = 0;
          for (i=1;i<=nres;i++) {
    	  if ( (ind_res[i]==ind_centeres[j])                      &&
                 (strcasecmp(res_chainid[i],centeres_chainid[j])==0)  )   {
    	        centeres_found = 1;
            }
          }
          if (centeres_found==0) {
     	    sprintf(err_msg, "\\center{%s:%d} was not"
                               "\nfound in \\residues{} in database line:\n%s",
    		               centeres_chainid[j],
    		               ind_centeres[j],
                                 database_line);
     	    errorMessage(err_msg);
          }
        }/*endfor*/

        for (j=1;j<=nrespair;j++) {
              pairedres1_found = 0;
              pairedres2_found = 0;
              for (i=1;i<=nres;i++) {
         	  if ( (ind_res[i]==ind_pairedres1[j])                      &&
                       (strcasecmp(res_chainid[i],pairedres1_chainid[j])==0)  )   {
       	                 pairedres1_found = 1;
                  }
         	  if ( (ind_res[i]==ind_pairedres2[j])                      &&
                       (strcasecmp(res_chainid[i],pairedres2_chainid[j])==0)  )   {
       	                 pairedres2_found = 1;
                  }
              }
              if (pairedres1_found==0) {
         	    sprintf(err_msg, "first element of \\residue_pair{%s:%d,%s:%d}"
                                     "\nwas not found in "
                                     "\\residues{} in database line:\n%s",
       		                    pairedres1_chainid[j],
       		                    ind_pairedres1[j],
                                    pairedres2_chainid[j],
       		                    ind_pairedres2[j],
                                    database_line);
         	    errorMessage(err_msg);
              }
              if (pairedres2_found==0) {
         	    sprintf(err_msg, "second element of \\residue_pair{%s:%d,%s:%d}"
                                     "\nwas not found in "
                                     "\\residues{} in database line:\n%s",
       		                    pairedres1_chainid[j],
       		                    ind_pairedres1[j],
                                    pairedres2_chainid[j],
       		                    ind_pairedres2[j],
                                    database_line);
         	    errorMessage(err_msg);
              }
        }/*endfor-j*/
    }/*endif*/

/*--------------------------------------------------------------------------*/
/* IV) DEBUG                                                                */
/*--------------------------------------------------------------------------*/

#ifdef DEBUG
    printf("~~~~~~~~~~~~~~~~~~~~~~~~~~Region~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
    printf("element_top_type %d \n",element_top_type);
    printf("dependency_type  %d \n", dependency_type);
    
    if (element_top_type==1) {
        printf("natom %d \n",natom);
        printf("prop_trans_sig %g \n",prop_trans_sig);
        printf("prop_rot_sig %g \n",prop_rot_sig);
    }
 getchar();
    if (element_top_type==2) {
        printf("nres %d \n",nres);
        printf("nrespair %d \n",nrespair);
        printf("ncenteres %d \n",ncenteres);
        printf("---------------center residues--------------\n");
        for (i=1;i<=ncenteres;i++) {
	  printf("ind_centeres[%d] %d \t",i,ind_centeres[i]);
	  printf("centeres_chainid[%d] #%s# \n",i,centeres_chainid[i]);
        }
        printf("\n");       
        printf("---------------residues--------------\n");
        for (i=1;i<=nres;i++) {
	  printf("ind_res[%d] %d \t",i,ind_res[i]);
	  printf("res_chainid[%d] #%s# \n",i,res_chainid[i]);
        }
        printf("\n");
        printf("------------residue pairs------------\n");
        for (i=1;i<=nrespair;i++) {
	  printf("pairedres1_chainid[%d] #%s# ",i,pairedres1_chainid[i]);
	  printf("ind_pairedres1[%d] %d  ",i,ind_pairedres1[i]);
	  printf("pairedres2_chainid[%d] #%s# ",i,pairedres2_chainid[i]);
	  printf("ind_pairedres2[%d] %d \n",i,ind_pairedres2[i]);
        }
        printf("\n");
        printf("--------------step size--------------\n");
        printf("prop_trans_sig %g \n",prop_trans_sig);
        printf("prop_rot_sig %g \n",prop_rot_sig);
        printf("prop_trans_sig_freeres %g \n",prop_trans_sig_freeres);
        printf("prop_rot_sig_freeres %g \n",prop_rot_sig_freeres);
        printf("prop_trans_sig_respair %g \n",prop_trans_sig_respair);
        printf("prop_rot_sig_respair %g \n",prop_rot_sig_respair);
        printf("prop_trans_sig_respair_internal %g \n",
                                     prop_trans_sig_respair_internal);
        printf("prop_rot_sig_respair_internal %g \n",
                                     prop_rot_sig_respair_internal);
    }
    getchar();
    if (element_top_type==3) {
        printf("nseg %d \n",nseg);
        printf("ncenteres %d \n",ncenteres);
        printf("\n");
        printf("---------------segments_firstresidue--------------\n");
        for (i=1;i<=nseg;i++) {
	  printf("ind_seg_firstres[%d] %d \t",i,ind_seg_firstres[i]);
	  printf("seg_firstres_chainid[%d] #%s# \n",i,seg_firstres_chainid[i]);
        }
        printf("\n");
        printf("---------------segments_lastresidue--------------\n");
        for (i=1;i<=nseg;i++) {
	  printf("ind_seg_lastres[%d] %d \t",i,ind_seg_lastres[i]);
	  printf("seg_firstres_chainid[%d] #%s# \n",i,seg_firstres_chainid[i]);
        }
        printf("\n");
        printf("---------------segments_baseresidue--------------\n");
        for (i=1;i<=nseg;i++) {
	  printf("ind_seg_baseres[%d] %d \t",i,ind_seg_baseres[i]);
	  printf("seg_baseres_chainid[%d] #%s# \n",i,seg_baseres_chainid[i]);
        }
        printf("\n");
        printf("---------------center residues--------------\n");
        for (i=1;i<=ncenteres;i++) {
	  printf("ind_centeres[%d] %d \t",i,ind_centeres[i]);
	  printf("centeres_chainid[%d] #%s# \n",i,centeres_chainid[i]);
        }
        printf("\n");
        printf("prop_trans_sig %g \n",prop_trans_sig);
        printf("prop_rot_sig %g \n",prop_rot_sig);
        printf("prop_trans_sig_freeres %g \n",prop_trans_sig_freeres);
        printf("prop_rot_sig_freeres %g \n",prop_rot_sig_freeres);
    }
#endif
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

