#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Maps::setup( Filenames *filenames, General *general )
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, iaatype;
  int action_type            = general->action_type;
  int control_design         = general->control_design;
  int n_chains_mc            = general->n_chains_mc;

  int iatm;
  int natm_dna_bb, natm_rna_bb;
  int natm_ade_base, natm_gua_base, natm_cyt_base;
  int natm_thy_base, natm_ura_base;


  char *design_filename;
  char line[LINE_NCHARMAX];
  char err_msg[LINE_NCHARMAX];
  FILE *design_file;

/*-------------------------------------------------------------------------*/
/* 0.a) Assign map class variables                                         */ 

  inter_on_3bond_conn       = general->inter_on_3bond_conn; 
  inter_on_4bond_conn       = general->inter_on_4bond_conn;
  cancel_res_inter_local    = general->cancel_res_inter_local;    
  cancel_res_inter_neighbor = general->cancel_res_inter_neighbor; 
  
  inter_on_3bond_conn       = general->inter_on_3bond_conn; 
  inter_on_4bond_conn       = general->inter_on_4bond_conn;

/*-------------------------------------------------------------------------*/
/* 0.b) Allocate map class variables                                       */ 

  order_X   = new int[n_chains_mc+1];     /* 0,1,...,n_chains_mc */

  naatype = 25;
  aa_label  = (char  **)cmall_int_mat(1, naatype, 0, NNAMECHARMAX); 
  aa1_code  = (char  **)cmall_int_mat(1, naatype, 0, NNAMECHARMAX); 
  aa3_code  = (char  **)cmall_int_mat(1, naatype, 0, NNAMECHARMAX); 

  natm_dna_bb   = 18;
  natm_rna_bb   = 19;
  natm_ade_base = 14;
  natm_gua_base = 15;
  natm_cyt_base = 11;
  natm_thy_base = 14;
  natm_ura_base = 11;

  atm_dna_bb_label  = 
               (char  **)cmall_int_mat(1, natm_dna_bb, 0, NNAMECHARMAX); 
  atm_rna_bb_label  = 
               (char  **)cmall_int_mat(1, natm_rna_bb, 0, NNAMECHARMAX); 
  atm_ade_base_label  = 
             (char  **)cmall_int_mat(1, natm_ade_base, 0, NNAMECHARMAX);
  atm_gua_base_label  = 
             (char  **)cmall_int_mat(1, natm_gua_base, 0, NNAMECHARMAX);
  atm_cyt_base_label  = 
             (char  **)cmall_int_mat(1, natm_cyt_base, 0, NNAMECHARMAX);
  atm_thy_base_label  = 
             (char  **)cmall_int_mat(1, natm_thy_base, 0, NNAMECHARMAX);
  atm_ura_base_label  = 
             (char  **)cmall_int_mat(1, natm_ura_base, 0, NNAMECHARMAX);

/*-------------------------------------------------------------------------*/
/* I) Assign constants for topology                                        */  

/*-------------------------------------------------------------------------*/
/* I.a) Primitive topology                                                 */ 

  char *aa_label_local[] = {"","ACA","CCA","DCA","ECA","FCA","GCA","HCA",
                               "ICA","KCA","LCA","MCA","NCA","PCA","QCA",
                               "RCA","SCA","TCA","VCA","WCA","YCA",
                               "aC4*","cC4*","gC4*","tC4*","uC4*"};
  char *aa1_code_local[] = {"","A","C","D","E","F","G","H",
                               "I","K","L","M","N","P","Q",
                               "R","S","T","V","W","Y",
                               "a","c","g","t","u"};
  char *aa3_code_local[] = {"","ALA","CYS","ASP","GLU","PHE","GLY","HIS",
			        "ILE","LYS","LEU","MET","ASN","PRO","GLN",
   			        "ARG","SER","THR","VAL","TRP","TYR",
                                "A","C","G","T","U"};  

  for (iaatype=1;iaatype<=naatype;iaatype++) {
      strcpy(aa_label[iaatype],aa_label_local[iaatype]);
      strcpy(aa1_code[iaatype],aa1_code_local[iaatype]);
      strcpy(aa3_code[iaatype],aa3_code_local[iaatype]);
  }    

/*-------------------------------------------------------------------------*/
/* I.b) All atom topology                                                  */ 


  char *atm_dna_bb_label_local[] = {"","P","O1P","O2P","O5'","C5'","H5'",
                                   "H5''","C4'","H4'","O4'","C1'","H1'",
                                   "C3'","H3'","C2'","H2'","H2''","O3'"};
  char *atm_rna_bb_label_local[] = {"","P","O1P","O2P","O5'","C5'","H5'",
                                   "H5''","C4'","H4'","O4'","C1'","H1'",
                                   "C3'","H3'","C2'","H2'","H2''","O3'","O2'"};
  char *atm_ade_base_label_local[] = {"","N9","C8","H8","N7","C5","C6","N6",
                                    "H61","H62","N1","C2","H2","N3","C4"};
  char *atm_gua_base_label_local[] = {"","N9","C8","H8","N7","C5","C6","O6",
                                    "N1","H1","C2","N2","H21","H22","N3","C4"};
  char *atm_cyt_base_label_local[] = {"","N1","C6","H6","C5","H5","C4","H41",
                                    "H42","N3","C2","O2"};
  char *atm_thy_base_label_local[] = {"","N1","C6","H6","C5","C5M","H51","H52",
                                    "H53","C4","O4","N3","H3","C2","O2"};
  char *atm_ura_base_label_local[] = {"","N1","C6","H6","C5","H5","C4","O4",
                                    "N3","H3","C2","O2"};


  for (iatm=1;iatm<=natm_dna_bb;iatm++) {
      strcpy(atm_dna_bb_label[iatm],atm_dna_bb_label_local[iatm]);
  }/*endfor*/
  for (iatm=1;iatm<=natm_rna_bb;iatm++) {
      strcpy(atm_rna_bb_label[iatm],atm_rna_bb_label_local[iatm]);
  }/*endfor*/
  for (iatm=1;iatm<=natm_ade_base;iatm++) {
      strcpy(atm_ade_base_label[iatm],atm_ade_base_label_local[iatm]);
  }/*endfor*/
  for (iatm=1;iatm<=natm_gua_base;iatm++) {
      strcpy(atm_gua_base_label[iatm],atm_gua_base_label_local[iatm]);
  }/*endfor*/
  for (iatm=1;iatm<=natm_cyt_base;iatm++) {
      strcpy(atm_cyt_base_label[iatm],atm_cyt_base_label_local[iatm]);
  }/*endfor*/
  for (iatm=1;iatm<=natm_thy_base;iatm++) {
      strcpy(atm_thy_base_label[iatm],atm_thy_base_label_local[iatm]);
  }/*endfor*/
  for (iatm=1;iatm<=natm_ura_base;iatm++) {
      strcpy(atm_ura_base_label[iatm],atm_ura_base_label_local[iatm]);
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* II) Open and read in design_file                                        */ 

  if ((action_type==3 || action_type==4) && control_design) {
     design_filename = filenames->design_filename;
     design_field = new int[5] - 1;
     if ((design_file=fopen(design_filename, "r")) == NULL) {
       sprintf(err_msg, "Cannot open parm file %s.", design_filename);
       errorMessage(err_msg);
     }
     if ((fgets(line, LINE_NCHARMAX, design_file))==NULL) {
       sprintf(err_msg, "Error in reading file %s",design_filename);
       errorMessage(err_msg);
     }
     if (sscanf(line,"%d %d %d %d %d",&(design_field[1]),
                                &(design_field[2]),&(design_field[3]),
                                &(design_field[4]),&(design_field[5]))!=5) {
       sprintf(err_msg, "Error in reading file %s .",design_filename);
       errorMessage(err_msg);
     }

     fclose(design_file);
  }

/*-------------------------------------------------------------------------*/
/* III) DEBUG                                                              */ 
 
#ifdef DEBUG
  for (iaatype=1;iaatype<=naatype;iaatype++) {
      printf("aa1_code[%d] from maps %s\n",iaatype,aa1_code[iaatype]);
      printf("aa3_code[%d] from maps %s\n",iaatype,aa3_code[iaatype]);
      printf("aa_label[%d] from maps %s\n",iaatype,aa_label[iaatype]);
  }
  for (i=1;i<=5;i++) {
      printf("design_field[%d] %d\n",i,design_field[i]);
  }
#endif

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
