#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
void System::parse_input(Filenames *filenames,char *input_filename) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
   
  int    i, n, nkey, key_defined;
  int    line_num;
  int   *key_found;
  int    int_arg;
  long   long_arg;
  double real_arg;
  char   str_arg[NCHARMAX];
  char   err_msg[LINE_NCHARMAX];

  char entry_line[LINE_NCHARMAX];
  char entry_name[NCHARMAX], key[NCHARMAX], arg[NCHARMAX];
  char *def;
  
  FILE *input_file;
  char *sim_gen_keys[]= {"",
			 "simulation_typ", "replica_number", "total_step_mc",
			 "local_step_md", "time_step_md", "statistics_freq",
                         "prob_eemc_jump", "temperature", "eemc_disk_size",
                         "burn_in_B", "burn_in_N", "extend_inter",
                         "EEMC_Emin", "EEMC_Emax", "write_energy_unit",
                         "energy_gap", "prop_type", "prop_tors_sig",
                         "cancel_res_inter", "inter_list", "num_procs",
                         "rinter_switch_length", "prop_tors_type", 
                         "minimize_tol", "minimize_type", "minimize_report",
                         "energy_report", "prop_rot_sig", "prop_trans_sig",
                         "postprop_minimize", "postprop_minimize_itmax", 
                         "postprop_minimize_energy", "prop_clos_sig",
                         "prop_notors_sig_scale", "prop_closcompl_type",
                         "postprop_minimize_gradclos_itmax", "random_seed",
                         "scale_mass","scale_mass_value","scale_mass_value_H",
                         "intra_respa_step", "sa_temp_first", "sa_temp_last",
                         "sa_rate", "stsamc_type", "stsamc_period",
                         "stsamc_ampl", "prop_DNA_closcompl_type",
                         "cancel_res_inter_replace", "neighbor_list_skin",
                         "lnk_list_skin","stsamc_shift", 
                         "rinter_exclude_length", "prop_regions_type"};
  char *sim_mol_keys[]= {"",
                         "mol_parm_file", "bond_database_file", 
                         "pos_init_file", "pos_out_file", "atom_pos_file", 
                         "epot_file", "einter_file", "design_file", 
                         "system_def", "cgres_model", "bend_database_file",
                         "tors_database_file", "onfo_database_file",
                         "inter_database_file", "tors_pos_file", 
                         "hessian_file", "implicit_solvent", "eighess_file",
                         "neutralize", "ddd", "ddd_D", "ddd_D0", "ddd_S", 
                         "ddd_c", "ddd_e", "force_const_cutoff", 
                         "force_const_cutoff_bond", "force_const_cutoff_bend",
                         "eigvechess_file", "inter_replace_database_file", 
                         "inter_replace_type", "inter_nonpairwise",
                         "energy_term", "cryo_em_database_file",
                         "cryo_em_expermnt_file", "region_database_file", 
                         "constraint_database_file"};
  
  if ( (input_file=fopen(input_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open input file %s.", input_filename);
    errorMessage(err_msg);
  }    
  
  key_found = new int[54] - 1;

/*------------------------------------------------------------------------*/
/*Read an entry */
  line_num = 0;
  while((n=readOneEntry(input_file,'~','[',']',entry_line, line_num))!=0) {
    if (n!=1) {
      sprintf(err_msg, "%s at line %d while reading file %s.",
	      (n==-1) ? "Syntax error" : "Internal error", 
	      line_num, input_filename);
      errorMessage(err_msg);
    }
    n=getEntryName(entry_line, '~', '[', entry_name, def);
    if (n!=1) {
      sprintf(err_msg, "Syntax error at input file line:\n%s", entry_line);
      errorMessage(err_msg);
    }

/*-------------------------------------------------------------------------*/
/* 1) ~sim_gen_def: alias ~method:                                         */
    if ( (strcasecmp(entry_name, "sim_gen_def")==0) || 
         (strcasecmp(entry_name, "method")==0) ) {
      nkey = 54;
      for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
      /*they have default values */
      key_found[12] = 1; /*If used this key controls two variables */  
       general.inter_on_3bond_conn = 0; 
       general.inter_on_4bond_conn = 0;
      key_found[18] = 1; /*For torsional propagation default:10% Pi*/
       general.prop_tors_sig = 0.1*M_PI;
      key_found[19] = 1;          /* Default cancel_res_inter{off} */
       general.cancel_res_inter_local = 0;
       general.cancel_res_inter_neighbor = 0;
      key_found[20] = 1;               /* Default inter_list{none} */
       general.lnk_list = 0;       
       general.neighbor_list = 0;       
      key_found[21] = 1;                    /* Default 1 processor */
       general.num_procs = 1;       
      key_found[22] = 1;       /* Default 1 A rinter_switch_length */
       inter.rswitch_length = 1.0/BOHR;
      key_found[23] = 1;          /* Default \prop_tors_type{full} */
       general.prop_tors_type = 1;
      key_found[24] = 1;          /* Default \minimize_tol{1.e-7}  */
       general.minimize_tol = 1.e-7;
      key_found[25] = 1;          /* Default \minimize_type{cg}    */
       general.minimize_type = 1;
      key_found[26] = 1;          /* Default \minimize_report{2}   */
       general.minimize_report = 2;
      key_found[27] = 1;          /* Default \energy_report{0}     */
       general.energy_report = 0;
      key_found[28] = 1; /*For rotational propagation default:10% Pi*/
       general.prop_rot_sig = 0.1*M_PI;
      key_found[29] = 1; /*For translational propagation default: 0.001 A*/
       general.prop_trans_sig = 1.e-3/BOHR;
      key_found[30] = 1; /*No post propagation minimization by default*/
       general.postprop_minimize = 0;
      key_found[31] = 1; /*5 steps of iteration minimization by default*/
        general.postprop_minimize_itmax = 5; 
      key_found[32] = 1; /*Default postprop min uses the total energy funct.*/
        general.postprop_minimize_energy = 5; 
      key_found[33] = 1; /*For closure propagation default: 0.001 A*/
       general.prop_clos_sig = 1.e-3/BOHR;
      key_found[34] = 1; /*No scaling of notors sigmas*/
       general.prop_notors_sig_scale = 1.0;
      key_found[35] = 1;   /* Default \prop_closcompl_type{trans_rot}    */
       general.prop_closcompl_type = 3;
      key_found[36] = 1;   /* Default \postprop_minimize_gradclos_itmax{1} */
       general.postprop_minimize_gradclos_itmax = 1;
      key_found[37] = 1;   /* Default general.seed is defined in General */
                           /* Constructor                                */
      key_found[38] = 1;   /* Default \scale_mass{none}                  */
       general.scale_mass = 0;                   
      key_found[39] = 1;   /* Default \scale_mass_value{1.0}            */
       general.scale_mass_value = 1.0;                   
      key_found[40] = 1;   /* Default \scale_mass_value_H{1.0}          */
       general.scale_mass_value_H = 1.0;                   
      key_found[41] = 1;   /* Default \intra_respa_step{1}              */
       general.intra_respa_step = 1;                   
      key_found[42] = 1;   /* Default \sa_temp_first{5000.0} in K          */
       general.sa_temp_first = 5000.0;                   
      key_found[43] = 1;   /* Default \sa_temp_last{0.1} in K              */
       general.sa_temp_last  = 0.1;                   
      key_found[44] = 1;   /* Default \sa_rate{0.9995}                     */
       general.sa_rate = 0.9995;                   
      key_found[45] = 1;   /* Default \stsamc_type{1} 1: trigonometric     */
       general.stsamc_type = 1;                   
      key_found[46] = 1;   /* Default \stsamc_period{4000}                 */
       general.stsamc_period = 4000;                   
      key_found[47] = 1;   /* Default \stsamc_ampl{800.0}                  */
       general.stsamc_ampl = 800.0;    
      key_found[48] = 1;   /* Default \prop_closcompl_type{intra_inter_level}*/
       general.prop_DNA_closcompl_type = 3;               
      key_found[49] = 1;   /* Default \cancel_res_inter_replace{off}*/
       general.cancel_res_inter_replace_local = 0;               
       general.cancel_res_inter_replace_neighbor = 0;               
      key_found[50] = 1;       /* Default 1 A neighbor_list_skin */
       inter.neighbor_list_skin = 1.0/BOHR;
      key_found[51] = 1;       /* Default 0 A lnk_list_skin */
       inter.lnk_list_skin = 0.0/BOHR;
      key_found[52] = 1;   /* Default \stsamc_shift{0.0}             */
       general.stsamc_shift = 0.0;    
      key_found[53] = 1;   /* Default \rinter_exclude_length{100.0}  */    
       inter.rexclude_length = 100.0/BOHR;
       inter.rexclude_length_defined = 0;       
      key_found[54] = 1;          /* Default \prop_regions_type{superimpose} */
       general.prop_regions_type = 1;

      /* the "null" methods */
      general.pt        = 0;
      general.ptemp     = 0;
      general.nmode     = 0;
      general.nmode_stg = 0;
      general.nmpt      = 0;
      general.minimize  = 0;
      general.md        = 0;
      general.ptrepswa  = 0;
      general.repswa    = 0;
      general.ptstage   = 0;
      general.stage     = 0;
      /* the "null" debug mode methods */
      general.debug           = 0;
      general.debug_force     = 0;
      general.debug_tors_defs = 0;
      /* the "null enm" methods */
      general.enm             = 0;
      general.nmode_enm       = 0;
      general.debug_force_enm = 0;           
      general.debug_force_stage = 0;           

      while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
	key_defined = 0;
	for ( i=1; i<=nkey; i++) {                     /*loop for key words.*/
	  if (strcasecmp(key, sim_gen_keys[i])==0) {   /*it's a keyword.    */
	    key_defined  = 1;
	    key_found[i] = 1;
	    switch (i) {
	    case 1:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg,"PT")==0)    {general.action_type=1;
	                                                general.pt   =1;
                                                        general.ptemp=1;      }
	      else if (strcasecmp(str_arg,"PTMC")==0)  {general.action_type=1;
	                                                general.pt   =1;
                                                        general.ptemp=1;      }
	      else if (strcasecmp(str_arg,"PTRPSW")==0)  
                                                       {general.action_type=1;
                                                        general.ptrepswa=1;
                                                        general.ptemp=1;
                                                        general.repswa=1;     }
	      else if (strcasecmp(str_arg,"PTRPSWMC")==0)  
                                                       {general.action_type=1;
                                                        general.ptrepswa=1;
                                                        general.ptemp=1;
                                                        general.repswa=1;     }
	      else if (strcasecmp(str_arg,"PTSTG")==0)  
                                                       {general.action_type=1;
                                                        general.ptstage=1;
                                                        general.ptemp=1;
                                                        general.stage=1;     }
	      else if (strcasecmp(str_arg,"PTSTGMC")==0)  
                                                       {general.action_type=1;
                                                        general.ptstage=1;
                                                        general.ptemp=1;
                                                        general.stage=1;     }
              else if (strcasecmp(str_arg,"NM")==0)    {general.action_type=1;
                                                        general.nmode=1;      }
              else if (strcasecmp(str_arg,"NMSTG")==0) {general.action_type=1;
                                                        general.nmode_stg=1;  
                                                        general.stage=1;      }
              else if (strcasecmp(str_arg,"NMEN")==0)  {general.action_type=1;
	                                                general.enm        =1;
                                                        general.nmode_enm  =1;}
              else if (strcasecmp(str_arg,"NMPT")==0)  {general.action_type=1;
                                                        general.nmpt=1;       }
              else if (strcasecmp(str_arg,"MIN")==0)   {general.action_type=1;
                                                        general.minimize=1;   }
              else if (strcasecmp(str_arg,"MD")==0)    {general.action_type=1;
                                                        general.md=1;   }
              else if (strcasecmp(str_arg,"DBFR")==0)  {general.action_type=1;
                                                        general.debug_force=1;
                                                        general.debug=1;      }
              else if (strcasecmp(str_arg,"DBTD")==0)
		                                   {general.debug_tors_defs=1;}
              else if (strcasecmp(str_arg,"DBFREN")==0){general.action_type=1;
                                                    general.debug_force_enm=1;
                                                    general.enm            =1;
                                                    general.debug          =1;}
              else if (strcasecmp(str_arg,"DBFRSTG")==0){general.action_type=1;
                                                 general.debug_force_stage =1;
						 general.stage             =1;
                                                 general.debug             =1;}
	      else if (strcasecmp(str_arg,"EEMC")==0)  {general.action_type=2;
                                                        general.eemc_type  =1;}
              else if (strcasecmp(str_arg,"EEMC_PT")==0){general.action_type=2;
                                                         general.eemc_type  =2;}
              else if (strcasecmp(str_arg,"SEQ_PT")==0){general.action_type=3;}
              else if (strcasecmp(str_arg,"SEQ_EEMC")==0){general.action_type=4;}
	      else { barf(key); }
	      break;
	    case 2:
              sscanf(arg, "%d", &int_arg);
	      general.n_chains_mc = int_arg;
	      break;	      
	    case 3:
              sscanf(arg, "%d", &int_arg);
	      general.nstep_mc = int_arg;
	      break;	      
	    case 4:
	      sscanf(arg, "%d", &int_arg);
	      general.nstep_md = int_arg;
	      break;
	    case 5:
	      sscanf(arg, "%lf", &real_arg);
	      real_arg /= TIME_CONV;
              general.dt = real_arg;
	      break; 
	    case 6:
	      sscanf(arg, "%d", &int_arg);
	      general.nstat = int_arg;
	      break;
	    case 7:
              sscanf(arg, "%lf", &real_arg);
	      general.pee = real_arg;
	      break;
            case 8:
              sscanf(arg, "%lf", &real_arg);
	      general.T = real_arg;
              general.text = real_arg;
	      break;
            case 9:
	      sscanf(arg, "%d", &int_arg);
	      general.mdisk = int_arg;
	      break;
            case 10:
	      sscanf(arg, "%d", &int_arg);
	      general.B = int_arg;
	      break;
            case 11:
	      sscanf(arg, "%d", &int_arg);
	      general.N = int_arg;
	      break;
            case 12:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "3bond_conn")==0) { 
                       general.inter_on_3bond_conn = 1; 
                       general.inter_on_4bond_conn = 1;
              }
	      else if (strcasecmp(str_arg, "4bond_conn")==0){ 
                       general.inter_on_3bond_conn = 0; 
                       general.inter_on_4bond_conn = 1;
              }
	      else if (strcasecmp(str_arg, "off")==0){ 
                       general.inter_on_3bond_conn = 0; 
                       general.inter_on_4bond_conn = 0;
              }            
	      else { barf(key); }
	      break;
	    case 13:
	      sscanf(arg, "%lf", &real_arg);
              energy_part.H_lower = real_arg;
	      break;
	    case 14:
	      sscanf(arg, "%lf", &real_arg);
              energy_part.H_upper = real_arg;
	      break;
            case 15:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "kcal")==0)
                      { general.write_energy_unit = 1;}
	      else if (strcasecmp(str_arg, "Ha")==0)
                      { general.write_energy_unit = 2;}
	      else { barf(key); }
	      break;
	    case 16:
	      sscanf(arg, "%lf", &real_arg);
              energy_part.energy_gap = real_arg;
	      break;
            case 17:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "cart")==0){ general.prop_type = 1;}
	      else if (strcasecmp(str_arg, "tors")==0){ general.prop_type = 2;}
	      else if (strcasecmp(str_arg, "bend")==0){ general.prop_type = 3;}
	      else { barf(key); }
	      break;
	    case 18:
	      sscanf(arg, "%lf", &real_arg);
              general.prop_tors_sig = real_arg;
	      break;
            case 19:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "local")==0){ 
                       general.cancel_res_inter_local = 1;
                       general.cancel_res_inter_neighbor = 0;
              }
	      else if (strcasecmp(str_arg, "neighbor")==0){ 
                       general.cancel_res_inter_local = 1;
                       general.cancel_res_inter_neighbor = 1;
              }
              else if (strcasecmp(str_arg, "off")==0){ 
                       general.cancel_res_inter_local = 0;
                       general.cancel_res_inter_neighbor = 0;
                      
              }
	      else { barf(key); }
	      break;
            case 20:
	      strcpy(str_arg, arg);
	      if   (strcasecmp(str_arg,"lnk_list")==0){ general.lnk_list = 1;}
	      else if (strcasecmp(str_arg,"neighbor_list")==0)
                                                 { general.neighbor_list = 1;}
	      else if (strcasecmp(str_arg, "none")==0){ general.lnk_list = 0;
                                                   general.neighbor_list = 0;}
	      else { barf(key); }
	      break;
            case 21:
	      sscanf(arg, "%d", &int_arg);
	      general.num_procs = int_arg;
	      break;
	    case 22:
              sscanf(arg, "%lf", &real_arg);
              real_arg /= BOHR;
	      inter.rswitch_length = real_arg;
	      break;
            case 23:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "full")==0)
                      { general.prop_tors_type = 1; }
	      else if (strcasecmp(str_arg, "side_chain")==0)
                      { general.prop_tors_type = 2; }
	      else { barf(key); }
	      break;
	    case 24:
              sscanf(arg, "%lf", &real_arg);
	      general.minimize_tol = real_arg;
	      break;
            case 25:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg,"cg")==0)
                      {general.minimize_type = 1;}
	      else if (strcasecmp(str_arg,"bfgs")==0)
                      {general.minimize_type = 2;}
	      else if (strcasecmp(str_arg,"samc")==0)
                      {general.minimize_type = 3;}
	      else if (strcasecmp(str_arg,"stsamc")==0)
                      {general.minimize_type = 4;}
	      else { barf(key); }
	      break;
            case 26:
	      sscanf(arg, "%d", &int_arg);
	      general.minimize_report = int_arg;
	      break;
            case 27:
	      sscanf(arg, "%d", &int_arg);
	      general.energy_report = int_arg;
	      break;
	    case 28:
	      sscanf(arg, "%lf", &real_arg);
              general.prop_rot_sig = real_arg;
	      break;
	    case 29:
	      sscanf(arg, "%lf", &real_arg);
              general.prop_trans_sig = real_arg/BOHR;
	      break;
            case 30:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "off")==0){ 
                       general.postprop_minimize = 0;
              }
	      else if (strcasecmp(str_arg, "cart")==0){
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 1;
              }
              else if (strcasecmp(str_arg, "tors")==0){ 
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 2;
              }
              else if (strcasecmp(str_arg, "bend")==0){ 
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 3;
              }
              else if (strcasecmp(str_arg, "bend_tors")==0){ 
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 4;
              }
              else if (strcasecmp(str_arg, "tors_bend")==0){ 
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 5;
              }
              else if (strcasecmp(str_arg, "clos")==0){ 
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 6;
              }
              else if (strcasecmp(str_arg, "gradclos_bend_tors")==0){ 
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 7;
              }
              else if (strcasecmp(str_arg, "side_chain")==0){ 
                       general.postprop_minimize = 1; 
	               general.postprop_minimize_type = 8;
              }
	      else { barf(key); }
	      break;
            case 31:
	      sscanf(arg, "%d", &int_arg);
	      general.postprop_minimize_itmax = int_arg;
	      break;
            case 32:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "bond")==0){ 
                       general.postprop_minimize_energy = 1;
              }
	      else if (strcasecmp(str_arg, "bond_bend")==0){
                       general.postprop_minimize_energy = 2; 
              }
              else if (strcasecmp(str_arg, "bond_bend_tors")==0){ 
                       general.postprop_minimize_energy = 3; 
              }
              else if (strcasecmp(str_arg, "bond_bend_tors_onfo")==0){ 
                       general.postprop_minimize_energy = 4; 
              }
              else if (strcasecmp(str_arg, "all")==0){ 
                       general.postprop_minimize_energy = 5; 
              }
	      else { barf(key); }
	      break;
	    case 33:
	      sscanf(arg, "%lf", &real_arg);
              general.prop_clos_sig = real_arg/BOHR;
	      break;
	    case 34:
	      sscanf(arg, "%lf", &real_arg);
              general.prop_notors_sig_scale = real_arg;
	      break;
            case 35:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "trans")==0){ 
                       general.prop_closcompl_type = 1;
              }
	      else if (strcasecmp(str_arg, "rot")==0){
                       general.prop_closcompl_type = 2;
              }
              else if (strcasecmp(str_arg, "trans_rot")==0){ 
                       general.prop_closcompl_type = 3;
              }
	      else { barf(key); }
	      break;
            case 36:
	      sscanf(arg, "%d", &int_arg);
	      general.postprop_minimize_gradclos_itmax = int_arg;
	      break;
            case 37:
	      sscanf(arg, "%d", &int_arg);
	      general.seed = int_arg;
	      break;
            case 38:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "none")==0){ 
                       general.scale_mass = 0;
              }
	      else if (strcasecmp(str_arg, "all")==0){
                       general.scale_mass = 1;
              }
              else if (strcasecmp(str_arg, "backbone")==0){ 
                       general.scale_mass = 2;
              }
	      else { barf(key); }
	      break;
	    case 39:
	      sscanf(arg, "%lf", &real_arg);
              general.scale_mass_value = real_arg;
	      break;
	    case 40:
	      sscanf(arg, "%lf", &real_arg);
              general.scale_mass_value_H = real_arg;
	      break;
            case 41:
	      sscanf(arg, "%d", &int_arg);
	      general.intra_respa_step = int_arg;
	      break;
	    case 42:
	      sscanf(arg, "%lf", &real_arg);
              general.sa_temp_first = real_arg;
	      break;
	    case 43:
	      sscanf(arg, "%lf", &real_arg);
              general.sa_temp_last = real_arg;
	      break;
	    case 44:
	      sscanf(arg, "%lf", &real_arg);
              general.sa_rate = real_arg;
	      break;
            case 45:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "trigonom")==0)
                      { general.stsamc_type = 1;}
	      else if (strcasecmp(str_arg, "linear")==0)
                      { general.stsamc_type = 2;}
	      else { barf(key); }
	      break;
            case 46:
	      sscanf(arg, "%d", &int_arg);
	      general.stsamc_period = int_arg;
	      break;
	    case 47:
	      sscanf(arg, "%lf", &real_arg);
              general.stsamc_ampl = real_arg;
	      break;
            case 48:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "intra_level")==0){ 
                       general.prop_DNA_closcompl_type = 1;
              }
	      else if (strcasecmp(str_arg, "inter_level")==0){
                       general.prop_DNA_closcompl_type = 2;
              }
              else if (strcasecmp(str_arg, "intra_inter_level")==0){ 
                       general.prop_DNA_closcompl_type = 3;
              }
	      else { barf(key); }
	      break;
            case 49:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "local")==0){ 
                       general.cancel_res_inter_replace_local = 1;
                       general.cancel_res_inter_replace_neighbor = 0;
              }
	      else if (strcasecmp(str_arg, "neighbor")==0){ 
                       general.cancel_res_inter_replace_local = 1;
                       general.cancel_res_inter_replace_neighbor = 1;
              }
              else if (strcasecmp(str_arg, "off")==0){ 
                       general.cancel_res_inter_replace_local = 0;
                       general.cancel_res_inter_replace_neighbor = 0;
                      
              }
	      else { barf(key); }
	      break;
	    case 50:
              sscanf(arg, "%lf", &real_arg);
              real_arg /= BOHR;
	      inter.neighbor_list_skin = real_arg;
	      break;
	    case 51:
              sscanf(arg, "%lf", &real_arg);
              real_arg /= BOHR;
	      inter.lnk_list_skin = real_arg;
	      break;
	    case 52:
	      sscanf(arg, "%lf", &real_arg);
              general.stsamc_shift = real_arg;
	      break;
	    case 53:
              sscanf(arg, "%lf", &real_arg);
              real_arg /= BOHR;
	      inter.rexclude_length = real_arg;
              inter.rexclude_length_defined = 1;
	      break;
            case 54:
	      strcpy(str_arg, arg);
	      if      (strcasecmp(str_arg, "superimpose")==0)
                      { general.prop_regions_type = 1; }
	      else if (strcasecmp(str_arg, "onebyone")==0)
                      { general.prop_regions_type = 2; }
	      else { barf(key); }
	      break;
	    }/*end switch*/
	  }/*end if */
	}/*end for */
	if (key_defined==0) {
	  sprintf(err_msg, "Key word %s unknown in input file line:\n%s",
		  key, entry_line);
	  errorMessage(err_msg);
	}
      }/*end while: getKeyArgPair */
      if (n!=0) {
	sprintf(err_msg, "Syntax error in input file line:\n%s",entry_line);
	errorMessage(err_msg);
      }
      for ( i=1; i<=nkey; i++) { 
	if (key_found[i]==0) {
	  sprintf(err_msg, "Key word %s missed in input file line:\n%s",
		  sim_gen_keys[i], entry_line);
	  errorMessage(err_msg);
	}
      }
    }/*end if: sim_gen_def*/

/*-------------------------------------------------------------------------*/
/* 2) ~sim_mol_def: alias ~system:                                         */
    else if ( (strcasecmp(entry_name, "sim_mol_def")==0) ||
              (strcasecmp(entry_name, "system")==0) ) {
      nkey = 37;
      for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
      key_found[1] = 1; key_found[2] = 1;                /*They have     */
      key_found[3] = 1; key_found[4] = 1;                /*default values*/
      /*temporarilly no default values*/
      key_found[1] = 0; key_found[2] = 0; 
      key_found[3] = 0; key_found[4] = 0;
      /*default is no control in design*/
      key_found[8] = 1; general.control_design = 0;
      /*default is all atom model      */  
      key_found[10] = 1; 
      general.cgres_model_KB = 0; 
      general.cgres_model_1pt = 0;   
      general.cgres_model_3pt = 0;
      general.cgres_model_5pt = 0;
      general.cgres_model_UA = 0; 
      general.cgres_model_UAPH = 0; 
      general.cgres_model_KB_UA = 0;
      general.cgres_model_KB_UAPH = 0;
      general.cgres_model_KB_5pt = 0;
      general.cgres_model_KB_3pt = 0;
      general.cgres_model_KB_1pt = 0;  
      general.cgres_model_Go  = 0;
      general.cgres_model_BNL = 0;
      general.cgres_model     = 0;
      general.allatm_model    = 1;
      /*default is not to write hessian*/
      key_found[16] = 1;
      general.write_hessian = 0;
      /*default is not to have implicit solvent*/
      key_found[17] = 1;
      inter.implicit_solvent = 0;
      inter.implicit_solvent_scp = 0;
      onfo.implicit_solvent = 0;
      onfo.implicit_solvent_scp = 0;
      /*default is not to write eighess*/
      key_found[18] = 1;
      general.write_eighess = 0;
      /*default is not to neutralize*/
      key_found[19] = 1;
      general.neutralize = 0;
      /*default is no distance dependent dielectric*/
      key_found[20] = 1;
      inter.ddd = 0;
      onfo.ddd = 0;
      /*default is D = 80.0*/
      key_found[21] = 1;
      inter.ddd_D = 80.0;
      onfo.ddd_D = 80.0;
      /*default is D0 = 4.0*/
      key_found[22] = 1;
      inter.ddd_D0 = 4.0;
      onfo.ddd_D0 = 4.0;
      /*default is S = 0.4 /Angstrom/^(-1) */
      key_found[23] = 1;
      inter.ddd_S = 0.4*BOHR;
      onfo.ddd_S = 0.4*BOHR;
      /*default is c = 2.5 /Angstrom */
      key_found[24] = 1;
      inter.ddd_c = 2.5/BOHR;
      onfo.ddd_c = 2.5/BOHR;
      /*default is e = 2.0 */
      key_found[25] = 1;
      inter.ddd_e = 2.0;
      onfo.ddd_e = 2.0;
     
      key_found[26] = 1;   /*default is \force_const_cutoff{off} */
        general.force_const_cutoff = 0;
      key_found[27] = 1;   /*default is \force_const_cutoff_bond{300000.0} */
        general.force_const_cutoff_bond = 300000.0;
        general.force_const_cutoff_bond *= BOHR*BOHR/BOLTZ;
      key_found[28] = 1;   /*default is \force_const_cutoff_bend{40000.0} */
        general.force_const_cutoff_bend = 40000.0;
        general.force_const_cutoff_bend /= BOLTZ;
      /*default is not to write eigvechess*/
      key_found[29] = 1;
      general.write_eigvechess = 0;
      /*default is no inter replace*/
      key_found[30] = 1;
      general.inter_replace = 0;
      /*default is KB_UA inter replace type*/
      key_found[31] = 1;
      general.inter_replace_KB_UA  = 1;
      general.inter_replace_KB_5pt = 0;
      /*default is not to add inter_nonpairwise*/
      key_found[32] = 1;
      general.inter_nprw = 0;
      inter.nprw         = 0;
      /*default is bond, bend, onfo, tors, inter, no cryo_em*/
      key_found[33] = 1;
      general.energy_term_bond  = 1;
      general.energy_term_bend  = 1;
      general.energy_term_tors  = 1;
      general.energy_term_onfo  = 1;
      general.energy_term_inter = 1;
      bond.energy_term = 1;
      bend.energy_term = 1;
      tors.energy_term = 1;
      onfo.energy_term = 1;
      inter.energy_term = 1;
      general.energy_term_cryo_em  = 0;     
      cryo_em.energy_term = 0;
      /*default is no cryo_em_database_file*/
      key_found[34] = 1;
      general.cryo_em_database_defined = 0;
      /*default is no cryo_em_expermnt_file*/
      key_found[35] = 1;
      general.cryo_em_expermnt_defined = 0;
      /*default is no region_database_file*/
      key_found[36] = 1;
      general.region_database_defined = 0;
      general.region_topology_defined = 0;
      /*default is no constraint_database_file*/
      key_found[37] = 1;
      general.constraint_database_defined = 0;


      while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
        key_defined = 0;
        for ( i=1; i<=nkey; i++) {                  /*loop for key words.*/
          if (strcasecmp(key, sim_mol_keys[i])==0) {/*it's a keyword.    */
            key_defined  = 1;
            key_found[i] = 1;
            switch(i) {
            case 1:
              strcpy(str_arg, arg);
              strcpy(filenames->mol_parm_filename, str_arg);
              break;
            case 2:
              strcpy(str_arg, arg);
              strcpy(filenames->bond_database_filename, str_arg);
              break;
            case 3:
              strcpy(str_arg, arg);
              strcpy(filenames->pos_init_filename, str_arg);
              break;
            case 4:
              strcpy(str_arg, arg);
              strcpy(filenames->pos_out_filename, str_arg);
              break;
            case 5:
              strcpy(str_arg, arg);
              strcpy(filenames->atom_pos_filename, str_arg);
              break;
            case 6:
              strcpy(str_arg, arg);
              strcpy(filenames->epot_filename, str_arg);
              break;
            case 7:
              strcpy(str_arg, arg);
              strcpy(filenames->einter_filename, str_arg);
              break;
            case 8:
              strcpy(str_arg, arg);
              strcpy(filenames->design_filename, str_arg);
              general.control_design = 1;
              break;
            case 9:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "primitive")==0){ 
                 general.system_definition = 1;
              }
	      else if (strcasecmp(str_arg, "residue")==0){ 
                 general.system_definition = 2;
              }
	      else { barf(key); }
              break;
            case 10:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "KB_1pt")==0){
                 general.cgres_model_KB = 1;
                 general.cgres_model_KB_1pt = 1;
                 general.cgres_model_1pt = 1; 
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "KB_3pt")==0){
                 general.cgres_model_KB = 1; 
                 general.cgres_model_KB_3pt = 1;
                 general.cgres_model_3pt = 1;  
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "KB_5pt")==0){
                 general.cgres_model_KB = 1; 
                 general.cgres_model_KB_5pt = 1;
                 general.cgres_model_5pt = 1;  
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "KB_UA")==0){
                 general.cgres_model_KB = 1; 
                 general.cgres_model_KB_UA = 1;
                 general.cgres_model_UA = 1; 
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "KB_UAPH")==0){
                 general.cgres_model_KB = 1; 
                 general.cgres_model_KB_UAPH = 1;
                 general.cgres_model_UAPH = 1; 
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "UA")==0){
                 general.cgres_model_UA = 1;
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "UAPH")==0){
                 general.cgres_model_UAPH = 1;
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "Go")==0){
                 general.cgres_model_Go  = 1;
                 general.cgres_model_1pt = 1;
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "BNL")==0){
                 general.cgres_model_BNL = 1;
                 general.cgres_model_1pt = 1;
                 general.cgres_model  = 1;
                 general.allatm_model = 0;
              }
	      else if (strcasecmp(str_arg, "off")==0){
                 general.allatm_model = 1;
              }
	      else { barf(key); }
              break;
            case 11:
              strcpy(str_arg, arg);
              strcpy(filenames->bend_database_filename, str_arg);
              break;
            case 12:
              strcpy(str_arg, arg);
              strcpy(filenames->tors_database_filename, str_arg);
              break;
            case 13:
              strcpy(str_arg, arg);
              strcpy(filenames->onfo_database_filename, str_arg);
              break;
            case 14:
              strcpy(str_arg, arg);
              strcpy(filenames->inter_database_filename, str_arg);
              break;
            case 15:
              strcpy(str_arg, arg);
              strcpy(filenames->tors_pos_filename, str_arg);
              break;
            case 16:
              strcpy(str_arg, arg);
              strcpy(filenames->hessian_filename, str_arg);
              general.write_hessian = 1;              
              break;
            case 17:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "scp")==0){ 
                 inter.implicit_solvent = 1;
                 inter.implicit_solvent_scp = 1;
                 onfo.implicit_solvent = 1;
                 onfo.implicit_solvent_scp = 1;
              }
	      else if (strcasecmp(str_arg, "off")==0){ 
                 inter.implicit_solvent = 0;
                 inter.implicit_solvent_scp = 0;
                 onfo.implicit_solvent = 0;
                 onfo.implicit_solvent_scp = 0;
              }
	      else { barf(key); }
              break;
            case 18:
              strcpy(str_arg, arg);
              strcpy(filenames->eighess_filename,str_arg);
              general.write_eighess = 1;              
              break;
            case 19:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "off")==0){ 
		 general.neutralize = 0;
              }
	      else if (strcasecmp(str_arg, "nucl")==0){ 
		 general.neutralize = 1;
              }
	      else if (strcasecmp(str_arg, "prot")==0){ 
		 general.neutralize = 2;
              }
	      else if (strcasecmp(str_arg, "all")==0){ 
		 general.neutralize = 3;
              }
	      else { barf(key); }
              break;
            case 20:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "DD0S")==0){ 
                 inter.ddd = 1;
                 onfo.ddd = 1;
              }
	      else if (strcasecmp(str_arg, "ce")==0){ 
                 inter.ddd = 2;
                 onfo.ddd = 2;
              }
	      else if (strcasecmp(str_arg, "off")==0){ 
                 inter.ddd = 0;
                 onfo.ddd = 0;
              }
	      else { barf(key); }
              break;
	    case 21:
              sscanf(arg, "%lf", &real_arg);
	      inter.ddd_D = real_arg;
              onfo.ddd_D = real_arg;
	      break;
	    case 22:
              sscanf(arg, "%lf", &real_arg);
	      inter.ddd_D0 = real_arg;
              onfo.ddd_D0 = real_arg;
	      break;
	    case 23:
              sscanf(arg, "%lf", &real_arg);
              real_arg   *= BOHR;
	      inter.ddd_S = real_arg;
              onfo.ddd_S = real_arg;
	      break;
	    case 24:
              sscanf(arg, "%lf", &real_arg);
              real_arg   /= BOHR;
	      inter.ddd_c = real_arg;
              onfo.ddd_c = real_arg;
	      break;
	    case 25:
              sscanf(arg, "%lf", &real_arg);
	      inter.ddd_e = real_arg;
              onfo.ddd_e = real_arg;
	      break;
            case 26:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "off")==0){ 
                 general.force_const_cutoff = 0;
              }
	      else if (strcasecmp(str_arg, "bond")==0){ 
                 general.force_const_cutoff = 1;
              }
	      else if (strcasecmp(str_arg, "bond_bend")==0){ 
                 general.force_const_cutoff = 2;
              }
	      else { barf(key); }
              break;
	    case 27:
              sscanf(arg, "%lf", &real_arg);
              real_arg *= BOHR*BOHR/BOLTZ;
	      general.force_const_cutoff_bond = real_arg;
	      break;
	    case 28:
              sscanf(arg, "%lf", &real_arg);
              real_arg /= BOLTZ;
	      general.force_const_cutoff_bend = real_arg;
	      break;
            case 29:
              strcpy(str_arg, arg);
              strcpy(filenames->eigvechess_filename,str_arg);
              general.write_eigvechess = 1;              
              break;
            case 30:
              strcpy(str_arg, arg);
              strcpy(filenames->inter_replace_database_filename, str_arg);
              general.inter_replace = 1;              
              break;
            case 31:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "KB_UA")==0){
                 general.inter_replace_KB_UA = 1;
              }
	      else if (strcasecmp(str_arg, "KB_5pt")==0){
                 general.inter_replace_KB_5pt = 1;
                 general.inter_replace_KB_UA  = 0;
              }
	      else { barf(key); }
              break;
            case 32:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "off")==0){ 
                 general.inter_nprw = 0;
                 inter.nprw = 0;
              }
	      else if (strcasecmp(str_arg, "ea")==0){ 
                 general.inter_nprw = 1;
                 inter.nprw = 1;
              }
	      else { barf(key); }
              break;
            case 33:
              strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "bond")==0){
                 general.energy_term_bond = 1;
                 bond.energy_term         = 1;
              }
	      else if (strcasecmp(str_arg, "bond_off")==0){
                 general.energy_term_bond = 0;
                 bond.energy_term         = 0;
              }
	      else if (strcasecmp(str_arg, "bend")==0){
                 general.energy_term_bend = 1;
                 bend.energy_term         = 1;
              }
	      else if (strcasecmp(str_arg, "bend_off")==0){
                 general.energy_term_bend = 0;
                 bend.energy_term         = 0;
              }
	      else if (strcasecmp(str_arg, "tors")==0){
                 general.energy_term_tors = 1;
                 tors.energy_term         = 1;
              }
	      else if (strcasecmp(str_arg, "tors_off")==0){
                 general.energy_term_tors = 0;
                 tors.energy_term         = 0;
              }
	      else if (strcasecmp(str_arg, "onfo")==0){
                 general.energy_term_onfo = 1;
                 onfo.energy_term         = 1;
              }
	      else if (strcasecmp(str_arg, "onfo_off")==0){
                 general.energy_term_onfo = 0;
                 onfo.energy_term         = 0;
              }
	      else if (strcasecmp(str_arg, "inter")==0){
                 general.energy_term_inter = 1;
                 inter.energy_term         = 1;
              }
	      else if (strcasecmp(str_arg, "inter_off")==0){
                 general.energy_term_inter = 0;
                 inter.energy_term         = 0;
              }
	      else if (strcasecmp(str_arg, "cryo_em")==0){
                 general.energy_term_cryo_em = 1;
                 cryo_em.energy_term         = 1;
              }
	      else if (strcasecmp(str_arg, "cryo_em_off")==0){
                 general.energy_term_cryo_em = 0;
                 cryo_em.energy_term         = 0;
              }
	      else if (strcasecmp(str_arg, "constraint")==0){
                 general.energy_term_constraint = 1;
                 constraint.energy_term         = 1;
              }
	      else if (strcasecmp(str_arg, "constraint_off")==0){
                 general.energy_term_constraint = 0;
                 constraint.energy_term         = 0;
              }
	      else { barf(key); }
              break;
            case 34:
              strcpy(str_arg, arg);
              strcpy(filenames->cryo_em_database_filename, str_arg);
              general.cryo_em_database_defined = 1;              
              break;
            case 35:
              strcpy(str_arg, arg);
              strcpy(filenames->cryo_em_expermnt_filename, str_arg);
              general.cryo_em_expermnt_defined = 1;              
              break;
            case 36:
              strcpy(str_arg, arg);
              strcpy(filenames->region_database_filename, str_arg);
              general.region_database_defined = 1;              
              general.region_topology_defined = 1; 
              break;
            case 37:
              strcpy(str_arg, arg);
              strcpy(filenames->constraint_database_filename, str_arg);
              general.constraint_database_defined = 1;              
              general.constraint_topology_defined = 1;
              break;
            }/*end switch*/
          }/*end if */
        }/*end for */
        if (key_defined==0) {
          sprintf(err_msg, "Key word %s unknown in input file line:\n%s",
                  key, entry_line);
          errorMessage(err_msg);
        }
      }/*end while: getKeyArgPair */
      if (n!=0) {
        sprintf(err_msg, "Syntax error in input file line:\n%s",entry_line);
        errorMessage(err_msg);
      }
      for ( i=1; i<=nkey; i++) {
        if (key_found[i]==0) {
          sprintf(err_msg, "Key word %s missed in input file line:\n%s",
                  sim_mol_keys[i], entry_line);
          errorMessage(err_msg);
        }
      }
    }/*end if: sim_mol_keys*/

/*-------------------------------------------------------------------------*/
/* 3) Error: not recodnized entry name*/
    else {
      sprintf(err_msg, "Entry name not defined in input file:\n%s",entry_name);
      errorMessage(err_msg);
    }
  }/*end while: readOneEntry*/

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void System::vomit_input(Filenames *filenames) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int  i,j;
  int  nkey;
  char *sim_gen_keys[]= {"",
			 "simulation_typ", "replica_number", "total_step_mc",
			 "local_step_md", "time_step_md", "statistics_freq",
                         "prob_eemc_jump", "temperature", "eemc_disk_size", 
                         "burn_in_B", "burn_in_N", "extend_inter",  
                         "EEMC_Emin", "EEMC_Emax", "write_energy_unit",
                         "energy_gap", "prop_type", "prop_tors_sig", 
                         "cancel_res_inter", "inter_list", "num_procs",
                         "rinter_switch_length", "prop_tors_type",
                         "minimize_tol", "minimize_type", "minimize_report",
                         "energy_report", "prop_rot_sig", "prop_trans_sig",  
                         "postprop_minimize", "postprop_minimize_itmax",
                         "postprop_minimize_energy", "prop_clos_sig", 
                         "prop_notors_sig_scale", "prop_closcompl_type",
                         "postprop_minimize_gradclos_itmax", "random_seed",
                         "scale_mass","scale_mass_value","scale_mass_value_H",
                         "intra_respa_step", "sa_temp_first", "sa_temp_last",
                         "sa_rate", "stsamc_type", "stsamc_period",
                         "stsamc_ampl", "prop_DNA_closcompl_type",
                         "cancel_res_inter_replace", "neighbor_list_skin",
                         "lnk_list_skin","stsamc_shift",
                         "rinter_exclude_length", "prop_regions_type"};
  char *sim_mol_keys[]= {"",
                         "mol_parm_file", "bond_database_file", 
                         "pos_init_file", "pos_out_file", "atom_pos_file", 
                         "epot_file", "einter_file", "design_file", 
                         "system_def", "cgres_model", "bend_database_file",
                         "tors_database_file", "onfo_database_file",
                         "inter_database_file", "tors_pos_file", 
                         "hessian_file", "implicit_solvent", "eighess_file",
                         "neutralize", "ddd", "ddd_D", "ddd_D0", "ddd_S", 
                         "ddd_c", "ddd_e", "force_const_cutoff", 
                         "force_const_cutoff_bond", "force_const_cutoff_bend",
                         "eigvechess_file", "inter_replace_database_file",
                         "inter_replace_type",  "inter_nonpairwise",
                         "energy_term", "cryo_em_database_file",
                         "cryo_em_expermnt_file","region_database_file",
                         "constraint_database_file"};
  FILE *output_file;

  output_file=fopen("sim_param.out", "w");

/*-------------------------------------------------------------------------*/
/* 1) ~sim_gen_def: alias ~method:                                         */
  fprintf(output_file,"~method[]\n");
  nkey = 54;
  for ( i=1; i<=nkey; i++) {                     /*loop for key words.*/
      switch (i) {
      case 1:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.action_type == 1) {
	  if (general.pt == 1)            {fprintf(output_file,"{PT} ");}
	  if (general.ptstage == 1)       {fprintf(output_file,"{PTSTG} ");}
	  if (general.ptrepswa == 1)      {fprintf(output_file,"{PTRPSW} ");}
          if (general.nmode == 1)         {fprintf(output_file,"{NM} ");}
          if (general.nmode_enm == 1)     {fprintf(output_file,"{NMEN} ");}
          if (general.nmpt  == 1)         {fprintf(output_file,"{NMPT} ");}
          if (general.minimize == 1)      {fprintf(output_file,"{MIN} ");}
          if (general.md == 1)            {fprintf(output_file,"{MD} ");}
          if (general.debug_force == 1)   {fprintf(output_file,"{DBFR} ");}
          if (general.debug_force_enm==1) {fprintf(output_file,"{DBFREN} ");}
          if (general.debug_force_stage==1){fprintf(output_file,"{DBFRSTG} ");}
        } 
        else if (general.action_type == 2) {
          fprintf(output_file, "{EEMC}");
        }
        else if (general.action_type == 3) {
          fprintf(output_file, "{SEQ_PT}");
        }
        else if (general.action_type == 4) {
          fprintf(output_file, "{SEQ_EEMC}");
        }
        fprintf(output_file,"{PT,PTMC,PTSTG,PTSTGMC,PTRPSW,PTRPSWMC,"
                            "NM,NMSTG,NMEN,NMPT,MIN,DBFR,DBTD,DBFREN,DBFRSTG,"
                            "EEMC,SEQ_PT,SEQ_EEMC}\n");
	break; 
      case 2:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.n_chains_mc);
        fprintf(output_file,"{integer>=0}\n");
        break;     
      case 3:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.nstep_mc);
        fprintf(output_file,"{integer>=1}\n");
        break;     
      case 4:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.nstep_md);
        fprintf(output_file,"{integer>=1}\n");
        break;     
      case 5:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",TIME_CONV*general.dt);
        fprintf(output_file,"{real>=0}\n");
        break;
      case 6:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.nstat);
        fprintf(output_file,"{integer>=1}\n");
        break;     
      case 7:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.pee);
        fprintf(output_file,"{real>=0}\n");
        break;     
      case 8:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.T);
        fprintf(output_file,"{real>=0}\n");
        break;     
      case 9:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.mdisk);
        fprintf(output_file,"{integer>=1}\n");
        break;     
      case 10:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.B);
        fprintf(output_file,"{integer>=0}\n");
        break;     
      case 11:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.N);
        fprintf(output_file,"{integer>=0}\n");
        break;     
      case 12:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.inter_on_3bond_conn==1) {
	  fprintf(output_file, "{3bond_conn} ");
        } else {
          if (general.inter_on_4bond_conn==1) {
	     fprintf(output_file, "{4bond_conn} ");
          } else {
             fprintf(output_file, "{off} ");
          }
        }
        fprintf(output_file,"{3bond_conn,4bond_conn,off}\n");
        break;
      case 13:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",energy_part.H_lower);
        fprintf(output_file,"{real}\n");
        break;
      case 14:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",energy_part.H_upper);
        fprintf(output_file,"{real}\n");
        break;     
      case 15:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.write_energy_unit == 1) {
	  fprintf(output_file, "{kcal} ");
        } 
        else if (general.write_energy_unit == 2) {
          fprintf(output_file, "{Ha} ");
        }
        fprintf(output_file,"{kcal,Ha}\n");
	break;
      case 16:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",energy_part.energy_gap);
        fprintf(output_file,"{real>=1.0}\n");
        break;
      case 17:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.prop_type == 1) {
	  fprintf(output_file, "{cart} ");
        } 
        else if (general.prop_type == 2) {
          fprintf(output_file, "{tors} ");
        }
        else if (general.prop_type == 3) {
          fprintf(output_file, "{bend} ");
        }
        fprintf(output_file,"{cart,tors,bend}\n");
	break;
      case 18:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.prop_tors_sig);
        fprintf(output_file,"{0 <= radian < 2 Pi}\n");
        break;
      case 19:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.cancel_res_inter_neighbor==1) {
	  fprintf(output_file, "{neighbor} ");
        } else {
          if (general.cancel_res_inter_local==1) {
	     fprintf(output_file, "{local} ");
          } else {
             fprintf(output_file, "{off} ");
          }
        }
        fprintf(output_file,"{neighbor,local,off}\n");
        break;
      case 20:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.lnk_list == 1) {
	  fprintf(output_file, "{lnk_list} ");
        } 
        else if (general.neighbor_list == 1) {
	  fprintf(output_file, "{neighbor_list} ");
        } 
        else if ((general.lnk_list == 0)&&(general.neighbor_list == 0)) {
          fprintf(output_file, "{none} ");
        }
        fprintf(output_file,"{lnk_list,neighbor_list,none}\n");
	break;
      case 21:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.num_procs);
        fprintf(output_file,"{integer>=1}\n");
        break;
      case 22:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",BOHR*inter.rswitch_length);
        fprintf(output_file,"{real>=0.0}\n");
        break;
      case 23:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.prop_tors_type == 1) {
	  fprintf(output_file, "{full} ");
        } 
        else if (general.prop_tors_type == 2) {
          fprintf(output_file, "{side_chain} ");
        }
        fprintf(output_file,"{full,side_chain}\n");
	break;
      case 24:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.minimize_tol);
        fprintf(output_file,"{real>0.0}\n");
        break;
      case 25:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.minimize_type == 1) {
	  fprintf(output_file, "{cg} ");
        } 
        else if (general.minimize_type == 2) {
          fprintf(output_file, "{bfgs} ");
        }
        else if (general.minimize_type == 3) {
          fprintf(output_file, "{samc} ");
        }
        else if (general.minimize_type == 4) {
          fprintf(output_file, "{stsamc} ");
        }
        fprintf(output_file,"{cg,bfgs,samc,stsamc}\n");
	break;
      case 26:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.minimize_report);
        fprintf(output_file,"{0,1,2}\n");
        break;
      case 27:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.energy_report);
        fprintf(output_file,"{0,1,2}\n");
        break;
      case 28:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.prop_rot_sig);
        fprintf(output_file,"{0 <= radian < 2 Pi}\n");
        break;
      case 29:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",(general.prop_trans_sig)*BOHR);
        fprintf(output_file,"{Angstrom >= 0.0}\n");
        break;
      case 30:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.postprop_minimize == 0) {
	  fprintf(output_file, "{off} ");
        } 
        else if (general.postprop_minimize == 1) {
	  if (general.postprop_minimize_type==1) {
	      fprintf(output_file, "{cart} ");
          }
	  else if (general.postprop_minimize_type==2) {
	      fprintf(output_file, "{tors} ");
          }
	  else if (general.postprop_minimize_type==3) {
	      fprintf(output_file, "{bend} ");
          }
	  else if (general.postprop_minimize_type==4) {
	      fprintf(output_file, "{bend_tors} ");
          }
	  else if (general.postprop_minimize_type==5) {
	      fprintf(output_file, "{tors_bend} ");
          }
	  else if (general.postprop_minimize_type==6) {
	      fprintf(output_file, "{clos} ");
          }
	  else if (general.postprop_minimize_type==7) {
	      fprintf(output_file, "{gradclos_bend_tors} ");
          }
	  else if (general.postprop_minimize_type==8) {
	      fprintf(output_file, "{side_chain} ");
          }
        }
        fprintf(output_file,"{off,cart,tors,bend,bend_tors,tors_bend,clos,"
                            "gradclos_bend_tors,side_chain}\n");
	break;
      case 31:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.postprop_minimize_itmax);
        fprintf(output_file,"{integer >= 0}\n");
        break;
      case 32:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
	  if (general.postprop_minimize_energy==1) {
	      fprintf(output_file, "{bond} ");
          }
	  else if (general.postprop_minimize_energy==2) {
	      fprintf(output_file, "{bond_bend} ");
          }
	  else if (general.postprop_minimize_energy==3) {
	      fprintf(output_file, "{bond_bend_tors} ");
          }
	  else if (general.postprop_minimize_energy==4) {
	      fprintf(output_file, "{bond_bend_tors_onfo} ");
          }
	  else if (general.postprop_minimize_energy==5) {
	      fprintf(output_file, "{all} ");
          }
        fprintf(output_file,"{bond,bond_bend,bond_bend_tors,"
                            "bond_bend_tors_onfo,all}\n");
	break;
      case 33:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",(general.prop_clos_sig)*BOHR);
        fprintf(output_file,"{Angstrom >= 0.0}\n");
        break;
      case 34:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.prop_notors_sig_scale);
        fprintf(output_file,"{real >= 0.0}\n");
        break;
      case 35:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
	  if (general.prop_closcompl_type==1) {
	      fprintf(output_file, "{trans} ");
          }
	  else if (general.prop_closcompl_type==2) {
	      fprintf(output_file, "{rot} ");
          }
	  else if (general.prop_closcompl_type==3) {
	      fprintf(output_file, "{trans_rot} ");
          }
        fprintf(output_file,"{trans,rot,trans_rot}\n");
	break;
      case 36:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.postprop_minimize_gradclos_itmax);
        fprintf(output_file,"{integer >= 0}\n");
        break;
      case 37:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.seed);
        fprintf(output_file,"{large integer}\n");
        break;
      case 38:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
	  if (general.scale_mass==0) {
	      fprintf(output_file, "{none} ");
          }
	  else if (general.scale_mass==1) {
	      fprintf(output_file, "{all} ");
          }
	  else if (general.scale_mass==2) {
	      fprintf(output_file, "{backbone} ");
          }
        fprintf(output_file,"{none,all,backbone}\n");
	break;
      case 39:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.scale_mass_value);
        fprintf(output_file,"{real > 0.0}\n");
        break;
      case 40:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.scale_mass_value_H);
        fprintf(output_file,"{real > 0.0}\n");
        break;
      case 41:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.intra_respa_step);
        fprintf(output_file,"{integer>=1}\n");
        break;
      case 42:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.sa_temp_first);
        fprintf(output_file,"{real > 0.0, in K}\n");
        break;  
      case 43:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.sa_temp_last);
        fprintf(output_file,"{real > 0.0, in K}\n");
        break;  
      case 44:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.sa_rate);
        fprintf(output_file,"{real > 0.0}\n");
        break;
      case 45:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.stsamc_type == 1) {
	  fprintf(output_file, "{trigonom} ");
        } 
        else if (general.stsamc_type == 2) {
          fprintf(output_file, "{linear} ");
        }
        fprintf(output_file,"{trigonom,linear}\n");
	break;
      case 46:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%d} ",general.stsamc_period);
        fprintf(output_file,"{integer >= 100, # of mc steps}\n");
        break;
      case 47:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.stsamc_ampl);
        fprintf(output_file,"{real > 0.0, in K}\n");
        break;
      case 48:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
	  if (general.prop_DNA_closcompl_type==1) {
	      fprintf(output_file, "{intra_level} ");
          }
	  else if (general.prop_DNA_closcompl_type==2) {
	      fprintf(output_file, "{inter_level} ");
          }
	  else if (general.prop_DNA_closcompl_type==3) {
	      fprintf(output_file, "{intra_inter_level} ");
          }
        fprintf(output_file,"{intra_level,inter_level,intra_inter_level}\n");
	break;
      case 49:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.cancel_res_inter_replace_neighbor==1) {
	  fprintf(output_file, "{neighbor} ");
        } else {
          if (general.cancel_res_inter_replace_local==1) {
	     fprintf(output_file, "{local} ");
          } else {
             fprintf(output_file, "{off} ");
          }
        }
        fprintf(output_file,"{neighbor,local,off}\n");
        break;
      case 50:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",BOHR*inter.neighbor_list_skin);
        fprintf(output_file,"{real>=0.0}\n");
        break;
      case 51:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",BOHR*inter.lnk_list_skin);
        fprintf(output_file,"{real>=0.0}\n");
        break;
      case 52:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",general.stsamc_shift);
        fprintf(output_file,"{real > 0.0, in K}\n");
        break;
      case 53:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        fprintf(output_file,"{%g} ",BOHR*inter.rexclude_length);
        fprintf(output_file,"{real>=0.0}\n");
        break;
      case 54:
        fprintf(output_file,"%s\t",sim_gen_keys[i]);
        if (general.prop_regions_type == 1) {
	  fprintf(output_file, "{superimpose} ");
        } 
        else if (general.prop_regions_type == 2) {
          fprintf(output_file, "{onebyone} ");
        }
        fprintf(output_file,"{superimpose,onebyone}\n");
	break;
      }
  }
  fprintf(output_file,"\n");
/*-------------------------------------------------------------------------*/
/* 2) ~sim_mol_def: alias ~system:                                         */
  fprintf(output_file,"~system[]\n");
  nkey = 37;
  for ( i=1; i<=nkey; i++) {                     /*loop for key words.*/
      switch (i) {
      case 1:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->mol_parm_filename);
        fprintf(output_file,"{filename}\n");
        break;     
      case 2:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->bond_database_filename);
        fprintf(output_file,"{filename}\n");
        break;     
      case 3:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->pos_init_filename);
        fprintf(output_file,"{filename}\n");
        break;     
      case 4:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->pos_out_filename);
        fprintf(output_file,"{filename}\n");
        break;     
      case 5:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->atom_pos_filename);
        fprintf(output_file,"{filename}\n");
        break;     
      case 6:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->epot_filename);
        fprintf(output_file,"{filename}\n");
        break;     
      case 7:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->einter_filename);
        fprintf(output_file,"{filename}\n");
        break;     
      case 8:
        if (general.control_design == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",
                                  filenames->design_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 9:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.system_definition == 1) {
	  fprintf(output_file, "{primitive} ");
        } 
        else if (general.system_definition == 2) {
          fprintf(output_file, "{residue} ");
        }
        fprintf(output_file,"{primitive,residue}\n");
	break;
      case 10:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.cgres_model == 1) {
            if (general.cgres_model_KB_1pt == 1) {
	        fprintf(output_file, "{KB_1pt} ");
            }
            else if (general.cgres_model_KB_3pt == 1) {
	        fprintf(output_file, "{KB_3pt} ");
            }
            else if (general.cgres_model_KB_5pt == 1) {
	        fprintf(output_file, "{KB_5pt} ");
            }
            else if (general.cgres_model_KB_UA == 1) {
	        fprintf(output_file, "{KB_UA} ");
            }
            else if (general.cgres_model_KB_UAPH == 1) {
	        fprintf(output_file, "{KB_UAPH} ");
            }
            else if (general.cgres_model_UA == 1) {
	        fprintf(output_file, "{UA} ");
            }
            else if (general.cgres_model_UAPH == 1) {
	        fprintf(output_file, "{UAPH} ");
            }
            else if (general.cgres_model_Go == 1) {
	        fprintf(output_file, "{Go} ");
            }
            else if (general.cgres_model_BNL == 1) {
	        fprintf(output_file, "{BNL} ");
            } 
        }
        else if (general.cgres_model == 0) {
          fprintf(output_file, "{off} ");
        }
        fprintf(output_file,
               "{off,KB_1pt,KB_3pt,KB_5pt,KB_UA,KB_UAPH,UA,UAPH,Go,BNL}\n");
	break;
      case 11:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->bend_database_filename);
        fprintf(output_file,"{filename}\n");
        break;
      case 12:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->tors_database_filename);
        fprintf(output_file,"{filename}\n");
        break;
      case 13:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->onfo_database_filename);
        fprintf(output_file,"{filename}\n");
        break;
      case 14:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->inter_database_filename);
        fprintf(output_file,"{filename}\n");
        break;
      case 15:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%s} ",filenames->tors_pos_filename);
        fprintf(output_file,"{filename}\n");
        break;
      case 16:
        if (general.write_hessian == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",filenames->hessian_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ",filenames->hessian_filename);
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 17:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (inter.implicit_solvent == 1) {
           if (inter.implicit_solvent_scp == 1) {
               fprintf(output_file, "{scp} ");
           }
        } 
        else if (inter.implicit_solvent == 0) {
           fprintf(output_file, "{off} ");
        }
        fprintf(output_file,"{off,scp}\n");
	break;
      case 18:
        if (general.write_eighess == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",filenames->eighess_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 19:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.neutralize == 0) {
           fprintf(output_file, "{off} ");
        }
        else if (general.neutralize == 1) {
           fprintf(output_file, "{nucl} ");
        }
        else if (general.neutralize == 2) {
           fprintf(output_file, "{prot} ");
        } 
        else if (general.neutralize == 3) {
           fprintf(output_file, "{all} ");
        } 
        fprintf(output_file,"{off,nucl,prot,all}\n");
	break;
      case 20:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (inter.ddd == 0) {
           fprintf(output_file, "{off} ");
        }
        else if (inter.ddd == 1) {
           fprintf(output_file, "{DD0S} ");
        } 
        else if (inter.ddd == 2) {
           fprintf(output_file, "{ce} ");
        } 
        fprintf(output_file,"{off,DD0S,ce}\n");
	break;
      case 21:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%g} ",inter.ddd_D);
        fprintf(output_file,"{real >= 0.0}\n");
        break;
      case 22:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%g} ",inter.ddd_D0);
        fprintf(output_file,"{real >= 0.0}\n");
        break;
      case 23:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%g} ",(inter.ddd_S)/BOHR);
        fprintf(output_file,"{real >= 0.0}\n");
        break;
      case 24:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%g} ",(inter.ddd_c)*BOHR);
        fprintf(output_file,"{real >= 0.0}\n");
        break;
      case 25:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%g} ",inter.ddd_e);
        fprintf(output_file,"{real >= 0.0}\n");
        break;
      case 26:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.force_const_cutoff == 0) {
           fprintf(output_file, "{off} ");
        }
        else if (general.force_const_cutoff == 1) {
           fprintf(output_file, "{bond} ");
        } 
        else if (general.force_const_cutoff == 2) {
           fprintf(output_file, "{bond_bend} ");
        } 
        fprintf(output_file,"{off,bond,bond_bend}\n");
	break;
      case 27:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%g} ",(general.force_const_cutoff_bond)
                                                    *BOLTZ/BOHR/BOHR);
        fprintf(output_file,"{real >= 0.0, in K/A^2}\n");
        break;
      case 28:
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        fprintf(output_file,"{%g} ",(general.force_const_cutoff_bend)
                                                    *BOLTZ);
        fprintf(output_file,"{real >= 0.0, in K/rad^2}\n");
        break;
      case 29:
        if (general.write_eigvechess == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",filenames->eigvechess_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 30:
        if (general.inter_replace == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",
                                  filenames->inter_replace_database_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 31:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.inter_replace_KB_UA == 1) {
	        fprintf(output_file, "{KB_UA} ");
        }
        if (general.inter_replace_KB_5pt == 1) {
	        fprintf(output_file, "{KB_5pt} ");
        }
        fprintf(output_file,
                        "{KB_UA,KB_5pt}\n");
	break;
      case 32:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.inter_nprw == 0) {
           fprintf(output_file, "{off} ");
        }
        else if (general.inter_nprw == 1) {
           fprintf(output_file, "{ea} ");
        } 
        fprintf(output_file,"{off,ea}\n");
	break;
      case 33:     
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.energy_term_bond == 1) {
           fprintf(output_file, "{bond} ");
        }
        if (general.energy_term_bond == 0) {
           fprintf(output_file, "{bond_off} ");
        }
        fprintf(output_file, "{bond,bond_off}\n");
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.energy_term_bend == 1) {
           fprintf(output_file, "{bend} ");
        }
        if (general.energy_term_bend == 0) {
           fprintf(output_file, "{bend_off} ");
        }
        fprintf(output_file, "{bend,bend_off}\n");
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.energy_term_tors == 1) {
           fprintf(output_file, "{tors} ");
        }
        if (general.energy_term_tors == 0) {
           fprintf(output_file, "{tors_off} ");
        }
        fprintf(output_file, "{tors,tors_off}\n");
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.energy_term_onfo == 1) {
           fprintf(output_file, "{onfo} ");
        }
        if (general.energy_term_onfo == 0) {
           fprintf(output_file, "{onfo_off} ");
        }
        fprintf(output_file, "{onfo,onfo_off}\n");
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.energy_term_inter == 1) {
           fprintf(output_file, "{inter} ");
        }
        if (general.energy_term_inter == 0) {
           fprintf(output_file, "{inter_off} ");
        }
        fprintf(output_file, "{inter,inter_off}\n");
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.energy_term_cryo_em == 1) {
           fprintf(output_file, "{cryo_em} ");
        }
        if (general.energy_term_cryo_em == 0) {
           fprintf(output_file, "{cryo_em_off} ");
        } 
        fprintf(output_file, "{cryo_em,cryo_em_off}\n");
        fprintf(output_file,"%s\t",sim_mol_keys[i]);
        if (general.energy_term_constraint == 1) {
           fprintf(output_file, "{constraint} ");
        }
        if (general.energy_term_constraint == 0) {
           fprintf(output_file, "{constraint_off} ");
        } 
        fprintf(output_file, "{constraint,constraint_off}\n");
	break;
      case 34:
        if (general.cryo_em_database_defined == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",
                                  filenames->cryo_em_database_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 35:
        if (general.cryo_em_expermnt_defined == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",
                                  filenames->cryo_em_expermnt_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 36:
        if (general.region_database_defined == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",
                                  filenames->region_database_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      case 37:
        if (general.constraint_database_defined == 1) {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{%s} ",
                                  filenames->constraint_database_filename);
            fprintf(output_file,"{filename}\n");
        } else {
            fprintf(output_file,"%s\t",sim_mol_keys[i]);
            fprintf(output_file,"{not defined} ");
            fprintf(output_file,"{filename}\n");
        }
        break;
      }
  }

  fclose(output_file);
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
  
