#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
void Pos::setup( Maps *maps, Restop_data *restop_database, 
                             Res_data_entry *res_database, General *general)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm, iatm_bb, iiatm, ires, iires, jres, irtfres;
  int one, ires1, ires2;
  int iresbond;
  int nbond, nbend, ntors, nimpr, nonfo, nexclude;  
  int ibond, ibend, itors, iimpr, ionfo, iexclude;
  int oneres, natm_1stres;
  int **bond_res;
  int *atm_res_id;
  int res_found;
  double xmin, ymin, zmin;
  double xmax, ymax, zmax;
  double scale_chrg, one_chrg;
  double res_chrg, res_chrg_pos, res_chrg_neg;
  char **atm_label, **upac_atm_label, **res_label;


  int    imol;
  int   *nres_mol;
  int   *npro_mol;
  int  **ind_res_mol;
  int   *res_mol_id;        
  int   *ind_atm_bb;
  char **mol_label, **mol_chainid;
  int CA_found, O_found, C4p_found, P_found, O2p_found, DUM_found;        
  int OH2_found, H1_found, H2_found;
  int ind_O1P, ind_O2P;

  int ind_N, ind_CA, ind_C;      
  int ind_P, ind_O5p, ind_C5p, ind_C4p, ind_C3p, ind_O3p;

  char   err_msg[LINE_NCHARMAX];  

/*-------------------------------------------------------------------------*/
/* I) Allocate variables in maps, pos                                      */

  residue = new Residue[nres] - 1;
  natom = 0;
  for (ires=1;ires<=nres;ires++) {
      natom += res_database[ires].natom;
  }/*end for*/  
  atm_label            = (char  **)cmall_int_mat(1, natom, 0, NNAMECHARMAX);
  upac_atm_label       = (char  **)cmall_int_mat(1, natom, 0, NNAMECHARMAX);
  res_label            = (char  **)cmall_int_mat(1, nres,  0, NNAMECHARMAX);
  atm_res_id           =  new int[natom]-1;
  maps->atm_label      = atm_label;
  maps->upac_atm_label = upac_atm_label;
  maps->res_label      = res_label;
  maps->atm_res_id     = atm_res_id;

  mass  = new double[natom]-1;
  chrg  = new double[natom]-1;  
  x     = new double[natom]-1;
  y     = new double[natom]-1;
  z     = new double[natom]-1;
  xi    = new double[natom]-1;
  yi    = new double[natom]-1;
  zi    = new double[natom]-1;
  vx    = new double[natom]-1;
  vy    = new double[natom]-1;
  vz    = new double[natom]-1;
  fx    = new double[natom]-1;
  fy    = new double[natom]-1;
  fz    = new double[natom]-1;

  if (general->neighbor_list==1) {
      x0     = new double[natom]-1;
      y0     = new double[natom]-1;
      z0     = new double[natom]-1;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* II) Set variables, Maps, Pos, General                                   */

  maps->natm_tot      = natom;
  maps->nres          = nres;
  maps->lnk_list      = general->lnk_list;
  maps->neighbor_list = general->neighbor_list;
  action_type         = general->action_type;
  
  allatm_model       = general->allatm_model;
  cgres_model_KB_UA  = general->cgres_model_KB_UA;
  cgres_model_KB_5pt = general->cgres_model_KB_5pt;               
  cgres_model_KB_3pt = general->cgres_model_KB_3pt;               
  cgres_model_KB_1pt = general->cgres_model_KB_1pt;               
  cgres_model_Go     = general->cgres_model_Go;                   
  cgres_model_BNL    = general->cgres_model_BNL;
  cgres_model_KB     = general->cgres_model_KB;
  cgres_model        = general->cgres_model; 
  cgres_model_1pt    = general->cgres_model_1pt;                    
  cgres_model_3pt    = general->cgres_model_3pt;               
  cgres_model_5pt    = general->cgres_model_5pt;               
  cgres_model_UA     = general->cgres_model_UA;               
             
  natm_tot          = natom;
  general->deg_free = 3*natm_tot;

/*-------------------------------------------------------------------------*/
/* III) Setup residue                                                      */
/*      + Fill in maps: atm_label                                          */
/*                pos:  mass, chrg, x, y, z                                */
  
  for (ires=1;ires<=nres;ires++) {
      res_found = 0;
      for (irtfres=1;irtfres<=nrtfres;irtfres++) {
	  if (strcasecmp(res_database[ires].residue_label, 
                         restop_database[irtfres].residue_name)==0) {
	                 res_found = 1;
                         strcpy(maps->res_label[ires],
                                            res_database[ires].residue_label);
	                 residue[ires].setup(maps,&(restop_database[irtfres]),
                                                  &(res_database[ires]),this);
                         break;
          }/*end if*/   
      }/*end for*/
      if (res_found==0) {
         sprintf(err_msg,"%s %d was not found in .rtf",
                 res_database[ires].residue_label,
                 res_database[ires].residue_number_old);
         errorMessage(err_msg);
      }/*end if*/
  }/*end for*/  

/*-------------------------------------------------------------------------*/
/* IV) Scale to atomic units                                               */
/*     Get boxl, boxw, boxh as the minimum expansion in x, y, z            */
  
  for (iatm=1;iatm<=natom;iatm++) {
      mass[iatm] *= PROT_MASS;
      x[iatm]    /= BOHR;
      y[iatm]    /= BOHR;
      z[iatm]    /= BOHR;
  }/*end for*/
  
  xmax = x[1]; xmin = x[1];
  ymax = y[1]; ymin = y[1];
  zmax = z[1]; zmin = z[1];
  for (iatm=2;iatm<=natom;iatm++) {
       xmax = MAX(x[iatm],xmax); xmin = MIN(x[iatm],xmin);
       ymax = MAX(y[iatm],ymax); ymin = MIN(y[iatm],ymin);
       zmax = MAX(z[iatm],zmax); zmin = MIN(z[iatm],zmin);
  }/*end for*/

  boxl = xmax - xmin;
  boxw = ymax - ymin;
  boxh = zmax - zmin; 

/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/* V) Get the list of residues to bond                                     */
/*    Assume that only residues with neighboring res indices might bond    */ 
/*    Conditions:                                                          */ 
/*    1) same chain_id;                                                    */ 
/*    2) neighboring res_number;                                           */
/*    3) for both residues nbranch_root > 0                                */

  nresbond = 0;
  bond_res = cmall_int_mat(1, nres, 1, 2); 
  for (ires=1;ires<=nres-1;ires++) {
       jres = ires + 1;
       if (  (strcasecmp(res_database[ires].chain_id,
                         res_database[jres].chain_id)==0) && 
             (res_database[jres].residue_number_old-
              res_database[ires].residue_number_old==1)   &&
	     ( (residue[ires].nbranch_root>0) && 
               (residue[jres].nbranch_root>0)  )         ) {
	      nresbond++;
              bond_res[nresbond][1] = ires;
              bond_res[nresbond][2] = jres;
       }/*end if*/
  }/*end for*/
  maps->nresbond  =  nresbond; 

/*-------------------------------------------------------------------------*/
/* VI A) Count the number of molecules: nmol                               */
/*       Residues within a molecule: nres_mol                              */
/*       Proline residues within a molecule: npro_mol                      */
/*       Residue indices within the molecule: ind_res_mol                  */
/*       Allocate and Define all the above                                 */

  nmol        = nres - nresbond;
  nres_mol    = new int[nmol]-1;
  npro_mol    = new int[nmol]-1;
  res_mol_id  = new int[nres]-1;
  mol_label   = (char **)cmall_int_mat(1, nmol, 0, NNAMECHARMAX);
  mol_chainid = (char **)cmall_int_mat(1, nmol, 0, NNAMECHARMAX);
  ind_res_mol = cmall_int_mat(1,nmol,1,nres);  
 
  for (imol=1;imol<=nmol;imol++) {nres_mol[imol] = 0;
                                  npro_mol[imol] = 0;}

  imol = 1;
  iresbond = 1;
  nres_mol[imol] = 1;
  for (ires=1;ires<=nres-1;ires++) {
       jres = ires + 1;
       if (  (ires==bond_res[iresbond][1])&&
             (jres==bond_res[iresbond][2])   ) {
	      nres_mol[imol]++;
	      iresbond++;
       } else {
     	      imol++;
              nres_mol[imol]++;
       }/*endif*/
  }/*end for*/

  ires = 0;
  for (imol=1;imol<=nmol;imol++) {

       if (nres_mol[imol]==1) {oneres = 1;} else {oneres = 0;}
       for (jres=1;jres<=nres_mol[imol];jres++) {
	    ires++;
            res_mol_id[ires] = imol;
            ind_res_mol[imol][jres] = ires;
            if (strcasecmp(residue[ires].residue_name,"PRO")==0) {
	        npro_mol[imol]++;
            }/*endif*/      
       }/*end for*/

       CA_found  = 0;
       O_found   = 0;
       C4p_found = 0;
       P_found   = 0;
       O2p_found = 0;
       OH2_found = 0;
       H1_found  = 0;
       H2_found  = 0;
       DUM_found = 0;
       natm_1stres = residue[ind_res_mol[imol][1]].natom;
       for (iatm=1;iatm<=natm_1stres;iatm++) {
         iiatm = residue[ind_res_mol[imol][1]].atom[iatm];
	 if (strcasecmp(maps->upac_atm_label[iiatm],"CA")==0) {CA_found = 1; }
         if (strcasecmp(maps->upac_atm_label[iiatm],"O")==0)  {O_found = 1;  }
         if (strcasecmp(maps->upac_atm_label[iiatm],"C4'")==0){C4p_found = 1;}
         if (strcasecmp(maps->upac_atm_label[iiatm],"P")==0)  {P_found = 1;  }
         if (strcasecmp(maps->upac_atm_label[iiatm],"O2'")==0){O2p_found = 1;}
         if (strcasecmp(maps->upac_atm_label[iiatm],"OH2")==0){OH2_found = 1;}
         if (strcasecmp(maps->upac_atm_label[iiatm],"H1")==0) {H1_found = 1; }
         if (strcasecmp(maps->upac_atm_label[iiatm],"H2")==0) {H2_found = 1; }
         if (strcasecmp(maps->upac_atm_label[iiatm],"DUM")==0){DUM_found = 1;}
       }/*endfor*/
       if ( (CA_found==1) && (O_found==1) ) {
            strcpy(mol_label[imol],"protein");
       } else if ( (P_found==1) && (C4p_found==1) ) {/*nucleic*/
	    if (O2p_found==1) strcpy(mol_label[imol],"RNA");
            if (O2p_found==0) strcpy(mol_label[imol],"DNA");
       } else if ( (OH2_found == 0) &&
                   (H1_found  == 0) && 
                   (H2_found  == 0) && 
                   (oneres == 1   ) &&  
                   (natm_1stres==3) ) {/*water*/
            strcpy(mol_label[imol],"water");
       } else if ( (DUM_found == 0) && 
                   (oneres == 1   ) &&  
                   (natm_1stres==1) ) {/*atom/ion*/
            strcpy(mol_label[imol],"atom/ion");
       } else if ( (DUM_found==1) ) {/*dummy*/
            strcpy(mol_label[imol],"DUM");
       } else  {
            strcpy(mol_label[imol],"other");
       }/*endif*/
       strcpy(mol_chainid[imol],res_database[ind_res_mol[imol][1]].chain_id);
  }/*end for*/

  maps->nmol        = nmol;
  maps->nres_mol    = nres_mol;
  maps->npro_mol    = npro_mol;
  maps->res_mol_id  = res_mol_id;
  maps->ind_res_mol = ind_res_mol;        
  maps->mol_label   = mol_label;          
  maps->mol_chainid = mol_chainid;          

/*-------------------------------------------------------------------------*/
/* VI B) Neutralize molecules if required                                  */
/*       Only if all atom model is present                                 */

  if ( general->allatm_model == 1 ) {/*all atom model only*/

   if ((general->neutralize ==1)||(general->neutralize ==3)){/*nucleic acids*/
     for (imol=1;imol<=nmol;imol++) {
       if ( (strcasecmp(mol_label[imol],"RNA")==0) ||
            (strcasecmp(mol_label[imol],"DNA")==0) ) {
             for (jres=1;jres<=nres_mol[imol];jres++) {
	       ires =  ind_res_mol[imol][jres]; 
               for (iatm=1;iatm<=residue[ires].natom;iatm++) {
                 iiatm = residue[ires].atom[iatm];
		 if (strcasecmp(maps->upac_atm_label[iiatm],"O1P")==0) {
		     ind_O1P = residue[ires].atom[iatm];
 		 }/*endif*/
                 if (strcasecmp(maps->upac_atm_label[iiatm],"O2P")==0) {
		     ind_O2P = residue[ires].atom[iatm];
 		 }/*endif*/  
               }/*endfor-iatm*/
               one_chrg = 1.0;
               scale_chrg = (chrg[ind_O1P]+chrg[ind_O2P]+one_chrg)/
                            (chrg[ind_O1P]+chrg[ind_O2P]);
               chrg[ind_O1P] *= scale_chrg;
               chrg[ind_O2P] *= scale_chrg;   
             }/*endfor-jres*/
       }/*endif*/
     }/*endfor-imol*/
   }/*end if*/

   if ( (general->neutralize == 2) || (general->neutralize == 3) ) {

       for (imol=1;imol<=nmol;imol++) {
        for (ires=1;ires<=maps->nres_mol[imol];ires++) {
         iires = maps->ind_res_mol[imol][ires];

         res_chrg = 0.0;
         for (iatm=1;iatm<=residue[iires].natom;iatm++) {
              iiatm = residue[iires].atom[iatm];
              res_chrg += chrg[iiatm];
         }/*end for-iatm*/

         if (res_chrg > 0.0) {
	    res_chrg_pos = 0.0;
            for (iatm=1;iatm<=residue[iires].natom;iatm++) {
              iiatm = residue[iires].atom[iatm];
              if (chrg[iiatm] > 0.0) {res_chrg_pos += chrg[iiatm];}
            }/*end for-iatm*/
            scale_chrg = (res_chrg_pos - res_chrg)/res_chrg_pos;
            for (iatm=1;iatm<=residue[iires].natom;iatm++) {
              iiatm = residue[iires].atom[iatm];
	      if (chrg[iiatm] > 0.0) {chrg[iiatm] *=  scale_chrg;}
            }/*end for-iatm*/
         }/*endif*/

         if (res_chrg < 0.0) {
	    res_chrg_neg = 0.0;
            for (iatm=1;iatm<=residue[iires].natom;iatm++) {
              iiatm = residue[iires].atom[iatm];
              if (chrg[iiatm] < 0.0) {res_chrg_neg += chrg[iiatm];}
            }/*end for-iatm*/
            scale_chrg = (res_chrg_neg - res_chrg)/res_chrg_neg;
            for (iatm=1;iatm<=residue[iires].natom;iatm++) {
              iiatm = residue[iires].atom[iatm];
	      if (chrg[iiatm] < 0.0) {chrg[iiatm] *=  scale_chrg;}
            }/*end for-iatm*/
         }/*endif*/


        }/*end for-ires*/
       }/*end for-imol*/


   }/*end if*/

  }/*if all_atom model*/

/*-------------------------------------------------------------------------*/
/* VI C) Scale molecule masses if required                                 */

  if (general->scale_mass != 0) {/*general scale mass*/

     if (general->scale_mass == 1) {/*all*/
         for (iatm=1;iatm<=natom;iatm++) {
              mass[iatm] *= (general->scale_mass_value);
         }/*endfor*/            
     }/*endif - scale_mass = 1 or all*/

     if (general->scale_mass == 2) {/*backbone*/
        for (imol=1;imol<=nmol;imol++) {
          if ( (strcasecmp(mol_label[imol],"RNA")==0) ||
               (strcasecmp(mol_label[imol],"DNA")==0) ) {
                for (jres=1;jres<=nres_mol[imol];jres++) {
      	          ires =  ind_res_mol[imol][jres]; 
                  for (iatm=1;iatm<=residue[ires].natom;iatm++) {
                    iiatm = residue[ires].atom[iatm];
                    if (strcasecmp(maps->upac_atm_label[iiatm],"P")==0) {
      		       ind_P = residue[ires].atom[iatm];
                       mass[ind_P]   *= (general->scale_mass_value);
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"O5'")==0) {
                       ind_O5p = residue[ires].atom[iatm];
                       mass[ind_O5p] *= (general->scale_mass_value);
                    }/*endif*/  
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C5'")==0) {
      		       ind_C5p = residue[ires].atom[iatm];
                       mass[ind_C5p] *= (general->scale_mass_value);
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C4'")==0) {
                       ind_C4p = residue[ires].atom[iatm];
                       mass[ind_C4p] *= (general->scale_mass_value);
                    }/*endif*/  
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C3'")==0) {
                       ind_C3p = residue[ires].atom[iatm];
                       mass[ind_C3p] *= (general->scale_mass_value);
                    }/*endif*/  
                    if (strcasecmp(maps->upac_atm_label[iiatm],"O3'")==0) {
                       ind_O3p = residue[ires].atom[iatm];
                       mass[ind_O3p] *= (general->scale_mass_value);
                    }/*endif*/  
                  }/*endfor-iatm*/
                }/*endfor-jres*/
          }/*endif-RNA-DNA*/
          if ( strcasecmp(mol_label[imol],"protein")==0 ) {
                for (jres=1;jres<=nres_mol[imol];jres++) {
      	          ires =  ind_res_mol[imol][jres];
                  for (iatm=1;iatm<=residue[ires].natom;iatm++) {
      		    iiatm = residue[ires].atom[iatm];
                    if (strcasecmp(maps->upac_atm_label[iiatm],"N")==0) {
      		       ind_N = residue[ires].atom[iatm];
                       mass[ind_N]   *= (general->scale_mass_value);
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"CA")==0) {
                       ind_CA = residue[ires].atom[iatm];
                       mass[ind_CA]  *= (general->scale_mass_value);
                    }/*endif*/  
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C")==0) {
      		       ind_C = residue[ires].atom[iatm];
                       mass[ind_C]   *= (general->scale_mass_value);
                    }/*endif*/
                  }/*endfor-iatm*/
                }/*endfor-jres*/
          }/*endif-protein*/
        }/*endfor-imol*/
     }/*endif - scale_mass = 2 or backbone*/
  }/*endif - scale_mass != 0*/


  if (general->scale_mass_value_H != 1.0) {/*H scale mass*/
       for (imol=1;imol<=nmol;imol++) {
          for (jres=1;jres<=nres_mol[imol];jres++) {
	    ires =  ind_res_mol[imol][jres]; 
            for (iatm=1;iatm<=residue[ires].natom;iatm++) {
              iiatm = residue[ires].atom[iatm];
              if ((maps->upac_atm_label[iiatm][0]) == 'H') {
                   mass[iiatm] *=  (general->scale_mass_value_H);
              }/*endif*/
            }/*endfor-iatm*/
          }/*endfor-jres*/
       }/*endfor-imol*/
  }/*endif H scale mass*/


/*-------------------------------------------------------------------------*/
/* VI D)  Count the number of all backbone atoms in any molecule(s)        */
/*        Allocate ind_atm_bb of length natm_bb                            */
/*        Fill in  ind_atm_bb                                              */

        natm_bb = 0;
        for (imol=1;imol<=nmol;imol++) {
          if ( (strcasecmp(mol_label[imol],"RNA")==0) ||
               (strcasecmp(mol_label[imol],"DNA")==0) ) {
	        natm_bb += 6*nres_mol[imol];              /*assume all atom*/
          }/*endif-RNA-DNA*/
          if ( strcasecmp(mol_label[imol],"protein")==0 ) {
	        natm_bb += 3*nres_mol[imol];              /*assume all atom*/
          }/*endif-protein*/
        }/*end for-imol*/
        ind_atm_bb = new int[natm_bb] - 1;

        natm_bb = 0;
        for (imol=1;imol<=nmol;imol++) {
          if ( (strcasecmp(mol_label[imol],"RNA")==0) ||
               (strcasecmp(mol_label[imol],"DNA")==0) ) {
                for (jres=1;jres<=nres_mol[imol];jres++) {
      	          ires =  ind_res_mol[imol][jres]; 
                  for (iatm=1;iatm<=residue[ires].natom;iatm++) {
                    iiatm = residue[ires].atom[iatm];
                    if (strcasecmp(maps->upac_atm_label[iiatm],"P")==0) {
      		       ind_P = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_P; 
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"O5'")==0) {
                       ind_O5p = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_O5p;
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C5'")==0) {
      		       ind_C5p = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_C5p;
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C4'")==0) {
                       ind_C4p = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_C4p;
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C3'")==0) {
                       ind_C3p = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_C3p;
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"O3'")==0) {
                       ind_O3p = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_O3p;
                    }/*endif*/
                  }/*endfor-iatm*/
                }/*endfor-jres*/
          }/*endif-RNA-DNA*/
          if ( strcasecmp(mol_label[imol],"protein")==0 ) {
                for (jres=1;jres<=nres_mol[imol];jres++) {
      	          ires =  ind_res_mol[imol][jres];
                  for (iatm=1;iatm<=residue[ires].natom;iatm++) {
      		    iiatm = residue[ires].atom[iatm];
                    if (strcasecmp(maps->upac_atm_label[iiatm],"N")==0) {
      		       ind_N = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_N;
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"CA")==0) {
                       ind_CA = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_CA;
                    }/*endif*/
                    if (strcasecmp(maps->upac_atm_label[iiatm],"C")==0) {
      		       ind_C = residue[ires].atom[iatm];
                       natm_bb++; ind_atm_bb[natm_bb] = ind_C;
                    }/*endif*/
                  }/*endfor-iatm*/
                }/*endfor-jres*/
          }/*endif-protein*/
        }/*endfor-imol*/
        maps->natm_bb    = natm_bb;
        maps->ind_atm_bb = ind_atm_bb;
 
/*-------------------------------------------------------------------------*/
/* VII)  Bond the residues                                                 */
/*  If 1pt model    -> All interactions go into ONE Residue_bond class     */
/*                  -> RESET local (nresbond = 1) and (maps->nresbond = 1) */
/*  If not 1pt model-> Each residue interaction goes into separate classes */
/*                  -> nresbond and maps->nresbond are kept INTACT         */

  if ( cgres_model_1pt==1 ) {/*1pt/res models*/
   one = 1;
   residue_bond = new Residue_bond[one] - 1;     
   residue_bond[one].setup_1pt(maps,residue,bond_res);
   nresbond = 1;
   maps->nresbond  =  nresbond; 
  }
  else {/*beyond 1pt/res models*/
   residue_bond = new Residue_bond[nresbond] - 1;  
   for (iresbond=1;iresbond<=nresbond;iresbond++) {
    ires1 = bond_res[iresbond][1];
    ires2 = bond_res[iresbond][2];
    residue_bond[iresbond].setup(maps,&(residue[ires1]),&(residue[ires2]),this);   }/*end for*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* VIII) Setup global indices for bond, bend, tors, impr, onfo             */
/*      Get global nbond, nbend, ntors, nimpr, nonfo                       */

  nbond = 0;
  nbend = 0;
  ntors = 0;
  nimpr = 0;
  nonfo = 0;
  nexclude = 0;

  for (ires=1;ires<=nres;ires++) {
      for (ibond=1;ibond<=residue[ires].nbond;ibond++) {
	  nbond++;
   	  residue[ires].bond[ibond] = nbond;
      }/*end for*/
      for (ibend=1;ibend<=residue[ires].nbend;ibend++) {
	  nbend++;
   	  residue[ires].bend[ibend] = nbend;
      }/*end for*/       
      for (itors=1;itors<=residue[ires].ntors;itors++) {
	  ntors++;
   	  residue[ires].tors[itors] = ntors;
      }/*end for*/       
      for (iimpr=1;iimpr<=residue[ires].nimpr;iimpr++) {
	  nimpr++;
   	  residue[ires].impr[iimpr] = nimpr;
      }/*end for*/       
      for (ionfo=1;ionfo<=residue[ires].nonfo;ionfo++) {
	  nonfo++;
   	  residue[ires].onfo[ionfo] = nonfo;
      }/*end for*/
      for (iexclude=1;iexclude<=residue[ires].nexclude;iexclude++) {
	  nexclude++;
   	  residue[ires].exclude[iexclude] = nexclude;
      }/*end for*/       
  }/*end for*/

  for (iresbond=1;iresbond<=nresbond;iresbond++) {
      for (ibond=1;ibond<=residue_bond[iresbond].nbond;ibond++) {
	  nbond++;
   	  residue_bond[iresbond].bond[ibond] = nbond;
      }/*end for*/
      for (ibend=1;ibend<=residue_bond[iresbond].nbend;ibend++) {
	  nbend++;
   	  residue_bond[iresbond].bend[ibend] = nbend;
      }/*end for*/       
      for (itors=1;itors<=residue_bond[iresbond].ntors;itors++) {
	  ntors++;
   	  residue_bond[iresbond].tors[itors] = ntors;
      }/*end for*/       
      for (iimpr=1;iimpr<=residue_bond[iresbond].nimpr;iimpr++) {
	  nimpr++;
   	  residue_bond[iresbond].impr[iimpr] = nimpr;
      }/*end for*/       
      for (ionfo=1;ionfo<=residue_bond[iresbond].nonfo;ionfo++) {
	  nonfo++;
   	  residue_bond[iresbond].onfo[ionfo] = nonfo;
      }/*end for*/
      for (iexclude=1;iexclude<=residue_bond[iresbond].nexclude;iexclude++) {
	  nexclude++;
   	  residue_bond[iresbond].exclude[iexclude] = nexclude;
      }/*end for*/
  }/*end for*/
  maps->nbond    = nbond;
  maps->nbend    = nbend;
  maps->ntors    = ntors;
  maps->nimpr    = nimpr;
  maps->nonfo    = nonfo;
  maps->nexclude = nexclude;

/*-------------------------------------------------------------------------*/
/* IX) Sample velocities                                                   */

  sampleVelocity(general, maps);

/*-------------------------------------------------------------------------*/
/*  X) For design calculations                                             */

  if (action_type==3 || action_type==4) {
      permute_seq      =  new int[nres] - 1;
      permute_seq_prop =  new int[nres] - 1;
      for (ires=1;ires<=nres;ires++) {permute_seq[ires] = ires;}
  }/*end for*/

/*-------------------------------------------------------------------------*/
/*  XI) If neighbor list is turned on                                      */

  if (general->neighbor_list == 1) {
        for (iatm=1;iatm<=natom;iatm++) {
            x0[iatm] = 0.0;
            y0[iatm] = 0.0;
            z0[iatm] = 0.0;
        }/*endfor*/        
  }/*endif*/


/*-------------------------------------------------------------------------*/
/* XII) Vomit topology                                                      */

#ifdef DEBUG
    printf("Entering Pos::setup DEBUG\n");
    getchar();
#endif


#ifdef DEBUG
   printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
   printf("                     Molecular Topology                      \n");
   printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
   printf("There is(are) %d molecule(s)\n",nmol);
   getchar();
   printf("Residue indices within the molecule(s)\n\n");
   ires = 0;
   for (imol=1;imol<=nmol;imol++) {
    printf("molecule %d\n\n",imol);
    printf("molecule type %s\n",maps->mol_label[imol]);
    printf("molecule chain id %s\n",maps->mol_chainid[imol]);
    printf("number of PRO residues %d\n",maps->npro_mol[imol]);
    for (jres=1;jres<=maps->nres_mol[imol];jres++) {
     printf(" residue %d is %d ",jres,maps->ind_res_mol[imol][jres]);
     printf(" %s\n",residue[maps->ind_res_mol[imol][jres]].residue_name);
    }/*end for*/
    getchar();
   }/*end for*/

   printf("\nNumber of backbone atoms: %d\n",maps->natm_bb);
   getchar();
   for (iatm=1;iatm<=maps->natm_bb;iatm++) {
        iatm_bb = ind_atm_bb[iatm];
        printf(" bbatom %d is %s of %d. residue\n",iatm,
                                                maps->upac_atm_label[iatm_bb],
                                                maps->atm_res_id[iatm_bb]);
   }/*endfor*/
   getchar();

   printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
   printf("                     Residue Topology                        \n");
   printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
   getchar();
   for (ires=1;ires<=nres;ires++) {
       residue[ires].write_alldata(maps,this);
       for (iresbond=1;iresbond<=nresbond;iresbond++) {
	 if ( ires == bond_res[iresbond][1] )  {
            residue_bond[iresbond].write_alldata(maps);
         }/*end if*/
       }/*end for*/
   }/*end for*/
#endif


#ifdef DEBUG
    printf("Leaving Pos::setup() DEBUG\n");
    getchar();
#endif


/*-------------------------------------------------------------------------*/
} /*end routine*/
/*-------------------------------------------------------------------------*/
/*=========================================================================*/


/*=========================================================================*/
void Pos::setup(char *parm_filename, Maps *maps, General *general)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int natm_curr, natm_curr_old;
  int i, n, nkey, key_defined;
  int line_num, atm_ind;
  int key_found[6];
  int natm_freeze, *atm_freeze, *aa_type;
  int mol_def_found;
  int iaatype,  naatype;

  double real_arg;
  char   str_arg[NCHARMAX];
  char   err_msg[LINE_NCHARMAX];
  
  char entry_line[LINE_NCHARMAX];
  char entry_name[NCHARMAX], key[NCHARMAX], arg[NCHARMAX];
  char *def, def_old[LINE_NCHARMAX];
  char **atm_label;
  FILE *parm_file;

  char *mol_name_keys[] = {"", "molecule_name", "natom"};
  char *atm_def_keys[]  = {"", "atom_typ", "atom_ind", 
			   "mass", "charge", "freeze"};
  
  if ((parm_file=fopen(parm_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open parm file %s.", parm_filename);
    errorMessage(err_msg);
  }

  
  /* For future multiple parm file development:
   * this part should be modified: 
   * natm_curr_old should be a parameter passed here
   */

  natm_curr_old = 0;
  atm_label  = NULL;
  atm_freeze = NULL;
  mass       = NULL;
  chrg       = NULL;
  
  natm_curr  = natm_curr_old;
  line_num   = 0;
  natm_freeze= 0;
  
  mol_def_found = 0;
  while((n=readOneEntry(parm_file,'~','[',']',entry_line, line_num))!=0) {
    if (n!=1) {
      sprintf(err_msg, "%s at line %d while reading file %s.",
	      (n==-1) ? "Syntax error" : "Internal error", 
	      line_num, parm_filename);
      errorMessage(err_msg);
    }
    n=getEntryName(entry_line, '~', '[', entry_name, def);
    if (n!=1) {
      sprintf(err_msg, "Syntax error at parm file line:\n%s", entry_line);
      errorMessage(err_msg);
    }

/*-------------------------------------------------------------------------*/
/* 1) ~molecule_name_def:      */
    if (strcasecmp(entry_name, "molecule_name_def")==0) {
      if (mol_def_found>=1) {
	sprintf(err_msg,"Multiple molecule_name_def entries not allowed: %s",
		parm_filename);
	errorMessage(err_msg);
      } else {
	mol_def_found++; 
      }
      nkey = 2;
      for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
      while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
	key_defined = 0;
	for ( i=1; i<=nkey; i++) {                  /*loop for key words.*/
	  if (strcasecmp(key, mol_name_keys[i])==0) {/*it's a keyword.  */
	    key_defined  = 1;
	    key_found[i] = 1;
	    switch (i) {
	    case 1:
	      strcpy(maps->mol_name, arg);
	      break;
	    case 2:
	      sscanf(arg, "%d", &n);
	      natm_curr += n;
	      break;
	    }/*end switch*/
	  }/*end if */
	}/*end for */
	if (key_defined==0) {
	  sprintf(err_msg, "Key word %s unknown in parm file line:\n%s",
		  key, entry_line);
	  errorMessage(err_msg);
	}
      }/*end while: getKeyArgPair */
      if (n!=0) {
	sprintf(err_msg, "Syntax error in parm file line:\n%s",entry_line);
	errorMessage(err_msg);
      }
      for ( i=1; i<=nkey; i++) { 
	if (key_found[i]==0) {
	  sprintf(err_msg, "Key word %s missed in parm file line:\n%s",
		  mol_name_keys[i], entry_line);
	  errorMessage(err_msg);
	}
      }
      if (natm_curr_old==0) {
	atm_label  = (char  **)cmall_int_mat(1, natm_curr, 0, NNAMECHARMAX); 
	atm_freeze = (int    *)cmalloc(natm_curr*sizeof(int))-1;
	aa_type    = (int    *)cmalloc(natm_curr*sizeof(int))-1;
	mass       = (double *)cmalloc(natm_curr*sizeof(double))-1;
	chrg       = (double *)cmalloc(natm_curr*sizeof(double))-1;
      } else {
	atm_label  = (char **)creall_int_mat((int **)atm_label, 
					     1, natm_curr_old, 
					     0, NNAMECHARMAX,
					     1, natm_curr,0, NNAMECHARMAX);
	atm_freeze = (int    *)crealloc(atm_freeze+1,natm_curr*sizeof(int))-1;
	aa_type    = (int    *)crealloc(atm_freeze+1,natm_curr*sizeof(int))-1;
	mass       = (double *)crealloc(mass+1, natm_curr*sizeof(double))-1;
	chrg       = (double *)crealloc(chrg+1, natm_curr*sizeof(double))-1;
      }
    }/*end if: molecule_name_def*/

/*-------------------------------------------------------------------------*/
/* 2) ~atom_def:     */
    if (strcasecmp(entry_name, "atom_def")==0) {
      if (mol_def_found==0) {
	sprintf(err_msg,"molecule_name_def should appear before atom_def: %s",
		parm_filename);
	errorMessage(err_msg);
      }
      atm_ind = 0;
      nkey    = 5;
      for ( i=1; i<=nkey; i++) { key_found[i] = 0; }
      strcpy(def_old, def);               /*Copy to store for future use. */
      while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
	if (strcasecmp(key, atm_def_keys[2])==0) { 
	  /*first find the atom index */
	  key_found[2] = 1;
	  sscanf(arg, "%d", &atm_ind);
	}
      }
      if (n!=0) { 
	sprintf(err_msg, "Syntax error at parm file line:\n%s", entry_line);
	errorMessage(err_msg);
      }
      if (key_found[2]==0) {
	sprintf(err_msg, "Key word %s missed in parm file line:\n%s",
		atm_def_keys[2], entry_line);
	errorMessage(err_msg);
      } 
      if (atm_ind<1 || atm_ind>natm_curr) {
	sprintf(err_msg,"Illegal atom index in parm file %s.",
		parm_filename);
	errorMessage(err_msg);
      }
      strcpy(def, def_old);                     /*Restore the saved line */
      while ((n=getKeyArgPair(def, key, arg, '\\', '{', '}', def))==1) {
	key_defined = 0;
	for ( i=1; i<=nkey; i++) {                  /*loop for key words.*/
	  if (strcasecmp(key, atm_def_keys[i])==0) {/*it's a keyword.  */
	    key_defined  = 1;
	    key_found[i] = 1;
	    switch (i) {
	    case 1:
	      strcpy(atm_label[atm_ind], arg);
	      break;
	    case 2:
	      break; /*already done */
	    case 3:
	      sscanf(arg, "%lf", &real_arg);
	      mass[atm_ind] = real_arg*PROT_MASS;
	      break;
	    case 4:
	      sscanf(arg, "%lf", &real_arg);
	      chrg[atm_ind] = real_arg;
	      break;
	    case 5:
	      strcpy(str_arg, arg);
	      if (strcasecmp(str_arg, "No")==0)     {atm_freeze[atm_ind]=0;}
	      else if (strcasecmp(str_arg,"Yes")==0){ 
		atm_freeze[atm_ind]=1;
		natm_freeze++;
	      }
	      else { barf(key); }
	      break;
	    }/*end switch*/
	  }/*end if */
	}/*end for */
	if (key_defined==0) {
	  sprintf(err_msg, "Key word %s unknown in parmfile line:\n%s",
		  key, entry_line);
	  errorMessage(err_msg);
	}
      }/*end while: getKeyArgPair */
      if (n!=0) {
	sprintf(err_msg, "Syntax error in parm file line:\n%s",entry_line);
	errorMessage(err_msg);
      }
      for ( i=1; i<=nkey; i++) { 
        if (i!=5) {/*make the freeze function optional*/
           if (key_found[i]==0) {
	      sprintf(err_msg, "Key word %s missed in parm file line:\n%s",
		     atm_def_keys[i], entry_line);
	      errorMessage(err_msg);
	   }
        }
      }
    }/*end if: atom_def*/
  }/*end while: readOneEntry */
  fclose(parm_file);
  
  natm_tot         = natm_curr;
  natom            = natm_curr;
  maps->natm_tot   = natm_tot;
  general->natm_tot= natm_tot;
  general->natom   = natm_tot;
  general->deg_free= 3*(natm_tot-natm_freeze);
  maps->atm_freeze = atm_freeze;
  maps->aa_type    = aa_type;
  maps->natm_freeze= natm_freeze;
  maps->atm_label  = atm_label;
  
  
  x    = new double[natm_tot]-1;
  y    = new double[natm_tot]-1;
  z    = new double[natm_tot]-1;
  xi   = new double[natm_tot]-1;
  yi   = new double[natm_tot]-1;
  zi   = new double[natm_tot]-1;
  vx   = new double[natm_tot]-1;
  vy   = new double[natm_tot]-1;
  vz   = new double[natm_tot]-1;
  fx   = new double[natm_tot]-1;
  fy   = new double[natm_tot]-1;
  fz   = new double[natm_tot]-1;
    
  sampleVelocity(general, maps);

/*-------------------------------------------------------------------------*/
/* for design calculations */
  action_type = general->action_type;
  if (action_type==3 || action_type==4) {
     permute_seq = new int[natm_tot] - 1;
     permute_seq_prop = new int[natm_tot] - 1;
     for (i=1;i<=natm_tot;i++) {
         permute_seq[i] = i;
     }
  }
/*-------------------------------------------------------------------------*/
/* aa_type temporarily definition based on CA model (ACA is CA in ALA)*/

  naatype = maps->naatype;
  for (i=1;i<=natm_tot;i++) {
    aa_type[i] = 1; /* default */
    strcpy(str_arg,atm_label[i]);
      for (iaatype=1;iaatype<=naatype;iaatype++) {
          if (strcasecmp(maps->aa_label[iaatype],str_arg)==0) {
	     aa_type[i] = iaatype; 
          }
      }
  }

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::setup( System &system, double Tg)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
    int i,j; 
    int nbend;
    long *seed    = &(system.general.seed);
    int ntors     = system.tors_defs.num_trans;
    int nmode     = system.general.nmode;
    int nmode_enm = system.general.nmode_enm;
    int nmode_stg = system.general.nmode_stg;
    int nmpt      = system.general.nmpt;
    int prop_type = system.general.prop_type;

    int *natm_trans = system.tors_defs.natm_trans;

    action_type = system.general.action_type;
    natom       = system.pos.natom;
    natm_bb     = system.pos.natm_bb;

    T = Tg;

/*-------------------------------------------------------------------------*/
/* 0) initialize system pointer                                            */
    system_ptr = &(system);

/*-------------------------------------------------------------------------*/
/* 00) evaluate the number of bends in tors_trans                          */
/*     this is not equal to the number of total bends in the system        */
    if ((prop_type==2) || (prop_type==3)) {/*torsional or bend*/
       nbend = 0;
       for (i=1;i<=ntors;i++) {
            nbend += natm_trans[i];
       }/*endfor*/
    }/*endif*/
    
/*-------------------------------------------------------------------------*/
/* I) malloc coordinates                                                   */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* I.0) general                                                            */  

    if ((prop_type==2) || (prop_type==3)) {/*torsional or bend*/
        tors_X = new double[ntors] - 1;
        t      = new double[ntors] - 1;
        ft     = new double[ntors] - 1;
        dt     = new double[ntors] - 1;
        t_scr  = new double[ntors] - 1;
        b      = new double[nbend] - 1;
        fb     = new double[nbend] - 1;
        db     = new double[nbend] - 1;
        b_scr  = new double[nbend] - 1;
    }/*endif*/
    mass   = new double[natom] - 1;
    x      = new double[natom] - 1;
    y      = new double[natom] - 1;
    z      = new double[natom] - 1;
    xi     = new double[natom] - 1;
    yi     = new double[natom] - 1;
    zi     = new double[natom] - 1;
    vx     = new double[natom] - 1;
    vy     = new double[natom] - 1;
    vz     = new double[natom] - 1;
    fx     = new double[natom] - 1;
    fy     = new double[natom] - 1;
    fz     = new double[natom] - 1;
    fxsa   = new double[natom] - 1;
    fysa   = new double[natom] - 1;
    fzsa   = new double[natom] - 1;

    fxri   = new double[natom] - 1;
    fyri   = new double[natom] - 1;
    fzri   = new double[natom] - 1;
    chrg   = new double[natom] - 1; 

    xyz    = new double[3*natom] - 1;
    dxyz   = new double[3*natom] - 1;
    fxyz   = new double[3*natom] - 1;

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* I.1) sequence sampling                                                  */  

    if (action_type==3 || action_type==4) {
       permute_seq = new int[natom] - 1;        /* for design calculations */
       permute_seq_prop = new int[natom] - 1;   /* for design calculations */
    }

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* I.2) normal modes                                                       */

    if ( (nmode == 1) || (nmode_enm==1) || (nmpt == 1) ) { 
      if (prop_type==1) {/*cartesian*/
          xc        =   new double[natom] - 1;
          yc        =   new double[natom] - 1;
          zc        =   new double[natom] - 1;
          dxc       =   new double[natom] - 1;
          dyc       =   new double[natom] - 1;
          dzc       =   new double[natom] - 1;
          xyzc      =   new double[3*natom] - 1;
          cnm       =   new double[3*natom] - 1;
          lam_hess  =   new double[3*natom] - 1;
          llam_hess =   new double[3*natom] - 1;
          hessian   =   cmall_mat(1,3*natom,1,3*natom);
          eig_hess  =   cmall_mat(1,3*natom,1,3*natom);
      }/*cartesian*/
      if (prop_type==2) {/*torsional*/
          tc        =   new double[ntors] - 1;
          tnm       =   new double[ntors] - 1;         
          lam_hess  =   new double[ntors] - 1;
          llam_hess =   new double[ntors] - 1;
          hessian   =   cmall_mat(1,ntors,1,ntors);
          eig_hess  =   cmall_mat(1,ntors,1,ntors);
      }/*torsional*/
    }/*endif*/


    if ( (nmode_stg == 1) ) {
      if (prop_type==1) {/*cartesian*/
          uxyzbb     =  new double[3*natm_bb] - 1;
          fuxyzbb    =  new double[3*natm_bb] - 1;
          duxyzbb    =  new double[3*natm_bb] - 1;
          uxyzcbb    =  new double[3*natm_bb] - 1;
          ubbnm      =  new double[3*natm_bb] - 1;
          lam_hess   =  new double[3*natm_bb] - 1;
          llam_hess  =  new double[3*natm_bb] - 1;
          hessian    =  cmall_mat(1,3*natm_bb,1,3*natm_bb);
          eig_hess   =  cmall_mat(1,3*natm_bb,1,3*natm_bb);
      }/*endif*/
    }/*endif*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* I.3) stage/repswa transformations                                       */

    if ( (system.general.repswa == 1) || (system.general.stage == 1) ) {
          ux  = new double[natom] - 1;
          uy  = new double[natom] - 1;
          uz  = new double[natom] - 1;
          fux = new double[natom] - 1;
          fuy = new double[natom] - 1;
          fuz = new double[natom] - 1;

          uxi  = new double[natom] - 1;
          uyi  = new double[natom] - 1;
          uzi  = new double[natom] - 1;
          fuxi = new double[natom] - 1;
          fuyi = new double[natom] - 1;
          fuzi = new double[natom] - 1;
    }/*endif*/


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* I.4) neighbor list                                                      */

    if (system.general.neighbor_list == 1) {
          x0  = new double[natom] - 1;
          y0  = new double[natom] - 1;
          z0  = new double[natom] - 1;
    }/*endif*/


/*-------------------------------------------------------------------------*/
/* II)  inherit coordinates and energies from system.pos                   */

    for (i=1;i<=natom;i++) {
        mass[i]     = system.pos.mass[i];
        x[i]        = system.pos.x[i];
        y[i]        = system.pos.y[i];
        z[i]        = system.pos.z[i];
        chrg[i]     = system.pos.chrg[i];
        if (action_type==3 || action_type==4) {
           permute_seq[i] = system.pos.permute_seq[i]; 
        }
    }

    ebond    = system.pos.ebond;
    ebend    = system.pos.ebend;
    etors    = system.pos.etors;
    eonfo    = system.pos.eonfo;
    einter   = system.pos.einter;
    epot     = system.pos.epot;
    ecryo_em = system.pos.ecryo_em;

    ebend_bond  = system.bend.vbond;
    ebend_bend  = system.bend.vbend;
    etors_tors  = system.tors.vtors;
    etors_imp   = system.tors.vimpr;
    eonfo_lj    = system.onfo.vlj;
    eonfo_coul  = system.onfo.vcoul;
    einter_lj   = system.inter.elj;
    einter_coul = system.inter.ecoul;
    einter_scp  = system.inter.escp;
    einter_kb   = system.inter.ekb;


/*-------------------------------------------------------------------------*/ 
/* III) initialize velocities                                              */
 
    for (i=1;i<=natom;i++) {
        vx[i] = mb(mass[i],T,seed);
        vy[i] = mb(mass[i],T,seed);
        vz[i] = mb(mass[i],T,seed);
    }/*end for*/

/*-------------------------------------------------------------------------*/ 
/* IV) initialize normal mode coordinates                                  */

    if ((nmode == 1) || (nmode_enm==1) || (nmpt == 1)) { 
        if (prop_type==1) {/*cartesian*/
            for (i=1;i<=natom;i++) {
                 xc[i]  = x[i];
                 yc[i]  = y[i];
                 zc[i]  = z[i];
            }
            for (i=1;i<=3*natom;i++) {
                 cnm[i]       = 0.0;
                 lam_hess[i]  = 0.0;
                 llam_hess[i] = 0.0;
                 for (j=1;j<=3*natom;j++) {
        	      hessian[i][j]  = 0.0;
                      eig_hess[i][j] = 0.0;
                 } 
            }
        }/*cartesian*/
        if (prop_type==2) {/*torsional*/
            for (i=1;i<=ntors;i++) {
                 tc[i]        = 0.0;
                 tnm[i]       = 0.0;
                 lam_hess[i]  = 0.0;
                 llam_hess[i] = 0.0;
                 for (j=1;j<=ntors;j++) {
      	              hessian[i][j]  = 0.0;
                      eig_hess[i][j] = 0.0;
                 } 
            }
        }/*torsional*/
    }/*nmode==1 || nmode_enm==1 || nmpt==1*/


    if ( (nmode_stg == 1) ) {
        if (prop_type==1) {/*cartesian*/
	  for (i=1;i<=3*natm_bb;i++) {
	      uxyzbb[i]    = 0.0;
	      fuxyzbb[i]   = 0.0;
	      duxyzbb[i]   = 0.0;
	      uxyzcbb[i]   = 0.0;
	      ubbnm[i]     = 0.0;
	      lam_hess[i]  = 0.0;
	      llam_hess[i] = 0.0; 
              for (j=1;j<=3*natm_bb;j++) {
		   hessian[i][j]  = 0.0;
                   eig_hess[i][j] = 0.0;
              }/*endfor*/
          }/*endfor*/
	}/*endif*/
    }/*endif*/



/*-------------------------------------------------------------------------*/ 
/* V) initialize staging coordinates                                       */

    if (system.general.stage == 1) {
        stage_transformXFXtoUFU();
    }/*endif*/


/*-------------------------------------------------------------------------*/ 
/* VI) initialize x0, y0, z0 coordinates                                   */
/*     plus move one atom over skin so that neighbor update is initialized */

    if (system.general.neighbor_list == 1) {
        for (i=1;i<=natom;i++) {
            x0[i] = 0.0;
            y0[i] = 0.0;
            z0[i] = 0.0;
        }/*endfor*/        
    }/*endif*/

/*-------------------------------------------------------------------------*/ 
/* VII) in case of cartesian propagation                                   */
/*      initialize forces                                                  */

    if (prop_type==1) {force(system);}
    

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Pos::sampleVelocity(General *general, Maps *maps)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int iatm;
  int *iseed, *iseed2; 
  double width, *qseed;
  double ekin_tmp, scale;

  printf("----------------------------------------------\n");
  printf("Sampling atom velocities\n...\n");

  iseed = general->iseed;
  iseed2= general->iseed2;
  qseed = general->qseed;
  
  gaussran(natm_tot, iseed, iseed2, qseed, vx);
  gaussran(natm_tot, iseed, iseed2, qseed, vy);
  gaussran(natm_tot, iseed, iseed2, qseed, vz);
  
  for (iatm=1; iatm<=natm_tot; iatm++) {
    width = sqrt(general->text/(mass[iatm]*BOLTZ));
    vx[iatm] *= width;
    vy[iatm] *= width;
    vz[iatm] *= width;
  } /*endfor*/
  
  ekin_tmp = 0;
  for (iatm=1; iatm<=natm_tot; iatm++) {
    ekin_tmp += mass[iatm]*vx[iatm]*vx[iatm];
    ekin_tmp += mass[iatm]*vy[iatm]*vy[iatm];
    ekin_tmp += mass[iatm]*vz[iatm]*vz[iatm];
  }

  scale = sqrt(general->text*(general->deg_free)/(ekin_tmp*BOLTZ));
  for (iatm=1; iatm<=natm_tot; iatm++) {
      vx[iatm] *= scale;
      vy[iatm] *= scale;
      vz[iatm] *= scale;
  }  

  printf("Atom velocities sampling done.\n");
  printf("----------------------------------------------\n");
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
double Pos::getEkin(General *general) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int iatm;
  double Ek=0;
  for (iatm=1; iatm<=natm_tot; iatm++) {
    Ek += 0.5*mass[iatm]*vx[iatm]*vx[iatm];
    Ek += 0.5*mass[iatm]*vy[iatm]*vy[iatm];
    Ek += 0.5*mass[iatm]*vz[iatm]*vz[iatm];
  }
  return Ek;
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*=========================================================================*/
void Pos::readCoord(char *pos_init_filename, General *general)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int iatm;  
  char line[LINE_NCHARMAX];
  char   err_msg[LINE_NCHARMAX];
  FILE *pos_init_file;

  if ((pos_init_file=fopen(pos_init_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open initial pos file %s.", pos_init_filename);
    errorMessage(err_msg);
  }
  
  if ((fgets(line, LINE_NCHARMAX, pos_init_file))==NULL) {
    sprintf(err_msg, "Error in reading file %s.", pos_init_filename);
    errorMessage(err_msg);
  }
  
  if (sscanf(line, "%d", &natm_tot)!=1) {
    sprintf(err_msg, "Error in reading file %s.", pos_init_filename);
    errorMessage(err_msg);
  }
  
  if (natm_tot != general->natm_tot) {
    sprintf(err_msg, "Wrong atom number in file %s.", pos_init_filename);
    errorMessage(err_msg);
  }
  
  for(iatm=1; iatm<=natm_tot; iatm++) {
    if ((fgets(line, LINE_NCHARMAX, pos_init_file))==NULL) {
      sprintf(err_msg, "Error in reading file %s for atmom %d.", 
	      pos_init_filename, iatm);
      errorMessage(err_msg);
    }
    
    if (sscanf(line,"%lf %lf %lf",&(x[iatm]),&(y[iatm]),&(z[iatm]))!=3) {
      sprintf(err_msg, "Error in reading file %s for atmom %d.", 
	      pos_init_filename, iatm);
      errorMessage(err_msg);
    }
    x[iatm] /= BOHR; 
    y[iatm] /= BOHR; 
    z[iatm] /= BOHR; 
  }
  if ((fgets(line, LINE_NCHARMAX, pos_init_file))==NULL) {
    sprintf(err_msg, "Error in reading file %s for box length.",
	    pos_init_filename);
    errorMessage(err_msg);
  }
    
  if (sscanf(line,"%lf",&boxl)!=1) {
    sprintf(err_msg, "Error in reading file %s for box length.",
	    pos_init_filename);
    errorMessage(err_msg);
  }
  
  if ((fgets(line, LINE_NCHARMAX, pos_init_file))==NULL) {
    sprintf(err_msg, "Error in reading file %s for box width.",
	    pos_init_filename);
    errorMessage(err_msg);
  }
    
  if (sscanf(line,"%*s %lf",&boxw)!=1) {
    sprintf(err_msg, "Error in reading file %s for box width.",
	    pos_init_filename);
    errorMessage(err_msg);
  }
    
  if ((fgets(line, LINE_NCHARMAX, pos_init_file))==NULL) {
    sprintf(err_msg, "Error in reading file %s for box height.",
	    pos_init_filename);
    errorMessage(err_msg);
  }
    
  if (sscanf(line,"%*s %*s %lf",&boxh)!=1) {
    sprintf(err_msg, "Error in reading file %s for box height.",
	    pos_init_filename);
    errorMessage(err_msg);
  }
  boxl /= BOHR; 
  boxw /= BOHR;
  boxh /= BOHR;
  
  fclose(pos_init_file);
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::write_res_arrangement( Maps *maps )
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

   int iatm, iiatm, ires, iires, imol;
   int natm_res;
   double res_chrg, mol_chrg, sys_chrg;

   int   *nres_mol;
   int  **ind_res_mol;        
   char **mol_label, **mol_chainid, **atm_label;

   int nmol_biopolymer, nmol_atom_ion, nmol_water;
   int nmol_dummy, nmol_other;

   nres_mol    = maps->nres_mol;
   ind_res_mol = maps->ind_res_mol;        
   mol_label   = maps->mol_label;          
   mol_chainid = maps->mol_chainid;
   atm_label   = maps->atm_label;          

/*-------------------------------------------------------------------------*/ 
/* I) count the number of multi-residue molecules, atoms/ions, and water   */
/*    and initialize system charge                                         */
  
   nmol_biopolymer = 0;
   nmol_atom_ion   = 0;
   nmol_water      = 0;
   nmol_dummy      = 0;
   nmol_other      = 0;
   for (imol=1;imol<=nmol;imol++) {
       if (  (strcasecmp(maps->mol_label[imol],"protein")==0) || 
             (strcasecmp(maps->mol_label[imol],"RNA")==0)     ||
             (strcasecmp(maps->mol_label[imol],"DNA")==0)     )  {
	      nmol_biopolymer++;
       }/*endif*/
       if (  (strcasecmp(maps->mol_label[imol],"atom/ion")==0) )  {
	      nmol_atom_ion++;
       }/*endif*/
       if (  (strcasecmp(maps->mol_label[imol],"water")==0) )  {
	      nmol_water++;
       }/*endif*/
       if (  (strcasecmp(maps->mol_label[imol],"DUM")==0) )  {
	      nmol_dummy++;
       }/*endif*/
       if (  (strcasecmp(maps->mol_label[imol],"other")==0) )  {
	      nmol_other++;
       }/*endif*/
   }/*endfor*/
   sys_chrg = 0.0;

/*-------------------------------------------------------------------------*/ 
/* II) write out counts                                                    */

   printf("Total number of molecules: %d\n\n",nmol);

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* II.1) biopolymer molecules                                              */


   printf("Number of biopolymer molecules: %d\n",nmol_biopolymer);
   printf("Itemizing...\n\n");

   for (imol=1;imol<=nmol;imol++) {

    if (((strcasecmp(maps->mol_label[imol],"protein")==0) || 
         (strcasecmp(maps->mol_label[imol],"RNA")==0)     ||
         (strcasecmp(maps->mol_label[imol],"DNA")==0))==0 )  
         {continue;}

    printf("%d. molecule \n",imol);
    printf("molecule type ~~~>%s<~~~\n",maps->mol_label[imol]);
    printf("molecule chain id ~~~>%s<~~~\n",maps->mol_chainid[imol]);
    mol_chrg = 0.0;
    for (ires=1;ires<=maps->nres_mol[imol];ires++) {
     iires = maps->ind_res_mol[imol][ires];
     printf(" %d. residue is %d ",ires,iires);
     printf(" %s ",residue[iires].residue_name);
     res_chrg = 0.0;
     for (iatm=1;iatm<=residue[iires].natom;iatm++) {
          iiatm = residue[iires].atom[iatm];
          res_chrg += chrg[iiatm];
     }/*end for-iatm*/
     if (fabs(res_chrg) < 1.e-15) {res_chrg = 0.0;} /*get rid of eps_m*/ 
     printf(" natom %d, charge %2.2g\n",residue[iires].natom,res_chrg);
     mol_chrg += res_chrg;
    }/*end for-ires*/
    printf("molecular charge %g\n\n",mol_chrg);
    sys_chrg += mol_chrg;
   }/*end for-imol*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* II.2) water molecules                                                   */

   printf("Number of water molecules: %d\n",nmol_water);

   for (imol=1;imol<=nmol;imol++) {
    if ((strcasecmp(maps->mol_label[imol],"water")==0)==0)
        {continue;}
    mol_chrg = 0.0;
    for (ires=1;ires<=maps->nres_mol[imol];ires++) {
     iires = maps->ind_res_mol[imol][ires];
     res_chrg = 0.0;
     for (iatm=1;iatm<=residue[iires].natom;iatm++) {
          iiatm = residue[iires].atom[iatm];
          res_chrg += chrg[iiatm];
     }/*end for-iatm*/
     if (fabs(res_chrg) < 1.e-15) {res_chrg = 0.0;} /*get rid of eps_m*/ 
     mol_chrg += res_chrg;
    }/*end for-ires*/
    sys_chrg += mol_chrg;
   }/*end for-imol*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* II.3) atom/ion molecules                                                */

   printf("Number of atoms/ions: %d \n",nmol_atom_ion);

   for (imol=1;imol<=nmol;imol++) {
    if ((strcasecmp(maps->mol_label[imol],"atom/ion")==0)==0)
        {continue;}
    mol_chrg = 0.0;
    for (ires=1;ires<=maps->nres_mol[imol];ires++) {
     iires = maps->ind_res_mol[imol][ires];
     res_chrg = 0.0;
     for (iatm=1;iatm<=residue[iires].natom;iatm++) {
          iiatm = residue[iires].atom[iatm];
          res_chrg += chrg[iiatm];
     }/*end for-iatm*/
     if (fabs(res_chrg) < 1.e-15) {res_chrg = 0.0;} /*get rid of eps_m*/ 
     mol_chrg += res_chrg;
    }/*end for-ires*/
    sys_chrg += mol_chrg;
   }/*end for-imol*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* II.4) dummy molecules                                                   */

   printf("Number of dummy atoms: %d\n",nmol_dummy);

   for (imol=1;imol<=nmol;imol++) {
    if ((strcasecmp(maps->mol_label[imol],"DUM")==0)==0)
        {continue;}
    mol_chrg = 0.0;
    for (ires=1;ires<=maps->nres_mol[imol];ires++) {
     iires = maps->ind_res_mol[imol][ires];
     res_chrg = 0.0;
     for (iatm=1;iatm<=residue[iires].natom;iatm++) {
          iiatm = residue[iires].atom[iatm];
          res_chrg += chrg[iiatm];
     }/*end for-iatm*/
     if (fabs(res_chrg) < 1.e-15) {res_chrg = 0.0;} /*get rid of eps_m*/ 
     mol_chrg += res_chrg;
    }/*end for-ires*/
    sys_chrg += mol_chrg;
   }/*end for-imol*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* II.5) other molecules                                                   */

   printf("Number of other molecules: %d \n",nmol_other);

   for (imol=1;imol<=nmol;imol++) {
    if ((strcasecmp(maps->mol_label[imol],"other")==0)==0)
        {continue;}
    mol_chrg = 0.0;
    for (ires=1;ires<=maps->nres_mol[imol];ires++) {
     iires = maps->ind_res_mol[imol][ires];
     res_chrg = 0.0;
     for (iatm=1;iatm<=residue[iires].natom;iatm++) {
          iiatm = residue[iires].atom[iatm];
          res_chrg += chrg[iiatm];
     }/*end for-iatm*/
     if (fabs(res_chrg) < 1.e-15) {res_chrg = 0.0;} /*get rid of eps_m*/ 
     mol_chrg += res_chrg;
    }/*end for-ires*/
    sys_chrg += mol_chrg;
   }/*end for-imol*/

/*-------------------------------------------------------------------------*/ 
/* III) output system charge                                               */

   printf("Total system charge: %g\n",sys_chrg);

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
