#include "../include/standard.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"
#include "../include/typedefs.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
/*=========================================================================*/
void System::set_pdb_data(const char *pdb_filename)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int ientry, jentry, n, n_entry_mall, n_entry_now;
  int line_num;

  int    nstride, ntorcompl, ncblc, ncloscompl;  
  char** stride;
  char** torcompl;
  char** closcompl;
  char*  cblc;

  char   err_msg[LINE_NCHARMAX];
  char   wrn_msg[LINE_NCHARMAX];
  char   pdb_entry_line[LINE_NCHARMAX];
  FILE *pdb_file;

  printf("----------------------------------------------\n");
  printf("Reading file %s\n...\n", pdb_filename);
  
  stride     = (char  **)cmall_int_mat(1, NSTRIDEMAX, 0, NRESMAX);
  torcompl   = (char  **)cmall_int_mat(1, NTORCOMPLMAX, 0, NMOLMAX);
  closcompl  = (char  **)cmall_int_mat(1, NCLOSCOMPLMAX, 0, NMOLMAX);
  cblc       =  new char[NMOLMAX];
  tors_defs.stride    = stride;
  tors_defs.torcompl  = torcompl;
  tors_defs.cblc      = cblc;
  tors_defs.closcompl = closcompl;
  

  if ((pdb_file=fopen(pdb_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open file %s.", pdb_filename);
    errorMessage(err_msg);
  }
  n_entry_mall = NMEM_MIN;
  pdb_database = new Pdb_data_entry[n_entry_mall]-1;
  
  general.stride_defined    = 0;
  general.torcompl_defined  = 0;
  general.closcompl_defined = 0;
  general.cblc_defined      = 0;
  general.clos_defined      = 0;

  nstride=0;
  ntorcompl=0;
  ncloscompl=0;
  ncblc=0;
  line_num=0;
  n_entry_now=0;
  while (fgets(pdb_entry_line, LINE_NCHARMAX, pdb_file) != NULL) {
    if (end_entry(pdb_entry_line)==1) break;     /* Leave at END   */
    if (stride_entry(pdb_entry_line)==1) {       /* STRIDE entry   */
       nstride++;
       if (nstride==1) {general.stride_defined = 1;}
       if (nstride>NSTRIDEMAX) {
        sprintf(err_msg,"max %d STRIDE is supported in .pdb",NSTRIDEMAX);
        errorMessage(err_msg);
       }/*endif*/
       if (strlen(pdb_entry_line)>NRESMAX) {
        sprintf(err_msg,"%d. STRIDE in.pdb is too long",nstride);
        errorMessage(err_msg);
       }/*endif*/
       get_stride(&(tors_defs.stride[nstride][0]),pdb_entry_line); 
    }/*endif*/
    if (torcompl_entry(pdb_entry_line)==1) {      /* TORCOMPL entry   */
       ntorcompl++;
       if (ntorcompl==1) {general.torcompl_defined = 1;}
       if (ntorcompl>NTORCOMPLMAX) {
        sprintf(err_msg,"max %d TORCOMPL is supported in .pdb",NTORCOMPLMAX);
        errorMessage(err_msg);
       }/*endif*/
       if (strlen(pdb_entry_line)>NMOLMAX) {
        sprintf(err_msg,"%d. TORCOMPL in .pdb is too long",ntorcompl);
        errorMessage(err_msg);
       }/*endif*/
       get_torcompl(&(tors_defs.torcompl[ntorcompl][0]),pdb_entry_line); 
    }/*endif*/
    if (closcompl_entry(pdb_entry_line)==1) {      /* CLOSCOMPL entry   */
       ncloscompl++;
       if (ncloscompl==1) {general.closcompl_defined = 1;}
       if (ncloscompl>NCLOSCOMPLMAX) {
        sprintf(err_msg,"max %d CLOSCOMPL is supported in .pdb",NCLOSCOMPLMAX);
        errorMessage(err_msg);
       }/*endif*/
       if (strlen(pdb_entry_line)>NMOLMAX) {
        sprintf(err_msg,"%d. CLOSCOMPL in .pdb is too long",ncloscompl);
        errorMessage(err_msg);
       }/*endif*/
       get_closcompl(&(tors_defs.closcompl[ncloscompl][0]),pdb_entry_line); 
    }/*endif*/ 
    if (cblc_entry(pdb_entry_line)==1) {      /* CBLC entry   */
       ncblc++;
       if (ncblc==1) { general.cblc_defined = 1;
                       general.clos_defined = 1; }
       if (ncblc>1) {
        sprintf(err_msg,"max 1 CBLC entry is supported in .pdb");
        errorMessage(err_msg);
       }/*endif*/
       if (strlen(pdb_entry_line)>NMOLMAX) {
        sprintf(err_msg,"CBLC in .pdb is too long");
        errorMessage(err_msg);
       }/*endif*/
       get_cblc(&(tors_defs.cblc[0]),pdb_entry_line); 
    }/*endif*/
    if (atom_entry(pdb_entry_line)!=1) continue; /* Only read ATOM/HETATM */
    n_entry_now++;
    if (n_entry_now>n_entry_mall) {
      n_entry_mall += NMEM_MIN;
      pdb_database  = (Pdb_data_entry *)
	realloc(pdb_database+1, (n_entry_mall)*sizeof(Pdb_data_entry))-1;
      for (ientry=1; ientry<n_entry_now-1; ientry++) {
	pdb_database[ientry].next = &(pdb_database[ientry+1]);
      }
    }
    pdb_database[n_entry_now].setup(pdb_entry_line);
    if (n_entry_now>1) {
      pdb_database[n_entry_now-1].next = &(pdb_database[n_entry_now]);
    }
  }
  pdb_database[n_entry_now].next = NULL;
  tors_defs.nstride = nstride;
  tors_defs.ntorcompl  = ntorcompl;
  tors_defs.ncloscompl = ncloscompl;
  fclose(pdb_file);


  printf("File reading done, total %d lines read.\n", n_entry_now);
  printf("----------------------------------------------\n");

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Pdb_data_entry::setup(const char *pdb_entry_line)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, ifnam, ianum, ianame, ialt, irnum, ichid, irname, ix, iy, iz;

    char field_field_name[NNAMECHARMAX];
    char field_atom_number[NNAMECHARMAX];
    char field_altloc[NNAMECHARMAX];
    char field_atom_name[NNAMECHARMAX];
    char field_residue_name[NNAMECHARMAX];
    char field_chain_id[NNAMECHARMAX];
    char field_residue_number[NNAMECHARMAX];
    char field_coord_x[NNAMECHARMAX];
    char field_coord_y[NNAMECHARMAX];
    char field_coord_z[NNAMECHARMAX];

    ifnam = 0; ianum = 0; 
    ianame = 0; ialt = 0; 
    ichid = 0; irnum = 0; 
    irname = 0;
    ix = 0; iy = 0; iz = 0;

    for (i=0;i<=strlen(pdb_entry_line)-1;i++) {
	if (i>=0 && i<=5) {/*atom_number PDB 7-11 column*/
	   field_field_name[ifnam] = pdb_entry_line[i];
           ifnam++;
        }
	else if (i>=6 && i<=10) {/*atom_number PDB 7-11 column*/
	   field_atom_number[ianum] = pdb_entry_line[i];
           ianum++;
        }
        else if (i>=12 && i<=15) {/*atom_name PDB 13-16 coulmn*/  
           field_atom_name[ianame] = pdb_entry_line[i];
           ianame++;
        }
        else if (i==16) {/*altLoc PDB 17 coulmn*/  
	   field_altloc[ialt] = pdb_entry_line[i];
           ialt++;
        }
        else if (i>=17 && i<=19) {/*residue_name PDB 18-20 coulmn*/  
           field_residue_name[irname] = pdb_entry_line[i];
           irname++;
        }
        else if (i==21) {/*chainID PDB 22 coulmn*/  
           field_chain_id[ichid] = pdb_entry_line[i];
           ichid++;
        }
        else if (i>=22 && i<=25) {/*residue_name PDB 23-26 coulmn*/  
           field_residue_number[irnum] = pdb_entry_line[i];
           irnum++;
        }
        else if (i>=30 && i<=37) {/*coord_x PDB 31-38 coulmn*/  
           field_coord_x[ix] = pdb_entry_line[i];
           ix++;
        }
        else if (i>=38 && i<=45) {/*coord_y PDB 39-46 coulmn*/  
           field_coord_y[iy] = pdb_entry_line[i];
           iy++;
        }
        else if (i>=46 && i<=53) {/*coord_z PDB 47-54 coulmn*/  
           field_coord_z[iz] = pdb_entry_line[i];
           iz++;
        }
    }
          
    field_field_name[ianum]  = '\0';
    field_atom_number[ianum] = '\0';
    field_atom_name[ianame]  = '\0';
    field_altloc[ialt]  = '\0';
    field_residue_name[irname]  = '\0';
    field_chain_id[ichid]  = '\0';
    field_residue_number[irnum]  = '\0';
    field_coord_x[ix]  = '\0';
    field_coord_y[iy]  = '\0';
    field_coord_z[iz]  = '\0';

    cutHeadEndBlank(field_field_name);
    cutHeadEndBlank(field_atom_number);
    cutHeadEndBlank(field_atom_name);
    cutHeadEndBlank(field_altloc);
    cutHeadEndBlank(field_residue_name);
    cutHeadEndBlank(field_chain_id);
    cutHeadEndBlank(field_residue_number);
    cutHeadEndBlank(field_coord_x);
    cutHeadEndBlank(field_coord_y);
    cutHeadEndBlank(field_coord_z);
    
    sscanf(field_field_name, "%s", &field_name);
    sscanf(field_atom_number, "%d", &atom_number);
    sscanf(field_atom_name, "%s", &atom_name);
    sscanf(field_altloc, "%s", &altloc);
    sscanf(field_residue_name, "%s", &residue_name);
    strcpy(chain_id,field_chain_id);
    sscanf(field_residue_number, "%d", &residue_number);
    sscanf(field_coord_x, "%lf", &coord_x);
    sscanf(field_coord_y, "%lf", &coord_y);
    sscanf(field_coord_z, "%lf", &coord_z);


#define DEBUG
#undef DEBUG
#ifdef DEBUG      
          printf("#%s# ", field_field_name);
          printf("#%s# ", field_atom_number);
          printf("#%s# ", field_atom_name);
          printf("#%s# ", field_altloc);
          printf("#%s# ", field_residue_name);
          printf("#%s# ", field_chain_id);
          printf("#%s# ", field_residue_number);
          printf("#%s# ", field_coord_x);
          printf("#%s# ", field_coord_y);
          printf("#%s#\n", field_coord_z);
          getchar();

          printf("#%s# ", field_name);
          printf("#%d# ", atom_number);
          printf("#%s# ", atom_name);
          printf("#%s# ", field_altloc);
          printf("#%s# ", residue_name);
          printf("#%s# ", chain_id);
          printf("#%d# ", residue_number);
          printf("#%g# ", coord_x);
          printf("#%g# ", coord_y);
          printf("#%g#\n", coord_z);
          getchar();
#endif      

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void System::set_res_data()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  int ientry, nentry;
  int ires, nres, nres_mall, res_num1, res_num2;
  int *natom_ires;
  const Pdb_data_entry *pdb;

  nentry = 0;
  pdb = &(pdb_database[1]);

  while (pdb!=NULL) {
       nentry++; 
       pdb = pdb->next;
  }/*end while */

  nres_mall = NMEM_MIN;
  natom_ires = new int[nres_mall]-1;  

  for (ires=1;ires<=nres_mall;ires++)  natom_ires[ires] = 0;

  nres = 1;
  natom_ires[nres]++;
  for (ientry=1;ientry<=nentry;ientry++) {
      if (ientry==1) continue;      
      res_num1 = pdb_database[ientry-1].residue_number;
      res_num2 = pdb_database[ientry].residue_number;
      if ((res_num2-res_num1)!=0) {nres++;}
      if (nres>nres_mall) {
         nres_mall += NMEM_MIN;
         natom_ires = (int *)realloc(natom_ires+1, 
                                     nres_mall*sizeof(int))-1;
         for (ires=nres;ires<=nres_mall;ires++)  
                                         natom_ires[ires] = 0;        
      }
      natom_ires[nres]++;
#define DEBUG
#undef DEBUG
#ifdef DEBUG
      printf(" atom_number %d ",pdb_database[ientry].atom_number);
      printf(" residue_number %d ",pdb_database[ientry].residue_number);
      printf(" nres %d ",nres);
      printf(" natom_ires[%d] %d\n ",nres,natom_ires[nres]);
#endif
#undef DEBUG
  }

  res_database = new Res_data_entry[nres]-1;
  general.nres = nres;
  pos.nres = nres;

  ientry = 0;
  for (ires=1;ires<=nres;ires++) {
       res_database[ires].setup(&(pdb_database[ientry]),&general,
                                   natom_ires[ires],ires,ientry+1); 
      ientry += natom_ires[ires];
#define DEBUG
#undef DEBUG
#ifdef DEBUG
      res_database[ires].write_alldata();
      getchar(); 
      printf(" ientry %d\n",ientry);
#endif
#undef DEBUG  
  }
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Res_data_entry::setup(Pdb_data_entry *pdb_database, General *general, 
                           int natm, int ires, int ind_atm1_global)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
     
    int iatm;
    char   err_msg[LINE_NCHARMAX];
    double dist_HD1_NE2, dist_HD1_ND1;
    int special_residue;
    int natom_prot_HIS;
    int natom_neut_HIS;
    int HE2_found, HD1_found;

/*-------------------------------------------------------------------------*/
/* I) Allocate class variables                                             */

    
    atom_number   = new int[natm] - 1;
    atom_label    = (char  **)cmall_int_mat(1, natm, 0, NNAMECHARMAX); 
    residue_label = new char[NNAMECHARMAX];
    chain_id      = new char[NNAMECHARMAX];
    coord_x       = new double[natm] - 1;
    coord_y       = new double[natm] - 1;
    coord_z       = new double[natm] - 1;

/*-------------------------------------------------------------------------*/
/* II) Fill in variables                                                   */

    natom = natm;
    residue_number = ires;
    residue_number_old  =  pdb_database[1].residue_number;
    atom1_global_number =  ind_atm1_global;  
    strcpy(chain_id,pdb_database[1].chain_id);                

    for (iatm=1;iatm<=natm;iatm++) {
        strcpy(atom_label[iatm],pdb_database[iatm].atom_name);
        atom_number[iatm] = iatm;
        coord_x[iatm]     = pdb_database[iatm].coord_x;              
        coord_y[iatm]     = pdb_database[iatm].coord_y;
        coord_z[iatm]     = pdb_database[iatm].coord_z;   
#undef DEBUG
#ifdef DEBUG
      printf(" atom_number %d  ",pdb_database[iatm].atom_number);
      printf(" residue_number %d\n ",pdb_database[iatm].residue_number);
#endif
#undef DEBUG
    }

/*-------------------------------------------------------------------------*/
/* III) Treating special residues                                          */
    special_residue = 0;
/*-------------------------------------------------------------------------*/
/* III) a) Amino acids                                                     */
             
/*-------------------------------------------------------------------------*/
/* Protonation states of HIS                                               */
/* HSE natom: 17, neutral HD1 bonded to NE2                                */
/* HSD natom: 17, HD1 bonded to ND1                                        */
/* HSP natom: 18, protonated                                               */
    natom_prot_HIS = 18;
    natom_neut_HIS = 17;
    if (strcasecmp(pdb_database[1].residue_name,"HIS")==0) {
       special_residue = 1;
       if (general->cgres_model==1) {   /* coarse grained model */
          strcpy(residue_label,"HIS");
       }
       else if (natom==natom_prot_HIS) {/* all atom model */
          strcpy(residue_label,"HSP");                
       }
       else if (natom==natom_neut_HIS) {/* all atom model */
         HE2_found = 0;
         HD1_found = 0;
	 for (iatm=1;iatm<=natom;iatm++) {
	     if (strcasecmp(atom_label[iatm],"HE2")==0) {
                 HE2_found = 1;
             }
             else if (strcasecmp(atom_label[iatm],"HD1")==0) {
                 HD1_found = 1;
             }
         }/*end for*/
         if ( HE2_found == 1 ) {strcpy(residue_label,"HSE");}                
         if ( HD1_found == 1 ) {strcpy(residue_label,"HSD");}
       }
       else {
            sprintf(err_msg, "HIS %d does not have the right number of atoms", 
                              residue_number_old);
            errorMessage(err_msg);
       }/*end if*/	
    }/*end if*/

/*-------------------------------------------------------------------------*/
/* III) b) Nucleic acids                                                   */             
    if (strcasecmp(pdb_database[1].residue_name,"A")==0) {
        strcpy(residue_label,"ADE");                
        special_residue = 1;
    }
    else if (strcasecmp(pdb_database[1].residue_name,"T")==0) {
        strcpy(residue_label,"THY");
        special_residue = 1;                
    }
    else if (strcasecmp(pdb_database[1].residue_name,"C")==0) {
        strcpy(residue_label,"CYT");
        special_residue = 1;                
    }
    else if (strcasecmp(pdb_database[1].residue_name,"G")==0) {
        strcpy(residue_label,"GUA");                
        special_residue = 1;
    }
    else if (strcasecmp(pdb_database[1].residue_name,"U")==0) {
        strcpy(residue_label,"URA"); 
        special_residue = 1;               
    }
/*-------------------------------------------------------------------------*/
/* IV) Other residues                                                      */             
    if (special_residue ==0) { 
        strcpy(residue_label,pdb_database[1].residue_name);                
    }
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Res_data_entry::write_alldata()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
   int iatm;

   printf(" residue %d, type %s\n",residue_number,residue_label);
   printf(" chain_id %s\n",chain_id);
   printf(" # of atoms %d\n",natom);
   printf(" starting global atom # %d\n",atom1_global_number);
   printf(" index  type  x  y  z\n");
   for (iatm=1;iatm<=natom;iatm++) {
        printf("%d  %s ",atom_number[iatm],atom_label[iatm]);
        printf("%g %g %g\n",coord_x[iatm],coord_y[iatm],coord_z[iatm]);
   }
  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

