#include "../include/standard.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"
#include "../include/typedefs.h"

#define DEBUG_OFF

/*==============================================================================*/
/*==============================================================================*/
void System::set_rtf_data(const char *rtf_filename)
/*------------------------------------------------------------------------------*/
{ /*begin routine */
/*------------------------------------------------------------------------------*/

  int i, ientry, jentry, n, n_entry_mall, n_entry_now;
  int n_locator_mall, n_locator_old, n_locator_now;
  int line_num, n_rtfres;
  int MASS_found;
  
  char   err_msg[LINE_NCHARMAX];
  char   wrn_msg[LINE_NCHARMAX];
  char   rtf_entry_line[LINE_NCHARMAX];

  Rtf_data_entry *rtf_database;  /* residue topology file line by line     */
  int **residue_locator;         /* [1][1] line locator of 1st MASS entry  */
                                 /* [1][2] # of MASS lines                 */
                                 /* [2][1] line locator of RESI 1 entry    */
                                 /* [2][2] # of RESI 1 lines               */
  int data_entry_type;           /* 0 - MASS, 1 - RESI, 2 - other          */
  int nline_resi, nline_mass;
  Rtf_data_entry *rtf_resi;      /* Pointer to RESI in rtf                 */
  Rtf_data_entry *rtf_mass;      /* Pointer to MASS in rtf                 */

  FILE *rtf_file;

/*------------------------------------------------------------------------------*/
/* I) Reading residue topology file */
  printf("----------------------------------------------\n");
  printf("Reading file %s\n...\n", rtf_filename);
  
  if ((rtf_file=fopen(rtf_filename, "r")) == NULL) {
    sprintf(err_msg, "Cannot open file %s.", rtf_filename);
    errorMessage(err_msg);
  }
  n_entry_mall = NMEM_MIN;
  rtf_database = new Rtf_data_entry[n_entry_mall]-1;     /*rtf line by line*/
  n_locator_mall = NMEM_MIN;
  residue_locator = cmall_int_mat(0,n_locator_mall,1,2);
  
  n_rtfres = 0;
  line_num = 0;
  n_entry_now = 0;
  n_locator_now = 0;
  MASS_found = 0;
  while (fgets(rtf_entry_line, LINE_NCHARMAX, rtf_file) != NULL) {
    line_num++;
    n_entry_now++;
    if (n_entry_now>n_entry_mall) {
      n_entry_mall += NMEM_MIN;
      rtf_database  = (Rtf_data_entry *)
	realloc(rtf_database+1, (n_entry_mall)*sizeof(Rtf_data_entry))-1;
      for (ientry=1; ientry<n_entry_now-1; ientry++) {
	rtf_database[ientry].next = &(rtf_database[ientry+1]);
      }
    }

#ifdef DEBUG
  printf("############################ RTFLINE ############################\n");
    printf("%s",rtf_entry_line);
  printf("############################ RTFLINE ############################\n");
  getchar();
#endif

    data_entry_type = rtf_database[n_entry_now].setup(rtf_entry_line);
    if (n_entry_now>1) {
      rtf_database[n_entry_now-1].next = &(rtf_database[n_entry_now]);
    }
    if (data_entry_type==0) {/* MASS entry */
       if (MASS_found==0) {
          n_locator_now++;
          if (n_locator_now>n_locator_mall) {
             n_locator_old  = n_locator_mall;
             n_locator_mall += NMEM_MIN;
             residue_locator = (int **)creall_int_mat((int **)residue_locator, 
		   			        0, n_locator_old, 
					        1, 2,
					        0, n_locator_mall,1, 2);
          }
          residue_locator[n_locator_now][1] = line_num;
          MASS_found = 1;
       }
       residue_locator[n_locator_now][2]++;
    }
    if (data_entry_type==1) {/* RESI or PRES entry */
          n_rtfres++;
          n_locator_now++;
          if (n_locator_now>n_locator_mall) {
             n_locator_old  = n_locator_mall;
             n_locator_mall += NMEM_MIN;
	     residue_locator = (int **)creall_int_mat((int **)residue_locator, 
		        			0, n_locator_old, 
					        1, 2,
					        0, n_locator_mall,1, 2);
          }
          residue_locator[n_locator_now][1] = line_num;
          if (n_locator_now>2) {/* 1 MASS, 2 RESI-1 */
	    residue_locator[n_locator_now-1][2] = 
                                   line_num-residue_locator[n_locator_now-1][1]; 
          }
    }
  }
  residue_locator[n_locator_now][2] = line_num-residue_locator[n_locator_now][1]; 
  rtf_database[n_entry_now].next = NULL;
  fclose(rtf_file);

#ifdef DEBUG
  printf("############################ RESLOC ############################\n");
  for (i=1;i<=n_locator_now;i++) {
    printf("residue_locator[%d][1]%d\n",i,residue_locator[i][1]);       
    printf("residue_locator[%d][2]%d\n",i,residue_locator[i][2]);       
  }
  printf("############################ RESLOC ############################\n");
  getchar();
#endif

/*------------------------------------------------------------------------------*/
/* II) Setting up residue topology database */

  restop_database = new Restop_data[n_rtfres] - 1;
  general.nrtfres = n_rtfres;
  pos.nrtfres = n_rtfres;

  nline_mass = residue_locator[1][2];
  rtf_mass   = &(rtf_database[residue_locator[1][1]])-1;
  for (i=1;i<=n_rtfres;i++) {
      nline_resi = residue_locator[i+1][2];
      rtf_resi = &(rtf_database[residue_locator[i+1][1]])-1;
      restop_database[i].setup(general,rtf_resi,nline_resi,rtf_mass,nline_mass);
  }
 
/*------------------------------------------------------------------------------*/

  printf("File reading done, total %d lines read.\n", n_entry_now);
  printf("----------------------------------------------\n");



/*------------------------------------------------------------------------------*/
} /*end routine */
/*==============================================================================*/


/*==============================================================================*/
/*==============================================================================*/
int Rtf_data_entry::setup(char *rtf_entry_line)
/*------------------------------------------------------------------------------*/
{ /*begin routine */
/*------------------------------------------------------------------------------*/
  int  i, n_entry_mall, n_entry_now, n_entry_old;

  char first_char;
  char* ptr;

  n_entry_mall = NMEM_MIN;
  field  = (char  **)cmall_int_mat(1, n_entry_mall, 0, NNAMECHARMAX); 

  n_entry_now = 0;
  ptr = strtok(rtf_entry_line," ");
  while (ptr!=NULL) {
        n_entry_now++;
        if (n_entry_now>n_entry_mall) {
	   n_entry_old   = n_entry_mall;
           n_entry_mall += NMEM_MIN;
	   field  = (char **)creall_int_mat((int **)field, 
					     1, n_entry_old, 
					     0, NNAMECHARMAX,
					     1, n_entry_mall,0, NNAMECHARMAX);
        }
        strcpy(field[n_entry_now],ptr);
        ptr = strtok(NULL," ");
  }

  first_char = field[n_entry_now][0];
  if (first_char=='\n') {
     nfield = n_entry_now-1;
  }
  else {
     ptr = strtok(field[n_entry_now],"\n");
     nfield = n_entry_now;
  }

  /* get rid of comments */
  for (i=1;i<=nfield;i++) {
      first_char = field[i][0];
      if (first_char=='!') {
	 nfield = i - 1;
         break;
      }
  }

#define DEBUG
#undef DEBUG
#ifdef DEBUG
  for (i=1;i<=nfield;i++) {
      printf("#%s#",field[i]);
  }
  printf("\n");
  getchar();
#endif
 
  if (strcasecmp(field[1],"MASS")==0) {
    return 0;
  }
  else if ((strcasecmp(field[1],"RESI")==0)||(strcasecmp(field[1],"PRES")==0)) {
    return 1;
  }
  else {return 2;}
  
/*------------------------------------------------------------------------------*/
} /*end routine */
/*==============================================================================*/


/*==============================================================================*/
/*==============================================================================*/
void Restop_data::setup(General general, Rtf_data_entry *rtf_resi, int nline_resi, 
                                         Rtf_data_entry *rtf_mass, int nline_mass)
/*------------------------------------------------------------------------------*/
{ /*begin routine */
/*------------------------------------------------------------------------------*/
  int i, j, iatom, jatom, ifield;
  int ibond, ibend, itors, iimpr, ionfo, iexclude;
  int jbond;
  int n_entry_mall, n_entry_now, n_entry_old;
  int n_entry_mall_, n_entry_now_, n_entry_old_;
  int *bond_list;
  int natom_list, iatom_list, jatom_list;
  int natom_list_1, iatom_list_1;
  int natom_list_2, iatom_list_2;
  int atm1, atm2;
  int *bond_list_1, *bond_list_2;
  int ibranch, iibranch;
  int found_root_1, found_root_2;
  int onfo_found, bend_found;

  int nbond_atm_max, ibond_atm;
  int valid;
  int iatm, jatm, katm, latm; 
  int kbond_atm, lbond_atm;
  int ichigrp, nchiprim;

  int isize, size;
  int natm_self;
  int first, second;                     

  int atm_field[5];
  int atom_found[5];
  int atom[5];
  char first_char;
  char atom_loc[5];  /* '.': this, '+': next, '-': previous residue */

  char root_f[10];   /* first root */
  char root_s[10];   /* second root */

  char   err_msg[LINE_NCHARMAX];

  int inter_on_3bond_conn = general.inter_on_3bond_conn;
  int inter_on_4bond_conn = general.inter_on_4bond_conn;

/*------------------------------------------------------------------------------*/
/* I) Fill in the variables                                                     */

  n_entry_mall = NMEM_MIN;  
  atom_label      = (char  **)cmall_int_mat(1, n_entry_mall, 0, NNAMECHARMAX); 
  upac_atom_label = (char  **)cmall_int_mat(1, n_entry_mall, 0, NNAMECHARMAX); 
  atom_gindex     =    (int *)cmalloc(n_entry_mall*sizeof(int))-1;  
  mass            = (double *)cmalloc(n_entry_mall*sizeof(double))-1;  
  charge          = (double *)cmalloc(n_entry_mall*sizeof(double))-1;  

  n_entry_mall_ = NMEM_MIN;
  group_begin     =    (int *)cmalloc(n_entry_mall_*sizeof(int))-1;  
  group_end       =    (int *)cmalloc(n_entry_mall_*sizeof(int))-1;  

  natom = 0;
  ngroup = 0;
  n_entry_now = 0;
  n_entry_now_ = 0;
  group_begin[1] = 1; /* first group begins with first atom */
  for (i=1;i<=nline_resi;i++) {
      if ( strcasecmp(rtf_resi[i].field[1],"RESI")==0 ||  
           strcasecmp(rtf_resi[i].field[1],"PRES")==0 ) {
          sscanf(rtf_resi[i].field[1], "%s", residue_type);
          sscanf(rtf_resi[i].field[2], "%s", residue_name);
          sscanf(rtf_resi[i].field[3], "%lf", &residue_chrg);
      }
      else if (strcasecmp(rtf_resi[i].field[1],"GROUP")==0) {
	  ngroup++;
          n_entry_now_++;
          if (n_entry_now_>n_entry_mall_) {
	     n_entry_old_  = n_entry_mall_;
             n_entry_mall_ += NMEM_MIN;
             group_begin = (int *)crealloc(group_begin+1,n_entry_mall_*sizeof(int))-1;
             group_end   = (int *)crealloc(group_end+1,n_entry_mall_*sizeof(int))-1;  
          }
          if (ngroup!=1) {
	     group_begin[ngroup] = natom + 1;
             group_end[ngroup-1] = natom;
          }
      }
      else if (strcasecmp(rtf_resi[i].field[1],"ATOM")==0) {
	  natom++;
          n_entry_now++;
          if (n_entry_now>n_entry_mall) {
	     n_entry_old   = n_entry_mall;
             n_entry_mall += NMEM_MIN;
	     atom_label = (char **)creall_int_mat((int **)atom_label, 
					     1, n_entry_old, 
					     0, NNAMECHARMAX,
					     1, n_entry_mall,
                                             0, NNAMECHARMAX);
             upac_atom_label  = (char **)creall_int_mat((int **)upac_atom_label, 
					     0, n_entry_old, 
					     0, NNAMECHARMAX,
					     0, n_entry_mall,
                                             0, NNAMECHARMAX);
	     atom_gindex = (int *)crealloc(atom_gindex+1,n_entry_mall*sizeof(int))-1;
	     mass   = (double *)crealloc(mass+1,n_entry_mall*sizeof(double))-1;
             charge = (double *)crealloc(charge+1,n_entry_mall*sizeof(double))-1;
          }
          atom_gindex[n_entry_now] = ngroup;    
          sscanf(rtf_resi[i].field[2], "%s", upac_atom_label[n_entry_now]);
          sscanf(rtf_resi[i].field[3], "%s", atom_label[n_entry_now]);
          sscanf(rtf_resi[i].field[4], "%lf", &(charge[n_entry_now]));
          for (j=1;j<=nline_mass;j++) {
	      if (strcasecmp(atom_label[n_entry_now],rtf_mass[j].field[3])==0) {
		 jatom = j;
                 break;
              }             
          }
          sscanf(rtf_mass[jatom].field[4], "%lf", &(mass[n_entry_now]));
      }/* end if */
  }/* end for */
  group_end[ngroup] = natom; /* last group ends with last atom */

/*------------------------------------------------------------------------------*/
/* II) Check if upac_atom_label is a unique list                                */
/*     Exit with an error only if there is any BOND/DOUBLE entry in the residue */

  unique_atom_list = 1;
  for (iatom=1;iatom<=natom-1;iatom++) {
      for (jatom=iatom+1;jatom<=natom;jatom++) {
          if (strcasecmp(upac_atom_label[iatom],upac_atom_label[jatom])==0) {
	      unique_atom_list = 0;
          }/*endif*/
      }/*endfor*/
  }/* end for */  

  nbond = 0;
  for (i=1;i<=nline_resi;i++) {
    if ( (strcasecmp(rtf_resi[i].field[1],"BOND")  ==0) ||
         (strcasecmp(rtf_resi[i].field[1],"DOUBLE")==0)   ) {nbond++;}
  }/*endfor*/

  if ((unique_atom_list==0) && (nbond!=0)) {
    for (iatom=1;iatom<=natom-1;iatom++) {
      for (jatom=iatom+1;jatom<=natom;jatom++) {
          if (strcasecmp(upac_atom_label[iatom],upac_atom_label[jatom])==0) {
             sprintf(err_msg,"either define residues without any connectivity \n"
                             "-no BONDS/DOUBLE- or have a unique list of atoms\n"
                             "%dth and %dth atoms are identical in residue %s", 
                    iatom,jatom,residue_name);
             errorMessage(err_msg);             
          }
      }
    }/* end for */  
  }/*endif*/

/*------------------------------------------------------------------------------*/
/* III) Fill in bond_atom                                                       */

  n_entry_mall = NMEM_MIN;  

  bond_atom  =  cmall_int_mat(1, n_entry_mall, 1, 2); 

  size = 2;
  nbond  = 0;
  ndbond = 0;
  n_entry_now = 0;
  for (i=1;i<=nline_resi;i++) {
    if ( (strcasecmp(rtf_resi[i].field[1],"BOND")==0) || 
          strcasecmp(rtf_resi[i].field[1],"DOUBLE")==0) {
      for (ifield=2;ifield<=rtf_resi[i].nfield;ifield+=2) {
        atm_field[1] = ifield;
        atm_field[2] = ifield+1; 
 
        for (isize=1;isize<=size;isize++) {/* atom[1], atom[2] */
          atom_found[isize] = 0;
          for (iatom=1;iatom<=natom;iatom++) {
            if (strcasecmp(upac_atom_label[iatom],
                           rtf_resi[i].field[atm_field[isize]])==0) {
               atom_found[isize] = 1;
               atom[isize] =  iatom; 
               break; 
            }/* end if */                
	  }/* end for */
          if (atom_found[isize]==0) {
            first_char =  rtf_resi[i].field[atm_field[isize]][0];
            if ( (first_char!='+') && (first_char!='-') &&
                 (strcasecmp(residue_type,"PRES")!=0) ) {
               sprintf(err_msg,"%dth atom not found on line %d in residue %s", 
                      atm_field[isize]-1,i,residue_name);
               errorMessage(err_msg);             
            }/* end if */
	  }/* end if */
        }/* end for isize<=size */

        if ( (atom_found[1]==1)&&(atom_found[2]==1) ) {/* new bond */
	     nbond++;
             n_entry_now++;
             if (n_entry_now>n_entry_mall) {
	        n_entry_old   = n_entry_mall;
                n_entry_mall += NMEM_MIN;
	        bond_atom     =  creall_int_mat((int **)bond_atom, 
					     1, n_entry_old, 1, 2,
					     1, n_entry_mall, 1, 2);
             }
             bond_atom[nbond][1] = atom[1];
             bond_atom[nbond][2] = atom[2];
        }/* end if */

      }/* end for ifield<=nfield */	  
    }/* end if "BOND" "DOUBLE" */     
  }/* end for i<=nline_resi */   

/*------------------------------------------------------------------------------*/
/* IV) Fill in impr_atom, bimpr_atom_f, bimpr_atom_s                            */

  n_entry_mall = NMEM_MIN;  

  impr_atom     =  cmall_int_mat(1, n_entry_mall, 1, 4); 
  bimpr_atom_f  =  (int *)cmalloc(4*sizeof(int))-1;  
  bimpr_atom_s  =  (int *)cmalloc(4*sizeof(int))-1;  

  size = 4;
  nimpr = 0;
  nbimpr = 0;
  n_entry_now = 0;
  for (i=1;i<=nline_resi;i++) {
    if (strcasecmp(rtf_resi[i].field[1],"IMPR")==0) {
      for (ifield=2;ifield<=rtf_resi[i].nfield;ifield+=4) {
        atm_field[1] = ifield;
        atm_field[2] = ifield+1; 
        atm_field[3] = ifield+2;
        atm_field[4] = ifield+3;  

        for (isize=1;isize<=size;isize++) {/* atom[1],atom[2],atom[3],atom[4] */
          atom_found[isize] = 0; 
          for (iatom=1;iatom<=natom;iatom++) {
            if (strcasecmp(upac_atom_label[iatom],
                           rtf_resi[i].field[atm_field[isize]])==0) {
               atom[isize] = iatom;
               atom_found[isize] = 1; 
               atom_loc[isize] = '.';
               break; 
            }/* end if */                
  	  }/* end for */
          if (atom_found[isize]==0) {
            first_char =  rtf_resi[i].field[atm_field[isize]][0];
            atom_loc[isize] = first_char;
            if ((first_char!='+')&&(first_char!='-')&& 
                (strcasecmp(residue_type,"PRES")!=0)) {
               sprintf(err_msg,"%dth atom not found on line %d in residue %s", 
                      atm_field[isize]-1,i,residue_name);
               errorMessage(err_msg);             
            }/* end if */
          }/* end if */
        }/* end for isize<=size */

        first = 0;
        second = 0;
        natm_self = 0;
        for (isize=1;isize<=size;isize++) {
	    if  (atom_loc[isize] == '.') {natm_self++;}
            if  (atom_loc[isize] == '-') {first = 1;}
            if  (atom_loc[isize] == '+') {second = 1;}
        }/* end for isize<=size */

        if ( natm_self==4 ) {/* new impr */
	     nimpr++;
             n_entry_now++;
             if (n_entry_now>n_entry_mall) {
	        n_entry_old   = n_entry_mall;
                n_entry_mall += NMEM_MIN;
	        impr_atom     =  creall_int_mat((int **)impr_atom, 
					     1, n_entry_old, 1, 4,
					     1, n_entry_mall, 1, 4);
             }
             impr_atom[nimpr][1] = atom[1];
             impr_atom[nimpr][2] = atom[2];
             impr_atom[nimpr][3] = atom[3];
             impr_atom[nimpr][4] = atom[4];
        }
        else if ( (natm_self==3)&&((first==1)||(second==1)) ) {/* new bimpr */
	   nbimpr++;
           if ( nbimpr > 2 ) {/* max 2 nbimpr */
              sprintf(err_msg,"more than 2 bond_improper in residue %s", 
                                                           residue_name);
              errorMessage(err_msg);             
           }/* end if */
	   if (first==1) {
              for (isize=1;isize<=size;isize++) {
		  if (atom_loc[isize] == '-') {
                      bimpr_atom_f[isize] =  0;
                  }
                  else {
		      bimpr_atom_f[isize] =  atom[isize];
                  }/* end if */                      
              }/* end for */
           }
           else if (second==1) {
              for (isize=1;isize<=size;isize++) {
		  if (atom_loc[isize] == '+') {
                      bimpr_atom_s[isize] =  0;
                  }
                  else {
		      bimpr_atom_s[isize] =  atom[isize];
                  }/* end if */                                    
              }/* end for */
           }/* end if */
        }
        else if ( ((first!=1)&&(second!=1)) ) {/* new bimpr */
	   nbimpr++;
           if ( nbimpr > 2 ) {/* max 2 nbimpr */
              sprintf(err_msg,"more than 2 bond_improper in residue %s", 
                                                           residue_name);
              errorMessage(err_msg);             
           }/* end if */
	   if (nbimpr==1) {
              for (isize=1;isize<=size;isize++) {
		  if (atom_loc[isize] != '.') {
                      bimpr_atom_f[isize] =  0;
                  }
                  else {
		      bimpr_atom_f[isize] =  atom[isize];
                  }/* end if */                      
              }/* end for */
           }
           else if (nbimpr==2) {
              for (isize=1;isize<=size;isize++) {
		  if (atom_loc[isize] != '.') {
                      bimpr_atom_s[isize] =  0;
                  }
                  else {
		      bimpr_atom_s[isize] =  atom[isize];
                  }/* end if */                                    
              }/* end for */
           }/* end if */
        }
        else {
            sprintf(err_msg,"invalid improper on line %d in residue %s", 
                            i,residue_name);
            errorMessage(err_msg);
        }/* end if */

      }/* end for ifield<=nfield */	  
    }/* end if "IMPR" */     
  }/* end for  i<=nline_resi */   

/*------------------------------------------------------------------------------*/
/* V) Fill in bend_atom                                                         */

  bond_list = new int[10]-1;

  n_entry_mall = NMEM_MIN;
  bend_atom  =  cmall_int_mat(1, n_entry_mall, 1, 3); 

  nbend = 0;
  n_entry_now = 0;
  for (iatom=1;iatom<=natom;iatom++) {
      natom_list = 0;
      for (ibond=1;ibond<=nbond;ibond++) {      
	  if (bond_atom[ibond][1]==iatom) {
  	     natom_list++;
	     bond_list[natom_list] = bond_atom[ibond][2];        
          } 
          else if (bond_atom[ibond][2]==iatom) {
             natom_list++;
             bond_list[natom_list] = bond_atom[ibond][1];  
          }  
      }/*end for*/
      for (iatom_list=1;iatom_list<=natom_list-1;iatom_list++) {
          for (jatom_list=iatom_list+1;jatom_list<=natom_list;jatom_list++) {
	      nbend++;
              n_entry_now++;
              if (n_entry_now>n_entry_mall) {
	         n_entry_old   = n_entry_mall;
                 n_entry_mall += NMEM_MIN;
	         bend_atom     =  creall_int_mat((int **)bend_atom, 
					     1, n_entry_old, 1, 3,
					     1, n_entry_mall, 1, 3);
              }
              bend_atom[nbend][1] =  bond_list[iatom_list];
              bend_atom[nbend][2] =  iatom;  
              bend_atom[nbend][3] =  bond_list[jatom_list];
          }
      }
  }/*end for*/

/*------------------------------------------------------------------------------*/
/* VI) Fill in tors_atom                                                        */

  bond_list_1 = new int[10]-1;
  bond_list_2 = new int[10]-1;

  n_entry_mall = NMEM_MIN;  
  tors_atom  =  cmall_int_mat(1, n_entry_mall, 1, 4); 
  ntors = 0;
  n_entry_now = 0;   

  for (ibond=1;ibond<=nbond;ibond++) {/* loop through bonds */

      natom_list_1 = 0;
      atm1 = bond_atom[ibond][1];
      for (jbond=1;jbond<=nbond;jbond++) {      
	  if (jbond!=ibond) {
	     if (bond_atom[jbond][1]==atm1) {
  	        natom_list_1++;
	        bond_list_1[natom_list_1] = bond_atom[jbond][2];        
             } 
             else if (bond_atom[jbond][2]==atm1) {
                      natom_list_1++;
                      bond_list_1[natom_list_1] = bond_atom[jbond][1];
             }  
          }/*end if*/
      }/*end for*/

      natom_list_2 = 0;
      atm2 = bond_atom[ibond][2];
      for (jbond=1;jbond<=nbond;jbond++) {      
          if (jbond!=ibond) {
	     if (bond_atom[jbond][1]==atm2) {
  	         natom_list_2++;
	         bond_list_2[natom_list_2] = bond_atom[jbond][2];
             } 
             else if (bond_atom[jbond][2]==atm2) {
                 natom_list_2++;
                 bond_list_2[natom_list_2] = bond_atom[jbond][1];
             }  
     	  }
      }/*end for*/   

      if ( (natom_list_1!=0) && (natom_list_2!=0) ) {
	 for (iatom_list_1=1;iatom_list_1<=natom_list_1;iatom_list_1++) {
             for (iatom_list_2=1;iatom_list_2<=natom_list_2;iatom_list_2++) {
	         ntors++;
                 n_entry_now++;
                 if (n_entry_now>n_entry_mall) {
	            n_entry_old   = n_entry_mall;
                    n_entry_mall += NMEM_MIN;
	            tors_atom     =  creall_int_mat((int **)tors_atom, 
			     		         1, n_entry_old, 1, 4,
					         1, n_entry_mall, 1, 4);
                 }
                 tors_atom[ntors][1] =  bond_list_1[iatom_list_1];
                 tors_atom[ntors][2] =  bond_atom[ibond][1];
                 tors_atom[ntors][3] =  bond_atom[ibond][2];
                 tors_atom[ntors][4] =  bond_list_2[iatom_list_2];
             }/*end for*/
         }/*end for*/
      }/*end if*/

  }/* for loop through bonds */


/*------------------------------------------------------------------------------*/
/* VII) Filter tors_atom with impr_atom                                         */
/* No need since there is non of them                                           */
  
/*------------------------------------------------------------------------------*/
/* VIII) Fill in onfo_atom                                                      */

  n_entry_mall = NMEM_MIN;  
  onfo_atom  =  cmall_int_mat(1, n_entry_mall, 1, 2); 
  nonfo = 0;
  n_entry_now = 0;   

  for (itors=1;itors<=ntors;itors++) {
      onfo_found = 0;
      for (ionfo=1;ionfo<=nonfo;ionfo++) {
	if ( ((onfo_atom[ionfo][1]==tors_atom[itors][1])&&
              (onfo_atom[ionfo][2]==tors_atom[itors][4])) ||
             ((onfo_atom[ionfo][1]==tors_atom[itors][4])&&
              (onfo_atom[ionfo][2]==tors_atom[itors][1])) ) {onfo_found=1;}
      }/*end for*/
      bend_found = 0;
      for (ibend=1;ibend<=nbend;ibend++) {
	if ( ((bend_atom[ibend][1]==tors_atom[itors][1])&&
              (bend_atom[ibend][3]==tors_atom[itors][4])) ||
             ((bend_atom[ibend][1]==tors_atom[itors][4])&&
              (bend_atom[ibend][3]==tors_atom[itors][1])) ) {bend_found=1;}
          
      }/*end for*/
      if ((onfo_found==0)&&(bend_found==0)) {
         nonfo++;
         n_entry_now++;
         if (n_entry_now>n_entry_mall) {
             n_entry_old   = n_entry_mall;
             n_entry_mall += NMEM_MIN;
             onfo_atom     =  creall_int_mat((int **)onfo_atom, 
       		   		         1, n_entry_old, 1, 2,
       				         1, n_entry_mall, 1, 2);
         }
         onfo_atom[nonfo][1] =  tors_atom[itors][1];
         onfo_atom[nonfo][2] =  tors_atom[itors][4];
      }/*end if*/
  }/*end for*/

  
/*------------------------------------------------------------------------------*/
/* IX) Fill in exclude_atom                                                     */

  nbond_atm = new int[natom]-1;

  for (iatm=1; iatm<=natom; iatm++) { nbond_atm[iatm] = 0; }
  for (ibond=1; ibond<=nbond; ibond++) {
    iatm = bond_atom[ibond][1];
    jatm = bond_atom[ibond][2];
    nbond_atm[iatm]++;
    nbond_atm[jatm]++;
  }/*end for*/
  nbond_atm_max = 0;
  for (iatm=1; iatm<=natom; iatm++) {
    nbond_atm_max = MAX(nbond_atm[iatm],nbond_atm_max);
  }/*end for*/
  bond_atm_map = cmall_int_mat(1, natom, 1, nbond_atm_max);
  for (iatm=1; iatm<=natom; iatm++) {
    ibond_atm = 0;
    for (ibond=1; ibond<=nbond; ibond++) {
      if (iatm==bond_atom[ibond][1]) {
	bond_atm_map[iatm][++ibond_atm] = bond_atom[ibond][2];
      }/*end if*/
      if (iatm==bond_atom[ibond][2]) {
	bond_atm_map[iatm][++ibond_atm] = bond_atom[ibond][1];
      }/*end if*/
    }/*end for*/
    if (ibond_atm!=nbond_atm[iatm]) {
      sprintf(err_msg,"Internal error of setting up atom bond map in residue %s", 
	                                                            residue_name);
      errorMessage(err_msg);
    }/*end if*/
  }/*end for*/
  
  n_entry_mall = NMEM_MIN;  
  exclude_atom  =  cmall_int_mat(1, n_entry_mall, 1, 2); 
  nexclude = 0;
  n_entry_now = 0;   

  for (iatm=1; iatm<=natom; iatm++) {
    for (jatm=iatm+1; jatm<=natom; jatm++) {
      if (general.cancel_res_inter_local) {/* if all inter turned off */
	  valid = 0;                       /* within the residue      */
      }
      else {
         valid = 1;     
         /*Check direct bond connection: iatm--jatm*/
         for (ibond_atm=1; ibond_atm<=nbond_atm[iatm]; ibond_atm++) {
 	   if (jatm==bond_atm_map[iatm][ibond_atm]) { 
	     valid = 0;
	     break;  /*Not necessary to continue. */
	   }/*end if*/
         }/*end for*/
      }/*end if*/
      /*Check three-atom bond connection: iatm--katm--jatm */
      if ((valid==1)&&(inter_on_3bond_conn==0)) {
	for (ibond_atm=1; ibond_atm<=nbond_atm[iatm]; ibond_atm++) {
	  katm=bond_atm_map[iatm][ibond_atm];
	  for (kbond_atm=1; kbond_atm<=nbond_atm[katm]; kbond_atm++) {
	    if(jatm== bond_atm_map[katm][kbond_atm]) {
	      valid = 0;
	      break;  /*Not necessary to continue. */
	    }/*end if*/
	  }/*end for*/ 
	}/*end for*/
        for (ibend=1; ibend<=nbend; ibend++) {
	  if ((iatm==bend_atom[ibend][1] ||
	       iatm==bend_atom[ibend][2] ||
	       iatm==bend_atom[ibend][3] ) &&
	      (jatm==bend_atom[ibend][1] ||
	       jatm==bend_atom[ibend][2] ||
	       jatm==bend_atom[ibend][3] )  ) { valid = 0;}
        }/*end for*/
      }/*end if*/
      /*Check four-atom bond connection: iatm--katm--latm--jatm */
      if ((valid==1)&&(inter_on_3bond_conn==0)&&(inter_on_4bond_conn==0)) {
	for (ibond_atm=1; ibond_atm<=nbond_atm[iatm]; ibond_atm++) {
	  katm=bond_atm_map[iatm][ibond_atm];
	  for (kbond_atm=1; kbond_atm<=nbond_atm[katm]; kbond_atm++) {
	    latm=bond_atm_map[katm][kbond_atm];
	    for (lbond_atm=1; lbond_atm<=nbond_atm[latm]; lbond_atm++) {
	      if (jatm==bond_atm_map[latm][lbond_atm]) {
		valid = 0;
		break;  /*Not necessary to continue. */
	      }/*end if*/
	    }/*end for*/ 
	  }/*end for*/
	}/*end for*/
        for (ionfo=1; ionfo<=nonfo; ionfo++) {
	  if ((iatm==onfo_atom[ionfo][1] ||
	       iatm==onfo_atom[ionfo][2] ) &&
              (jatm==onfo_atom[ionfo][1] ||
	       jatm==onfo_atom[ionfo][2] )  ) { valid = 0;}
        }/*end for*/
        for (itors=1; itors<=ntors; itors++) {
	  if ((iatm==tors_atom[itors][1] ||
	       iatm==tors_atom[itors][2] ||
	       iatm==tors_atom[itors][3] ||
	       iatm==tors_atom[itors][4] ) &&
	      (jatm==tors_atom[itors][1] ||
	       jatm==tors_atom[itors][2] ||
	       jatm==tors_atom[itors][3] ||
	       jatm==tors_atom[itors][4] )  ) { valid = 0;}
        }/*end for*/
      }/*end if*/
      if (valid==0) {
         nexclude++;
         n_entry_now++;
         if (n_entry_now>n_entry_mall) {
             n_entry_old   = n_entry_mall;
             n_entry_mall += NMEM_MIN;
             exclude_atom     =  creall_int_mat((int **)exclude_atom, 
       		   		         1, n_entry_old, 1, 2,
       				         1, n_entry_mall, 1, 2);
         }/*end if*/
         exclude_atom[nexclude][1] = iatm;
         exclude_atom[nexclude][2] = jatm;
      }/*end if*/
    }/*end for-jatm */
  }/*end for-iatm*/

/*------------------------------------------------------------------------------*/
/* X) Fill in natm_chigrp, chigrp_atom, chiprim_atom                            */

  natm_chigrp  =  (int *)cmalloc(NCHIGRPMAX*sizeof(int))-1;  
  chigrp_atom  =  cmall_int_mat(1, NCHIGRPMAX, 1, NATM_CHIGRPMAX); 
  chiprim_atom =  cmall_int_mat(1, NCHIGRPMAX, 1, 4);

  nchigrp  = 0;
  for (i=1;i<=nline_resi;i++) {
    if (strcasecmp(rtf_resi[i].field[1],"CHIGROUP")==0) {
      nchigrp++;
      if (nchigrp>NCHIGRPMAX) {
          sprintf(err_msg,"nchigrp > NCHIGRPMAX");
          errorMessage(err_msg);             
      } 
      for (ifield=2;ifield<=rtf_resi[i].nfield;ifield++) {
        atm_field[1] = ifield; 
 
        size = 1; 
        for (isize=1;isize<=size;isize++) {/* atom[1] */
          atom_found[isize] = 0;
          for (iatom=1;iatom<=natom;iatom++) {
            if (strcasecmp(upac_atom_label[iatom],
                           rtf_resi[i].field[atm_field[isize]])==0) {
               atom_found[isize] = 1;
               atom[isize] =  iatom; 
               break; 
            }/* end if */                
	  }/* end for */
          if (atom_found[isize]==0) {
               sprintf(err_msg,"%dth atom not found on line %d in residue %s", 
                      atm_field[isize]-1,i,residue_name);
               errorMessage(err_msg);             
	  }/* end if */
        }/* end for isize<=size */

        if ( (atom_found[1]==1) ) {/* new group atom */
	     natm_chigrp[nchigrp]++;
             if (natm_chigrp[nchigrp]>NATM_CHIGRPMAX) {
               sprintf(err_msg,"natm_chigrp[nchigrp]>NATM_CHIGRPMAX ");
               errorMessage(err_msg);             
             }
             chigrp_atom[nchigrp][natm_chigrp[nchigrp]] = atom[1];
        }/* end if */

      }/* end for ifield<=nfield */	  
    }/* end if "CHIGROUP" */     
  }/* end for i<=nline_resi */   

  nchiprim = 0;
  for (i=1;i<=nline_resi;i++) {
    if (strcasecmp(rtf_resi[i].field[1],"CHIPRIM")==0) {
      nchiprim++;
      if (nchiprim>NCHIGRPMAX) {
          sprintf(err_msg,"nchiprim > NCHIGRPMAX");
          errorMessage(err_msg);             
      }
      for (ifield=2;ifield<=rtf_resi[i].nfield;ifield+=4) {
        atm_field[1] = ifield;
        atm_field[2] = ifield+1; 
        atm_field[3] = ifield+2;
        atm_field[4] = ifield+3;  

        size = 4;
        for (isize=1;isize<=size;isize++) {/* atom[1],atom[2],atom[3],atom[4] */
          atom_found[isize] = 0; 
          for (iatom=1;iatom<=natom;iatom++) {
            if (strcasecmp(upac_atom_label[iatom],
                           rtf_resi[i].field[atm_field[isize]])==0) {
               atom[isize] = iatom;
               atom_found[isize] = 1; 
               break; 
            }/* end if */                
  	  }/* end for */
          if (atom_found[isize]==0) {
              sprintf(err_msg,"%dth atom not found on line %d in residue %s", 
                      atm_field[isize]-1,i,residue_name);
              errorMessage(err_msg);             
          }/* end if */
        }/* end for isize<=size */
        chiprim_atom[nchiprim][1] =  atom[1];
        chiprim_atom[nchiprim][2] =  atom[2];
        chiprim_atom[nchiprim][3] =  atom[3];
        chiprim_atom[nchiprim][4] =  atom[4];
      }/* end for ifield<=nfield */	  
    }/* end if "CHIPRIM" */

  }/* end for i<=nline_resi */   

  if (nchiprim != nchigrp) {
      sprintf(err_msg,"NCHIPRIM != NCHIGRP in residue %s",residue_name);
      errorMessage(err_msg);             
  }/*endif*/

/*------------------------------------------------------------------------------*/
/* XI) Generate Root structures                                                 */
/* One atom cannot have more than 5 connections                                 */

   nbranch_root = 0; /* no root is assumed */ 

   if (general.cgres_model_Go==1) {/* Go model regardless of residue_name */ 
               strcpy(root_f,"CA");
               strcpy(root_s,"CA");
               nbranch_root = 2;
   }/*endif*/
   if ( (strcasecmp(residue_name,"ALA")==0) ||
        (strcasecmp(residue_name,"ARG")==0) ||
        (strcasecmp(residue_name,"ASN")==0) ||
        (strcasecmp(residue_name,"ASP")==0) || 
        (strcasecmp(residue_name,"ASC")==0) || 
        (strcasecmp(residue_name,"CYS")==0) || 
        (strcasecmp(residue_name,"GLN")==0) || 
        (strcasecmp(residue_name,"GLU")==0) || 
        (strcasecmp(residue_name,"GLY")==0) ||
        (strcasecmp(residue_name,"HIS")==0) || 
        (strcasecmp(residue_name,"HSD")==0) || 
        (strcasecmp(residue_name,"HDN")==0) || 
        (strcasecmp(residue_name,"HSE")==0) || 
        (strcasecmp(residue_name,"HSP")==0) || 
        (strcasecmp(residue_name,"ILE")==0) || 
        (strcasecmp(residue_name,"LEU")==0) || 
        (strcasecmp(residue_name,"LEN")==0) || 
        (strcasecmp(residue_name,"LYS")==0) || 
        (strcasecmp(residue_name,"LYN")==0) || 
        (strcasecmp(residue_name,"MET")==0) || 
        (strcasecmp(residue_name,"PHE")==0) || 
        (strcasecmp(residue_name,"PRO")==0) || 
        (strcasecmp(residue_name,"SER")==0) || 
        (strcasecmp(residue_name,"SEC")==0) || 
        (strcasecmp(residue_name,"THR")==0) || 
        (strcasecmp(residue_name,"TRP")==0) || 
        (strcasecmp(residue_name,"TYR")==0) || 
        (strcasecmp(residue_name,"VAL")==0) ) {
         if (general.cgres_model==1) {/*coarse model*/
	     if (general.cgres_model_1pt==1) {
                  strcpy(root_f,"CA");
                  strcpy(root_s,"CA");
             }
	     else if  (general.cgres_model_3pt==1) {
                  strcpy(root_f,"CA");
                  strcpy(root_s,"O");
             }
	     else if  (general.cgres_model_5pt==1) {
                  strcpy(root_f,"CA");
                  strcpy(root_s,"O");
             }
             else if  (general.cgres_model_UA==1) {
                  strcpy(root_f,"N");
                  strcpy(root_s,"C");
             }
             else if  (general.cgres_model_UAPH==1) {
                  strcpy(root_f,"N");
                  strcpy(root_s,"C");
             }
         }
         else if (general.allatm_model==1) {/*aa model*/
             strcpy(root_f,"N");
             strcpy(root_s,"C");
         }
         else {/*no model*/
            sprintf(err_msg,"No model is defined in Restop_data::setup:XI");
            errorMessage(err_msg);
         }
         nbranch_root = 2;
   }
   else if ( (strcasecmp(residue_name,"GUA")==0) ||
             (strcasecmp(residue_name,"ADE")==0) ||
             (strcasecmp(residue_name,"CYT")==0) || 
             (strcasecmp(residue_name,"THY")==0) || 
             (strcasecmp(residue_name,"URA")==0)  ) {
              if (general.cgres_model==1) {/*coarse model*/
	          if (general.cgres_model_1pt==1) {
                       strcpy(root_f,"C4*");
                       strcpy(root_s,"C4*");
                  }
	          else if  (general.cgres_model_3pt==1) {
                       strcpy(root_f,"P");
                       strcpy(root_s,"C4*");
                  }
	          else if  (general.cgres_model_5pt==1) {
                       strcpy(root_f,"P");
                       strcpy(root_s,"C4*");
                  }
                  else if  (general.cgres_model_UA==1) {
                       strcpy(root_f,"P");
                       strcpy(root_s,"O3'");
                  }
              }
              else  if (general.allatm_model==1) {/*aa model*/      
                 strcpy(root_f,"P");
                 strcpy(root_s,"O3'");
              }
              else {/*no model*/
                sprintf(err_msg,"No model is defined in Restop_data::setup:XI");
                errorMessage(err_msg);
              }
              nbranch_root = 2;
   }
   else if ( (strcasecmp(residue_name,"NTER")==0) ||
             (strcasecmp(residue_name,"GLYP")==0)  ) {
              strcpy(root_f,"CA");
              nbranch_root = 1;
   }
   else if ( (strcasecmp(residue_name,"CTER")==0) ||
             (strcasecmp(residue_name,"CT2")==0) ||
             (strcasecmp(residue_name,"CT3")==0) ) {
              strcpy(root_f,"C");
              nbranch_root = 1;
   }
    

   branch_root_f  =  cmall_int_mat(0, 10, 0, 10);
   branch_root_s  =  cmall_int_mat(0, 10, 0, 10);

   if (nbranch_root == 1) {/* get the one root */
      found_root_1 = 0;
      for (iatom=1;iatom<=natom;iatom++) {
          if (strcasecmp(upac_atom_label[iatom],root_f)==0) {
   	     branch_root_f[0][0] = iatom;
             found_root_1 = 1;
             break;
          }   
      }  
      if ( found_root_1 == 0 ) {
        sprintf(err_msg,"root %s was not found in residue %s", 
                         root_f,residue_name);
        errorMessage(err_msg);
      }             
   } 
   else if (nbranch_root == 2) {/* get the two roots */
      found_root_1 = 0;
      found_root_2 = 0;
      for (iatom=1;iatom<=natom;iatom++) {
          if (strcasecmp(upac_atom_label[iatom],root_f)==0) {
   	     branch_root_f[0][0] = iatom;
             found_root_1 = 1;
          }   
          if (strcasecmp(upac_atom_label[iatom],root_s)==0) {
	     branch_root_s[0][0] = iatom;
             found_root_2 = 1;
          }
      }
      if ( found_root_1 == 0 ) {
        sprintf(err_msg,"root %s was not found in residue %s", 
                         root_f,residue_name);
        errorMessage(err_msg);
      }
      if ( found_root_2 == 0 ) {
        sprintf(err_msg,"root %s was not found in residue %s", 
                         root_s,residue_name);
        errorMessage(err_msg);
      }
   }

/*------------------------------------------------------------------------------*/
/* Get leafs at depth 1                                                         */   

  if (nbranch_root != 0) {
     nbranch_f1 = 0;
     nbranch_s1 = 0;
     for (ibond=1;ibond<=nbond;ibond++) {/* loop through bonds */
          if (bond_atom[ibond][1]==branch_root_f[0][0]) {
     	      nbranch_f1++;
	      branch_root_f[nbranch_f1][0] = bond_atom[ibond][2];       
          } 
          else if (bond_atom[ibond][2]==branch_root_f[0][0]) {
	      nbranch_f1++;
              branch_root_f[nbranch_f1][0] = bond_atom[ibond][1];       
          }
          if (nbranch_root == 2) {
             if (bond_atom[ibond][1]==branch_root_s[0][0]) {
  	         nbranch_s1++;
	         branch_root_s[nbranch_s1][0] = bond_atom[ibond][2];
             } 
             else if (bond_atom[ibond][2]==branch_root_s[0][0]) {
	         nbranch_s1++;
                 branch_root_s[nbranch_s1][0] = bond_atom[ibond][1];       
             }
          }/*end if*/     
     }/*end for*/
  }/*end if*/

/*------------------------------------------------------------------------------*/
/* Get leafs at depth 2                                                         */

  if (nbranch_root != 0) {
  nbranch_f2 = new int[nbranch_f1] - 1;
  for (ibranch=1;ibranch<=nbranch_f1;ibranch++) { 
      nbranch_f2[ibranch] = 0;
      for (ibond=1;ibond<=nbond;ibond++) {/* loop through bonds */
          if (bond_atom[ibond][1]==branch_root_f[ibranch][0]) {
	     if (bond_atom[ibond][2]!= branch_root_f[0][0]) { 
  	         nbranch_f2[ibranch]++;
                 branch_root_f[ibranch][nbranch_f2[ibranch]] = bond_atom[ibond][2];       
             } 
          } 
          else if (bond_atom[ibond][2]==branch_root_f[ibranch][0]) {
             if (bond_atom[ibond][1]!= branch_root_f[0][0]) { 
                 nbranch_f2[ibranch]++;
                 branch_root_f[ibranch][nbranch_f2[ibranch]] = bond_atom[ibond][1];       
             }
          }/*end if*/     
      }/*end for*/      
  }/*end for*/
  }/*end if*/
  if (nbranch_root == 2) {
  nbranch_s2 = new int[nbranch_s1] - 1;   
  for (ibranch=1;ibranch<=nbranch_s1;ibranch++) { 
      nbranch_s2[ibranch] = 0;
      for (ibond=1;ibond<=nbond;ibond++) {/* loop through bonds */
          if (bond_atom[ibond][1]==branch_root_s[ibranch][0]) {
	     if (bond_atom[ibond][2]!= branch_root_s[0][0]) { 
  	         nbranch_s2[ibranch]++;
                 branch_root_s[ibranch][nbranch_s2[ibranch]] = bond_atom[ibond][2];       
             } 
          } 
          else if (bond_atom[ibond][2]==branch_root_s[ibranch][0]) {
             if (bond_atom[ibond][1]!= branch_root_s[0][0]) { 
                 nbranch_s2[ibranch]++;
                 branch_root_s[ibranch][nbranch_s2[ibranch]] = bond_atom[ibond][1];    
             }
          }/*end if*/     
      }/*end for*/      
  }/*end for*/
  }/*end if*/


/*------------------------------------------------------------------------------*/
/* XII) DEBUG                                                                   */

#define DEBUG
#undef DEBUG
#ifdef DEBUG
  printf("############################ RESI ############################\n");
  for (i=1;i<=nline_resi;i++) {
    for (j=1;j<=rtf_resi[i].nfield;j++) {
        printf("%s ",rtf_resi[i].field[j]);    
    }
    printf("\n");
  }
  printf("############################ RESI ############################\n");
  getchar();
#endif
#undef DEBUG

#define DEBUG
#undef DEBUG
#ifdef DEBUG
  printf("############################ MASS ############################\n");
  for (i=1;i<=nline_mass;i++) {
    for (j=1;j<=rtf_mass[i].nfield;j++) {
        printf("%s ",rtf_mass[i].field[j]);    
    }
    printf("\n");
  }
  printf("############################ MASS ############################\n");
  getchar();
#endif
#undef DEBUG

#define DEBUG
#undef DEBUG
#ifdef DEBUG
  printf("############################ RESTOP ############################\n");
  printf("residue_type : %s\n",residue_type);
  printf("residue_name : %s\n",residue_name);
  printf("natom : %d\n",natom);
  printf("residue_chrg : %g\n",residue_chrg);
  for (j=1;j<=ngroup;j++) {
    printf("----------group----------\n");
    printf("atom_ind : group_ind : upac_atom : atom : charge : mass \n");      
      for (i=group_begin[j];i<=group_end[j];i++) {
          printf(" %d :",i);
   	  printf(" %d :",atom_gindex[i]);
          printf(" %s :",upac_atom_label[i]);
          printf(" %s :",atom_label[i]);
          printf(" %g :",charge[i]);
          printf(" %g \n",mass[i]);
      }
  }
  printf("\n----------bonds----------\n");
  printf("bond : ind_atm1 : ind_atm2 : lbl_atm1 : lbl_atm2\n");
  for (ibond=1;ibond<=nbond;ibond++) {
       printf("%d : %d : %d : %s : %s\n",ibond,bond_atom[ibond][1],bond_atom[ibond][2],
             upac_atom_label[bond_atom[ibond][1]],upac_atom_label[bond_atom[ibond][2]]);
  }
  printf("\n----------impr----------\n");
  printf("impr : ind_atm1 : ind_atm2 : ind_atm3 : ind_atm4\n");
  printf("impr : lbl_atm1 : lbl_atm2 : lbl_atm3 : lbl_atm4\n");
  for (iimpr=1;iimpr<=nimpr;iimpr++) {
      printf("%d : %d : %d : %d : %d\n",iimpr,impr_atom[iimpr][1],impr_atom[iimpr][2],
                                              impr_atom[iimpr][3],impr_atom[iimpr][4]);
      printf("%d : ",iimpr);
      for (i=1;i<=4;i++) {
          printf("%s ",upac_atom_label[impr_atom[iimpr][i]]);
      }
      printf("\n");
  }
  printf("\n----------bimpr----------\n");
  printf("impr : ind_atm1 : ind_atm2 : ind_atm3 : ind_atm4\n");
  printf("impr : lbl_atm1 : lbl_atm2 : lbl_atm3 : lbl_atm4\n");
  if (nbimpr>0) {
      printf("bimpr_atom_f\n");
      printf("%d : %d : %d : %d\n",bimpr_atom_f[1],bimpr_atom_f[2],
                                              bimpr_atom_f[3],bimpr_atom_f[4]);
      for (i=1;i<=4;i++) {
	  if (bimpr_atom_f[i]!=0) {
             printf("%s ",upac_atom_label[bimpr_atom_f[i]]);
          }
          else {
             printf("- ");
          }
      }
      printf("\n");
  }/* end if */
  if (nbimpr>1) {
      printf("bimpr_atom_s\n");
      printf("%d : %d : %d : %d\n",bimpr_atom_s[1],bimpr_atom_s[2],
                                              bimpr_atom_s[3],bimpr_atom_s[4]);
      for (i=1;i<=4;i++) {
	  if (bimpr_atom_s[i]!=0) {
             printf("%s ",upac_atom_label[bimpr_atom_s[i]]);
          }
          else {
             printf("+ ");
          }
      }
      printf("\n");
  }/* end if */
  
  printf("\n----------bends----------\n");
  printf("bend : ind_atm1 : ind_atm2 : ind_atm3\n");
  printf("bend : lbl_atm1 : lbl_atm2 : lbl_atm3\n");
  for (ibend=1;ibend<=nbend;ibend++) {
      printf("%d : %d : %d : %d\n",ibend,bend_atom[ibend][1],
                                         bend_atom[ibend][2],
                                         bend_atom[ibend][3]);
      printf("%d : %s : %s : %s\n",ibend,upac_atom_label[bend_atom[ibend][1]],
                                         upac_atom_label[bend_atom[ibend][2]],
                                         upac_atom_label[bend_atom[ibend][3]]);
  }
  printf("\n----------tors----------\n");
  printf("tors : ind_atm1 : ind_atm2 : ind_atm3 : ind_atm4\n");
  printf("tors : lbl_atm1 : lbl_atm2 : lbl_atm3 : lbl_atm4\n");
  for (itors=1;itors<=ntors;itors++) {
      printf("%d : %d : %d : %d : %d\n",itors,tors_atom[itors][1],
                                         tors_atom[itors][2],
                                         tors_atom[itors][3],
                                         tors_atom[itors][4]);
      printf("%d : %s : %s : %s : %s\n",itors,upac_atom_label[tors_atom[itors][1]],
                                         upac_atom_label[tors_atom[itors][2]],
                                         upac_atom_label[tors_atom[itors][3]],
                                         upac_atom_label[tors_atom[itors][4]]);
  }
  printf("\n----------onfos----------\n");
  printf("onfo : ind_atm1 : ind_atm2 : lbl_atm1 : lbl_atm2\n");
  for (ionfo=1;ionfo<=nonfo;ionfo++) {
       printf("%d : %d : %d : %s : %s\n",ionfo,onfo_atom[ionfo][1],onfo_atom[ionfo][2],
             upac_atom_label[onfo_atom[ionfo][1]],upac_atom_label[onfo_atom[ionfo][2]]);
  }
  printf("\n----------excludes----------\n");
  printf("exclude : ind_atm1 : ind_atm2 : lbl_atm1 : lbl_atm2\n");
  for (iexclude=1;iexclude<=nexclude;iexclude++) {
       printf("%d : %d : %d : %s : %s\n",iexclude,exclude_atom[iexclude][1],
                                                  exclude_atom[iexclude][2],
                                         upac_atom_label[exclude_atom[iexclude][1]],
                                         upac_atom_label[exclude_atom[iexclude][2]]);
  }
  printf("\n----------chigrp_atoms----------\n");
  printf("chigrp_atoms : ind_atm : lbl_atm\n");
  for (ichigrp=1;ichigrp<=nchigrp;ichigrp++) {
       printf("group %d ->",ichigrp);
       for (iatom=1;iatom<=natm_chigrp[ichigrp];iatom++) {
	    printf("%d:%d ",iatom,chigrp_atom[ichigrp][iatom]);
       } printf(" | ");
       for (iatom=1;iatom<=natm_chigrp[ichigrp];iatom++) {
            printf("%d:%s ",iatom,upac_atom_label[chigrp_atom[ichigrp][iatom]]);
       } printf("\n");
       printf("primtor %d ->",ichigrp);
       for (iatom=1;iatom<=4;iatom++) {
	    printf("%d:%d ",iatom,chiprim_atom[ichigrp][iatom]);
       } printf(" | ");
       for (iatom=1;iatom<=4;iatom++) {
            printf("%d:%s ",iatom,upac_atom_label[chiprim_atom[ichigrp][iatom]]);
       } printf("\n");
  }
  if (nbranch_root != 0) {
  printf("\n----------f_branch----------\n");
  printf("node : ind_atm : lbl_atm\n");
  printf("branch_root_f[0][0]:%d %s\n",branch_root_f[0][0],
                                       upac_atom_label[branch_root_f[0][0]]);
  for (ibranch=1;ibranch<=nbranch_f1;ibranch++) {
      printf("branch_root_f[%d][0]: %d %s\n",ibranch,
                                 branch_root_f[ibranch][0],
                                 upac_atom_label[branch_root_f[ibranch][0]]);
      for (iibranch=1;iibranch<=nbranch_f2[ibranch];iibranch++) {
          printf("  branch_root_f[%d][%d]: %d %s\n",ibranch,iibranch,
                                 branch_root_f[ibranch][iibranch],
                                 upac_atom_label[branch_root_f[ibranch][iibranch]]);
      }
  }/*end for*/
  }/*end if*/
  if (nbranch_root == 2) {
  printf("\n----------s_branch----------\n");
  printf("node : ind_atm : lbl_atm\n");
  printf("branch_root_s[0][0]:%d %s\n",branch_root_s[0][0],
                                       upac_atom_label[branch_root_s[0][0]]);
  for (ibranch=1;ibranch<=nbranch_s1;ibranch++) {
      printf("branch_root_s[%d][0]: %d %s\n",ibranch,
                                 branch_root_s[ibranch][0],
                                 upac_atom_label[branch_root_s[ibranch][0]]);
      for (iibranch=1;iibranch<=nbranch_s2[ibranch];iibranch++) {
          printf("  branch_root_s[%d][%d]: %d %s\n",ibranch,iibranch,
                                 branch_root_s[ibranch][iibranch],
                                 upac_atom_label[branch_root_s[ibranch][iibranch]]);
      }
  }/*end for*/
  }/*end if*/
  printf("############################ RESTOP ############################\n");
  getchar();
#endif

/*------------------------------------------------------------------------------*/
} /*end routine */
/*==============================================================================*/
