#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
/*-------------------------------------------------------------------------*/
void Regions::setup( const Region_data_entry *database, System *system )
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iregion, nentry;
  
/*-------------------------------------------------------------------------*/
/* I) Count the number of entries in database                              */

  const Region_data_entry *database_entry;
  
  nentry = 0;
  database_entry = database;
  while (database_entry!=NULL) {
    nentry++;
    database_entry = database_entry->next;
  }/*end while*/
  nregion = nentry;

/*-------------------------------------------------------------------------*/
/* II) Allocate variables                                                  */

  region = new REgion[nregion] - 1;

/*-------------------------------------------------------------------------*/
/* III) Setup each region                                                  */

  iregion = 0;
  database_entry = database;
  while (database_entry!=NULL) {
    iregion++;
    region[iregion].setup(database_entry,system,iregion);
    database_entry = database_entry->next;
  }/*end while*/

/*-------------------------------------------------------------------------*/
/* IV) Allocate related variables in maps                                  */

  Maps *maps = &(system->maps);

  int  nmol;
  int nrseg_imol, ncseg_imol;
  int imol,iseg,ires,ibres_mol_id;
  int ifres,ibres,ieres;

  int  *nrseg_mol;              /* # of rigid sgmnts per molecule, Lth: nmol*/
  int **ind_rseg_begin;         /*   residue index, Lth: nmol  x nseg       */
  int **ind_rseg_central;       /*   residue index, Lth: nmol  x nseg       */
  int **ind_rseg_end;           /*   residue index, Lth: nmol  x nseg       */
  int **rseg_length;            /*   segment length, Lth: nmol x nseg       */

  int  *ncseg_mol;              /* # of flxb sgmnts per molecule, Lth: nmol */
  int **ind_cseg_begin;         /*   residue index, Lth: nmol  x ncseg_imol */
  int **ind_cseg_end;           /*   residue index, Lth: nmol  x ncseg_imol */
  int **cseg_length;            /*   segment length, Lth: nmol x ncseg_imol */

  nmol             = maps->nmol;

  nrseg_mol        = new int[nmol]  - 1;
  ind_rseg_begin   = new int*[nmol] - 1;
  ind_rseg_central = new int*[nmol] - 1;
  ind_rseg_end     = new int*[nmol] - 1;
  rseg_length      = new int*[nmol] - 1;

  ncseg_mol        = new int[nmol]  - 1;
  ind_cseg_begin   = new int*[nmol] - 1;
  ind_cseg_end     = new int*[nmol] - 1;
  cseg_length      = new int*[nmol] - 1;

  for (imol=1;imol<=nmol;imol++) {
       nrseg_imol = 0;
       ncseg_imol = 0;
       for (iregion=1;iregion<=nregion;iregion++) {
            if ( (region[iregion].element_top_type==3) &&
                 (region[iregion].dependency_type==1) ) {
                  for (iseg=1;iseg<=region[iregion].nseg;iseg++) {
                       ibres = region[iregion].ind_seg_baseres[iseg];
                       ibres_mol_id = maps->res_mol_id[ibres];
                       if (ibres_mol_id == imol) {nrseg_imol++;}
                  }/*endfor-iseg*/
            }/*endif*/
            if ( (region[iregion].element_top_type==3) &&
                 (region[iregion].dependency_type==2) ) {
                  for (iseg=1;iseg<=region[iregion].nseg;iseg++) {
                       ibres = region[iregion].ind_seg_baseres[iseg];
                       ibres_mol_id = maps->res_mol_id[ibres];
                       if (ibres_mol_id == imol) {ncseg_imol++;}
                  }/*endfor-iseg*/
            }/*endif*/
       }/*endfor-iregion*/
       nrseg_mol[imol]        = nrseg_imol;
       ind_rseg_begin[imol]   = new int[nrseg_imol];
       ind_rseg_central[imol] = new int[nrseg_imol];
       ind_rseg_end[imol]     = new int[nrseg_imol];
       rseg_length[imol]      = new int[nrseg_imol];

       ncseg_mol[imol]        = ncseg_imol;
       ind_cseg_begin[imol]   = new int[ncseg_imol];
       ind_cseg_end[imol]     = new int[ncseg_imol];
       cseg_length[imol]      = new int[nrseg_imol];

       nrseg_imol = 0;
       ncseg_imol = 0;
       for (iregion=1;iregion<=nregion;iregion++) {
            if ( (region[iregion].element_top_type==3) &&
                 (region[iregion].dependency_type==1) ) {
                  for (iseg=1;iseg<=region[iregion].nseg;iseg++) {
	               ifres = region[iregion].ind_seg_firstres[iseg];
   	               ibres = region[iregion].ind_seg_baseres[iseg];
                       ieres = region[iregion].ind_seg_lastres[iseg];
                       ibres_mol_id = maps->res_mol_id[ibres];
                       if (ibres_mol_id == imol) {
                           nrseg_imol++;
                           ind_rseg_begin[imol][nrseg_imol]   = ifres;
                           ind_rseg_central[imol][nrseg_imol] = ibres;
                           ind_rseg_end[imol][nrseg_imol]     = ieres;
                           rseg_length[imol][nrseg_imol] = ieres - ifres + 1;
                       }/*endif*/
                  }/*endfor-iseg*/
            }/*endif*/
            if ( (region[iregion].element_top_type==3) &&
                 (region[iregion].dependency_type==2) ) {
                  for (iseg=1;iseg<=region[iregion].nseg;iseg++) {
	               ifres = region[iregion].ind_seg_firstres[iseg];
   	               ibres = region[iregion].ind_seg_baseres[iseg];
                       ieres = region[iregion].ind_seg_lastres[iseg];
                       ibres_mol_id = maps->res_mol_id[ibres];
                       if (ibres_mol_id == imol) {
                           ncseg_imol++;
                           ind_cseg_begin[imol][ncseg_imol]   = ifres;
                           ind_cseg_end[imol][ncseg_imol]     = ieres;
                           cseg_length[imol][nrseg_imol] = ieres - ifres + 1;
                       }/*endif*/
                  }/*endfor-iseg*/
            }/*endif*/
       }/*endfor-iregion*/       
  }/*endfor-imol*/

  maps->nrseg_mol        = nrseg_mol;
  maps->ind_rseg_begin   = ind_rseg_begin;
  maps->ind_rseg_central = ind_rseg_central;
  maps->ind_rseg_end     = ind_rseg_end;
  maps->rseg_length      = rseg_length;

  maps->ncseg_mol        = ncseg_mol;
  maps->ind_cseg_begin   = ind_cseg_begin;
  maps->ind_cseg_end     = ind_cseg_end;
  maps->cseg_length      = cseg_length;


/*-------------------------------------------------------------------------*/
/* V) DEBUG                                                                */

#ifdef DEBUG
  for (iregion=1;iregion<=nregion;iregion++) {
    //       region[iregion].write_alldata(system);
  }/*endfor*/
  for (imol=1;imol<=nmol;imol++) {
       printf("~~~~~~~~~~~~~~~~~~~~~~ Segment assignment ~~~~~~~~~~~~~~~~~~~~~~\n");
       printf("in molecule : %d\n",imol);
       printf("nrseg_mol[%d] : %d\n",imol,nrseg_mol[imol]);
       for (iseg=1;iseg<=nrseg_mol[imol];iseg++) {
	 printf("ind_rseg_begin[%d][%d]: %d\n",imol,iseg,ind_rseg_begin[imol][iseg]);
	 printf("ind_rseg_end[%d][%d]: %d\n",imol,iseg,ind_rseg_end[imol][iseg]);
	 printf("ind_rseg_central[%d][%d]: %d\n",imol,iseg,ind_rseg_central[imol][iseg]);
	 printf("rseg_length[%d][%d]: %d\n",imol,iseg,rseg_length[imol][iseg]);
       }/*endfor*/
      
       printf("ncseg_mol[%d] : %d\n",imol,ncseg_mol[imol]);
       for (iseg=1;iseg<=ncseg_mol[imol];iseg++) {
       	 printf("ind_cseg_begin[%d][%d]: %d\n",imol,iseg,ind_cseg_begin[imol][iseg]);
	 printf("ind_cseg_end[%d][%d]: %d\n",imol,iseg,ind_cseg_end[imol][iseg]);
	 printf("cseg_length[%d][%d]: %d\n",imol,iseg,cseg_length[imol][iseg]);
       }/*endfor*/
      getchar();
  }/*endfor*/
#endif

/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/



/*=========================================================================*/
/*-------------------------------------------------------------------------*/
void REgion::setup( const Region_data_entry *database, System *system,
                                                       int index)
/*-------------------------------------------------------------------------*/
{ /*begin routine*/
/*-------------------------------------------------------------------------*/

  int ires, jres, imol;
  int ifreeres, iclbase, nres_global, num_clbase;
  int nres_local;

  int res_chainid_found, ind_res_found, clbase_found;
  int pairedres1_chainid_found, ind_pairedres1_found;
  int pairedres2_chainid_found, ind_pairedres2_found;
  int centeres_chainid_found, ind_centeres_found;
  int pairedres_found;
  
  int seg_firstres_chainid_found, ind_seg_firstres_found;
  int seg_lastres_chainid_found, ind_seg_lastres_found;
  int seg_baseres_chainid_found, ind_seg_baseres_found;

  int cblc_residue;

  int four, three;
  int itrans, num_trans;
  int ind_local_centeres, iclbase_centeres;
  int  *natm_trans;
  int **ind_trans;
  int **ind_prim_tors;
  int **ind_atm_clbase;

  char err_msg[LINE_NCHARMAX];

  Res_data_entry *res_database;
  Tors_defs      *tors_defs_sys;
  Maps           *maps;

/*-------------------------------------------------------------------------*/
/* 0) system variables                                                     */
 
  nres_global    = system->pos.nres;
  res_database   = system->res_database;
  tors_defs_sys      = &(system->tors_defs);
  maps           = &(system->maps);
  num_clbase     = tors_defs_sys->num_clbase;
  ind_atm_clbase = tors_defs_sys->ind_atm_clbase;
  
/*-------------------------------------------------------------------------*/
/* I.0) Setup scalar variables                                             */
  
  region_index              =  index;
  element_top_type          =  database->element_top_type;
  dependency_type           =  database->dependency_type;
  natom                     =  database->natom;
  nres                      =  database->nres;
  nseg                      =  database->nseg;
  nrespair                  =  database->nrespair;
  ncenteres                 =  database->ncenteres;

  prop_trans_sig            =  database->prop_trans_sig;
  prop_rot_sig              =  database->prop_rot_sig;
  prop_trans_sig_freeres    =  database->prop_trans_sig_freeres;
  prop_rot_sig_freeres      =  database->prop_rot_sig_freeres;
  prop_trans_sig_respair    =  database->prop_trans_sig_respair;
  prop_rot_sig_respair      =  database->prop_rot_sig_respair;
  prop_trans_sig_respair_internal = database->prop_trans_sig_respair_internal;
  prop_rot_sig_respair_internal   = database->prop_rot_sig_respair_internal;

  
/*-------------------------------------------------------------------------*/
/* I.1) Decide on element_top_type type                                    */

  if (element_top_type == 2) {
      nseg = 0;
  }
  else if (element_top_type == 3) {
      nres     = 0;
      nrespair = 0;
  }
  else {
	sprintf(err_msg, "the current element_top_type is not supported");
        errorMessage(err_msg);  
  }/*endif*/
  
  nfreeres                  =  nres - 2*nrespair; /* make an error message */
  
/*-------------------------------------------------------------------------*/
/* II) Allocate variables                                                  */

 if (element_top_type == 2) {/*residue*/
  ind_res                   =      new int[nres] - 1;
  ind_clbase_res            =      new int[nres] - 1;
  ind_freeres               =      new int[nfreeres] - 1;
  ind_clbase_freeres        =      new int[nfreeres] - 1;
  ind_pairedres1            =      new int[nrespair] - 1;
  ind_pairedres2            =      new int[nrespair] - 1;
  ind_clbase_pairedres1     =      new int[nrespair] - 1;
  ind_clbase_pairedres2     =      new int[nrespair] - 1;
  ind_centeres              =      new int[ncenteres] - 1;
  ind_clbase_centeres       =      new int[ncenteres] - 1;
 }/*endif*/

 if (element_top_type == 3) {/*segment*/
  ind_seg_firstres          =      new int[nseg] - 1;
  ind_seg_lastres           =      new int[nseg] - 1;
  ind_seg_baseres           =      new int[nseg] - 1;
  ind_clbase_seg_baseres    =      new int[nseg] - 1;
  ind_centeres              =      new int[ncenteres] - 1;
  ind_clbase_centeres       =      new int[ncenteres] - 1;
 }/*endif*/

/*-------------------------------------------------------------------------*/
/* III) Find global residue indices                                        */

  for (ires=1;ires<=nres_global;ires++) {
       for (jres=1;jres<=ncenteres;jres++) {/*residue & segment*/
            centeres_chainid_found = 0;
            ind_centeres_found     = 0;
            if (strcasecmp(res_database[ires].chain_id,
                database->centeres_chainid[jres])==0)
               {centeres_chainid_found = 1;}
            if (res_database[ires].residue_number_old==
	        database->ind_centeres[jres])
	       {ind_centeres_found = 1;}
            if ((centeres_chainid_found == 1) && (ind_centeres_found == 1))
	       {ind_centeres[jres] = ires;}
       }/*endfor*/

       for (jres=1;jres<=nres;jres++) {/*residue*/
            res_chainid_found = 0;
            ind_res_found     = 0;
            if (strcasecmp(res_database[ires].chain_id,
                      database->res_chainid[jres])==0)
                     {res_chainid_found = 1;}
            if (res_database[ires].residue_number_old==
	              database->ind_res[jres])
	             {ind_res_found = 1;}
            if ( (res_chainid_found == 1) &&
                  (ind_res_found == 1) )
	          {ind_res[jres] = ires; break;}
       }/*endfor-jres*/
       
       for (jres=1;jres<=nrespair;jres++) {/*residue*/
            pairedres1_chainid_found = 0;
            ind_pairedres1_found     = 0;
            pairedres2_chainid_found = 0;
            ind_pairedres2_found     = 0;
            if (strcasecmp(res_database[ires].chain_id,
                      database->pairedres1_chainid[jres])==0)
                     {pairedres1_chainid_found = 1;}
            if (res_database[ires].residue_number_old==
	              database->ind_pairedres1[jres])
	             {ind_pairedres1_found = 1;}
            if ( (pairedres1_chainid_found == 1) &&
                  (ind_pairedres1_found == 1) )
	          {ind_pairedres1[jres] = ires;}

            if (strcasecmp(res_database[ires].chain_id,
                      database->pairedres2_chainid[jres])==0)
                     {pairedres2_chainid_found = 1;}
            if (res_database[ires].residue_number_old==
	              database->ind_pairedres2[jres])
	             {ind_pairedres2_found = 1;}
            if ( (pairedres2_chainid_found == 1) &&
                  (ind_pairedres2_found == 1) )
	          {ind_pairedres2[jres] = ires;}

       }/*endfor-jres*/

       for (jres=1;jres<=nseg;jres++) {/*segment*/
            seg_firstres_chainid_found = 0;
            ind_seg_firstres_found = 0;
            if (strcasecmp(res_database[ires].chain_id,
                database->seg_firstres_chainid[jres])==0)
               {seg_firstres_chainid_found = 1;}
            if (res_database[ires].residue_number_old==
	        database->ind_seg_firstres[jres])
	       {ind_seg_firstres_found = 1;}
            if ((seg_firstres_chainid_found == 1) && (ind_seg_firstres_found == 1))
	       {ind_seg_firstres[jres] = ires;}
       }/*endfor*/

       for (jres=1;jres<=nseg;jres++) {/*segment*/
            seg_lastres_chainid_found = 0;
            ind_seg_lastres_found = 0;
            if (strcasecmp(res_database[ires].chain_id,
                database->seg_lastres_chainid[jres])==0)
               {seg_lastres_chainid_found = 1;}
            if (res_database[ires].residue_number_old==
	        database->ind_seg_lastres[jres])
	       {ind_seg_lastres_found = 1;}
            if ((seg_lastres_chainid_found == 1) && (ind_seg_lastres_found == 1))
	       {ind_seg_lastres[jres] = ires;}
       }/*endfor*/

       for (jres=1;jres<=nseg;jres++) {/*segment*/
            seg_baseres_chainid_found = 0;
            ind_seg_baseres_found = 0;
            if (strcasecmp(res_database[ires].chain_id,
                database->seg_baseres_chainid[jres])==0)
               {seg_baseres_chainid_found = 1;}
            if (res_database[ires].residue_number_old==
	        database->ind_seg_baseres[jres])
	       {ind_seg_baseres_found = 1;}
            if ((seg_baseres_chainid_found == 1) && (ind_seg_baseres_found == 1))
	       {ind_seg_baseres[jres] = ires;}
       }/*endfor*/

  }/*endfor-ires*/

  ifreeres = 0;
  for (ires=1;ires<=nres;ires++) {
       pairedres_found = 0;
       for (jres=1;jres<=nrespair;jres++) {
	    if ((ind_res[ires]==ind_pairedres1[jres]) ||
                (ind_res[ires]==ind_pairedres2[jres]) ) {
                 pairedres_found = 1;
                 break;
            }/*endif*/
       }/*endfor-jres*/
       if (pairedres_found==0) {
           ifreeres++;
           ind_freeres[ifreeres] = ind_res[ires];
       }/*endif*/
  }/*endfor-ires*/


/*-------------------------------------------------------------------------*/
/* IV.0) Find clbase indices                                               */
/*     Check if residues belong to cblc molecules                          */ 

 if (element_top_type == 2) {/*residue*/
  for (ires=1;ires<=nres;ires++) {
    cblc_residue = 0;
    for (imol=1;imol<=tors_defs_sys->nmol_cblc;imol++) {
      if (maps->res_mol_id[ind_res[ires]] == tors_defs_sys->ind_mol_cblc[imol])
	 {cblc_residue = 1;}
    }/*endfor*/
    clbase_found = 0;
    for (iclbase=1;iclbase<=num_clbase;iclbase++) {
      if (ind_res[ires] == tors_defs_sys->res_id_clbase[iclbase]) {
	  ind_clbase_res[ires] = iclbase;
          clbase_found = 1;
      }/*endif*/
    }/*endfor*/
    if (cblc_residue==0) {
	sprintf(err_msg, "the host chain of residue %s:%d "
		         "is not in CBLC",database->res_chainid[ires],
                                          database->ind_res[ires]);
        errorMessage(err_msg);
    }/*endif*/
    if (clbase_found==0) {
	sprintf(err_msg, "residue %s:%d does not have a clbase"
		                            ,database->res_chainid[ires]
		                            ,database->ind_res[ires]);
        errorMessage(err_msg);
    }/*endif*/
  }/*endfor*/
 }/*endif*/

 if (element_top_type == 3) {/*segment*/
  for (ires=1;ires<=nseg;ires++) {
    clbase_found = 0;
    for (iclbase=1;iclbase<=num_clbase;iclbase++) {
      if (ind_seg_baseres[ires] == tors_defs_sys->res_id_clbase[iclbase]) {
	  ind_clbase_seg_baseres[ires] = iclbase;
          clbase_found = 1;
      }/*endif*/
    }/*endfor*/
    if (clbase_found==0) {
	sprintf(err_msg, "residue %s:%d does not have a clbase"
		                            ,database->seg_baseres_chainid[ires]
		                            ,database->ind_seg_baseres[ires]);
        errorMessage(err_msg);
    }/*endif*/
  }/*endfor*/
 }/*endif*/


/*-------------------------------------------------------------------------*/
/* IV.1) ind_clbase_centeres                                               */

  for (jres=1;jres<=ncenteres;jres++) {
   if (element_top_type == 2) {/*residue*/
    for (ires=1;ires<=nres;ires++) {
         if (ind_res[ires] == ind_centeres[jres]) {
	     ind_local_centeres = ires; break;
         }/*endif*/
    }/*endfor*/ 
    ind_clbase_centeres[jres] = ind_clbase_res[ind_local_centeres];
   }/*endif*/

   if (element_top_type == 3) {/*segment*/
    clbase_found = 0;
    for (ires=1;ires<=nseg;ires++) {
         if (ind_seg_baseres[ires] == ind_centeres[jres]) {
	     ind_local_centeres = ires; 
             clbase_found = 1;  
             break;
         }/*endif*/
    }/*endfor*/
    if (clbase_found==0) {
	sprintf(err_msg, "\\centers{%s:%d} is not in \\segments_baseres{}"
		                            ,database->centeres_chainid[jres]
		                            ,database->ind_centeres[jres]);
        errorMessage(err_msg);
    }/*endif*/ 
    ind_clbase_centeres[jres] = ind_clbase_seg_baseres[ind_local_centeres];
   }/*endif*/
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* IV.1) ind_clbase_freeres                                                */

  for (ires=1;ires<=nfreeres;ires++) {
       for (jres=1;jres<=nres;jres++) {
            if (ind_res[jres] == ind_freeres[ires]) {
	         ind_clbase_freeres[ires] = ind_clbase_res[jres]; 
	         break;
            }/*endif*/
       }/*endif*/
  }/*endfor*/


/*-------------------------------------------------------------------------*/
/* IV.3) ind_clbase_pairedres1&2                                           */

  for (ires=1;ires<=nrespair;ires++) {
       for (jres=1;jres<=nres;jres++) {
            if (ind_res[jres] == ind_pairedres1[ires]) {
	         ind_clbase_pairedres1[ires] = ind_clbase_res[jres];
            }/*endif*/
            if (ind_res[jres] == ind_pairedres2[ires]) {
	         ind_clbase_pairedres2[ires] = ind_clbase_res[jres];
            }/*endif*/
       }/*endif*/
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* V) Allocate and set up tors_defs                                        */

  three = 3;
  four  = 4;

  tors_defs     = new Tors_defs[ncenteres] - 1;

  if (element_top_type == 2) {nres_local = nres;}
  if (element_top_type == 3) {nres_local = nseg;}
  
  num_trans     = nres_local - 1;

  for (jres=1;jres<=ncenteres;jres++) {
    tors_defs[jres].num_trans     = num_trans;
    tors_defs[jres].natm_trans    = new int[num_trans] - 1;
    tors_defs[jres].ind_trans     = cmall_int_mat(1, num_trans, 1, NATM_TRANS_MAX);
    tors_defs[jres].ind_prim_tors = cmall_int_mat(1, num_trans, 1, four);
  
    natm_trans    =  tors_defs[jres].natm_trans;
    ind_trans     =  tors_defs[jres].ind_trans;
    ind_prim_tors =  tors_defs[jres].ind_prim_tors;


    for (itrans=1;itrans<=num_trans;itrans++) {
         natm_trans[itrans] = three;
    }/*endfor*/
  
    iclbase_centeres    = ind_clbase_centeres[jres];
  
    itrans = 0;
    for (ires=1;ires<=nres_local;ires++) {
       if (element_top_type == 2) {/*residue*/
         if (ind_res[ires] != ind_centeres[jres]) {
             itrans++;
             iclbase = ind_clbase_res[ires];
             ind_trans[itrans][1] = ind_atm_clbase[iclbase][1];
             ind_trans[itrans][2] = ind_atm_clbase[iclbase][2];
             ind_trans[itrans][3] = ind_atm_clbase[iclbase][3];
             ind_prim_tors[itrans][4] = ind_atm_clbase[iclbase][1];
             ind_prim_tors[itrans][3] = ind_atm_clbase[iclbase_centeres][3];
             ind_prim_tors[itrans][2] = ind_atm_clbase[iclbase_centeres][2];
             ind_prim_tors[itrans][1] = ind_atm_clbase[iclbase_centeres][1];
         }/*endif*/
       }/*endif*/
       if (element_top_type == 3) {/*segment*/
         if (ind_seg_baseres[ires] != ind_centeres[jres]) {
             itrans++;
             iclbase = ind_clbase_seg_baseres[ires];
             ind_trans[itrans][1] = ind_atm_clbase[iclbase][1];
             ind_trans[itrans][2] = ind_atm_clbase[iclbase][2];
             ind_trans[itrans][3] = ind_atm_clbase[iclbase][3];
             ind_prim_tors[itrans][4] = ind_atm_clbase[iclbase][1];
             ind_prim_tors[itrans][3] = ind_atm_clbase[iclbase_centeres][3];
             ind_prim_tors[itrans][2] = ind_atm_clbase[iclbase_centeres][2];
             ind_prim_tors[itrans][1] = ind_atm_clbase[iclbase_centeres][1];
         }/*endif*/
       }/*endif*/
    }/*endfor*/ 
  
    tors_defs[jres].rotate_utils.setup();
  }/*endfor*/
   

/*-------------------------------------------------------------------------*/
}/*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/



/*=========================================================================*/
/*-------------------------------------------------------------------------*/
void REgion::write_alldata(System *system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm, ires, itrans;

  printf("~~~~~~~~~~~~~~~~~~~~~~ Region ~~~~~~~~~~~~~~~~~~~~~~\n");
  printf("region_index : %d\n",region_index);
  printf("dependency_type : %d\n",dependency_type);
  printf("element_top_type : %d\n",element_top_type);
  
  
  if (element_top_type==1) {
      printf("----------atoms----------\n");
      printf("natom : %d\n",natom);
      printf("atm_ind_glob : atm_ind_loc : atom_label : charge : mass \n");
  }


  if (element_top_type==2) {
      printf("----------center residues----------\n");
      printf("ncenteres : %d\n",ncenteres);
      printf("res_ind_glob : res_ind_loc : res_label : clbase_ind\n");
      for (ires=1;ires<=ncenteres;ires++) {
           printf(" %d : %d :",ind_centeres[ires],ires);
           printf(" %s :",system->maps.res_label[ind_centeres[ires]]);
           printf(" %d\n",ind_clbase_centeres[ires]);
      }/*endfor*/
      LINE

      printf("----------residues----------\n");
      printf("nres : %d\n",nres);
      printf("res_ind_glob : res_ind_loc : res_label : clbase_ind\n");
      for (ires=1;ires<=nres;ires++) {
           printf(" %d : %d :",ind_res[ires],ires);
           printf(" %s :",system->maps.res_label[ind_res[ires]]);
           printf(" %d\n",ind_clbase_res[ires]);
      }/*endfor*/
      LINE

      printf("----------free residues----------\n");
      printf("nfreeres : %d\n",nfreeres);
      printf("res_ind_glob : res_ind_loc : res_label \n");
      for (ires=1;ires<=nfreeres;ires++) {
           printf(" %d : %d :",ind_res[ires],ires);
           printf(" %s\n",system->maps.res_label[ind_res[ires]]);
      }/*endfor*/
      LINE

      printf("----------residue pairs----------\n");
      printf("nrespair : %d\n",nrespair);
      printf("pairedres1_ind_glob : pairedres1_ind_loc : pairedres1_label \n");      
      for (ires=1;ires<=nrespair;ires++) {
           printf(" %d : %d :",ind_pairedres1[ires],ires);
           printf(" %s\n",system->maps.res_label[ind_pairedres1[ires]]);
      }/*endfor*/
      printf("pairedres2_ind_glob : pairedres2_ind_loc : pairedres2_label \n");
      for (ires=1;ires<=nrespair;ires++) {
           printf(" %d : %d :",ind_pairedres2[ires],ires);
           printf(" %s\n",system->maps.res_label[ind_pairedres2[ires]]);
      }/*endfor*/
      LINE

      printf("----------step sizes----------\n");
      printf("prop_trans_sig : %g\n",prop_trans_sig);
      printf("prop_rot_sig : %g\n",prop_rot_sig);
      printf("prop_trans_sig_freeres : %g\n",prop_trans_sig_freeres);
      printf("prop_rot_sig_freeres : %g\n",prop_rot_sig_freeres);
      printf("prop_trans_sig_respair : %g\n",prop_trans_sig_respair);
      printf("prop_rot_sig_respair : %g\n",prop_rot_sig_respair);
      printf("prop_trans_sig_respair_internal : %g\n",
                                    prop_trans_sig_respair_internal);
      printf("prop_rot_sig_respair_internal : %g\n",
                                      prop_rot_sig_respair_internal);
      LINE

      printf("----------tors_defs----------\n");
      for (ires=1;ires<=ncenteres;ires++) {
          printf("------center residue------\n");
          printf("res_ind_local: %d | res_ind_global: %d\n",ires,ind_centeres[ires]);
          printf("num_trans : %d\n",tors_defs[ires].num_trans);
          for (itrans=1;itrans<=tors_defs[ires].num_trans;itrans++) {
             printf("natm_trans[%d] : %d\n",itrans,tors_defs[ires].natm_trans[itrans]);
             printf("ind_trans[%d][1] : %d\n",itrans,tors_defs[ires].ind_trans[itrans][1]);
             printf("ind_trans[%d][2] : %d\n",itrans,tors_defs[ires].ind_trans[itrans][2]);
             printf("ind_trans[%d][3] : %d\n",itrans,tors_defs[ires].ind_trans[itrans][3]);
             printf("ind_prim_tors[%d][1] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][1]);
             printf("ind_prim_tors[%d][2] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][2]);
             printf("ind_prim_tors[%d][3] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][3]);
             printf("ind_prim_tors[%d][4] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][4]);
          }/*endfor*/
      }/*endfor*/    
  }/*residues*/
  getchar();


  if (element_top_type==3) {
      printf("----------center residues----------\n");
      printf("ncenteres : %d\n",ncenteres);
      printf("res_ind_glob : res_ind_loc : res_label : clbase_ind\n");
      for (ires=1;ires<=ncenteres;ires++) {
           printf(" %d : %d :",ind_centeres[ires],ires);
           printf(" %s :",system->maps.res_label[ind_centeres[ires]]);
           printf(" %d\n",ind_clbase_centeres[ires]);
      }/*endfor*/
      LINE

      printf("----------segments base residue----------\n");
      printf("nseg : %d\n",nseg);
      printf("res_ind_glob : res_ind_loc : res_label : clbase_ind\n");
      for (ires=1;ires<=nseg;ires++) {
           printf(" %d : %d :",ind_seg_baseres[ires],ires);
           printf(" %s :",system->maps.res_label[ind_seg_baseres[ires]]);
           printf(" %d\n",ind_clbase_seg_baseres[ires]);
      }/*endfor*/
      LINE

      printf("----------segments first residue----------\n");
      printf("res_ind_glob : res_ind_loc : res_label\n");
      for (ires=1;ires<=nseg;ires++) {
           printf(" %d : %d :",ind_seg_firstres[ires],ires);
           printf(" %s\n",system->maps.res_label[ind_seg_firstres[ires]]);
      }/*endfor*/
      LINE

      printf("----------segments last residue----------\n");
      printf("res_ind_glob : res_ind_loc : res_label\n");
      for (ires=1;ires<=nseg;ires++) {
           printf(" %d : %d :",ind_seg_lastres[ires],ires);
           printf(" %s\n",system->maps.res_label[ind_seg_lastres[ires]]);
      }/*endfor*/
      LINE

      printf("----------step sizes----------\n");
      printf("prop_trans_sig : %g\n",prop_trans_sig);
      printf("prop_rot_sig : %g\n",prop_rot_sig);
      printf("prop_trans_sig_freeres : %g\n",prop_trans_sig_freeres);
      printf("prop_rot_sig_freeres : %g\n",prop_rot_sig_freeres);
      LINE

      printf("----------tors_defs----------\n");
      for (ires=1;ires<=ncenteres;ires++) {
          printf("------center residue------\n");
          printf("res_ind_local: %d | res_ind_global: %d\n",ires,ind_centeres[ires]);
          printf("num_trans : %d\n",tors_defs[ires].num_trans);
          for (itrans=1;itrans<=tors_defs[ires].num_trans;itrans++) {
             printf("natm_trans[%d] : %d\n",itrans,tors_defs[ires].natm_trans[itrans]);
             printf("ind_trans[%d][1] : %d\n",itrans,tors_defs[ires].ind_trans[itrans][1]);
             printf("ind_trans[%d][2] : %d\n",itrans,tors_defs[ires].ind_trans[itrans][2]);
             printf("ind_trans[%d][3] : %d\n",itrans,tors_defs[ires].ind_trans[itrans][3]);
             printf("ind_prim_tors[%d][1] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][1]);
             printf("ind_prim_tors[%d][2] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][2]);
             printf("ind_prim_tors[%d][3] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][3]);
             printf("ind_prim_tors[%d][4] : %d\n",itrans,
                                        tors_defs[ires].ind_prim_tors[itrans][4]);
          }/*endfor*/
      }/*endfor*/
   }/*segments*/


/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/
