#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG

/*=========================================================================*/
/*-------------------------------------------------------------------------*/
void Residue::setup( Maps *maps, Restop_data *restop_database, 
                                 Res_data_entry *res_database, Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/


  int iatm, jatm, atm_found;
  int iatm_global, ind_atm1_global;
  int ibranch, iibranch;
  int index_trans_loc2glob;
  int ibond, ibend, itors, iimpr, ionfo, iexclude, ichigrp;
  int itype;
  int size, nucl;

  int iatom_shared, iatom_type;
  char **atm_bb_label;
  char **atm_type_base_label;
  char   err_msg[LINE_NCHARMAX];  

#define DEBUG
#undef DEBUG
#ifdef DEBUG
  printf("res_database[%d].residue_label %s\n",
	                               res_database->residue_number,
                                       res_database->residue_label);
#endif
#undef DEBUG

/*-------------------------------------------------------------------------*/
/* I) Error checks in res_database, restop_database and between            */
/*    Atom number, unique atom list, missing atoms in database             */
  
  if (res_database->natom != restop_database->natom) {
     sprintf(err_msg,"Total %d atoms in %d. residue(%s) doesn't fit topology",
	              res_database->natom,
                      res_database->residue_number,
                      res_database->residue_label);
                      errorMessage(err_msg);
  }/*endif*/

  if (restop_database->unique_atom_list==1) { 
   for (iatm=1;iatm<=res_database->natom-1;iatm++) {
      for (jatm=iatm+1;jatm<=res_database->natom;jatm++) {
	  if (strcasecmp(res_database->atom_label[iatm],
                         res_database->atom_label[jatm])==0) {
                 sprintf(err_msg,"No unique atm label list in %s %d",
                                  res_database->residue_label,
                                  res_database->residue_number_old);
                 errorMessage(err_msg);
          } 
      }/*end for*/
   }/*end for*/
  }/*endif*/

  for (iatm=1;iatm<=res_database->natom;iatm++) {
      atm_found = 0;
      for (jatm=1;jatm<=restop_database->natom;jatm++) {
	  if (strcasecmp(res_database->atom_label[iatm],
                         restop_database->upac_atom_label[jatm])==0) {
                 atm_found = 1;
                 break;
          } 
      }/*end for*/
      if (atm_found==0) {
         sprintf(err_msg,"Atom %s in %s %d was not found in .rtf",
                 res_database->atom_label[iatm],
                 res_database->residue_label,
                 res_database->residue_number_old);
         errorMessage(err_msg);
      }/*end if*/
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* II) Fill in maps, pos variables                                         */

  ind_atm1_global = res_database->atom1_global_number;

  for (iatm=1;iatm<=restop_database->natom;iatm++) {/*top*/
      iatm_global = ind_atm1_global - 1 + iatm;
      strcpy(maps->atm_label[iatm_global],
                                  restop_database->atom_label[iatm]);
      strcpy(maps->upac_atm_label[iatm_global],
                                  restop_database->upac_atom_label[iatm]);
      maps->atm_res_id[iatm_global] = res_database->residue_number;
      pos->mass[iatm_global] = restop_database->mass[iatm];
      pos->chrg[iatm_global] = restop_database->charge[iatm];
      if (restop_database->unique_atom_list==1) {
        for (jatm=1;jatm<=res_database->natom;jatm++) {/*pdb*/
          if (strcasecmp(restop_database->upac_atom_label[iatm],
                         res_database->atom_label[jatm])==0) {
                 pos->x[iatm_global] =  res_database->coord_x[jatm];
                 pos->y[iatm_global] =  res_database->coord_y[jatm];
                 pos->z[iatm_global] =  res_database->coord_z[jatm];
                 break;
          }/*end if*/
        }/*end for*/
      } else {
                 pos->x[iatm_global] =  res_database->coord_x[iatm];
                 pos->y[iatm_global] =  res_database->coord_y[iatm];
                 pos->z[iatm_global] =  res_database->coord_z[iatm];
      }/*endif*/
  }/*end for*/   

/*-------------------------------------------------------------------------*/
/* III) Fill in scalar/allocated residue variables                         */

  strcpy(residue_name,restop_database->residue_name);
  residue_index = res_database->residue_number;
  natom         = restop_database->natom;
  nbond         = restop_database->nbond;
  ndbond        = restop_database->ndbond;
  nbend         = restop_database->nbend;
  ntors         = restop_database->ntors;
  nimpr         = restop_database->nimpr; 
  nbimpr        = restop_database->nbimpr; 
  nonfo         = restop_database->nonfo;
  nexclude      = restop_database->nexclude;
  nchigrp       = restop_database->nchigrp;
  nbranch_root  = restop_database->nbranch_root;
  nbranch_f1    = restop_database->nbranch_f1;
  nbranch_s1    = restop_database->nbranch_s1;

/*-------------------------------------------------------------------------*/
/* IV) Allcate residue variables                                           */

  atom          =  new int[natom] - 1;
  bond          =  new int[nbond] - 1;  
  bend          =  new int[nbend] - 1;
  tors          =  new int[ntors] - 1;
  impr          =  new int[nimpr] - 1;
  onfo          =  new int[nonfo] - 1;
  exclude       =  new int[nexclude] - 1;
  natm_chigrp   =  new int[nchigrp] - 1;

  bond_atom     =  cmall_int_mat(1, nbond, 1, 2); 
  bend_atom     =  cmall_int_mat(1, nbend, 1, 3); 
  tors_atom     =  cmall_int_mat(1, ntors, 1, 4);
  impr_atom     =  cmall_int_mat(1, nimpr, 1, 4); 
  onfo_atom     =  cmall_int_mat(1, nonfo, 1, 2); 
  exclude_atom  =  cmall_int_mat(1, nexclude, 1, 2); 
  chigrp_atom   =  cmall_int_mat(1, nchigrp, 1, NATM_CHIGRPMAX);
  chiprim_atom  =  cmall_int_mat(1, nchigrp, 1, 4);

  size = 4;
  bimpr_atom_f = new int[size] - 1;  
  bimpr_atom_s = new int[size] - 1;  

  branch_root_f  =  cmall_int_mat(0,  MAX_BRANCH, 0,  MAX_BRANCH);
  branch_root_s  =  cmall_int_mat(0,  MAX_BRANCH, 0,  MAX_BRANCH);

  nbranch_f2  =  new int[MAX_BRANCH] - 1; 
  nbranch_s2  =  new int[MAX_BRANCH] - 1; 

/*-------------------------------------------------------------------------*/
/* V) Complete filling in residue variables                                */

  index_trans_loc2glob = ind_atm1_global - 1;

  for (iatm=1;iatm<=natom;iatm++) {
      atom[iatm]  =  iatm + index_trans_loc2glob;
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* V.a) Local residue connectivity                                         */

  size = 2;
  for (ibond=1;ibond<=nbond;ibond++) {
    for (jatm=1;jatm<=size;jatm++) {
        bond_atom[ibond][jatm]  = restop_database->bond_atom[ibond][jatm];
        bond_atom[ibond][jatm] += index_trans_loc2glob;
    }/*end for*/
  }/*end for*/

  size = 3;
  for (ibend=1;ibend<=nbend;ibend++) {
    for (jatm=1;jatm<=size;jatm++) {
        bend_atom[ibend][jatm]  = restop_database->bend_atom[ibend][jatm];
        bend_atom[ibend][jatm] += index_trans_loc2glob;
    }/*end for*/
  }/*end for*/

  size = 4;
  for (itors=1;itors<=ntors;itors++) {
    for (jatm=1;jatm<=size;jatm++) {
        tors_atom[itors][jatm]  = restop_database->tors_atom[itors][jatm];
        tors_atom[itors][jatm] += index_trans_loc2glob;
    }/*end for*/
  }/*end for*/

  size = 4;
  for (iimpr=1;iimpr<=nimpr;iimpr++) {
    for (jatm=1;jatm<=size;jatm++) {
        impr_atom[iimpr][jatm]  = restop_database->impr_atom[iimpr][jatm];
        impr_atom[iimpr][jatm] += index_trans_loc2glob;
    }/*end for*/
  }/*end for*/

  size = 2;
  for (ionfo=1;ionfo<=nonfo;ionfo++) {
    for (jatm=1;jatm<=size;jatm++) {
        onfo_atom[ionfo][jatm]  = restop_database->onfo_atom[ionfo][jatm];
        onfo_atom[ionfo][jatm] += index_trans_loc2glob;
    }/*end for*/
  }/*end for*/

  size = 2;
  for (iexclude=1;iexclude<=nexclude;iexclude++) {
    for (jatm=1;jatm<=size;jatm++) {
        exclude_atom[iexclude][jatm] = restop_database->exclude_atom[iexclude][jatm];
        exclude_atom[iexclude][jatm]+= index_trans_loc2glob;
    }/*end for*/
  }/*end for*/

  size = 4;
  if (nbimpr>0) {
     for (jatm=1;jatm<=size;jatm++) {
	 bimpr_atom_f[jatm]  = restop_database->bimpr_atom_f[jatm];
         if ( bimpr_atom_f[jatm] != 0 ) {
            bimpr_atom_f[jatm] += index_trans_loc2glob;
         }/*end if*/
     }/*end for*/
  }/*end if*/
  if (nbimpr>1) {
     for (jatm=1;jatm<=size;jatm++) {
	 bimpr_atom_s[jatm]  = restop_database->bimpr_atom_s[jatm];
         if ( bimpr_atom_s[jatm] != 0 ) {
            bimpr_atom_s[jatm] += index_trans_loc2glob;
         }/*end if*/
     }/*end for*/
  }/*end if*/

/*-------------------------------------------------------------------------*/
/* V.b) Side chain structure                                               */

  for (ichigrp=1;ichigrp<=nchigrp;ichigrp++) {
   natm_chigrp[ichigrp] = restop_database->natm_chigrp[ichigrp];
   for (iatm=1;iatm<=natm_chigrp[ichigrp];iatm++) {
    chigrp_atom[ichigrp][iatm]=restop_database->chigrp_atom[ichigrp][iatm];
    chigrp_atom[ichigrp][iatm]+=index_trans_loc2glob;
   }/*end for*/ 
   for (iatm=1;iatm<=4;iatm++) {
    chiprim_atom[ichigrp][iatm]=restop_database->chiprim_atom[ichigrp][iatm];
    chiprim_atom[ichigrp][iatm]+=index_trans_loc2glob;
   }/*end for*/ 
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* V.c) Branch root structure                                              */

  if (nbranch_root != 0) {
    branch_root_f[0][0] =  restop_database->branch_root_f[0][0];
    branch_root_f[0][0] += index_trans_loc2glob;
   for (ibranch=1;ibranch<=nbranch_f1;ibranch++) {
      branch_root_f[ibranch][0] = restop_database->branch_root_f[ibranch][0];
      branch_root_f[ibranch][0] += index_trans_loc2glob;
      nbranch_f2[ibranch] =  restop_database->nbranch_f2[ibranch];
      for (iibranch=1;iibranch<=nbranch_f2[ibranch];iibranch++) {
       branch_root_f[ibranch][iibranch] = restop_database->branch_root_f[ibranch][iibranch];
       branch_root_f[ibranch][iibranch] += index_trans_loc2glob;
      }/*end for*/
   }/*end for*/
  }/*end if*/

  if (nbranch_root == 2) {
    branch_root_s[0][0] =  restop_database->branch_root_s[0][0];
    branch_root_s[0][0] += index_trans_loc2glob;
   for (ibranch=1;ibranch<=nbranch_s1;ibranch++) {
      branch_root_s[ibranch][0] = restop_database->branch_root_s[ibranch][0];
      branch_root_s[ibranch][0] += index_trans_loc2glob;
      nbranch_s2[ibranch] =  restop_database->nbranch_s2[ibranch];
      for (iibranch=1;iibranch<=nbranch_s2[ibranch];iibranch++) {
       branch_root_s[ibranch][iibranch] = restop_database->branch_root_s[ibranch][iibranch];
       branch_root_s[ibranch][iibranch] += index_trans_loc2glob;
      }/*end for*/
   }/*end for*/
  }/*end if*/

/*-------------------------------------------------------------------------*/
/* VI) Decide if a residue has mutiple type                                */
/*     Currently supported mutiple type residues:                          */
/*     MBD: Multiple Base DNA, MBR: Multiple Base RNA,                     */
/*     MSP: Multiple Side Chain Protein                                    */
/*     Allocate and fill in related variables                              */  

   multiple_type = 0;
   nucl          = 0;

   if ( (strcasecmp(residue_name,"MBR")==0) ||
        (strcasecmp(residue_name,"MBD")==0) ) {nucl = 1;multiple_type = 1;}
         
   if (multiple_type == 0) {return;}

   if (nucl == 1) {
       ntype         = 4;       /* A C G U/T */
       natom_shared  = 18;
       if (strcasecmp(residue_name,"MBR")==0) {natom_shared++;}
   }/*endif*/


   natom_type       = new int[ntype] - 1;
   ind_atom_shared  = new int[natom_shared] - 1;
   type_weight      = new double[ntype] - 1;
   type_label       = (char **)cmall_int_mat(1,ntype,0,NNAMECHARMAX);

   if (nucl==1) {
       strcpy(type_label[1],"A"); natom_type[1] = 14;        
       strcpy(type_label[2],"C"); natom_type[2] = 12;        
       strcpy(type_label[3],"G"); natom_type[3] = 15;
       if (strcasecmp(residue_name,"MBD")==0) {
           strcpy(type_label[4],"T");
           natom_type[4] = 14;
       }/*endif*/ 
       if (strcasecmp(residue_name,"MBR")==0) {
           strcpy(type_label[4],"U");
           natom_type[4] = 11;
       }/*endif*/
   }/*endif*/

   max_natom_type = 0;
   for (itype=1;itype<=ntype;itype++) {
        max_natom_type = MAX(natom_type[itype],max_natom_type);
   }/*endfor*/   
   ind_atom_type = cmall_int_mat(1,ntype,1,max_natom_type);
   
   iatom_shared = 0;
   for (iatm=1;iatm<=natom;iatm++) {
          if (strcasecmp(residue_name,"MBR")==0)
           {atm_bb_label = maps->atm_rna_bb_label;}
          if (strcasecmp(residue_name,"MBD")==0)
           {atm_bb_label = maps->atm_dna_bb_label;}  
          for (jatm=1;jatm<=natom_shared;jatm++) {
            if (strcasecmp(atm_bb_label[jatm],
                     maps->atm_label[atom[iatm]])==0) {
	           iatom_shared++;
	           ind_atom_shared[iatom_shared] = iatm;
            }/*endif*/
          }/*endfor*/
   }/*iatm-endfor*/

   for (itype=1;itype<=ntype;itype++) {
       iatom_type = 0;
       if (strcasecmp(type_label[itype],"A")==0)
        {atm_type_base_label = maps->atm_ade_base_label;}
       if (strcasecmp(type_label[itype],"C")==0)
        {atm_type_base_label = maps->atm_cyt_base_label;}
       if (strcasecmp(type_label[itype],"G")==0)
        {atm_type_base_label = maps->atm_gua_base_label;}
       if (strcasecmp(type_label[itype],"T")==0)
        {atm_type_base_label = maps->atm_thy_base_label;}
       if (strcasecmp(type_label[itype],"U")==0)
        {atm_type_base_label = maps->atm_ura_base_label;}

        for (jatm=1;jatm<=natom_type[itype];jatm++) {
             for (iatm=1;iatm<=natom;iatm++) {
               if (strcasecmp(atm_type_base_label[jatm],
                    maps->atm_label[atom[iatm]])==0) {
    	              iatom_type++;
    	              ind_atom_type[itype][iatom_type] = iatm; 
               }/*endif*/
             }/*iatm-endfor*/
        }/*jatm-endfor*/
   }/*itype-endfor*/
 
/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/


/*=========================================================================*/
/*-------------------------------------------------------------------------*/
void Residue::write_alldata(Maps *maps, Pos *pos)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm;
  int ibond, ibend, itors, iimpr, ionfo, iexclude, ichigrp;
  int ibranch, iibranch;

  char   err_msg[LINE_NCHARMAX];  

  printf("~~~~~~~~~~~~~~~~~~~~~~ Residue ~~~~~~~~~~~~~~~~~~~~~~\n");
  printf("residue_name : %s\n",residue_name);

  printf("----------atoms----------\n");
  printf("natom : %d\n",natom);
  printf("atm_ind_glob : atm_ind_loc : atom_label : charge : mass \n");      
  for (iatm=1;iatm<=natom;iatm++) {
          printf(" %d : %d :",atom[iatm],iatm);
          printf(" %s :",maps->atm_label[atom[iatm]]);
          printf(" %g :",pos->chrg[atom[iatm]]);
          printf(" %g \n",pos->mass[atom[iatm]]);
  }

  printf("----------bonds----------\n");
  printf("nbond %d\n",nbond);
  printf("ind_glob ind_loc-> ");
  printf(" ind_atm1 : ind_atm2 |");
  printf(" lbl_atm1 : lbl_atm2\n");
  for (ibond=1;ibond<=nbond;ibond++) {
      printf("%d %d-> ",bond[ibond],ibond);
      printf("%d : %d | ",bond_atom[ibond][1],bond_atom[ibond][2]);
      printf("%s : %s\n",maps->atm_label[bond_atom[ibond][1]], 
                         maps->atm_label[bond_atom[ibond][2]]);
  }/*end for*/

  printf("----------impr----------\n");
  printf("nimpr %d\n",nimpr);
  printf("ind_glob ind_loc-> ");
  printf(" iatm1 : iatm2 : iatm3 : iatm4 |");
  printf(" latm1 : latm2 : latm3 : latm4\n");
  for (iimpr=1;iimpr<=nimpr;iimpr++) {
      printf("%d %d-> ",impr[iimpr],iimpr);
      printf("%d : %d : %d : %d | ",impr_atom[iimpr][1],impr_atom[iimpr][2],
                                    impr_atom[iimpr][3],impr_atom[iimpr][4]);
      printf("%s : %s : %s : %s\n",maps->atm_label[impr_atom[iimpr][1]], 
                                   maps->atm_label[impr_atom[iimpr][2]],
                                   maps->atm_label[impr_atom[iimpr][3]],
                                   maps->atm_label[impr_atom[iimpr][4]]);
  }/*end for*/

  printf("----------bimpr----------\n");
  printf("nbimpr %d\n",nbimpr);
  printf("bimpr_ind-> ");
  printf(" iatm1 : iatm2 : iatm3 : iatm4 |");
  printf(" latm1 : latm2 : latm3 : latm4\n");
  if (nbimpr>0) {
      printf("bimpr_atom_f-> ");
      printf("%d : %d : %d : %d |",bimpr_atom_f[1],bimpr_atom_f[2],
                                    bimpr_atom_f[3],bimpr_atom_f[4]);
      for (iatm=1;iatm<=4;iatm++) {
	  if (bimpr_atom_f[iatm]!=0) {
	     if (iatm==4) {
                printf(" %s",maps->atm_label[bimpr_atom_f[iatm]]);
             }
             else { 
                printf(" %s :",maps->atm_label[bimpr_atom_f[iatm]]);
             }/*end if*/
          }
          else {
             if (iatm==4) {
                printf(" -");
             }
             else {
                printf(" - :");
             }/*end if*/
          }/*end if*/
      }/*end for*/
      printf("\n");
  }/*end if*/
  if (nbimpr>1) {
      printf("bimpr_atom_s-> ");
      printf("%d : %d : %d : %d |",bimpr_atom_s[1],bimpr_atom_s[2],
                                    bimpr_atom_s[3],bimpr_atom_s[4]);
      for (iatm=1;iatm<=4;iatm++) {
	  if (bimpr_atom_s[iatm]!=0) {
	     if (iatm==4) {
                printf(" %s",maps->atm_label[bimpr_atom_s[iatm]]);
             }
             else { 
                printf(" %s :",maps->atm_label[bimpr_atom_s[iatm]]);
             }/*end if*/
          }
          else {
             if (iatm==4) {
                printf(" +");
             }
             else {
                printf(" + :");
             }/*end if*/
          }/*end if*/
      }/*end for*/
      printf("\n");
  }/*end if*/

  printf("----------bends----------\n");
  printf("nbend %d\n",nbend);
  printf("ind_glob ind_loc-> ");
  printf(" iatm1 : iatm2 : iatm3 |");
  printf(" latm1 : latm2 : latm3\n");
  for (ibend=1;ibend<=nbend;ibend++) {
       printf("%d %d-> ",bend[ibend],ibend);
       printf("%d : %d : %d | ",bend_atom[ibend][1],bend_atom[ibend][2],
                                                    bend_atom[ibend][3]);
       printf("%s : %s : %s\n",maps->atm_label[bend_atom[ibend][1]], 
                               maps->atm_label[bend_atom[ibend][2]],
                               maps->atm_label[bend_atom[ibend][3]]);
  }/*end for*/

  printf("----------tors----------\n");
  printf("ntors %d\n",ntors);
  printf("ind_glob ind_loc-> ");
  printf(" iatm1 : iatm2 : iatm3 : iatm4 |");
  printf(" latm1 : latm2 : latm3 : latm4\n");
  for (itors=1;itors<=ntors;itors++) {
      printf("%d %d-> ",tors[itors],itors);
      printf("%d : %d : %d : %d | ",tors_atom[itors][1],tors_atom[itors][2],
                                    tors_atom[itors][3],tors_atom[itors][4]);
      printf("%s : %s : %s : %s\n",maps->atm_label[tors_atom[itors][1]], 
                                   maps->atm_label[tors_atom[itors][2]],
                                   maps->atm_label[tors_atom[itors][3]],
                                   maps->atm_label[tors_atom[itors][4]]);
  }/*end for*/

  printf("----------onfo----------\n");
  printf("nonfo %d\n",nonfo);
  printf("ind_glob ind_loc-> ");
  printf(" ind_atm1 : ind_atm2 |");
  printf(" lbl_atm1 : lbl_atm2\n");
  for (ionfo=1;ionfo<=nonfo;ionfo++) {
      printf("%d %d-> ",onfo[ionfo],ionfo);
      printf("%d : %d | ",onfo_atom[ionfo][1],onfo_atom[ionfo][2]);
      printf("%s : %s\n",maps->atm_label[onfo_atom[ionfo][1]], 
                         maps->atm_label[onfo_atom[ionfo][2]]);
  }/*end for*/

  printf("----------exclude----------\n");
  printf("nexclude %d\n",nexclude);
  printf("ind_glob ind_loc-> ");
  printf(" ind_atm1 : ind_atm2 |");
  printf(" lbl_atm1 : lbl_atm2\n");
  for (iexclude=1;iexclude<=nexclude;iexclude++) {
      printf("%d %d-> ",exclude[iexclude],iexclude);
      printf("%d : %d | ",exclude_atom[iexclude][1],exclude_atom[iexclude][2]);
      printf("%s : %s\n",maps->atm_label[exclude_atom[iexclude][1]], 
                         maps->atm_label[exclude_atom[iexclude][2]]);
  }/*end for*/

  printf("\n----------chigrp_atoms----------\n");
  printf("chigrp_atoms : ind_atm : lbl_atm\n");
  for (ichigrp=1;ichigrp<=nchigrp;ichigrp++) {
       printf("group %d ->",ichigrp);
       for (iatm=1;iatm<=natm_chigrp[ichigrp];iatm++) {
	    printf("%d:%d ",iatm,chigrp_atom[ichigrp][iatm]);
       } 
       printf(" | ");
       for (iatm=1;iatm<=natm_chigrp[ichigrp];iatm++) {
            printf("%d:%s ",iatm,maps->atm_label[chigrp_atom[ichigrp][iatm]]);
       } 
       printf("\n");
       printf("primtor %d ->",ichigrp);
       for (iatm=1;iatm<=4;iatm++) {
	    printf("%d:%d ",iatm,chiprim_atom[ichigrp][iatm]);
       } 
       printf(" | ");
       for (iatm=1;iatm<=4;iatm++) {
            printf("%d:%s ",iatm,maps->atm_label[chiprim_atom[ichigrp][iatm]]);
       } 
       printf("\n");
  }/*end for*/

  if (nbranch_root != 0) {
  printf("\n----------f_branch----------\n");
  printf("node : ind_atm : lbl_atm\n");
  printf("branch_root_f[0][0]: %d : %s\n",branch_root_f[0][0],
                                          maps->atm_label[branch_root_f[0][0]]);
  for (ibranch=1;ibranch<=nbranch_f1;ibranch++) {
      printf("branch_root_f[%d][0]: %d : %s\n",ibranch,
	                       branch_root_f[ibranch][0],
                               maps->atm_label[branch_root_f[ibranch][0]]);
      for (iibranch=1;iibranch<=nbranch_f2[ibranch];iibranch++) {
          printf("  branch_root_f[%d][%d]: %d : %s\n",ibranch,iibranch,
                               branch_root_f[ibranch][iibranch],
                               maps->atm_label[branch_root_f[ibranch][iibranch]]);
      }
  }/*end for*/
  }/*end if*/
  if (nbranch_root == 2) {
  printf("\n----------s_branch----------\n");
  printf("node : ind_atm\n");
  printf("branch_root_s[0][0]: %d : %s\n",branch_root_s[0][0],
                                          maps->atm_label[branch_root_s[0][0]]);
  for (ibranch=1;ibranch<=nbranch_s1;ibranch++) {
      printf("branch_root_s[%d][0]: %d : %s\n",ibranch,
                                 branch_root_s[ibranch][0],
                                 maps->atm_label[branch_root_s[ibranch][0]]);
      for (iibranch=1;iibranch<=nbranch_s2[ibranch];iibranch++) {
          printf("  branch_root_s[%d][%d]: %d : %s\n",ibranch,iibranch,
                                 branch_root_s[ibranch][iibranch],
                                 maps->atm_label[branch_root_s[ibranch][iibranch]]);
      }
  }/*end for*/
  }/*end if*/
  printf("~~~~~~~~~~~~~~~~~~~~~~ End Residue ~~~~~~~~~~~~~~~~~~~~~~\n");
  getchar();

  
/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/


/*=========================================================================*/
/*-------------------------------------------------------------------------*/
void Residue::clean()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm, size;
  int ibond, iimpr, nbond_new, nimpr_new;
  int clean;
  int **bond_atom_save, **impr_atom_save;


/*-------------------------------------------------------------------------*/
/* I) Clean bonds from "0, -1" indices                                     */             
  size = 2;
  bond_atom_save  =  cmall_int_mat(1, nbond, 1, 2); 
  for (ibond=1;ibond<=nbond;ibond++) {
      for (iatm=1;iatm<=size;iatm++) {
   	  bond_atom_save[ibond][iatm] =  bond_atom[ibond][iatm]; 
      }/*end for*/
  }/*end for*/
  nbond_new = 0;
  for (ibond=1;ibond<=nbond;ibond++) {
      clean = 0;
      for (iatm=1;iatm<=size;iatm++) {
          if ( (bond_atom_save[ibond][iatm]==0) || 
               (bond_atom_save[ibond][iatm]==-1) ) {
	       clean = 1;
               break;     
	  }/*end if*/
      }/*end if*/
      if (clean==0) {
	 nbond_new++;
         for (iatm=1;iatm<=size;iatm++) {
	     bond_atom[nbond_new][iatm] = bond_atom_save[ibond][iatm];
         }/*end for*/
      }/*end if*/
  }/*end if*/   
  cfree_int_mat(bond_atom_save,1,nbond,1,2);
  nbond = nbond_new; 


/*-------------------------------------------------------------------------*/
/* I) Clean imprs from "0, -1" indices                                     */             
  size = 4;
  impr_atom_save  =  cmall_int_mat(1, nimpr, 1, 4); 
  for (iimpr=1;iimpr<=nimpr;iimpr++) {
      for (iatm=1;iatm<=size;iatm++) {
   	  impr_atom_save[iimpr][iatm] =  impr_atom[iimpr][iatm]; 
      }/*end for*/
  }/*end for*/
  nimpr_new = 0;
  for (iimpr=1;iimpr<=nimpr;iimpr++) {
      clean = 0;
      for (iatm=1;iatm<=size;iatm++) {
          if ( (impr_atom_save[iimpr][iatm]==0) || 
               (impr_atom_save[iimpr][iatm]==-1) ) {
	       clean = 1;
               break;     
	  }/*end if*/
      }/*end if*/
      if (clean==0) {
	 nimpr_new++;
         for (iatm=1;iatm<=size;iatm++) {
	     impr_atom[nimpr_new][iatm] = impr_atom_save[iimpr][iatm];
         }/*end for*/
      }/*end if*/
  }/*end if*/   
  cfree_int_mat(impr_atom_save,1,nimpr,1,4);
  nimpr = nimpr_new; 

/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/
