#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/global_system.h"


#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
void System::setup(char* input_filename) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/* 0) external global pointer to system defined in global_system.h         */
/*    initialize pos.system pointer                                        */
  global_system  = this;
  pos.system_ptr = this;


/*-------------------------------------------------------------------------*/
/* I) Setup all database files                                             */

  PAS_title();
  print_doubleline();
  printf("Reading simulation input file %s\n",input_filename);
  print_line();
  parse_input(&filenames,input_filename);          
  vomit_input(&filenames);                        
  checks();
  print_line();
  printf("Completed reading file %s\n",input_filename);
  print_doubleline();
  LINE
  LINE

  print_doubleline();
  printf("Reading potential database files \n");
  print_line();
  set_pot_data("bond",filenames.bond_database_filename);      
  set_pot_data("bend",filenames.bend_database_filename);      
  set_pot_data("tors",filenames.tors_database_filename);      
  set_pot_data("onfo",filenames.onfo_database_filename);      
  set_pot_data("inter",filenames.inter_database_filename);      
  if (general.inter_replace == 1)
     {set_pot_data("inter_replace",filenames.inter_replace_database_filename);}
  if (general.cryo_em_database_defined == 1)
     {set_pot_data("cryo_em",filenames.cryo_em_database_filename);}
  if (general.region_database_defined == 1)
     {set_region_data("region",filenames.region_database_filename);}
  if (general.constraint_database_defined == 1)
     {set_constraint_data("constraint",filenames.constraint_database_filename);}
  print_line();
  printf("Completed reading potential database files \n");
  print_doubleline();
  LINE
  LINE

  print_doubleline();
  printf("Creating atom maps\n");
  print_line();
  maps.setup(&filenames, &general);
  print_line();
  printf("Creating atom maps done\n");
  print_doubleline();
  LINE
  LINE


/*-------------------------------------------------------------------------*/
/* II) Setup system based on primitive topology                            */

  if (general.system_definition==1) {/*primitive topology*/
    print_doubleline();
    printf("Reading molecular parameter file %s\n",filenames.mol_parm_filename);
    print_line();
    pos.setup(filenames.mol_parm_filename, &maps, &general);
    print_line();
    printf("Completed reading molecular parameter file %s\n",filenames.mol_parm_filename);
    print_doubleline();
    LINE
    LINE

    print_doubleline();
    printf("Reading initial position file %s\n",filenames.pos_init_filename);
    print_line();
    pos.readCoord(filenames.pos_init_filename, &general);
    print_line();
    printf("Completed reading initial position file %s\n",filenames.pos_init_filename);
    print_doubleline();
    LINE
    LINE

    print_doubleline();
    printf("Setting up connectivity\n");
    print_line();
    if (general.energy_term_bond==1) 
      {bond.setup(&(bondprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_bend==1) 
      {bend.setup(&(bendprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_tors==1) 
      {tors.setup(&(torsprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_onfo==1) 
      {onfo.setup(&(onfoprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_inter==1) 
       {inter.setup(&(interprm_database[0]), &maps, this);}
    if (general.energy_term_cryo_em==1) 
       {cryo_em.setup(&(cryo_emprm_database[0]), this);}
    if (general.inter_nprw != 0)
       {inter.nonpair_setup(&(interprm_database[0]), &maps, &pos);}
    if (general.inter_replace == 1)
       {inter.replace_setup(&(inter_replaceprm_database[0]), &maps, this);}
    print_line();
    printf("Connectivity setup done\n");
    print_doubleline();
    LINE
    LINE
 }/*endif-primitive topology*/


/*-------------------------------------------------------------------------*/
/* III) Setup system based on residue topology                             */
 
 if (general.system_definition==2) {/*residue topology*/

    print_doubleline();
    printf("Reading file %s\n",filenames.pos_init_filename);
    print_line();
    set_pdb_data(filenames.pos_init_filename);
    set_res_data();
    print_line();
    printf("Completed reading file %s\n",filenames.pos_init_filename);
    print_doubleline();
    LINE
    LINE

    print_doubleline();
    printf("Reading residue topology file %s\n",filenames.mol_parm_filename);
    print_line();
    set_rtf_data(filenames.mol_parm_filename);
    print_line();
    printf("Completed reading file %s\n",filenames.mol_parm_filename);    
    print_doubleline();
    LINE
    LINE

    print_doubleline();
    printf("Setting up residue topology\n");
    print_line();
    pos.setup(&maps,restop_database,res_database,&general);
    print_line();
    printf("Residue topology setup done\n");    
    print_doubleline();
    LINE
    LINE

    if ( (general.prop_type==2) ||
         (general.prop_type==3) ) { /* 1: cartesian, 2: torsional, 3: bend */
        print_doubleline();
        printf("Setting up torsional transformations\n");
        print_line();
        tors_defs.setup(*this);
        print_line();
        printf("Torsional transformations setup done\n");    
        print_doubleline();
        LINE
        LINE
    }/*endif*/

    if ( (general.stage==1) ) {
        print_doubleline();
        printf("Setting up staging transformations\n");
        print_line();
        stage_defs.setup(&maps,&pos,&general);
        print_line();
        printf("Staging transformations setup done\n");    
        print_doubleline();
        LINE
        LINE
    }/*endif*/

    if ( (general.region_topology_defined==1) ) {
        print_doubleline();
        printf("Setting up region topology\n");
        print_line();
        regions.setup(&(regionprm_database[0]),this);
        print_line();
        printf("Region topology setup done\n");    
        print_doubleline();
        LINE
        LINE
    }/*endif*/


    print_doubleline();
    printf("Setting up connectivity\n");
    print_line();
    if (general.energy_term_bond==1) 
      {bond.setup(&(bondprm_database[0]), &maps, &pos);}
    if (general.energy_term_bend==1) 
      {bend.setup(&(bendprm_database[0]), &maps, &pos);}
    if (general.energy_term_tors==1) 
      {tors.setup(&(torsprm_database[0]), &maps, &pos);}
    if (general.energy_term_onfo==1) 
      {onfo.setup(&(onfoprm_database[0]), &maps, &pos);}
    if (general.energy_term_inter==1) 
       {inter.setup(&(interprm_database[0]), &maps, &pos);}
    if (general.energy_term_cryo_em==1) 
       {cryo_em.setup(&(cryo_emprm_database[0]), this);}
    if (general.inter_nprw != 0)
       {inter.nonpair_setup(&(interprm_database[0]), &maps, &pos);}
    if (general.inter_replace == 1)
       {inter.replace_setup(&(inter_replaceprm_database[0]), &maps, this);}
    print_line();
    printf("Connectivity setup done\n");
    print_doubleline();
    LINE
    LINE

    if ( (general.constraint_topology_defined==1) ) {
        print_doubleline();
        printf("Setting up constraint topology\n");
        print_line();
        constraint.setup(&(constraintprm_database[0]), &maps, &pos);
        print_line();
        printf("Constraint topology setup done\n");    
        print_doubleline();
        LINE
        LINE
    }/*endif*/

    print_doubleline();
    printf("System setup \n");
    print_line();
    pos.write_res_arrangement(&maps);
    print_line();
    printf("System setup done\n");    
    print_doubleline();
    LINE
    LINE

 }/*endif-residue topology*/


/*-------------------------------------------------------------------------*/
/* IV) Setup elastic network model                                         */

  if (general.enm==1) {
    print_doubleline();
    printf("Setting up Elastic Network Model\n");
    print_line();
    bond.setup_enm(&general, &maps, &pos);
    print_line();
    printf("Elastic Network Model setup done\n");
    print_doubleline();
    LINE
    LINE
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* V) Get initial energies                                                 */
/*    always      write basic energy terms: bond, bend, tors, onfo, inter  */
/*    if present  write additional energy terms: cryo_em                   */

  print_doubleline();
  pos.ebond    =  bond.getPot(&pos);
  pos.ebend    =  bend.getPot(&pos);
  pos.etors    =  tors.getPot(&pos);
  pos.eonfo    =  onfo.getPot(&pos);
  pos.einter   =  inter.getPot(&pos);
  pos.ecryo_em =  cryo_em.getPot(&pos);
  pos.econstraint = constraint.getPot(&pos);
  pos.epot = pos.ebond+pos.ebend+pos.etors+pos.eonfo+pos.einter+pos.ecryo_em
                      +pos.econstraint;
  if (general.write_energy_unit==1) {
     printf("Initial energies in kcal/mol\n");
     print_line();
     printf("Initial Bond energy %.16g\n",KCAL*pos.ebond);
     printf("Initial Bend energy %.16g\n",KCAL*pos.ebend);
     printf("Initial Torsion energy %.16g\n",KCAL*pos.etors);
     printf("Initial Onefour energy %.16g\n",KCAL*pos.eonfo);
     printf("Initial Inter energy %.16g\n",KCAL*pos.einter);
     if (general.energy_term_cryo_em==1) {
        printf("Initial Cryo-em energy %.16g\n",KCAL*pos.ecryo_em);
     }
     printf("Initial Constraint energy %.16g\n",KCAL*pos.econstraint);
     printf("Initial Total energy %.16g\n",KCAL*pos.epot);
  }
  else if (general.write_energy_unit==2) {
     printf("Initial energies in Ha\n");
     print_line();
     printf("Initial Bond energy %.16g\n",pos.ebond);
     printf("Initial Bend energy %.16g\n",pos.ebend);
     printf("Initial Torsion energy %.16g\n",pos.etors);
     printf("Initial Onefour energy %.16g\n",pos.eonfo);
     printf("Initial Inter energy %.16g\n",pos.einter);
     if (general.energy_term_cryo_em==1) {
        printf("Initial Cryo-em energy %.16g\n",pos.ecryo_em);
     }
     printf("Initial Constraint energy %.16g\n",pos.econstraint);
     printf("Initial Total energy %.16g\n",pos.epot);
  }
  print_line();
  printf("Initial energies\n");
  print_doubleline();
  LINE
  LINE


/*-------------------------------------------------------------------------*/
/* VI) Setup muticanonical algorithms                                      */

  if ((general.ptemp==1)||         /*PT  */
      (general.nmpt ==1)||         /*NMPT*/
      (general.action_type==2) )   /*EEMC, EEMC_PT*/
  {/*beginif*/ 
      print_doubleline();
      printf("Energy/Temparature Domain Partitioning\n");
      print_line();
      energy_part.setup(general);
      print_line();
      printf("Energy/Temparature Domain Partitioning Done\n");
      print_doubleline();
      LINE
      LINE
  } else {
      energy_part.setup_silent(general);
  }/*endif*/

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void System::setup_silent(char* input_filename) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/* external global pointer to system defined in global_system.h            */
/* initialize pos.system_ptr pointer                                       */
  global_system  = this;
  pos.system_ptr = this;


/*-------------------------------------------------------------------------*/
/* I) Setup all database files                                             */

  parse_input(&filenames,input_filename);          
  vomit_input(&filenames);                        
  checks();

  set_pot_data("bond",filenames.bond_database_filename);      
  set_pot_data("bend",filenames.bend_database_filename);      
  set_pot_data("tors",filenames.tors_database_filename);      
  set_pot_data("onfo",filenames.onfo_database_filename);      
  set_pot_data("inter",filenames.inter_database_filename);      
  if (general.inter_replace == 1)
     {set_pot_data("inter_replace",filenames.inter_replace_database_filename);}
  if (general.cryo_em_database_defined == 1)
     {set_pot_data("cryo_em",filenames.cryo_em_database_filename);}
  if (general.region_database_defined == 1)
     {set_region_data("region",filenames.region_database_filename);}

  maps.setup(&filenames, &general);


/*-------------------------------------------------------------------------*/
/* II) Setup system based on primitive topology                            */

  if (general.system_definition==1) {/*primitive topology*/

    pos.setup(filenames.mol_parm_filename, &maps, &general);
    pos.readCoord(filenames.pos_init_filename, &general);

    if (general.energy_term_bond==1) 
      {bond.setup(&(bondprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_bend==1) 
      {bend.setup(&(bendprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_tors==1) 
      {tors.setup(&(torsprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_onfo==1) 
      {onfo.setup(&(onfoprm_database[0]), &maps, filenames.mol_parm_filename);}
    if (general.energy_term_inter==1) 
       {inter.setup(&(interprm_database[0]), &maps, this);}
    if (general.energy_term_cryo_em==1) 
       {cryo_em.setup(&(cryo_emprm_database[0]), this);}
    if (general.inter_nprw != 0)
       {inter.nonpair_setup(&(interprm_database[0]), &maps, &pos);}
    if (general.inter_replace == 1) 
       {inter.replace_setup(&(inter_replaceprm_database[0]), &maps, this);}
 }/*primitive topology*/


/*-------------------------------------------------------------------------*/
/* III) Setup system based on residue topology                             */
 
 if (general.system_definition==2) {/*residue topology*/


    set_pdb_data(filenames.pos_init_filename);
    set_res_data();
    set_rtf_data(filenames.mol_parm_filename);
    pos.setup(&maps,restop_database,res_database,&general);

    if ( (general.prop_type==2) ||
         (general.prop_type==3) ) { /* 1: cartesian, 2: torsional, 3: bend */
        tors_defs.setup(*this);
    }/*endif*/

    if ( (general.stage==1) ) {
        stage_defs.setup(&maps,&pos,&general);
    }/*endif*/

    if ( (general.region_topology_defined==1) ) {
        regions.setup(&(regionprm_database[0]),this);
    }/*endif*/

    if (general.energy_term_bond==1) 
       {bond.setup(&(bondprm_database[0]), &maps, &pos);}
    if (general.energy_term_bend==1) 
       {bend.setup(&(bendprm_database[0]), &maps, &pos);}
    if (general.energy_term_tors==1)  
       {tors.setup(&(torsprm_database[0]), &maps, &pos);}
    if (general.energy_term_onfo==1) 
       {onfo.setup(&(onfoprm_database[0]), &maps, &pos);}
    if (general.energy_term_inter==1) 
       {inter.setup(&(interprm_database[0]), &maps, &pos);}
    if (general.energy_term_cryo_em==1) 
       {cryo_em.setup(&(cryo_emprm_database[0]), this);}
    if (general.inter_nprw != 0)
       {inter.nonpair_setup(&(interprm_database[0]), &maps, &pos);}
    if (general.inter_replace == 1)
       {inter.replace_setup(&(inter_replaceprm_database[0]), &maps, this);}
 }/*endif*/


/*-------------------------------------------------------------------------*/
/* IV) Setup elastic network model                                         */

  if (general.enm==1) {
    bond.setup_enm(&general, &maps, &pos);
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* V) Get initial energies                                                 */

  print_doubleline();
  pos.ebond    =  bond.getPot(&pos);
  pos.ebend    =  bend.getPot(&pos);
  pos.etors    =  tors.getPot(&pos);
  pos.eonfo    =  onfo.getPot(&pos);
  pos.einter   =  inter.getPot(&pos);
  pos.ecryo_em =  cryo_em.getPot(&pos);
  pos.epot = pos.ebond+pos.ebend+pos.etors+pos.eonfo+pos.einter+pos.ecryo_em;


/*-------------------------------------------------------------------------*/
/* VI) Setup muticanonical algorithms                                      */

  energy_part.setup_silent(general);

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Energy_part::setup(General &general)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int K;

  // energy system
  K = general.n_chains_mc;
  
  // energy bounds inherited from folded/unfolded conf in 
  // internal energy units [0.015,0.125] => [0,0.14]
  // H_lower  = -1.5;  
  // H_upper  = 1.5;
  H_ladder = new double [K+2];
  // temperature
  T        = new double[K+1];
  // energy partition the system
  partition(general);  

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Energy_part::setup_silent(General &general)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int K;

  // energy system
  K = general.n_chains_mc;
  
  // energy bounds inherited from folded/unfolded conf in 
  // internal energy units [0.015,0.125] => [0,0.14]
  // H_lower  = -1.5;  
  // H_upper  = 1.5;
  H_ladder = new double [K+2];
  // temperature
  T        = new double[K+1];
  // energy partition the system
  partition_silent(general);  

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Energy_part::partition(General &general) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i;
  double c,l1,l2; 
  int K       = general.n_chains_mc;
  double T0   = general.T;
  double H0   = H_lower;
  double HKp1 = H_upper;
  double* H   = H_ladder;
  double* H_  = new double[K+2];
     
  for (i=0;i<=K+1;i++) {
      H_[i] = pow(energy_gap,i);
  }

  l1 = H_[K+1] - H_[0];  
  l2 = HKp1 - H0;
  
  cout << " Levels in energy partitioning " << endl << endl;  

  for (i=0;i<=K+1;i++) {
      H[i] = H0+(H_[i]-H_[0])*l2/l1;
      cout << setprecision(3) << H[i] << " "; 
  }
  c = (H[1] - H[0])/T0;  // such that T[0] = T0
 
  cout << endl << endl;
  cout << " Temperature ladder" << endl << endl;
  
  for (i=0;i<=K;i++) {   
      T[i] = (H[i+1]-H[i])/c;
      cout << setprecision(3) << T[i] << " "; 
  }      
  cout << endl << endl;

  cout << " Temperature ladder in energy units" << endl << endl;
  
  for (i=0;i<=K;i++) {   
      T[i] /= BOLTZ;
      cout << setprecision(3) << T[i] << " "; 
  }      
  cout << endl << endl;

  cout << "current c value " << (H[1] - H[0])/T[0] << endl; 

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Energy_part::partition_silent(General &general) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i;
  double c,l1,l2; 
  int K       = general.n_chains_mc;
  double T0   = general.T;
  double H0   = H_lower;
  double HKp1 = H_upper;
  double* H   = H_ladder;
  double* H_  = new double[K+2];
     
  for (i=0;i<=K+1;i++) {
      H_[i] = pow(energy_gap,i);
  }

  l1 = H_[K+1] - H_[0];  
  l2 = HKp1 - H0;

  for (i=0;i<=K+1;i++) {
      H[i] = H0+(H_[i]-H_[0])*l2/l1;
  }
  c = (H[1] - H[0])/T0;  // such that T[0] = T0
 
  for (i=0;i<=K;i++) {   
      T[i] = (H[i+1]-H[i])/c;
  }      
  
  for (i=0;i<=K;i++) {   
      T[i] /= BOLTZ;
  }      

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
