#include "../include/standard_mpi.h"
#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/typedefs_mpi.h"
#include "../include/definitions.h"
#include "../include/definitions_mpi.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
/*=========================================================================*/
void conf_sampl_pt_mpi( System &system, Commun commun )
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
    int i,j,n,k,ii,iip1;
    double pi_iy,pi_ix,pi_ip1y,pi_ip1x;
    double mypot;
    int exch, myprop;
    long  *seed = &(system.general.seed);

/*-------------------------------------------------------------------------*/
/* 0) get system variables                                                 */

    int    nstep_mc = system.general.nstep_mc;
    int    nstat    = system.general.nstat;
    double pee      = system.general.pee;
    double pacc;
    double x_pot,y_pot;
    Coord  y;

    int B = system.general.B;                            /*burn in periods*/
    int N = system.general.N;
 
    int K        = system.general.n_chains_mc;
    int *order_X = system.maps.order_X;
    Chain *X;
    X = new Chain[K+1];
    double T[K+1];

    double pi_iy_p_ix;
    double pi_ip1x_p_ip1y;

/*-------------------------------------------------------------------------*/
/* I) get commun variables                                                 */
   
    MPI_Comm world = commun.world;
    int myid       = commun.myid;
    int numprocs   = commun.numprocs;

/*-------------------------------------------------------------------------*/
/* II) set up individual chains                                            */
 
    for (k=0;k<=K;k++) {
#ifdef DEBUG
   printf(" Check point III\n");
   getchar();
#endif 
        X[k].setup_scalar(k,system);
        order_X[k] = k;
#ifdef DEBUG
   printf(" Check point IV\n");
   getchar();
#endif 
        T[k] = system.energy_part.T[k];
    }
    X[myid].setup(myid,system);

/*-------------------------------------------------------------------------*/
/* III) Perform nstep_mc MCMC iterations                                   */
/* III.A) Make exchanges                                                   */
/* III.B) Propagate the individual chains                                  */

    if (pee>0) {exch = ((int)(1.0/pee));}
    else {exch = nstep_mc + 1;}
    for (n=1;n<=nstep_mc;n++) {
        /* III.A) */
        if ( ( (n % exch ) == 0 ) && ( n>(K)*(B+N) ) ) {
            for (i=1;i<=K;i++) {
	         MPI_Reduce(&(X[i].pos.epot),&mypot,1,
                                   MPI_DOUBLE,MPI_SUM,0,world);
                 if (myid==0) {X[i].pos.epot   = mypot-X[i].pos.epot;}
	         MPI_Reduce(&(X[i].pos.einter),&mypot,1,
                                   MPI_DOUBLE,MPI_SUM,0,world);
                 if (myid==0) {X[i].pos.einter = mypot-X[i].pos.einter;}
            }/*endfor*/
            MPI_Barrier(world);
            if (myid==0) {
                for (i=0;i<K;i+=2) {
                       ii   = order_X[i];
                       iip1 = order_X[i+1];

                       y_pot   = X[iip1].potential(system);
                       x_pot   = X[ii].potential(system);
                       pi_iy_p_ix  = (x_pot-y_pot)/T[i];
                       pi_ip1x_p_ip1y = (x_pot-y_pot)/T[i+1];
                       pacc    = MIN(1.0,exp(pi_iy_p_ix)*exp(pi_ip1x_p_ip1y));
                          X[i].nptprop += 1;
                          if ( pacc > ran1(seed) ) {
                             X[iip1].T     = T[i];
                             X[iip1].pos.T = T[i];
                             X[ii].T       = T[i+1];
                             X[ii].pos.T   = T[i+1];
                             order_X[i+1]  = ii;
                             order_X[i]    = iip1;
                             X[iip1].order = ii;
                             X[ii].order   = iip1;
                             X[i].nptacc += 1;
                          }
	        }/*endfor-K*/
                for (i=1;i<K;i+=2) {
                       ii   = order_X[i];
                       iip1 = order_X[i+1];    
                       y_pot   = X[iip1].potential(system);
                       x_pot   = X[ii].potential(system);
                       pi_iy_p_ix  = (x_pot-y_pot)/T[i];
                       pi_ip1x_p_ip1y = (x_pot-y_pot)/T[i+1];
                       pacc    = MIN(1.0,exp(pi_iy_p_ix)*exp(pi_ip1x_p_ip1y));
                          X[i].nptprop += 1;
                          if ( pacc > ran1(seed) ) {
                             X[iip1].T     = T[i];
                             X[iip1].pos.T = T[i];
                             X[ii].T       = T[i+1];
                             X[ii].pos.T   = T[i+1];
                             order_X[i+1]  = ii;
                             order_X[i]    = iip1;
                             X[iip1].order = ii;
                             X[ii].order   = iip1; 
                             X[i].nptacc += 1;
                          }
	        }/*endfor-K*/
            }/*endif-myid==0*/
            for (i=0;i<=K;i++) {
                 MPI_Barrier(world);
	         MPI_Bcast(&(X[i].order),1,MPI_INT,0,world);
                 MPI_Bcast(&(X[i].T),1,MPI_DOUBLE,0,world);
                 MPI_Bcast(&(X[i].pos.T),1,MPI_DOUBLE,0,world);
	    }/*endfor*/
            MPI_Bcast(&(system.maps.order_X[0]),numprocs,MPI_INT,0,world);
        } else {
           /* III.B) */
           ii   = order_X[myid];
           X[myid].prop(system);
           if (X[myid].order != X[myid].index) {//order != index//
               X[ii].nprop++;
               X[ii].ntprop++;
               if (X[myid].acc == 1) {X[ii].nacc++;}
           }//order != index//
        }/*endif-exchange*/

/*-------------------------------------------------------------------------*/
/* IV) Output coordinates                                                  */

        if ( (n>(K)*(B+N)) && ((n % nstat) == 0) ) {
            for (i=1;i<=K;i++) {
	         MPI_Reduce(&(X[i].pos.epot),&mypot,1,
                                   MPI_DOUBLE,MPI_SUM,0,world);
                 if (myid==0) {X[i].pos.epot   = mypot-X[i].pos.epot;}
	         MPI_Reduce(&(X[i].pos.einter),&mypot,1,
                                   MPI_DOUBLE,MPI_SUM,0,world);
                 if (myid==0) {X[i].pos.einter = mypot-X[i].pos.einter;}
            }/*endfor*/
#ifdef DEBUG
   printf(" Check point VIII\n");
   getchar();
#endif
           output_mpi(X,system,commun);           
#ifdef DEBUG
   printf(" Check point IX\n");
   getchar();
#endif 
        }/*endfor-output*/
    }/*endfor-nstep_mc*/ 

#ifdef DEBUG
   printf(" Check point X\n");
   getchar();
#endif 

/*-------------------------------------------------------------------------*/
/* V) Output Transition statistics                                         */

    MPI_Barrier(world);
    for (i=0;i<=K;i++) {
         MPI_Reduce(&(X[i].nprop),&myprop,1,MPI_INT,MPI_SUM,0,world);
         if (myid==0) {X[i].nprop = myprop;}
         MPI_Reduce(&(X[i].ntprop),&myprop,1,MPI_INT,MPI_SUM,0,world);
         if (myid==0) {X[i].ntprop = myprop;}
         MPI_Reduce(&(X[i].nacc),&myprop,1,MPI_INT,MPI_SUM,0,world);
         if (myid==0) {X[i].nacc = myprop;}
    }/*endfor*/

    if (myid==0) {
        statistics(X,system);
    }/*end if*/
#define DEBUG_ACC_OFF
#ifdef DEBUG_ACC
   printf(" Check point XI\n");
   printf("myid %d, X[myid].nacc %d, X[myid].nprop %d\n",myid,
                                                         X[myid].nacc,
                                                         X[myid].nprop);
//   getchar();
#endif 
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
