#include "../include/standard.h"
#include "../include/utillib.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"

#define DEBUG

/*=========================================================================*/
/*=========================================================================*/
int main(int argc, char *argv[])
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
//  error handling
    char err_msg[LINE_NCHARMAX];

//  input file 
    char *input_filename;
    input_filename = argv[1];

//  setup system
    System system;
    system.setup(input_filename);

//  action types
    int action_type  = system.general.action_type;

    int eemc_type    = system.general.eemc_type;
    int pt           = system.general.pt;
    int ptrepswa     = system.general.ptrepswa;
    int ptstage      = system.general.ptstage;
    int nmode        = system.general.nmode;
    int nmode_stg    = system.general.nmode_stg;
    int nmode_enm    = system.general.nmode_enm;
    int minim        = system.general.minimize;
    int mdrun        = system.general.md;
    int debug_on     = system.general.debug;

    int num_procs    = system.general.num_procs;
    int n_all_chains = system.general.n_chains_mc + 1; // 0,...,n_chains_mc

//  serial version
    if (num_procs != 1) {
        sprintf(err_msg,"serial version \\num_procs must be set to 1");
        errorMessage(err_msg);
    }

    switch (action_type) {
    case 1:   // conformational "sampling" and/or parallel tempering
        if (pt==1)        {conf_sampl_pt(system);}        // PT/PTMC
        if (ptrepswa==1)  {conf_sampl_ptrepswa(system);}  // PTRPSW/PTRPSWMC
        if (ptstage==1)   {conf_sampl_ptstage(system);}  // PTSTG/PTSTGMC
        if (nmode==1)     {normal_mode(system);}    // regular NM
        if (nmode_stg==1) {normal_mode_stg(system);}// regular NMSTG
        if (nmode_enm==1) {normal_mode_enm(system);}// regular NMEN
        if (minim==1)     {minimize(system);}       // regular MIN
        if (mdrun==1)     {md(system);}             // regular MD
        if (debug_on)     {debug(system);}          // debug system
      break; 
    case 2:   // equi-energy mc conformation sampling
        if (eemc_type==1) {conf_sampl_eemc(system);}     // pure eemc
        if (eemc_type==2) {conf_sampl_eemc_pt(system);}  // eemc with pt
      break;
    case 3:   // parallel tempering sequence sampling
        seq_sampl_pt(system);
      break;
    case 4:   // equi-energy mc sequence sampling
        seq_sampl_eemc(system);   
      break;
    }

// indicate the end of PAS
    PAS_end();
    return 0;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
