#include "../include/standard_mpi.h" 
#include "../include/standard.h" 
#include "../include/utillib.h"
#include "../include/typedefs.h"
#include "../include/typedefs_mpi.h"
#include "../include/definitions.h"
#include "../include/definitions_mpi.h"

#define DEBUG

/*=========================================================================*/
/*=========================================================================*/
int main(int argc, char *argv[])
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
    int iprocs;
    int mpi_supported;

// Initialize MPI
    Commun commun;
    MPI_Init(&argc,&argv);
    commun.world = MPI_COMM_WORLD;
    MPI_Comm_size(commun.world,&(commun.numprocs));
    MPI_Comm_rank(commun.world,&(commun.myid));

//  error handling
    char err_msg[LINE_NCHARMAX];

//  input file 
    char *input_filename;
    input_filename = argv[1];

//  setup system on each process
    System system;
    if ((commun.myid) == 0) {
         system.setup(input_filename);
    }
    MPI_Barrier((commun.world));
    for (iprocs=1;iprocs<commun.numprocs;iprocs++) {
         if (iprocs == (commun.myid)) {
             LINE
             LINE
             print_doubleline();
             printf("Setting up system on process %d\n",iprocs);
             print_line();
             system.setup_silent(input_filename);
             print_doubleline();
             printf("Completed setup on process %d\n",iprocs);
             print_line();
             LINE
             LINE
         }
         MPI_Barrier((commun.world));
    }/*end for*/

//  action types
    int action_type  = system.general.action_type;

    int eemc_type    = system.general.eemc_type;
    int pt           = system.general.pt;
    int ptrepswa     = system.general.ptrepswa;
    int ptstage      = system.general.ptstage;
    int nmode        = system.general.nmode;
    int nmode_stg    = system.general.nmode_stg;
    int nmode_enm    = system.general.nmode_enm;
    int minim        = system.general.minimize;
    int debug_on     = system.general.debug;


    int num_procs    = system.general.num_procs;
    int n_all_chains = system.general.n_chains_mc + 1; // 0,...,n_chains_mc


//  parallel version  
    if (num_procs <= 1) {
        sprintf(err_msg,"parallel version -> num_procs > 1 must hold");
        errorMessage(err_msg);
    }
    if (n_all_chains != num_procs) {
        sprintf(err_msg,"replica_number == num_procs +1 must hold\n");
        sprintf(err_msg+strlen(err_msg),"replicas: 0,...,replica_number");
        errorMessage(err_msg);
    }

//  MPI supported
    mpi_supported = 0;
    mpi_supported = pt + ptstage;

    if (mpi_supported == 0) {
        sprintf(err_msg,"current simulation type is not supported by MPI\n"
                        "presently supported: simulation_typ{PT,PTMC}");
        errorMessage(err_msg);
    }


    switch (action_type) {
    case 1:    // conformational sampling  
      if (pt == 1)      {conf_sampl_pt_mpi(system,commun);}
      if (ptstage == 1) {conf_sampl_ptstage_mpi(system,commun);}
      break; 
    default:   // equi-energy mc conformation sampling
        sprintf(err_msg, "Only simulation_typ{PT,PTMC,PTSTG,PTSTGMC}" 
                         "if num_procs > 1");
        errorMessage(err_msg); 
      break;
    }


// indicate the end of PAS
    if ((commun.myid) == 0) {
         PAS_end();
    }

//---------------------------------------------------------//
// close MPI environment
    MPI_Finalize();
    return 0;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

