#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
void minimize( System &system )
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

    int n;
    double pfunc;
    int* K              = &(system.general.n_chains_mc);
    int minimize_type   = system.general.minimize_type;
    int minimize_report = system.general.minimize_report;
    int prop_type       = system.general.prop_type;
    int clos_defined    = system.general.clos_defined;
    int nstep_mc        = system.general.nstep_mc;
    int nstat           = system.general.nstat;
    double T            = system.general.T;
    double sa_temp_first = system.general.sa_temp_first;
    double sa_temp_last  = system.general.sa_temp_last;
    double sa_rate       = system.general.sa_rate;

    int stsamc_type     = system.general.stsamc_type;
    int stsamc_period   = system.general.stsamc_period;
    double stsamc_ampl  = system.general.stsamc_ampl;
    double stsamc_shift = system.general.stsamc_shift;

    Chain X;
                               
    char   err_msg[LINE_NCHARMAX];
    char   warn_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Incompatibilities and Warnings                                       */

    if ( (prop_type     == 2) &&     /* tors     */
         (minimize_type <= 2) &&     /* cg, bfgs */
         (clos_defined  == 1) ) {    /* closure  */
      sprintf(warn_msg, "{cg} or {bfgs} minimization is performed over  \n"
                        "non continuous torsional space defined for \n"
                        "closure relations and might cause unusual  \n"
                        "growing of bonds -> use {samc} instead" );
      warnMessage(warn_msg);
    }/*endif*/
                     
/*-------------------------------------------------------------------------*/
/* 1) Initialize local parameters                                          */
 

    sa_temp_first   /= BOLTZ;
    sa_temp_last   /= BOLTZ;

    stsamc_ampl    /= BOLTZ;
    stsamc_shift   /= BOLTZ;

/*-------------------------------------------------------------------------*/
/* 2) Initialize the "one member" chain                                    */

    (*K) = 0;
    X.setup((*K),system);

/*-------------------------------------------------------------------------*/
/* 3) Minimize                                                             */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* 3.A) gradient based minimizations                                       */

    if (minimize_type <= 2) {/*cg,bfgs*/
          X.pos.move_tomin(system);
    }/*endif*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* 3.B) stochastic minimizations                                           */

    if (minimize_type == 3) {/*samc*/        
        X.T = sa_temp_first;
        for (n=1;n<=nstep_mc;n++) {    
	     X.T *= sa_rate;
             X.pos.T = X.T;
             X.prop(system);
             if (X.T <= sa_temp_last) {break;}
             if (((n % nstat)==0) && (minimize_report == 2)) {
                 system.general.istep = n;
	         output(&(X.pos),system);
             }/*endif*/
        }/*endfor*/
        output(&(X.pos),system);
        statistics(&X,system);
    }/*endif*/

    if (minimize_type == 4) {/*stsamc*/
        for (n=1;n<=nstep_mc;n++) {
          if (stsamc_type == 1) {/*trigonometric*/
	      pfunc = sin(2.0*M_PI*((double)n)/((double)stsamc_period));
          }/*endif*/
          if (stsamc_type == 2) {/*linear*/
	      pfunc = linsin(2.0*M_PI*((double)n)/((double)stsamc_period));
          }/*endif*/
	  X.T  = stsamc_ampl*pfunc + stsamc_ampl + stsamc_shift;
          X.pos.T = X.T;
          X.prop(system);
          if (((n % nstat)==0) && (minimize_report == 2)) {
                system.general.istep = n;
	        output(&(X.pos),system);
          }/*endif*/
        }/*endfor*/
     
      output(&(X.pos),system);
      statistics(&X,system);
    }/*endif*/


// Output position if debug

#ifdef DEBUG
    output(&(X.pos),system);
#endif

/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/

