#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_OFF

/*=========================================================================*/
/*=========================================================================*/
void normal_mode_enm( System &system )
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

    int i,j,n,k;
    int natom, nlownm, ntors_tr;
    double cmax, cmin, cresol;
// system parameters
    double T        = system.general.T;

    Chain X;

/*-------------------------------------------------------------------------*/
/* I) get and diagonalize the hessian                                      */

// Initialize
    X.pos.setup(system,T);

// Get the Hessian
    X.pos.get_hessian_enm(system);

// Solve Hessian
    X.pos.diagon_hessian(system);

/*-------------------------------------------------------------------------*/
/* II) write out small fluctuations in top 5-20 % low energy modes         */

  LINE
  LINE
  print_doubleline();
  printf(" constructing normal modes \n");
  print_line(); 


  if (system.general.prop_type==1) {/*cartesian*/
    natom  = X.pos.natom;
    nlownm = (int)(0.05*3*natom);                  /* 5 % of all modes  */
    if (nlownm<6) {nlownm = (int)(0.2*3*natom);}   /* 20 % of all modes */
    cmin =  -40.0;
    cmax =   40.0;
    cresol = 10.0;
    printf("output perturbed top %d low energy cartesian modes\n",nlownm-6);
    printf("for each mode -> cmax %g cmin %g cresol %g\n",cmax,cmin,cresol);
    for (j=3*natom-6;j>=(3*natom-nlownm);j--) {
       for (i=1;i<=3*natom;i++) {
           X.pos.cnm[i] = 0.0;
       }
       X.pos.cnm[j] = cmin;
       while ( X.pos.cnm[j] <= cmax ) {
            X.pos.nm_transformCNMtoX();
            X.pos.update_x_y_z();
            X.pos.force(system);
            output(&(X.pos),system);
            X.pos.cnm[j] += cresol;
       }
    }
  }/*cartesian*/

  if (system.general.prop_type==2) {/*torsional*/
    ntors_tr  = system.tors_defs.num_trans;
    nlownm    = (int)(0.1*ntors_tr);                /* 10 % of all modes */
    if (nlownm<6) {nlownm = (int)(0.4*ntors_tr);}   /* 40 % of all modes */
    printf("output fluctuations in top %d low energy torsional modes\n",nlownm);    for (j=ntors_tr;j>=(ntors_tr-nlownm);j--) {
       for (i=1;i<=ntors_tr;i++) { 
           X.pos.tnm[i] = 0.0; 
       }
       X.pos.tnm[j] = 0.0;
       for (i=1;i<=10;i++) {
            X.pos.tnm[j] += i*0.1; 
            X.pos.update_xyz();
            X.pos.update_xi_yi_zi();
            X.pos.tors_transformXtoT();
            X.pos.nm_transformTNMtoT();
            X.pos.tors_transformTtoX();
            X.pos.acc_propose();   /* xi,yi,zi -> x,y,z */
            X.pos.force(system);
            output(&(X.pos),system);
       }
    }
  }/*torsional*/


  LINE
  LINE
  print_line();
  printf(" normal mode calculations completed \n");
  print_doubleline();

#ifdef DEBUG
   printf(" End of DEBUG Pos::normal_mode\n");
   getchar();
#endif 

/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/

