/*-------------------------------------------------------------------------*/
/*                 Class Constructors and Destructors, MOSAICS             */ 
/* Contributed by                                                          */
/* Sam Nakagawa 06/02/2009                                                 */
/*-------------------------------------------------------------------------*/

#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG
#define CRYO_EM_OFF

/*=========================================================================*/
/*                             class Bond                                  */
/*=========================================================================*/
Bond::Bond()
/*-------------------------------------------------------------------------*/
/* This is the constructor of Bond class                                   */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        bond_atm_map = NULL;
        atom1 =     NULL;
        atom2 =     NULL;
        fk =        NULL;
        r0 =        NULL;
        nbond_atm = NULL;
        atom1_enm = NULL;
        atom2_enm = NULL;
        fk_enm =    NULL;
        r0_enm =    NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Bond::~Bond()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Bond class                                    */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Bond class\n");
  getchar();
#endif

        if(bond_atm_map != NULL) {
                cfree_int_mat(bond_atm_map, 1, natm_tot, 1, nbond_atm_max);
        }

        if(atom1 != NULL) atom1++;
        delete [] atom1;
        if(atom2 != NULL) atom2++;
        delete [] atom2;
        if(fk != NULL) fk++;
        delete [] fk;
        if(r0 != NULL) r0++;
        delete [] r0;
        if(nbond_atm != NULL) nbond_atm++;
        delete [] nbond_atm;
        if(atom1_enm != NULL) atom1_enm++;
        delete [] atom1_enm;
        if(atom2_enm != NULL) atom2_enm++;
        delete [] atom2_enm;
        if(fk_enm != NULL) fk_enm++;
        delete [] fk_enm;
        if(r0_enm != NULL) r0_enm++;
        delete [] r0_enm;
#ifdef DEBUG
  printf("leave destructor for Bond class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*                             class Bend                                  */
/*=========================================================================*/
Bend::Bend()
/*-------------------------------------------------------------------------*/
/* This is the constructor of Bend class                                   */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
                atom1 =   NULL;
                atom2 =   NULL;
                atom3 =   NULL;
                fk_bend = NULL;
                fk_bond = NULL;
                r0 =      NULL;
                theta0 =  NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Bend::~Bend()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Bend class                                    */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Bend class\n");
  getchar();
#endif
        if (atom1 != NULL) atom1++;
        delete [] atom1;
        if (atom2 != NULL) atom2++;
        delete [] atom2;
        if (atom3 != NULL) atom3++;
        delete [] atom3;
        if (fk_bend != NULL) fk_bend++;
        delete [] fk_bend;
        if (fk_bond != NULL) fk_bond++;
        delete [] fk_bond;
        if (r0 != NULL) r0++;
        delete [] r0;
        if (theta0 != NULL) theta0++;
        delete [] theta0;
#ifdef DEBUG
  printf("leave destructor for Bend class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*                             class Tors                                  */
/*=========================================================================*/
Tors::Tors()
/*-------------------------------------------------------------------------*/
/* This is the constructor of Tors class                                   */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        atom1 = NULL;
        atom2 = NULL;
        atom3 = NULL;
        atom4 = NULL;
        improper = NULL;
        type = NULL;
        p0 = NULL;
        p1 = NULL;
        p2 = NULL;
        p3 = NULL;
        p4 = NULL;
        p5 = NULL;
        p6 = NULL;
        s0 = NULL;
        s1 = NULL;
        s2 = NULL;
        s3 = NULL;
        s4 = NULL;
        s5 = NULL;
        s6 = NULL;
        sp2 = NULL;
        sp3 = NULL;
        sp4 = NULL;
        sp5 = NULL;
        sp6 = NULL;
        eq = NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Tors::~Tors()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Tors class                                    */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Tors class\n");
  getchar();
#endif
        if (atom1 != NULL) atom1++;
        delete [] atom1;
        if (atom2 != NULL) atom2++;
        delete [] atom2;
        if (atom3 != NULL) atom3++;
        delete [] atom3;
        if (atom4 != NULL) atom4++;
        delete [] atom4;
        if (improper != NULL) improper++;
        delete [] improper;
        if (type != NULL) type++;
        delete [] type;
        if (p0 != NULL) p0++;
        delete [] p0;
        if (p1 != NULL) p1++;
        delete [] p1;
        if (p2 != NULL) p2++;
        delete [] p2;
        if (p3 != NULL) p3++;
        delete [] p3;
        if (p4 != NULL) p4++;
        delete [] p4;
        if (p5 != NULL) p5++;
        delete [] p5;
        if (p6 != NULL) p6++;
        delete [] p6;
        if (s0 != NULL) s0++;
        delete [] s0;
        if (s1 != NULL) s1++;
        delete [] s1;
        if (s2 != NULL) s2++;
        delete [] s2;
        if (s3 != NULL) s3++;
        delete [] s3;
        if (s4 != NULL) s4++;
        delete [] s4;
        if (s5 != NULL) s5++;
        delete [] s5;
        if (s6 != NULL) s6++;
        delete [] s6;
        if (sp2 != NULL) sp2++;
        delete [] sp2;
        if (sp3 != NULL) sp3++;
        delete [] sp3;
        if (sp4 != NULL) sp4++;
        delete [] sp4;
        if (sp5 != NULL) sp5++;
        delete [] sp5;
        if (sp6 != NULL) sp6++;
        delete [] sp6;
        if (eq != NULL) eq++;
        delete [] eq;
#ifdef DEBUG
  printf("leave destructor for Tors class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*                             class Onfo                                  */
/*=========================================================================*/
Onfo::Onfo()
/*-------------------------------------------------------------------------*/
/*  This is the constructor of Onfo class                                  */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        atom1 = NULL;
        atom2 = NULL;
        type = NULL;
        eps  = NULL;
        feps = NULL;
        sig  = NULL;
        beta = NULL;
        scale= NULL;
        r0   = NULL;
        awill= NULL;
        bwill= NULL;
        c6m  = NULL;
        c8m  = NULL;
        c10m = NULL;
        r_min= NULL;
        r_max= NULL;
        r_res= NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Onfo::~Onfo()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Onfo class                                    */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Onfo class\n");
  getchar();
#endif
        if (atom1 != NULL) atom1++;
        delete [] atom1;
        if (atom2 != NULL) atom2++;
        delete [] atom2;
        if (type != NULL) type++;
        delete [] type;
        if (eps  != NULL) eps++;
        delete [] eps;
        if (feps != NULL) feps++;
        delete [] feps;
        if (sig  != NULL) sig++;
        delete [] sig;
        if (beta != NULL) beta++;
        delete [] beta;
        if (scale!= NULL) scale++;
        delete [] scale;
        if (r0   != NULL) r0++;
        delete [] r0;
        if (awill!= NULL) awill++;
        delete [] awill;
        if (bwill!= NULL) bwill++;
        delete [] bwill;
        if (c6m  != NULL) c6m++;
        delete [] c6m;
        if (c8m  != NULL) c8m++;
        delete [] c8m;
        if (c10m != NULL) c10m++;
        delete [] c10m;
        if (r_min!= NULL) r_min++;
        delete [] r_min;
        if (r_max!= NULL) r_max++;
        delete [] r_max;
        if (r_res!= NULL) r_res++;
        delete [] r_res;
#ifdef DEBUG
  printf("leave destructor for Onfo class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*                             class Inter                                 */
/*=========================================================================*/
Inter::Inter()
/*-------------------------------------------------------------------------*/
/* This is the constructor of Inter class                                  */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
        atom1         = NULL;
        atom2         = NULL;
        exclude_atom1 = NULL;
        exclude_atom2 = NULL;
        type          = NULL;
        atm1_type     = NULL;
        atm2_type     = NULL;
        numeric_size  = NULL;
        inter_off     = NULL;
        type_index    = NULL;

        scale_iinter = NULL;
        eps   = NULL;
        feps  = NULL;
        sig   = NULL;
        beta  = NULL;
        scale = NULL;
        r0    = NULL;
        awill = NULL;
        bwill = NULL;
        c6m   = NULL;
        c8m   = NULL;
        c10m  = NULL;
        r_min = NULL;
        r_max = NULL;
        r_res = NULL;

        // inter_setup
        numeric_x     = NULL;
        numeric_V_x   = NULL;
        numeric_d2V_x = NULL;

        // inter_nonpair_setup
        nfunc_nprw_type    = NULL;
        ind_func_nprw_type = NULL;
        nprw_numeric_size  = NULL;

        nprw_numeric_x     = NULL;
        nprw_numeric_V_x   = NULL;
        nprw_numeric_d2V_x = NULL;

        excl_num      = NULL;
        inter_index   = NULL;
        excl_list     = NULL;

        lbox_ll       = NULL;
        rcell_ll      = NULL;
        ncell_ll      = NULL;
        vcell_ll      = NULL;
        vcell1_ll     = NULL;
        lscl_ll       = NULL;
        head_ll       = NULL;

/*--------------------------------------------------------------------------*/
}/*end routine*/
/*==========================================================================*/

/*==========================================================================*/
Inter::~Inter()
/*--------------------------------------------------------------------------*/
/* This is the destructor of Inter class                                    */
/*--------------------------------------------------------------------------*/
{/*begin routine*/
/*--------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Inter class\n");
  getchar();
#endif
        if (atom1       != NULL) atom1++;
        delete [] atom1;
        if (atom2       != NULL) atom2++;
        delete [] atom2;
        if (exclude_atom1 != NULL) exclude_atom1++;
        delete [] exclude_atom1;
        if (exclude_atom2 != NULL) exclude_atom2++;
        delete [] exclude_atom2;
        if (type   != NULL) type++;
        delete [] type;
        if (atm1_type   != NULL) atm1_type++;
        delete [] atm1_type;
        if (atm2_type   != NULL) atm2_type++;
        delete [] atm2_type;
        if (numeric_size  != NULL) numeric_size++;
        delete [] numeric_size;
        if (inter_off   != NULL) inter_off++;
        delete [] inter_off;
        if (type_index  != NULL) type_index++;
        delete [] type_index;

        if (scale_iinter   != NULL) scale_iinter++;
        delete [] scale_iinter;
        if (eps   != NULL) eps++;
        delete [] eps;
        if (feps  != NULL) feps++;
        delete [] feps;
        if (sig   != NULL) sig++;
        delete [] sig;
        if (beta  != NULL) beta++;
        delete [] beta;
        if (scale != NULL) scale++;
        delete [] scale;
        if (r0    != NULL) r0++;
        delete [] r0;
        if (awill != NULL) awill++;
        delete [] awill;
        if (bwill != NULL) bwill++;
        delete [] bwill;
        if (c6m   != NULL) c6m++;
        delete [] c6m;
        if (c8m   != NULL) c8m++;
        delete [] c8m;
        if (c10m  != NULL) c10m++;
        delete [] c10m;
        if (r_min != NULL) r_min++;
        delete [] r_min;
        if (r_max != NULL) r_max++;
        delete [] r_max;
        if (r_res != NULL) r_res++;
        delete [] r_res;

        // inter_setup
        if (numeric_x != NULL) {
                // The following assume that the value of ntype
                // did not change between the time memory was allocated
                // and now.
                for (int i = 1; i <= ntype; i++) {
		     if (numeric_x[i] != NULL) {
                         delete [] (numeric_x[i]+1);
                     }
                }
                delete (numeric_x+1);
        }


        if (numeric_d2V_x != NULL) {

                // The following assume that the value of ntype
                // did not change between the time memory was allocated
                // and now.
                for (int i = 1; i <= ntype; i++) {
		     if (numeric_d2V_x[i] != NULL) {
                        delete [] (numeric_d2V_x[i]+1);
                     }
                }    
                delete (numeric_d2V_x+1);
        }
        if (numeric_V_x != NULL) {

                // The following assume that the value of ntype
                // did not change between the time memory was allocated
                // and now.
                for (int i = 1; i <= ntype; i++) {
		     if (numeric_V_x[i] != NULL) {
                        delete [] (numeric_V_x[i]+1);
                     }
                }
                delete (numeric_V_x+1);
        }

        // inter_nonpair_setup
        if (nfunc_nprw_type != NULL) nfunc_nprw_type++;
        delete [] nfunc_nprw_type;

        if (ind_func_nprw_type != NULL) 
	  {free_imatrix(ind_func_nprw_type,1,max_nfunc_nprw_type,1,ntype);}

     
        if (nprw_numeric_size != NULL) nprw_numeric_size++;
        delete [] nprw_numeric_size;

        if (nprw_numeric_x != NULL) {

                // The following assume that the value of nfunc_nprw
                // did not change between the time memory was allocated
                // and now.
                for (int i = 1; i <= nfunc_nprw; i++) {

                        delete [] (nprw_numeric_x[i]+1);
                }
                delete (nprw_numeric_x+1);
        }
        if (nprw_numeric_d2V_x != NULL) {

                // The following assume that the value of nfunc_nprw
                // did not change between the time memory was allocated
                // and now.
                for (int i = 1; i <= nfunc_nprw; i++) {

                        delete [] (nprw_numeric_d2V_x[i]+1);
                }
                delete (nprw_numeric_d2V_x+1);
        }
        if (nprw_numeric_V_x != NULL) {

                // The following assume that the value of nfunc_nprw
                // did not change between the time memory was allocated
                // and now.
                for (int i = 1; i <= nfunc_nprw; i++) {

                        delete [] (nprw_numeric_V_x[i]+1);
                }
                delete (nprw_numeric_V_x+1);
        }


        if (inter_index != NULL) 
          {free_imatrix(inter_index,1,natom,1,natom);}
        if (excl_list != NULL) 
          {free_imatrix(excl_list,1,excl_num_mall,1,natom);}

        if (excl_num    != NULL) excl_num++;
        delete [] excl_num;


        if (lbox_ll     != NULL) lbox_ll++;
        delete [] lbox_ll;
        if (rcell_ll    != NULL) rcell_ll++;
        delete [] rcell_ll;
        if (ncell_ll    != NULL) ncell_ll++;
        delete [] ncell_ll;
        if (vcell_ll    != NULL) vcell_ll++;
        delete [] vcell_ll;
        if (vcell1_ll   != NULL) vcell1_ll++;
        delete [] vcell1_ll;
        if (lscl_ll     != NULL) lscl_ll++;
        delete [] lscl_ll;
        if (head_ll     != NULL) head_ll++;
        delete [] head_ll;
#ifdef DEBUG
  printf("leave destructor for Inter class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
}/*end routine*/
/*=========================================================================*/


/*=========================================================================*/
/*                             class Cryo_em                               */
/*=========================================================================*/
Cryo_em::Cryo_em()
/*-------------------------------------------------------------------------*/
/* This is the constructor of Cryo_em class                                */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

        ea_az       = NULL;
        ea_alt      = NULL;
        ea_phi      = NULL;
        ea_range    = NULL;
        ea_interval = NULL;
        pixel_size  = NULL;
        resol_blur  = NULL;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Cryo_em::~Cryo_em()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Cryo_em class                                 */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Cryo_em class\n");
  getchar();
#endif

        if(ea_az != NULL) ea_az++;
        delete [] ea_az;
        if(ea_alt != NULL) ea_alt++;
        delete [] ea_alt;
        if(ea_phi != NULL) ea_phi++;
        delete [] ea_phi;
        if(ea_range != NULL) ea_range++;
        delete [] ea_range;
        if(ea_interval != NULL) ea_interval++;
        delete [] ea_interval;
        if(pixel_size != NULL) pixel_size++;
        delete [] pixel_size;
        if(resol_blur != NULL) resol_blur++;
        delete [] resol_blur;

#ifdef CRYO_EM
        delete eman2_pa;
#endif
#ifdef DEBUG
  printf("leave destructor for Cryo_em class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
/*                             class Constraint                            */
/*=========================================================================*/
Constraint::Constraint()
/*-------------------------------------------------------------------------*/
/* This is the constructor of Constraint class                             */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        bond_atm_map = NULL;
        atom1 =     NULL;
        atom2 =     NULL;
        fk =        NULL;
        r0 =        NULL;
        nbond_atm = NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Constraint::~Constraint()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Constraint class                              */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Constraint class\n");
  getchar();
#endif

        if(bond_atm_map != NULL) {
                cfree_int_mat(bond_atm_map, 1, natm_tot, 1, nbond_atm_max);
        }

        if(atom1 != NULL) atom1++;
        delete [] atom1;
        if(atom2 != NULL) atom2++;
        delete [] atom2;
        if(fk != NULL) fk++;
        delete [] fk;
        if(r0 != NULL) r0++;
        delete [] r0;
        if(nbond_atm != NULL) nbond_atm++;
        delete [] nbond_atm;
#ifdef DEBUG
  printf("leave destructor for Constraint class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
/*                             class Coord                                 */
/*=========================================================================*/
Coord::Coord()
/*-------------------------------------------------------------------------*/
/* This is the constructor for Coord class                                 */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        x = NULL;
        y = NULL;
        z = NULL;
        permute_seq = NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Coord::~Coord()
/*-------------------------------------------------------------------------*/
/* This is the destructor for Coord class                                  */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Coord class\n");
  getchar();
#endif
        if (x != NULL) x++;
        delete [] x;
        if (y != NULL) y++;
        delete [] y;
        if (z != NULL) z++;
        delete [] z;
        if (permute_seq != NULL) permute_seq++;
        delete [] permute_seq;
#ifdef DEBUG
  printf("leave destructor for Coord class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*                             class Residue                               */
/*=========================================================================*/
Residue::Residue()
/*-------------------------------------------------------------------------*/
/* This is a constructor of Residue class                                  */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        atom          = NULL;
        bond          = NULL;
        bend          = NULL;
        tors          = NULL;
        impr          = NULL;
        onfo          = NULL;
        exclude       = NULL;
        natm_chigrp   = NULL;
        nbranch_f2    = NULL;
        nbranch_s2    = NULL;
        bimpr_atom_f  = NULL;
        bimpr_atom_s  = NULL;

        ind_atom_shared = NULL;
        natom_type      = NULL;
        type_weight     = NULL;


        bond_atom     = NULL; 
        dbond_atom    = NULL;
        bend_atom     = NULL; 
        tors_atom     = NULL;
        impr_atom     = NULL;
        onfo_atom     = NULL;
        exclude_atom  = NULL;
        chigrp_atom   = NULL;
        chiprim_atom  = NULL;
        branch_root_f = NULL;
        branch_root_s = NULL;
        ind_atom_type = NULL;
        type_label    = NULL;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
Residue::~Residue()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Residue class                                 */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Residue class\n");
  getchar();
#endif
        if (atom          != NULL) atom++;
        delete [] atom;
        if (bond          != NULL) bond++;
        delete [] bond;
        if (bend          != NULL) bend++;
        delete [] bend;
        if (tors          != NULL) tors++;
        delete [] tors;
        if (impr          != NULL) impr++;
        delete [] impr;
        if (onfo          != NULL) onfo++;
        delete [] onfo;
        if (exclude       != NULL) exclude++;
        delete [] exclude;
        if (natm_chigrp   != NULL) natm_chigrp++;
        delete [] natm_chigrp;
        if (nbranch_f2  != NULL) nbranch_f2++;
        delete [] nbranch_f2;
        if (nbranch_s2  != NULL) nbranch_s2++;
        delete [] nbranch_s2;
        if (bimpr_atom_f != NULL) bimpr_atom_f++;
        delete [] bimpr_atom_f;
        if (bimpr_atom_s != NULL) bimpr_atom_s++;
        delete [] bimpr_atom_s;
        if (ind_atom_shared  != NULL) ind_atom_shared++;
        delete [] ind_atom_shared;
        if (natom_type  != NULL) natom_type++;
        delete [] natom_type;
        if (type_weight  != NULL) type_weight++;
        delete [] type_weight;

        if (bond_atom != NULL) 
	  {cfree_int_mat(bond_atom,1,nbond,1,2);}
        if (dbond_atom != NULL) 
          {cfree_int_mat(dbond_atom,1,ndbond,1,2);}
        if (bend_atom != NULL) 
	  {cfree_int_mat(bend_atom,1,nbend,1,3);}
        if (tors_atom != NULL) 
	  {cfree_int_mat(tors_atom,1,ntors,1,4);}
        if (impr_atom != NULL) 
	  {cfree_int_mat(impr_atom,1,nimpr,1,4);}
        if (onfo_atom != NULL) 
	  {cfree_int_mat(onfo_atom,1,nonfo,1,2);}
        if (exclude_atom != NULL) 
	  {cfree_int_mat(exclude_atom,1,nexclude,1,2);}
        if (chigrp_atom != NULL) 
	  {cfree_int_mat(chigrp_atom,1,nchigrp,1,NATM_CHIGRPMAX);}
        if (chiprim_atom != NULL) 
	  {cfree_int_mat(chiprim_atom,1,nchigrp,1,4);}
        if (branch_root_f != NULL) 
	  {cfree_int_mat(branch_root_f,0,MAX_BRANCH,0,MAX_BRANCH);}
        if (branch_root_s != NULL) 
	  {cfree_int_mat(branch_root_s,0,MAX_BRANCH,0,MAX_BRANCH);}
        if (ind_atom_type != NULL) 
	  {cfree_int_mat(ind_atom_type,1,ntype,1,max_natom_type);}
	/*        
         if (type_label != NULL) 
	  {cfree_int_mat(type_label,1,ntype,0,NNAMECHARMAX);}
	*/
#ifdef DEBUG
  printf("leave destructor for Residue class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*=========================================================================*/
Residue_bond::Residue_bond()
/*-------------------------------------------------------------------------*/
// This is a constructor of Residue_bond class
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        bond          = NULL;
        bend          = NULL;
        tors          = NULL;
        impr          = NULL;
        onfo          = NULL;
        exclude       = NULL;
        bond_atom     = NULL;
        bend_atom     = NULL;
        tors_atom     = NULL;
        impr_atom     = NULL;
        onfo_atom     = NULL;
        exclude_atom  = NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
/*=========================================================================*/
Residue_bond::~Residue_bond()
/*-------------------------------------------------------------------------*/
// This is a destructor of Residue_bond class
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Residue_bond class\n");
  getchar();
#endif
        if (bond          != NULL) bond++;
        delete [] bond;
        if (bend          != NULL) bend++;
        delete [] bend;
        if (tors          != NULL) tors++;
        delete [] tors;
        if (impr          != NULL) impr++;
        delete [] impr;
        if (onfo          != NULL) onfo++;
        delete [] onfo;
        if (exclude       != NULL) exclude++;
        delete [] exclude;

        if (bond_atom != NULL)
	  {cfree_int_mat(bond_atom,1,nbond,1,2);}
        if (bend_atom != NULL) 
	  {cfree_int_mat(bend_atom,1,nbend,1,3);}
        if (tors_atom != NULL) 
	  {cfree_int_mat(tors_atom,1,ntors,1,4);}
        if (impr_atom != NULL) 
	  {cfree_int_mat(impr_atom,1,nimpr,1,4);}
        if (onfo_atom != NULL) 
	  {cfree_int_mat(onfo_atom,1,nonfo,1,2);}
        if (exclude_atom != NULL) 
	  {cfree_int_mat(exclude_atom,1,nexclude,1,2);}
#ifdef DEBUG
  printf("leave destructor for Residue_bond class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
Restop_data::Restop_data()
/*-------------------------------------------------------------------------*/
/* This is the constructor of Restop_data class                            */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter constructor for Restop_data class\n");
  getchar();
#endif
        natm_chigrp   = NULL;
        atom_gindex   = NULL;
        mass          = NULL;
        charge        = NULL;
        group_begin   = NULL;
        group_end     = NULL;

        nbranch_f2    = NULL;
        nbranch_s2    = NULL;
        bimpr_atom_f  = NULL;
        bimpr_atom_s  = NULL;
        nbond_atm     = NULL;

        atom_label      = NULL;
        upac_atom_label = NULL;
               
        bond_atom       = NULL;
        dbond_atom      = NULL;
        bend_atom       = NULL;
        tors_atom       = NULL;
        impr_atom       = NULL;
        onfo_atom       = NULL;
        exclude_atom    = NULL;
        chigrp_atom     = NULL;
        chiprim_atom    = NULL;
        branch_root_f   = NULL;
        branch_root_s   = NULL;
        bond_atm_map    = NULL;

#ifdef DEBUG
  printf("leave constructor for Restop_data class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
Restop_data::~Restop_data()
/*-------------------------------------------------------------------------*/
/* This is the destructor of Restop_data class                             */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Restop_data class\n");
  getchar();
#endif

  /*
        if (natm_chigrp != NULL)  natm_chigrp++;
        delete []  natm_chigrp;
        if (atom_gindex != NULL)  atom_gindex++;
        delete []  atom_gindex;
        if (mass != NULL)  mass++;
        delete []  mass;
        if (charge != NULL)  charge++;
        delete []  charge;
        if (group_begin != NULL)  group_begin++;
        delete []  group_begin;
        if (group_end != NULL)  group_end++;
        delete []  group_end;

        if (nbranch_f2  != NULL) nbranch_f2++;
        delete [] nbranch_f2;
        if (nbranch_s2  != NULL) nbranch_s2++;
        delete [] nbranch_s2;
        if (bimpr_atom_f != NULL) bimpr_atom_f++;
        delete [] bimpr_atom_f;
        if (bimpr_atom_s != NULL) bimpr_atom_s++;
        delete [] bimpr_atom_s;
        if (nbond_atm != NULL) nbond_atm++;
        delete [] nbond_atm;
  */
#ifdef DEBUG
  printf("leave destructor for Restop_data class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*=========================================================================*/
Stage_defs::Stage_defs()
/*-------------------------------------------------------------------------*/
// This is a constructor of Stage_defs class
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        res_id = NULL;
        ityp_trans = NULL;
        natm_trans = NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
/*=========================================================================*/
Stage_defs::~Stage_defs()
/*-------------------------------------------------------------------------*/
// This is a destructor of Stage_defs class
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Stage_defs class\n");
  getchar();
#endif
        if (res_id != NULL) res_id++;
        delete [] res_id;
        if (ityp_trans != NULL) ityp_trans++;
        delete [] ityp_trans;
        if (natm_trans != NULL) natm_trans++;
        delete [] natm_trans;
#ifdef DEBUG
  printf("leave destructor for Stage_defs class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
Tors_defs::Tors_defs()
/*-------------------------------------------------------------------------*/
// This is a constructor of Tors_defs class
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
        res_id = NULL;
        ityp_trans = NULL;
        off_trans = NULL;
        natm_trans = NULL;
        secondary_only = NULL;
        ntbend_trans = NULL;
        prop_sig_scale = NULL;
        ind_stride_mol = NULL;
        nmol_torcompl = NULL;
        nmol_closcompl = NULL;
        nlevel_closcompl = NULL;
        nclbase_mol = NULL;
        ind_clbase_free = NULL;
        ind_mol_notors = NULL;
        ind_clos_trans = NULL;
        natm_clos = NULL;
        ityp_clos = NULL;
        ityp_clos_postprop = NULL;
        natm_clbase = NULL;
        res_id_clbase = NULL;
        freeze_clbase = NULL;
        ind_mol_cblc = NULL;
        ind_mol_free = NULL;
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

/*=========================================================================*/
/*=========================================================================*/
Tors_defs::~Tors_defs()
/*-------------------------------------------------------------------------*/
// This is a destructor of Tors_defs class
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
#ifdef DEBUG
  printf("enter destructor for Tors_defs class\n");
  getchar();
#endif
        if (res_id != NULL) res_id++;
        delete [] res_id;
        if (ityp_trans != NULL) ityp_trans++;
        delete [] ityp_trans;
        if (off_trans != NULL) off_trans++;
        delete [] off_trans;
        if (natm_trans != NULL) natm_trans++;
        delete [] natm_trans;
        if (secondary_only != NULL) secondary_only++;
        delete [] secondary_only;
        if (ntbend_trans != NULL) ntbend_trans++;
        delete [] ntbend_trans;
        if (prop_sig_scale != NULL) prop_sig_scale++;
        delete [] prop_sig_scale;
        if (ind_stride_mol != NULL) ind_stride_mol++;
        delete [] ind_stride_mol;
        if (nmol_torcompl != NULL) nmol_torcompl++;
        delete [] nmol_torcompl;
        if (nmol_closcompl != NULL) nmol_closcompl++;
        delete [] nmol_closcompl;
        if (nlevel_closcompl != NULL) nlevel_closcompl++;
        delete [] nlevel_closcompl;
        if (nclbase_mol != NULL) nclbase_mol++;
        delete [] nclbase_mol;
        if (ind_clbase_free != NULL) ind_clbase_free++;
        delete [] ind_clbase_free;
        if (ind_mol_notors != NULL) ind_mol_notors++;
        delete [] ind_mol_notors;
        if (ind_clos_trans != NULL) ind_clos_trans++;
        delete [] ind_clos_trans;
        if (natm_clos != NULL) natm_clos++;
        delete [] natm_clos;
        if (ityp_clos != NULL) ityp_clos++;
        delete [] ityp_clos;
        if (ityp_clos_postprop != NULL) ityp_clos_postprop++;
        delete [] ityp_clos_postprop;
        if (natm_clbase != NULL) natm_clbase++;
        delete [] natm_clbase;
        if (res_id_clbase != NULL) res_id_clbase++;
        delete [] res_id_clbase;
        if (freeze_clbase != NULL) freeze_clbase++;
        delete [] freeze_clbase;
        if (ind_mol_cblc != NULL) ind_mol_cblc++;
        delete [] ind_mol_cblc;
        if (ind_mol_free != NULL) ind_mol_free++;       
        delete [] ind_mol_free;
#ifdef DEBUG
  printf("leave destructor for Tors_defs class\n");
  getchar();
#endif
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
