/***********************************************************/
/*-------------------------- C-----------------------------*/
/***********************************************************/
/*                                                         */
/*           Some math routine for intra calcluations      */
/*                                                         */
/***********************************************************/
/*-------------------------- C-----------------------------*/
/***********************************************************/

#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/utillib.h"
#include "../include/definitions.h"

/***********************************************************/
/*-------------------------- C++ --------------------------*/
/***********************************************************/
void cprd(double ax, double ay, double az, 
          double bx, double by, double bz,
          double *cx, double *cy, double *cz) 
/*---------------------------------------------------------*/
     /* c_vec = a_vec X b_vec -- cross product */
/*---------------------------------------------------------*/
{/*begin*/
/*---------------------------------------------------------*/
    *cx = ay*bz - az*by;
    *cy = az*bx - ax*bz;
    *cz = ax*by - ay*bx; 
/*---------------------------------------------------------*/
}/*end*/ 

/***********************************************************/
/*-------------------------- C++ --------------------------*/
/***********************************************************/
double getDistOnfo(double *x, double *y, double *z,
		    int j1, int j4) 
/*=======================================================================*/
{   /*begin routine*/
/*=======================================================================*/
/*            Local variable declarations                                */
   double dx,dy,dz;
   double d14;

   dx = x[j4] - x[j1];
   dy = y[j4] - y[j1];
   dz = z[j4] - z[j1];

   d14 = sqrt(dx*dx+dy*dy+dz*dz);

   return d14;

}/*end*/


/***********************************************************/
/*-------------------------- C++ --------------------------*/
/***********************************************************/
double getTorsAngle(double *x, double *y, double *z,
		    int j1, int j2, int j3, int j4) 
/*=======================================================================*/
{   /*begin routine*/
/*=======================================================================*/
/*            Local variable declarations                                */
  double phi, cost;
  double r21[3], r23[3], r32[3], r34[3], tmpVec[3];
  double n123[3], n234[3];
  
  r21[0] = x[j1] - x[j2];
  r21[1] = y[j1] - y[j2];
  r21[2] = z[j1] - z[j2];

  r23[0] = x[j3] - x[j2];
  r23[1] = y[j3] - y[j2];
  r23[2] = z[j3] - z[j2];

  r32[0] = x[j2] - x[j3];
  r32[1] = y[j2] - y[j3];
  r32[2] = z[j2] - z[j3];
  
  r34[0] = x[j4] - x[j3];
  r34[1] = y[j4] - y[j3];
  r34[2] = z[j4] - z[j3];
  
  /* get the normal unit vector of plane j1-j2-j3. */    
  DCross( r21, r23, n123, 3);
  DScale(1.0/DNorm(n123, 3), n123, 3);
  
  /* get the normal unit vector of plane j2-j3-j4. */   
  DCross( r32, r34, n234, 3);
  DScale(1.0/DNorm(n234, 3), n234, 3);
  
  /* get the torsion from the product of the 
     above two normal vectors. */
  cost = DDot(n123, n234, 3);
  if (cost > 1.0) cost = 1.0;
  if (cost <-1.0) cost =-1.0;
  phi = acos(cost);
  
  /* adjustmest for >PI torsion. */
  DCross(n123, n234, tmpVec, 3);
  if ( DDot(tmpVec, r23, 3)<0) {
    phi = 2.0*M_PI - phi;
  }
  return phi;
      
/*-----------------------------------------------------------------------*/
}/*end routine*/
/*==========================================================================*/

/***********************************************************/
/*-------------------------- C++ --------------------------*/
/***********************************************************/
double getBendAngle(double *x, double *y, double *z,
		    int j1, int j2, int j3) 
/*=======================================================================*/
{   /*begin routine*/
/*=======================================================================*/
/*            Local variable declarations                                */

  double theta,cost; 
  double dx21,dy21,dz21;
  double dx23,dy23,dz23;
  double rr21,r21,rr23,r23;
 
    dx21 = x[j1] - x[j2];
    dy21 = y[j1] - y[j2];
    dz21 = z[j1] - z[j2];
    rr21 = dx21*dx21+dy21*dy21+dz21*dz21;
    r21  = sqrt(rr21);
    dx23 = x[j3] - x[j2];
    dy23 = y[j3] - y[j2];
    dz23 = z[j3] - z[j2];
    rr23 = dx23*dx23+dy23*dy23+dz23*dz23;
    r23  = sqrt(rr23);
    cost = (dx21*dx23 + dy21*dy23 + dz21*dz23)/(r21*r23);
    if (cost> 1) cost = 1;
    if (cost<-1) cost =-1;
    theta  = acos(cost);

    return theta;
/*-----------------------------------------------------------------------*/
}/*end routine*/
/*==========================================================================*/

