#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#define CLOCKS_PER_SEC_C  1000000
#define MAXTIME 2147.48
static double dsqrarg;
#define DSQR(a) ((dsqrarg=(a)) == 0.0 ? 0.0 : dsqrarg*dsqrarg)
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
#define NR_END 0
#define FREE_ARG char*


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
void spline(double x[], double y[], int n, double yp1, double ypn, double y2[])
/* Given arrays x[1..n] and y[1..n] containing a tabulated function, i.e.,    */
/* y_i = f(x_i), with x_1 < x_2 < ... < x_N, and given values yp1 and ypn for */
/* the first derivative of the interpolating function at points 1 and n,      */ 
/* respectively, this routine returns an array y2[1..n] that contains the     */
/* second derivatives of the interpolating function at the tabulated points   */
/* x_i. If yp1 and/or ypn are equal to 1 x 10^(30) or larger, the routine     */
/* signaled to set the corresponding boundary condition for a natural spline, */
/* with zero second derivative on that boundary.                              */ 
/*==========================================================================*/
{
  int i,k;
  double p,qn,sig,un,*u;

  u = dvector(1,n-1);
  if (yp1 > 0.99e30)     
      y2[1] = u[1] = 0.0;      
  else {
      y2[1] = -0.5;
      u[1]  = (3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
  }    

  for (i=2;i<=n-1;i++) {
      sig = (x[i]-x[i-1])/(x[i+1]-x[i-1]);
      p = sig*y2[i-1]+2.0;
      y2[i] = (sig-1.0)/p;
      u[i] = (y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
      u[i] = (6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
  }     

  if (ypn > 0.99e30) 
     qn = un = 0.0;
  else {
     qn = 0.5;
     un = (3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
  }
  y2[n] = (un-qn*u[n-1])/(qn*y2[n-1]+1.0);
  for (k=n-1;k>=1;k--)
      y2[k] = y2[k]*y2[k+1]+u[k];
  free_dvector(u,1,n-1);  
}
/*==========================================================================*/

//=========================== C++ =======================================//
//***********************************************************************//
//=========================== C++ =======================================//
//void nspline(double* x, double* y, int N, double* b, double* c, double* d)
void nspline(double x[], double y[], int N, double derivatives[])
// The array derivatives[] should have the size of 3*N
// derivatives[1..N]     the second derivatives
// derivatives[N+1..2N]  the first derivatives
// derivatives[2N+1..3N] the third derivatives
{

       // Step 3
        double *h = dvector(1, N-1);
        for (int i = 1; i <= N-1; i++) {h[i] = x[i+1] - x[i];}

        // Step 4
        double *alpha = dvector(1, N-1);
        for (int i = 2; i <= N-1; i++) {
                alpha[i] = 3.*(y[i+1] - y[i])/h[i] - 3.*(y[i] - y[i-1])/h[i-1];
        }

        // Step 5
        double *l = dvector(1, N);
        double *u = dvector(1, N);
        double *z = dvector(1, N);

        // Step 6
        l[1] = 1.0;
        u[1] = 0.0;  // Ensures the second derivative at x[1] is zero.
        z[1] = 0.0;  // Ensures the second derivative at x[1] is zero.

        // Step 7
        for (int i = 2; i <= N-1; i++) {
                l[i] = 2.*(x[i+1] - x[i-1]) - h[i-1]*u[i-1];
                u[i] = h[i] / l[i];
                z[i] = (alpha[i] - h[i-1]*z[i-1])/l[i];
        }

        // Step 8
        l[N] = 1.0;
        z[N] = 0.0;
        derivatives[N] = 0.0;    // Ensures the second derivative at x[N] is zero.
                                 // The same as c[N] = 0.0
        derivatives[3*N] = 0.0;  // Since c[N] is zero, d[N] should also be zero.
                                 // The same as d[N] = 0.0
        derivatives[2*N] = (y[N] - y[N-1]) / h[N-1];
                                 // Simple backward difference approximation.
                                 // The same as b[N] = (y[N] - y[N-1]) / h[N-1]
        // Step 9
        for (int j = N-1; j >= 1; j--) {

                // c[j] = z[j] - u[j]*c[j+1];
                derivatives[j] = z[j] - u[j]*derivatives[j+1];

                // b[j] = (y[j+1] - y[j])/h[j] - h[j]*(c[j+1] + 2.0*c[j])/3.0;
                derivatives[j+N] = (y[j+1] - y[j])/h[j]
                                 - h[j]*(derivatives[j+1] + 2.0*derivatives[j])/3.0;

                // d[j] = (c[j+1] - c[j])/3.0/h[j];
                derivatives[j+2*N] = (derivatives[j+1] - derivatives[j])/3.0/h[j];
        }

        // Step 10
        // De-allocate memory
        free_dvector(h, 1, N-1);
        free_dvector(l, 1, N);
        free_dvector(alpha, 1, N-1);
        free_dvector(u, 1, N);
        free_dvector(z, 1, N);
}
/*==========================================================================*/


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
void splint(double xa[], double ya[], double y2a[], int n, double x, double *y)
/* Given the arrays xa[1..n] and ya[1..n], which tabulate a function and given */
/* the array y2a[1..n], which is the output from 'spline' above, and given a   */
/* value of x, this routine returns the cubic-spline interpolated value y.     */
/*==========================================================================*/
{
  int i,klo,khi,k;
  double h,b,a;

  double delta = 1.e-3;

// get the max(ya) min(ya)
   double max_ya = -1.e6;
   double min_ya =  1.e6;
   for (i=1;i<=n;i++) {
        if (ya[i]>max_ya) {max_ya=ya[i];}
        if (ya[i]<min_ya) {min_ya=ya[i];}
   }

  klo=1;
  khi=n;
  while (khi-klo > 1) {
    k = (khi+klo) >> 1;
    if (xa[k] > x) khi=k;
    else klo = k;
  }
  h = xa[khi]-xa[klo];
  if (h == 0.0) nrerror("Bad xa input to routine splint");
  a = (xa[khi]-x)/h;
  b = (x-xa[klo])/h;
  *y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;

// First order approximation in case of extremes

   if ( (*y > (max_ya+delta)) || (*y < (min_ya-delta)) )  {
         *y=a*ya[klo]+b*ya[khi];
   }

}
/*==========================================================================*/


//=========================== C++ =======================================//
//***********************************************************************//
//=========================== C++ =======================================//
void nsplint(double x[], double y[], double derivatives[], int n,
             double xintrp, double *yintrp)
{

  int ks = 1;
  int kf;
  double ytmp;
  nsplint(x, y, derivatives, n, ks, kf, xintrp, &ytmp);
  *yintrp = ytmp;
}
/*==========================================================================*/


//=========================== C++ =======================================//
//***********************************************************************//
//=========================== C++ =======================================//
void nsplint(double x[], double y[], double derivatives[], int n,
             int ks, int& kf, double xintrp, double *yintrp)
{

  int klo = ks;
  int khi = klo + 1;                // set kho to klo + 1 or n whichever smaller

  // If cache miss, find the new index
  if ( !((x[klo] <= xintrp) && (xintrp < x[khi]))) {   // cache miss

    // Binary search the index
    klo = 1;
    khi = n;
    int k;
    while (khi-klo > 1) {
      k = (khi+klo) >> 1;
      if (x[k] > xintrp) khi = k;
      else klo = k;
    }
  }

  // Store the current index in cache
  kf = klo;

  // Compute the interpolated y
  double dx = xintrp - x[klo];
  double b = derivatives[klo+n];
  double c = derivatives[klo];
  double d = derivatives[klo+n+n];
  *yintrp = y[klo] + (b + ((c + d * dx) * dx)) * dx;

}
/*==========================================================================*/


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
void dsplint(double xa[], double ya[], double y2a[], int n, double x, double *dy_dx)
/* Given the arrays xa[1..n] and ya[1..n], which tabulate a function and given   */
/* the array y2a[1..n], which is the output from 'spline' above, and given a     */
/* value of x, this routine returns the cubic-spline interpolated value of dy/dx.*/
/*==========================================================================*/
{
  double dx = 1.e-5;
  double x_b, x_f, y_b, y_f;
  
  x_b = x - dx;
  splint(xa,ya,y2a,n,x_b,&y_b);
  x_f = x + dx;
  splint(xa,ya,y2a,n,x_f,&y_f);
    
  *dy_dx = (y_f - y_b)/(2.0*dx);
}
/*==========================================================================*/

//=========================== C++ =======================================//
//***********************************************************************//
//=========================== C++ =======================================//
void ndsplint(double x[], double y[], double derivatives[],
              int n, double xintrp, double *dy_dx)
{
        double delta = 1.0e-5;
        double y1, y2;
        int ks = 1;
        int kf;
        nsplint(x, y, derivatives, n, ks, kf, xintrp + delta, &y2);
        nsplint(x, y, derivatives, n, kf, kf, xintrp - delta, &y1);
        *dy_dx = (y2 - y1) / 2. / delta;
}
/*==========================================================================*/

//=========================== C++ =======================================//
//***********************************************************************//
//=========================== C++ =======================================//
void ndsplint(double x[], double y[], double derivatives[],
              int n, int ks, int& kf, double xintrp, double *dy_dx)
{
        double delta = 1.0e-5;
        double y1, y2;
        nsplint(x, y, derivatives, n, ks, kf, xintrp + delta, &y2);
        nsplint(x, y, derivatives, n, kf, kf, xintrp - delta, &y1);
        *dy_dx = (y2 - y1) / 2. / delta;
}
/*==========================================================================*/


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
void cputime(double *time)

/*==========================================================================*/
{
  int itime;
  static double to=0.,tn=0.;

  itime = clock();
  tn = (double)((double)itime/(double)CLOCKS_PER_SEC_C);
  *time = tn;
  if(tn >= 0 && to >= 0){*time=tn;}
  if(tn < 0  && to >= 0){*time=MAXTIME*2.0+tn;}
  if(tn >= 0 && to <  0){*time=tn+MAXTIME;}
  if(tn <  0 && to <  0){*time=MAXTIME+tn;}

  to = tn;
}
/*==========================================================================*/


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
double heavyside(double x)
    // analytic approximation to a heavyside function
{
    double temp;
    double n = 1000000.0;
      
    if ( x >= M_PI/(2.0*n) ) 
       temp = 1.0;
    else if ( x <= -M_PI/(2.0*n) )
       temp =-1.0;
    else 
       temp = sin(n*x);
    return temp*0.5+0.5;
//--------------------------- C++ ---------------------------------------// 
}
//=========================== C++ =======================================//

//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
double heavyside_dot(double x)
    // analytic approximation to a heavyside function
{
    double temp;
    double n = 1000000.0;
      
    if ( x >= M_PI/(2.0*n) ) 
       temp = 0.0;
    else if ( x <= -M_PI/(2.0*n) )
       temp = 0.0;
    else 
       temp = n*cos(n*x);
    return temp*0.5;
//--------------------------- C++ ---------------------------------------// 
}
//=========================== C++ =======================================//
 
//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
double switch_left(double x, double a, double b, int type)
   // switch from left a to left b such that
   // returns 0 at a and 1 at b !!! b>a 
   // type C_0 ... C_4 equivalent (1...4) 
   // see more 
   // M. Watanabe & W. P. Reinhardt
   // Phys. Rev. Lett. Volume 65, Number 26 
   // page 3303 
{
   double tau = (x-a)/(b-a);
   double tau2 = tau*tau;
   double tau3 = tau2*tau; 
   double tau4 = tau2*tau2;
   double tau5 = tau3*tau2;
   
   if (type == 0)
      return tau;
   else if (type == 1) 
      return -tau2*(2.0*tau-3.0);
   else if (type == 2)
      return tau-sin(2.0*M_PI*tau)/(2.0*M_PI);
   else if (type == 3)
      return -tau4*(20.0*tau3-70.0*tau2+84.0*tau-35.0); 
   else if (type == 4) 
      return tau5*(70.0*tau4-315.0*tau3+540.0*tau2-420.0*tau+126.0); 
   //   else if (type == 5)
   //   return exp(tau4);
   return 0.0;
//--------------------------- C++ ---------------------------------------// 
} 
//=========================== C++ =======================================// 


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
double switch_right(double x, double a, double b, int type)
   // switch from right from a to right b such that
   // returns 0 at b and 1 at a !!! b>a 
   // type C_0 ... C_4 equivalent (1...4) 
   // see more 
   // M. Watanabe & W. P. Reinhardt
   // Phys. Rev. Lett. Volume 65, Number 26 
   // page 3303 
{
   double tau = (b-x)/(b-a);
   double tau2 = tau*tau;
   double tau3 = tau2*tau; 
   double tau4 = tau2*tau2;
   double tau5 = tau3*tau2;
   
   if (type == 0)
      return tau;
   else if (type == 1) 
      return -tau2*(2.0*tau-3.0);
   else if (type == 2)
      return tau-sin(2.0*M_PI*tau)/(2.0*M_PI);
   else if (type == 3)
      return -tau4*(20.0*tau3-70.0*tau2+84.0*tau-35.0); 
   else if (type == 4) 
      return tau5*(70.0*tau4-315.0*tau3+540.0*tau2-420.0*tau+126.0); 
   //else if (type == 5)
   //   return exp(tau4);
   return 0.0;
//=========================== C++ =======================================// 
} 
//=========================== C++ =======================================// 


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
double dx_switch_right(double x, double a, double b, int type)
   // first derivative of switch_right 
   // switch from right from a to right b such that
   // returns 0 at b and 1 at a !!! b>a 
   // type C_0 ... C_4 equivalent (1...4) 
   // see more 
   // M. Watanabe & W. P. Reinhardt
   // Phys. Rev. Lett. Volume 65, Number 26 
   // page 3303 
{
   double tau  = (b-x)/(b-a); double dx_tau = 1.0/(a-b);
   double tau2 = tau*tau;  
   double tau3 = tau2*tau; 
   double tau4 = tau2*tau2;
   double tau5 = tau3*tau2; 
   
   if (type == 0)
      return dx_tau;
   else if (type == 1) 
     // switch_right returns -tau2*(2.0*tau-3.0);
     return -tau*dx_tau*(6.0*tau + 12.0); 
   else if (type == 4)
     // switch_right returns 
     // tau5*(70.0*tau4-315.0*tau3+540.0*tau2-420.0*tau+126.0);  
     return tau4*dx_tau*(  630.0*tau4 - 2520.0*tau3 +
                          3780.0*tau2 - 2520.0*tau  + 630.0 ); 
   else {
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     printf("type %d is not supported in dx_switch_right\n",type);
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     fflush(stdout);
     exit(1);
   }  
//=========================== C++ =======================================// 
} 
//=========================== C++ =======================================// 


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
double ddd_eps(double r, double par1, double par2, double par3, int type)
  // distant dependent dielectric formulas
  //
  // type == 1 : Lavery type, Biophysics Journal 76 2760 (1999)
  // Computes D-(D-D0)/2*( (r S)^2 + 2 r S +2 ) exp(-r S)
  // 
//=========================== C++ =======================================// 
{
//=========================== C++ =======================================//

  double D, D0, S, rS;
  double e,c, arg, A, B;
  double value;

  if (type==1) {
      D  = par1;
      D0 = par2;
      S  = par3;
      rS = r*S;

      value = D - (D-D0)/2.0*( rS*(rS+2.0) + 2.0 )*exp(-rS);
      return value;
  }
  else if (type==2) {
      c  = par1;
      e  = par2;
      arg = r/(2.0*c);

      A  = exp(e*log(arg));
      B  = exp(e*log(sinh(arg)));
   
      value = 78.3 - 77.3*A/B;
      return value;
  } else {
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     printf("type %d is not supported in ddd_eps \n",type);
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     fflush(stdout);
     exit(1);
  }/*endif*/  

//=========================== C++ =======================================// 
} 
//=========================== C++ =======================================// 


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 
double ddd_deps(double r, double par1, double par2, double par3, int type)
  // distant dependent dielectric formulas
  //
  // type == 1 : Lavery type, Biophysics Journal 76 2760 (1999)
  // Computes d/dr [D-(D-D0)/2*( (r S)^2 + 2 r S +2 ) exp(-r S)]
  // 
//=========================== C++ =======================================// 
{
//=========================== C++ =======================================//

  double D, D0, S, rS;
  double e,c, arg, argp, A, B, Ap, Bp;
  double value;

  if (type==1) {
      D  = par1;
      D0 = par2;   
      S  = par3;
      rS = r*S;

      value = 0.5*(D-D0)*S*rS*rS*exp(-rS);

      return value;
  }
  else if (type==2) {
      c     = par1;
      e     = par2;
      arg   = r/(2.0*c);
      argp = 1.0/(2.0*c);

      A  = exp(e*log(arg));
      B  = exp(e*log(sinh(arg)));
      Ap = (e/r)*A;
      Bp = e*argp*(cosh(arg)/sinh(arg))*B;

      value = 77.3*(A*Bp/(B*B) - Ap/B);

#define DEBUG_DDD_DEPS_OFF
#ifdef DEBUG_DDD_DEPS
      printf("debug eps\n");
      getchar();
      double an,num,eps;
      eps = 1.e-5;
      num = (ddd_eps(r+eps,par1,par2,par3,type)- 
             ddd_eps(r    ,par1,par2,par3,type))/eps;
      an  =  value;
      printf("debug DDD_EPS type %d\n",type);
      printf("numerical %.16g, analitical %.16g\n",num,an);   
      printf("r %g, c %g, e %g, A %g, B %g, Ap %g, Bp %g\n",r,c,e,A,B,Ap,Bp);
      getchar();
#endif
      return value;
  } else {
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     printf("type %d is not supported in ddd_eps \n",type);
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     fflush(stdout);
     exit(1);
  }/*endif*/  
  
//=========================== C++ =======================================// 
} 
//=========================== C++ =======================================// 


//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 

double pythag(double a, double b)
  // 
  // Computes (a^2+b^2)^(1/2) without destructive
  // underflow or overflow
{
   double absa,absb;
   absa=fabs(a);
   absb=fabs(b);
   if (absa > absb) return absa*sqrt(1.0+DSQR(absb/absa));
   else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+DSQR(absa/absb)));
}

//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 

double *dvector(long nl, long nh)
{
   double *v;

   v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
   if (!v) nrerror("allocation failure in dvector()");
   return v-nl+NR_END;
} 

//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 

int *ivector(long nl, long nh)
{
   int *v;

   v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
   if (!v) nrerror("allocation failure in ivector()");
   return v-nl+NR_END;
} 

//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 

char *cvector(long nl, long nh)
{
   char *v;

   v=(char *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(char)));
   if (!v) nrerror("allocation failure in cvector()");
   return v-nl+NR_END;
} 

//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 

void free_dvector(double *v, long nl, long nh)
  // free a double vector allocated with dvector() 
{
  free((FREE_ARG) (v+nl-NR_END));
}

//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 

void free_ivector(int *v, long nl, long nh)
  // free an int vector allocated with ivector() 
{
  free((FREE_ARG) (v+nl-NR_END));
}

//=========================== C++ =======================================// 
//***********************************************************************//
//=========================== C++ =======================================// 

void free_cvector(char *v, long nl, long nh)
  // free a char vector allocated with cvector() 
{
  free((FREE_ARG) (v+nl-NR_END));
}

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* cmall_mat: Careful malloc a matrix of type double                        */
/*==========================================================================*/
double **dmatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a double matrix with subscript range m[nrl...nrh][ncl...nch] */
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  double **m;

  if(nrow <= 0 || ncol <= 0) return (double **)NULL;
/* allocate pointers to rows */
  m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
  if(!m)
    {
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     printf("allocation failure of row pointers in cmall_mat\n");
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     fflush(stdout);
     exit(1);
    }
  m += NR_END;
  m -= nrl;

/* allocate rows and set pointers to them */
  m[nrl]=(double *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
  if(!m[nrl])
    {
      printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
      printf("allocation failure of rows in cmall_mat\n");
      printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
      fflush(stdout);
      exit(1);
    }
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1;i<=nrh;i++)
    m[i]=m[i-1]+ncol;

/* return pointer to array of pointers to rows */
   return m; 
}/* end routine */



/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* cmall_mat: Careful malloc a matrix of type int                           */
/*==========================================================================*/
int **imatrix(long nrl, long nrh, long ncl, long nch)
/* allocate an int matrix with subscript range m[nrl...nrh][ncl...nch] */
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  int **m;

  if(nrow <= 0 || ncol <= 0) return (int **)NULL;
/* allocate pointers to rows */
  m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int*)));
  if(!m)
    {
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     printf("allocation failure of row pointers in cmall_mat\n");
     printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
     fflush(stdout);
     exit(1);
    }
  m += NR_END;
  m -= nrl;

/* allocate rows and set pointers to them */
  m[nrl]=(int *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(int)));
  if(!m[nrl])
    {
      printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
      printf("allocation failure of rows in cmall_mat\n");
      printf("@@@@@@@@@@@@@@@@@@@@_ERROR_@@@@@@@@@@@@@@@@@@@@\n");
      fflush(stdout);
      exit(1);
    }
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1;i<=nrh;i++)
    m[i]=m[i-1]+ncol;

/* return pointer to array of pointers to rows */
   return m; 
}/* end routine */


/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* cfree_mat: Careful free a double matrix                                  */
/*==========================================================================*/

void free_dmatrix(double **m,long nrl, long nrh, long ncl, long nch)
{
  free((char *) (m[nrl]+ncl-NR_END));
  free((char *) (m+nrl-NR_END));
}/* end routine */

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* cfree_mat: Careful free an int matrix                                    */
/*==========================================================================*/

void free_imatrix(int **m,long nrl, long nrh, long ncl, long nch)
{
  free((char *) (m[nrl]+ncl-NR_END));
  free((char *) (m+nrl-NR_END));
}/* end routine */

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* mat_mat: matrix-matrix multiplication                                    */
/*==========================================================================*/
void mat_mat(double **a, double **b, double **c, int n)
// multiply square matrices of dimensions n x n
// a,b E R^(n x n) : input 
// c E R^(n x n) : output 
{//begin//
 for (int i=1;i<=n;i++) {
     for (int j=1;j<=n;j++) {
          c[i][j] = 0.0;
          for (int k=1;k<=n;k++) 
              c[i][j] += a[i][k]*b[k][j]; 
     } 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* mat_add: matrix-matrix addition                                          */
/*==========================================================================*/
void mat_add(double **a, double **b, double **c, int n)
// add together square matrices of dimensions n x n
// a,b E R^(n x n) : input 
// c E R^(n x n) : output 
{//begin//
 for (int i=1;i<=n;i++) {
     for (int j=1;j<=n;j++) {
              c[i][j] = a[i][j]+b[i][j]; 
     } 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* mat_add: matrix-matrix addition                                          */
/*==========================================================================*/
void mat_add(double **a, double **b, double **c, double **d, int n)
// add together square matrices of dimensions n x n
// a,b,c E R^(n x n) : input 
// d E R^(n x n) : output 
{//begin//
 for (int i=1;i<=n;i++) {
     for (int j=1;j<=n;j++) {
              d[i][j] = a[i][j]+b[i][j]+c[i][j]; 
     } 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* mat_app: matrix-matrix append                                            */
/*==========================================================================*/
void mat_app(double **a, double scal, double **b, int n)
// append together square matrices of dimensions n x n
// a,b E R^(n x n), scal  : input 
// a E R^(n x n) : output  ( a = a + scal * b ) 
{//begin//
 for (int i=1;i<=n;i++) {
     for (int j=1;j<=n;j++) {
              a[i][j] = a[i][j]+scal*b[i][j]; 
     } 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* vec_add: vector-vector addition                                          */
/*==========================================================================*/
void vec_add(double *a, double *b, double *c, int n)
// add together vectors of dimensions n x n
// a,b E R^(n) : input 
// c E R^(n) : output 
{//begin//
 for (int i=1;i<=n;i++) {
              c[i] = a[i]+b[i]; 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* vec_add: vector-vector addition                                          */
/*==========================================================================*/
void vec_add(double *a, double *b, double *c, double *d, int n)
// add together vectors of dimensions n x n
// a,b,c E R^(n) : input 
// d E R^(n) : output 
{//begin//
 for (int i=1;i<=n;i++) {
              d[i] = a[i]+b[i]+c[i]; 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* scal_mat:scalar-matrix multiplication                                    */
/*==========================================================================*/
void scal_mat(double scal, double **a, int n)
// scale square matrix A of dimensions n x n with scal 
// a E R^(n x n) : input,output 
{//begin//
 for (int i=1;i<=n;i++) {
     for (int j=1;j<=n;j++) {
              a[i][j] = scal*a[i][j]; 
     } 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* scal_vec:scalar-vector multiplication                                    */
/*==========================================================================*/
void scal_vec(double scal, double *a, int n)
// scale vector a of dimensions n with scal 
// a E R^(n) : input,output 
{//begin//
 for (int i=1;i<=n;i++) {
              a[i] = scal*a[i]; 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* vec_vec: vector-vector multiplication                                    */
/*==========================================================================*/
double vec_vec(double *a, double *b, int n) 
// scalar product of vectors 
// a E R^(n x 1) : input 
// b E R^(n x 1) : input 
// c E R^(1) : output 
{//begin//
 double c = 0.0;
 for (int i=1;i<=n;i++) {
         c += a[i]*b[i];
 }
 return c;
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* mat_vec: matrix-vector multiplication                                    */
/*==========================================================================*/
void mat_vec(double **a, double *b, double *c, int n) 
// multiply square matrices of dimensions n x n
// a E R^(n x n) : input 
// b E R^(n x 1) : input 
// c E R^(n x 1) : output 
{//begin//
 for (int i=1;i<=n;i++) {
     c[i] = 0.0;
     for (int j=1;j<=n;j++)  
         c[i] += a[i][j]*b[j];
 }
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* vec_mat: vector-matrix multiplication                                    */
/*==========================================================================*/
void vec_mat(double *b, double **a, double *c, int n) 
// multiply square matrices of dimensions n x n
// a E R^(n x n) : input 
// b E R^(1 x n) : input 
// c E R^(1 x n) : output 
{//begin//
 for (int i=1;i<=n;i++) {
     c[i] = 0.0;
     for (int j=1;j<=n;j++) 
         c[i] += a[j][i]*b[j]; 
 } 
}//end//

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* transpose: transpose real symmetric matrix                               */
/*==========================================================================*/
void mat_trn(double **a, double **a_tr, int n) 
// get the transpose of a to a_tr 
// a E R^(n x n) : input 
// a_tr E R^(n x n) : output 
{//begin//
 for (int i=1;i<=n;i++) 
     for (int j=1;j<=n;j++) 
         a_tr[j][i] = a[i][j];
}//end// 

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* write out a vector                                                       */
/*==========================================================================*/
void vec_out(double *a, int n) 
// a E R^n : input 
{//begin//
   for (int i=1;i<=n;i++) {
          cout<<setprecision(16)<<a[i]<<'\t';
   }
   cout<<endl;
}//end//     

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* write out a matrix                                                       */
/*==========================================================================*/
void mat_out(double **a, int n) 
// a E R^(n x n) : input 
{//begin//
   for (int i=1;i<=n;i++) {
      for (int j=1;j<=n;j++)
          cout<<setprecision(16)<<a[i][j]<<'\t';
      cout<<endl;
   }
   cout<<endl;
}//end//     

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* write out two matrix in comparison                                       */
/*==========================================================================*/
void mat_comp_out(double **a, double **b, int n) 
// a,b E R^(n x n) : input 
{//begin//
   cout<<"dimension: "<<'\t'<<n<<endl;
   cout<<" matrix 1. and matrix 2. rowise "<<endl;
   for (int i=1;i<=n;i++) {
      cout<<" row # "<<'\t'<<i<<endl; 
      for (int j=1;j<=n;j++)
          cout<<setprecision(16)<<a[i][j]<<'\t';
      cout<<endl;
      for (int j=1;j<=n;j++)
          cout<<setprecision(16)<<b[i][j]<<'\t';
      cout<<endl<<endl;
   }
   cout<<endl;
}//end//     
 
/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* write out two vectors in comparison                                      */
/*==========================================================================*/
void vec_comp_out(double *a, double *b, int n) 
// a,b E R^(n) : input 
{//begin//
   if ( n <= 3 ) {
      for (int j=1;j<=n;j++)
          cout<<setprecision(16)<<a[j]<<'\t';
      cout<<endl;
      for (int j=1;j<=n;j++)
          cout<<setprecision(16)<<b[j]<<'\t';
      cout<<endl<<endl;
   }
   else {
      for (int j=1;j<=n;j++) { 
          cout<<setprecision(16)<<a[j]<<'\t';
          cout<<setprecision(16)<<b[j]<<endl;
      }
   } 
}//end//     

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* copy a column of a matrix to a vector                                    */
/*==========================================================================*/
void colcopy_mat_to_vec(double **a, double *b, int n, int m) 
// m column number : input
// a E R^(n x $) : input  (!! $ >= m) 
// b E R^(n) : output 
{//begin//
   for (int i=1;i<=n;i++)  
       b[i] = a[i][m]; 

}//end//     

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* copy a vector to an other vector                                         */
/*==========================================================================*/
void copy_vec_to_vec(double *a, double *b, int n) 
// n length : input
// a E R^(n) : input
// b E R^(n) : output 
{//begin//
   for (int i=1;i<=n;i++)  
       b[i] = a[i]; 

/*--------------------------------------------------------------------------*/
}//end//     
/*==========================================================================*/

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* copy 3 1D vectors (source) to another 3 1D vectors (target)              */
/*==========================================================================*/
void copy_3vec_to_3vec(double *as, double *bs, double *cs,
                       double *at, double *bt, double *ct, int n) 
// n length : input
// as E R^(n) : input
// bs E R^(n) : input
// cs E R^(n) : input
// at E R^(n) : output 
// bt E R^(n) : output 
// ct E R^(n) : output 
{//begin//
  for (int i=1;i<=n;i++) { 
       at[i] = as[i];
       bt[i] = bs[i]; 
       ct[i] = cs[i];
  }//endfor//
/*--------------------------------------------------------------------------*/
}//end//     
/*==========================================================================*/

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* copy 3 1D vectors of length n  (source) to                               */
/*      a 1D vector  of length 3n (target)                                  */
/*==========================================================================*/
void copy_3vec_to_vec(double *a, double *b, double *c, double *abc, int n) 
// n length : input
// a E R^(n) : input
// b E R^(n) : input
// c E R^(n) : input
// abc E R^(3n) : output  
{//begin//
  for (int i=1;i<=n;i++) { 
       abc[3*(i-1)+1] = a[i];
       abc[3*(i-1)+2] = b[i];
       abc[3*(i-1)+3] = c[i];
  }//endfor//
/*--------------------------------------------------------------------------*/
}//end//     
/*==========================================================================*/

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* copy a 1D vector  of length 3n (source) to                               */
/*      3 1D vectors of length n  (target)                                  */
/*==========================================================================*/
void copy_vec_to_3vec( double *abc, double *a, double *b, double *c, int n) 
// n length : input
// abc E R^(3n) : input 
// a E R^(n) : output
// b E R^(n) : output
// c E R^(n) : output
{//begin//
  for (int i=1;i<=n;i++) { 
       a[i] = abc[3*(i-1)+1];
       b[i] = abc[3*(i-1)+2];
       c[i] = abc[3*(i-1)+3];
  }//endfor//
/*--------------------------------------------------------------------------*/
}//end//     
/*==========================================================================*/

/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* nrerror                                                                  */
/*==========================================================================*/

void nrerror(char error_text[])
  // Standard error handler
{
 fprintf(stderr,"Numerical run-time error...\n");
 fprintf(stderr,"%s\n",error_text);
 fprintf(stderr,"...now exiting to system...\n"); 
 exit(1);
} 


/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* tred2                                                                    */
/*==========================================================================*/
 
void tred2(double** a, int n, double d[], double e[])
// Householder reduction of a real, symmetric matrix a[1..n][1..n]. On 
// output, a is replaced by the orthogonal matrix Q effecting the 
// transformation. d[1..n] returns the diagonal elements of the 
// tridiagonal matrix, and e[1..n] the off-diagonal elements, with
// e[1]=0. Several statements, as noted in comments, can be omitted
// if only eigenvalues are to be found, in which case a contains no
// useful information on output. Otherwise they are to be included.
{
	int l,k,j,i;
	double scale,hh,h,g,f;

	for (i=n;i>=2;i--) {
		l=i-1;
		h=scale=0.0;
		if (l > 1) {
			for (k=1;k<=l;k++)
				scale += fabs(a[i][k]);
			if (scale == 0.0)
				e[i]=a[i][l];
			else {
				for (k=1;k<=l;k++) {
					a[i][k] /= scale;
					h += a[i][k]*a[i][k];
				}
				f=a[i][l];
				g = f>0 ? -sqrt(h) : sqrt(h);
				e[i]=scale*g;
				h -= f*g;
				a[i][l]=f-g;
				f=0.0;
				for (j=1;j<=l;j++) {
				/* Next statement can be omitted if eigenvectors not wanted */
					a[j][i]=a[i][j]/h;
					g=0.0;
					for (k=1;k<=j;k++)
						g += a[j][k]*a[i][k];
					for (k=j+1;k<=l;k++)
						g += a[k][j]*a[i][k];
					e[j]=g/h;
					f += e[j]*a[i][j];
				}
				hh=f/(h+h);
				for (j=1;j<=l;j++) {
					f=a[i][j];
					e[j]=g=e[j]-hh*f;
					for (k=1;k<=j;k++)
						a[j][k] -= (f*e[k]+g*a[i][k]);
				}
			}
		} else
			e[i]=a[i][l];
		d[i]=h;
	}
	/* Next statement can be omitted if eigenvectors not wanted */
	d[1]=0.0;
	e[1]=0.0;
	/* Contents of this loop can be omitted if eigenvectors not
			wanted except for statement d[i]=a[i][i]; */
	for (i=1;i<=n;i++) {
		l=i-1;
		if (d[i]) {
			for (j=1;j<=l;j++) {
				g=0.0;
				for (k=1;k<=l;k++)
					g += a[i][k]*a[k][j];
				for (k=1;k<=l;k++)
					a[k][j] -= g*a[k][i];
			}
		}
		d[i]=a[i][i];
		a[i][i]=1.0;
		for (j=1;j<=l;j++) a[j][i]=a[i][j]=0.0;
	}
}


/*==========================================================================*/
/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*==========================================================================*/
/* tred2                                                                    */
/*==========================================================================*/
 

//****************************************************************//
//****************************************************************//

void tqli(double d[], double e[], int n, double **z)
// Computes eigenvalues, eigenvectors of a tridiagonal matrix
// using the QL algorithm with implicit shifts. 
// A real symmetric matrix could be reduced to tridiagonal
// for by tred2 (see. above).
// d[1..n] contains the diagonal elements of the matrix 
// which is overwritten by the eigenvalues (output).
// e[1..n] imputs the subdiagonal elements of the tridiagonal
// matrix, with e[1] is arbitrary. On output e is lost.
// If the eigenvectors of a tridiagonal matrix are desired,
// the matrix z[1..n][1..n] is input as the identity matrix.
// If eigenvectors of a matrix that has been reduced by tred2 are 
// reqired, then z is input as the matrix output by tred2.
// In either case, the k^(th) column of z returns the normalized 
// eigenvector corresponding to d[k]. 
{
  int m,l,iter,i,k;
  double s,r,p,g,f,dd,c,b;

  for (i=2;i<=n;i++) e[i-1]=e[i];
  e[n]=0.0;
  for (l=1;l<=n;l++) {
      iter=0;
      do {
	for (m=l;m<=n-1;m++) {
            dd=fabs(d[m])+fabs(d[m+1]);
            if ((double)(fabs(e[m])+dd)==dd) break;
	}
        if (m!=l) {
           if (iter++ == 30) nrerror("Too many iterations in tqli");
           g=(d[l+1]-d[l])/(2.0*e[l]);
           r=pythag(g,1.0);
           g=d[m]-d[l]+e[l]/(g+SIGN(r,g));
           s=c=1.0;
           p=0.0;
	   for (i=m-1;i>=l;i--) {
               f=s*e[i];
               b=c*e[i];
               e[i+1]=(r=pythag(f,g));
               if (r == 0.0) {
                  d[i+1]-=p;
                  e[m]=0.0;
                  break;
	       }
               s=f/r;
               c=g/r;
               g=d[i+1]-p;
               r=(d[i]-g)*s+2.0*c*b;
               d[i+1]=g+(p=s*r);
               g=c*r-b;
               for (k=1;k<=n;k++) {
                   f=z[k][i+1];
                   z[k][i+1]=s*z[k][i]+c*f;
                   z[k][i]=c*z[k][i]-s*f;
	       }
	   }
           if (r == 0.0 && i>=1) continue;
           d[l] -= p;
           e[l]=g;
           e[m]=0.0;
	}
      } while (m!=l);
  }
}

//****************************************************************//
//****************************************************************//

void eigsrt(double d[], double **v, int n)
  // Given the eigenvalues d[1..n] and eigenvectors v[1..n][1..n]
  // as output from jacobi (N.R. in C (11.1)) or tqli (N.R. in C
  // (11.3)), this routine sorts the eigenvalues into descending
  // order, and rearranges the columns of v correspondingly. 
  // The method is straight insertion.
{
    int k,j,i;
    double p;

    for (i=1;i<n;i++) {
        p=d[k=i];
        for (j=i+1;j<=n;j++)
            if (d[j] >= p) p=d[k=j];
        if (k != i) {
           d[k]=d[i];
           d[i]=p;
           for (j=1;j<=n;j++) {
               p=v[j][i];
               v[j][i]=v[j][k];
               v[j][k]=p;
	   }
	}
    }
}

//****************************************************************//
//****************************************************************//

void gaucof(int n, double a[], double b[], double amu0, double x[], double w[])
  // Computes the abscissas and weights for a Gaussian quadrature formula
  // from the Jacobi matrix. On input, a[1..n] and b[1..n] are the coefficients
  // of the recurrence relation for the set of monic orthogonal polynomials.
  // The quantity mu0 = /Int_{a}^b W(x) dx is input as amu0. The abscissas
  // x[1..n] are returned in decreasing order, with the corresponding weights
  // in w[1..n]. The arrays a and b are modified. Execution can speed up by
  // modifying tqli and eigsrt to compute only the first component of each
  // eigenvector.
{
  int i,j;
  double **z;

  z=dmatrix(1,n,1,n);
  for (i=1;i<=n;i++) {
    if (i != 1) b[i]=b[i];    // Set up superdiagonal of Jacobi matrix.
    for (j=1;j<=n;j++) z[i][j]=(double)(i == j);
    // Set up identity matrix for tqli to compute eigenvectors.
  }

  tqli(a,b,n,z);
  eigsrt(a,z,n);                    // Sort eigenvalues into decr. order.
  for (i=1;i<=n;i++) {
      x[i]=a[n+1-i];
      w[i]=amu0*z[1][n+1-i]*z[1][n+1-i];
  }
  free_dmatrix(z,1,n,1,n);
}


// RANDOM GENERATOR FUNCTION
void ran_md_gen( double &number )
//------------------------------------------------------------------//
{//begin//

  int ixl = 90293872;
  int ial = 29760349;
  int iml = 23230348;

  static int xl = ixl;

  xl = (ial*xl % iml);
  number = fabs((double(xl))/(double(iml)));

}//end//
//------------------------------------------------------------------//


// periodic boundary condition on [0, 2Pi)
void pbc_0_2Pi( double *t, int n )
//------------------------------------------------------------------//
{//begin//

  int i;

  for (i=1;i<=n;i++) {
       if (t[i] < 0) {
	   t[i] += 2.0*M_PI;
       }
       if (t[i] >= 2.0*M_PI) {
	   t[i] -= 2.0*M_PI;
       }
  }

}//end//
//------------------------------------------------------------------//


// linear approximation to sin(x) on [0, 2Pi]
double linsin( double x )
//------------------------------------------------------------------//
{//begin//

      
      double frac, func;
      double unit, onefourth, threefourth, four, two;

      unit        = 2.0*M_PI;
      onefourth   = unit/4.0;
      threefourth = 3.0/4.0*unit;
      four        = 4.0;
      two         = 2.0;
      
      frac = x - floor(x/unit)*unit;

      if (frac < onefourth) {func = four*frac/unit;}
      if ( (frac >= onefourth) && (frac <= threefourth) ) {
            func = -four*frac/unit + two;
      }/*endif*/
      if (frac > threefourth) {func = four*frac/unit - four;}

      return func;

}//end//
//------------------------------------------------------------------//
