#include "../../include/standard.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/bfgs_min.h"
#include "../../include/utillib.h"

#define DEBUG
#undef DEBUG
#define REPORT


/*===========================================================================*/
/*===========================================================================*/
/* Broyden-Fletcher-Goldfarb-Shanno minimization                             */
/* implementation of numerical recipies >>dfpmin<< routine                   */
void dfpmin(double *p, int n, double ftol, int *iter, double *fret,
            double (*func)(double *), void (*dfunc)(double*,double*))
/*---------------------------------------------------------------------------*/
{/*begin routine*/
/*---------------------------------------------------------------------------*/

	int j,i,its,itmax;
	double fp,fae,fad,fac;
	double *xi,*g,*dg,*hdg;
	double **hessin;

        extern  System  *global_system;
        System *system = global_system;
        double *xt, *yt, *zt, *pt;

        void linmin(double*,double*,int,double*,double (*)(double*));
        void nrerror(char*);
        double *dvector(long, long);
	void free_dvector(double*, long, long);
        double **dmatrix(long, long, long, long);
        void free_dmatrix(double**, long, long, long, long);

        //_____________________________________________________postprop
        int  postprop = system->general.postprop_minimize;
        int *postprop_succeed = &(system->maps.postprop_succeed);
        //_____________________________________________________postprop

        //_____________________________________________________itmax
        if (postprop == 1) {
	    itmax = system->general.postprop_minimize_itmax;
	} else {
	    itmax = ITMAX;
        }/*endif*/  
        //_____________________________________________________itmax

	hessin=dmatrix(1,n,1,n);
	xi=dvector(1,n);
	g=dvector(1,n);
	dg=dvector(1,n);
	hdg=dvector(1,n);
	fp=(*func)(p);
	(*dfunc)(p,g);
	for (i=1;i<=n;i++) {
		for (j=1;j<=n;j++) hessin[i][j]=0.0;
		hessin[i][i]=1.0;
		xi[i] = -g[i];
	}
	for (its=1;its<=itmax;its++) {
		*iter=its;
#ifdef REPORT
                //_____________________________________________________output
                if (its==1) {
                  print_doubleline();         
		  printf("BFGS minimization\n");
                  printf("BFGS: Broyden-Fletcher-Goldfarb-Shanno\n");
                  printf("Current Desired Conditions ");
                  printf(">> 2.0 |A-B| <= ftol (|A|+|B|+EPS) <<\n");
                  printf("{ ftol: %.16g, ",ftol);
                  printf("EPS: %.16g }\n",EPS);
                  print_line();
                }
                if ( ((its % (system->general.nstat)) == 0) && 
                     (system->general.minimize_report >  0) ) {/*nstat*/
                    if (system->general.write_energy_unit==1) {
                          printf("energy %.16g kcal/mol  ||  ",fp*KCAL);
                    }
                    if (system->general.write_energy_unit==2) {
                          printf("energy %.16g Ha  ||  ",fp);
                    }
                    if ( system->general.minimize_report >  1) {/*report*/ 
                         if ( (system->general.prop_type==2) || /*torsional*/
                              (system->general.prop_type==3) ) {/*bend*/

                              if  (system->general.prop_type==2) {
       		                  system->pos.tors_transformXtoT();
                                  pt = system->pos.t;
                                  system->pos.t = p;
                                  system->pos.tors_transformTtoX();
                                  system->pos.t = pt;
                              }/*endif-prop_type==2*/
                              if  (system->general.prop_type==3) {
       		                  system->pos.tors_transformXtoB();
                                  pt = system->pos.b;
                                  system->pos.b = p;
                                  system->pos.tors_transformBtoX();
                                  system->pos.b = pt;
                              }/*endif-prop_type==3*/
                           
                              xt = system->pos.x; 
                              yt = system->pos.y; 
                              zt = system->pos.z;
                              system->pos.x = system->pos.xi;
                              system->pos.y = system->pos.yi;
                              system->pos.z = system->pos.zi;
                         }/*torsional*/
                         output(&(system->pos),*system);
                         if ( (system->general.prop_type==2) || /*torsional*/
                              (system->general.prop_type==3) ) {/*bend*/
                              system->pos.x = xt; 
                              system->pos.y = yt; 
                              system->pos.z = zt;
                         }/*torsional*/
                    }/*report*/
                }/*nstat*/
                //_____________________________________________________output
#endif

                //_____________________________________________________quit
                if (its==itmax) {
                  printf("\n");
                  print_line();         
		  printf("Maximum number of iterations %d reached\n",itmax);
		  printf("BFGS minimization terminated\n");
                  print_doubleline();
		        free_dvector(hdg,1,n);
			free_dvector(dg,1,n);
			free_dvector(g,1,n);
			free_dvector(xi,1,n);
			free_dmatrix(hessin,1,n,1,n);
			return;
                }
                //_____________________________________________________quit


#ifdef DEBUG
                //_____________________________________________________debug
		printf("Check Point I in frprm\n");
	        printf("*fret %.16g fp %.16g \n",*fret,fp);
                printf("max gradients \n");
                double max_grad;
                max_grad = 0.0;
                for (j=1;j<=n;j++) {
		     max_grad = MAX(max_grad,fabs(xi[j]));
                }
                printf("max_gradient %.16g\n",max_grad);
                getchar();
                //_____________________________________________________debug
#endif

		linmin(p,xi,n,fret,func);

                //_____________________________________________________quit
		if ( (*fret > fp) && (postprop==1) ) {
                  printf("\n");
                  print_line();         
		  printf("A > B -> minimization is not improving\n");
                  printf("A = %.16g \t\t\t B = %.16g\n\n",*fret,fp);
		  printf("Conjugate gradient minimization terminated\n");
                  print_doubleline();
                  
		  *postprop_succeed = 0;

		        free_dvector(hdg,1,n);
			free_dvector(dg,1,n);
			free_dvector(g,1,n);
			free_dvector(xi,1,n);
			free_dmatrix(hessin,1,n,1,n);
			return;
                }
                //_____________________________________________________quit

#ifdef REPORT
                //_____________________________________________________output
                if ( ((its % (system->general.nstat)) == 0) && 
                     (system->general.minimize_report >  0) ) {
                    printf("A = %.16g\n \t\t\t\t\tB = %.16g\n\n",*fret,fp);
                }
                //_____________________________________________________output
#endif

		if (2.0*fabs(*fret-fp) <= ftol*(fabs(*fret)+fabs(fp)+EPS)) {
#ifdef REPORT
                //_____________________________________________________output
	            fp=(*func)(p);
                    if (system->general.write_energy_unit==1) {
                          printf("energy %.16g kcal/mol  ||  ",fp*KCAL);
                    }
                    if (system->general.write_energy_unit==2) {
                          printf("energy %.16g Ha  ||  ",fp);
                    }
                    printf("A = %.16g\n \t\t\t\t\tB = %.16g\n\n",*fret,fp);
                    if ( system->general.minimize_report >  1) {/*report*/
                         if ( (system->general.prop_type==2) || /*torsional*/
                              (system->general.prop_type==3) ) {/*bend*/
                              xt = system->pos.x; 
                              yt = system->pos.y; 
                              zt = system->pos.z;
                              system->pos.x = system->pos.xi;
                              system->pos.y = system->pos.yi;
                              system->pos.z = system->pos.zi;
                         }/*torsional*/
                         output(&(system->pos),*system);
                         if ( (system->general.prop_type==2) || /*torsional*/
                              (system->general.prop_type==3) ) {/*bend*/
                              system->pos.x = xt; 
                              system->pos.y = yt; 
                              system->pos.z = zt;
                         }/*torsional*/
                    }/*report*/
                  print_line();
                  printf("Desired condition is satistied\n");
		  printf("BFGS minimization terminated\n");
                  print_doubleline();
                //_____________________________________________________output
#endif
			free_dvector(hdg,1,n);
			free_dvector(dg,1,n);
			free_dvector(g,1,n);
			free_dvector(xi,1,n);
			free_dmatrix(hessin,1,n,1,n);
			return;
		}
		fp=(*fret);
		for (i=1;i<=n;i++) dg[i]=g[i];
		*fret=(*func)(p);
		(*dfunc)(p,g);
		for (i=1;i<=n;i++) dg[i]=g[i]-dg[i];
		for (i=1;i<=n;i++) {
			hdg[i]=0.0;
			for (j=1;j<=n;j++) hdg[i] += hessin[i][j]*dg[j];
		}
		fac=fae=0.0;
		for (i=1;i<=n;i++) {
			fac += dg[i]*xi[i];
			fae += dg[i]*hdg[i];
		}
		fac=1.0/fac;
		fad=1.0/fae;
		for (i=1;i<=n;i++) dg[i]=fac*xi[i]-fad*hdg[i];
		for (i=1;i<=n;i++)
			for (j=1;j<=n;j++)
				hessin[i][j] += fac*xi[i]*xi[j]
					-fad*hdg[i]*hdg[j]+fae*dg[i]*dg[j];
		for (i=1;i<=n;i++) {
			xi[i]=0.0;
			for (j=1;j<=n;j++) xi[i] -= hessin[i][j]*g[j];
		}
	}
	nrerror("Too many iterations in DFPMIN");
/*---------------------------------------------------------------------------*/
}/*end routine*/
/*===========================================================================*/
