#include "../../include/standard.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/linmin.h"
#include "../../include/utillib.h"

#define DEBUG
#undef DEBUG

/*===========================================================================*/
/*===========================================================================*/
void linmin(double *p,double *xi,int n,double *fret,double (*func)(double*))
/*---------------------------------------------------------------------------*/
{/*begin routine*/
/*---------------------------------------------------------------------------*/

	int j;
	double xx,xmin,fx,fb,fa,bx,ax;


        double brent(double,double,double,double (*f)(double),double,double *);
        double f1dim(double);

        void mnbrak(double*,double*,double*,double*,
                    double*,double*,double (*)(double));

        double *dvector(long, long);
	void free_dvector(double*, long, long);

	ncom=n;
	pcom=dvector(1,n);
	xicom=dvector(1,n);
	nrfunc=func;
	for (j=1;j<=n;j++) {
		pcom[j]=p[j];
		xicom[j]=xi[j];
	}
	ax=0.0;
	xx=1.0;
	bx=2.0;
	mnbrak(&ax,&xx,&bx,&fa,&fx,&fb,f1dim);
	*fret=brent(ax,xx,bx,f1dim,TOL_LINMIN,&xmin);
	for (j=1;j<=n;j++) {
		xi[j] *= xmin;
		p[j] += xi[j];
	}
	free_dvector(xicom,1,n);
	free_dvector(pcom,1,n);

/*---------------------------------------------------------------------------*/
}/*end routine*/
/*===========================================================================*/


/*===========================================================================*/
/*===========================================================================*/
double brent(double ax, double bx, double cx, double (*f)(double), double tol, 
                                                               double *xmin)
/*---------------------------------------------------------------------------*/
{/*begin routine*/
/*---------------------------------------------------------------------------*/

	int iter;
	double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
	double e=0.0;
	void nrerror(char *);

	a=((ax < cx) ? ax : cx);
	b=((ax > cx) ? ax : cx);
	x=w=v=bx;
	fw=fv=fx=(*f)(x);
	for (iter=1;iter<=ITMAX_BRENT;iter++) {
		xm=0.5*(a+b);
		tol2=2.0*(tol1=tol*fabs(x)+ZEPS_BRENT);
		if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
			*xmin=x;
			return fx;
		}
		if (fabs(e) > tol1) {
			r=(x-w)*(fx-fv);
			q=(x-v)*(fx-fw);
			p=(x-v)*q-(x-w)*r;
			q=2.0*(q-r);
			if (q > 0.0) p = -p;
			q=fabs(q);
			etemp=e;
			e=d;
			if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
				d=CGOLD_BRENT*(e=(x >= xm ? a-x : b-x));
			else {
				d=p/q;
				u=x+d;
				if (u-a < tol2 || b-u < tol2)
					d=SIGN_BRENT(tol1,xm-x);
			}
		} else {
			d=CGOLD_BRENT*(e=(x >= xm ? a-x : b-x));
		}
		u=(fabs(d) >= tol1 ? x+d : x+SIGN_BRENT(tol1,d));
		fu=(*f)(u);
		if (fu <= fx) {
			if (u >= x) a=x; else b=x;
			SHFT(v,w,x,u)
			SHFT(fv,fw,fx,fu)
		} else {
			if (u < x) a=u; else b=u;
			if (fu <= fw || w == x) {
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} else if (fu <= fv || v == x || v == w) {
				v=u;
				fv=fu;
			}
		}
	}
	nrerror("Too many iterations in BRENT");
	*xmin=x;
	return fx;

/*---------------------------------------------------------------------------*/
}/*end routine*/
/*===========================================================================*/


/*===========================================================================*/
/*===========================================================================*/
double f1dim(double x)
/*---------------------------------------------------------------------------*/
{/*begin routine*/
/*---------------------------------------------------------------------------*/

	int j;
	double f,*xt;

        double *dvector(long, long);
	void free_dvector(double*, long, long);

	xt=dvector(1,ncom);
	for (j=1;j<=ncom;j++) xt[j]=pcom[j]+x*xicom[j];
	f=(*nrfunc)(xt);
	free_dvector(xt,1,ncom);
	return f;

/*---------------------------------------------------------------------------*/
}/*end routine*/
/*===========================================================================*/


/*===========================================================================*/
/*===========================================================================*/
void mnbrak(double *ax, double *bx, double *cx, 
            double *fa, double *fb, double *fc, double (*func)(double))
/*---------------------------------------------------------------------------*/
{/*begin routine*/
/*---------------------------------------------------------------------------*/

	double ulim,u,r,q,fu,dum;

	*fa=(*func)(*ax);
	*fb=(*func)(*bx);
	if (*fb > *fa) {
		SHFT_MNBRAK(dum,*ax,*bx,dum)
		SHFT_MNBRAK(dum,*fb,*fa,dum)
	}
	*cx=(*bx)+GOLD_MNBRAK*(*bx-*ax);
	*fc=(*func)(*cx);
	while (*fb > *fc) {
		r=(*bx-*ax)*(*fb-*fc);
		q=(*bx-*cx)*(*fb-*fa);
		u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/
			(2.0*SIGN_MNBRAK(MAX_MNBRAK(fabs(q-r),TINY_MNBRAK),q-r));
		ulim=(*bx)+GLIMIT_MNBRAK*(*cx-*bx);
		if ((*bx-u)*(u-*cx) > 0.0) {
			fu=(*func)(u);
			if (fu < *fc) {
				*ax=(*bx);
				*bx=u;
				*fa=(*fb);
				*fb=fu;
				return;
			} else if (fu > *fb) {
				*cx=u;
				*fc=fu;
				return;
			}
			u=(*cx)+GOLD_MNBRAK*(*cx-*bx);
			fu=(*func)(u);
		} else if ((*cx-u)*(u-ulim) > 0.0) {
			fu=(*func)(u);
			if (fu < *fc) {
				SHFT_MNBRAK(*bx,*cx,u,*cx+GOLD_MNBRAK*(*cx-*bx))
				SHFT_MNBRAK(*fb,*fc,fu,(*func)(u))
			}
		} else if ((u-ulim)*(ulim-*cx) >= 0.0) {
			u=ulim;
			fu=(*func)(u);
		} else {
			u=(*cx)+GOLD_MNBRAK*(*cx-*bx);
			fu=(*func)(u);
		}
		SHFT_MNBRAK(*ax,*bx,*cx,u)
		SHFT_MNBRAK(*fa,*fb,*fc,fu)
	}
/*---------------------------------------------------------------------------*/
}/*end routine*/
/*===========================================================================*/
