#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_OFF


/*=========================================================================*/
/*=========================================================================*/
void output(Chain *X, System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, nres;
  int  imol, ires, jres, iatm;
  double einter,epot,econv;

  int system_definition    = system.general.system_definition;
  int K                    = system.general.n_chains_mc;
  int natom                = system.pos.natom;
  int ntors                = system.tors_defs.num_trans;
  int write_energy_unit    = system.general.write_energy_unit;
  int action_type          = system.general.action_type;
  int *aa_type             = system.maps.aa_type;
  char **aa3_code          = system.maps.aa3_code;
  char **aa1_code          = system.maps.aa1_code;
  char *atom_pos_filename  = system.filenames.atom_pos_filename;
  char *tors_pos_filename  = system.filenames.tors_pos_filename;
  char *pos_out_filename   = system.filenames.pos_out_filename;
  char *epot_filename      = system.filenames.epot_filename;
  char *einter_filename    = system.filenames.einter_filename;
  double *tors_X           = X[0].pos.tors_X;
  double *x                = X[0].pos.x;
  double *y                = X[0].pos.y;
  double *z                = X[0].pos.z; 
  int *permute_seq         = X[0].pos.permute_seq;
  int static output_count  = 0;

  int    nmol              = system.maps.nmol;
  int   *nres_mol          = system.maps.nres_mol;
  int  **ind_res_mol       = system.maps.ind_res_mol;
  char **mol_label         = system.maps.mol_label;
  char **mol_chainid       = system.maps.mol_chainid;

  char **upac_atm_label    = system.maps.upac_atm_label;
  Residue *residue         = system.pos.residue;
  FILE *output_file;

  output_count++;


/*-------------------------------------------------------------------------*/
/* 0) Report detailed energy terms for target chain                        */

  X[0].pos.report_energy(system);

/*-------------------------------------------------------------------------*/
/* I) Coordinates and trajectory                                           */

  output_file = fopen(atom_pos_filename, "a");
  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     fprintf(output_file,"MODEL %d\n",output_count);
     if (system_definition==1) {
        for (i=1;i<=natom;i++) {
            fprintf(output_file,"ATOM   %4d  CA  %3s A%4d    %8.3f%8.3f%8.3f\n",
                   i,aa3_code[aa_type[i]],i,x[i]*BOHR,y[i]*BOHR,z[i]*BOHR);
        }
     }/*end if*/
     if (system_definition==2) {
       for (imol=1;imol<=nmol;imol++) {
	nres = nres_mol[imol];
        for (jres=1;jres<=nres;jres++) {
	    ires = ind_res_mol[imol][jres];
            for (iatm=1;iatm<=residue[ires].natom;iatm++) {
	 /*1-6*/fprintf(output_file,"ATOM  ");  
        /*7-11*/fprintf(output_file,"%5d",residue[ires].atom[iatm]);
	  /*12*/fprintf(output_file," ");
       /*13-16*/fprintf(output_file,"%4s",upac_atm_label[residue[ires].atom[iatm]]);
	  /*17*/fprintf(output_file," "); /*altLoc*/
       /*18-20*/fprintf(output_file,"%3s",residue[ires].residue_name);
          /*21*/fprintf(output_file," ");
	  /*22*/fprintf(output_file,"%1s",mol_chainid[imol]); /*chainID*/
       /*23-26*/fprintf(output_file,"%4d",ires);
       /*27-30*/fprintf(output_file,"    ");
       /*31-54*/fprintf(output_file,"%8.3f%8.3f%8.3f\n",
                                        x[residue[ires].atom[iatm]]*BOHR,
                                        y[residue[ires].atom[iatm]]*BOHR,
                                        z[residue[ires].atom[iatm]]*BOHR);
            }/*end for-iatm*/
	}/*end for-jres*/
       }/*end for-imol*/
     }/*end if*/
     fprintf(output_file,"TER\nENDMDL\n");
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     fprintf(output_file,">\n");
     for (i=1;i<=natom;i++) {
          fprintf(output_file,"%s",aa1_code[aa_type[permute_seq[i]]]);
     }
     fprintf(output_file,"\n");
  }/*if*/ 
  fclose(output_file);

  output_file = fopen(pos_out_filename, "w");
  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     if (system_definition==1) {
        for (i=1;i<=natom;i++) {
             fprintf(output_file,"ATOM   %4d  CA  %3s A%4d    %8.3f%8.3f%8.3f\n",
                         i,aa3_code[aa_type[i]],i,x[i]*BOHR,y[i]*BOHR,z[i]*BOHR);
        }/*end for*/
     }/*end if*/
     if (system_definition==2) {
       if (system.general.stride_defined==1) {
	   for (i=1;i<=system.tors_defs.nstride;i++) {
                fprintf(output_file,"STRIDE %s\n",
                                    &(system.tors_defs.stride[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.torcompl_defined==1) {
	   for (i=1;i<=system.tors_defs.ntorcompl;i++) {
                fprintf(output_file,"TORCOMPL %s\n",
                                    &(system.tors_defs.torcompl[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.closcompl_defined==1) {
	   for (i=1;i<=system.tors_defs.ncloscompl;i++) {
                fprintf(output_file,"CLOSCOMPL %s\n",
                                    &(system.tors_defs.closcompl[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.cblc_defined==1) {
                fprintf(output_file,"CBLC %s\n",
                                    &(system.tors_defs.cblc[0]));
       }/*end if*/
       for (imol=1;imol<=nmol;imol++) {
	nres = nres_mol[imol];
        for (jres=1;jres<=nres;jres++) {
	    ires = ind_res_mol[imol][jres];
            for (iatm=1;iatm<=residue[ires].natom;iatm++) {
	 /*1-6*/fprintf(output_file,"ATOM  ");  
        /*7-11*/fprintf(output_file,"%5d",residue[ires].atom[iatm]);
	  /*12*/fprintf(output_file," ");
       /*13-16*/fprintf(output_file,"%4s",upac_atm_label[residue[ires].atom[iatm]]);
	  /*17*/fprintf(output_file," "); /*altLoc*/
       /*18-20*/fprintf(output_file,"%3s",residue[ires].residue_name);
          /*21*/fprintf(output_file," ");
	  /*22*/fprintf(output_file,"%1s",mol_chainid[imol]); /*chainID*/
       /*23-26*/fprintf(output_file,"%4d",ires);
       /*27-30*/fprintf(output_file,"    ");
       /*31-54*/fprintf(output_file,"%8.3f%8.3f%8.3f\n",
                                        x[residue[ires].atom[iatm]]*BOHR,
                                        y[residue[ires].atom[iatm]]*BOHR,
                                        z[residue[ires].atom[iatm]]*BOHR);
            }/*end for-iatm*/
	}/*end for-jres*/
       }/*end for-imol*/
     }/*end if*/
     fprintf(output_file,"TER\nENDMDL\n");
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     fprintf(output_file,">\n");
     for (i=1;i<=natom;i++) {
          fprintf(output_file,"%s",aa1_code[aa_type[permute_seq[i]]]);
     }
     fprintf(output_file,"\n");
  }/*if*/
  fclose(output_file);

/*-------------------------------------------------------------------------*/
/* II) Torsional dihedral                                                  */
  output_file = fopen(tors_pos_filename, "a");
  if (output_count==1) {
      if (system.general.stride_defined==1) {
          for (i=1;i<=system.tors_defs.nstride;i++) {
               fprintf(output_file,"STRIDE %s\n",
                                   &(system.tors_defs.stride[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.torcompl_defined==1) {
	  for (i=1;i<=system.tors_defs.ntorcompl;i++) {
               fprintf(output_file,"TORCOMPL %s\n",
                                   &(system.tors_defs.torcompl[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.closcompl_defined==1) {
	  for (i=1;i<=system.tors_defs.ncloscompl;i++) {
               fprintf(output_file,"CLOSCOMPL %s\n",
                                   &(system.tors_defs.closcompl[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.cblc_defined==1) {
               fprintf(output_file,"CBLC %s\n",
                                   &(system.tors_defs.cblc[0]));
      }/*end if*/
  }/*end if*/
  fprintf(output_file,"MODEL %d ",output_count);
  if (tors_X != 0) {
     for (i=1;i<=ntors;i++) {
          fprintf(output_file,"%8.3f",tors_X[i]);
     }/*endfor*/
  }/*endif*/
  fprintf(output_file,"\n");
  fclose(output_file);

/*-------------------------------------------------------------------------*/
/* III) Potential energy                                                   */

  output_file = fopen(epot_filename, "a");
  if (output_count==1) {
      if (write_energy_unit==1) {/*kcal/mol*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in kcal/mol\n",K+1);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in Ha\n ",K+1);
      }
  }
  for (i=0;i<=K;i++) {
      epot   = X[i].pos.epot;
      if (write_energy_unit==1) {/*kcal/mol*/
         fprintf(output_file,"%g ",epot*KCAL);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
         fprintf(output_file,"%g ",epot);
      }
  }
  fprintf(output_file,"\n");
  fclose(output_file);

/*-------------------------------------------------------------------------*/
/* Inter energy */

  output_file = fopen(einter_filename, "a");
  if (output_count==1) {
      if (write_energy_unit==1) {/*kcal/mol*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in kcal/mol\n",K+1);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in Ha\n",K+1);
      }
  }
  for (i=0;i<=K;i++) {
      if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
         einter = X[i].pos.einter;
      }
      else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
         einter = system.inter.getPotPermute(&(X[i].pos));
      }
      if (write_energy_unit==1) {/*kcal/mol*/
         fprintf(output_file,"%g ",einter*KCAL);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
         fprintf(output_file,"%g ",einter);
      }
  }    
  fprintf(output_file,"\n");
  fclose(output_file);

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void output(Pos *pos, System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, nres;
  int imol, ires, jres, iatm;
  double einter,epot,econv;

  int system_definition    = system.general.system_definition;
  int K                    = system.general.n_chains_mc;
  int minimize             = system.general.minimize;
  int mdrun                = system.general.md;
  int minimize_type        = system.general.minimize_type;
  int minimize_report      = system.general.minimize_report;
  int natom                = system.pos.natom;
  int ntors                = system.tors_defs.num_trans;
  int write_energy_unit    = system.general.write_energy_unit;
  int action_type          = system.general.action_type;
  int *aa_type             = system.maps.aa_type;
  char **aa3_code          = system.maps.aa3_code;
  char **aa1_code          = system.maps.aa1_code;
  char *atom_pos_filename  = system.filenames.atom_pos_filename;
  char *tors_pos_filename  = system.filenames.tors_pos_filename;
  char *pos_out_filename   = system.filenames.pos_out_filename;
  char *epot_filename      = system.filenames.epot_filename;
  char *einter_filename    = system.filenames.einter_filename;
  double *tors_X           = pos->tors_X;
  double *x                = pos->x;
  double *y                = pos->y;
  double *z                = pos->z; 
  int *permute_seq         = pos->permute_seq;
  int static output_count  = 0;

  int    nmol              = system.maps.nmol;
  int   *nres_mol          = system.maps.nres_mol;
  int  **ind_res_mol       = system.maps.ind_res_mol;
  char **mol_label         = system.maps.mol_label;
  char **mol_chainid       = system.maps.mol_chainid;

  char **upac_atm_label    = system.maps.upac_atm_label;
  Residue *residue         = system.pos.residue;
  FILE *output_file;

  output_count++;



/*-------------------------------------------------------------------------*/
/* 0) Report detailed energy terms                                         */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* 0.A) minimization                                                       */
/*  If simulated annealing Monte Carlo (samc) minimization is on           */
/*  If simulated tempering simulated annealing (stsamc) minimization is on */
/*  Other types (cg,bfgs) of minimizations output these terms inteernally  */

  if ( (minimize        == 1) && 
       (minimize_type   >= 3) &&
       (minimize_report >= 1) ) {
        pos->report_energy(system);
  }/*endif*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* 0.B) md                                                                 */
/*    If simple md is performed                                            */

  if (mdrun == 1) {pos->report_energy(system);}


/*-------------------------------------------------------------------------*/
/* I) Coordinates and trajectory                                           */

  output_file = fopen(atom_pos_filename, "a");
  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     fprintf(output_file,"MODEL %d\n",output_count);
     if (system_definition==1) {
        for (i=1;i<=natom;i++) {
            fprintf(output_file,"ATOM   %4d  CA  %3s A%4d    %8.3f%8.3f%8.3f\n",
                   i,aa3_code[aa_type[i]],i,x[i]*BOHR,y[i]*BOHR,z[i]*BOHR);
        }
     }/*end if*/
     if (system_definition==2) {
       for (imol=1;imol<=nmol;imol++) {
	nres = nres_mol[imol];
        for (jres=1;jres<=nres;jres++) {
	    ires = ind_res_mol[imol][jres];
            for (iatm=1;iatm<=residue[ires].natom;iatm++) {
	 /*1-6*/fprintf(output_file,"ATOM  ");  
        /*7-11*/fprintf(output_file,"%5d",residue[ires].atom[iatm]);
	  /*12*/fprintf(output_file," ");
       /*13-16*/fprintf(output_file,"%4s",upac_atm_label[residue[ires].atom[iatm]]);
	  /*17*/fprintf(output_file," "); /*altLoc*/
       /*18-20*/fprintf(output_file,"%3s",residue[ires].residue_name);
          /*21*/fprintf(output_file," ");
	  /*22*/fprintf(output_file,"%1s",mol_chainid[imol]); /*chainID*/
       /*23-26*/fprintf(output_file,"%4d",ires);
       /*27-30*/fprintf(output_file,"    ");
       /*31-54*/fprintf(output_file,"%8.3f%8.3f%8.3f\n",
                                        x[residue[ires].atom[iatm]]*BOHR,
                                        y[residue[ires].atom[iatm]]*BOHR,
                                        z[residue[ires].atom[iatm]]*BOHR);
            }/*end for-iatm*/
	}/*end for-jres*/
       }/*end for-imol*/
     }/*end if*/
     fprintf(output_file,"TER\nENDMDL\n");
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     fprintf(output_file,">\n");
     for (i=1;i<=natom;i++) {
          fprintf(output_file,"%s",aa1_code[aa_type[permute_seq[i]]]);
     }
     fprintf(output_file,"\n");
  }/*if*/ 
  fclose(output_file);


  output_file = fopen(pos_out_filename, "w");
  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     if (system_definition==1) {
        for (i=1;i<=natom;i++) {
             fprintf(output_file,"ATOM   %4d  CA  %3s A%4d    %8.3f%8.3f%8.3f\n",
                         i,aa3_code[aa_type[i]],i,x[i]*BOHR,y[i]*BOHR,z[i]*BOHR);
        }/*end for*/
     }/*end if*/
     if (system_definition==2) {
       if (system.general.stride_defined==1) {
	   for (i=1;i<=system.tors_defs.nstride;i++) {
                fprintf(output_file,"STRIDE %s\n",
                                    &(system.tors_defs.stride[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.torcompl_defined==1) {
	   for (i=1;i<=system.tors_defs.ntorcompl;i++) {
                fprintf(output_file,"TORCOMPL %s\n",
                                    &(system.tors_defs.torcompl[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.closcompl_defined==1) {
	   for (i=1;i<=system.tors_defs.ncloscompl;i++) {
                fprintf(output_file,"CLOSCOMPL %s\n",
                                    &(system.tors_defs.closcompl[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.cblc_defined==1) {
                fprintf(output_file,"CBLC %s\n",
                                    &(system.tors_defs.cblc[0]));
       }/*end if*/
       for (imol=1;imol<=nmol;imol++) {
	nres = nres_mol[imol];
        for (jres=1;jres<=nres;jres++) {
	    ires = ind_res_mol[imol][jres];
            for (iatm=1;iatm<=residue[ires].natom;iatm++) {
	 /*1-6*/fprintf(output_file,"ATOM  ");  
        /*7-11*/fprintf(output_file,"%5d",residue[ires].atom[iatm]);
	  /*12*/fprintf(output_file," ");
       /*13-16*/fprintf(output_file,"%4s",upac_atm_label[residue[ires].atom[iatm]]);
	  /*17*/fprintf(output_file," "); /*altLoc*/
       /*18-20*/fprintf(output_file,"%3s",residue[ires].residue_name);
          /*21*/fprintf(output_file," ");
	  /*22*/fprintf(output_file,"%1s",mol_chainid[imol]); /*chainID*/
       /*23-26*/fprintf(output_file,"%4d",ires);
       /*27-30*/fprintf(output_file,"    ");
       /*31-54*/fprintf(output_file,"%8.3f%8.3f%8.3f\n",
                                        x[residue[ires].atom[iatm]]*BOHR,
                                        y[residue[ires].atom[iatm]]*BOHR,
                                        z[residue[ires].atom[iatm]]*BOHR);
            }/*end for-iatm*/
	}/*end for-jres*/
       }/*end for-imol*/
     }/*end if*/
     fprintf(output_file,"TER\nENDMDL\n");
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     fprintf(output_file,">\n");
     for (i=1;i<=natom;i++) {
          fprintf(output_file,"%s",aa1_code[aa_type[permute_seq[i]]]);
     }
     fprintf(output_file,"\n");
  }/*if*/
  fclose(output_file);

/*-------------------------------------------------------------------------*/
/* II) Torsional dihedral                                                  */
  output_file = fopen(tors_pos_filename, "a");
  if (output_count==1) {
      if (system.general.stride_defined==1) {
          for (i=1;i<=system.tors_defs.nstride;i++) {
               fprintf(output_file,"STRIDE %s\n",
                                   &(system.tors_defs.stride[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.torcompl_defined==1) {
	  for (i=1;i<=system.tors_defs.ntorcompl;i++) {
               fprintf(output_file,"TORCOMPL %s\n",
                                   &(system.tors_defs.torcompl[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.closcompl_defined==1) {
	  for (i=1;i<=system.tors_defs.ncloscompl;i++) {
               fprintf(output_file,"CLOSCOMPL %s\n",
                                   &(system.tors_defs.closcompl[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.cblc_defined==1) {
               fprintf(output_file,"CBLC %s\n",
                                   &(system.tors_defs.cblc[0]));
      }/*end if*/
  }/*end if*/
  fprintf(output_file,"MODEL %d ",output_count);
  if (tors_X != 0) {
     for (i=1;i<=ntors;i++) {
          fprintf(output_file,"%8.3f",tors_X[i]);
     }
  }/*end if*/
  fprintf(output_file,"\n");
  fclose(output_file);

/*-------------------------------------------------------------------------*/
/* III) Potential energy                                                   */

  output_file = fopen(epot_filename, "a");
  if (output_count==1) {
      if (write_energy_unit==1) {/*kcal/mol*/
	fprintf(output_file,"Potential Energy in kcal/mol\n");
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
	fprintf(output_file,"Potential Energy in Ha\n ");
      }
  }
  epot   = pos->epot;
  if (write_energy_unit==1) {/*kcal/mol*/
     fprintf(output_file,"%g ",epot*KCAL);
  }
  else if (write_energy_unit==2) {/*atomic unit: Ha*/
     fprintf(output_file,"%g ",epot);
  }
  fprintf(output_file,"\n");
  fclose(output_file);

/*-------------------------------------------------------------------------*/
/* Inter energy */

  output_file = fopen(einter_filename, "a");
  if (output_count==1) {
      if (write_energy_unit==1) {/*kcal/mol*/
	fprintf(output_file,"Potential Energy in kcal/mol\n");
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
	fprintf(output_file,"Potential Energy in Ha\n");
      }
  }

  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     einter = pos->einter;
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     einter = system.inter.getPotPermute(pos);
  }
  if (write_energy_unit==1) {/*kcal/mol*/
     fprintf(output_file,"%g ",einter*KCAL);
  }
  else if (write_energy_unit==2) {/*atomic unit: Ha*/
     fprintf(output_file,"%g ",einter);
  }
   
  fprintf(output_file,"\n");
  fclose(output_file);

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
/*=========================================================================*/
void statistics(Chain *X, System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
     
  int i;
  int K =  system.general.n_chains_mc;
  int action_type = system.general.action_type;
  int eemc_type   = system.general.eemc_type;

  // acceptance ratio
  print_doubleline();
  cout << " Acceptance ratio for MH Chains " <<endl;
  print_line();
  cout << "       " <<  "# of accepted   ||  # of total   || acceptance ratio" << endl; 
  for (i=0;i<=K;i++) {
      cout <<" Chain "<<i;
      if (X[i].nprop == 0) { 
        cout <<" Not initialized yet => Increse # of steps " << endl;
      } else {
        cout <<"     "<< X[i].nacc <<'\t'; 
        cout <<"     "<<  X[i].nprop <<'\t'; 
        cout <<"     "<< (double)X[i].nacc/(double)X[i].nprop << endl;      
      }
  }
 
  if ( (action_type==1)||(action_type==3) ||
       ((action_type==2)&&(eemc_type==2)) ) {/* PT */
   cout << endl << endl;
   cout << " Acceptance ratio for PT jumps " <<endl<<endl;
   cout << "       " <<  "# of accepted   ||  # of total   || acceptance ratio" << endl; 
   for (i=0;i<=K-1;i++) {
       cout <<" Chain "<<i; 
       if (X[i].nptprop == 0) { 
         cout <<" Not initialized yet  " << endl;
       } else {
         cout <<"     "<< X[i].nptacc <<'\t'; 
         cout <<"     "<<  X[i].nptprop <<'\t'; 
         cout <<"     "<< (double)X[i].nptacc/(double)X[i].nptprop << endl;      
      }
   }
  } 
  if ((action_type==2)||(action_type==4)) {/* EEMC */
   cout << endl << endl;
   cout << " Acceptance ratio for EE jumps " <<endl<<endl;
   cout << "       " <<  "# of accepted   ||  # of total   || acceptance ratio" << endl; 
   for (i=0;i<=K-1;i++) {
       cout <<" Chain "<<i; 
       if (X[i].neeprop == 0) { 
         cout <<" Not initialized yet  " << endl;
       } else {
         cout <<"     "<< X[i].neeacc <<'\t'; 
         cout <<"     "<<  X[i].neeprop <<'\t'; 
         cout <<"     "<< (double)X[i].neeacc/(double)X[i].neeprop << endl;      
      }
   }
  }
  print_line();
  cout << " Acceptance ratio for MH Chains " <<endl;
  print_doubleline();
  LINE
  LINE


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Pos::write()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

     
  int iatm;
  for (iatm=1;iatm<=natom;iatm++) {
    cout <<iatm<<" "<<"x "<<"y "<<"z "<<"   "; 
    cout <<P16<<x[iatm]<<" "<<P16<<y[iatm]<<" "<<P16<<z[iatm]<<endl;
    cout <<iatm<<" "<<"ux "<<"uy "<<"uz "<<"   "; 
    cout <<P16<<ux[iatm]<<" "<<P16<<uy[iatm]<<" "<<P16<<uz[iatm]<<endl;
    cout <<iatm<<" "<<"xi "<<"yi "<<"zi "<<"   "; 
    cout <<P16<<xi[iatm]<<" "<<P16<<yi[iatm]<<" "<<P16<<zi[iatm]<<endl;
    cout <<iatm<<" "<<"vx "<<"vy "<<"vz "<<"   "; 
    cout <<P16<<vx[iatm]<<" "<<P16<<vy[iatm]<<" "<<P16<<vz[iatm]<<endl;
    cout <<iatm<<" "<<"fx "<<"fy "<<"fz "<<" "; 
    cout <<P16<<fx[iatm]<<" "<<P16<<fy[iatm]<<" "<<P16<<fz[iatm]<<endl;
    cout <<iatm<<" "<<"fux "<<"fuy "<<"fuz "<<" "; 
    cout <<P16<<fux[iatm]<<" "<<P16<<fuy[iatm]<<" "<<P16<<fuz[iatm]<<endl;
  }


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
/*=========================================================================*/
void Pos::write_tors_defs(Tors_defs &tors_defs, int level)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iatm, tatm;

  System *system = this->system_ptr;                                       
  Maps *maps = &(system->maps);

  int **ind_trans     = tors_defs.ind_trans;
  int **ind_prim_tors = tors_defs.ind_prim_tors;
  int *natm_trans     = tors_defs.natm_trans;  
  int *res_id         = tors_defs.res_id;   


  printf("----------group----------\n");
  printf("level->  %d\n",level);
  printf("residue id->  %d\n",res_id[level]);
  printf("residue label-> %s\n",maps->res_label[res_id[level]]);
  printf("natm_trans-> : %d\n",natm_trans[level]);
  printf("atm_ind : atm_ind_glob : atom_label : upac_atom_label\n");      
  for (iatm=1;iatm<=natm_trans[level];iatm++) {
       printf(" %d : %d :",iatm,ind_trans[level][iatm]);
       printf(" %s :",maps->atm_label[ind_trans[level][iatm]]);
       printf(" %s \n",maps->upac_atm_label[ind_trans[level][iatm]]);
  }/*end for*/

  for (tatm=1;tatm<=natm_trans[level];tatm++) {
    iatm = ind_trans[level][tatm];
    cout <<iatm<<" "<<"xi "<<"yi "<<"zi "<<"   "; 
    cout <<P16<<xi[iatm]<<" "<<P16<<yi[iatm]<<" "<<P16<<zi[iatm]<<endl;
    cout <<iatm<<" "<<"fx "<<"fy "<<"fz "<<" "; 
    cout <<P16<<fx[iatm]<<" "<<P16<<fy[iatm]<<" "<<P16<<fz[iatm]<<endl;
  }

  printf("----------ind_prim_tors----------\n");
  printf(" iatm1:iatm2:iatm3:iatm4 |");
  printf(" latm1:latm2:latm3:latm4 |");
  printf(" uatm1:uatm2:uatm3:uatm4 \n");

char();

  printf("%d : %d : %d : %d | ",ind_prim_tors[level][1],ind_prim_tors[level][2],
                                ind_prim_tors[level][3],ind_prim_tors[level][4]);

char();

  printf("%s : %s : %s : %s | ",maps->atm_label[ind_prim_tors[level][1]], 
                               maps->atm_label[ind_prim_tors[level][2]],
                               maps->atm_label[ind_prim_tors[level][3]],
                               maps->atm_label[ind_prim_tors[level][4]]);
char();
  printf("%s : %s : %s : %s\n",maps->upac_atm_label[ind_prim_tors[level][1]], 
                                  maps->upac_atm_label[ind_prim_tors[level][2]],
                                  maps->upac_atm_label[ind_prim_tors[level][3]],
                                  maps->upac_atm_label[ind_prim_tors[level][4]]);


  for (tatm=1;tatm<=4;tatm++) {
    iatm = ind_prim_tors[level][tatm];
    cout <<iatm<<" "<<"xi "<<"yi "<<"zi "<<"   "; 
    cout <<P16<<xi[iatm]<<" "<<P16<<yi[iatm]<<" "<<P16<<zi[iatm]<<endl;
    cout <<iatm<<" "<<"fx "<<"fy "<<"fz "<<" "; 
    cout <<P16<<fx[iatm]<<" "<<P16<<fy[iatm]<<" "<<P16<<fz[iatm]<<endl;
  }
  
  cout << endl;



/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
