#include "../include/standard_mpi.h" 
#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/typedefs_mpi.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

/*=========================================================================*/
/*=========================================================================*/
void output_mpi(Chain *X, System &system, Commun commun)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, nres;
  int imol, ires, jres, iatm;
  double einter, epot, econv;

/*-------------------------------------------------------------------------*/
/* 0) get commun variables                                                 */
   
  MPI_Comm world = commun.world;
  int myid       = commun.myid;
  int numprocs   = commun.numprocs;

/*-------------------------------------------------------------------------*/
/* I) get system variables                                                 */

  int system_definition    = system.general.system_definition;
  int K                    = system.general.n_chains_mc;
  int natom                = system.pos.natom;
  int ntors                = system.tors_defs.num_trans;
  int write_energy_unit    = system.general.write_energy_unit;
  int action_type          = system.general.action_type;
  int *aa_type             = system.maps.aa_type;
  char **aa3_code          = system.maps.aa3_code;
  char **aa1_code          = system.maps.aa1_code;
  int  *order_X            = system.maps.order_X;
  char *atom_pos_filename  = system.filenames.atom_pos_filename;
  char *tors_pos_filename  = system.filenames.tors_pos_filename;
  char *pos_out_filename   = system.filenames.pos_out_filename;
  char *epot_filename      = system.filenames.epot_filename;
  char *einter_filename    = system.filenames.einter_filename;

/*-------------------------------------------------------------------------*/
/* II) get position related variables                                      */

  double *tors_X           = X[order_X[0]].pos.tors_X;
  double *x                = X[order_X[0]].pos.x;
  double *y                = X[order_X[0]].pos.y;
  double *z                = X[order_X[0]].pos.z; 
  int *permute_seq         = X[order_X[0]].pos.permute_seq;
  int static output_count  = 0;

/*-------------------------------------------------------------------------*/
/* III) get molecular/residue related variables                            */

  int    nmol              = system.maps.nmol;
  int   *nres_mol          = system.maps.nres_mol;
  int  **ind_res_mol       = system.maps.ind_res_mol;
  char **mol_label         = system.maps.mol_label;
  char **mol_chainid       = system.maps.mol_chainid;

  char **upac_atm_label    = system.maps.upac_atm_label;
  Residue *residue         = system.pos.residue;
  FILE *output_file;

  output_count++;

/*-------------------------------------------------------------------------*/
/* IV) Coordinates and trajectory                                          */
	
if (myid==order_X[0]) {
  output_file = fopen(atom_pos_filename, "a");
  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     fprintf(output_file,"MODEL %d\n",output_count);
     if (system_definition==1) {
        for (i=1;i<=natom;i++) {
            fprintf(output_file,"ATOM   %4d  CA  %3s A%4d    %8.3f%8.3f%8.3f\n",
                   i,aa3_code[aa_type[i]],i,x[i]*BOHR,y[i]*BOHR,z[i]*BOHR);
        }
     }/*end if*/
     if (system_definition==2) {
       for (imol=1;imol<=nmol;imol++) {
	nres = nres_mol[imol];
        for (jres=1;jres<=nres;jres++) {
	    ires = ind_res_mol[imol][jres];
            for (iatm=1;iatm<=residue[ires].natom;iatm++) {
	 /*1-6*/fprintf(output_file,"ATOM  ");  
        /*7-11*/fprintf(output_file,"%5d",residue[ires].atom[iatm]);
	  /*12*/fprintf(output_file," ");
       /*13-16*/fprintf(output_file,"%4s",upac_atm_label[residue[ires].atom[iatm]]);
	  /*17*/fprintf(output_file," "); /*altLoc*/
       /*18-20*/fprintf(output_file,"%3s",residue[ires].residue_name);
          /*21*/fprintf(output_file," ");
	  /*22*/fprintf(output_file,"%1s",mol_chainid[imol]); /*chainID*/
       /*23-26*/fprintf(output_file,"%4d",ires);
       /*27-30*/fprintf(output_file,"    ");
       /*31-54*/fprintf(output_file,"%8.3f%8.3f%8.3f\n",
                                        x[residue[ires].atom[iatm]]*BOHR,
                                        y[residue[ires].atom[iatm]]*BOHR,
                                        z[residue[ires].atom[iatm]]*BOHR);
            }/*end for-iatm*/
	}/*end for-jres*/
       }/*end for-imol*/
     }/*end if*/
     fprintf(output_file,"TER\nENDMDL\n");
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     fprintf(output_file,">\n");
     for (i=1;i<=natom;i++) {
          fprintf(output_file,"%s",aa1_code[aa_type[permute_seq[i]]]);
     }
     fprintf(output_file,"\n");
  }/*if*/ 
  fclose(output_file);

  output_file = fopen(pos_out_filename, "w");
  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     if (system_definition==1) {
        for (i=1;i<=natom;i++) {
             fprintf(output_file,"ATOM   %4d  CA  %3s A%4d    %8.3f%8.3f%8.3f\n",
                         i,aa3_code[aa_type[i]],i,x[i]*BOHR,y[i]*BOHR,z[i]*BOHR);
        }/*end for*/
     }/*end if*/
     if (system_definition==2) {
       if (system.general.stride_defined==1) {
           for (i=1;i<=system.tors_defs.nstride;i++) {
                fprintf(output_file,"STRIDE %s\n",
                                    &(system.tors_defs.stride[i][0]));
           }/*end for*/           
       }/*end if*/
       if (system.general.torcompl_defined==1) {
	   for (i=1;i<=system.tors_defs.ntorcompl;i++) {
                fprintf(output_file,"TORCOMPL %s\n",
                                    &(system.tors_defs.torcompl[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.closcompl_defined==1) {
	   for (i=1;i<=system.tors_defs.ncloscompl;i++) {
                fprintf(output_file,"CLOSCOMPL %s\n",
                                    &(system.tors_defs.closcompl[i][0]));
           }/*end for*/
       }/*end if*/
       if (system.general.cblc_defined==1) {
                fprintf(output_file,"CBLC %s\n",
                                   &(system.tors_defs.cblc[0]));
       }/*end if*/
       for (imol=1;imol<=nmol;imol++) {
	nres = nres_mol[imol];
        for (jres=1;jres<=nres;jres++) {
	    ires = ind_res_mol[imol][jres];
            for (iatm=1;iatm<=residue[ires].natom;iatm++) {
	 /*1-6*/fprintf(output_file,"ATOM  ");  
        /*7-11*/fprintf(output_file,"%5d",residue[ires].atom[iatm]);
	  /*12*/fprintf(output_file," ");
       /*13-16*/fprintf(output_file,"%4s",upac_atm_label[residue[ires].atom[iatm]]);
	  /*17*/fprintf(output_file," "); /*altLoc*/
       /*18-20*/fprintf(output_file,"%3s",residue[ires].residue_name);
          /*21*/fprintf(output_file," ");
	  /*22*/fprintf(output_file,"%1s",mol_chainid[imol]); /*chainID*/
       /*23-26*/fprintf(output_file,"%4d",ires);
       /*27-30*/fprintf(output_file,"    ");
       /*31-54*/fprintf(output_file,"%8.3f%8.3f%8.3f\n",
                                        x[residue[ires].atom[iatm]]*BOHR,
                                        y[residue[ires].atom[iatm]]*BOHR,
                                        z[residue[ires].atom[iatm]]*BOHR);
            }/*end for-iatm*/
	}/*end for-jres*/
       }/*end for-imol*/
     }/*end if*/
     fprintf(output_file,"TER\nENDMDL\n");
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     fprintf(output_file,">\n");
     for (i=1;i<=natom;i++) {
          fprintf(output_file,"%s",aa1_code[aa_type[permute_seq[i]]]);
     }
     fprintf(output_file,"\n");
  }/*if*/
  fclose(output_file);
}/*endif-myid==order_X[0]-*/

/*-------------------------------------------------------------------------*/
/* V) Torsional dihedral                                                   */

if (myid==order_X[0]) {
  output_file = fopen(tors_pos_filename, "a");
  if (output_count==1) {
      if (system.general.stride_defined==1) {
        for (i=1;i<=system.tors_defs.nstride;i++) {
             fprintf(output_file,"STRIDE %s\n",
                                 &(system.tors_defs.stride[i][0]));
        }/*end for*/
      }/*end if*/
      if (system.general.torcompl_defined==1) {
	  for (i=1;i<=system.tors_defs.ntorcompl;i++) {
               fprintf(output_file,"TORCOMPL %s\n",
                                   &(system.tors_defs.torcompl[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.closcompl_defined==1) {
	  for (i=1;i<=system.tors_defs.ncloscompl;i++) {
               fprintf(output_file,"CLOSCOMPL %s\n",
                                   &(system.tors_defs.closcompl[i][0]));
          }/*end for*/
      }/*end if*/
      if (system.general.cblc_defined==1) {
                fprintf(output_file,"CBLC %s\n",
                                   &(system.tors_defs.cblc[0]));
      }/*end if*/
  }/*end if*/
  fprintf(output_file,"MODEL %d ",output_count);
  if (tors_X != 0) {
      for (i=1;i<=ntors;i++) {
           fprintf(output_file,"%8.3f",tors_X[i]);
      }/*endfor*/
  }/*endif*/
  fprintf(output_file,"\n");
  fclose(output_file);
}/*endif-myid==order_X[0]-*/

/*-------------------------------------------------------------------------*/
/* VI) Potential energy                                                    */

if (myid==order_X[0]) {

    X[order_X[0]].pos.report_energy(system); /*report energy on screen*/

}/*endif-myid==order_X[0]-*/

if (myid==0) {

  output_file = fopen(epot_filename, "a");
  if (output_count==1) {
      if (write_energy_unit==1) {/*kcal/mol*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in kcal/mol\n",K+1);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in Ha\n ",K+1);
      }
  }
  for (i=0;i<=K;i++) {
      epot   = X[order_X[i]].pos.epot;
      if (write_energy_unit==1) {/*kcal/mol*/
         fprintf(output_file,"%g ",epot*KCAL);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
         fprintf(output_file,"%g ",epot);
      }
  }
  fprintf(output_file,"\n");
  fclose(output_file);
}/*endif-myid==0-*/

/*-------------------------------------------------------------------------*/
/* VII) Inter energy                                                       */

if (myid==0) {
  output_file = fopen(einter_filename, "a");
  if (output_count==1) {
      if (write_energy_unit==1) {/*kcal/mol*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in kcal/mol\n",K+1);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
	fprintf(output_file,"Potential Energy for %d Replica(s) in Ha\n",K+1);
      }
  }
  for (i=0;i<=K;i++) {
      if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
         einter = X[order_X[i]].pos.einter;
      }
      else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
         einter = system.inter.getPotPermute(&(X[order_X[i]].pos));
      }
      if (write_energy_unit==1) {/*kcal/mol*/
         fprintf(output_file,"%g ",einter*KCAL);
      }
      else if (write_energy_unit==2) {/*atomic unit: Ha*/
         fprintf(output_file,"%g ",einter);
      }
  }    
  fprintf(output_file,"\n");
  fclose(output_file);
}/*endif-myid==0-*/

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/

