#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#ifdef DEBUG
  printf(" Check point IV %d\n",index);
  printf(" chain %d\n",index);
  printf(" pacc %g\n",pacc);
  printf(" Hp %g H %g\n",Hp,H);
  getchar();
#endif

#define DEBUG
#undef DEBUG
#define DEBUG_CLOS_OFF
#define DEBUG_ACC_OFF
#define DEBUG_TIMESTEP_OFF

/*=========================================================================*/
void Chain::setup(int indexg, System &system) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

    int i; 
    long *seed  = &(system.general.seed); 

    index       = indexg;    
    order       = index;

    T           = system.energy_part.T[index];
    nstat       = system.general.nstat;    
    action_type = system.general.action_type;

    pos.setup(system,T);

    nprop = 0;
    nacc  = 0;
    neeprop = 0;
    neeacc  = 0;
    ntprop  = 0;
    nptprop = 0;
    nptacc  = 0;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Chain::setup_scalar(int indexg, System &system) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
/* Only sets up/initializes some of the the scalar variables               */ 
/* associated with Chain and Chain.Pos but no coordinate allocation        */
/* for Class Pos is perfomed                                               */
/* The routine is mainly used to support the MPI version                   */
/*-------------------------------------------------------------------------*/

    int i; 
    long *seed  = &(system.general.seed); 

    index       = indexg;    
    order       = index;

    T           = system.energy_part.T[index];
    nstat       = system.general.nstat;    
    action_type = system.general.action_type;

    nprop = 0;
    nacc  = 0;
    neeprop = 0;
    neeacc  = 0;
    ntprop  = 0;
    nptprop = 0;
    nptacc  = 0;


    pos.T      = T;
    pos.ebond  = 0.0;
    pos.ebend  = 0.0;
    pos.etors  = 0.0;
    pos.eonfo  = 0.0;
    pos.einter = 0.0;
    pos.epot   = 0.0;       /* energies for x y z */
 
    pos.action_type = action_type;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Chain::prop(System &system) 
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

    double H,Hp,pacc;

    int real        =   1;
    int propose     =   0; 
    
    int action_type   = system.general.action_type;
    int prop_type     = system.general.prop_type;
    int clos_defined  = system.general.clos_defined;
    int clos_succeed  = pos.clos_succeed;

    long  *seed     = &(system.general.seed);

    double acc_rate_cum, acc_rate_targ;
    double *prop_tors_sig = &(system.general.prop_tors_sig);
    int *postprop_succeed = &(system.maps.postprop_succeed);

    char   warn_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Make it compatible with MPI                                          */

    if (order==index)                          /* compatibility with mpi */
       {nprop++; ntprop++;}


/*-------------------------------------------------------------------------*/
/* I) If needed sample velocities                                          */
 
    if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
         if (prop_type==1) {/*cartesian propagation, HMC*/ 
             pos.vel_sampl(system);
         }/*endif*/
    }/*endif*/    


#ifdef DEBUG
  printf(" Pos before move");
  pos.write();
  getchar();
#endif

/*-------------------------------------------------------------------------*/
/* II) Move system and evaluate acceptance                                 */


    H = pos.energy(system,real);

    pos.init_propose(system);

    *postprop_succeed = 1;
    if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
         if (prop_type==1) {/*cartesian propagation, HMC*/ 
	     pos.force_save();
             pos.move_md(system);
             Hp = pos.epotp + pos.kinetic();
         }
         else if (prop_type==2) {/*torsional propagation*/
             pos.move_tors(system);
             if (clos_defined==1) {/*closure*/
                 if ((pos.clos_succeed) == 1) {
		     pos.postprop(system);
                     if ((*postprop_succeed)==1) 
		        {Hp = pos.energy(system,propose);}
                 }/*endif*/
             } else {/*no closure*/
	         pos.postprop(system);  
                 if ((*postprop_succeed)==1)
		    {Hp = pos.energy(system,propose);}
             }/*endif*/
         }/*endif*/
    }/*endif*/

    pacc = MIN(1.0,exp(-(Hp-H)/T));


/*-------------------------------------------------------------------------*/
/* III) Rejecting state if closure or post propagation conditions are not  */
/*      satisfied properly                                                 */


    if ( (prop_type   ==2) && 
         (clos_defined==1) &&
         ((pos.clos_succeed)==0) ) {/*torsional closure failure*/
#ifdef DEBUG_CLOS
          sprintf(warn_msg,"Closure is not healed => rejecting state");
          warnMessage(warn_msg);
#endif
          pacc = 0;
    }/*endif*/        

    if (  (*postprop_succeed)== 0 )   {pacc = 0;}        

/*-------------------------------------------------------------------------*/
/* IV) Evaluate aceptance                                                  */

    acc = 0;
    if (pacc > ran1(seed)) {
        acc = 1;
        pos.acc_propose();
        if (order==index)                       /* compatibility with mpi */
           {nacc++;} 
    } else {
        if (prop_type==1) {pos.force_restore();}
    }/*endif*/

/*-------------------------------------------------------------------------*/
/* V) If adaptive stepsize is required                                     */
/*      Currently with (general.clos_defined == 1) only                    */
/*      Only perform it at every nstat step                                */

#ifdef DEBUG_TIMESTEP    
    acc_rate_targ = 0.3;
    if ( (prop_type   ==2) && (clos_defined==1) ) {/*torsional closure*/
       if ( (ntprop % nstat) == 0 ) {
             acc_rate_cum = ((double)nacc)/((double)nprop);
             if (acc_rate_cum < acc_rate_targ) {(*prop_tors_sig) /= 2.0;}
             if (acc_rate_cum > acc_rate_targ) {(*prop_tors_sig) *= 2.0;}
       }/*endif*/
    }/*endif*/
#endif

/*-------------------------------------------------------------------------*/
/* VI) DEBUG                                                               */


#ifdef DEBUG_ACC
  printf("pos.clos_succeed %d\n",pos.clos_succeed);
  printf(" pacc %g acc %d\n",pacc,acc);
  printf(" Hp %g H %g\n",Hp,H);
  printf(" delH %g\n",Hp-H);
  getchar();
#endif


#ifdef DEBUG
  printf(" Pos after move");
  pos.write();
  getchar();
#endif


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Chain::get_I(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

     int i;
     int real = 1;
     double V  = pos.potential(system,real);
     double *H = system.energy_part.H_ladder;
     int K     = system.general.n_chains_mc;  

     I = K;
     for (i=0;i<=K;i++) {
         if ((V >= H[i]) && (V < H[i+1])) {
            I = i;
         }
     }

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Chain::ee_jump(double pacc, Coord y, long* seed)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
     
     neeprop++; ntprop++;
     eeacc = 0;
     if (pacc > ran1(seed)) {
        eeacc = 1;
        neeacc++;
        pos = y;
     }               

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
