#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"

#define DEBUG_OFF

/*=========================================================================*/
void Pos::move_md(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int  nstep_md   =   system.general.nstep_md;
  
  move_md(system,nstep_md);


/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::move_md(System &system, int nstep_md)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int  i,j;
  int irespa, nrespa;

  double dt, dt2;
  double *xs,*ys,*zs;
  double *fxs,*fys,*fzs;

  int  stage      =   system.general.stage;

  dt      =   system.general.dt;
  nrespa = system.general.intra_respa_step;  


/*-------------------------------------------------------------------------*/
/* 0) Initialize U space                                                   */


  if (stage == 1) {
      stage_transformXFXtoUFU();
      xs = xi; ys = yi; zs = zi;
      xi = ux; yi = uy; zi = uz; 
      fxs = fx; fys = fy; fzs = fz;
      fx = fux; fy = fuy; fz = fuz; 
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Move the system with intra RESPA                                     */

  dt  /= ((double)(nrespa));
  dt2 = dt/2.0;

  for (i=1;i<=nstep_md;i++) {

      
    if (stage == 1) {
        xi = xs;  yi = ys;  zi = zs;
        fx = fxs; fy = fys; fz = fzs;
        stage_transformUtoX();
        respa_intra(system);
        respa_inter(system);
    }/*endif*/

    if (stage == 1) {
       stage_transformXFXtoUFU();
       xs = xi; ys = yi; zs = zi;
       xi = ux; yi = uy; zi = uz; 
       fxs = fx; fys = fy; fzs = fz;
       fx = fux; fy = fuy; fz = fuz; 
    }/*endif*/

    for (irespa=1;irespa<=nrespa;irespa++) {

         for (j=1;j<=natom;j++) {
             vx[j] = vx[j] + fx[j]/mass[j]*dt2;
             vy[j] = vy[j] + fy[j]/mass[j]*dt2;
             vz[j] = vz[j] + fz[j]/mass[j]*dt2;
      
             xi[j] = xi[j] + vx[j]*dt;
             yi[j] = yi[j] + vy[j]*dt;
             zi[j] = zi[j] + vz[j]*dt;
         }/*endfor*/
      
         if (stage == 1) {
             xi = xs;  yi = ys;  zi = zs;
             fx = fxs; fy = fys; fz = fzs;
             stage_transformUtoX();
         }/*endif*/

         respa_intra(system);
         if (irespa==nrespa) 
            {respa_inter(system);}
      
         if (stage == 1) {
             stage_transformXFXtoUFU();
             xs = xi; ys = yi; zs = zi;
             xi = ux; yi = uy; zi = uz; 
             fxs = fx; fys = fy; fzs = fz;
             fx = fux; fy = fuy; fz = fuz; 
         }/*endif*/
      
         for (j=1;j<=natom;j++) {
             vx[j] = vx[j] + fx[j]/mass[j]*dt2;
             vy[j] = vy[j] + fy[j]/mass[j]*dt2;
             vz[j] = vz[j] + fz[j]/mass[j]*dt2;
         }/*endfor*/

    }/*irespa*/

#ifdef DEBUG
    FILE *output_file;
    char* epot_filename = system.filenames.epot_filename;
    output_file = fopen(epot_filename,"a");
    fprintf(output_file,"energy %.16g\n",epotp+kinetic());
    fclose(output_file);
#endif

  }/*endfor*/

#ifdef DEBUG
  printf("Completed MD\n");
  getchar();
#endif


/*-------------------------------------------------------------------------*/
/* II) Restore pointers                                                    */

  if (stage == 1) {
      xi = xs;  yi = ys;  zi = zs;
      fx = fxs; fy = fys; fz = fzs;
  }/*endif*/

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::init_propose(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/
  
   int j;
   int index1, index2, index1_found, index2_found;
   int  control_design = system.general.control_design; 
   int *design_field   = system.maps.design_field;
   long *seed = &(system.general.seed);
   int natom_design;
   int control_design_type;     
   int datm_first;
   int datm_last;
   int cdatm_first;
   int cdatm_last;

   if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
      for (j=1;j<=natom;j++) {
          xi[j] = x[j];
          yi[j] = y[j];
          zi[j] = z[j];
      }
   }
   else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/

      control_design_type = design_field[1];     
      datm_first          = design_field[2];
      datm_last           = design_field[3];
      cdatm_first         = design_field[4];
      cdatm_last          = design_field[5];
      if (control_design==1) {
         
         natom_design = datm_last - datm_first + 1;

	 /*  index1       = (int)((natom_design-1)*ran1(seed)) + 1; */
         /*  index2       = (int)((natom_design-1)*ran1(seed)) + 1; */
   
         /* !! should use some uniform integer random generator */
         /* temporary solution */
         index1_found = 0;
         index2_found = 0;
         while ( (index1_found==0) || (index2_found==0) ) {
	       index1_found = 0;
               index2_found = 0;
               index1       = (int)((natom_design)*ran1(seed)) + 1;
               index2       = (int)((natom_design)*ran1(seed)) + 1;
               if (index1 <= natom_design) {index1_found = 1;}
               if (index2 <= natom_design) {index2_found = 1;}
	 }
         /* temporary solution */

         index1       = datm_first - 1 + index1;
         index2       = datm_first - 1 + index2;               

         for (j=1;j<=natom;j++) {

             if (j==index1) {
                permute_seq_prop[j] = permute_seq[index2];
             }
             else if (j==index2) {
                permute_seq_prop[j] = permute_seq[index1];
             }
             else {
	        permute_seq_prop[j] = permute_seq[j];
             }
         }

         if (control_design_type==1) {/*complementary design*/

	    index1 = cdatm_last - (index1 - datm_first);
            index2 = cdatm_last - (index2 - datm_first);   
            for (j=cdatm_first;j<=cdatm_last;j++) {

                if (j==index1) {
                   permute_seq_prop[j] = permute_seq[index2];
                }
                else if (j==index2) {
                    permute_seq_prop[j] = permute_seq[index1];
                }
                else {
	            permute_seq_prop[j] = permute_seq[j];
                }

            }
         }/*end if*/

      }
      else if (control_design==0) {
         index1 = (int)((natom-1)*ran1(seed)) + 1;
         index2 = (int)((natom-1)*ran1(seed)) + 1;

         for (j=1;j<=natom;j++) {
             if (j==index1) {
                permute_seq_prop[j] = permute_seq[index2];
             }
             else if (j==index2) {
                permute_seq_prop[j] = permute_seq[index1];
             }
             else {
	        permute_seq_prop[j] = permute_seq[j];
             }
         }

      }/*end if control_design*/ 

   }/*end if action_type*/

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::acc_propose()
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int j;

  if ((action_type==1)||(action_type==2)) {/*conformation sampling*/
     for (j=1;j<=natom;j++) {
         x[j] = xi[j];
         y[j] = yi[j];
         z[j] = zi[j];
     }
     ebond    = ebondp;
     ebend    = ebendp;
     etors    = etorsp;
     eonfo    = eonfop;
     einter   = einterp;
     ecryo_em = ecryo_emp;
     econstraint = econstraintp;
     epot     = epotp;

     ebend_bond  =  ebendp_bond;
     ebend_bend  =  ebendp_bend;
     etors_tors  =  etorsp_tors;
     etors_imp   =  etorsp_imp;
     eonfo_lj    =  eonfop_lj;   
     eonfo_coul  =  eonfop_coul;
     einter_lj   =  einterp_lj;  
     einter_coul =  einterp_coul;
     einter_scp  =  einterp_scp;
     einter_kb   =  einterp_kb; 
  }
  else if ((action_type==3)||(action_type==4)) {/*sequence sampling*/
     for (j=1;j<=natom;j++) {
         permute_seq[j] = permute_seq_prop[j];
     }   
  }

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::force_save()
/*-------------------------------------------------------------------------*/
{/*begin routine */
/*-------------------------------------------------------------------------*/
     int j;
     for (j=1;j<=natom;j++) {
          fxsa[j] = fx[j];
          fysa[j] = fy[j];
          fzsa[j] = fz[j];
     }/*endfor*/ 
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::force_restore()
/*-------------------------------------------------------------------------*/
{/*begin routine */
/*-------------------------------------------------------------------------*/
     int j;
     for (j=1;j<=natom;j++) {
          fx[j] = fxsa[j];
          fy[j] = fysa[j];
          fz[j] = fzsa[j];
     }/*endfor*/ 
/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
