#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"

#define DEBUG_OFF

/*=========================================================================*/
void Pos::move_tomin(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, iter, n;
  double max_grad, av_grad;
  double fret, ftol;
  double *xt,*yt,*zt;

/*-------------------------------------------------------------------------*/
/* I) perform minimization                                                 */  
/*    minimization is performed in "system.pos.xyz", thus results should   */
/*    be further processed/migrated into current Pos class                 */
/*    effected system.pos  vectors:                                        */
/*                      system.pos.$  $={x,y,z}                            */
/*                      system.pos.f$ $={x,y,z}                            */ 
/*    effected system.pos pointers: xyz, dxyz                              */
/*                                                                         */
/*                                                                         */

  ftol = system.general.minimize_tol;

  if (system.general.prop_type==1) {/*cartesian propagation*/
      n = 3*natom;
      update_xyz();
      if (system.general.minimize_type==1)
          frprmn(xyz,n,ftol,&iter,&fret,potential_cart,dpotential_cart);
      if (system.general.minimize_type==2)
          dfpmin(xyz,n,ftol,&iter,&fret,potential_cart,dpotential_cart);
  }/*cartesian propagation*/
 
  if (system.general.prop_type==2) {/*torsional propagation*/
      n = system.tors_defs.num_trans;
      init_propose(system);
      force(system);
      tors_transformXFXtoTFT();
      system.pos.tors_X = t_scr;
      system.pos.t      = t_scr;
      system.pos.init_propose(system);
      if (system.general.minimize_type==1)
          frprmn(t,n,ftol,&iter,&fret,potential_tors,dpotential_tors);
      if (system.general.minimize_type==2)
          dfpmin(t,n,ftol,&iter,&fret,potential_tors,dpotential_tors);
  }/*torsional propagation*/

  if (system.general.prop_type==3) {/*bend propagation*/
      n = system.tors_defs.num_tbend;
      init_propose(system);
      force(system);
      tors_transformXFXtoBFB();
      system.pos.tors_X = t_scr;
      system.pos.b      = b_scr;
      system.pos.init_propose(system);
      if (system.general.minimize_type==1)
          frprmn(b,n,ftol,&iter,&fret,potential_bend,dpotential_bend);
      if (system.general.minimize_type==2)
          dfpmin(b,n,ftol,&iter,&fret,potential_bend,dpotential_bend);
  }/*bend propagation*/

/*-------------------------------------------------------------------------*/
/* II) migrate positions and forces to current Pos class and restore       */
/*     system.pos to its original state                                    */

  if (system.general.prop_type==1) {/*cartesian propagation*/
      xt = x; yt =y; zt = z;
     
      x  = system.pos.x;
      y  = system.pos.y;
      z  = system.pos.z;
      force(system);
      update_xyz();
      update_fxyz();
      
      system.pos.x = xt;
      system.pos.y = yt;
      system.pos.z = zt;
      system.pos.force(system);
      system.pos.xyz  = 0;
      system.pos.dxyz = 0;
  }/*cartesian propagation*/

  if (system.general.prop_type==2) {/*torsional propagation*/
      /* here only *t is changed, and */
      /* xi,yi,zi,  x,y,z is adjusted */
      /* accordingly                  */
      tors_transformTtoX();      
      acc_propose();
      force(system); 
      system.pos.init_propose(system); 
  }/*torsional propagation*/

  if (system.general.prop_type==3) {/*bend propagation*/
      /* here only *b is changed, and */
      /* xi,yi,zi,  x,y,z is adjusted */
      /* accordingly                  */
      tors_transformBtoX();      
      acc_propose();
      force(system); 
      system.pos.init_propose(system);      
  }/*bend propagation*/

/*-------------------------------------------------------------------------*/
/* III) test and debug minimization protocol                               */

#ifdef DEBUG

  if (system.general.prop_type==1) {/*cartesian propagation*/

     xt = x; yt =y; zt = z;
     
     x  = system.pos.x;
     y  = system.pos.y;
     z  = system.pos.z;
     force(system);
     update_xyz();
     
     system.pos.x = xt;
     system.pos.y = yt;
     system.pos.z = zt;
     system.pos.force(system);
     
     //____________________________________________________________output
     printf("before frprmn\n");
     force(system);
     update_fxyz();
     max_grad = 0.0;
     av_grad = 0.0;
     for (i=1;i<=3*natom;i++) {
       max_grad = MAX(max_grad,fabs(fxyz[i]));
       av_grad += fabs(fxyz[i]);
     }
     av_grad /= (3*natom);
     printf("max_gradient: %.16g,  average_gradient: %.16g\n",max_grad,av_grad);     getchar();
     //____________________________________________________________output
     
     frprmn(xyz,n,ftol,&iter,&fret,potential_cart,dpotential_cart);     
     
     xt = x; yt =y; zt = z;
     
     x  = system.pos.x;
     y  = system.pos.y;
     z  = system.pos.z;
     force(system);
     update_xyz();
     update_fxyz();
     
     system.pos.x = xt;
     system.pos.y = yt;
     system.pos.z = zt;
     system.pos.force(system);
     
     //____________________________________________________________output 
     printf("after frprmn\n");
     force(system);
     update_fxyz();
     max_grad = 0.0;
     av_grad = 0.0;
     for (i=1;i<=3*natom;i++) {
       max_grad = MAX(max_grad,fabs(fxyz[i]));
       av_grad += fabs(fxyz[i]);
     }
     av_grad /= (3*natom);
     printf("max_gradient: %.16g,  average_gradient: %.16g\n",max_grad,av_grad);
     getchar();
     //____________________________________________________________output

  }/*cartesian propagation*/


  if (system.general.prop_type==2) {/*torsional propagation*/

     for (i=1;i<=3*natom;i++) {     
          x[i] = system.pos.x[i];
          y[i] = system.pos.y[i];
          z[i] = system.pos.z[i];   
     }
     
     init_propose(system);
     force(system);
     tors_transformXFXtoTFT();

     //____________________________________________________________output
     printf("before frprmn\n");
     max_grad = 0.0;
     av_grad = 0.0;
     for (i=1;i<=n;i++) {
       max_grad = MAX(max_grad,fabs(ft[i]));
       av_grad += fabs(ft[i]);
     }
     av_grad /= n;
     printf("max_gradient: %.16g,  average_gradient: %.16g\n",max_grad,av_grad);     getchar();
     //____________________________________________________________output

     frprmn(t,n,ftol,&iter,&fret,potential_tors,dpotential_tors);

     tors_transformTtoX();  
     acc_propose();
     force(system);
     tors_transformXFXtoTFT();
          
     //____________________________________________________________output
     printf("after frprmn\n");
     max_grad = 0.0;
     av_grad = 0.0;
     for (i=1;i<=n;i++) {
       max_grad = MAX(max_grad,fabs(ft[i]));
       av_grad += fabs(ft[i]);
     }
     av_grad /= n;
     printf("max_gradient: %.16g,  average_gradient: %.16g\n",max_grad,av_grad);     getchar();
     //____________________________________________________________output

  }/*torsional propagation*/

#endif



/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/
