#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"

#define DEBUG_OFF

/*=========================================================================*/
void Pos::postprop(System &system)
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int iter;
  int gradclos_itmax;
  int prop_type_save; 
  int *postprop_succeed = &(system.maps.postprop_succeed);

  
/*-------------------------------------------------------------------------*/
/* 0) Initialize                                                           */

  *postprop_succeed = 1;
  
/*-------------------------------------------------------------------------*/
/* I) Perform minimization if required                                     */

#ifdef DEBUG
      printf("Before postprop minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif

  if (system.general.postprop_minimize == 1) {

    if (system.general.postprop_minimize_type==1) {/*cartesian*/
        prop_type_save = system.general.prop_type;
        system.general.prop_type = 1;              /*cart type*/
        postprop_minimize_cart(system);
        system.general.prop_type = prop_type_save;
	if ((*postprop_succeed)==0) return;
#ifdef DEBUG
      printf("After postprop cart minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("system energy xi,yi,zi %g\n",system.pos.potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif

    }/*cartesian*/
     
    if (system.general.postprop_minimize_type==2) {/*torsional*/
        postprop_minimize_tors(system);
	if ((*postprop_succeed)==0) return;
#ifdef DEBUG
      printf("After postprop tors minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("system energy xi,yi,zi %g\n",system.pos.potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif
    }/*torsional*/ 
  
    if (system.general.postprop_minimize_type==3) {/*bend*/      
            prop_type_save = system.general.prop_type;
            system.general.prop_type = 3;              /*bend type*/    
            postprop_minimize_bend(system);
            system.general.prop_type = prop_type_save;
	    if ((*postprop_succeed)==0) return;
#ifdef DEBUG
      printf("After postprop bend minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("system energy xi,yi,zi %g\n",system.pos.potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif
    }/*bend*/
  
    if (system.general.postprop_minimize_type==4) {/*bend+torsion*/

        prop_type_save = system.general.prop_type;
        system.general.prop_type = 3;              /*bend type*/    
        postprop_minimize_bend(system);
        system.general.prop_type = prop_type_save;
	if ((*postprop_succeed)==0) return;
#ifdef DEBUG
      printf("After postprop bend minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif
        postprop_minimize_tors(system);
	if ((*postprop_succeed)==0) return;
#ifdef DEBUG
      printf("After postprop tors minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif
    }/*bend+torsion*/

    if (system.general.postprop_minimize_type==5) {/*torsion+bend*/

        postprop_minimize_tors(system);
        if ((*postprop_succeed)==0) return;

        prop_type_save = system.general.prop_type;
        system.general.prop_type = 3;              /*bend type*/    
        postprop_minimize_bend(system);
        system.general.prop_type = prop_type_save;
        if ((*postprop_succeed)==0) return;

    }/*torsion+bend*/

    if (system.general.postprop_minimize_type==6) {/*closure*/
#ifdef DEBUG
      printf("Before postprop clos minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif
        postprop_minimize_clos(system);
	if ((*postprop_succeed)==0) return;
#ifdef DEBUG
      printf("After postprop clos minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif
    }/*closure*/

    if (system.general.postprop_minimize_type==7) {/*grad_clos-bend+tors*/
      gradclos_itmax = system.general.postprop_minimize_gradclos_itmax;
      for (iter=1;iter<=gradclos_itmax;iter++) {

        prop_type_save = system.general.prop_type;
        system.general.prop_type = 3;              /*bend type*/    
        postprop_minimize_bend(system);
        system.general.prop_type = prop_type_save;
        if ((*postprop_succeed)==0) return;

        postprop_minimize_tors(system);
        if ((*postprop_succeed)==0) return;


#ifdef DEBUG
      printf("After postprop igc-bend-tors <bend-tors> minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif

        postprop_move_clos(system);           /*xi yi zi of pos is stored */
        if ((*postprop_succeed)==0) return;

#ifdef DEBUG
      printf("After postprop igc-bend-tors <bend-tors-clos> minimization\n");
      printf("energy xi,yi,zi %g\n",potential(system,0));
      printf("epotp %g\n",epotp);
      getchar();
#endif
      }/*endfor-iter*/

    }/*grad_clos-bend+torsion*/

     
    if (system.general.postprop_minimize_type==8) {/*sidechain torsional*/
        postprop_minimize_sidechain(system);
	if ((*postprop_succeed)==0) return;
    }/*sidechain torsional*/ 

  }/*endif-postprop_minimize*/

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::postprop_minimize_cart(System &system)
/*-------------------------------------------------------------------------*/
/* perform postpropagation cartesian minimization                          */
/* effected variables: xi, yi, zi                                          */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, iter, n;
  double fret, ftol;
  double *xt,*yt,*zt;

/*-------------------------------------------------------------------------*/
/* I) perform minimization                                                 */  
/*    minimization is performed in "system.pos.xyz", thus results should   */
/*    be further processed/migrated into current Pos class                 */
/*    effected system.pos  vectors:                                        */
/*                      system.pos.$  $={x,y,z}                            */
/*                      system.pos.f$ $={x,y,z}                            */ 
/*    effected system.pos pointers: xyz, dxyz                              */
/*                                                                         */
/*    effected pos variables: xyz                                          */
/*                                                                         */


  ftol = system.general.minimize_tol;

      n = 3*natom;
      copy_3vec_to_vec(xi,yi,zi,xyz,natom);
      if (system.general.minimize_type==1)
          frprmn(xyz,n,ftol,&iter,&fret,potential_cart,dpotential_cart);
      if (system.general.minimize_type==2)
          dfpmin(xyz,n,ftol,&iter,&fret,potential_cart,dpotential_cart);


/*-------------------------------------------------------------------------*/
/* II) migrate positions to current Pos class xi, yi, zi by switching      */
/*     pointers                                                            */

      xt = xi; yt =yi; zt = zi;

      xi  = system.pos.x;
      yi  = system.pos.y;
      zi  = system.pos.z;

      system.pos.x = xt;
      system.pos.y = yt;
      system.pos.z = zt;

      system.pos.xyz  = 0;
      system.pos.dxyz = 0;

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/



/*=========================================================================*/
void Pos::postprop_minimize_tors(System &system)
/*-------------------------------------------------------------------------*/
/* perform postpropagation torsional minimization                          */
/* effected variables: xi, yi, zi                                          */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, iter, n;
  double fret, ftol;
  double *xt,*yt,*zt;

  int *postprop_succeed = &(system.maps.postprop_succeed);

/*-------------------------------------------------------------------------*/
/* 0) Initialize xi, yi, zi and x, y, z in system based on xi,yi,zi in pos */


  copy_3vec_to_3vec(xi,yi,zi,&(system.pos.xi[0]),
                             &(system.pos.yi[0]),
                             &(system.pos.zi[0]),natom);

  copy_3vec_to_3vec(xi,yi,zi,&(system.pos.x[0]),
                             &(system.pos.y[0]),
                             &(system.pos.z[0]),natom);

/*-------------------------------------------------------------------------*/
/* I) perform minimization                                                 */  
/*    effected system.pos pointers: ft                                     */
/*                                                                         */
/*    effected pos vartiables: t                                           */
/*                                                                         */

  ftol = system.general.minimize_tol;

      n = system.tors_defs.num_trans;
      system.pos.tors_X = t_scr;
      system.pos.t      = t_scr;

      tors_transformXtoT();
      if (system.general.minimize_type==1)
          frprmn(t,n,ftol,&iter,&fret,potential_tors,dpotential_tors);
      if (system.general.minimize_type==2)
          dfpmin(t,n,ftol,&iter,&fret,potential_tors,dpotential_tors);

      system.pos.ft = 0; 

/*-------------------------------------------------------------------------*/
/* II)if postprop is successful => translate postions to xi, yi, zi        */
/*     here only *t is changed, and                                        */
/*     xi,yi,zi are adjusted                                               */
/*     accordingly                                                         */
/*    if postprop is not successful => restore xi, yi, zi                  */

      if ((*postprop_succeed) == 1) {
           tors_transformTtoX(); 
      } else {
           copy_3vec_to_3vec(&(system.pos.x[0]),
                             &(system.pos.y[0]),
                             &(system.pos.z[0]),xi,yi,zi,natom);
      }/*endif*/ 

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::postprop_minimize_sidechain(System &system)
/*-------------------------------------------------------------------------*/
/* perform postpropagation sidechain torsional minimization                */
/* effected variables: xi, yi, zi                                          */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/*  I) Set system.tors_defs_ptr to system.tors_defs_sidechain              */

      system.tors_defs_ptr = &(system.tors_defs_sidechain);

/*-------------------------------------------------------------------------*/
/*  II) Perform minimization in sidechain torsional space                  */
     
      postprop_minimize_tors(system);

/*-------------------------------------------------------------------------*/
/*  III) Set system.tors_defs_ptr to system.tors_defs                      */

      system.tors_defs_ptr = &(system.tors_defs);

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 


/*=========================================================================*/
void Pos::postprop_minimize_bend(System &system)
/*-------------------------------------------------------------------------*/
/* perform postpropagation bend minimization                               */
/* effected variables: xi, yi, zi                                          */
/*-------------------------------------------------------------------------*/
{ /*begin routine */
/*-------------------------------------------------------------------------*/

  int i, iter, n;
  double fret, ftol;
  double *xt,*yt,*zt;

  int *postprop_succeed = &(system.maps.postprop_succeed);

/*-------------------------------------------------------------------------*/
/* 0) Initialize xi, yi, zi and x, y, z in system based on xi,yi,zi in pos */


  copy_3vec_to_3vec(xi,yi,zi,&(system.pos.xi[0]),
                             &(system.pos.yi[0]),
                             &(system.pos.zi[0]),natom);

  copy_3vec_to_3vec(xi,yi,zi,&(system.pos.x[0]),
                             &(system.pos.y[0]),
                             &(system.pos.z[0]),natom);

/*-------------------------------------------------------------------------*/
/* I) perform minimization                                                 */  
/*    effected system.pos pointers: fb                                     */
/*                                                                         */
/*    effected pos vartiables: b                                           */
/*                                                                         */

  ftol = system.general.minimize_tol;

      n = system.tors_defs.num_tbend;
      system.pos.tors_X = t_scr;
      system.pos.b      = b_scr;

      tors_transformXtoB();
      if (system.general.minimize_type==1)
          frprmn(b,n,ftol,&iter,&fret,potential_bend,dpotential_bend);
      if (system.general.minimize_type==2)
          dfpmin(b,n,ftol,&iter,&fret,potential_bend,dpotential_bend);

      system.pos.fb = 0; 


/*-------------------------------------------------------------------------*/
/* II)if postprop is successful => translate postions to xi, yi, zi        */
/*     here only *t is changed, and                                        */
/*     xi,yi,zi are adjusted                                               */
/*     accordingly                                                         */
/*    if postprop is not successful => restore xi, yi, zi                  */

      if ((*postprop_succeed) == 1) {
           tors_transformBtoX();    
      } else {
           copy_3vec_to_3vec(&(system.pos.x[0]),
                             &(system.pos.y[0]),
                             &(system.pos.z[0]),xi,yi,zi,natom);
      }/*endif*/ 

/*-------------------------------------------------------------------------*/
} /*end routine */
/*=========================================================================*/


/*=========================================================================*/
void Pos::postprop_minimize_clos(System &system)
/*-------------------------------------------------------------------------*/
/*  Moves system in torsional space                                        */
/*-------------------------------------------------------------------------*/
{/*begin routine*/  
/*-------------------------------------------------------------------------*/

 int n;
 int istep_mc;
 int nstep_mc          = system.general.postprop_minimize_itmax;  
 int *postprop_succeed = &(system.maps.postprop_succeed);
 long *seed            = &(system.general.seed);
 double pacc, epot_i, epot_ip1;
 double T_loc          = T;
 double rate_anneal    = pow(1.0/T,1/nstep_mc);

/*-------------------------------------------------------------------------*/
/* I) Perform local Monte Carlo on the closure relation                    */ 

 n = 3*natom;
 copy_3vec_to_vec(xi,yi,zi,xyz,natom);
 epot_i   = potential_cart(xyz);
 for (istep_mc=1;istep_mc<=nstep_mc;istep_mc++) {   
      postprop_move_clos(system);           /*xi yi zi of pos is stored */
      if ((*postprop_succeed)==0) {return;}
      copy_3vec_to_vec(xi,yi,zi,xyz,natom); /*   in xi yi zi of system  */
      epot_ip1 = potential_cart(xyz);
      pacc = MIN(1.0,exp(-(epot_ip1-epot_i)/T_loc));
      
      if (pacc < ran1(seed)) {
	// if (epot_ip1 > epot_i) { 
           copy_3vec_to_3vec(&(system.pos.xi[0]),
                             &(system.pos.yi[0]),
                             &(system.pos.zi[0]),xi,yi,zi,natom);
      } else {epot_i = epot_ip1;}
      T_loc *= rate_anneal;
 }/*endfor*/
 
/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 


/*=========================================================================*/
void Pos::postprop_move_clos(System &system)
/*-------------------------------------------------------------------------*/
/*  Moves system in torsional space                                        */
/*-------------------------------------------------------------------------*/
{/*begin routine*/  
/*-------------------------------------------------------------------------*/
                                                                
 int level,ilevel,iatm,trans_any;
 int clclosed, clsucceed;

 double *xt, *yt, *zt;

 General   *general   = &(system.general);
 Tors_defs *tors_defs = &(system.tors_defs);
 int num_tr           =  tors_defs->num_trans; 
 int *ityp_tr         =  tors_defs->ityp_trans; 
 int *ityp_clos       =  tors_defs->ityp_clos;
 int *postprop_succeed = &(system.maps.postprop_succeed);

 char   warn_msg[LINE_NCHARMAX];


/*-------------------------------------------------------------------------*/
/* 0) Pointer operations                                                   */
/*    Restore in II)                                                       */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.A) due to adjust x, y, z temporarily has to have the same values as   */
/*    xi, yi, zi, therefore xi, yi, zi of system is initialized based on   */
/*    xi,yi,zi in pos, then temporarily x,y,z will point to xi, yi, zi in  */
/*    system                                                               */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */


  copy_3vec_to_3vec(xi,yi,zi,&(system.pos.xi[0]),
                             &(system.pos.yi[0]),
                             &(system.pos.zi[0]),natom);

  xt = x;
  yt = y;
  zt = z;

  x = system.pos.xi;
  y = system.pos.yi;
  z = system.pos.zi;
    
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.B) Change ityp_clos to ityp_clos_postprop                             */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   tors_defs->ityp_clos = tors_defs->ityp_clos_postprop;

/*-------------------------------------------------------------------------*/
/* I) Transformation loop                                                  */
/*    Move Closure Basis (if any) After Backward Half                      */  
/*-------------------------------------------------------------------------*/

   
   adjust(this,system,1);

   for (level = num_tr;level >= 1;level--) {/*backward half*/
       /* 1st three steps involve primary tors and group atoms */
       /* i)forward translate */
          translate(this,tors_defs,level,1);  
       /* ii)forward rotate */
          rotate(this,tors_defs,level,1); 
       /* iii)forward resolve */
          resolve(this,tors_defs,level,1);
       /* iv) group atoms */
          group(this,tors_defs,level,1);
       /* v) adjust atoms */
          adjust(this,system,level,1);
   }/*end of backward half*/ 

   if (general->cblc_defined==1) {
       clos_succeed = 1;
   }

   for (level = 1;level <= num_tr;level++) {/*forward half*/
       /* 0) adjust atoms */
          adjust(this,system,level,-1);
       /* 00) ungroup atoms */
          group(this,tors_defs,level,-1);
       /* i) backward resolve */
          resolve(this,tors_defs,level,-1);
       /* ii) backward  rotate */
          rotate(this,tors_defs,level,-1);
       /* iii) backward translate */
          translate(this,tors_defs,level,-1);
       /* iv) closure         */
          if (ityp_tr[level]==2) {
             clclosed      =  closure(this,tors_defs,level,-1);
             clsucceed     =  clclosed;
             if ( clclosed == 0 ) {
                  clos_succeed  = 0; 
                  sprintf(warn_msg,"postprop_move_clos => closure is\n"
                                   "not healed!");
                  warnMessage(warn_msg);
                 *postprop_succeed = 0;
                  break;
             }
          }/*endif*/
   }/*end of forward half*/

   adjust(this,system,-1);


/*-------------------------------------------------------------------------*/
/* II) Restore ityp_clos and  x,y,z                                        */
/*-------------------------------------------------------------------------*/

   tors_defs->ityp_clos = ityp_clos;

   x = xt;
   y = yt;
   z = zt;

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 

