#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG


/*=========================================================================*/
 void Pos::move_closcompl(System &system)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for All Atom model                 */
/*-------------------------------------------------------------------------*/
{/*begin routine*/

  int icloscompl;

  Tors_defs *tors_defs   = &(system.tors_defs);  
  int ncloscompl         = tors_defs->ncloscompl;
  char **closcompl_label = tors_defs->closcompl_label; 

  for (icloscompl=1;icloscompl<=ncloscompl;icloscompl++) {
        if (strcasecmp(closcompl_label[icloscompl],"DNA_duplex")==0) {
  	    move_closcompl_DNA_duplex(system,icloscompl);
        }/*endif*/
  }/*endfor*/

/*-------------------------------------------------------------------------*/
}/*end routine*/
/*-------------------------------------------------------------------------*/


/*=========================================================================*/
 void Pos::move_closcompl_DNA_duplex(System &system,int icloscompl)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for All Atom model                 */
/*-------------------------------------------------------------------------*/
{/*begin routine*/


  int ilevel;
  int type, ndim;
  int i, iatm, jatm;
  int atm11, atm12, atm13, atm21, atm22, atm23; 
  int atm11p1, atm12p1, atm13p1, atm21p1, atm22p1, atm23p1; 
  int move_basepair, move_levelpair;

  int  clb1, clb2;
  int *atm_clb1, *atm_clb2;
  double norm, sig, dx, dy, dz;
  double xci, yci, zci, xcip1, ycip1, zcip1;
  double eyez;

  int prop_closcompl_type     = system.general.prop_closcompl_type;
  int prop_DNA_closcompl_type = system.general.prop_DNA_closcompl_type;

  double T0   = system.general.T;
  double T    = (this->T)*BOLTZ;
  double TpT0 = T/T0; 
  double one  = 1.0;
  double zero = 0.0;

  long *seed       = &(system.general.seed);
  double sig_trans = (system.general.prop_trans_sig)*TpT0;
  double sig_rot   = (system.general.prop_rot_sig)*TpT0;

  Tors_defs *tors_defs   = &(system.tors_defs);  

  double *e1 = tors_defs->rotate_utils.e1;
  double *e2 = tors_defs->rotate_utils.e2;
  double *ex = tors_defs->rotate_utils.ex;
  double *ey = tors_defs->rotate_utils.ey;
  double *ez = tors_defs->rotate_utils.ez;
  double *e0 = tors_defs->rotate_utils.e0;

  int **ind_clbase_level = tors_defs->ind_clbase_level_closcompl[icloscompl];
  int *nlevel_closcompl  = tors_defs->nlevel_closcompl;
  int **ind_atm_clbase   = tors_defs->ind_atm_clbase;

  char   err_msg[LINE_NCHARMAX];


/*-------------------------------------------------------------------------*/
/* I) Move the levels basis                                                */

    ndim = 3;
    zero = 1.e-10;

    move_basepair  = 0;
    move_levelpair = 0;

    if (prop_DNA_closcompl_type==1) {type=1;}
    if (prop_DNA_closcompl_type==2) {type=2;}
    if (prop_DNA_closcompl_type==3) {
        type = MIN(2,(int)(2.0*ran1(seed))+1);
    }/*endif*/

    if (type==1) {move_basepair  = 1;}
    if (type==2) {move_levelpair = 1;}

/*-------------------------------------------------------------------------*/
/* I.A) Move the basepairs at each level                                   */
    
    if (move_basepair==1) {
       for (ilevel=1;ilevel<=nlevel_closcompl[icloscompl];ilevel++) {
            /* - - - - - - - - - - - - - - - - - - - */
            /*  ignore level if any clbase is fixed  */
            /* - - - - - - - - - - - - - - - - - - - */
            clb1 = ind_clbase_level[ilevel][1];
            clb2 = ind_clbase_level[ilevel][2];
            if ( ((tors_defs->freeze_clbase[clb1])==1) ||
  	         ((tors_defs->freeze_clbase[clb2])==1) ) {continue;}

         /* - - - - - - - - - - - - - - - - - - - */
         /*  decide on propagation type           */
         /* - - - - - - - - - - - - - - - - - - - */
	 if (prop_closcompl_type==1)
	    {type = MIN(3,(int)(3.0*ran1(seed))+1);}
         if (prop_closcompl_type==2) 
            {type = 3 + MIN(3,(int)(3.0*ran1(seed))+1);}
         if (prop_closcompl_type==3) 
            {type = MIN(6,(int)(6.0*ran1(seed))+1);}
            /* - - - - - - */
            /*  find cip1  */
            /* - - - - - - */
            if (ilevel<nlevel_closcompl[icloscompl]) {
                clb1 = ind_clbase_level[ilevel+1][1];
                clb2 = ind_clbase_level[ilevel+1][2];
                atm_clb1 = ind_atm_clbase[clb1];
                atm_clb2 = ind_atm_clbase[clb2];
                atm12 =  atm_clb1[2]; 
                atm22 =  atm_clb2[2];
                ey[1] = xi[atm12] - xi[atm22];
                ey[2] = yi[atm12] - yi[atm22];
                ey[3] = zi[atm12] - zi[atm22];
                xcip1 = xi[atm22] + 0.5*ey[1];    
                ycip1 = yi[atm22] + 0.5*ey[2];
                zcip1 = zi[atm22] + 0.5*ey[3];
            }/*endif*/
   
            clb1 = ind_clbase_level[ilevel][1];
            clb2 = ind_clbase_level[ilevel][2];
            atm_clb1 = ind_atm_clbase[clb1];
            atm_clb2 = ind_atm_clbase[clb2];
            atm11 =  atm_clb1[1];
            atm12 =  atm_clb1[2]; 
            atm13 =  atm_clb1[3];
            atm21 =  atm_clb2[1];
            atm22 =  atm_clb2[2];
            atm23 =  atm_clb2[3];
            /* - - - - - - */
            /*  find ey,e0 */
            /* - - - - - - */
            ey[1] = xi[atm12] - xi[atm22];
            ey[2] = yi[atm12] - yi[atm22];
            ey[3] = zi[atm12] - zi[atm22];
            xci = xi[atm22] + 0.5*ey[1];    
            yci = yi[atm22] + 0.5*ey[2];
            zci = zi[atm22] + 0.5*ey[3];
            norm  = sqrt(ey[1]*ey[1]+ey[2]*ey[2]+ey[3]*ey[3]);
            for (i=1;i<=ndim;i++) {ey[i] /= norm;}
            e0[1] = xci;
            e0[2] = yci;
            e0[3] = zci;
            /* - - - - - - */
            /*  find ez    */
            /* - - - - - - */
            if (ilevel<nlevel_closcompl[icloscompl]) {
                ez[1] = xcip1 - xci;
                ez[2] = ycip1 - yci;
                ez[3] = zcip1 - zci;
            }/*endif*/
            eyez = ey[1]*ez[1] + ey[2]*ez[2] + ey[3]*ez[3];
            ez[1] = ez[1] - ey[1]*eyez;
            ez[2] = ez[2] - ey[2]*eyez;
            ez[3] = ez[3] - ey[3]*eyez;
            norm  = sqrt(ez[1]*ez[1]+ez[2]*ez[2]+ez[3]*ez[3]);
            for (i=1;i<=ndim;i++) {ez[i] /= norm;}
            /* - - - - - - */
            /*  find ex    */
            /* - - - - - - */
            cprd(ey[1],ey[2],ey[3],ez[1],ez[2],ez[3],
                                 &(ex[1]),&(ex[2]),&(ex[3]));
            norm  = sqrt(ex[1]*ex[1]+ex[2]*ex[2]+ex[3]*ex[3]);
            for (i=1;i<=ndim;i++) {ex[i] /= norm;}
   
            switch (type) {
    	    case 1:/*Stretch*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ey[1]; dy = sig*ey[2]; dz = sig*ey[3];
                   translate_entry(system, dx, dy, dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 2:/*Shear*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ex[1]; dy = sig*ex[2]; dz = sig*ex[3];
                   translate_entry(system, dx, dy, dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 3:/*Stagger*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ez[1]; dy = sig*ez[2]; dz = sig*ez[3];
                   translate_entry(system, dx, dy, dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 4:/*Buckle*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0, sig,zero,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,-sig,zero,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 5:/*Propeller*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero, sig,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,-sig,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 6:/*Opening*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero, sig,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero,-sig,
                                                     atm21,atm22,atm23);
   	      break;
            }/*switch*/
            
       }/*endfor*/
    }/*move_basepair*/


/*-------------------------------------------------------------------------*/
/* I.B) Move the successive level pairs                                    */

    
    if (move_levelpair==1) {
       for (ilevel=1;ilevel<=nlevel_closcompl[icloscompl]-1;ilevel++) {
            /* - - - - - - - - - - - - - - - - - - - */
            /*  ignore level if any clbase is fixed  */
            /* - - - - - - - - - - - - - - - - - - - */
            clb1 = ind_clbase_level[ilevel][1];
            clb2 = ind_clbase_level[ilevel][2];
            if ( ((tors_defs->freeze_clbase[clb1])==1) ||
  	         ((tors_defs->freeze_clbase[clb2])==1) ) {continue;}
            clb1 = ind_clbase_level[ilevel+1][1];
            clb2 = ind_clbase_level[ilevel+1][2];
            if ( ((tors_defs->freeze_clbase[clb1])==1) ||
  	         ((tors_defs->freeze_clbase[clb2])==1) ) {continue;}

         /* - - - - - - - - - - - - - - - - - - - */
         /*  decide on propagation type           */
         /* - - - - - - - - - - - - - - - - - - - */
	 if (prop_closcompl_type==1)
	    {type = MIN(3,(int)(3.0*ran1(seed))+1);}
         if (prop_closcompl_type==2) 
            {type = 3 + MIN(3,(int)(3.0*ran1(seed))+1);}
         if (prop_closcompl_type==3) 
            {type = MIN(6,(int)(6.0*ran1(seed))+1);}
            /* - - - - - - */
            /*  find cip1  */
            /* - - - - - - */
            clb1 = ind_clbase_level[ilevel+1][1];
            clb2 = ind_clbase_level[ilevel+1][2];
            atm_clb1 = ind_atm_clbase[clb1];
            atm_clb2 = ind_atm_clbase[clb2];
            atm11p1 =  atm_clb1[1];
            atm12p1 =  atm_clb1[2]; 
            atm13p1 =  atm_clb1[3];
            atm21p1 =  atm_clb2[1];
            atm22p1 =  atm_clb2[2];
            atm23p1 =  atm_clb2[3];

            ey[1] = xi[atm12p1] - xi[atm22p1];
            ey[2] = yi[atm12p1] - yi[atm22p1];
            ey[3] = zi[atm12p1] - zi[atm22p1];
            xcip1 = xi[atm22p1] + 0.5*ey[1];    
            ycip1 = yi[atm22p1] + 0.5*ey[2];
            zcip1 = zi[atm22p1] + 0.5*ey[3];
   
            clb1 = ind_clbase_level[ilevel][1];
            clb2 = ind_clbase_level[ilevel][2];
            atm_clb1 = ind_atm_clbase[clb1];
            atm_clb2 = ind_atm_clbase[clb2];
            atm11 =  atm_clb1[1];
            atm12 =  atm_clb1[2]; 
            atm13 =  atm_clb1[3];
            atm21 =  atm_clb2[1];
            atm22 =  atm_clb2[2];
            atm23 =  atm_clb2[3];
            /* - - - - - - */
            /*  find ey,e0 */
            /* - - - - - - */
            ey[1] = xi[atm12] - xi[atm22];
            ey[2] = yi[atm12] - yi[atm22];
            ey[3] = zi[atm12] - zi[atm22];
            xci = xi[atm22] + 0.5*ey[1];    
            yci = yi[atm22] + 0.5*ey[2];
            zci = zi[atm22] + 0.5*ey[3];
            norm  = sqrt(ey[1]*ey[1]+ey[2]*ey[2]+ey[3]*ey[3]);
            for (i=1;i<=ndim;i++) {ey[i] /= norm;}
            e0[1] = xci;
            e0[2] = yci;
            e0[3] = zci;
            /* - - - - - - */
            /*  find ez    */
            /* - - - - - - */
            ez[1] = xcip1 - xci;
            ez[2] = ycip1 - yci;
            ez[3] = zcip1 - zci;

            eyez = ey[1]*ez[1] + ey[2]*ez[2] + ey[3]*ez[3];
            ez[1] = ez[1] - ey[1]*eyez;
            ez[2] = ez[2] - ey[2]*eyez;
            ez[3] = ez[3] - ey[3]*eyez;
            norm  = sqrt(ez[1]*ez[1]+ez[2]*ez[2]+ez[3]*ez[3]);
            for (i=1;i<=ndim;i++) {ez[i] /= norm;}
            /* - - - - - - */
            /*  find ex    */
            /* - - - - - - */
            cprd(ey[1],ey[2],ey[3],ez[1],ez[2],ez[3],
                                 &(ex[1]),&(ex[2]),&(ex[3]));
            norm  = sqrt(ex[1]*ex[1]+ex[2]*ex[2]+ex[3]*ex[3]);
            for (i=1;i<=ndim;i++) {ex[i] /= norm;}
   
            switch (type) {
    	    case 1:/*Slide*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ey[1]; dy = sig*ey[2]; dz = sig*ey[3];
                   translate_entry(system, dx, dy, dz,atm11p1,atm12p1,atm13p1);
                   translate_entry(system, dx, dy, dz,atm21p1,atm22p1,atm23p1);
                   translate_entry(system,-dx,-dy,-dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 2:/*Shift*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ex[1]; dy = sig*ex[2]; dz = sig*ex[3];
                   translate_entry(system, dx, dy, dz,atm11p1,atm12p1,atm13p1);
                   translate_entry(system, dx, dy, dz,atm21p1,atm22p1,atm23p1);
                   translate_entry(system,-dx,-dy,-dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 3:/*Rise*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ez[1]; dy = sig*ez[2]; dz = sig*ez[3];
                   translate_entry(system, dx, dy, dz,atm11p1,atm12p1,atm13p1);
                   translate_entry(system, dx, dy, dz,atm21p1,atm22p1,atm23p1);
                   translate_entry(system,-dx,-dy,-dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 4:/*Tilt*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0, sig,zero,zero,
                                                     atm11p1,atm12p1,atm13p1);
                   rotate_entry(system,ex,ey,ez,e0, sig,zero,zero,
                                                     atm21p1,atm22p1,atm23p1);
                   rotate_entry(system,ex,ey,ez,e0,-sig,zero,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,-sig,zero,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 5:/*Roll*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero, sig,zero,
                                                     atm11p1,atm12p1,atm13p1);
                   rotate_entry(system,ex,ey,ez,e0,zero, sig,zero,
                                                     atm21p1,atm22p1,atm23p1);
                   rotate_entry(system,ex,ey,ez,e0,zero,-sig,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,-sig,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 6:/*Twist*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero, sig,
                                                     atm11p1,atm12p1,atm13p1);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero, sig,
                                                     atm21p1,atm22p1,atm23p1);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero,-sig,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero,-sig,
                                                     atm21,atm22,atm23);
   	      break;
            }/*switch*/
            
       }/*endfor*/
    }/*move_basepair*/

/*-------------------------------------------------------------------------*/
/* II) DEBUG                                                               */

#ifdef DEBUG
  getchar();
#endif



/*-------------------------------------------------------------------------*/
}/*end routine*/
/*-------------------------------------------------------------------------*/
