#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG


/*=========================================================================*/
 void Tors_defs::setup_closcompl(const Maps *maps,
                                 const Pos *pos,
                                 const General *general)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for All Atom model                 */
/*-------------------------------------------------------------------------*/
{/*begin routine*/

  int icloscompl;
  int imol, iimol, iclbase, jclbase, jjclbase;
  int found_in_closcompl;
  
  for (icloscompl=1;icloscompl<=ncloscompl;icloscompl++) {
        if (strcasecmp(closcompl_label[icloscompl],"DNA_duplex")==0) {
  	    setup_closcompl_DNA_duplex(maps,pos,general,icloscompl);
        }/*endif*/
  }/*endfor*/

  num_clbase_free = 0;
  for (iclbase=1;iclbase<=num_clbase;iclbase++) {
       found_in_closcompl = 0;
       for (icloscompl=1;icloscompl<=ncloscompl;icloscompl++) {
	    for (imol=1;imol<=nmol_closcompl[icloscompl];imol++) {
	         iimol = ind_mol_closcompl[icloscompl][imol];
                 for (jclbase=1;jclbase<=nclbase_mol[iimol];jclbase++) {
		      jjclbase = ind_clbase_mol[iimol][jclbase];
                      if (iclbase == jjclbase) {found_in_closcompl = 1;}
                 }/*endfor*/
            }/*endfor*/
       }/*endfor*/
       if (found_in_closcompl==0) {
	   num_clbase_free++;
           ind_clbase_free[num_clbase_free] = iclbase;  
       }/*endif*/
  }/*endfor*/


/*-------------------------------------------------------------------------*/
}/*end routine*/
/*-------------------------------------------------------------------------*/


/*=========================================================================*/
 void Tors_defs::setup_closcompl_DNA_duplex(const Maps *maps,
                                            const Pos *pos,
                                            const General *general, 
                                            int icloscompl)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for All Atom model                 */
/*-------------------------------------------------------------------------*/
{/*begin routine*/

  int iatm, jatm;
  int ind_mol1, ind_mol2; 
  int ilevel, jlevel, iclbase, jclbase, iiclbase, jjclbase;
  int jclbase_start;

  char chainid;

  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;
  char **mol_label   = maps->mol_label;
  char **mol_chainid = maps->mol_chainid;

  int **ind_clbase_level =  ind_clbase_level_closcompl[icloscompl];

  double *x = pos->x;
  double *y = pos->y;
  double *z = pos->z;

  double infinity, min_dist, dist_ij;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Incompatibilities                                                    */


  ind_mol1 = ind_mol_closcompl[icloscompl][1];
  ind_mol2 = ind_mol_closcompl[icloscompl][2];


  if ( nmol_closcompl[icloscompl] != 2 ) {
       sprintf(err_msg,"In DNA_duplex (%d. CLOSCOMPL), the \n"
                       "number of molecules is not 2       \n",icloscompl);
       errorMessage(err_msg);
  }/*endif*/


  if ( nclbase_mol[ind_mol1] !=  nclbase_mol[ind_mol2] ) {
       sprintf(err_msg,"In DNA_duplex (%d. CLOSCOMPL), the \n"
                       "the # of clbase is not the same in \n"
                       "the two strands",icloscompl);
       errorMessage(err_msg);
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Get the number of levels as the number of clbase in the first strand */
/*    Fill in ind_clbase_level                                             */


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A) Get the number of levels                                           */

  nlevel_closcompl[icloscompl] =  nclbase_mol[ind_mol1];

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.B) Fill in the first level                                            */ 
/*      ind_clbase_level[1][1], ind_clbase_level[1][2]                     */

  infinity = 1000.0;

       ilevel   = 1; 
       iclbase  = ilevel;
       iiclbase = ind_clbase_mol[ind_mol1][iclbase];
       ind_clbase_level[ilevel][1] =  iiclbase;
       if (natm_clbase[iiclbase] != 3) {
           chainid = mol_chainid[ind_mol1][0];
           sprintf(err_msg,"In DNA_duplex (%d. CLOSCOMPL), the \n"
                           "the # of clbase atoms is not 3 in  \n"
                           "%d. clbase in molecule/chain %c",
                                        icloscompl,iclbase,chainid);
           errorMessage(err_msg);
       }/*endif*/
       min_dist = infinity;
       iatm = ind_atm_clbase[iiclbase][2];
       for (jclbase=1;jclbase<=nclbase_mol[ind_mol2];jclbase++) {
	    jjclbase = ind_clbase_mol[ind_mol2][jclbase];
            jatm    = ind_atm_clbase[jjclbase][2];
            dist_ij = sqrt( (x[iatm]-x[jatm])*(x[iatm]-x[jatm]) 
                           +(y[iatm]-y[jatm])*(y[iatm]-y[jatm]) 
			   +(z[iatm]-z[jatm])*(z[iatm]-z[jatm]) );
            if (min_dist > dist_ij) {
	        min_dist = dist_ij;
                ind_clbase_level[ilevel][2] =  jjclbase;
                jclbase_start = jclbase; 
            }/*endif*/
       }/*endfor*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.C) Check for jclbase_start                                            */
/*       jclbase_start must be either 1 or nclbase_mol[ind_mol2]           */

  if (abs(jclbase_start - nclbase_mol[ind_mol2]) > (jclbase_start - 1)) {
      for (ilevel=2;ilevel<=nlevel_closcompl[icloscompl];ilevel++) {
	   jlevel = ilevel;
           iclbase = ind_clbase_mol[ind_mol1][ilevel];
           ind_clbase_level[ilevel][1] = iclbase;
           jclbase = ind_clbase_mol[ind_mol2][jlevel];
           ind_clbase_level[ilevel][2] = jclbase;
      }/*endfor*/
  }
  else if (abs(jclbase_start - nclbase_mol[ind_mol2]) < (jclbase_start - 1)) {
      for (ilevel=2;ilevel<=nlevel_closcompl[icloscompl];ilevel++) {
	   jlevel = nlevel_closcompl[icloscompl] - ilevel + 1;
           iclbase = ind_clbase_mol[ind_mol1][ilevel];
           ind_clbase_level[ilevel][1] = iclbase;
           jclbase = ind_clbase_mol[ind_mol2][jlevel];
           ind_clbase_level[ilevel][2] = jclbase;
      }/*endfor*/
  }
  else {
       sprintf(err_msg,"Tors_defs::setup_closcompl_DNA_duplex \n"
                       "jclbase_start is exactly between 1    \n" 
                       "and nclbase_mol[%d]",ind_mol2,nclbase_mol[ind_mol2]);
       errorMessage(err_msg);
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* II) DEBUG                                                               */

#ifdef DEBUG
  printf("DEBUG Tors_defs::setup_closcompl_DNA_duplex\n");
  for (jclbase=1;jclbase<=nclbase_mol[ind_mol1];jclbase++) {
       printf("ind_clbase_mol[%d][%d]: %d\n",ind_mol1,jclbase,
                                           ind_clbase_mol[ind_mol1][jclbase]);
  }/*endfor*/
  for (jclbase=1;jclbase<=nclbase_mol[ind_mol2];jclbase++) {
       printf("ind_clbase_mol[%d][%d]: %d\n",ind_mol2,jclbase,
                                           ind_clbase_mol[ind_mol2][jclbase]);
  }/*endfor*/
  for (ilevel=1;ilevel<=nlevel_closcompl[icloscompl];ilevel++) {
    printf("ind_clbase_level[%d][1]=%d\n",ilevel,ind_clbase_level[ilevel][1]);
    printf("ind_clbase_level[%d][2]=%d\n",ilevel,ind_clbase_level[ilevel][2]);
  }/*endfor*/
  getchar();
#endif



/*-------------------------------------------------------------------------*/
}/*end routine*/
/*-------------------------------------------------------------------------*/
