#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG


/*=========================================================================*/
void Pos::move_clbase(System &system)
/*-------------------------------------------------------------------------*/
/*  Translates and rotates the closure base                                */
/*                                                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/
  if (system.general.region_topology_defined==1) {
      move_clbase_regions(system);
  } else {
      move_clbase_regular(system);
  }/*endif*/

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 



/*=========================================================================*/
void Pos::move_clbase_regular(System &system)
/*-------------------------------------------------------------------------*/
/*  Translates and rotates the closure base                                */
/*                                                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/
                                                                
 int iatm, iclbase, iiclbase;
 int iatm1, iatm2, iatm3;
 double sig_trans, sig_rot, sig_scale;

 General   *general   = &(system.general);
 Maps      *maps      = &(system.maps);
 Tors_defs *tors_defs = &(system.tors_defs);

 int  num_clbase      = tors_defs->num_clbase;
 int  num_clbase_free = tors_defs->num_clbase_free;
 int *ind_clbase_free = tors_defs->ind_clbase_free;
 int *natm_clbase     = tors_defs->natm_clbase;
 int **ind_atm_clbase = tors_defs->ind_atm_clbase;

 char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Choose the propagation width with respect to torsional sigma         */

 sig_scale = 1.0;
 sig_trans = sig_scale*(general->prop_trans_sig);
 sig_rot   = sig_scale*(general->prop_rot_sig);

/*-------------------------------------------------------------------------*/
/* I) Loop over all free clbase and move them                              */

 for (iclbase=1;iclbase<=num_clbase_free;iclbase++) {
      iiclbase = ind_clbase_free[iclbase];
      if ((tors_defs->freeze_clbase[iiclbase])==1) {continue;}
      if (natm_clbase[iiclbase]==3) {
          iatm1 = ind_atm_clbase[iiclbase][1];
          iatm2 = ind_atm_clbase[iiclbase][2];
          iatm3 = ind_atm_clbase[iiclbase][3];
          move_entry(system,iatm1,iatm2,iatm3,sig_trans,sig_rot);
      } else {
 	sprintf(err_msg,"Pos::move_clbase:natm_clbase[iclbase]!=3\n"
                        "natm_clbase[%d]: %g",iclbase,natm_clbase[iclbase]);
        errorMessage(err_msg);
      }/*endif*/
 }/*endfor*/

/*-------------------------------------------------------------------------*/
/* I) Movel all clbase in all CLOSCOMPL                                    */
 
 move_closcompl(system);

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 


/*=========================================================================*/
void Pos::move_clbase_regular(System &system, int iclbase, double sig_trans,
                                                           double sig_rot)
/*-------------------------------------------------------------------------*/
/*  Translates and rotates the closure base                                */
/*                                                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/
                                                                
 int iatm1, iatm2, iatm3;

 Tors_defs *tors_defs = &(system.tors_defs);

 int *natm_clbase     = tors_defs->natm_clbase;
 int **ind_atm_clbase = tors_defs->ind_atm_clbase;

 char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* I) Loop over all free clbase and move them                              */


      if ((tors_defs->freeze_clbase[iclbase])==1) {return;}
      if (natm_clbase[iclbase]==3) {
          iatm1 = ind_atm_clbase[iclbase][1];
          iatm2 = ind_atm_clbase[iclbase][2];
          iatm3 = ind_atm_clbase[iclbase][3];
          move_entry(system,iatm1,iatm2,iatm3,sig_trans,sig_rot);
      } else {
 	sprintf(err_msg,"Pos::move_clbase:natm_clbase[iclbase]!=3\n"
                        "natm_clbase[%d]: %g",iclbase,natm_clbase[iclbase]);
        errorMessage(err_msg);
      }/*endif*/

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 

