#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
void Pos::move_clbase_regions(System &system)
/*-------------------------------------------------------------------------*/
/*  Translates and rotates the closure base                                */
/*                                                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  long *seed;                                       
  int iregion, jregion, nregion;
  double max_ran;

  nregion = system.regions.nregion;
 
#ifdef DEBUG
 printf("Debug I from move_clbase_regions\n");
 getchar();
#endif

 if (system.general.prop_regions_type == 1) {/*superimpose*/
  for (iregion=1;iregion<=nregion;iregion++) {
       move_clbase_region(system,iregion);
  }/*endfor*/
 }/*endif*/
  

 if (system.general.prop_regions_type == 2) {/*onebyone*/
  seed      = &(system.general.seed);  
  max_ran   = -1;
  for (jregion=1;jregion<=nregion;jregion++) {
       if (max_ran < ran1(seed)) {
    	   max_ran = ran1(seed);
           iregion = jregion;
       }/*endif*/
  }/*endfor*/
  move_clbase_region(system,iregion);
 }/*endif*/


 
#ifdef DEBUG
 printf("Debug II from move_clbase_regions\n");
 getchar();
#endif



/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 


/*=========================================================================*/
void Pos::move_clbase_region(System &system, int iregion)
/*-------------------------------------------------------------------------*/
/*  Translates and rotates the closure base                                */
/*                                                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/
                         
 long *seed;                                       
 int iclbase, num_trans, level;
 int nfreeres, nrespair, ires, jres, nseg;
 double sig_trans, sig_rot, max_ran;
 
 int *ind_clbase_freeres, *ind_clbase_seg_baseres;

 Tors_defs *tors_defs;

 Regions   *regions   = &(system.regions);
 REgion    *region    = &(regions->region[iregion]);


/*-------------------------------------------------------------------------*/
/* 0) Choose the propagation width with respect to torsional sigma         */


/*-------------------------------------------------------------------------*/
/* I) Global move of region                                                */
/*    Choose cnter residue randomly from [1,ncenteres]                     */

 seed      = &(system.general.seed);  
 max_ran   = -1;
 for (jres=1;jres<=region->ncenteres;jres++) {
      if (max_ran < ran1(seed)) {
  	  max_ran = ran1(seed);
          ires    = jres;
      }/*endif*/
 }/*endfor*/

 tors_defs = &(region->tors_defs[ires]);
 iclbase   = region->ind_clbase_centeres[ires]; 

 sig_trans = region->prop_trans_sig;
 sig_rot   = region->prop_rot_sig;

 num_trans = tors_defs->num_trans;
 for (level = 1;level<=num_trans;level++) {
      translate(this,tors_defs,level,1);
      rotate(this,tors_defs,level,1);
      resolve(this,tors_defs,level,1);
 }/*endfor*/
 
 move_clbase_regular(system,iclbase,sig_trans,sig_rot);

 for (level = 1;level<=num_trans;level++) {
      resolve(this,tors_defs,level,-1);
      rotate(this,tors_defs,level,-1);
      translate(this,tors_defs,level,-1);
 }/*endfor*/

/*-------------------------------------------------------------------------*/
/* II) Move residue pairs                                                  */

 nrespair = region->nrespair;
 for (ires=1;ires<=nrespair;ires++) {
      move_clbase_respair(system,region,ires);
 }/*endfor*/
 

/*-------------------------------------------------------------------------*/
/* III) Move free residues                                                 */

 nfreeres           = region->nfreeres;
 ind_clbase_freeres = region->ind_clbase_freeres;
 sig_trans          = region->prop_trans_sig_freeres;
 sig_rot            = region->prop_rot_sig_freeres;
 for (ires=1;ires<=nfreeres;ires++) {
      iclbase = ind_clbase_freeres[ires];      
      move_clbase_regular(system,iclbase,sig_trans,sig_rot);
 }/*endfor*/


/*-------------------------------------------------------------------------*/
/* IV) Move segment base residues                                          */

 nseg                   = region->nseg;
 ind_clbase_seg_baseres = region->ind_clbase_seg_baseres;
 sig_trans              = region->prop_trans_sig_freeres;
 sig_rot                = region->prop_rot_sig_freeres;
 for (ires=1;ires<=nseg;ires++) {
      iclbase = ind_clbase_seg_baseres[ires];      
      move_clbase_regular(system,iclbase,sig_trans,sig_rot);
 }/*endfor*/

/*-------------------------------------------------------------------------*/
/* I) Loop over all free clbase and move them                              */

/*-------------------------------------------------------------------------*/
}/*                              End                                       */
/*-------------------------------------------------------------------------*/ 
