#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG


/*=========================================================================*/
void Pos::move_clbase_respair(System &system,REgion *region, int irespair)
/*-------------------------------------------------------------------------*/
/*                 Moves clbase for a residue pair                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/

  int i, ilevel;
  int type, ndim;
  int atm11, atm12, atm13, atm21, atm22, atm23;
  int move_local, move_global;

  int  clb1, clb2;
  int *atm_clb1, *atm_clb2;
  double norm, sig, dx, dy, dz;
  double xci, yci, zci;
  double eyez;


  int prop_closcompl_type     = system.general.prop_closcompl_type;

  double T0    = system.general.T;
  double T     = (this->T)*BOLTZ;
  double sTpT0 = sqrt(T/T0); 
  double one  = 1.0;
  double zero = 0.0;

  long *seed       = &(system.general.seed);
  double sig_trans = (region->prop_trans_sig_respair)*sTpT0;
  double sig_rot   = (region->prop_rot_sig_respair)*sTpT0;

  double sig_trans_internal = (region->prop_trans_sig_respair_internal)*sTpT0;
  double sig_rot_internal   = (region->prop_rot_sig_respair_internal)*sTpT0;

  Tors_defs *tors_defs   = &(system.tors_defs);  

  double *ex = tors_defs->rotate_utils.ex;
  double *ey = tors_defs->rotate_utils.ey;
  double *ez = tors_defs->rotate_utils.ez;
  double *e0 = tors_defs->rotate_utils.e0;

  int **ind_atm_clbase        = tors_defs->ind_atm_clbase;
  int  *ind_clbase_pairedres1 = region->ind_clbase_pairedres1;
  int  *ind_clbase_pairedres2 = region->ind_clbase_pairedres2;

  char   err_msg[LINE_NCHARMAX];


/*-------------------------------------------------------------------------*/
/* 0) Initializa variables                                                 */

    ndim = 3;
    zero = 1.e-10;
    move_local  = 1;
    move_global = 1;

/*-------------------------------------------------------------------------*/
/* I) Get the local coordinate system                                      */

            /* - - - - - - - - - - - - - - - - - - - */
            /*  ignore level if any clbase is fixed  */
            /* - - - - - - - - - - - - - - - - - - - */
            clb1 = ind_clbase_pairedres1[irespair];
            clb2 = ind_clbase_pairedres2[irespair];
            if ( ((tors_defs->freeze_clbase[clb1])==1) ||
  	         ((tors_defs->freeze_clbase[clb2])==1) ) {return;}

         /* - - - - - - - - - - - - - - - - - - - */
         /*  decide on propagation type           */
         /* - - - - - - - - - - - - - - - - - - - */
	 if (prop_closcompl_type==1)
	    {type = MIN(3,(int)(3.0*ran1(seed))+1);}
         if (prop_closcompl_type==2) 
            {type = 3 + MIN(3,(int)(3.0*ran1(seed))+1);}
         if (prop_closcompl_type==3) 
            {type = MIN(6,(int)(6.0*ran1(seed))+1);}
   
            clb1 = ind_clbase_pairedres1[irespair];
            clb2 = ind_clbase_pairedres2[irespair];
            atm_clb1 = ind_atm_clbase[clb1];
            atm_clb2 = ind_atm_clbase[clb2];
            atm11 =  atm_clb1[1];
            atm12 =  atm_clb1[2]; 
            atm13 =  atm_clb1[3];
            atm21 =  atm_clb2[1];
            atm22 =  atm_clb2[2];
            atm23 =  atm_clb2[3];
            /* - - - - - - */
            /*  find ey,e0 */
            /* - - - - - - */
            ey[1] = xi[atm12] - xi[atm22];
            ey[2] = yi[atm12] - yi[atm22];
            ey[3] = zi[atm12] - zi[atm22];
            xci = xi[atm22] + 0.5*ey[1];    
            yci = yi[atm22] + 0.5*ey[2];
            zci = zi[atm22] + 0.5*ey[3];
            norm  = sqrt(ey[1]*ey[1]+ey[2]*ey[2]+ey[3]*ey[3]);
            for (i=1;i<=ndim;i++) {ey[i] /= norm;}
            e0[1] = xci;
            e0[2] = yci;
            e0[3] = zci;
            /* - - - - - - */
            /*  find ez    */
            /* - - - - - - */

            ez[1] = xi[atm11] - xi[atm13];
            ez[2] = yi[atm11] - yi[atm13];
            ez[3] = zi[atm11] - zi[atm13];

            eyez = ey[1]*ez[1] + ey[2]*ez[2] + ey[3]*ez[3];
            ez[1] = ez[1] - ey[1]*eyez;
            ez[2] = ez[2] - ey[2]*eyez;
            ez[3] = ez[3] - ey[3]*eyez;
            norm  = sqrt(ez[1]*ez[1]+ez[2]*ez[2]+ez[3]*ez[3]);
            for (i=1;i<=ndim;i++) {ez[i] /= norm;}
            /* - - - - - - */
            /*  find ex    */
            /* - - - - - - */
            cprd(ey[1],ey[2],ey[3],ez[1],ez[2],ez[3],
                                 &(ex[1]),&(ex[2]),&(ex[3]));
            norm  = sqrt(ex[1]*ex[1]+ex[2]*ex[2]+ex[3]*ex[3]);
            for (i=1;i<=ndim;i++) {ex[i] /= norm;}

/*-------------------------------------------------------------------------*/
/* III) Local moves                                                        */
   
    if (move_local==1) {
            switch (type) {
    	    case 1:/*Stretch*/	       
                   sig = mb(one,sig_trans_internal,seed);
                   dx = sig*ey[1]; dy = sig*ey[2]; dz = sig*ey[3];
                   translate_entry(system, dx, dy, dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 2:/*Shear*/	       
                   sig = mb(one,sig_trans_internal,seed);
                   dx = sig*ex[1]; dy = sig*ex[2]; dz = sig*ex[3];
                   translate_entry(system, dx, dy, dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 3:/*Stagger*/	       
                   sig = mb(one,sig_trans_internal,seed);
                   dx = sig*ez[1]; dy = sig*ez[2]; dz = sig*ez[3];
                   translate_entry(system, dx, dy, dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 4:/*Buckle*/	       
                   sig = mb(one,sig_rot_internal,seed);
                   rotate_entry(system,ex,ey,ez,e0, sig,zero,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,-sig,zero,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 5:/*Propeller*/	       
                   sig = mb(one,sig_rot_internal,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero, sig,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,-sig,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 6:/*Opening*/	       
                   sig = mb(one,sig_rot_internal,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero, sig,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero,-sig,
                                                     atm21,atm22,atm23);
   	      break;
            }/*switch*/
    }/*move_local*/

/*-------------------------------------------------------------------------*/
/* IV) Global moves                                                        */

    if (move_global==1) {   
            switch (type) {
    	    case 1:/*Slide*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ey[1]; dy = sig*ey[2]; dz = sig*ey[3];
                   translate_entry(system,-dx,-dy,-dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 2:/*Shift*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ex[1]; dy = sig*ex[2]; dz = sig*ex[3];
                   translate_entry(system,-dx,-dy,-dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 3:/*Rise*/	       
                   sig = mb(one,sig_trans,seed);
                   dx = sig*ez[1]; dy = sig*ez[2]; dz = sig*ez[3];
                   translate_entry(system,-dx,-dy,-dz,atm11,atm12,atm13);
                   translate_entry(system,-dx,-dy,-dz,atm21,atm22,atm23);
   	      break;
    	    case 4:/*Tilt*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0,-sig,zero,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,-sig,zero,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 5:/*Roll*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero,-sig,zero,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,-sig,zero,
                                                     atm21,atm22,atm23);
   	      break;
    	    case 6:/*Twist*/	       
                   sig = mb(one,sig_rot,seed);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero,-sig,
                                                     atm11,atm12,atm13);
                   rotate_entry(system,ex,ey,ez,e0,zero,zero,-sig,
                                                     atm21,atm22,atm23);
   	      break;
            }/*switch*/
    }/*move_global*/

/*-------------------------------------------------------------------------*/
/* V) DEBUG                                                                */

#ifdef DEBUG
  getchar();
#endif

/*-------------------------------------------------------------------------*/
}/*end routine*/
/*-------------------------------------------------------------------------*/
