#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Pos::move_notors(System &system)
/*-------------------------------------------------------------------------*/
/*  Tranlates and/or rotates molecules that do not have torsional          */
/*  degrees of freedom                                                     */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/
                                                                
 int iatm, ires, imol, imol_notors;
 int iatm1, iatm2, iatm3;
 double sig_trans, sig_rot, sig_scale;

 General   *general   = &(system.general);
 Tors_defs *tors_defs = &(system.tors_defs);
 Maps      *maps      = &(system.maps);
 Residue   *residue   = system.pos.residue;

 int  nmol_notors      = tors_defs->nmol_notors;
 int   *ind_mol_notors = tors_defs->ind_mol_notors;
 int   *nres_mol       = maps->nres_mol;        
 int  **ind_res_mol    = maps->ind_res_mol;  

 char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Choose the propagation width with respect to torsional sigma         */

 sig_scale = general->prop_notors_sig_scale;
 sig_trans = sig_scale*(general->prop_trans_sig);
 sig_rot   = sig_scale*(general->prop_rot_sig);

/*-------------------------------------------------------------------------*/
/* I) Loop over the basis and move them                                    */

 for (imol_notors=1;imol_notors<=nmol_notors;imol_notors++) {
      imol = ind_mol_notors[imol_notors];
      if (nres_mol[imol]!=1) {
          sprintf(err_msg,"Pos::move_notors:nres_mol[%d]!=1\n"
                          "nres_mol[%d] %d",imol,imol,nres_mol[imol]);
          errorMessage(err_msg);
      }/*endif*/
      ires = ind_res_mol[imol][1];
      if (residue[ires].natom == 3) {
          iatm1 = residue[ires].atom[1];
          iatm2 = residue[ires].atom[2];
          iatm3 = residue[ires].atom[3];
          move_entry(system,iatm1,iatm2,iatm3,sig_trans,sig_rot);
      }
      else if (residue[ires].natom == 1) {
          iatm1 = residue[ires].atom[1];
          move_entry(system,iatm1,sig_trans);
      } else {
 	sprintf(err_msg,"Pos::move_notors:currently only molecules with"
                        "either one or three atoms are supported!");
        errorMessage(err_msg);
      }/*endif*/
 }/*endfor*/

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 
