#include "../include/standard.h"
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG

/*=========================================================================*/
void Pos::move_torcomplbase(System &system)
/*-------------------------------------------------------------------------*/
/*  Translates and rotates the closure base                                */
/*                                                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

 int itorcompl;

 double sig_trans, sig_rot;

 General   *general   = &(system.general);
 Tors_defs *tors_defs = &(system.tors_defs);

 int  ntorcompl       = tors_defs->ntorcompl;

/*-------------------------------------------------------------------------*/
/* 0) Choose the propagation width with respect to torsional sigma         */

 sig_trans =  general->prop_trans_sig;
 sig_rot   =  general->prop_rot_sig;

/*-------------------------------------------------------------------------*/
/* I) Loop over all torcompl                                               */

 for (itorcompl=1;itorcompl<=ntorcompl;itorcompl++) {
      move_torcomplbase(system,itorcompl,sig_trans,sig_rot);
 }/*endfor*/

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 


/*=========================================================================*/
void Pos::move_torcomplbase(System &system, int itorcompl, double sig_trans,
                                                           double sig_rot)
/*-------------------------------------------------------------------------*/
/*  Translates and rotates the closure base                                */
/*                                                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/
                                                                
 int iatm1, iatm2, iatm3, iatm4;

 Tors_defs *tors_defs = &(system.tors_defs);

 int  *ityp_torcompl        = tors_defs->ityp_torcompl;
 int  *natm_torcomplbase    = tors_defs->natm_torcomplbase;
 int **ind_atm_torcomplbase = tors_defs->ind_atm_torcomplbase;

 char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* I) move torcomplbase of itorcompl                                       */
/*    DNA_duplex                                                           */
/*    protein_D6                                                           */


   if (ityp_torcompl[itorcompl]==1) {/*DNA_duplex*/
      if (natm_torcomplbase[itorcompl]==4) {
          iatm1 = ind_atm_torcomplbase[itorcompl][1];
          iatm2 = ind_atm_torcomplbase[itorcompl][2];
          iatm3 = ind_atm_torcomplbase[itorcompl][3];
          iatm4 = ind_atm_torcomplbase[itorcompl][4];
          move_entry(system,iatm1,iatm2,iatm3,sig_trans,sig_rot);
          xi[iatm4] = xi[iatm2] + (xi[iatm2] - xi[iatm3]);
          yi[iatm4] = yi[iatm2] + (yi[iatm2] - yi[iatm3]);
          zi[iatm4] = zi[iatm2] + (zi[iatm2] - zi[iatm3]);
      } else {
 	sprintf(err_msg,"Pos::move_torcomplbase:natm_torcomplbase!=4\n"
                        "natm_torcomplbase[%d] for DNA_duplex: %g",
                                            itorcompl,
                                            natm_torcomplbase[itorcompl]);
        errorMessage(err_msg);
      }/*endif*/
   }/*endif-DNA_duplex*/

   if (ityp_torcompl[itorcompl]==2) {/*protein_D6*/
      if (natm_torcomplbase[itorcompl]==3) {
          iatm1 = ind_atm_torcomplbase[itorcompl][1];
          iatm2 = ind_atm_torcomplbase[itorcompl][2];
          iatm3 = ind_atm_torcomplbase[itorcompl][3];
          move_entry(system,iatm1,iatm2,iatm3,sig_trans,sig_rot);
      } else {
 	sprintf(err_msg,"Pos::move_torcomplbase:natm_torcomplbase!=3\n"
                        "natm_torcomplbase[%d] for protein_D6: %g",
                                            itorcompl,
                                            natm_torcomplbase[itorcompl]);
        errorMessage(err_msg);
      }/*endif*/
   }/*endif-protein_D6*/

/*--------------------------------------------------------------------------*/
}/*                              End                                        */
/*--------------------------------------------------------------------------*/
