#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
void Pos::move_tors(System &system)
/*-------------------------------------------------------------------------*/
/*  Moves system in torsional space                                        */
/*-------------------------------------------------------------------------*/
{/*begin routine*/  
/*-------------------------------------------------------------------------*/
                                                                
 int level,ilevel,iatm,trans_any;
 int clclosed, clsucceed;
 int clrd;                                       /* closure recovery depth */

 General   *general   = &(system.general);
 Tors_defs *tors_defs = &(system.tors_defs);
 int num_tr           =  tors_defs->num_trans; 
 int *ityp_tr         =  tors_defs->ityp_trans; 


/*-------------------------------------------------------------------------*/
/* 0) Move Molecules with no torsional degrees                             */
/*-------------------------------------------------------------------------*/

   move_notors(system);

/*-------------------------------------------------------------------------*/
/* I) Transformation loop                                                  */
/*    Move Closure Basis (if any) After Backward Half                      */  
/*-------------------------------------------------------------------------*/
   
   adjust(this,system,1);

   clrd = 3;
   for (level = num_tr;level >= 1;level--) {/*backward half*/
       /* 1st three steps involve primary tors and group atoms */
       /* i)forward translate */
          translate(this,tors_defs,level,1);  
       /* ii)forward rotate */
          rotate(this,tors_defs,level,1); 
       /* iii)forward resolve */
          resolve(this,tors_defs,level,1);
       /* iv) group atoms */
          group(this,tors_defs,level,1);
       /* v) adjust atoms */
          adjust(this,system,level,1);
       /* vi) propagate torsion */
          switch(ityp_tr[level]){
             case 0 : break;
             case 1 : 
               prop_tors(this,general,tors_defs,level);
               break;
             case 2 : 
               prop_tors(this,general,tors_defs,level);
               break;
             case 3 : break;
             case 4 : break;
	  }/*end switch */
   }/*end of backward half*/ 


   if (general->cblc_defined==1) { 
       move_clbase(system); 
       clos_succeed = 1;
   }
   if (general->torcompl_defined==1) {
       move_torcomplbase(system);
   }



   for (level = 1;level <= num_tr;level++) {/*forward half*/
       /* 0) adjust atoms */
          adjust(this,system,level,-1);
       /* 00) ungroup atoms */
          group(this,tors_defs,level,-1);
       /* i) backward resolve */
          resolve(this,tors_defs,level,-1);
       /* ii) backward  rotate */
          rotate(this,tors_defs,level,-1);
       /* iii) backward translate */
          translate(this,tors_defs,level,-1);
       /* iv) closure         */

#ifdef DEBUG
	  printf("ityp_tr[%d] : %d \n",level,ityp_tr[level]);
          getchar();
#endif
          if (ityp_tr[level]==2) {
             clclosed      =  closure(this,tors_defs,level,-1);
             clsucceed     =  clclosed;
             if ( clclosed == 0 ) {clos_succeed  = 0; break;}
          }/*endif*/

#ifdef DEBUG
	  printf("clclosed %d \n",clclosed);
          getchar();
#endif
   }/*end of forward half*/

   adjust(this,system,-1);

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 
