#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_cgresmodel_3pt(const Maps *maps, const Pos *pos,
                                                        const General *general)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for any model                      */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

   int imol, nmol;
   int ires, iatm;
   int protein, nucleic;
   int ichar, istride;
   int region, region_condition;
   int rb_cblc_stride, rb_scblc_stride;

   char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* I) Define if STRIDE defines rigid bodies for cblc                       */

  if (general->cblc_defined==1) {
      rb_cblc_stride  = 0;
      rb_scblc_stride = 0;
      if (general->stride_defined==1) {
	  for (istride=1;istride<=nstride;istride++) {
	      for (ichar=1;ichar<=strlen(&stride[istride][0])-1;ichar++) {
		   if (stride[istride][ichar] == 'R') {
		       if (cblc[0] == '>') {rb_cblc_stride  = 1;}
                       if (cblc[0] == '~') {rb_scblc_stride = 1;}
                   }/*endif*/
              }/*endfor*/
          }/*endfor*/
      }/*endif*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* II) Decide on different torsional 3pt setup options                     */
/*     if (region == 0)                                                    */
/*       setup_3pt_rb_cblc,setup_3pt_cblc,setup_3pt has nmol == 1 checks   */
/*       setup_3pt_rb_scblc can handle nmol == 1 and nmol > 1              */
/*     if (region == 1)                                                    */
/*       CBLC ~ needed, setup_3pt_rb_scblc_mulchn_region handles nmol >= 1 */             


  if (general->region_topology_defined == 1)
       {region = 1;}
  else {region = 0;}

  if (region == 0) {
  if (general->cblc_defined==1) {
      if (rb_cblc_stride==1) {
	 setup_3pt_rb_cblc(maps,pos,general);
      } else if (rb_scblc_stride==1) {
      	 setup_3pt_rb_scblc(maps,pos,general);
      } else {
         setup_3pt_cblc(maps,pos,general);
      }/*endif*/
  } else {
        setup_3pt(maps,pos,general);
  }}/*endif*/

  if (region == 1) {
      region_condition = 0;
      if (cblc[0] == '~') {region_condition++;}
      if (general->cblc_defined==1) {region_condition++;}
      if (region_condition != 2) {
          sprintf(err_msg,"//region_database_file option only works with \n"
                          "CBLC ~\n");
          errorMessage(err_msg);
      }/*endif*/
      nucleic = 0;
      protein = 0;
      nmol    = maps->nmol;
      for (imol=1;imol<=nmol;imol++) {
          ires    = maps->ind_res_mol[imol][1];          
          iatm    = pos->residue[ires].branch_root_f[0][0];
          if (strcasecmp(maps->upac_atm_label[iatm],"CA")==0){protein = 1;}  
          if (strcasecmp(maps->upac_atm_label[iatm],"P")==0) {nucleic = 1;}
     }/*endfor*/
      if ((protein == 1) && (nucleic == 0)) {
         setup_3pt_region(maps,pos,general);
      }/*endif*/
      if ((protein == 1) && (nucleic == 1)) {
         setup_3pt_region_gen(maps,pos,general);
      }/*endif*/
  }/*endfor*/


/*-------------------------------------------------------------------------*/
}/*end routine*/
/*=========================================================================*/

