#include "../../include/standard.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
 void Tors_defs::setup_3pt_rb_scblc(const Maps *maps, const Pos *pos,
                                                      const General *general)
/*-------------------------------------------------------------------------*/
/*  Sets up all the torsional angles for cgres_model_3pt_rb_cblc           */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int ires, itrans, iclbase, iclos;
  int iatm, iiatm, ibranch;

  int istride, nres;
  int iseg, rseg, cseg, nrseg, ncseg;

  int *ind_rseg_central;
  int *ind_rseg_begin, *ind_rseg_end, *rseg_length;
  int *ind_cseg_begin, *ind_cseg_end, *cseg_length;

  int odd, even;
  int ind_CA, ind_O, ind_CMA;
  int ind_prevres_CA, ind_prevres_O, ind_prevres_CMA;
  int ind_nextres_CA, ind_nextres_O, ind_nextres_CMA;
  int ind_nnextres_CA, ind_nnextres_O, ind_nnextres_CMA;

  double dx, dy, dz;
  double *x = pos->x;
  double *y = pos->y;
  double *z = pos->z;

  char  *atm_label;
  char   err_msg[LINE_NCHARMAX];

  int    nmol  =  maps->nmol;

/*-------------------------------------------------------------------------*/
/* 0) Only one molecule is supported                                       */

  if (nmol!=1) {
      setup_3pt_rb_scblc_mulchn(maps,pos,general);
      return;
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Count the number of rigid and flexible segments in STRIDE            */
/*    and allocate arrays related to segment definitions                   */

  nres = strlen(&stride[1][0])-1;

  ind_rseg_central = new int[nres] - 1;
  ind_rseg_begin   = new int[nres] - 1;
  ind_rseg_end     = new int[nres] - 1;
      rseg_length  = new int[nres] - 1;

  ind_cseg_begin   = new int[nres] - 1;
  ind_cseg_end     = new int[nres] - 1;
      cseg_length  = new int[nres] - 1;

  nrseg = 0;
  ncseg = 0;
  rseg  = 0;
  cseg  = 0;
  for (ires=1;ires<=nres;ires++) {
       if ((stride[1][ires] == 'R') && (rseg == 0)) {
            nrseg++;
            ind_rseg_begin[nrseg] = ires;
            rseg = 1;
            cseg = 0;
       }/*endif*/
       else if ((stride[1][ires] == 'C') && (cseg == 0)) {
            ncseg++;
            ind_cseg_begin[ncseg] = ires;
            cseg = 1;
            rseg = 0;
       }/*endif*/
       if (ires != nres) {
           if ((stride[1][ires] == 'R') && (stride[1][ires+1] != 'R')) {
                ind_rseg_end[nrseg] = ires;
           }/*endif*/
           if ((stride[1][ires] == 'C') && (stride[1][ires+1] != 'C')) {
                ind_cseg_end[ncseg] = ires;
           }/*endif*/
       }/*endif*/
       if (ires == nres) {
           if (stride[1][ires] == 'R') {
                ind_rseg_end[nrseg] = ires;
           }/*endif*/
           if (stride[1][ires] == 'C') {
                ind_cseg_end[ncseg] = ires;
           }/*endif*/
       }/*endif*/
       if ((stride[1][ires] != 'R') && (stride[1][ires] != 'C'))  {
             sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc\n"
                             "'STRIDE >' use either 'C' or 'R' only");
             errorMessage(err_msg);
       }/*endif*/
  }/*endfor*/

  for (iseg=1;iseg<=nrseg;iseg++) {
       rseg_length[iseg] = ind_rseg_end[iseg] - ind_rseg_begin[iseg] + 1;
       if (rseg_length[iseg] < 3) {
           sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc\n"
                    "'STRIDE > %d. 'R' segment shorter than 3 residues",iseg);
           errorMessage(err_msg);
       }/*endif*/
       odd = 0; even = 0;
       if ( (rseg_length[iseg] % 2) == 0 ) {even = 1;}
       else {odd = 1;}
       if (even == 1) {
           ind_rseg_central[iseg] = rseg_length[iseg]/2
                                  + ind_rseg_begin[iseg] - 1;
       }/*endif*/
       if (odd  == 1) {
           ind_rseg_central[iseg] = (rseg_length[iseg]-1)/2
                                  +  ind_rseg_begin[iseg];
       }/*endif*/
  }/*endfor*/

  for (iseg=1;iseg<=ncseg;iseg++) { 
       cseg_length[iseg] = ind_cseg_end[iseg] - ind_cseg_begin[iseg] + 1;
  }/*endfor*/

#ifdef DEBUG
  printf("nrseg %d, ncseg %d\n",nrseg,ncseg);
  for (iseg=1;iseg<=nrseg;iseg++) { 
    printf("ind_rseg_begin[%d] %d\n",iseg,ind_rseg_begin[iseg]);
    printf("ind_rseg_end[%d] %d\n",iseg,ind_rseg_end[iseg]);
    printf("ind_rseg_central[%d] %d\n",iseg,ind_rseg_central[iseg]);
    printf("    rseg_length[%d] %d\n",iseg,rseg_length[iseg]);
  }/*endfor*/

  for (iseg=1;iseg<=ncseg;iseg++) { 
    printf("ind_cseg_begin[%d] %d\n",iseg,ind_cseg_begin[iseg]);
    printf("ind_cseg_end[%d] %d\n",iseg,ind_cseg_end[iseg]);
    printf("    cseg_length[%d] %d\n",iseg,cseg_length[iseg]);
  }/*endfor*/
  getchar();
#endif

/*-------------------------------------------------------------------------*/
/* II) Get the number of transformations, closures and closure bases       */

  num_trans  = 0;
  num_clbase = 0;
  num_clos   = 0;
  for (iseg=1;iseg<=nrseg;iseg++) {
       num_trans  += 2*(rseg_length[iseg]-1);
       num_clbase += 1;
  }/*endfor*/

  for (iseg=1;iseg<=ncseg;iseg++) {
    if (ind_cseg_begin[iseg]==1) {/*head C*/
       num_trans  += 2*cseg_length[iseg] + 1;
    } 
    else if (ind_cseg_end[iseg]==nres) {/*tail C*/
       num_trans  += 2*cseg_length[iseg];
    }
    else {/*bulk C*/
       num_trans  += 3*cseg_length[iseg];
       num_clos   += 2*cseg_length[iseg];
    }/*endif*/
  }/*endfor*/


#ifdef DEBUG
    printf("num_trans %d\n",num_trans);
    printf("num_clbase %d\n",num_clbase);
    printf("num_clos %d\n",num_clos);
    getchar();
#endif

/*-------------------------------------------------------------------------*/
/* III)                                                                    */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* III.A) Allocate variables                                               */

  res_id         =  new int[num_trans]-1;
  ityp_trans     =  new int[num_trans]-1;
  off_trans      =  new int[num_trans]-1;
  natm_trans     =  new int[num_trans]-1;
  ind_clos_trans =  new int[num_trans]-1;
  prop_sig_scale =  new double[num_trans]-1;
  secondary_only =  new int[num_trans]-1;

  natm_clos          =  new int[num_clos]-1;
  ityp_clos          =  new int[num_clos]-1;
  ityp_clos_postprop =  new int[num_clos]-1;

  natm_clbase     =  new int[num_clbase]-1;
  ind_clbase_free =  new int[num_clbase]-1;
  res_id_clbase   =  new int[num_clbase]-1;
  freeze_clbase   =  new int[num_clbase]-1;

  ind_trans      =  cmall_int_mat(1, num_trans, 1, NATM_TRANS_MAX);
  ind_prim_tors  =  cmall_int_mat(1, num_trans, 1, 4);
  ind_atm_clos   =  cmall_int_mat(1, num_clos, 1, NATM_CLOS_MAX);
  ind_atm_clbase =  cmall_int_mat(1, num_clbase, 1, NATM_CLBASE_MAX);
  const_clos     =  cmall_mat(1, num_clos, 1, NATM_CLOS_MAX);

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* III.B) Initialize default variables                                     */

  for (itrans=1;itrans<=num_trans;itrans++) {
       ityp_trans[itrans]     = 1;
       off_trans[itrans]      = 0;
       secondary_only[itrans] = 0;
       ind_clos_trans[itrans] = 0;
       prop_sig_scale[itrans] = 1.0;
  }/*endfor*/

  num_clbase_free = num_clbase;
  for (iclbase=1;iclbase<=num_clbase;iclbase++) {
       freeze_clbase[iclbase] = 0;
       ind_clbase_free[iclbase] = iclbase;
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* IV.A)  Decide if 3pt protein or nucleic acid                            */
  int nucleic = 0;
  int protein = 0;
  iatm = pos->residue[1].branch_root_f[0][0];
  if (strcasecmp(maps->upac_atm_label[iatm],"CA")==0){protein = 1;}  
  if (strcasecmp(maps->upac_atm_label[iatm],"P")==0) {nucleic = 1;}
#ifdef DEBUG
      if (protein) {printf("This is a protein in 3pt representation\n");}
      if (nucleic) {printf("This is a nucleic acid in 3pt representation\n");}
      getchar();
#endif

/*-------------------------------------------------------------------------*/
/* IV.B) Define the group of atoms to be transformed for proteins          */

 if (protein) {

  itrans  = 0;
  iclbase = 0;
  iclos   = 0;

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.C.1) Run through rigid segments                                      */

  for (iseg=1;iseg<=nrseg;iseg++) {

    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
    /* IV.C.1)a Base                                                       */
    ires=ind_rseg_central[iseg];
    for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
         iiatm     =  pos->residue[ires].atom[iatm];
         atm_label =  maps->upac_atm_label[iiatm];

         if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
         if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
         if (strcasecmp(atm_label,"CMA")==0) {ind_CMA = iiatm;}
    }/*endfor*/
    iclbase++;
    if (iclbase>num_clbase) {
	     sprintf(err_msg,"Tors_defs:setup_3pt_rb_cblc:\n"
                        "iclbase>num_clbase");
          errorMessage(err_msg);
    }/*endif*/
    res_id_clbase[iclbase] = ires;
    natm_clbase[iclbase] = 3;
    ind_atm_clbase[iclbase][1] = ind_CMA;         /*CMA*/
    ind_atm_clbase[iclbase][2] = ind_CA;          /*CA*/
    ind_atm_clbase[iclbase][3] = ind_O;           /*O*/   

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.1)b Forward                                                     */
   for (ires=ind_rseg_central[iseg]+1;ires<=ind_rseg_end[iseg];ires++) {

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     =  pos->residue[ires].atom[iatm];
            atm_label =  maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"CMA")==0) {ind_CMA = iiatm;}
       }/*endfor*/
       
       for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
            iiatm     = pos->residue[ires-1].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"CA")==0) {ind_prevres_CA  = iiatm;}
            if (strcasecmp(atm_label,"O")==0)  {ind_prevres_O   = iiatm;}
            if (strcasecmp(atm_label,"CMA")==0){ind_prevres_CMA = iiatm;}
       }/*endfor*/
      
       itrans++;
       if (itrans>num_trans) {
	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                           "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_CA;              /*CA*/
       ind_prim_tors[itrans][4] = ind_CA;          /*CA*/
       ind_prim_tors[itrans][3] = ind_prevres_O;   /*O*/
       ind_prim_tors[itrans][2] = ind_prevres_CA;  /*CA*/
       ind_prim_tors[itrans][1] = ind_prevres_CMA; /*CMA*/

       itrans++;
       if (itrans>num_trans) {
  	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                           "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
       res_id[itrans] = ires;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1] = ind_O;               /*O*/
       ind_trans[itrans][2] = ind_CMA;             /*CMA*/
       ind_prim_tors[itrans][4] = ind_O;           /*O*/
       ind_prim_tors[itrans][3] = ind_CA;          /*CA*/
       ind_prim_tors[itrans][2] = ind_prevres_O;   /*O*/
       ind_prim_tors[itrans][1] = ind_prevres_CA;  /*CA*/

   }/*endfor-ires-forward*/

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.1)c Backward                                                    */

   for (ires=ind_rseg_central[iseg]-1;ires>=ind_rseg_begin[iseg];ires--) {

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     =  pos->residue[ires].atom[iatm];
            atm_label =  maps->upac_atm_label[iiatm];

            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"CMA")==0) {ind_CMA = iiatm;}
       }/*endfor*/

       for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
            iiatm     = pos->residue[ires+1].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"CA")==0) {ind_nextres_CA  = iiatm;}
            if (strcasecmp(atm_label,"O")==0)  {ind_nextres_O   = iiatm;}
            if (strcasecmp(atm_label,"CMA")==0){ind_nextres_CMA = iiatm;}
       }/*endfor*/

       itrans++;
       if (itrans>num_trans) {
	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_O;               /*O*/
       ind_prim_tors[itrans][4] = ind_O;           /*O*/
       ind_prim_tors[itrans][3] = ind_nextres_CA;  /*CA*/
       ind_prim_tors[itrans][2] = ind_nextres_O;   /*O*/
       ind_prim_tors[itrans][1] = ind_nextres_CMA; /*CMA*/

       itrans++;
       if (itrans>num_trans) {
  	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                           "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
       res_id[itrans] = ires;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1] = ind_CA;              /*CA*/
       ind_trans[itrans][2] = ind_CMA;             /*CMA*/
       ind_prim_tors[itrans][4] = ind_CA;          /*CA*/
       ind_prim_tors[itrans][3] = ind_O;           /*O*/
       ind_prim_tors[itrans][2] = ind_nextres_CA;  /*CA*/
       ind_prim_tors[itrans][1] = ind_nextres_O;   /*O*/

   }/*endfor-ires-backward*/

  }/*endfor-rigid segments*/


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.C.2) Run through coil segments                                       */

  for (iseg=1;iseg<=ncseg;iseg++) {   
 
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  /* IV.C.2)a Head                                                        */
    if (ind_cseg_begin[iseg]==1) {
      for (ires=ind_cseg_end[iseg];ires>=ind_cseg_begin[iseg];ires--) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
  
              if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0) {ind_CMA = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
              iiatm     = pos->residue[ires+1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0) {ind_nextres_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)  {ind_nextres_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0){ind_nextres_CMA = iiatm;}
         }/*endfor*/
         for (iatm=1;iatm<=pos->residue[ires+2].natom;iatm++) {
              iiatm     = pos->residue[ires+2].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0) {ind_nnextres_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)  {ind_nnextres_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0){ind_nnextres_CMA = iiatm;}
         }/*endfor*/


         if (ires==ind_cseg_end[iseg]) {
            itrans++;
            if (itrans>num_trans) {
       	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                                "itrans>num_trans");
                  errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 1;
            ind_trans[itrans][1] = ind_O;               /*O*/
            ind_prim_tors[itrans][4] = ind_O;           /*O*/
            ind_prim_tors[itrans][3] = ind_nextres_CA;  /*CA*/
            ind_prim_tors[itrans][2] = ind_nextres_O;   /*O*/
            ind_prim_tors[itrans][1] = ind_nextres_CMA; /*CMA*/
	 }/*endif*/

         if (ires!=ind_cseg_end[iseg]) {
            itrans++;
            if (itrans>num_trans) {
       	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                                "itrans>num_trans");
                  errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 2;
            ind_trans[itrans][1] = ind_O;               /*O*/
            ind_trans[itrans][2] = ind_nextres_CMA;     /*CMA*/
            ind_prim_tors[itrans][4] = ind_O;           /*O*/
            ind_prim_tors[itrans][3] = ind_nextres_CA;  /*CA*/
            ind_prim_tors[itrans][2] = ind_nextres_O;   /*O*/
            ind_prim_tors[itrans][1] = ind_nnextres_CA; /*CA*/
	 }/*endif*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_CA;              /*CA*/
         ind_prim_tors[itrans][4] = ind_CA;          /*CA*/
         ind_prim_tors[itrans][3] = ind_O;           /*O*/
         ind_prim_tors[itrans][2] = ind_nextres_CA;  /*CA*/
         ind_prim_tors[itrans][1] = ind_nextres_O;   /*O*/

         if (ires==ind_cseg_begin[iseg]) {
            itrans++;
            if (itrans>num_trans) {
       	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                                "itrans>num_trans");
                  errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 1;
            ind_trans[itrans][1] = ind_CMA;             /*CMA*/
            ind_prim_tors[itrans][4] = ind_CMA;         /*CMA*/
            ind_prim_tors[itrans][3] = ind_CA;          /*CA*/
            ind_prim_tors[itrans][2] = ind_O;           /*O*/
            ind_prim_tors[itrans][1] = ind_nextres_CA;  /*CA*/
	 }/*endif*/
      }/*endfor-ires*/
    }/*endif-Head*/

  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.2)b Tail                                                        */
    if (ind_cseg_end[iseg]==nres) {
      for (ires=ind_cseg_begin[iseg];ires<=ind_cseg_end[iseg];ires++) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0) {ind_CMA = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
              iiatm     = pos->residue[ires-1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0) {ind_prevres_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)  {ind_prevres_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0){ind_prevres_CMA = iiatm;}
         }/*endfor*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_CA;              /*CA*/
         ind_prim_tors[itrans][4] = ind_CA;          /*CA*/
         ind_prim_tors[itrans][3] = ind_prevres_O;   /*O*/
         ind_prim_tors[itrans][2] = ind_prevres_CA;  /*CA*/
         ind_prim_tors[itrans][1] = ind_prevres_CMA; /*CMA*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 2;
         ind_trans[itrans][1] = ind_O;               /*O*/
         ind_trans[itrans][2] = ind_CMA;             /*CMA*/
         ind_prim_tors[itrans][4] = ind_O;           /*O*/
         ind_prim_tors[itrans][3] = ind_CA;          /*CA*/
         ind_prim_tors[itrans][2] = ind_prevres_O;   /*O*/
         ind_prim_tors[itrans][1] = ind_prevres_CA;  /*CA*/

      }/*endfor-ires*/
    }/*endif-Tail*/


  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.2)c Bulk                                                        */
    if ((ind_cseg_begin[iseg]!=1)&&(ind_cseg_end[iseg]!=nres)) {


#ifdef DEBUG
  printf("Bulk coil segment setup\n");
  getchar();
#endif

      for (ires=ind_cseg_begin[iseg];ires<=ind_cseg_end[iseg];ires++) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0) {ind_CMA = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
              iiatm     = pos->residue[ires-1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0) {ind_prevres_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)  {ind_prevres_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0){ind_prevres_CMA = iiatm;}
         }/*endfor*/
         for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
              iiatm     = pos->residue[ires+1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0) {ind_nextres_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)  {ind_nextres_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0){ind_nextres_CMA = iiatm;}
         }/*endfor*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_CA;              /*CA*/
         ind_prim_tors[itrans][4] = ind_CA;          /*CA*/
         ind_prim_tors[itrans][3] = ind_prevres_O;   /*O*/
         ind_prim_tors[itrans][2] = ind_prevres_CA;  /*CA*/
         ind_prim_tors[itrans][1] = ind_prevres_CMA; /*CMA*/


         /*____________________/closure/____________________*/
         iclos++;
         if (iclos>num_clos) {
    	    sprintf(err_msg,"Tors_defs::setup_cgresmodel_3pt_cblc:\n"
                            "iclos>num_clos");
            errorMessage(err_msg);
         }/*endif*/
         ind_clos_trans[itrans] = iclos;
         if (ires==ind_cseg_begin[iseg]) {
             ityp_clos[iclos]       = 0;             /*cblc*/
	 } else {
             ityp_clos[iclos]       = 5;             /*scblc*/
         }/*endif*/
         natm_clos[iclos]       = 5;
         ind_atm_clos[iclos][5] = ind_O;            /*O*/
         ind_atm_clos[iclos][4] = ind_CA;           /*CA*/
         ind_atm_clos[iclos][3] = ind_prevres_O;    /*O*/
         ind_atm_clos[iclos][2] = ind_prevres_CA;   /*CA*/
         ind_atm_clos[iclos][1] = ind_prevres_CMA;  /*CMA*/
         dx = x[ind_O] - x[ind_CA];
         dy = y[ind_O] - y[ind_CA];
         dz = z[ind_O] - z[ind_CA];
         const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
         dx = x[ind_prevres_O] - x[ind_CA];
         dy = y[ind_prevres_O] - y[ind_CA];
         dz = z[ind_prevres_O] - z[ind_CA];
         const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/


         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         if (ires==ind_cseg_end[iseg]) {
             ityp_trans[itrans] = 2;
         } else {
             ityp_trans[itrans] = 1;
         }/*endif*/
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_O;               /*O*/
         ind_prim_tors[itrans][4] = ind_O;           /*O*/
         ind_prim_tors[itrans][3] = ind_CA;          /*CA*/
         ind_prim_tors[itrans][2] = ind_prevres_O;   /*O*/
         ind_prim_tors[itrans][1] = ind_prevres_CA;  /*CA*/


         /*____________________/closure/____________________*/
         iclos++;
         if (iclos>num_clos) {
    	    sprintf(err_msg,"Tors_defs::setup_cgresmodel_3pt_cblc:\n"
                            "iclos>num_clos");
            errorMessage(err_msg);
         }/*endif*/
         ind_clos_trans[itrans] = iclos;
         ityp_clos[iclos]       = 5;             /*scblc*/
         natm_clos[iclos]       = 5;
         ind_atm_clos[iclos][5] = ind_nextres_CA;   /*CA*/
         ind_atm_clos[iclos][4] = ind_O;            /*O*/
         ind_atm_clos[iclos][3] = ind_CA;           /*CA*/
         ind_atm_clos[iclos][2] = ind_prevres_O;    /*O*/
         ind_atm_clos[iclos][1] = ind_prevres_CA;   /*CA*/
         dx = x[ind_nextres_CA] - x[ind_O];
         dy = y[ind_nextres_CA] - y[ind_O];
         dz = z[ind_nextres_CA] - z[ind_O];
         const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
         dx = x[ind_CA] - x[ind_O];
         dy = y[ind_CA] - y[ind_O];
         dz = z[ind_CA] - z[ind_O];
         const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/

      }/*endfor-ires-backbone*/

      for (ires=ind_cseg_begin[iseg];ires<=ind_cseg_end[iseg];ires++) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0) {ind_CMA = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
              iiatm     = pos->residue[ires-1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0) {ind_prevres_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)  {ind_prevres_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0){ind_prevres_CMA = iiatm;}
         }/*endfor*/
         for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
              iiatm     = pos->residue[ires+1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
              if (strcasecmp(atm_label,"CA")==0) {ind_nextres_CA  = iiatm;}
              if (strcasecmp(atm_label,"O")==0)  {ind_nextres_O   = iiatm;}
              if (strcasecmp(atm_label,"CMA")==0){ind_nextres_CMA = iiatm;}
         }/*endfor*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         res_id[itrans] = ires;
         ityp_trans[itrans] = 0;
         secondary_only[itrans] = 2*itrans-2*cseg_length[iseg]; 
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_CMA;              /*CMA*/
         ind_prim_tors[itrans][4] = ind_CMA;          /*CMA*/
         ind_prim_tors[itrans][3] = ind_CA;           /*CA*/
         ind_prim_tors[itrans][2] = ind_prevres_O;    /*O*/
         ind_prim_tors[itrans][1] = ind_prevres_CA;   /*CA*/
      }/*endfor-ires-CMA*/


    }/*endif-Bulk*/
  }/*endfor-coil segments*/
 }/*end if-protein*/


/*-------------------------------------------------------------------------*/
/* III.C) Define the group of atoms to be transformed for nucleic acids    */

 if (nucleic) {
    sprintf(err_msg,"Tors_defs::setup_cgresmodel_3pt_cblc\n"
                    "currently nucleic acids are not supported");
    errorMessage(err_msg);
 }/*end if*/


/*-------------------------------------------------------------------------*/
/* IV) If stride is defined in .pdb, only move basis                       */
/*       along 'C' secondary structure type residue                        */
 
   if (general->stride_defined==1) {
     if (nstride!=1) {
       sprintf(err_msg, "Tors_defs::setup_cgresmodel_3pt_rb_cblc: nstride!=1");
       errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[1][0]))-1!=pos->nres) {
       sprintf(err_msg, "Tors_defs::setup_cgresmodel_3pt_rb_cblc:\n" 
                        "corrupted pos.stride \n");
       errorMessage(err_msg);
     }/*endif*/
//     for (iclbase=1;iclbase<=num_clbase;iclbase++) {
//          freeze_clbase[iclbase] = 1;
//          if (stride[1][res_id_clbase[iclbase]] == 'C') {
// 	     freeze_clbase[iclbase] = 0;
//          }/*endif*/
//     }/*endfor*/
   }/*end if*/

/*-------------------------------------------------------------------------*/
/* V) Setup rotate_utils                                                 */

  rotate_utils.setup();

  
/*-------------------------------------------------------------------------*/
/* VI) Setup ityp_clos_potprop if postprop_minimize is on                  */

  if (general->postprop_minimize==1) {
      for (iclos=1;iclos<=num_clos;iclos++) {
  	   ityp_clos_postprop[iclos] = ityp_clos[iclos];
      }/*endfor*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* VII) Debug                                                              */

#ifdef DEBUG
  printf("############################ All Torsions ############################\n");
  write_alldata(maps);
#endif

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/
