#include "../../include/standard.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG_OFF

/*=========================================================================*/
 void Tors_defs::setup_3pt_region_gen(const Maps *maps, const Pos *pos,
                                                    const General *general)
/*-------------------------------------------------------------------------*/
/*  Sets up all the torsional angles for cgres_model_3pt_rb_cblc           */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int ires, itrans, iclbase, iclos;
  int iatm, iiatm, ibranch, imol, jmol;

  int istride, nres;
  int iseg, rseg, cseg;
  int nres_global;

  int  *nrseg, *ncseg;
  int  *ires_offset;
  int **ind_rseg_central;
  int **ind_rseg_begin, **ind_rseg_end, **rseg_length;
  int **ind_cseg_begin, **ind_cseg_end, **cseg_length;

  int odd, even;
  int ind_BRF, ind_BRS, ind_BRFS;
  int ind_prevres_BRF, ind_prevres_BRS, ind_prevres_BRFS;
  int ind_nextres_BRF, ind_nextres_BRS, ind_nextres_BRFS;
  int ind_nnextres_BRF, ind_nnextres_BRS, ind_nnextres_BRFS;
  int nucleic, protein;
  int chainid_found;

  double dx, dy, dz;
  double *x = pos->x;
  double *y = pos->y;
  double *z = pos->z;

  char   chainid;
  char  *atm_label;

  int    nmol        = maps->nmol;
  char **mol_chainid = maps->mol_chainid;

  char   err_msg[LINE_NCHARMAX];

  printf("Call region_gen\n");

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.A) Get information about molecules treated with closure               */
/*      Currently supported:                                               */ 
/*      Constant Bond Length Closure (CBLC)                                */

  if (general->cblc_defined==0) {nmol_cblc = 0;}    
  if (general->cblc_defined==1) {    
      nmol_cblc    = strlen(&(cblc[0])) - 1;
      ind_mol_cblc = new int[nmol_cblc] - 1;
      for (imol=1;imol<=nmol_cblc;imol++) {
	   chainid = cblc[imol];
           for (jmol=1;jmol<=nmol;jmol++) {
	        if (mol_chainid[jmol][0] == chainid) {
		    ind_mol_cblc[imol] = jmol;
                }/*endif*/
           }/*endfor*/
      }/*endfor*/
  }/*endif*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.B) All molecules should be in closure                                 */

  if (general->cblc_defined==1) {
      nmol_cblc    = strlen(&(cblc[0])) - 1;
      ind_mol_cblc = new int[nmol_cblc] - 1;
      for (imol=1;imol<=nmol_cblc;imol++) {
	   chainid = cblc[imol];
           chainid_found = 0;
           for (jmol=1;jmol<=nmol;jmol++) {
	        if (mol_chainid[jmol][0] == chainid) {
		    ind_mol_cblc[imol] = jmol;
                    chainid_found = 1;
                }/*endif*/
           }/*endfor*/
           if (chainid_found==0) {
	       sprintf(err_msg,"'CBLC >' There is no chain with id %s\n",chainid);
               errorMessage(err_msg);
           }/*endif*/
      }/*endfor*/
      if (nmol_cblc != nmol) {
          sprintf(err_msg,"In the CBLC treatments of the 3pt coarse grain model\n"
                          "All molecules should be treated as CBLC");
          errorMessage(err_msg);
      }/*endif*/
  }/*endif*/


/*-------------------------------------------------------------------------*/
/* 0.C) Get information about STRIDE's                                     */
/*      If STRIDE is defined all molecules should possess STRIDE           */

     if (general->stride_defined==1) {
       if (nmol!=nstride) {
        sprintf(err_msg,"# of STRIDE does not match the # of molecules");
        errorMessage(err_msg);
       }/*endif*/
       for (imol=1;imol<=nmol;imol++) {       
          if (strlen(&(stride[imol][0]))-1!=maps->nres_mol[imol]) {
              sprintf(err_msg, "Tors_defs::setup_cgresmodel_3pt_rb_scblc:\n" 
		               "%d. STRIDE is corrupted\n",imol);
              errorMessage(err_msg);
          }/*endif*/
       }/*endfor*/
     }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Count the number of rigid and flexible segments in STRIDES           */
/*    and allocate arrays related to segment definitions                   */

  nrseg            = new int[nstride] - 1;
  ncseg            = new int[nstride] - 1;

  ires_offset      = new int[nstride] - 1;

  ind_rseg_central = new int*[nstride] - 1;
  ind_rseg_begin   = new int*[nstride] - 1;
  ind_rseg_end     = new int*[nstride] - 1;
      rseg_length  = new int*[nstride] - 1;

  ind_cseg_begin   = new int*[nstride] - 1;
  ind_cseg_end     = new int*[nstride] - 1;
      cseg_length  = new int*[nstride] - 1;

 
 nres_global = 0;
 for (istride=1;istride<=nstride;istride++) {
  nres = strlen(&stride[istride][0])-1;

  ind_rseg_central[istride] = new int[nres] - 1;
  ind_rseg_begin[istride]   = new int[nres] - 1;
  ind_rseg_end[istride]     = new int[nres] - 1;
      rseg_length[istride]  = new int[nres] - 1;

  ind_cseg_begin[istride]   = new int[nres] - 1;
  ind_cseg_end[istride]     = new int[nres] - 1;
      cseg_length[istride]  = new int[nres] - 1;

  ires_offset[istride]      = nres_global;
  nres_global              += nres; 

  nrseg[istride] = 0;
  ncseg[istride] = 0;
  rseg  = 0;
  cseg  = 0;
  for (ires=1;ires<=nres;ires++) {
       if ((stride[istride][ires] == 'R') && (rseg == 0)) {
            nrseg[istride]++;
            ind_rseg_begin[istride][nrseg[istride]] = ires + ires_offset[istride];
            rseg = 1;
            cseg = 0;
       }/*endif*/
       else if ((stride[istride][ires] == 'C') && (cseg == 0)) {
            ncseg[istride]++;
            ind_cseg_begin[istride][ncseg[istride]] = ires + ires_offset[istride];
            cseg = 1;
            rseg = 0;
       }/*endif*/
       if (ires != nres) {
           if ((stride[istride][ires] == 'R') && (stride[istride][ires+1] != 'R')) {
                ind_rseg_end[istride][nrseg[istride]] = ires + ires_offset[istride];
           }/*endif*/
           if ((stride[istride][ires] == 'C') && (stride[istride][ires+1] != 'C')) {
                ind_cseg_end[istride][ncseg[istride]] = ires + ires_offset[istride];
           }/*endif*/
       }/*endif*/
       if (ires == nres) {
           if (stride[istride][ires] == 'R') {
                ind_rseg_end[istride][nrseg[istride]] = ires + ires_offset[istride];
           }/*endif*/
           if (stride[istride][ires] == 'C') {
                ind_cseg_end[istride][ncseg[istride]] = ires + ires_offset[istride];
           }/*endif*/
       }/*endif*/
       if ((stride[istride][ires] != 'R') && (stride[istride][ires] != 'C'))  {
             sprintf(err_msg,"Tors_defs::setup_3pt_rb_scblc\n"
		     "%d. 'STRIDE >' use either 'C' or 'R' only",istride);
             errorMessage(err_msg);
       }/*endif*/
  }/*endfor*/

  for (iseg=1;iseg<=nrseg[istride];iseg++) {
       rseg_length[istride][iseg] = ind_rseg_end[istride][iseg] 
                                  - ind_rseg_begin[istride][iseg] + 1;
       if (rseg_length[istride][iseg] < 3) {
           sprintf(err_msg,"Tors_defs::setup_3pt_rb_scblc\n"
		   "%d. 'STRIDE >' %d. 'R' segment shorter than 3 residues",istride,
                                                                              iseg);
           errorMessage(err_msg);
       }/*endif*/
       odd = 0; even = 0;
       if ( (rseg_length[istride][iseg] % 2) == 0 ) {even = 1;}
       else {odd = 1;}
       if (even == 1) {
           ind_rseg_central[istride][iseg] = rseg_length[istride][iseg]/2
                                           + ind_rseg_begin[istride][iseg] - 1;
       }/*endif*/
       if (odd  == 1) {
           ind_rseg_central[istride][iseg] = (rseg_length[istride][iseg]-1)/2
                                           +  ind_rseg_begin[istride][iseg];
       }/*endif*/
  }/*endfor*/

  for (iseg=1;iseg<=ncseg[istride];iseg++) { 
       cseg_length[istride][iseg] = ind_cseg_end[istride][iseg] 
                                  - ind_cseg_begin[istride][iseg] + 1;
  }/*endfor*/

#ifdef DEBUG
  printf("istride %d\n",istride);
  printf("nrseg %d, ncseg %d\n",nrseg[istride],ncseg[istride]);
  for (iseg=1;iseg<=nrseg[istride];iseg++) {
    printf("ind_rseg_begin[%d] %d\n",iseg,ind_rseg_begin[istride][iseg]);
    printf("ind_rseg_end[%d] %d\n",iseg,ind_rseg_end[istride][iseg]);
    printf("ind_rseg_central[%d] %d\n",iseg,ind_rseg_central[istride][iseg]);
    printf("    rseg_length[%d] %d\n",iseg,rseg_length[istride][iseg]);
  }/*endfor*/

  for (iseg=1;iseg<=ncseg[istride];iseg++) {
    printf("ind_cseg_begin[%d] %d\n",iseg,ind_cseg_begin[istride][iseg]);
    printf("ind_cseg_end[%d] %d\n",iseg,ind_cseg_end[istride][iseg]);
    printf("    cseg_length[%d] %d\n",iseg,cseg_length[istride][iseg]);
  }/*endfor*/

  getchar();
#endif
 }/*endfor-istride*/

/*-------------------------------------------------------------------------*/
/* II) Get the number of transformations, closures and closure bases       */

  num_trans  = 0;
  num_clbase = 0;
  num_clos   = 0;
 for (istride=1;istride<=nstride;istride++) {
  nres = strlen(&stride[istride][0])-1;  
  for (iseg=1;iseg<=nrseg[istride];iseg++) {
       num_trans  += 2*(rseg_length[istride][iseg]-1);
       num_clbase += 1;
  }/*endfor*/

  for (iseg=1;iseg<=ncseg[istride];iseg++) {
    if (ind_cseg_begin[istride][iseg]==(1+ires_offset[istride])) {/*head C*/
       num_trans  += 2*cseg_length[istride][iseg] + 1;
    } 
    else if (ind_cseg_end[istride][iseg]==(nres+ires_offset[istride])) {/*tail C*/
       num_trans  += 2*cseg_length[istride][iseg];
    }
    else {/*bulk C*/
       num_trans  += 3*cseg_length[istride][iseg];
       num_clos   += 2*cseg_length[istride][iseg];
    }/*endif*/
  }/*endfor*/
 }/*endfor-istride*/

#ifdef DEBUG
    printf("num_trans %d\n",num_trans);
    printf("num_clbase %d\n",num_clbase);
    printf("num_clos %d\n",num_clos);
    getchar();
#endif

/*-------------------------------------------------------------------------*/
/* III)                                                                    */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* III.A) Allocate variables                                               */

  res_id         =  new int[num_trans]-1;
  ityp_trans     =  new int[num_trans]-1;
  off_trans      =  new int[num_trans]-1;
  natm_trans     =  new int[num_trans]-1;
  ind_clos_trans =  new int[num_trans]-1;
  prop_sig_scale =  new double[num_trans]-1;
  secondary_only =  new int[num_trans]-1;

  natm_clos          =  new int[num_clos]-1;
  ityp_clos          =  new int[num_clos]-1;
  ityp_clos_postprop =  new int[num_clos]-1;

  natm_clbase     =  new int[num_clbase]-1;
  ind_clbase_free =  new int[num_clbase]-1;
  res_id_clbase   =  new int[num_clbase]-1;
  freeze_clbase   =  new int[num_clbase]-1;

  ind_trans      =  cmall_int_mat(1, num_trans, 1, NATM_TRANS_MAX);
  ind_prim_tors  =  cmall_int_mat(1, num_trans, 1, 4);
  ind_atm_clos   =  cmall_int_mat(1, num_clos, 1, NATM_CLOS_MAX);
  ind_atm_clbase =  cmall_int_mat(1, num_clbase, 1, NATM_CLBASE_MAX);
  const_clos     =  cmall_mat(1, num_clos, 1, NATM_CLOS_MAX);

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* III.B) Initialize default variables                                     */

  for (itrans=1;itrans<=num_trans;itrans++) {
       ityp_trans[itrans]     = 1;
       off_trans[itrans]      = 0;
       secondary_only[itrans] = 0;
       ind_clos_trans[itrans] = 0;
       prop_sig_scale[itrans] = 1.0;
  }/*endfor*/

  num_clbase_free = num_clbase;
  for (iclbase=1;iclbase<=num_clbase;iclbase++) {
       freeze_clbase[iclbase] = 0;
       ind_clbase_free[iclbase] = iclbase;
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* IV.A/B) Define the groups to be transformed for proteins/nucleic acids  */
/*       Run through all STRIDES                                           */

  itrans  = 0;
  iclbase = 0;
  iclos   = 0;

 for (istride=1;istride<=nstride;istride++) {
  nres = strlen(&stride[istride][0])-1;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.C.1) Run through rigid segments                                      */
  for (iseg=1;iseg<=nrseg[istride];iseg++) {
    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
    /* IV.C.1)a Base                                                       */
    ires=ind_rseg_central[istride][iseg];
    for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
         iiatm     =  pos->residue[ires].atom[iatm];
         atm_label =  maps->upac_atm_label[iiatm];

         if ( (strcasecmp(atm_label,"CA")==0) ||
              (strcasecmp(atm_label,"P")==0)    )  {ind_BRF  = iiatm;}
	 if ( (strcasecmp(atm_label,"O")==0) ||
              (strcasecmp(atm_label,"C4*")==0)  )   {ind_BRS   = iiatm;}
         if ( (strcasecmp(atm_label,"CMA")==0) ||
              (strcasecmp(atm_label,"CMN")==0)  ) {ind_BRFS = iiatm;}
    }/*endfor*/
    iclbase++;

#ifdef DEBUG
      printf("istride %d, iclbace %d, num_clbase %d\n",istride,iclbase,num_clbase);
      printf("iseg %d, nseg %d\n",iseg,nrseg[istride]);
      getchar();
#endif

    if (iclbase>num_clbase) {
	     sprintf(err_msg,"Tors_defs:setup_3pt_rb_cblc:\n"
                        "iclbase>num_clbase");
          errorMessage(err_msg);
    }/*endif*/
    res_id_clbase[iclbase] = ires;
    natm_clbase[iclbase] = 3;
    ind_atm_clbase[iclbase][1] = ind_BRFS;         /*BRFS*/
    ind_atm_clbase[iclbase][2] = ind_BRF;          /*BRF*/
    ind_atm_clbase[iclbase][3] = ind_BRS;           /*BRS*/   

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.1)b Forward                                                     */
   for (ires=ind_rseg_central[istride][iseg]+1;
        ires<=ind_rseg_end[istride][iseg];ires++) {

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     =  pos->residue[ires].atom[iatm];
            atm_label =  maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )   {ind_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  ) {ind_BRFS = iiatm;}
       }/*endfor*/
       
       for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
            iiatm     = pos->residue[ires-1].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_prevres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_prevres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_prevres_BRFS = iiatm;}
       }/*endfor*/
      
       itrans++;
       if (itrans>num_trans) {
	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                           "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_BRF;              /*BRF*/
       ind_prim_tors[itrans][4] = ind_BRF;          /*BRF*/
       ind_prim_tors[itrans][3] = ind_prevres_BRS;   /*BRS*/
       ind_prim_tors[itrans][2] = ind_prevres_BRF;  /*BRF*/
       ind_prim_tors[itrans][1] = ind_prevres_BRFS; /*BRFS*/

       itrans++;
       if (itrans>num_trans) {
  	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                           "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
       res_id[itrans] = ires;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1] = ind_BRS;               /*BRS*/
       ind_trans[itrans][2] = ind_BRFS;             /*BRFS*/
       ind_prim_tors[itrans][4] = ind_BRS;           /*BRS*/
       ind_prim_tors[itrans][3] = ind_BRF;          /*BRF*/
       ind_prim_tors[itrans][2] = ind_prevres_BRS;   /*BRS*/
       ind_prim_tors[itrans][1] = ind_prevres_BRF;  /*BRF*/

   }/*endfor-ires-forward*/

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.1)c Backward                                                    */

   for (ires=ind_rseg_central[istride][iseg]-1;
        ires>=ind_rseg_begin[istride][iseg];ires--) {

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     =  pos->residue[ires].atom[iatm];
            atm_label =  maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_BRFS = iiatm;}
       }/*endfor*/

       for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
            iiatm     = pos->residue[ires+1].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_nextres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_nextres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_nextres_BRFS = iiatm;}
       }/*endfor*/

       itrans++;
       if (itrans>num_trans) {
	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_BRS;               /*BRS*/
       ind_prim_tors[itrans][4] = ind_BRS;           /*BRS*/
       ind_prim_tors[itrans][3] = ind_nextres_BRF;  /*BRF*/
       ind_prim_tors[itrans][2] = ind_nextres_BRS;   /*BRS*/
       ind_prim_tors[itrans][1] = ind_nextres_BRFS; /*BRFS*/

       itrans++;
       if (itrans>num_trans) {
  	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                           "itrans>num_trans");
             errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
       res_id[itrans] = ires;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1] = ind_BRF;              /*BRF*/
       ind_trans[itrans][2] = ind_BRFS;             /*BRFS*/
       ind_prim_tors[itrans][4] = ind_BRF;          /*BRF*/
       ind_prim_tors[itrans][3] = ind_BRS;           /*BRS*/
       ind_prim_tors[itrans][2] = ind_nextres_BRF;  /*BRF*/
       ind_prim_tors[itrans][1] = ind_nextres_BRS;   /*BRS*/

   }/*endfor-ires-backward*/

  }/*endfor-rigid segments*/


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* IV.C.2) Run through coil segments                                       */

  for (iseg=1;iseg<=ncseg[istride];iseg++) {   
 
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  /* IV.C.2)a Head                                                        */
    if (ind_cseg_begin[istride][iseg]==1) {
      for (ires=ind_cseg_end[istride][iseg];
           ires>=ind_cseg_begin[istride][iseg];ires--) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_BRFS = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
              iiatm     = pos->residue[ires+1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_nextres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_nextres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_nextres_BRFS = iiatm;}
         }/*endfor*/
         for (iatm=1;iatm<=pos->residue[ires+2].natom;iatm++) {
              iiatm     = pos->residue[ires+2].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_nnextres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_nnextres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_nnextres_BRFS = iiatm;}
         }/*endfor*/


         if (ires==ind_cseg_end[istride][iseg]) {
            itrans++;
            if (itrans>num_trans) {
       	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                                "itrans>num_trans");
                  errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 1;
            ind_trans[itrans][1] = ind_BRS;               /*BRS*/
            ind_prim_tors[itrans][4] = ind_BRS;           /*BRS*/
            ind_prim_tors[itrans][3] = ind_nextres_BRF;  /*BRF*/
            ind_prim_tors[itrans][2] = ind_nextres_BRS;   /*BRS*/
            ind_prim_tors[itrans][1] = ind_nextres_BRFS; /*BRFS*/
	 }/*endif*/

         if (ires!=ind_cseg_end[istride][iseg]) {
            itrans++;
            if (itrans>num_trans) {
       	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                                "itrans>num_trans");
                  errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 2;
            ind_trans[itrans][1] = ind_BRS;               /*BRS*/
            ind_trans[itrans][2] = ind_nextres_BRFS;     /*BRFS*/
            ind_prim_tors[itrans][4] = ind_BRS;           /*BRS*/
            ind_prim_tors[itrans][3] = ind_nextres_BRF;  /*BRF*/
            ind_prim_tors[itrans][2] = ind_nextres_BRS;   /*BRS*/
            ind_prim_tors[itrans][1] = ind_nnextres_BRF; /*BRF*/
	 }/*endif*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_BRF;              /*BRF*/
         ind_prim_tors[itrans][4] = ind_BRF;          /*BRF*/
         ind_prim_tors[itrans][3] = ind_BRS;           /*BRS*/
         ind_prim_tors[itrans][2] = ind_nextres_BRF;  /*BRF*/
         ind_prim_tors[itrans][1] = ind_nextres_BRS;   /*BRS*/

         if (ires==ind_cseg_begin[istride][iseg]) {
            itrans++;
            if (itrans>num_trans) {
       	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                                "itrans>num_trans");
                  errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 1;
            ind_trans[itrans][1] = ind_BRFS;             /*BRFS*/
            ind_prim_tors[itrans][4] = ind_BRFS;         /*BRFS*/
            ind_prim_tors[itrans][3] = ind_BRF;          /*BRF*/
            ind_prim_tors[itrans][2] = ind_BRS;           /*BRS*/
            ind_prim_tors[itrans][1] = ind_nextres_BRF;  /*BRF*/
	 }/*endif*/
      }/*endfor-ires*/
    }/*endif-Head*/

  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.2)b Tail                                                        */
    if (ind_cseg_end[istride][iseg]==(nres+ires_offset[istride])) {
      for (ires=ind_cseg_begin[istride][iseg];
           ires<=ind_cseg_end[istride][iseg];ires++) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_BRFS = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
              iiatm     = pos->residue[ires-1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_prevres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_prevres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_prevres_BRFS = iiatm;}
         }/*endfor*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_BRF;              /*BRF*/
         ind_prim_tors[itrans][4] = ind_BRF;          /*BRF*/
         ind_prim_tors[itrans][3] = ind_prevres_BRS;   /*BRS*/
         ind_prim_tors[itrans][2] = ind_prevres_BRF;  /*BRF*/
         ind_prim_tors[itrans][1] = ind_prevres_BRFS; /*BRFS*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 2;
         ind_trans[itrans][1] = ind_BRS;               /*BRS*/
         ind_trans[itrans][2] = ind_BRFS;             /*BRFS*/
         ind_prim_tors[itrans][4] = ind_BRS;           /*BRS*/
         ind_prim_tors[itrans][3] = ind_BRF;          /*BRF*/
         ind_prim_tors[itrans][2] = ind_prevres_BRS;   /*BRS*/
         ind_prim_tors[itrans][1] = ind_prevres_BRF;  /*BRF*/

      }/*endfor-ires*/
    }/*endif-Tail*/


  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /* IV.C.2)c Bulk                                                        */
    if ((ind_cseg_begin[istride][iseg]!=(1+ires_offset[istride]))&&
        (ind_cseg_end[istride][iseg]!=(nres+ires_offset[istride]))) {

#ifdef DEBUG
  printf("Bulk coil segment setup\n");
  getchar();
#endif

      for (ires=ind_cseg_begin[istride][iseg];
           ires<=ind_cseg_end[istride][iseg];ires++) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_BRFS = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
              iiatm     = pos->residue[ires-1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_prevres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_prevres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_prevres_BRFS = iiatm;}
         }/*endfor*/
         for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
              iiatm     = pos->residue[ires+1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_nextres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_nextres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_nextres_BRFS = iiatm;}
         }/*endfor*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_BRF;              /*BRF*/
         ind_prim_tors[itrans][4] = ind_BRF;          /*BRF*/
         ind_prim_tors[itrans][3] = ind_prevres_BRS;   /*BRS*/
         ind_prim_tors[itrans][2] = ind_prevres_BRF;  /*BRF*/
         ind_prim_tors[itrans][1] = ind_prevres_BRFS; /*BRFS*/


         /*____________________/closure/____________________*/
         iclos++;
         if (iclos>num_clos) {
    	    sprintf(err_msg,"Tors_defs::setup_cgresmodel_3pt_cblc:\n"
                            "iclos>num_clos");
            errorMessage(err_msg);
         }/*endif*/
         ind_clos_trans[itrans] = iclos;
         if (ires==ind_cseg_begin[istride][iseg]) {
             ityp_clos[iclos]       = 0;             /*cblc*/
	 } else {
             ityp_clos[iclos]       = 5;             /*scblc*/
         }/*endif*/
         natm_clos[iclos]       = 5;
         ind_atm_clos[iclos][5] = ind_BRS;            /*BRS*/
         ind_atm_clos[iclos][4] = ind_BRF;           /*BRF*/
         ind_atm_clos[iclos][3] = ind_prevres_BRS;    /*BRS*/
         ind_atm_clos[iclos][2] = ind_prevres_BRF;   /*BRF*/
         ind_atm_clos[iclos][1] = ind_prevres_BRFS;  /*BRFS*/
         dx = x[ind_BRS] - x[ind_BRF];
         dy = y[ind_BRS] - y[ind_BRF];
         dz = z[ind_BRS] - z[ind_BRF];
         const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
         dx = x[ind_prevres_BRS] - x[ind_BRF];
         dy = y[ind_prevres_BRS] - y[ind_BRF];
         dz = z[ind_prevres_BRS] - z[ind_BRF];
         const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/


         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         if (ires==ind_cseg_end[istride][iseg]) {
             ityp_trans[itrans] = 2;
         } else {
             ityp_trans[itrans] = 1;
         }/*endif*/
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_BRS;               /*BRS*/
         ind_prim_tors[itrans][4] = ind_BRS;           /*BRS*/
         ind_prim_tors[itrans][3] = ind_BRF;          /*BRF*/
         ind_prim_tors[itrans][2] = ind_prevres_BRS;   /*BRS*/
         ind_prim_tors[itrans][1] = ind_prevres_BRF;  /*BRF*/


         /*____________________/closure/____________________*/
         iclos++;
         if (iclos>num_clos) {
    	    sprintf(err_msg,"Tors_defs::setup_cgresmodel_3pt_cblc:\n"
                            "iclos>num_clos");
            errorMessage(err_msg);
         }/*endif*/
         ind_clos_trans[itrans] = iclos;
         ityp_clos[iclos]       = 5;             /*scblc*/
         natm_clos[iclos]       = 5;
         ind_atm_clos[iclos][5] = ind_nextres_BRF;   /*BRF*/
         ind_atm_clos[iclos][4] = ind_BRS;            /*BRS*/
         ind_atm_clos[iclos][3] = ind_BRF;           /*BRF*/
         ind_atm_clos[iclos][2] = ind_prevres_BRS;    /*BRS*/
         ind_atm_clos[iclos][1] = ind_prevres_BRF;   /*BRF*/
         dx = x[ind_nextres_BRF] - x[ind_BRS];
         dy = y[ind_nextres_BRF] - y[ind_BRS];
         dz = z[ind_nextres_BRF] - z[ind_BRS];
         const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
         dx = x[ind_BRF] - x[ind_BRS];
         dy = y[ind_BRF] - y[ind_BRS];
         dz = z[ind_BRF] - z[ind_BRS];
         const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/

      }/*endfor-ires-backbone*/

      for (ires=ind_cseg_begin[istride][iseg];
           ires<=ind_cseg_end[istride][iseg];ires++) {
         for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
              iiatm     =  pos->residue[ires].atom[iatm];
              atm_label =  maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_BRFS = iiatm;}
         }/*endfor*/         
         for (iatm=1;iatm<=pos->residue[ires-1].natom;iatm++) {
              iiatm     = pos->residue[ires-1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_prevres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_prevres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_prevres_BRFS = iiatm;}
         }/*endfor*/
         for (iatm=1;iatm<=pos->residue[ires+1].natom;iatm++) {
              iiatm     = pos->residue[ires+1].atom[iatm];
              atm_label = maps->upac_atm_label[iiatm];
            if ( (strcasecmp(atm_label,"CA")==0) ||
                 (strcasecmp(atm_label,"P")==0)    )  {ind_nextres_BRF  = iiatm;}
	    if ( (strcasecmp(atm_label,"O")==0) ||
                 (strcasecmp(atm_label,"C4*")==0)  )  {ind_nextres_BRS   = iiatm;}
            if ( (strcasecmp(atm_label,"CMA")==0) ||
                 (strcasecmp(atm_label,"CMN")==0)  )  {ind_nextres_BRFS = iiatm;}
         }/*endfor*/

         itrans++;
         if (itrans>num_trans) {
    	     sprintf(err_msg,"Tors_defs::setup_3pt_rb_cblc:\n"
                             "itrans>num_trans");
               errorMessage(err_msg);
         }/*endif*/
         res_id[itrans] = ires;
         ityp_trans[itrans] = 0;
         secondary_only[itrans] = 2*itrans-2*cseg_length[istride][iseg]; 
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_BRFS;              /*BRFS*/
         ind_prim_tors[itrans][4] = ind_BRFS;          /*BRFS*/
         ind_prim_tors[itrans][3] = ind_BRF;           /*BRF*/
         ind_prim_tors[itrans][2] = ind_prevres_BRS;    /*BRS*/
         ind_prim_tors[itrans][1] = ind_prevres_BRF;   /*BRF*/
      }/*endfor-ires-CMA*/


    }/*endif-Bulk*/
  }/*endfor-coil segments*/
 }/*endfor-istride*/


/*-------------------------------------------------------------------------*/
/* IV) If stride is defined in .pdb, only move basis                       */
/*       along 'C' secondary structure type residue                        */
 
//   if (general->stride_defined==1) {
//     if (nstride!=1) {
//       sprintf(err_msg, "Tors_defs::setup_cgresmodel_3pt_rb_cblc: nstride!=1");
//       errorMessage(err_msg);
//     }/*endif*/
//     if (strlen(&(stride[1][0]))-1!=pos->nres) {
//       sprintf(err_msg, "Tors_defs::setup_cgresmodel_3pt_rb_cblc:\n" 
//                        "corrupted pos.stride \n");
//       errorMessage(err_msg);
//     }/*endif*/
//     for (iclbase=1;iclbase<=num_clbase;iclbase++) {
//          freeze_clbase[iclbase] = 1;
//          if (stride[1][res_id_clbase[iclbase]] == 'C') {
// 	     freeze_clbase[iclbase] = 0;
//          }/*endif*/
//     }/*endfor*/
//   }/*end if*/

/*-------------------------------------------------------------------------*/
/* V) Setup rotate_utils                                                 */

  rotate_utils.setup();

  
/*-------------------------------------------------------------------------*/
/* VI) Setup ityp_clos_potprop if postprop_minimize is on                  */

  if (general->postprop_minimize==1) {
      for (iclos=1;iclos<=num_clos;iclos++) {
  	   ityp_clos_postprop[iclos] = ityp_clos[iclos];
      }/*endfor*/
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* VII) Debug                                                              */

#ifdef DEBUG
  printf("############################ All Torsions ############################\n");
  write_alldata(maps);
#endif

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/
