#include "../../include/standard.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_model_AA(const Maps *maps,
                                const Pos *pos,
                                const General *general)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for All Atom model                 */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int i, imol, iimol, jmol, jjmol, nres;
  int ires, iresl, itrans, iclos, iclbase;
  int istride, imol_notors;
  int num_prot_or_nucl, cblc_type;
  int itbend, ntbend;

  int ibond, iresb;
  int valid, natm_bonded;
  int iatm, iiatm, jatm, iatmb, jatmb;

  int imol_free, imol_cblc, jmol_found;
  int max_nmol_closcompl, max_nlevel_closcompl;
  char chainid;
  char *residue_name;

  int protein_found;
  int DNA_found;
  int DUM_found;
  int clos_found;
  int protein_D6;
  int DNA_duplex;
  int supported_torcompl, supported_closcompl;

  int    nmol        = maps->nmol;
  int   *nres_mol    = maps->nres_mol;
  int   *npro_mol    = maps->npro_mol;
  int  **ind_res_mol = maps->ind_res_mol;
  char **mol_label   = maps->mol_label;
  char **mol_chainid = maps->mol_chainid;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0.A) Get information about molecules (protein, DNA, RNA)                */
/*      and get the indices of molecules with no torsional degs of freedom */

     num_prot_or_nucl = 0;
     nmol_notors = 0;
     for (imol=1;imol<=nmol;imol++) {
       if (strcasecmp(mol_label[imol],"protein")==0) {
           num_prot_or_nucl++;
       }/*endif*/
       if (strcasecmp(mol_label[imol],"DNA")==0) {
           num_prot_or_nucl++;
       }/*endif*/
       if (strcasecmp(mol_label[imol],"RNA")==0) {
           num_prot_or_nucl++;
       }/*endif*/
       if ( (strcasecmp(mol_label[imol],"water")   ==0) ||
            (strcasecmp(mol_label[imol],"atom/ion")==0) ) {
	     nmol_notors++;
       }/*endif*/
     }/*endfor*/

     ind_mol_notors = new int[nmol_notors] - 1;
     imol_notors = 0;
     for (imol=1;imol<=nmol;imol++) {
       if ( (strcasecmp(mol_label[imol],"water")   ==0) ||
            (strcasecmp(mol_label[imol],"atom/ion")==0) ) {
	     imol_notors++;
             ind_mol_notors[imol_notors] = imol;
       }/*endif*/
     }/*endfor*/

/*-------------------------------------------------------------------------*/
/* 0.B) Handeling closure relations                                        */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.B.1) Get information about molecules treated with closure             */
/*      Currently supported:                                               */ 
/*       Constant Bond Length Closure (CBLC)                               */

  if (general->cblc_defined==0) {nmol_cblc = 0;}    
  if (general->cblc_defined==1) {    
      nmol_cblc    = strlen(&(cblc[0])) - 1;
      ind_mol_cblc = new int[nmol_cblc] - 1;
      for (imol=1;imol<=nmol_cblc;imol++) {
	   chainid = cblc[imol];
           for (jmol=1;jmol<=nmol;jmol++) {
	        if (mol_chainid[jmol][0] == chainid) {
		    ind_mol_cblc[imol] = jmol;
                }/*endif*/
           }/*endfor*/
      }/*endfor*/
  }/*endif*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.B.2) Get the number of closure bases per molecule                     */

  if (general->clos_defined==1) {
    nclbase_mol =  new int[nmol] - 1;
    for (jmol=1;jmol<=nmol;jmol++) {
        nclbase_mol[jmol] = 0;     
        for (imol=1;imol<=nmol_cblc;imol++) {
             if (ind_mol_cblc[imol] == jmol) {
               if (strcasecmp(mol_label[jmol],"DNA")==0) {
                   nclbase_mol[jmol] = nres_mol[jmol];
               }
               else if (strcasecmp(mol_label[jmol],"RNA")==0) {
                   nclbase_mol[jmol] = nres_mol[jmol];
               }
               else if (strcasecmp(mol_label[jmol],"protein")==0) {
                   nclbase_mol[jmol] = nres_mol[jmol] - 1;
               } else {
                 sprintf(err_msg,"currently supported molecule types \n"
                                 "in CBLC are: DNA and protein");
                 errorMessage(err_msg);
               }/*endif*/		   	       
             }/*endif*/
        }/*endfor*/
    }/*endfor*/
  }/*endif*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.B.3) Get information about closure complexes (if any)                 */
/*    Allocate necessary variables                                         */
/*    Supported setups:                                                    */
/*    A) DNA_duplex                                                        */
/*       nmol_closcompl[this] = 2                                          */
/*       2 DNA strands in a closure complex                                */


  if (general->closcompl_defined==0) {ncloscompl=0;}
  if (general->closcompl_defined==1) {
      nmol_closcompl = new int[ncloscompl] - 1;
      max_nmol_closcompl = 0;
      for (i=1;i<=ncloscompl;i++) {
	   nmol_closcompl[i]  = strlen(closcompl[i]) - 1;
           max_nmol_closcompl = MAX(max_nmol_closcompl,nmol_closcompl[i]);
      }/*endfor*/
      
      ind_mol_closcompl = cmall_int_mat(1, ncloscompl, 1, max_nmol_closcompl);
      for (i=1;i<=ncloscompl;i++) {
      	  for (imol=1;imol<=nmol_closcompl[i];imol++) {
	       chainid = closcompl[i][imol];
               for (jmol=1;jmol<=nmol;jmol++) {
		    if (mol_chainid[jmol][0] == chainid) {
		        ind_mol_closcompl[i][imol] = jmol;
                    }/*endif*/
               }/*endfor*/
          }/*endfor*/
      }/*endfor*/

      for (i=1;i<=ncloscompl;i++) {
      	  for (imol=1;imol<=nmol_closcompl[i];imol++) {
               clos_found = 0;
               iimol = ind_mol_closcompl[i][imol];
               for (jmol=1;jmol<=nmol_cblc;jmol++) {/*cblc*/
                    jjmol = ind_mol_cblc[jmol];
                    if (iimol == jjmol) {
		       clos_found = 1;
                    }
               }/*endfor*/
	       if (clos_found==0) {
                   chainid = closcompl[i][imol];
                   sprintf(err_msg,"chain/molecule %c present in CLOSCOMPL\n"
                                 "is not referenced as a closure molecule\n"
                                 "e.g. not part of CBLC",chainid);
                   errorMessage(err_msg);
               }/*endif*/
          }/*endfor*/
      }/*endfor*/

      closcompl_label = (char **)cmall_int_mat(1, ncloscompl, 0, NNAMECHARMAX);
      for (i=1;i<=ncloscompl;i++) {
	  protein_found = 0;
          DNA_found = 0;
          DUM_found = 0; 
          DNA_duplex = 0;
          supported_closcompl = 0;
          for (imol=1;imol<=nmol_closcompl[i];imol++) {
               iimol = ind_mol_closcompl[i][imol];
               if (strcasecmp(mol_label[iimol],"DNA")==0) {DNA_found++;}
          }/*endfor*/
          if ( (DNA_found==2) && (nmol_closcompl[i]==2) ){
                DNA_duplex = 1;
                strcpy(closcompl_label[i],"DNA_duplex");
          }/*endif*/
          supported_closcompl =  DNA_duplex;
          if ( supported_closcompl == 0 ) {
             sprintf(err_msg,"%d. CLOSCOMPL > setup in .pdb is not supported\n"
                    "Supported setups:                                  \n"
                    "A) DNA_duplex                                      \n"
                    "   2 DNA strands in one complex.                   \n"
		    "   E.g. CLOSCOMPL >AB                              \n",i);
             errorMessage(err_msg);
          }/*endif*/
      }/*endfor-i-complex*/
  }/*endif-closcompl_defined*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* 0.B.4) Allocate closure complex levels                                  */

  nlevel_closcompl = new int[ncloscompl] - 1;

  max_nlevel_closcompl = 0;        /* no more than all clbase in closcompl */
  for (i=1;i<=ncloscompl;i++) {
       for (imol=1;imol<=nmol_closcompl[i];imol++) {
            iimol = ind_mol_closcompl[i][imol];
            max_nlevel_closcompl += nclbase_mol[iimol];
       }/*endfor*/
  }/*endfor*/

  ind_clbase_level_closcompl = cmall_itens3(1,ncloscompl,
                                            1,max_nlevel_closcompl,
                                            1,MAXCLBPL);

/*-------------------------------------------------------------------------*/
/* 0.C) Get information about STRIDE's                                     */
/*      If STRIDE is defined both protein and DNA/RNA must possess STRIDE  */

     if (general->stride_defined==1) {
       if (num_prot_or_nucl!=nstride) {
        sprintf(err_msg,"# of STRIDE does not match # of prot and nucl acids");
        errorMessage(err_msg);
       }/*endif*/
       ind_stride_mol = new int[nmol] - 1;
       istride = 0;
       for (imol=1;imol<=nmol;imol++) {
         if ( (strcasecmp(mol_label[imol],"protein")==0) ||
              (strcasecmp(mol_label[imol],"DNA"    )==0) ||
              (strcasecmp(mol_label[imol],"RNA"    )==0)   ) {
               istride++;
               ind_stride_mol[imol] = istride;
         } else {
	       ind_stride_mol[imol] = 0;
         }/*endif*/
       }/*endfor*/
     }/*endif*/

/*-------------------------------------------------------------------------*/
/* 0.D) Get information about torsional complexes  (if any)                */
/*    Allocate necessary variables                                         */
/*    TORSIONAL MOLECULAR COMPLEXES~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/*    Supported setups:                                                    */
/*    A) DNA_duplex                                                        */
/*       nmol_torcompl[this] = 10                                          */
/*       2 DNA strands with 8 dummy molecules (atoms) in one complex       */
/*    B) protein_D6                                                        */
/*       1 protein with 6 dummy molecules (atoms) in                       */
/*       one complex                                                       */

  if (general->torcompl_defined==0) {ntorcompl=0;}
  if (general->torcompl_defined==1) {
      nmol_torcompl = new int[ntorcompl] - 1;
      max_nmol_torcompl = 0;
      for (i=1;i<=ntorcompl;i++) {
	   nmol_torcompl[i]  = strlen(torcompl[i]) - 1;
           max_nmol_torcompl = MAX(max_nmol_torcompl,nmol_torcompl[i]);
      }/*endfor*/
      
      ind_mol_torcompl = cmall_int_mat(1, ntorcompl, 1, max_nmol_torcompl);
      for (i=1;i<=ntorcompl;i++) {
      	  for (imol=1;imol<=nmol_torcompl[i];imol++) {
	       chainid = torcompl[i][imol];
               for (jmol=1;jmol<=nmol;jmol++) {
		    if (mol_chainid[jmol][0] == chainid) {
		        ind_mol_torcompl[i][imol] = jmol;
                    }/*endif*/
               }/*endfor*/
          }/*endfor*/
      }/*endfor*/

      for (i=1;i<=ntorcompl;i++) {
      	  for (imol=1;imol<=nmol_torcompl[i];imol++) {
               iimol = ind_mol_torcompl[i][imol];
               for (jmol=1;jmol<=nmol_cblc;jmol++) {
                    jjmol = ind_mol_cblc[jmol];
                    if (iimol == jjmol) {
		       chainid = mol_chainid[jmol][0];
                       sprintf(err_msg,"chain/molecule: %c is refrenced in\n"
                                       "both TORCOMPL and CBLC. Such setup\n"
                                       "is currently not supported",chainid);
                       errorMessage(err_msg);
                    }
               }/*endfor*/
          }/*endfor*/
      }/*endfor*/

      ityp_torcompl  =  new int[ntorcompl] - 1;
      torcompl_label = (char **)cmall_int_mat(1, ntorcompl, 0, NNAMECHARMAX);
      for (i=1;i<=ntorcompl;i++) {
	  protein_found = 0;
          DNA_found = 0;
          DUM_found = 0; 
          DNA_duplex = 0;
          protein_D6 = 0;
          supported_torcompl = 0;
          for (imol=1;imol<=nmol_torcompl[i];imol++) {
               iimol = ind_mol_torcompl[i][imol];
               if (strcasecmp(mol_label[iimol],"protein")==0){protein_found++;}
               if (strcasecmp(mol_label[iimol],"DNA")==0) {DNA_found++;}
               if (strcasecmp(mol_label[iimol],"DUM")==0) {DUM_found++;}
          }/*endfor*/
          if (  (DNA_found==2) && (DUM_found==8) && (nmol_torcompl[i] == 10) ){
                 DNA_duplex = 1;
                 strcpy(torcompl_label[i],"DNA_duplex");
                 ityp_torcompl[i] = 1;
          }/*endif*/
          if ((protein_found==1)&&(DUM_found==6) && (nmol_torcompl[i] == 7 ) ){
                 protein_D6 = 1;
                 strcpy(torcompl_label[i],"protein_D6");
                 ityp_torcompl[i] = 2;
          }/*endif*/
          supported_torcompl =  DNA_duplex + protein_D6;
          if ( supported_torcompl == 0 ) {
             sprintf(err_msg,"%d. TORCOMPL > setup in .pdb is not supported\n" 
                     "Supported setups:                                  \n"
                     "A) DNA_duplex                                      \n"
                     "   2 DNA strands with 8 dummy molecules (atoms)    \n"
                     "   in one complex.                                 \n"
                     "   E.g. TORCOMPL >ABCDEFGHIJ                       \n"
                     "   A, B : two DNA strands with chain id A and B    \n"
                     "   C,..,J : eight atoms of DUM label and residue   \n"
                     "            type with chain id C,..,J respectively \n"
                     "B) protein_D6                                      \n"
                     "   1 protein with 6 dummy molecules (atoms) in     \n"
                     "   one complex                                     \n"
                     "   E.g. TORCOMPL >ABCDEFG                          \n"
                     "   A : protein                                     \n"
                     "   B,...,G : atoms of DUM label with chain id      \n"
                     "   B,...,G respectively                            \n",i);
             errorMessage(err_msg);
          }/*endif*/
      }/*endfor-i-complex*/
  }/*endif-torcompl_defined*/


/*-------------------------------------------------------------------------*/
/* 0.E) Find molecules (free) neither in TORCOMPL nor in CBLC              */

      ind_mol_free = new int[nmol] - 1;
      nmol_free = 0;

      for (jmol=1;jmol<=nmol;jmol++) {
	 jmol_found = 0;
         for (i=1;i<=ntorcompl;i++) {
      	     for (imol=1;imol<=nmol_torcompl[i];imol++) {
	          if (jmol==ind_mol_torcompl[i][imol]) {jmol_found=1;}
             }/*endfor*/
         }/*endfor*/
         for (imol=1;imol<=nmol_cblc;imol++) {
             if (jmol==ind_mol_cblc[imol]) {jmol_found=1;}
         }/*endfor*/

         if (jmol_found==0) {
	     nmol_free++;
             ind_mol_free[nmol_free] = jmol;
         }/*endif*/
      }/*endfor*/


/*-------------------------------------------------------------------------*/
/* I) Get the number of transformations, closures, closure bases           */
/*                                                                         */
/*    CONSTANT BOND LENGTH CLOSURE MOLECULES (CBLC)~~~~~~~~~~~~~~~~~~~~~~~~*/
/*    A) PROTEINS                                                          */
/*       '>' or regular cblc                                               */
/*       Backbone: Assume two transformations/bulk residues                */
/*                        three transformations/terminal residues          */
/*                        additional four transformations/proline residues */  
/*       Base torsions: Get the number of chi groups from topology         */
/*       Closure bases: Assume one/residue - 1                             */
/*       Closures: Assume one closure/closure base - 1                     */
/*                 Plus one extra for each proline residue                 */ 
/*                                                                         */
/*    B) DNA                                                               */
/*       '>' or regular cblc                                               */
/*       Backbone: Assume seven transformations/residue                    */
/*       Base torsions: Get the number of chi groups from topology         */
/*       Closures: Assume two closures/residue - 1 (first residue only one)*/
/*       Closure bases: Assume one/residue                                 */
/*                                                                         */
/*       '~' or successive cblc                                            */
/*       Backbone: Assume nine transformations/residue                     */
/*       Base torsions: Get the number of chi groups from topology         */
/*       Closures: Assume four closures/residue - 3                        */
/*       Closure bases: Assume one/residue                                 */
/*                                                                         */
/*       '-' or regular cblc on '~' topology                               */
/*       Backbone: Assume nine transformations/residue                     */
/*       Base torsions: Get the number of chi groups from topology         */
/*       Closures: Assume four closures/residue - 3                        */
/*       Closure bases: Assume one/residue                                 */
/*    B) RNA                                                               */
/*       '>' or regular cblc                                               */
/*       Backbone: Assume eight transformations/residue                    */
/*       Base torsions: Get the number of chi groups from topology         */
/*       Closures: Assume two closures/residue - 1 (first residue only one)*/
/*       Closure bases: Assume one/residue                                 */
/*                                                                         */
/*       '~' or successive cblc                                            */
/*       Backbone: Assume ten transformations/residue                      */
/*       Base torsions: Get the number of chi groups from topology         */
/*       Closures: Assume four closures/residue - 3                        */
/*       Closure bases: Assume one/residue                                 */
/*                                                                         */
/*                                                                         */
/*    TORSIONAL MOLECULAR COMPLEXES~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/*    A) DNA_duplex                                                        */
/*       Backbone: Assume five transformations per residue per strand      */
/*                 This includes the tranformations for relative orient    */
/*       Base torsions: Get the number of chi groups from topology         */
/*    B) protein_D6                                                        */
/*       Backbone: Assume two transformations per residue in protein       */
/*                 Plus two transformations on each terminal (N,C)         */
/*                 Plus one extra tranformation on the center residue      */
/*                 connected to the dummy atoms                            */
/*                 The above include the 3 transformations with dummy      */
/*                 primary torsions (2 of them transform the central res)  */
/*                 In addition the presence of each PRO will decrease the  */
/*                 number of transformations by one                        */
/*       Sidechain: Get the number of chi groups from topology             */
/*                                                                         */
/*    FREE MOLECULES ONLY~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/*    Loop through the molecules                                           */
/*    A) PROTEINS:                                                         */
/*       Backbone: Two transformations/residue, except for first residue   */
/*       Sidechain: Get the number of chi groups from topology             */
/*    B) DNA:                                                              */
/*       Backbone: Five transformations/residue, except for last residue   */
/*       Base torsions: Get the number of chi groups from topology         */
/*    C) RNA:                                                              */
/*       Backbone: Six transformations/residue, except for last residue    */
/*       Base torsions: Get the number of chi groups from topology         */

     num_trans  = 0;
     num_clos   = 0;
     num_clbase = 0;
     for (i=1;i<=ntorcompl;i++) {
         if (strcasecmp(torcompl_label[i],"DNA_duplex")==0) {
             for (imol=1;imol<=nmol_torcompl[i];imol++) {
                  iimol = ind_mol_torcompl[i][imol];
                  if (strcasecmp(mol_label[iimol],"DNA")==0) {
      		      nres = nres_mol[iimol];
                      num_trans += 5*nres;                     /*DNA Backbone*/
                      for (iresl=1;iresl<=nres;iresl++) {
                         ires = ind_res_mol[iimol][iresl];
                         num_trans += pos->residue[ires].nchigrp; /*BaseTors*/
                      }/*end for*/
                  }/*DNA*/
             }/*endfor*/
         }/*endif-DNA_duplex*/
         if (strcasecmp(torcompl_label[i],"protein_D6")==0) {
             for (imol=1;imol<=nmol_torcompl[i];imol++) {
                  iimol = ind_mol_torcompl[i][imol];
                  if (strcasecmp(mol_label[iimol],"protein")==0) {
      		      nres = nres_mol[iimol];
                      num_trans += 2*nres + 3;            /*Protein Backbone*/
                      for (iresl=1;iresl<=nres;iresl++) {
                         ires = ind_res_mol[iimol][iresl];
                         num_trans += pos->residue[ires].nchigrp; /*BaseTors*/
                         residue_name = pos->residue[ires].residue_name;
                         if  ( strcasecmp(residue_name,"PRO")==0 ) {
			       num_trans--; /*one less transform*/
                         }/*endif*/
                      }/*end for*/
                  }/*DNA*/
             }/*endfor*/
         }/*endif-protein_D6*/
     }/*endfor-i-complex*/

     for (imol_cblc=1;imol_cblc<=nmol_cblc;imol_cblc++) {
          imol = ind_mol_cblc[imol_cblc];
          if (strcasecmp(mol_label[imol],"protein")==0) {
              nres = nres_mol[imol];
              if ( (cblc[0] == '>') ||
                   (cblc[0] == '~') ||
                   (cblc[0] == '-') ) {/*currently only regular cblc*/
                 num_trans  += 2*nres + 2;                /*Protein Backbone*/
                 num_trans  += 4*npro_mol[imol];
                 num_clos   += nres - 2;
                 num_clos   += npro_mol[imol];
                 num_clbase += nres - 1;
                 for (iresl=1;iresl<=nres;iresl++) {
                      ires = ind_res_mol[imol][iresl];
                      num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
                 }/*end for*/
              }/*endif*/
          }/*endif*/
          if (strcasecmp(mol_label[imol],"DNA")==0) {
       	      nres = nres_mol[imol];
              if ( cblc[0] == '>' ) {/*regular cblc*/
                 num_trans  += 7*nres;                       /*DNA Backbone*/
                 num_clos   += 2*nres - 1;
                 num_clbase += nres;
                 for (iresl=1;iresl<=nres;iresl++) {
                      ires = ind_res_mol[imol][iresl];
                      num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
                 }/*end for*/
              } else if ( cblc[0] == '~' ) {/*successive cblc*/
                 num_trans  += 9*nres;                       /*DNA Backbone*/
                 num_clos   += 4*nres - 3;
                 num_clbase += nres;
                 for (iresl=1;iresl<=nres;iresl++) {
                      ires = ind_res_mol[imol][iresl];
                      num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
                 }/*end for*/
              } else if ( cblc[0] == '-' ) {/*regular cblc on scblc topology*/
                 num_trans  += 9*nres;                       /*DNA Backbone*/
                 num_clos   += 4*nres - 3;
                 num_clbase += nres;
                 for (iresl=1;iresl<=nres;iresl++) {
                      ires = ind_res_mol[imol][iresl];
                      num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
                 }/*end for*/
              }/*endif*/
          }/*endif*/
          if (strcasecmp(mol_label[imol],"RNA")==0) {
       	      nres = nres_mol[imol];
              if ( cblc[0] == '>' ) {/*regular cblc*/
                 num_trans  += 8*nres;                       /*RNA Backbone*/
                 num_clos   += 2*nres - 1;
                 num_clbase += nres;
                 for (iresl=1;iresl<=nres;iresl++) {
                      ires = ind_res_mol[imol][iresl];
                      num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
                 }/*end for*/
              } else if ( cblc[0] == '~' ) {/*successive cblc*/
                 num_trans  += 10*nres;                      /*RNA Backbone*/
                 num_clos   += 4*nres - 3;
                 num_clbase += nres;
                 for (iresl=1;iresl<=nres;iresl++) {
                      ires = ind_res_mol[imol][iresl];
                      num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
                 }/*end for*/
              }/*endif*/
          }/*endif*/
     }/*endfor*/

     for (imol_free=1;imol_free<=nmol_free;imol_free++) {
          imol = ind_mol_free[imol_free];
          if (strcasecmp(mol_label[imol],"protein")==0) {
              nres = nres_mol[imol];
              num_trans += 2*(nres-1);                 /*Protein Backbone*/
              for (iresl=1;iresl<=nres;iresl++) {
                   ires = ind_res_mol[imol][iresl];
                   num_trans += pos->residue[ires].nchigrp;  /*Sidechain*/
              }/*end for*/
          }/*endif*/
          if (strcasecmp(mol_label[imol],"DNA")==0) {
       	      nres = nres_mol[imol];
              num_trans += 5*(nres-1) + 3;                /*DNA Backbone*/
              for (iresl=1;iresl<=nres;iresl++) {
                   ires = ind_res_mol[imol][iresl];
                   num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
              }/*end for*/
          }/*endif*/
          if (strcasecmp(mol_label[imol],"RNA")==0) {
       	      nres = nres_mol[imol];
              num_trans += 6*(nres-1) + 4;                /*DNA Backbone*/
              for (iresl=1;iresl<=nres;iresl++) {
                   ires = ind_res_mol[imol][iresl];
                   num_trans += pos->residue[ires].nchigrp;   /*BaseTors*/
              }/*end for*/
          }/*endif*/
     }/*endfor*/

/*-------------------------------------------------------------------------*/
/* II/A) Allocate variables                                                */

  res_id         =  new int[num_trans]-1;
  ityp_trans     =  new int[num_trans]-1;
  off_trans      =  new int[num_trans]-1;
  natm_trans     =  new int[num_trans]-1;
  ntbend_trans   =  new int[num_trans]-1;
  secondary_only =  new int[num_trans]-1;
  ind_clos_trans =  new int[num_trans]-1;
  prop_sig_scale =  new double[num_trans]-1;
  natm_clos          =  new int[num_clos]-1;
  ityp_clos          =  new int[num_clos]-1;
  ityp_clos_postprop =  new int[num_clos]-1;
  natm_clbase    =  new int[num_clbase]-1;
  ind_clbase_free = new int[num_clbase]-1;
  res_id_clbase   = new int[num_clbase]-1;             
  freeze_clbase   = new int[num_clbase]-1;
  natm_torcomplbase  =  new int[ntorcompl]-1;

  ind_trans      =  cmall_int_mat(1, num_trans, 1, NATM_TRANS_MAX);
  ind_tbend_trans=  cmall_int_mat(1, num_trans, 1, NATM_TRANS_MAX);
  ind_prim_tors  =  cmall_int_mat(1, num_trans, 1, 4);
  ind_atm_clos   =  cmall_int_mat(1, num_clos, 1, NATM_CLOS_MAX);
  ind_atm_clbase =  cmall_int_mat(1, num_clbase, 1, NATM_CLBASE_MAX);
  const_clos     =  cmall_mat(1, num_clos, 1, NATM_CLOS_MAX);
  ind_clbase_mol =  cmall_int_mat(1, nmol, 1, num_clbase);
  ind_atm_torcomplbase = cmall_int_mat(1,ntorcompl,1,NATM_TORCOMPLBASE_MAX);

/*-------------------------------------------------------------------------*/
/* II/B) Initialize default values                                         */
/*     These values might have default initialization/assignment elsewhere */

  for (i=1;i<=num_trans;i++) {
       ityp_trans[i]     = 1;
       off_trans[i]      = 0;
       secondary_only[i] = 0;
       ind_clos_trans[i] = 0;
       prop_sig_scale[i] = 1.0;
  }/*endfor*/  

  for (i=1;i<=num_clbase;i++) {
       freeze_clbase[i]  = 0;
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* III) Setup torsional transformation for molecules                       */
/*      Subsequently check for inconsistencies                             */

  itrans  = 0;
  iclos   = 0;
  iclbase = 0;

     for (i=1;i<=ntorcompl;i++) {
          if (strcasecmp(torcompl_label[i],"DNA_duplex")==0) {
	      setup_AA_DNA_duplex(maps,pos,general,i,&itrans);
          }/*endif-DNA_duplex*/
          if (strcasecmp(torcompl_label[i],"protein_D6")==0) {
	      setup_AA_protein_D6(maps,pos,general,i,&itrans);
          }/*endif-protein_D6*/
     }/*endfor-i-complex*/

     for (imol_cblc=1;imol_cblc<=nmol_cblc;imol_cblc++) {
        imol = ind_mol_cblc[imol_cblc];
        if (strcasecmp(mol_label[imol],"protein")==0) {
 	  setup_AA_protein_cblc(maps,pos,general,imol,&itrans,&iclos,&iclbase);
        }/*endif*/
        if (strcasecmp(mol_label[imol],"DNA")==0) { 
           if ( cblc[0] == '>' ) {/*regular cblc*/
	     setup_AA_DNA_cblc(maps,pos,general,imol,&itrans,&iclos,&iclbase);
           } else if ( cblc[0] == '~' ) {/*successive cblc*/
             setup_AA_DNA_scblc(maps,pos,general,imol,&itrans,&iclos,&iclbase);
           } else if ( cblc[0] == '-' ) {/*regular cblc on ~ topology*/
             setup_AA_DNA_cblc_top_scblc(maps,pos,general,imol,&itrans,
                                                               &iclos,
                                                               &iclbase);
	   }/*endif*/
        }/*endif*/
        if (strcasecmp(mol_label[imol],"RNA")==0) { 
           if ( cblc[0] == '>' ) {/*regular cblc*/
	     setup_AA_RNA_cblc(maps,pos,general,imol,&itrans,&iclos,&iclbase);
           } else if ( cblc[0] == '~' ) {/*successive cblc*/
             setup_AA_RNA_scblc(maps,pos,general,imol,&itrans,&iclos,&iclbase);
	   }/*endif*/
        }/*endif*/
     }/*endfor*/

     for (imol_free=1;imol_free<=nmol_free;imol_free++) {
        imol = ind_mol_free[imol_free];
        if (strcasecmp(mol_label[imol],"protein")==0) {
 	   setup_AA_protein(maps,pos,general,imol,&itrans);
        }/*endif*/
        if (strcasecmp(mol_label[imol],"DNA")==0) {
 	   setup_AA_DNA_strand(maps,pos,general,imol,&itrans);
        }/*endif*/
        if (strcasecmp(mol_label[imol],"RNA")==0) {
 	   setup_AA_RNA_strand(maps,pos,general,imol,&itrans);
        }/*endif*/
     }/*endfor*/


  if (itrans!=num_trans) {
      sprintf(err_msg,"Tors_defs::setup_model_AA -> itrans != num_trans\n"
                      "itrans = %d, num_trans = %d",itrans,num_trans);
      errorMessage(err_msg);
  }/*endif*/


  if (iclos!=num_clos) {
      sprintf(err_msg,"Tors_defs::setup_model_AA -> iclos != num_clos\n"
                      "iclos = %d, num_clos = %d",iclos,num_clos);
      errorMessage(err_msg);
  }/*endif*/


  if (iclbase!=num_clbase) {
      sprintf(err_msg,"Tors_defs::setup_model_AA -> iclbase != num_clbase\n"
                      "iclbase = %d, num_clbase = %d",iclbase,num_clbase);
      errorMessage(err_msg);
  }/*endif*/


  for (i=1;i<=num_trans;i++) {
    if ( (secondary_only[i]!=0) && (ityp_trans[i]!=0) ) {
          sprintf(err_msg,"Tors_defs::setup_model_AA -> \n"
                          "(secondary_only[%d]!=0) && (ityp_trans[%d]!=0)\n"
                          "secondary_only[%d] = %d,ityp_trans[%d] = %d "
                          ,i,i,i,secondary_only[i],i,ityp_trans[i]);
          errorMessage(err_msg);
    }/*endif*/
  }/*endfor*/  
  

/*-------------------------------------------------------------------------*/
/* IV) Adjust transformation types                                         */
/*     if off_trans == 1 => ityp_trans = 0 (identical)                     */

  for (itrans=1;itrans<=num_trans;itrans++) {
       if (off_trans[itrans] == 1) {ityp_trans[itrans] = 0;}
  }/*end for*/


/*-------------------------------------------------------------------------*/
/* V) Define tbends                                                        */

  num_tbend = 0;
  for (itrans=1;itrans<=num_trans;itrans++) {
       valid = 0;
       jatm = ind_prim_tors[itrans][3]; 
       natm_bonded = 0;
       for (iiatm=1;iiatm<=natm_trans[itrans];iiatm++) {
	    iatm = ind_trans[itrans][iiatm];
            for (ires=1;ires<=pos->nres;ires++) {
	         for (ibond=1;ibond<=pos->residue[ires].nbond;ibond++) {
		      iatmb = pos->residue[ires].bond_atom[ibond][1];
                      jatmb = pos->residue[ires].bond_atom[ibond][2];
                      if ( ((iatm==iatmb)&&(jatm==jatmb)) || 
                           ((iatm==jatmb)&&(jatm==iatmb)) ) {
			     natm_bonded++;
                      }/*endif*/
                 }/*endfor*/      
            }/*endfor*/
            for (iresb=1;iresb<=pos->nresbond;iresb++) {
	         for (ibond=1;ibond<=pos->residue_bond[iresb].nbond;ibond++) {
		      iatmb = pos->residue_bond[iresb].bond_atom[ibond][1];
                      jatmb = pos->residue_bond[iresb].bond_atom[ibond][2];
                      if ( ((iatm==iatmb)&&(jatm==jatmb)) || 
                           ((iatm==jatmb)&&(jatm==iatmb)) ) {
			     natm_bonded++;
                      }/*endif*/  
                 }/*endfor*/	      
            }/*endfor*/
       }/*end for*/
       if (natm_trans[itrans]==natm_bonded) {valid=1;}
       if (valid==1) {ntbend_trans[itrans]   = natm_trans[itrans];}
       else          {ntbend_trans[itrans]   = 0;}
       for (itbend=1;itbend<=ntbend_trans[itrans];itbend++) {
	    num_tbend++;
            ind_tbend_trans[itrans][itbend] = num_tbend;
       }/*endfor*/
  }/*end for*/ 

/*-------------------------------------------------------------------------*/
/* VI) Setup rotate_utils                                                  */

  rotate_utils.setup();

/*-------------------------------------------------------------------------*/
/* VII) Setup closure complexes                                            */

  setup_closcompl(maps,pos,general);


/*-------------------------------------------------------------------------*/
/* VIII) Setup ityp_clos_postprop if postprop_minimize is on               */

  if (general->postprop_minimize==1) {
      for (i=1;i<=num_clos;i++) {
           ityp_clos_postprop[i] =  ityp_clos[i];
      }/*endfor*/  
  }/*endif*/


/*-------------------------------------------------------------------------*/
/* IX) Debug                                                               */


#define DEBUG_CLOS_OFF
#ifdef DEBUG_CLOS
      for (i=1;i<=num_clos;i++) {
	     if (ityp_clos[i] != 0) {
	         ityp_clos[i] = 5;
	     }/*endfor*/
      }/*endfor*/  
#endif


#ifdef DEBUG

   if (general->torcompl_defined==1) {
     for (i=1;i<=ntorcompl;i++) {
       printf("torcompl[%d]: %s\n",i,torcompl[i]);
     }
     for (i=1;i<=ntorcompl;i++) {
       printf("complex %d\n",i);
       for (imol=1;imol<=nmol_torcompl[i];imol++) {
         printf("ind_mol_torcompl[%d][%d] %d, chain %s\n",i,imol,
        ind_mol_torcompl[i][imol],mol_chainid[ind_mol_torcompl[i][imol]]);
       }/*endfor*/
     }/*endfor*/
     for (i=1;i<=nmol_free;i++) {
       printf("ind_mol_free[%d] %d, chain %s\n",i,ind_mol_free[i],
                                     mol_chainid[ind_mol_free[i]]);
     }/*endfor*/          
   }/*endif-torcompl_defined*/   
   printf("\n");

   if (general->stride_defined==1) {
     for (i=1;i<=nstride;i++) {
       printf("stride[%d]: %s\n",i,stride[i]);
     }
   }/*endif*/   
   printf("\n");

  printf("~~~~~~~~~~~~~~~~~~~~~~~~~ All Torsions ~~~~~~~~~~~~~~~~~~~~~~~~~\n");
  write_alldata(maps);
#endif

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/

