#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG


/*=========================================================================*/
 void Tors_defs::setup_cgresmodel_UA(const Maps *maps, const Pos *pos, 
                                                       const General *general)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for the United Atom model          */
/*                                              All non-hydrogen model     */
/*-------------------------------------------------------------------------*/
{/*begin routine*/  
/*-------------------------------------------------------------------------*/

  int ires, itrans;
  int iatm, iiatm, ibranch;  
  int ichigrp;
  int num_trans_backbone;  
  char   err_msg[LINE_NCHARMAX];

  int    nmol        = maps->nmol;

/*-------------------------------------------------------------------------*/
/* 0) Only one molecule is supported                                       */

  if (nmol!=1) {
   sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA: nmol!=1 not supported");
   errorMessage(err_msg);
  }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Get the number of transformations                                    */
/*    Backbone: Assume two transformations/residue, except for first resid.*/
/*    Sidechain: Get the number of chi groups from topology                */

  num_trans = 0;  
  num_trans += 2*((pos->nres)-1); /* Backbone */

  for (ires=1;ires<=pos->nres;ires++) {
       num_trans += pos->residue[ires].nchigrp; /* Sidechain */
  }/*end for*/  

/*-------------------------------------------------------------------------*/
/* II) Allocate variables                                                  */

  res_id         =  new int[num_trans]-1;
  ityp_trans     =  new int[num_trans]-1;
  off_trans      =  new int[num_trans]-1;
  natm_trans     =  new int[num_trans]-1;

  ind_trans      =  cmall_int_mat(1, num_trans, 1, NATM_TRANS_MAX);
  ind_prim_tors  =  cmall_int_mat(1, num_trans, 1, 4);

/*-------------------------------------------------------------------------*/
/* III) Define the groups of atoms to be transformed for proteins          */
/*-------------------------------------------------------------------------*/
/* III.A) Define the groups of backbone atoms                              */

  itrans = 0;
  for (ires=1;ires<=pos->nres;ires++) {
    if (ires==1) {continue;}

     if (strcasecmp(pos->residue[ires].residue_name,"PRO")==0) {
       itrans++;
       if (itrans>num_trans) {
 	 sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1] = pos->residue[ires  ].branch_root_f[0][0];  /* N*/
       ind_trans[itrans][2] = pos->residue[ires-1].branch_root_s[2][0];  /*OC*/
       ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
       ind_prim_tors[itrans][3] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
       ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_f[1][0];/*CA*/
       ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_f[0][0];/*N*/

       itrans++;
       if (itrans>num_trans) {
           sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
           errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 5;
       ind_trans[itrans][1] = pos->residue[ires].branch_root_f[1][0];  /*CA*/
       ind_trans[itrans][2] = pos->residue[ires].branch_root_s[0][0]; /*C*/
       ind_trans[itrans][3] = pos->residue[ires].branch_root_f[1][2]; /*CB*/
       ind_trans[itrans][4] = pos->residue[ires].branch_root_f[2][0];  /*CD*/
       ind_trans[itrans][5] = pos->residue[ires].branch_root_f[2][1];  /*CG*/
       if (ires==pos->nres) {
           natm_trans[itrans] = 6;
           ind_trans[itrans][6] = pos->residue[ires].branch_root_s[2][0];/*OC*/
       }/*end if*/


       ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[1][0];/*CA*/
       ind_prim_tors[itrans][3] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
       ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
       ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_s[1][0];/*CA*/
       continue;
     }/*PRO residue*/

 if (strcasecmp(pos->residue[ires].residue_name,"GLY")==0) {
     itrans++;
     if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
         errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = 3;
     ind_trans[itrans][1] = pos->residue[ires  ].branch_root_f[0][0];  /* N*/
     ind_trans[itrans][2] = pos->residue[ires  ].branch_root_f[1][0];  /*CA*/
     ind_trans[itrans][3] = pos->residue[ires-1].branch_root_s[2][0];  /*OC*/
     ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
     ind_prim_tors[itrans][3] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
     ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_f[1][0];/*CA*/
     ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_f[0][0];/*N*/

     itrans++;
     if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
         errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = 1;
     ind_trans[itrans][1] = pos->residue[ires].branch_root_s[0][0]; /*C*/
     if (ires==pos->nres) {
         natm_trans[itrans] = 2;
         ind_trans[itrans][2] = pos->residue[ires].branch_root_s[2][0];/*OC*/
     }/*end if*/

     ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_s[0][0];/*C*/
     ind_prim_tors[itrans][3] = pos->residue[ires  ].branch_root_s[1][0];/*CA*/
     ind_prim_tors[itrans][2] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
     ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
     continue;
  }/*Gly residue*/

     itrans++;
     if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
         errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = 3;
     ind_trans[itrans][1] = pos->residue[ires  ].branch_root_f[0][0];  /* N*/
     ind_trans[itrans][2] = pos->residue[ires  ].branch_root_f[1][0];  /*CA*/
     ind_trans[itrans][3] = pos->residue[ires-1].branch_root_s[2][0];  /*OC*/
     ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
     ind_prim_tors[itrans][3] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
     ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_f[1][0];/*CA*/
     ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_f[0][0];/*N*/

     itrans++;
     if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
         errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = 2;
     ind_trans[itrans][1] = pos->residue[ires].branch_root_s[0][0]; /*C*/
     ind_trans[itrans][2] = pos->residue[ires].branch_root_s[1][1]; /*CB*/
     if (ires==pos->nres) {
         natm_trans[itrans] = 3;
         ind_trans[itrans][3] = pos->residue[ires].branch_root_s[2][0];/*OC*/
     }/*end if*/

     ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_s[0][0];/*C*/
     ind_prim_tors[itrans][3] = pos->residue[ires  ].branch_root_s[1][0];/*CA*/
     ind_prim_tors[itrans][2] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
     ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
  }/*end for*/
  num_trans_backbone = itrans;

/*-------------------------------------------------------------------------*/
/* III.B) Define the groups of sidechain atoms                             */

  for (ires=1;ires<=pos->nres;ires++) {
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     if (itrans>num_trans) {
        sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
        errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* IV) Transformation types                                                */
/*     By default, everybody is transformed                                */

   for (itrans=1;itrans<=num_trans;itrans++) {
        ityp_trans[itrans] = 1;
        off_trans[itrans]  = 0;
   }/*end for*/

/*-------------------------------------------------------------------------*/
/* V) If stride is defined in .pdb, only sample backbone                   */
/*       along 'C' secondary structure type residue                        */
/*       sidechain torsions are always transformed                         */   

   if (general->stride_defined==1) {
     if (nstride!=1) {
       sprintf(err_msg, "Tors_defs::setup_cgresmodel_UA: nstride!=1");
       errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[1][0]))-1!=pos->nres) {
       sprintf(err_msg, "Tors_defs::setup_cgresmodel_UA: corrupted STRIDE");
       errorMessage(err_msg);
     }/*endif*/
     for (itrans=1;itrans<=num_trans_backbone;itrans++) {
          ityp_trans[itrans] = 1;
          off_trans[itrans]  = 1;
          if (stride[1][res_id[itrans]] == 'C') {
              off_trans[itrans] = 0;
          }/*endif*/
     }/*end for*/
   }/*end if*/


/*-------------------------------------------------------------------------*/
/* VI) Adjust transformation types                                         */
/*     if off_trans == 1 => ityp_trans = 0 (identical)                     */
/*     if off_trans == 0 => ityp_trans = 1 (prop Gauss)                    */

  for (itrans=1;itrans<=num_trans;itrans++) {
    if (off_trans[itrans] == 1) {ityp_trans[itrans] = 0;}
    if (off_trans[itrans] == 0) {ityp_trans[itrans] = 1;}
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* VII) Setup rotate_utils                                                 */

  rotate_utils.setup();

/*-------------------------------------------------------------------------*/
/* VIII) Debug                                                             */

#ifdef DEBUG
  printf("############################ All Torsions ############################\n");
  write_alldata(maps);
#endif

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/

