#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup(System &system)
/*-------------------------------------------------------------------------*/
/*     Sets up all the torsional angles for any model                      */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/* 0) Initialize system.tors_defs_ptr                                      */

        system.tors_defs_ptr = &(system.tors_defs);

/*-------------------------------------------------------------------------*/
/* I) Initialize classes                                                   */

        const Maps *maps       = &(system.maps);
        const Pos  *pos        = &(system.pos);
        const General *general = &(system.general);

/*-------------------------------------------------------------------------*/
/* II) Setup torsional transformations                                     */

        if (general->allatm_model==1) {
	    if (general->prop_tors_type == 1) /* full torsional */
	      {setup_model_AA(maps,pos,general);}
            if (general->prop_tors_type == 2) /* sidechain torsional */
	      {setup_sidechains(maps,pos,general);}
            if (general->postprop_minimize == 1) {
	      if (general->postprop_minimize_type == 8) {/*side_chain*/
		system.tors_defs_sidechain.setup_sidechains(maps,pos,general);
              }/*endif*/
            }/*endif*/
        }
        if (general->cgres_model==1) {
            if (general->cgres_model_1pt==1) {
                setup_cgresmodel_1pt(maps,pos,general);
            }
            if (general->cgres_model_3pt==1) {
                setup_cgresmodel_3pt(maps,pos,general);
            }
            if (general->cgres_model_5pt==1) {
                setup_cgresmodel_5pt(maps,pos,general);
            }
            if (general->cgres_model_UA==1) {
	        if (general->prop_tors_type == 1) /* full torsional */
	            {setup_cgresmodel_UA(maps,pos,general);}
                if (general->prop_tors_type == 2) /* sidechain torsional */
	            {setup_sidechains(maps,pos,general);}
                if ( (general->postprop_minimize      == 1) &&  
	             (general->postprop_minimize_type == 8) ){/*side_chain*/
		 system.tors_defs_sidechain.setup_sidechains(maps,pos,general);
                }/*endif*/
            }/*end if*/
            if (general->cgres_model_UAPH==1) {
	        if (general->prop_tors_type == 1) /* full torsional */
	            {setup_cgresmodel_UAPH(maps,pos,general);}
                if (general->prop_tors_type == 2) /* sidechain torsional */
	            {setup_sidechains(maps,pos,general);}
                if ( (general->postprop_minimize      == 1) &&  
	             (general->postprop_minimize_type == 8) ){/*side_chain*/
		 system.tors_defs_sidechain.setup_sidechains(maps,pos,general);
                }/*endif*/
            }/*end if*/
        }/*end if*/


/*-------------------------------------------------------------------------*/
/* III) Debug torsional definitions                                        */
        
        if (system.general.debug_tors_defs==1) {
            write_alldata(maps);       
        }/*endif*/ 

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/

