#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_sidechains(const Maps *maps, const Pos *pos, 
                                                     const General *general)
/*-------------------------------------------------------------------------*/
/*   Sets up only side chain torsional angles                              */
/*-------------------------------------------------------------------------*/
{/*begin routine*/  
/*-------------------------------------------------------------------------*/

  int iatm, ires, itrans;
  int ichigrp;
/*-------------------------------------------------------------------------*/
/* I) Get the number of transformations                                    */

  num_trans = 0;  
  for (ires=1;ires<=pos->nres;ires++) {
       num_trans += pos->residue[ires].nchigrp;
  }/*end for*/  

/*-------------------------------------------------------------------------*/
/* II) Allocate variables                                                  */

  res_id         =  new int[num_trans]-1;
  ityp_trans     =  new int[num_trans]-1;
  off_trans      =  new int[num_trans]-1;
  natm_trans     =  new int[num_trans]-1;

  ind_trans      =  cmall_int_mat(1, num_trans, 1, NATM_TRANS_MAX);
  ind_prim_tors  =  cmall_int_mat(1, num_trans, 1, 4);

  /*  later for REPSWA  */
  /*  ityp_prim_tors =  new int[num_trans]-1;                              */
  /*  ityp_prim_onfo =  new int[num_trans]-1;                              */
  /*  nsec_tors      =  new int[num_trans]-1;                              */
  /*  prim_upper_lim =  new int[num_trans]-1;                              */
  /*  prim_lower_lim =  new int[num_trans]-1;                              */
  /*  ind_sec_tors   =  cmall_itens3(1, num_trans, 1, NSEC_TORS_MAX, 1, 4);*/   
  /*  ityp_sec_tors  =  cmall_int_mat(1, num_trans, 1, NSEC_TORS_MAX);     */
  /*  ityp_sec_onfo  =  cmall_int_mat(1, num_trans, 1, NSEC_TORS_MAX);     */
  
/*-------------------------------------------------------------------------*/
/* III) Get the group of atoms to be transformed                           */

  itrans = 0;
  for (ires=1;ires<=pos->nres;ires++) {
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* IV) Transformation types                                                */

  for (itrans=1;itrans<=num_trans;itrans++) {
       ityp_trans[itrans] = 1;
       off_trans[itrans] = 0;
  }/*end for*/

/*-------------------------------------------------------------------------*/
/* V) Setup rotate_utils                                                   */

  rotate_utils.setup();
  
/*-------------------------------------------------------------------------*/
/* VI) Debug                                                               */

#ifdef DEBUG
  printf("############################ Sidechains ############################\n");
  write_alldata(maps);
#endif

/*-------------------------------------------------------------------------*/
} /*end routine */
/*-------------------------------------------------------------------------*/
/*=========================================================================*/

