#include "../../include/standard.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Pos::tors_transformBFBtoXFX()
/*-------------------------------------------------------------------------*/
/*  transformation from torsional (r,theta,phi) to cartesian (x,y,z) space */
/*  along the predefined torsions available from class system->tors_defs   */
/*  and bends are initialized from b                                       */
/*                                                                         */
/*  Effected class variables:                                              */
/*  $i $ = {x,y,z}                                                         */
/*  f$ $ = {x,y,z}                                                         */
/*  bends are initialized from b (bends)                                   */
/*  bend forces are initialized from fb (bend forces)                      */
/*  note here bend refers to theta, or is Pi - bend                        */
/*-------------------------------------------------------------------------*/
{/*begin routine*/  
/*-------------------------------------------------------------------------*/

 int iatm, level, itbend;                         

 System *system = this->system_ptr;                                       

 General   *general   = &(system->general); 
 Tors_defs *tors_defs = &(system->tors_defs); 
 int num_tr           =  tors_defs->num_trans; 
 int **ind_group      =  tors_defs->ind_trans;
 int *natm_trans      =  tors_defs->natm_trans;
 int **ind_tbend_trans =  tors_defs->ind_tbend_trans;
 int *ntbend_trans     =  tors_defs->ntbend_trans;

/*-------------------------------------------------------------------------*/
/*                       Transformation loop                               */ 
/*-------------------------------------------------------------------------*/

   for (level = 1;level <= num_tr;level++) {/*forward half*/
       /* i) y == Pi - bend  <- theta   */
       /*    fy              <- f_theta */
          for (itbend=1;itbend<=ntbend_trans[level];itbend++) {
	     iatm = itbend;
             yi[ind_group[level][iatm]] = b[ind_tbend_trans[level][itbend]];
             fy[ind_group[level][iatm]] = fb[ind_tbend_trans[level][itbend]];
          }/*endfor*/
       /* ii) ungroup atoms */
          group(this,tors_defs,level,-2);
       /* iii) backward resolve */
          resolve(this,tors_defs,level,-2);
       /* iv) backward  rotate */
          rotate(this,tors_defs,level,-2);
       /* v) backward translate */
          translate(this,tors_defs,level,-2);   
   }/*end of forward half*/

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 
