#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Pos::tors_transformXFXtoBFB()
/*-------------------------------------------------------------------------*/
/*  transformation from cartesian (x,y,z) to torsional (r,theta,phi) space */
/*  along the predefined torsions available from  class system->tors_defs  */
/*                                                                         */
/*  Effected class variables:                                              */
/*  $i $ = {x,y,z}                                                         */
/*  f$ $ = {x,y,z}                                                         */
/*  b (bends), fb (force along bends)                                      */
/*  note that here bend refers to theta, which is Pi - bend                */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

 int  iatm, level, itbend;                         

 System *system = this->system_ptr;                                       

 General   *general   = &(system->general);
 Tors_defs *tors_defs = &(system->tors_defs);
 int num_tr           =  tors_defs->num_trans; 
 int **ind_group      =  tors_defs->ind_trans;
 int *natm_trans      =  tors_defs->natm_trans;
 int **ind_tbend_trans =  tors_defs->ind_tbend_trans;
 int *ntbend_trans     =  tors_defs->ntbend_trans;

/*-------------------------------------------------------------------------*/
/*                       Transformation loop                               */ 
/*-------------------------------------------------------------------------*/

   for (level = num_tr;level >= 1;level--) {/*backward half*/
       /* 1st three steps involve primary tors and group atoms */
       /* i)forward translate */
          translate(this,tors_defs,level,2);  
       /* ii)forward rotate */
          rotate(this,tors_defs,level,2); 
       /* iii)forward resolve */
          resolve(this,tors_defs,level,2);
       /* iv) group atoms */
          group(this,tors_defs,level,2);
       /* v) theta   <- y == Pi - bend  */
       /*    f_theta <- fy              */
          for (itbend=1;itbend<=ntbend_trans[level];itbend++) {
	     iatm = itbend;
             b[ind_tbend_trans[level][itbend]]  = yi[ind_group[level][iatm]];
             fb[ind_tbend_trans[level][itbend]] = fy[ind_group[level][iatm]];
          }/*endfor*/
   }/*end of backward half*/ 

/*--------------------------------------------------------------------------*/
}/*                              End                                        */ 
/*--------------------------------------------------------------------------*/ 
