#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#ifdef DEBUG
  printf(" in prop_tors\n");
  printf(" torsion %g\n",uz[patm]);
  getchar();
#endif

#define DEBUG
#undef DEBUG

void adjust(Pos *pos,System &system,int i_trans,int type) 

/*-------------------------------------------------------------------------*/
/*                                 adjust                                  */
/* type ==  1 : position forward                                           */
/* type == -1 : position backward                                          */
/* type ==  2 : position + force forward (not supported)                   */
/* type == -2 : position + force backward (not supported)                  */
/*                                                                         */
/* Based on type and certain properties of individual transformations      */
/* this routine is making adjustments on transformation groups             */
/*-------------------------------------------------------------------------*/
/*ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
  {/*begin*/ 
/*-------------------------------------------------------------------------*/

  int iatm,jatm,patm,plevel;
  int  natm_group_save;  

  double *ux  = pos->xi;
  double *uy  = pos->yi;
  double *uz  = pos->zi;

  double *x   = pos->x;
  double *y   = pos->y;
  double *z   = pos->z;

  Tors_defs *tors_defs = &(system.tors_defs);

  int allatm_model    = system.general.allatm_model;
  int prop_tors_type  = system.general.prop_tors_type;
  int *secondary_only = tors_defs->secondary_only;
  int **ind_prim_tors = tors_defs->ind_prim_tors;
  int **ind_group     = tors_defs->ind_trans;
  int *natm_group     = tors_defs->natm_trans;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* I) Adjust if secondary_only[i_trans] != 0                               */
/*    If type == 1                                                         */
/*       We restore the primary atom in the group using the 'x y z'        */
/*       coordinate set because this atom is also part of a subsequent     */
/*       (in the forward half) transformation group that transforms the    */
/*       primary atom                                                      */
/*    If type == -1                                                        */
/*       We translate+rotate+resolve the primary atom in the group         */
/*       because this atom has been already built in the backward half     */
/*                                                                         */
/*       Currently adjustments are only supported in all atom models       */
/*       and full torsional propagations                                   */

 if ((allatm_model==1) && (prop_tors_type==1)) {
  if (secondary_only[i_trans]!=0) {  
      patm = ind_group[i_trans][1];
      switch(type){
         case -2:
           sprintf(err_msg,"From void adjust, currently adjustments are not\n"
                           "supporting force transformations");
           errorMessage(err_msg);
           break;
         case -1:
           natm_group_save = natm_group[i_trans];
           natm_group[i_trans] = 1; 
           translate(pos,tors_defs,i_trans,1);
           rotate(pos,tors_defs,i_trans,1);
           resolve(pos,tors_defs,i_trans,1);
           natm_group[i_trans] = natm_group_save;   
           break;
         case 1:
              /* (ux,uy,uz) = (r theta phi) */
              /* (x,y,z) = (x, y, z)        */
              ux[patm] = x[patm];
              uy[patm] = y[patm];
              uz[patm] = z[patm];
           break;
         case 2:
           sprintf(err_msg,"From void adjust, currently adjustments are not\n"
                           "supporting force transformations");
           errorMessage(err_msg);
           break;
      }/*end switch*/ 
  }/*endif*/
 }/*endif*/
/*-------------------------------------------------------------------------*/
  }/*end of adjust*/
/*-------------------------------------------------------------------------*/



void adjust(Pos *pos,System &system,int type) 

/*-------------------------------------------------------------------------*/
/*                                 adjust                                  */
/* this function loops through all transformation groups                   */
/* type ==  1 : executed before forward half                               */
/* type == -1 : executed after backward half                               */
/*                                                                         */
/* Based on type and certain properties of individual transformations      */
/* this routine is making adjustments on transformation groups             */
/*-------------------------------------------------------------------------*/
/*ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
  {/*begin*/ 
/*-------------------------------------------------------------------------*/

  int iatm,jatm,patm,plevel,i_trans;
  int  natm_group_save,ip_trans,ip_atom;;

  double *ux  = pos->xi;
  double *uy  = pos->yi;
  double *uz  = pos->zi;

  double *x   = pos->x;
  double *y   = pos->y;
  double *z   = pos->z;

  Tors_defs *tors_defs = &(system.tors_defs);

  int allatm_model    = system.general.allatm_model;
  int prop_tors_type  = system.general.prop_tors_type;
  int  num_trans      = tors_defs->num_trans;
  int *secondary_only = tors_defs->secondary_only;
  int **ind_prim_tors = tors_defs->ind_prim_tors;
  int **ind_group     = tors_defs->ind_trans;
  int *natm_group     = tors_defs->natm_trans;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/*       Currently adjustments are only supported in all atom models       */
/*       and full torsional propagations                                   */

/*-------------------------------------------------------------------------*/
/* I) Adjust if secondary_only[i_trans] != 0                               */
/*    If type == 1                                                         */
/*       We insert the primary atom in the group using the pointer         */
/*       ip_trans to the corresponding primary torsion and this type       */
/*       should be executed before the forward half transformation cycle   */
/*                                                                         */
/*       The corresponding primary torsion could coincide with the current */
/*       torsion (ip_trans == i_trans), if the dihedral is crossing        */ 
/*       closure boundaries. In this case, the corresponding primary atom  */
/*       is stored in ind_group[i_trans][natm_group[i_trans]+1]            */
/*    If type == -1                                                        */
/*       Changes produced by type 1 is restored                            */
/*                                                                         */

 if ((allatm_model==1) && (prop_tors_type==1)) {
  for (i_trans=1;i_trans<=num_trans;i_trans++) {
    if (secondary_only[i_trans]!=0) {  
        patm = ind_group[i_trans][1];
        switch(type){
	  case 1:
             ip_trans = secondary_only[i_trans];
             if (ip_trans == i_trans) {
                 ip_atom = ind_group[i_trans][natm_group[i_trans]+1];
             }/*endif*/
             for (iatm=natm_group[i_trans];iatm>=1;iatm--) {
	          ind_group[i_trans][iatm+1] = ind_group[i_trans][iatm];  
             }/*endfor*/
             if (ip_trans == i_trans) {
                 ind_group[i_trans][1] =  ip_atom;
             } else {
                 ind_group[i_trans][1] =  ind_group[ip_trans][1];
             }/*endif*/
             natm_group[i_trans]++;
             ind_prim_tors[i_trans][4] = ind_group[i_trans][1]; 
	     break;
	  case -1:
             ip_trans = secondary_only[i_trans];
             if (ip_trans == i_trans) {
                 ip_atom = ind_group[i_trans][1];
             }/*endif*/
             natm_group[i_trans]--;
             for (iatm=1;iatm<=natm_group[i_trans];iatm++) {
	          ind_group[i_trans][iatm] = ind_group[i_trans][iatm+1];  
             }/*endfor*/
             if (ip_trans == i_trans) {
                 ind_group[i_trans][natm_group[i_trans]+1] = ip_atom;
             }/*endif*/
             ind_prim_tors[i_trans][4] = ind_group[i_trans][1];
	     break;
        }/*end switch*/
    }/*endif*/
  }/*endfor*/
 }/*endif*/
/*-------------------------------------------------------------------------*/
}/*end of adjust*/
/*-------------------------------------------------------------------------*/



