#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

#ifdef DEBUG
  printf(" in prop_tors\n");
  printf(" torsion %g\n",uz[patm]);
  getchar();
#endif

#define DEBUG
#undef DEBUG

/*-------------------------------------------------------------------------*/
/*                          group                                          */
/* type ==  1 : position forward                                           */
/* type == -1 : position backward                                          */
/* type ==  2 : position + force forward                                   */
/* type == -2 : position + force backward                                  */
/*-------------------------------------------------------------------------*/
/*ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*-------------------------------------------------------------------------*/

void group(Pos *pos,Tors_defs *tors_defs,int i_trans,int type) 

/*-------------------------------------------------------------------------*/
  {/*begin*/ 
/*-------------------------------------------------------------------------*/

  int iatm,jatm,patm;
 
  /*       For REPSWA       */
  /* double *ux  = pos.ux;  */
  /* double *uy  = pos.uy;  */
  /* double *uz  = pos.uz;  */
  /* double *fux = pos.fux; */
  /* double *fuy = pos.fuy; */
  /* double *fuz = pos.fuz; */

  double *tors_X = pos->tors_X;
  double *ux  = pos->xi;
  double *uy  = pos->yi;
  double *uz  = pos->zi;
  double *fux = pos->fx;
  double *fuy = pos->fy;
  double *fuz = pos->fz;

  int **ind_prim_tors = tors_defs->ind_prim_tors;
  int **ind_group     = tors_defs->ind_trans;
  int *natm_group     = tors_defs->natm_trans;

/*-------------------------------------------------------------------------*/
/*        Group atoms together to transform as a rigid body                */
/*          ind_group[][1] must be a primary atom                          */ 
  patm = ind_group[i_trans][1]; 
  switch(type){
     case -2:
       for (iatm=2;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
           jatm = ind_group[i_trans][iatm]; 
           /* (ux,uy,uz) = (r theta phi) */
           uz[jatm] = uz[jatm]+uz[patm];
           /* (fux,fuy,fuz) = (f_r,f_theta,f_phi) */ 
           fuz[patm] = fuz[patm]-fuz[jatm];
       }/*group atoms*/
       break;
     case -1:
       for (iatm=2;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
           jatm = ind_group[i_trans][iatm]; 
           /* (ux,uy,uz) = (r theta phi) */
           uz[jatm] = uz[jatm]+uz[patm];
       }/*group atoms*/
       break;
     case 1:
       for (iatm=2;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
          jatm = ind_group[i_trans][iatm]; 
          /* (ux,uy,uz) = (r theta phi) */
          uz[jatm] = uz[jatm]-uz[patm];
       }/*group atoms*/
       break;
     case 2:
       for (iatm=2;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
          jatm = ind_group[i_trans][iatm]; 
          /* (ux,uy,uz) = (r theta phi) */
          uz[jatm] = uz[jatm]-uz[patm];
          /* (fux,fuy,fuz) = (f_r,f_theta,f_phi) */ 
          fuz[patm] = fuz[patm]+fuz[jatm];
       }/*group atoms*/
       break;
   }/*end switch*/ 
/*-------------------------------------------------------------------------*/
/* Record the primary torsion                                              */
/*          ind_group[][1] must be a primary atom                          */ 
   tors_X[i_trans] = uz[patm];
/*-------------------------------------------------------------------------*/
  }/*end of repswa_group*/
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*                          group                                          */
/*          Propagate torsinal space with a gauss distribution             */
/*-------------------------------------------------------------------------*/
/*ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*-------------------------------------------------------------------------*/

void prop_tors(Pos *pos,General *general,Tors_defs *tors_defs,int i_trans) 

/*-------------------------------------------------------------------------*/
  {/*begin*/ 
/*-------------------------------------------------------------------------*/

  int patm;
  double sig, sig_scale;

  int num_tr             =  tors_defs->num_trans; 
  int *ityp_tr           =  tors_defs->ityp_trans; 
  int *off_tr            =  tors_defs->off_trans; 
  int **ind_group        =  tors_defs->ind_trans;
  double *prop_sig_scale =  tors_defs->prop_sig_scale;

  double *ux  = pos->xi;
  double *uy  = pos->yi;
  double *uz  = pos->zi;
  double *fux = pos->fx;
  double *fuy = pos->fy;
  double *fuz = pos->fz;
  
  double T0   =  general->T;                    /* Repli.0 temperature in K */
  double T    = (pos->T)*BOLTZ;                 /* Replica temperature in K */
  double TpT0 =  T/T0;

  int nres_ac = general->nres;                  /* Actual Residue Length    */
  int nres_st = 100;                            /* Standard Residue Length  */
  double slen = ((double)(nres_st))/((double)(nres_ac));   /* Length Scale  */
  double slen2= slen*slen; 

#ifdef DEBUG
  printf(" in prop_tors\n");
  printf(" T0 %g\n",T0);
  printf(" in prop_tors\n");
  printf(" T %g\n",T);
  getchar();
#endif

  long *seed  = &(general->seed);
  double one  = 1.0;


/*-------------------------------------------------------------------------*/
/* 0) Define sigma  Both T and Length scale                                */
/*    models with scaling options:                                         */
/*                                 a) all_atom full torsional              */
/*                                 b) cgres with cblc                      */

  sig_scale = 1.0;
  if ( (general->allatm_model==1) &&  (general->prop_tors_type==1) ) 
     {sig_scale = prop_sig_scale[i_trans];}
  if ( (general->cgres_model==1) &&  (general->cblc_defined==1) ) 
     {sig_scale = prop_sig_scale[i_trans];}



  if ( general->cblc_defined == 1 ) { 
     sig  = sig_scale*slen2*sqrt(sqrt(sqrt(TpT0)))*general->prop_tors_sig;
  } else {  
     sig  = sig_scale*slen2*TpT0*general->prop_tors_sig;
  }/*endif*/
/*-------------------------------------------------------------------------*/
/* I) Propagate the torsion angle                                          */
/*    Normal distribution with zero mean:                                  */
/*      f(x) = 1/(sig*sqrt(2Pi)) exp(-x^2/(2 sig^2))                       */
/*      double mb(1,sig,seed) from f(x)                                    */

   patm = ind_group[i_trans][1]; 

   uz[patm] += mb(one,sig,seed);

/*-------------------------------------------------------------------------*/
/* II) Applying periodic boundaries in [0,2 Pi)                            */

   if (uz[patm] < 0) {
       uz[patm] += 2.0*M_PI;
   }/*endif*/ 

   if (uz[patm] >= 2.0*M_PI) {
       uz[patm] -= 2.0*M_PI;
   }/*endif*/ 

/*-------------------------------------------------------------------------*/
  }/*end of prop_tors*/
/*-------------------------------------------------------------------------*/
