#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"

/*-------------------------------------------------------------------------*/
/*                          resolve                                        */
/* type ==  1 : position forward                                           */
/* type == -1 : position backward                                          */
/* type ==  2 : position + force forward                                   */
/* type == -2 : position + force backward                                  */
/*-------------------------------------------------------------------------*/
/*ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*-------------------------------------------------------------------------*/

void resolve(Pos *pos,Tors_defs *tors_defs,int i_trans,int type) 

/*-------------------------------------------------------------------------*/
  {/*begin*/ 
/*-------------------------------------------------------------------------*/

  int j1,j2,j3,j4; 
  int iatm,jatm;
  double r,theta,phi;
  double ux4,uy4,uz4;
  double r2d;
  double f_r,f_theta,f_phi;
  double fux4,fuy4,fuz4;
  double sin_theta,cos_theta,sin_phi,cos_phi;
 
  /*       For REPSWA       */
  /* double *ux  = pos.ux;  */
  /* double *uy  = pos.uy;  */
  /* double *uz  = pos.uz;  */
  /* double *fux = pos.fux; */
  /* double *fuy = pos.fuy; */
  /* double *fuz = pos.fuz; */

  double *ux  = pos->xi;
  double *uy  = pos->yi;
  double *uz  = pos->zi;
  double *fux = pos->fx;
  double *fuy = pos->fy;
  double *fuz = pos->fz;

  int **ind_prim_tors = tors_defs->ind_prim_tors;
  int **ind_group     = tors_defs->ind_trans;
  int *natm_group     = tors_defs->natm_trans;

/*-------------------------------------------------------------------------*/
/* In order to employ REPSWA for the torsion angle : Convert to            */
/*  spherical polar coordinates where phi is the torsion angle             */
/*  e.g. the in-plane angle */

  switch(type){
     case -2:
       for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
           jatm = ind_group[i_trans][iatm]; 
           /* resolve (r theta phi) -> (ux4,uy4,uz4) */
           ux4 = ux[jatm]*sin(uy[jatm])*cos(uz[jatm]);
           uy4 = ux[jatm]*sin(uy[jatm])*sin(uz[jatm]);
           uz4 = ux[jatm]*cos(uy[jatm]);
           /* resolve (f_r,f_theta,f_phi) -> (fux_4,fuy_4,fuz_4) */ 
           r = sqrt(ux4*ux4+uy4*uy4+uz4*uz4);
           sin_theta = sin(uy[jatm]);
           cos_theta = cos(uy[jatm]);
           sin_phi = sin(uz[jatm]);
           cos_phi = cos(uz[jatm]);
           fux4 = fux[jatm]*sin_theta*cos_phi +
                  fuy[jatm]*cos_theta*cos_phi/r -
                  fuz[jatm]*sin_phi/(r*sin_theta);
           fuy4 = fux[jatm]*sin_theta*sin_phi +
                  fuy[jatm]*cos_theta*sin_phi/r +
                  fuz[jatm]*cos_phi/(r*sin_theta);
           fuz4 = fux[jatm]*cos_theta -
                  fuy[jatm]*sin_theta/r;
           /* assign the positions, forces */ 
           ux[jatm] = ux4;
           uy[jatm] = uy4;
           uz[jatm] = uz4;
           fux[jatm] = fux4;
           fuy[jatm] = fuy4;
           fuz[jatm] = fuz4;
       }/*group atoms*/
       break;
     case -1:
       for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
           jatm = ind_group[i_trans][iatm]; 
           /* resolve (r theta phi) -> (ux4,uy4,uz4) */ 
           ux4 = ux[jatm]*sin(uy[jatm])*cos(uz[jatm]);
           uy4 = ux[jatm]*sin(uy[jatm])*sin(uz[jatm]);
           uz4 = ux[jatm]*cos(uy[jatm]);
           /* assign the positions, forces */ 
           ux[jatm] = ux4;
           uy[jatm] = uy4;
           uz[jatm] = uz4;
       }/*group atoms*/
       break;
     case 1:
       for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
          jatm = ind_group[i_trans][iatm]; 
          /* resolve  (ux4,uy4,uz4) -> (r theta phi) */
          r     = sqrt(ux[jatm]*ux[jatm]+uy[jatm]*uy[jatm]+uz[jatm]*uz[jatm]);
          theta = acos(uz[jatm]/r);
          r2d   = sqrt(ux[jatm]*ux[jatm]+uy[jatm]*uy[jatm]);
          phi   = acos(ux[jatm]/r2d);
          if ( uy[jatm] < 0.0 ) phi = 2.0*M_PI-phi;
             /* assign the positions */ 
          ux[jatm] = r;
          uy[jatm] = theta;
          uz[jatm] = phi;
       }/*group atoms*/
       break;
     case 2:
       for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
          jatm = ind_group[i_trans][iatm]; 
          /* resolve  (ux4,uy4,uz4) -> (r theta phi) */
          r     = sqrt(ux[jatm]*ux[jatm]+uy[jatm]*uy[jatm]+uz[jatm]*uz[jatm]);
          theta = acos(uz[jatm]/r);
          r2d   = sqrt(ux[jatm]*ux[jatm]+uy[jatm]*uy[jatm]);
          phi   = acos(ux[jatm]/r2d);
          if ( uy[jatm] < 0.0 ) phi = 2.0*M_PI-phi;
          /* resolve  (fux_4,fuy_4,fuz_4) -> (f_r,f_theta,f_phi) */ 
          f_r = fux[jatm]*sin(theta)*cos(phi) +
                fuy[jatm]*sin(theta)*sin(phi) +
                fuz[jatm]*cos(theta);
          f_theta =  fux[jatm]*r*cos(theta)*cos(phi) +
                     fuy[jatm]*r*cos(theta)*sin(phi) -
                     fuz[jatm]*r*sin(theta);
          f_phi = -fux[jatm]*r*sin(theta)*sin(phi) +
                   fuy[jatm]*r*sin(theta)*cos(phi);
          /* assign the positions, forces */ 
          ux[jatm] = r;
          uy[jatm] = theta;
          uz[jatm] = phi;
          fux[jatm] = f_r;
          fuy[jatm] = f_theta;
          fuz[jatm] = f_phi;
       }/*group atoms*/
       break;
   }/*end switch*/ 

/*-------------------------------------------------------------------------*/
  }/*end of translate*/
/*-------------------------------------------------------------------------*/
