#include "../include/standard.h" 
#include "../include/typedefs.h"
#include "../include/definitions.h"
#include "../include/utillib.h"
#include "../include/cmalloc.h"



/*-------------------------------------------------------------------------*/
/*                            rotate                                       */
/* type ==  1 : position forward                                           */
/* type == -1 : position backward                                          */
/* type ==  2 : position + force forward                                   */
/* type == -2 : position + force backward                                  */
/*-------------------------------------------------------------------------*/
/*ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*-------------------------------------------------------------------------*/

void rotate(Pos *pos,Tors_defs *tors_defs,int i_trans,int type) 

/*-------------------------------------------------------------------------*/
  {/*begin*/ 
/*-------------------------------------------------------------------------*/

  int j1,j2,j3,j4; 
  int i,j,iatm,jatm;
  int ndim;
  double e1_norm2;
  double e2_norm2;
  double ey_norm2;

  /*      For REPSWA        */
  /* double *ux  = pos.ux;  */
  /* double *uy  = pos.uy;  */
  /* double *uz  = pos.uz;  */
  /* double *fux = pos.fux; */
  /* double *fuy = pos.fuy; */
  /* double *fuz = pos.fuz; */

  double *ux  = pos->xi;
  double *uy  = pos->yi;
  double *uz  = pos->zi;
  double *fux = pos->fx;
  double *fuy = pos->fy;
  double *fuz = pos->fz;

  int **ind_prim_tors = tors_defs->ind_prim_tors;
  int **ind_group     = tors_defs->ind_trans;
  int *natm_group     = tors_defs->natm_trans;

/*-------------------------------------------------------------------------*/
/* variables which were available in crotate class */

  double*  a   =  tors_defs->rotate_utils.a;
  double*  b   =  tors_defs->rotate_utils.b;
  double** A   =  tors_defs->rotate_utils.A;
  double** B   =  tors_defs->rotate_utils.B;
  double** C   =  tors_defs->rotate_utils.C;
  double** I   =  tors_defs->rotate_utils.I;

  double*  vec1scr_3x1 = tors_defs->rotate_utils.vec1scr_3x1;
  double*  vec2scr_3x1 = tors_defs->rotate_utils.vec2scr_3x1;
  double** mat1scr_3x3 = tors_defs->rotate_utils.mat1scr_3x3;
  double** mat2scr_3x3 = tors_defs->rotate_utils.mat2scr_3x3;

  double*  r1  =  tors_defs->rotate_utils.r1;
  double*  r2  =  tors_defs->rotate_utils.r2;
  double*  r3  =  tors_defs->rotate_utils.r3;
  double*  r4  =  tors_defs->rotate_utils.r4;
  double*  r4_ =  tors_defs->rotate_utils.r4_;
  double*  f1  =  tors_defs->rotate_utils.f1;
  double*  f2  =  tors_defs->rotate_utils.f2;
  double*  f3  =  tors_defs->rotate_utils.f3;
  double*  f4  =  tors_defs->rotate_utils.f4;

  double** R        =  tors_defs->rotate_utils.R;
  double** RT       =  tors_defs->rotate_utils.RT;
  double** dr4_dr1  =  tors_defs->rotate_utils.dr4_dr1;
  double** dr4_dr2  =  tors_defs->rotate_utils.dr4_dr2;
  double** dr4_dr3  =  tors_defs->rotate_utils.dr4_dr3;

  double*  e1  =  tors_defs->rotate_utils.e1;
  double*  e2  =  tors_defs->rotate_utils.e2;
  double*  ex  =  tors_defs->rotate_utils.ex;
  double*  ey  =  tors_defs->rotate_utils.ey;
  double*  ez  =  tors_defs->rotate_utils.ez;

  double** de1_dr2  =  tors_defs->rotate_utils.de1_dr2;
  double** de1_dr3  =  tors_defs->rotate_utils.de1_dr3;
  double** de2_dr1  =  tors_defs->rotate_utils.de2_dr1;
  double** de2_dr2  =  tors_defs->rotate_utils.de2_dr2;
  double** dey_de1  =  tors_defs->rotate_utils.dey_de1;
  double** dey_de2  =  tors_defs->rotate_utils.dey_de2;
  double** dex_dey  =  tors_defs->rotate_utils.dex_dey;
  double** dex_dez  =  tors_defs->rotate_utils.dex_dez;

  double** dex_dr1  =  tors_defs->rotate_utils.dex_dr1;
  double** dex_dr2  =  tors_defs->rotate_utils.dex_dr2;
  double** dex_dr3  =  tors_defs->rotate_utils.dex_dr3;
  double** dey_dr1  =  tors_defs->rotate_utils.dey_dr1;
  double** dey_dr2  =  tors_defs->rotate_utils.dey_dr2;
  double** dey_dr3  =  tors_defs->rotate_utils.dey_dr3;
  double** dez_dr1  =  tors_defs->rotate_utils.dez_dr1;
  double** dez_dr2  =  tors_defs->rotate_utils.dez_dr2;
  double** dez_dr3  =  tors_defs->rotate_utils.dez_dr3;
  
  double** dR_dr1x  =  tors_defs->rotate_utils.dR_dr1x;
  double** dR_dr1y  =  tors_defs->rotate_utils.dR_dr1y;
  double** dR_dr1z  =  tors_defs->rotate_utils.dR_dr1z;
  double** dR_dr2x  =  tors_defs->rotate_utils.dR_dr2x;
  double** dR_dr2y  =  tors_defs->rotate_utils.dR_dr2y;
  double** dR_dr2z  =  tors_defs->rotate_utils.dR_dr2z;
  double** dR_dr3x  =  tors_defs->rotate_utils.dR_dr3x;
  double** dR_dr3y  =  tors_defs->rotate_utils.dR_dr3y;
  double** dR_dr3z  =  tors_defs->rotate_utils.dR_dr3z;

  double** dRT_dr1x  =  tors_defs->rotate_utils.dRT_dr1x;
  double** dRT_dr1y  =  tors_defs->rotate_utils.dRT_dr1y;
  double** dRT_dr1z  =  tors_defs->rotate_utils.dRT_dr1z;
  double** dRT_dr2x  =  tors_defs->rotate_utils.dRT_dr2x;
  double** dRT_dr2y  =  tors_defs->rotate_utils.dRT_dr2y;
  double** dRT_dr2z  =  tors_defs->rotate_utils.dRT_dr2z;
  double** dRT_dr3x  =  tors_defs->rotate_utils.dRT_dr3x;
  double** dRT_dr3y  =  tors_defs->rotate_utils.dRT_dr3y;
  double** dRT_dr3z  =  tors_defs->rotate_utils.dRT_dr3z;
/*-------------------------------------------------------------------------*/
/*                4-3-2-1   : R_3    : origin                              */
/*                          : z-axis : R_23 about which you tors           */
/*                          : x-axis : linear comb R12 R23 perp to z       */
/*                          : y-axis : R23 x R12                           */
/*                          : e1     : R_23                                */
/*                          : e2     : R_12                                */
/* in u() R3 is the origin                                                 */
/* Define coordinate system in term of 1-2-3                               */
/* Place atom 4 and its group in that coordinate system                    */
/* Rotate about the z-axis of the internal cooridinate system defined      */
/* by 1-2-3.                                                               */
/*-------------------------------------------------------------------------*/
/* primary torsion: (j1,j2,j3,j4) */
      j1 = ind_prim_tors[i_trans][1]; 
      j2 = ind_prim_tors[i_trans][2]; 
      j3 = ind_prim_tors[i_trans][3]; 
      j4 = ind_prim_tors[i_trans][4]; 
/*-------------------------------------------------------------------------*/
/* DEB */ 
/*      printf("j1 %d\n",j1);
/*      getchar(); 
/* DEB */ 

    
/* assign base: (r1, r2, r3) */
      r1[1] = ux[j1]; r1[2] = uy[j1]; r1[3] = uz[j1];
      r2[1] = ux[j2]; r2[2] = uy[j2]; r2[3] = uz[j2];
      r3[1] = ux[j3]; r3[2] = uy[j3]; r3[3] = uz[j3];
/*-------------------------------------------------------------------------*/
/* assign base forces: (f1, f2, f3) */
      f1[1] = fux[j1]; f1[2] = fuy[j1]; f1[3] = fuz[j1];
      f2[1] = fux[j2]; f2[2] = fuy[j2]; f2[3] = fuz[j2];
      f3[1] = fux[j3]; f3[2] = fuy[j3]; f3[3] = fuz[j3];

/*-------------------------------------------------------------------------*/
/* assign e1, e2 and get the derivatives wrt r1, r2, r3 */ 
      ndim = 3;
      e1_norm2 = 0.0;
      e2_norm2 = 0.0;
      for (i=1;i<=ndim;i++) {
          e1[i] = r3[i]-r2[i];
          e2[i] = r1[i]-r2[i];
          e1_norm2 += e1[i]*e1[i];
          e2_norm2 += e2[i]*e2[i];
      }
      for (i=1;i<=ndim;i++) {
          e1[i] /= sqrt(e1_norm2);
          e2[i] /= sqrt(e2_norm2);
      }

/*-------------------------------------------------------------------------*/
/* construct de1_dr2,de1_dr3,de2_dr1,de2_dr2 3x3 matrices                  */ 
      de_drb_(r3,r2,e1,de1_dr2);
      de_dra_(r3,r2,e1,de1_dr3);
      de_dra_(r1,r2,e2,de2_dr1);
      de_drb_(r1,r2,e2,de2_dr2);

/*-------------------------------------------------------------------------*/
/* get the unit vectors ez, ey, ex : from e1 and e2                        */ 
      for (i=1;i<=ndim;i++){ez[i] = e1[i];}
      cprd(e1[1],e1[2],e1[3],e2[1],e2[2],e2[3],&(ey[1]),&(ey[2]),&(ey[3]));
/* construct dey_de1,dey_de2 3x3 matrices */ 
      d_axb_n_da_(ey,e2,dey_de1,mat1scr_3x3,mat2scr_3x3);
      d_axb_n_db_(ey,e1,dey_de2,mat1scr_3x3,mat2scr_3x3);
/* normalize ey */ 
      ey_norm2 = ey[1]*ey[1]+ey[2]*ey[2]+ey[3]*ey[3];
      for (i=1;i<=ndim;i++){ey[i] /= sqrt(ey_norm2);}
      cprd(ey[1],ey[2],ey[3],ez[1],ez[2],ez[3],&(ex[1]),&(ex[2]),&(ex[3]));
/* construct dex_dey,dex_dez */ 
      d_axb_da_(ez,dex_dey);
      d_axb_db_(ey,dex_dez);
/*-------------------------------------------------------------------------*/
/* set up the rotation matrix : R into the internal frame                  */ 
      RT[1][1]=ex[1]; RT[1][2]=ey[1]; RT[1][3]=ez[1];
      RT[2][1]=ex[2]; RT[2][2]=ey[2]; RT[2][3]=ez[2];
      RT[3][1]=ex[3]; RT[3][2]=ey[3]; RT[3][3]=ez[3];
      mat_trn(RT,R,ndim);
/*-------------------------------------------------------------------------*/
/* construct basic derivative matrices */
 /* dez_dr1 */ 
      for (i=1;i<=ndim;i++){
	for (j=1;j<=ndim;j++){
	  dez_dr1[i][j] = 0.0;
      }}
 /* dez_dr2 */
      mat_mat(I,de1_dr2,dez_dr2,ndim);
 /* dez_dr3 */
      mat_mat(I,de1_dr3,dez_dr3,ndim);
 /* dey_dr1 */ 
      mat_mat(dey_de2,de2_dr1,dey_dr1,ndim);
 /* dey_dr2 */
      mat_mat(dey_de1,de1_dr2,A,ndim);
      mat_mat(dey_de2,de2_dr2,B,ndim);
      mat_add(A,B,dey_dr2,ndim);
 /* dey_dr3 */ 
      mat_mat(dey_de1,de1_dr3,dey_dr3,ndim);
 /* dex_dr1 */ 
      mat_mat(dex_dey,dey_dr1,dex_dr1,ndim);
 /* dex_dr2 */ 
      mat_mat(dex_dey,dey_dr2,A,ndim);
      mat_mat(dex_dez,dez_dr2,B,ndim);
      mat_add(A,B,dex_dr2,ndim);
 /* dex_dr3 */ 
      mat_mat(dex_dey,dey_dr3,A,ndim);
      mat_mat(dex_dez,dez_dr3,B,ndim);
      mat_add(A,B,dex_dr3,ndim);
/*-------------------------------------------------------------------------*/
/* Rotate R4 and his group into the internal frame                         */
  switch(type){
     case -2:
         /* construct dR/dr1,dR/dr2,dR/dr3 tensor */
         dR_dr_(dex_dr1,dey_dr1,dez_dr1,dR_dr1x,dR_dr1y,dR_dr1z);
         dR_dr_(dex_dr2,dey_dr2,dez_dr2,dR_dr2x,dR_dr2y,dR_dr2z);
         dR_dr_(dex_dr3,dey_dr3,dez_dr3,dR_dr3x,dR_dr3y,dR_dr3z);
         for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
             jatm = ind_group[i_trans][iatm]; 
              r4[1] = ux[jatm]; f4[1] = fux[jatm];
              r4[2] = uy[jatm]; f4[2] = fuy[jatm];
              r4[3] = uz[jatm]; f4[3] = fuz[jatm];
              mat_vec(RT,r4,r4_,ndim);
              /* get dr4_dr1,dr4_dr2,dr4_dr3 */ 
              dRa_dr_(dR_dr1x,dR_dr1y,dR_dr1z,r4_,dr4_dr1,vec1scr_3x1);
              dRa_dr_(dR_dr2x,dR_dr2y,dR_dr2z,r4_,dr4_dr2,vec1scr_3x1);
              dRa_dr_(dR_dr3x,dR_dr3y,dR_dr3z,r4_,dr4_dr3,vec1scr_3x1);
              /* update the forces */ 
              vec_mat(f4,dr4_dr1,a,ndim);
              for (i=1;i<=ndim;i++) f1[i] += a[i];
              vec_mat(f4,dr4_dr2,a,ndim);
              for (i=1;i<=ndim;i++) f2[i] += a[i];
              vec_mat(f4,dr4_dr3,a,ndim);
              for (i=1;i<=ndim;i++) f3[i] += a[i];
              vec_mat(f4,R,a,ndim);
              for (i=1;i<=ndim;i++) f4[i] = a[i];
              ux[jatm] = r4_[1]; fux[jatm] = f4[1];
              uy[jatm] = r4_[2]; fuy[jatm] = f4[2];
              uz[jatm] = r4_[3]; fuz[jatm] = f4[3];
         }/*group atoms*/
         break;
     case -1: 
         for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
             jatm = ind_group[i_trans][iatm]; 
              r4[1] = ux[jatm];
              r4[2] = uy[jatm];
              r4[3] = uz[jatm];
              mat_vec(RT,r4,r4_,ndim);
              ux[jatm] = r4_[1];
              uy[jatm] = r4_[2];
              uz[jatm] = r4_[3];
         }/*group atoms*/
         break;
     case 1: 
         for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
             jatm = ind_group[i_trans][iatm]; 
              r4[1] = ux[jatm];
              r4[2] = uy[jatm];
              r4[3] = uz[jatm];
              mat_vec(R,r4,r4_,ndim);
              ux[jatm] = r4_[1];
              uy[jatm] = r4_[2];
              uz[jatm] = r4_[3];
	 }
         break;
     case 2:
         /* construct dRT/dr1,dRT/dr2,dRT/dr3 tensor */ 
         dRT_dr_(dex_dr1,dey_dr1,dez_dr1,dRT_dr1x,dRT_dr1y,dRT_dr1z);
         dRT_dr_(dex_dr2,dey_dr2,dez_dr2,dRT_dr2x,dRT_dr2y,dRT_dr2z);
         dRT_dr_(dex_dr3,dey_dr3,dez_dr3,dRT_dr3x,dRT_dr3y,dRT_dr3z);


         for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
             jatm = ind_group[i_trans][iatm]; 
              r4[1] = ux[jatm]; f4[1] = fux[jatm];
              r4[2] = uy[jatm]; f4[2] = fuy[jatm];
              r4[3] = uz[jatm]; f4[3] = fuz[jatm];
              mat_vec(R,r4,r4_,ndim);
              /* get dr4_dr1,dr4_dr2,dr4_dr3 */ 
              dRa_dr_(dRT_dr1x,dRT_dr1y,dRT_dr1z,r4_,dr4_dr1,vec1scr_3x1);
              dRa_dr_(dRT_dr2x,dRT_dr2y,dRT_dr2z,r4_,dr4_dr2,vec1scr_3x1);
              dRa_dr_(dRT_dr3x,dRT_dr3y,dRT_dr3z,r4_,dr4_dr3,vec1scr_3x1);
              /* update the forces */ 
              vec_mat(f4,dr4_dr1,a,ndim);
              for (i=1;i<=ndim;i++) f1[i] += a[i];
              vec_mat(f4,dr4_dr2,a,ndim);
              for (i=1;i<=ndim;i++) f2[i] += a[i];
              vec_mat(f4,dr4_dr3,a,ndim);
              for (i=1;i<=ndim;i++) f3[i] += a[i];
              vec_mat(f4,RT,a,ndim);
              for (i=1;i<=ndim;i++) f4[i] = a[i];
              ux[jatm] = r4_[1]; fux[jatm] = f4[1];
              uy[jatm] = r4_[2]; fuy[jatm] = f4[2];
              uz[jatm] = r4_[3]; fuz[jatm] = f4[3];
         }/*group atoms*/
         break;
      }/*switch*/ 
      /* assign base forces: (f1, f2, f3) */
      fux[j1] = f1[1]; fuy[j1] = f1[2]; fuz[j1] = f1[3];
      fux[j2] = f2[1]; fuy[j2] = f2[2]; fuz[j2] = f2[3];
      fux[j3] = f3[1]; fuy[j3] = f3[2]; fuz[j3] = f3[3];

/*-------------------------------------------------------------------------*/
  }/*end of rotate*/
/*-------------------------------------------------------------------------*/
