#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_AA_DNA_duplex(   const Maps *maps,
                                        const Pos *pos,
                                        const General *general,
                                        int  itorcompl,
                                        int *itrans_glob        )
/*-------------------------------------------------------------------------*/
/*   sets up all the torsional angles for an all atom DNA duplex           */
/*   and eight dummy atoms responsible for the realtive orientaion of      */
/*   strands                                                               */
/*   locally these molecules are called:                                   */ 
/*                                       DNA1, D11, D12, D13               */
/*                                             D00, D10                    */
/*                                       DNA2, D21, D22, D23               */
/*   torsional complex index: itorcompl                                    */
/*   transformation offset: itrans_glob                                    */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int i;
  int ires, iresl, nres;
  int itrans, ichigrp, istride, transform;
  int res_id_local;

  int iatm, iiatm, imol, iimol;
  int num_trans_backbone, num_trans_sidechain, num_trans_mol;

  int ind_O1P, ind_O2P, ind_P;
  int ind_O2p, ind_O3p, ind_O4p, ind_O5p;
  int ind_C1p, ind_C2p, ind_C3p, ind_C4p, ind_C5p;
  int ind_H1p, ind_H2p, ind_H3p, ind_H4p, ind_H5p, ind_H2pp, ind_H5pp;
  int ind_N9, ind_N1;
  int ind_prevres_O3p;
  int ind_nextres_O1P, ind_nextres_O2P, ind_nextres_P, ind_nextres_O5p;
  int ind_DNA1, ind_DNA2, ind_D00, ind_D10;
  int ind_D11,  ind_D12,  ind_D13;
  int ind_D21,  ind_D22,  ind_D23;
  int ind_rcent_DNA1, ind_rcent_DNA2;
  int ind_D1, ind_D2, ind_D3;
  int iDNA, ind_DNA, ind_rcent_DNA;
 
  char *atm_label;
  char *DNA1_chainid, *DNA2_chainid;

  int    nmol        = maps->nmol;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;
  char **mol_label   = maps->mol_label;
  char **mol_chainid = maps->mol_chainid;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0.A) Get the global indices of molecules DNA1, DNA2, D00, D10           */ 
/*                                and atoms D11,  D12,  D13                */
/*                                          D21,  D22,  D23                */
/*-------------------------------------------------------------------------*/
 
  ind_DNA1 = 0;
  ind_DNA2 = 0;
  for (imol=1;imol<=nmol_torcompl[itorcompl];imol++) {
       iimol = ind_mol_torcompl[itorcompl][imol];
       if ( (strcasecmp(mol_label[iimol],"DNA")==0) &&
            (ind_DNA1 == 0)                             ) {
	   ind_DNA1 = iimol;
           DNA1_chainid = mol_chainid[iimol];
       }/*endif*/
       if ( (strcasecmp(mol_label[iimol],"DNA")==0) &&
            ( iimol != ind_DNA1 )                       ) {       
	   ind_DNA2 = iimol;
           DNA2_chainid = mol_chainid[iimol];
       }/*endif*/
  }/*endfor*/


  for (imol=1;imol<=nmol_torcompl[itorcompl];imol++) {
       iimol = ind_mol_torcompl[itorcompl][imol];
       if (nres_mol[iimol]==1) {
           ires  = ind_res_mol[iimol][1];
           if (strcasecmp(pos->residue[ires].residue_name,"D00")==0) {
	       ind_D00 = pos->residue[ires].atom[1];
           }/*endif*/
           if (strcasecmp(pos->residue[ires].residue_name,"D10")==0) {
	       ind_D10 = pos->residue[ires].atom[1];
           }/*endif*/
           if (strcasecmp(pos->residue[ires].residue_name,"D11")==0) {
	       ind_D11 = pos->residue[ires].atom[1];
           }/*endif*/
           if (strcasecmp(pos->residue[ires].residue_name,"D12")==0) {
	       ind_D12 = pos->residue[ires].atom[1];
           }/*endif*/
           if (strcasecmp(pos->residue[ires].residue_name,"D13")==0) {
	       ind_D13 = pos->residue[ires].atom[1];
           }/*endif*/
           if (strcasecmp(pos->residue[ires].residue_name,"D21")==0) {
	       ind_D21 = pos->residue[ires].atom[1];
           }/*endif*/
           if (strcasecmp(pos->residue[ires].residue_name,"D22")==0) {
	       ind_D22 = pos->residue[ires].atom[1];
           }/*endif*/
           if (strcasecmp(pos->residue[ires].residue_name,"D23")==0) {
	       ind_D23 = pos->residue[ires].atom[1];
           }/*endif*/
       }/*endif*/
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* 0.B) Get the indices of central residues in DNA1, DNA2                  */
/*-------------------------------------------------------------------------*/

   if ( nres_mol[ind_DNA1] != nres_mol[ind_DNA2] ) {
 	sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex           \n"
                        "The number of residues in the two DNA    \n"
                        "strands are different!                   \n");
        errorMessage(err_msg);
   }/*endif*/   
  
   if ( (nres_mol[ind_DNA1] % 2) == 0 ) {/*even*/
         ind_rcent_DNA1 = nres_mol[ind_DNA1]/2;
         ind_rcent_DNA2 = ind_rcent_DNA1 + 1;
   }/*endif*/

   if ( (nres_mol[ind_DNA1] % 2) == 1 ) {/*odd*/
         ind_rcent_DNA1 = (nres_mol[ind_DNA1]-1)/2 + 1;
         ind_rcent_DNA2 = ind_rcent_DNA1;
   }/*endif*/


#ifdef DEBUG
   printf("molecular indices\n");
   printf("ind_DNA1 %d, ind_DNA2 %d\n",ind_DNA1,ind_DNA2);
   printf("residue indices\n");
   printf("ind_rcent_DNA1 %d\n",ind_rcent_DNA1);
   printf("ind_rcent_DNA2 %d\n",ind_rcent_DNA2);
   printf("atomic indices\n");
   printf("ind_D00 %d, ind_D10\n",ind_D00, ind_D10);
   printf("ind_D11 %d, ind_D12 %d, ind_D13 %d\n",ind_D11,ind_D12,ind_D13);
   printf("ind_D21 %d, ind_D22 %d, ind_D23 %d\n",ind_D21,ind_D22,ind_D23);
#endif


/*-------------------------------------------------------------------------*/
/* 0.C) Set the number and indices of base atoms                           */
/*-------------------------------------------------------------------------*/

   natm_torcomplbase[itorcompl]       = 4;
   ind_atm_torcomplbase[itorcompl][1] = ind_D00;
   ind_atm_torcomplbase[itorcompl][2] = ind_D10;
   ind_atm_torcomplbase[itorcompl][3] = ind_D11;
   ind_atm_torcomplbase[itorcompl][4] = ind_D21;

/*-------------------------------------------------------------------------*/
/* I) Define the groups of backbone atoms to be transformed for DNA duplex */

  itrans = *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I.A) Define tors groups out of atoms DUM and atoms in ind_rcent_DNA     */
/*      Plus any atoms in overlaping groups with previous residue          */
/*      and any groups out of next residue atoms rooted on atoms in        */
/*      this residue                                                       */

 for (iDNA=1;iDNA<=2;iDNA++) {

  if (iDNA==1) {
      ind_D1        = ind_D11;
      ind_D2        = ind_D12;
      ind_D3        = ind_D13;
      ind_DNA       = ind_DNA1;
      ind_rcent_DNA = ind_rcent_DNA1;
  }/*endif*/

  if (iDNA==2) {
      ind_D1        = ind_D21;
      ind_D2        = ind_D22;
      ind_D3        = ind_D23;
      ind_DNA       = ind_DNA2;
      ind_rcent_DNA = ind_rcent_DNA2;
  }/*endif*/

  ires = ind_res_mol[ind_DNA][ind_rcent_DNA];
  for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
       iiatm     = pos->residue[ires].atom[iatm];
       atm_label = maps->upac_atm_label[iiatm];
       if (strcasecmp(atm_label,"O1P")==0) {ind_O1P = iiatm;}   
       if (strcasecmp(atm_label,"O2P")==0) {ind_O2P = iiatm;}   
       if (strcasecmp(atm_label,"P")==0)   {ind_P   = iiatm;}
       if (strcasecmp(atm_label,"O3'")==0) {ind_O3p = iiatm;}   
       if (strcasecmp(atm_label,"O4'")==0) {ind_O4p = iiatm;}
       if (strcasecmp(atm_label,"O5'")==0) {ind_O5p = iiatm;}
       if (strcasecmp(atm_label,"C1'")==0) {ind_C1p = iiatm;}
       if (strcasecmp(atm_label,"C2'")==0) {ind_C2p = iiatm;}
       if (strcasecmp(atm_label,"C3'")==0) {ind_C3p = iiatm;}
       if (strcasecmp(atm_label,"C4'")==0) {ind_C4p = iiatm;}
       if (strcasecmp(atm_label,"C5'")==0) {ind_C5p = iiatm;}
       if (strcasecmp(atm_label,"H1'")==0) {ind_H1p = iiatm;}
       if (strcasecmp(atm_label,"H2'")==0) {ind_H2p = iiatm;}
       if (strcasecmp(atm_label,"H3'")==0) {ind_H3p = iiatm;}
       if (strcasecmp(atm_label,"H4'")==0) {ind_H4p = iiatm;}
       if (strcasecmp(atm_label,"H5'")==0) {ind_H5p = iiatm;}
       if (strcasecmp(atm_label,"H2''")==0){ind_H2pp = iiatm;}
       if (strcasecmp(atm_label,"H5''")==0){ind_H5pp = iiatm;}
       if (strcasecmp(atm_label,"N9")==0)  {ind_N9   = iiatm;}
       if (strcasecmp(atm_label,"N1")==0)  {ind_N1   = iiatm;}
  }/*endfor*/

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   natm_trans[itrans] = 3;
   ind_trans[itrans][1] = ind_C1p;      /*C1'*/
   ind_trans[itrans][2] = ind_D2;       /*D$2 if ind_DNA$, $={1,2}*/
   ind_trans[itrans][3] = ind_D3;       /*D$3 if ind_DNA$, $={1,2}*/
   ind_prim_tors[itrans][4] = ind_C1p;  /*C1'*/
   ind_prim_tors[itrans][3] = ind_D1;   /*D$1 if ind_DNA$, $={1,2}*/
   ind_prim_tors[itrans][2] = ind_D10;  /*D10*/
   ind_prim_tors[itrans][1] = ind_D00;  /*D00*/

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_DNA_duplex:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   natm_trans[itrans] = 12;
   ind_trans[itrans][1]  = ind_O4p;  /*O4'*/
   ind_trans[itrans][2]  = ind_C4p;  /*C4'*/
   ind_trans[itrans][3]  = ind_C3p;  /*C3'*/
   ind_trans[itrans][4]  = ind_C2p;  /*C2'*/
   ind_trans[itrans][5]  = ind_H1p;  /*H1'*/
   ind_trans[itrans][6]  = ind_H2p;  /*H2'*/
   ind_trans[itrans][7]  = ind_H2pp; /*H2''*/
   ind_trans[itrans][8]  = ind_H3p;  /*H3'*/
   ind_trans[itrans][9]  = ind_O3p;  /*O3'*/
   ind_trans[itrans][10] = ind_H4p;  /*H4'*/
   ind_trans[itrans][11] = ind_C5p;  /*C5'*/
   if ( (strcasecmp(pos->residue[ires].residue_name,"ADE")==0) ||
        (strcasecmp(pos->residue[ires].residue_name,"GUA")==0) ) {
         ind_trans[itrans][12]  = ind_N9;  /*N9*/
   }/*endif*/
   if ( (strcasecmp(pos->residue[ires].residue_name,"THY")==0) ||
        (strcasecmp(pos->residue[ires].residue_name,"CYT")==0) ) {
         ind_trans[itrans][12]  = ind_N1;  /*N1*/
   }/*endif*/
   ind_prim_tors[itrans][4] = ind_O4p;  /*O4'*/
   ind_prim_tors[itrans][3] = ind_C1p;  /*C1'*/
   ind_prim_tors[itrans][2] = ind_D3;   /*D$3 if ind_DNA$, $={1,2}*/
   ind_prim_tors[itrans][1] = ind_D2;   /*D$2 if ind_DNA$, $={1,2}*/

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   natm_trans[itrans] = 3;
   ind_trans[itrans][1] = ind_O5p;      /* O5' */
   ind_trans[itrans][2] = ind_H5p;      /* H5' */
   ind_trans[itrans][3] = ind_H5pp;     /* H5''*/
   ind_prim_tors[itrans][4] = ind_O5p;  /* O5' */
   ind_prim_tors[itrans][3] = ind_C5p;  /* C5' */
   ind_prim_tors[itrans][2] = ind_C4p;  /* C4' */
   ind_prim_tors[itrans][1] = ind_O4p;  /* O4' */

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   natm_trans[itrans] = 1;
   ind_trans[itrans][1] = ind_P;        /* P  */
   ind_prim_tors[itrans][4] = ind_P;    /* P  */
   ind_prim_tors[itrans][3] = ind_O5p;  /* O5'*/
   ind_prim_tors[itrans][2] = ind_C5p;  /* C5'*/
   ind_prim_tors[itrans][1] = ind_C4p;  /* C4'*/

   if ( ind_rcent_DNA > 1 ) {
   ires = ind_res_mol[ind_DNA][ind_rcent_DNA-1]; /*~~~~previous residue~~~~*/
   for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
       iiatm     = pos->residue[ires].atom[iatm];
       atm_label = maps->upac_atm_label[iiatm];
       if (strcasecmp(atm_label,"O3'")==0) {ind_prevres_O3p = iiatm;}
   }/*endfor*/
   ires = ind_res_mol[ind_DNA][ind_rcent_DNA];
   }/*endif-ind_rcent_DNA>1*/

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   natm_trans[itrans] = 2;
   if ( ind_rcent_DNA > 1 ) 
        natm_trans[itrans] = 3;   
   ind_trans[itrans][1] = ind_O1P;         /* O1P */
   ind_trans[itrans][2] = ind_O2P;         /* O2P */
   if ( ind_rcent_DNA > 1 ) 
        ind_trans[itrans][3] = ind_prevres_O3p; /* O3p */
   ind_prim_tors[itrans][4] = ind_O1P;     /* O1P */
   ind_prim_tors[itrans][3] = ind_P;       /* P   */
   ind_prim_tors[itrans][2] = ind_O5p;     /* O5' */
   ind_prim_tors[itrans][1] = ind_C5p;     /* C5' */

   if (ind_rcent_DNA < nres_mol[ind_DNA]) {
    ires = ind_res_mol[ind_DNA][ind_rcent_DNA+1];/*~~~~next residue~~~~*/
    for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
         iiatm     = pos->residue[ires].atom[iatm];
         atm_label = maps->upac_atm_label[iiatm];
         if (strcasecmp(atm_label,"O1P")==0) {ind_nextres_O1P = iiatm;}   
         if (strcasecmp(atm_label,"O2P")==0) {ind_nextres_O2P = iiatm;}   
         if (strcasecmp(atm_label,"P")==0)   {ind_nextres_P   = iiatm;}
         if (strcasecmp(atm_label,"O5'")==0) {ind_nextres_O5p = iiatm;}
    }/*endfor*/

    itrans++;
    if (itrans>num_trans) {
	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
       errorMessage(err_msg);
    }/*endif*/
    res_id[itrans] = ires;
    natm_trans[itrans] = 1;
    ind_trans[itrans][1] = ind_nextres_P; /*P*/
    ind_prim_tors[itrans][4] = ind_nextres_P; /*P*/
    ind_prim_tors[itrans][3] = ind_O3p; /*O3'*/
    ind_prim_tors[itrans][2] = ind_C3p; /*C3'*/
    ind_prim_tors[itrans][1] = ind_C4p; /*C4'*/

    itrans++;
    if (itrans>num_trans) {
	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
       errorMessage(err_msg);
    }/*endif*/
    res_id[itrans] = ires;
    natm_trans[itrans] = 3;
    ind_trans[itrans][1] = ind_nextres_O5p; /*O5'*/
    ind_trans[itrans][2] = ind_nextres_O1P; /*O1P*/
    ind_trans[itrans][3] = ind_nextres_O2P; /*O2P*/
    ind_prim_tors[itrans][4] = ind_nextres_O5p; /*O5'*/
    ind_prim_tors[itrans][3] = ind_nextres_P;  /*P*/
    ind_prim_tors[itrans][2] = ind_O3p; /*O3'*/
    ind_prim_tors[itrans][1] = ind_C3p; /*C3'*/
   }/*endif-ind_rcent_DNA < nres_mol[ind_DNA]*/
/*-------------------------------------------------------------------------*/
/* I.B) Define tors groups on residues backwards from ind_rcent_DNA to the */
/*      first residue                                                      */

  ind_nextres_P   = ind_P;
  ind_nextres_O5p = ind_O5p;
  for (iresl=ind_rcent_DNA-1;iresl>=1;iresl--) {/*backwards*/
       ires = ind_res_mol[ind_DNA][iresl];
       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"O1P")==0) {ind_O1P = iiatm;}   
            if (strcasecmp(atm_label,"O2P")==0) {ind_O2P = iiatm;}   
            if (strcasecmp(atm_label,"P")==0)   {ind_P   = iiatm;}
            if (strcasecmp(atm_label,"O3'")==0) {ind_O3p = iiatm;}   
            if (strcasecmp(atm_label,"O4'")==0) {ind_O4p = iiatm;}
            if (strcasecmp(atm_label,"O5'")==0) {ind_O5p = iiatm;}
            if (strcasecmp(atm_label,"C1'")==0) {ind_C1p = iiatm;}
            if (strcasecmp(atm_label,"C2'")==0) {ind_C2p = iiatm;}
            if (strcasecmp(atm_label,"C3'")==0) {ind_C3p = iiatm;}
            if (strcasecmp(atm_label,"C4'")==0) {ind_C4p = iiatm;}
            if (strcasecmp(atm_label,"C5'")==0) {ind_C5p = iiatm;}
            if (strcasecmp(atm_label,"H1'")==0) {ind_H1p = iiatm;}
            if (strcasecmp(atm_label,"H2'")==0) {ind_H2p = iiatm;}
            if (strcasecmp(atm_label,"H3'")==0) {ind_H3p = iiatm;}
            if (strcasecmp(atm_label,"H4'")==0) {ind_H4p = iiatm;}
            if (strcasecmp(atm_label,"H5'")==0) {ind_H5p = iiatm;}
            if (strcasecmp(atm_label,"H2''")==0){ind_H2pp = iiatm;}
            if (strcasecmp(atm_label,"H5''")==0){ind_H5pp = iiatm;}
            if (strcasecmp(atm_label,"N9")==0)  {ind_N9   = iiatm;}
            if (strcasecmp(atm_label,"N1")==0)  {ind_N1   = iiatm;}
       }/*endfor*/
       
       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1]     = ind_C3p;         /*C3'*/
       ind_prim_tors[itrans][4] = ind_C3p;         /*C3'*/
       ind_prim_tors[itrans][3] = ind_O3p;         /*O3'*/
       ind_prim_tors[itrans][2] = ind_nextres_P;   /*P  */
       ind_prim_tors[itrans][1] = ind_nextres_O5p; /*O5'*/

       itrans++;
       if (itrans>num_trans) {
          sprintf(err_msg,"Tors_defs::setup_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 11;
       ind_trans[itrans][1]  = ind_O4p;  /*O4'*/
       ind_trans[itrans][2]  = ind_C4p;  /*C4'*/
       ind_trans[itrans][3]  = ind_C2p;  /*C2'*/
       ind_trans[itrans][4]  = ind_C1p;  /*C1'*/
       ind_trans[itrans][5]  = ind_H1p;  /*H1'*/
       ind_trans[itrans][6]  = ind_H2p;  /*H2'*/
       ind_trans[itrans][7]  = ind_H2pp; /*H2''*/
       ind_trans[itrans][8]  = ind_H3p;  /*H3'*/
       ind_trans[itrans][9]  = ind_H4p;  /*H4'*/
       ind_trans[itrans][10] = ind_C5p;  /*C5'*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"ADE")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"GUA")==0) ) {
             ind_trans[itrans][11]  = ind_N9;  /*N9*/
       }/*endif*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"THY")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"CYT")==0) ) {
             ind_trans[itrans][11]  = ind_N1;  /*N1*/
       }/*endif*/
       ind_prim_tors[itrans][4] = ind_O4p;  /*O4'*/
       ind_prim_tors[itrans][3] = ind_C3p;  /*C3'*/
       ind_prim_tors[itrans][2] = ind_O3p;  /*O3p*/
       ind_prim_tors[itrans][1] = ind_nextres_P;  /*P*/

       itrans++;
       if (itrans>num_trans) {
          sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 3;
       ind_trans[itrans][1] = ind_O5p;      /* O5' */
       ind_trans[itrans][2] = ind_H5p;      /* H5' */
       ind_trans[itrans][3] = ind_H5pp;     /* H5''*/
       ind_prim_tors[itrans][4] = ind_O5p;  /* O5' */
       ind_prim_tors[itrans][3] = ind_C5p;  /* C5' */
       ind_prim_tors[itrans][2] = ind_C4p;  /* C4' */
       ind_prim_tors[itrans][1] = ind_O4p;  /* O4' */

       itrans++;
       if (itrans>num_trans) {
          sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_P;        /* P  */
       ind_prim_tors[itrans][4] = ind_P;    /* P  */
       ind_prim_tors[itrans][3] = ind_O5p;  /* O5'*/
       ind_prim_tors[itrans][2] = ind_C5p;  /* C5'*/
       ind_prim_tors[itrans][1] = ind_C4p;  /* C4'*/

       if ( iresl != 1 ) {
            ires = ind_res_mol[ind_DNA][iresl-1]; /*~~previous residue~~*/
            for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
                 iiatm     = pos->residue[ires].atom[iatm];
                 atm_label = maps->upac_atm_label[iiatm];
                 if (strcasecmp(atm_label,"O3'")==0) {ind_prevres_O3p = iiatm;}
            }/*endfor*/
            ires = ind_res_mol[ind_DNA][iresl];
       }/*endif*/
      
       itrans++;
       if (itrans>num_trans) {
          sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       if ( iresl != 1 ) {
            natm_trans[itrans] = 3;
            ind_trans[itrans][1] = ind_O1P;         /* O1P */
            ind_trans[itrans][2] = ind_O2P;         /* O2P */
            ind_trans[itrans][3] = ind_prevres_O3p; /* O3p */
       } else {
            natm_trans[itrans] = 2;
            ind_trans[itrans][1] = ind_O1P;         /* O1P */
            ind_trans[itrans][2] = ind_O2P;         /* O2P */
       }/*endif*/
       ind_prim_tors[itrans][4] = ind_O1P;     /* O1P */
       ind_prim_tors[itrans][3] = ind_P;       /* P   */
       ind_prim_tors[itrans][2] = ind_O5p;     /* O5' */
       ind_prim_tors[itrans][1] = ind_C5p;     /* C5' */

       ind_nextres_P   = ind_P;
       ind_nextres_O5p = ind_O5p;

  }/*endfor-iresl*/


#ifdef DEBUG
   printf("Check Point I\n");
   getchar();
#endif

/*-------------------------------------------------------------------------*/
/* I.C) Define tors groups on residues forward from ind_rcent_DNA+1 to the */
/*      last residue                                                       */


  nres   = nres_mol[ind_DNA];
  for (iresl=ind_rcent_DNA+1;iresl<=nres;iresl++) {
       ires = ind_res_mol[ind_DNA][iresl];
       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"O1P")==0) {ind_O1P = iiatm;}   
            if (strcasecmp(atm_label,"O2P")==0) {ind_O2P = iiatm;}   
            if (strcasecmp(atm_label,"P")==0)   {ind_P   = iiatm;}
            if (strcasecmp(atm_label,"O3'")==0) {ind_O3p = iiatm;}   
            if (strcasecmp(atm_label,"O4'")==0) {ind_O4p = iiatm;}
            if (strcasecmp(atm_label,"O5'")==0) {ind_O5p = iiatm;}
            if (strcasecmp(atm_label,"C1'")==0) {ind_C1p = iiatm;}
            if (strcasecmp(atm_label,"C2'")==0) {ind_C2p = iiatm;}
            if (strcasecmp(atm_label,"C3'")==0) {ind_C3p = iiatm;}
            if (strcasecmp(atm_label,"C4'")==0) {ind_C4p = iiatm;}
            if (strcasecmp(atm_label,"C5'")==0) {ind_C5p = iiatm;}
            if (strcasecmp(atm_label,"H1'")==0) {ind_H1p = iiatm;}
            if (strcasecmp(atm_label,"H2'")==0) {ind_H2p = iiatm;}
            if (strcasecmp(atm_label,"H3'")==0) {ind_H3p = iiatm;}
            if (strcasecmp(atm_label,"H4'")==0) {ind_H4p = iiatm;}
            if (strcasecmp(atm_label,"H5'")==0) {ind_H5p = iiatm;}
            if (strcasecmp(atm_label,"H2''")==0){ind_H2pp = iiatm;}
            if (strcasecmp(atm_label,"H5''")==0){ind_H5pp = iiatm;}
            if (strcasecmp(atm_label,"N9")==0)  {ind_N9   = iiatm;}
            if (strcasecmp(atm_label,"N1")==0)  {ind_N1   = iiatm;}
       }/*endfor*/


#ifdef DEBUG
   printf("Check Point II\n");
   getchar();
#endif

       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][4] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][3] = ind_O5p;  /*O5'*/
       ind_prim_tors[itrans][2] = ind_P;    /*P*/
       ind_prim_tors[itrans][1] = ind_O1P;  /*O1P*/

#ifdef DEBUG
   printf("Check Point III\n");
   getchar();
#endif

       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 3;
       ind_trans[itrans][1] = ind_C4p;  /*C4'*/
       ind_trans[itrans][2] = ind_H5p;  /*H5'*/
       ind_trans[itrans][3] = ind_H5pp; /*H5''*/
       ind_prim_tors[itrans][4] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][3] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][2] = ind_O5p;  /*O5'*/
       ind_prim_tors[itrans][1] = ind_P;    /*P*/

#ifdef DEBUG
   printf("Check Point IV\n");
   getchar();
#endif

       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 11;
       ind_trans[itrans][1]  = ind_O4p;  /*O4'*/
       ind_trans[itrans][2]  = ind_C1p;  /*C1'*/
       ind_trans[itrans][3]  = ind_C2p;  /*C2'*/
       ind_trans[itrans][4]  = ind_C3p;  /*C3'*/
       ind_trans[itrans][5]  = ind_H1p;  /*H1'*/
       ind_trans[itrans][6]  = ind_H2pp; /*H2''*/
       ind_trans[itrans][7]  = ind_H3p;  /*H3'*/
       ind_trans[itrans][8]  = ind_O3p;  /*O3'*/
       ind_trans[itrans][9]  = ind_H4p;  /*H4'*/
       ind_trans[itrans][10] = ind_H2p;  /*H2'*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"ADE")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"GUA")==0) ) {
             ind_trans[itrans][11]  = ind_N9;  /*N9*/
       }/*endif*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"THY")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"CYT")==0) ) {
             ind_trans[itrans][11]  = ind_N1;  /*N1*/
       }/*endif*/
       ind_prim_tors[itrans][4] = ind_O4p;  /*O4'*/
       ind_prim_tors[itrans][3] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][2] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][1] = ind_O5p;  /*O5'*/


#ifdef DEBUG
   printf("Check Point V, iresl %d, nres %d\n",iresl,nres);
   getchar();
#endif

      
    if (iresl==nres) {continue;}   /*~~~~~~~~~~~~~next residue~~~~~~~~~~~~*/
    ires = ind_res_mol[ind_DNA][iresl+1];
    for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
         iiatm     = pos->residue[ires].atom[iatm];
         atm_label = maps->upac_atm_label[iiatm];
         if (strcasecmp(atm_label,"O1P")==0) {ind_nextres_O1P = iiatm;}   
         if (strcasecmp(atm_label,"O2P")==0) {ind_nextres_O2P = iiatm;}   
         if (strcasecmp(atm_label,"P")==0)   {ind_nextres_P   = iiatm;}
         if (strcasecmp(atm_label,"O5'")==0) {ind_nextres_O5p = iiatm;}
    }/*endfor*/

    itrans++;
    if (itrans>num_trans) {
	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
       errorMessage(err_msg);
    }/*endif*/
    res_id[itrans] = ires;
    natm_trans[itrans] = 1;
    ind_trans[itrans][1] = ind_nextres_P; /*P*/
    ind_prim_tors[itrans][4] = ind_nextres_P; /*P*/
    ind_prim_tors[itrans][3] = ind_O3p; /*O3'*/
    ind_prim_tors[itrans][2] = ind_C3p; /*C3'*/
    ind_prim_tors[itrans][1] = ind_C4p; /*C4'*/

    itrans++;
    if (itrans>num_trans) {
	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
       errorMessage(err_msg);
    }/*endif*/
    res_id[itrans] = ires;
    natm_trans[itrans] = 3;
    ind_trans[itrans][1] = ind_nextres_O5p; /*O5'*/
    ind_trans[itrans][2] = ind_nextres_O1P; /*O1P*/
    ind_trans[itrans][3] = ind_nextres_O2P; /*O2P*/
    ind_prim_tors[itrans][4] = ind_nextres_O5p; /*O5'*/
    ind_prim_tors[itrans][3] = ind_nextres_P;  /*P*/
    ind_prim_tors[itrans][2] = ind_O3p; /*O3'*/
    ind_prim_tors[itrans][1] = ind_C3p; /*C3'*/

  }/*endfor-iresl*/

 }/*endfor-iDNA*/

 num_trans_backbone = itrans - *itrans_glob;

#ifdef DEBUG
   printf("Check Point VI\n");
   printf("num_trans_backbone %d\n",num_trans_backbone);
   getchar();
#endif

/*-------------------------------------------------------------------------*/
/* II) Define the groups of sidechain atoms on the DNA strands             */

 for (imol=1;imol<=nmol_torcompl[itorcompl];imol++) {
  iimol = ind_mol_torcompl[itorcompl][imol];
  if  (strcasecmp(mol_label[iimol],"DNA")!=0) {continue;}
  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[iimol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     if (itrans>num_trans) {
        sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
        errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/
 }/*end for-imol*/

 num_trans_sidechain = itrans - *itrans_glob - num_trans_backbone;
 num_trans_mol       = num_trans_backbone + num_trans_sidechain;

/*-------------------------------------------------------------------------*/
/* II) Transformation types                                                */
/*     By default, everybody is transformed                                */

   for (itrans=1;itrans<=num_trans_mol;itrans++) {
        ityp_trans[itrans+(*itrans_glob)] = 1;
        off_trans[itrans+(*itrans_glob)]  = 0;
   }/*end for*/

/*-------------------------------------------------------------------------*/
/* III) If stride is defined in .pdb, only sample backbone                 */
/*       along 'C' secondary structure type residue                        */
/*       sidechain torsions are always transformed                         */ 
/*       Only consider STRIDE in case of mol_label == "DNA"                */

   if (general->stride_defined==1) {
    for (imol=1;imol<=nmol_torcompl[itorcompl];imol++) {
     iimol = ind_mol_torcompl[itorcompl][imol];
     if (strcasecmp(mol_label[iimol],"DNA")!=0) {continue;}
     istride = ind_stride_mol[iimol];
     if (istride==0) {
      sprintf(err_msg,"%d. molecule does not have a STRIDE",iimol);
      errorMessage(err_msg);
     }/*endif*/
     if (istride>nstride) {
      sprintf(err_msg,"%d. molecule istride > nstride",iimol);
      errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[istride][0]))-1!=nres_mol[iimol]) {
      sprintf(err_msg,"%d. STRIDE in PDB is corrupted",istride);
      errorMessage(err_msg);
     }/*endif*/
     for (itrans=1;itrans<=num_trans_backbone;itrans++) {
          ityp_trans[itrans+(*itrans_glob)] = 1;
          off_trans[itrans+(*itrans_glob)]  = 1;
          res_id_local = res_id[itrans+(*itrans_glob)]-ind_res_mol[iimol][1]+1;
          if (stride[istride][res_id_local] == 'C') {
              off_trans[itrans+(*itrans_glob)] = 0;
          }/*endif*/
     }/*end for*/
    }/*end for-imol*/
   }/*end if*/

/*-------------------------------------------------------------------------*/
/* IV) Adjust itrans_glob                                                  */  
   *itrans_glob += num_trans_mol; 

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/
