#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_AA_DNA_scblc(     const Maps *maps,
                                         const Pos *pos,
                                         const General *general,
                                         int  imol,
                                         int *itrans_glob,
                                         int *iclos_glob,
                                         int *iclbase_glob      )
/*-------------------------------------------------------------------------*/
/*     sets up all the torsional angles for an all atom DNA strand         */
/*     treated with the method of successive constant bond length closure  */
/*     CBLC J. Comp. Chem. 27: 309-315 (2006)                              */
/*     molecular index: imol                                               */
/*     transformation offset: itrans_glob                                  */
/*     closure offset: iclos_glob                                          */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int ires, itrans, iclos, iclbase;
  int iatm, iiatm, jatm, jjatm;
  int ichigrp;
  int istride;
  int transform;
  int num_trans_backbone, num_trans_sidechain, num_trans_mol;
  int num_clos_mol, num_clbase_mol;
  char   err_msg[LINE_NCHARMAX];

  int ind_O1P, ind_O2P, ind_P;
  int ind_O2p, ind_O3p, ind_O4p, ind_O5p;
  int ind_C1p, ind_C2p, ind_C3p, ind_C4p, ind_C5p;
  int ind_H1p, ind_H2p, ind_H3p, ind_H4p, ind_H5p, ind_H2pp, ind_H5pp;
  int ind_N9, ind_N1;

  int itrans_O5p, itrans_C5p;

  int ind_nextres_O1P, ind_nextres_O2P, ind_nextres_P;
  int ind_prevres_O3p, ind_nextres_O5p;

  int    res_id_local;
  int    nres, iresl;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;

  double dx, dy, dz;
  double *x = pos->x;
  double *y = pos->y;
  double *z = pos->z;

  char *atm_label;


/*-------------------------------------------------------------------------*/
/* 0) Indicate the type of torsional setup                                 */
/*-------------------------------------------------------------------------*/

  printf("Constant Bond Length Double Closure (CBLC) for DNA\n");


/*-------------------------------------------------------------------------*/
/* I) Define the groups of atoms to be transformed for nucleic acids       */
/*    >>ityp_trans<< needs to be defined everywhere explicitly!            */
/*-------------------------------------------------------------------------*/
/* I.A) Define the groups of backbone atoms                                */

  itrans  = *itrans_glob;
  iclos   = *iclos_glob;
  iclbase = *iclbase_glob;
  nres   = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
       ires = ind_res_mol[imol][iresl];
       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];

            if (strcasecmp(atm_label,"O1P")==0) {ind_O1P = iiatm;}   
            if (strcasecmp(atm_label,"O2P")==0) {ind_O2P = iiatm;}   
            if (strcasecmp(atm_label,"P")==0)   {ind_P   = iiatm;}
            if (strcasecmp(atm_label,"O3'")==0) {ind_O3p = iiatm;}   
            if (strcasecmp(atm_label,"O4'")==0) {ind_O4p = iiatm;}
            if (strcasecmp(atm_label,"O5'")==0) {ind_O5p = iiatm;}
            if (strcasecmp(atm_label,"C1'")==0) {ind_C1p = iiatm;}
            if (strcasecmp(atm_label,"C2'")==0) {ind_C2p = iiatm;}
            if (strcasecmp(atm_label,"C3'")==0) {ind_C3p = iiatm;}
            if (strcasecmp(atm_label,"C4'")==0) {ind_C4p = iiatm;}
            if (strcasecmp(atm_label,"C5'")==0) {ind_C5p = iiatm;}
            if (strcasecmp(atm_label,"H1'")==0) {ind_H1p = iiatm;}
            if (strcasecmp(atm_label,"H2'")==0) {ind_H2p = iiatm;}
            if (strcasecmp(atm_label,"H3'")==0) {ind_H3p = iiatm;}
            if (strcasecmp(atm_label,"H4'")==0) {ind_H4p = iiatm;}
            if (strcasecmp(atm_label,"H5'")==0) {ind_H5p = iiatm;}
            if (strcasecmp(atm_label,"H2''")==0){ind_H2pp = iiatm;}
            if (strcasecmp(atm_label,"H5''")==0){ind_H5pp = iiatm;}
            if (strcasecmp(atm_label,"N9")==0)  {ind_N9   = iiatm;}
            if (strcasecmp(atm_label,"N1")==0)  {ind_N1   = iiatm;}
	    if (strcasecmp(atm_label,"O2'")==0) {ind_O2p = iiatm;}  /*RNA*/
       }/*endfor*/

       if (iresl!=1) {
           ires = ind_res_mol[imol][iresl-1];/*~~~~~~~~prev residue~~~~~~~~~*/
           for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
                iiatm     = pos->residue[ires].atom[iatm];
                atm_label = maps->upac_atm_label[iiatm];
                if (strcasecmp(atm_label,"O3'")==0) {ind_prevres_O3p = iiatm;} 
           }/*endfor*/
           ires = ind_res_mol[imol][iresl];     
       }/*endif*/


       iclbase++;
       if (iclbase>num_clbase) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:iclbase>num_clbase");
          errorMessage(err_msg);
       }/*endif*/
       res_id_clbase[iclbase] = ires;
       ind_clbase_mol[imol][iclbase-(*iclbase_glob)] = iclbase;
       natm_clbase[iclbase] = 3;
       ind_atm_clbase[iclbase][1] = ind_O4p;  /*O4'*/
       ind_atm_clbase[iclbase][2] = ind_C1p;  /*C1'*/
       ind_atm_clbase[iclbase][3] = ind_C2p;  /*C2'*/


       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1;
       prop_sig_scale[itrans] = 0.02; // 0.05 0.1
       res_id[itrans] = ires;
       natm_trans[itrans] = 3;
       ind_trans[itrans][1] = ind_C3p;  /*C3'*/
       ind_trans[itrans][2] = ind_H2p;  /*H2'*/
       ind_trans[itrans][3] = ind_H2pp; /*H2''*/
       ind_prim_tors[itrans][4] = ind_C3p;  /*C3'*/
       ind_prim_tors[itrans][3] = ind_C2p;  /*C2'*/
       ind_prim_tors[itrans][2] = ind_C1p;  /*C1'*/
       ind_prim_tors[itrans][1] = ind_O4p;  /*O4'*/



       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 2;          /*needs closure*/
       prop_sig_scale[itrans] = 0.02; // 0.05 0.1
       res_id[itrans] = ires;
       natm_trans[itrans] = 3;
       ind_trans[itrans][1] = ind_C4p;  /*C4'*/
       ind_trans[itrans][2] = ind_O3p;  /*O3'*/
       ind_trans[itrans][3] = ind_H3p;  /*H3'*/
       ind_prim_tors[itrans][4] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][3] = ind_C3p;  /*C3'*/
       ind_prim_tors[itrans][2] = ind_C2p;  /*C2'*/
       ind_prim_tors[itrans][1] = ind_C1p;  /*C1'*/
       /*____________________/closure/____________________*/
       iclos++;
       if (iclos>num_clos) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:iclos>num_clos");
          errorMessage(err_msg);
       }/*endif*/
       ind_clos_trans[itrans] = iclos;
       ityp_clos[iclos]       = 0;        /*cblc*/
       natm_clos[iclos]       = 5;
       ind_atm_clos[iclos][5] = ind_O4p;  /*O4'*/
       ind_atm_clos[iclos][4] = ind_C4p;  /*C4'*/
       ind_atm_clos[iclos][3] = ind_C3p;  /*C3'*/
       ind_atm_clos[iclos][2] = ind_C2p;  /*C2'*/
       ind_atm_clos[iclos][1] = ind_C1p;  /*C1'*/
       dx = x[ind_O4p] - x[ind_C4p];
       dy = y[ind_O4p] - y[ind_C4p];
       dz = z[ind_O4p] - z[ind_C4p];
       const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
       dx = x[ind_C3p] - x[ind_C4p];
       dy = y[ind_C3p] - y[ind_C4p];
       dz = z[ind_C3p] - z[ind_C4p];
       const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/



       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 0;
       res_id[itrans] = ires;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1]  = ind_H1p;  /*H1'*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"ADE")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"GUA")==0) ) {
             ind_trans[itrans][2]  = ind_N9;  /*N9*/
       }/*endif*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"THY")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"CYT")==0) ) {
             ind_trans[itrans][2]  = ind_N1;  /*N1*/
       }/*endif*/
       ind_prim_tors[itrans][4] = ind_H1p;  /*H1'*/
       ind_prim_tors[itrans][3] = ind_C1p;  /*C1'*/
       ind_prim_tors[itrans][2] = ind_C2p;  /*C2'*/
       ind_prim_tors[itrans][1] = ind_C3p;  /*C3'*/



       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       itrans_C5p = itrans;
       ityp_trans[itrans] = 0;
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1]  = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][4] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][3] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][2] = ind_C3p;  /*C3'*/
       ind_prim_tors[itrans][1] = ind_C2p;  /*C2'*/
       if (iresl != 1) {
           iclos++;
           if (iclos>num_clos) {
     	      sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:iclos>num_clos");
              errorMessage(err_msg);
           }/*endif*/
           ind_clos_trans[itrans] = iclos;
           ityp_clos[iclos]       = 0;        /*scblc*/
           natm_clos[iclos]       = 5;
           ind_atm_clos[iclos][5] = ind_O5p;  /*O5'*/
           ind_atm_clos[iclos][4] = ind_C5p;  /*C5'*/
           ind_atm_clos[iclos][3] = ind_C4p;  /*C4'*/
           ind_atm_clos[iclos][2] = ind_C3p;  /*C3'*/
           ind_atm_clos[iclos][1] = ind_C2p;  /*C2'*/
           dx = x[ind_O5p] - x[ind_C5p];
           dy = y[ind_O5p] - y[ind_C5p];
           dz = z[ind_O5p] - z[ind_C5p];
           const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
           dx = x[ind_C5p] - x[ind_C4p];
           dy = y[ind_C5p] - y[ind_C4p];
           dz = z[ind_C5p] - z[ind_C4p];
           const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/
       }/*endif*/



       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       itrans_O5p = itrans;
       ityp_trans[itrans] = 1; // 1        
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_O5p;  /*O5'*/
       ind_prim_tors[itrans][4] = ind_O5p;  /*O5'*/
       ind_prim_tors[itrans][3] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][2] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][1] = ind_C3p;  /*C3'*/
       if (iresl != 1) {
           iclos++;
           if (iclos>num_clos) {
               sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:iclos>num_clos");
               errorMessage(err_msg);
           }/*endif*/
           ind_clos_trans[itrans] = iclos;
           ityp_clos[iclos]       = 5;        /*scblc 1 2 3 4 5*/
           natm_clos[iclos]       = 5;
           ind_atm_clos[iclos][5] = ind_P;    /*P*/
           ind_atm_clos[iclos][4] = ind_O5p;  /*O5'*/
           ind_atm_clos[iclos][3] = ind_C5p;  /*C5'*/
           ind_atm_clos[iclos][2] = ind_C4p;  /*C4'*/
           ind_atm_clos[iclos][1] = ind_C3p;  /*C3'*/
           dx = x[ind_P] - x[ind_O5p];
           dy = y[ind_P] - y[ind_O5p];
           dz = z[ind_P] - z[ind_O5p];
           const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r45*/
           dx = x[ind_C5p] - x[ind_O5p];
           dy = y[ind_C5p] - y[ind_O5p];
           dz = z[ind_C5p] - z[ind_O5p];
           const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/
       }/*endif*/



       itrans++;
       if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
         errorMessage(err_msg);
       }/*endif*/
       ityp_trans[itrans] = 1; // 1
       prop_sig_scale[itrans] = 0.0; // 0.05 0.1
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1]  = ind_P;  /*P*/
       ind_prim_tors[itrans][4] = ind_P;    /*P*/
       ind_prim_tors[itrans][3] = ind_O5p;  /*O5'*/
       ind_prim_tors[itrans][2] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][1] = ind_C4p;  /*C4'*/           
       if (iresl != 1) {
           iclos++;
           if (iclos>num_clos) {
               sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:iclos>num_clos");
               errorMessage(err_msg);
           }/*endif*/
           ityp_trans[itrans] = 2;
           ind_clos_trans[itrans] = iclos;
           ityp_clos[iclos]       = 5;            /*cblc 0 scblc 1 2 3 4 5*/
           natm_clos[iclos]       = 5;
           ind_atm_clos[iclos][5] = ind_prevres_O3p;  /*O3'*/
           ind_atm_clos[iclos][4] = ind_P;            /*P*/
           ind_atm_clos[iclos][3] = ind_O5p;          /*O5'*/
           ind_atm_clos[iclos][2] = ind_C5p;          /*C5'*/
           ind_atm_clos[iclos][1] = ind_C4p;          /*C4'*/
           dx = x[ind_P] - x[ind_prevres_O3p];
           dy = y[ind_P] - y[ind_prevres_O3p];
           dz = z[ind_P] - z[ind_prevres_O3p];
           const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r45*/
           dx = x[ind_P] - x[ind_O5p];
           dy = y[ind_P] - y[ind_O5p];
           dz = z[ind_P] - z[ind_O5p];
           const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/
       }/*endif*/



       itrans++;/*~~~~~~~~~~~~~~~secondary_only~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       ityp_trans[itrans] = 0;
       secondary_only[itrans] = itrans_O5p;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1] = ind_H5p;  /*H5'*/
       ind_trans[itrans][2] = ind_H5pp; /*H5''*/
       ind_prim_tors[itrans][4] = ind_H5p;  /*H5'*/
       ind_prim_tors[itrans][3] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][2] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][1] = ind_C3p;  /*C3'*/



       itrans++;/*~~~~~~~~~~~~~~~secondary_only~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       ityp_trans[itrans] = 0;
       secondary_only[itrans] = itrans_C5p;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1]  = ind_H4p;  /*H4'*/
       ind_prim_tors[itrans][4] = ind_H4p;  /*H4'*/
       ind_prim_tors[itrans][3] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][2] = ind_C3p;  /*C3'*/
       ind_prim_tors[itrans][1] = ind_C2p;  /*C2'*/


       if (iresl != 1) {  
           itrans++;/*~~~~~~~~~~~~~~~secondary_only~~~~~~~~~~~~~~~~~~~~~~~~~~~*/           if (itrans>num_trans) {
             sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
             errorMessage(err_msg);
           }/*endif*/
           ityp_trans[itrans] = 0;
           secondary_only[itrans] = itrans;
           res_id[itrans] = ires;
           natm_trans[itrans] = 2;       /* 3rd is on other side of closure */
           ind_trans[itrans][1]  = ind_O1P;             /*O1P*/
           ind_trans[itrans][2]  = ind_O2P;             /*O2P*/
	   ind_trans[itrans][3]  = ind_prevres_O3p;     /*O3p*/
           ind_prim_tors[itrans][4] = ind_O1P;          /*O3p*/
           ind_prim_tors[itrans][3] = ind_P;            /*P*/
           ind_prim_tors[itrans][2] = ind_O5p;          /*O5'*/
           ind_prim_tors[itrans][1] = ind_C5p;          /*C5'*/           
       }
       else {
           itrans++;
           if (itrans>num_trans) {
             sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
             errorMessage(err_msg);
           }/*endif*/
           ityp_trans[itrans] = 0;
           res_id[itrans] = ires;
           natm_trans[itrans] = 2;
           ind_trans[itrans][1]  = ind_O1P;  /*O1P*/
           ind_trans[itrans][2]  = ind_O2P;  /*O2P*/
           ind_prim_tors[itrans][4] = ind_O1P;  /*O1P*/
           ind_prim_tors[itrans][3] = ind_P;    /*P*/
           ind_prim_tors[itrans][2] = ind_O5p;  /*O5'*/
           ind_prim_tors[itrans][1] = ind_C5p;  /*C5'*/           
       }/*endif*/

  }/*endfor-iresl*/

  num_trans_backbone = itrans - *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I.B) Define the groups of sidechain atoms                               */

  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[imol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     if (itrans>num_trans) {
        sprintf(err_msg,"Tors_defs::setup_AA_DNA_scblc:itrans>num_trans");
        errorMessage(err_msg);
     }/*endif*/
     ityp_trans[itrans] = 1;
     prop_sig_scale[itrans] = 0.02; // 0.05 0.1
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/
  num_trans_sidechain = itrans - *itrans_glob - num_trans_backbone;

  num_trans_mol       = num_trans_backbone + num_trans_sidechain;
  num_clos_mol        =  iclos   - *iclos_glob;
  num_clbase_mol      =  iclbase - *iclbase_glob;

/*-------------------------------------------------------------------------*/
/* II)  By default no transformation is turned off                         */

   for (itrans=1;itrans<=num_trans_mol;itrans++) {
        off_trans[itrans+(*itrans_glob)] = 0;
   }/*endfor*/

/*-------------------------------------------------------------------------*/
/* III)  If stride is defined in .pdb, only sample clbase                  */
/*       along 'C' type residue                                            */

   if (general->stride_defined==1) {
     istride = ind_stride_mol[imol];
     if (istride==0) {
      sprintf(err_msg,"%d. molecule does not have a STRIDE",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (istride>nstride) {
      sprintf(err_msg,"%d. molecule istride > nstride",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[istride][0]))-1!=nres_mol[imol]) {
      sprintf(err_msg,"%d. STRIDE in PDB is corrupted",istride);
      errorMessage(err_msg);
     }/*endif*/
     for (iclbase=1;iclbase<=num_clbase_mol;iclbase++) {
          freeze_clbase[iclbase+(*iclbase_glob)]  = 1;
          res_id_local = res_id_clbase[iclbase+(*iclbase_glob)]
                       - ind_res_mol[imol][1]+1;
          if (stride[istride][res_id_local] == 'C') {
              freeze_clbase[iclbase+(*iclbase_glob)] = 0;
          }/*endif*/
     }/*end for*/
   }/*end if*/

/*-------------------------------------------------------------------------*/
/* IV) Adjust itrans_glob, iclos_glob, iclbase_glob                        */  
   *itrans_glob  += num_trans_mol;
   *iclos_glob   += num_clos_mol;
   *iclbase_glob += num_clbase_mol;

/*-------------------------------------------------------------------------*/
}/*end routine*/
/*=========================================================================*/
