#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_AA_RNA_strand(   const Maps *maps,
                                        const Pos *pos,
                                        const General *general,
                                        int  imol,
                                        int *itrans_glob        )
/*-------------------------------------------------------------------------*/
/*     sets up all the torsional angles for an all atom RNA strand         */
/*     molecular index: imol                                               */
/*     transformation offset: itrans_glob                                  */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int ires, itrans;
  int iatm, iiatm, jatm, jjatm;
  int ichigrp;
  int istride;
  int transform;
  int num_trans_backbone, num_trans_sidechain, num_trans_mol;
  char   err_msg[LINE_NCHARMAX];

  int ind_O1P, ind_O2P, ind_P;
  int ind_O2p, ind_O3p, ind_O4p, ind_O5p;
  int ind_C1p, ind_C2p, ind_C3p, ind_C4p, ind_C5p;
  int ind_H1p, ind_H2p, ind_H3p, ind_H4p, ind_H5p, ind_H2pp, ind_H5pp;
  int ind_N9, ind_N1;

  int ind_nextres_O1P, ind_nextres_O2P, ind_nextres_P, ind_nextres_O5p;

  int    res_id_local;
  int    nres, iresl;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;

  char *atm_label;

/*-------------------------------------------------------------------------*/
/* I) Define the groups of atoms to be transformed for nucleic acids       */
/*-------------------------------------------------------------------------*/
/* I.A) Define the groups of backbone atoms                                */

  itrans = *itrans_glob;
  nres   = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
       ires = ind_res_mol[imol][iresl];
#ifdef DEBUG
       printf("from setup_nucleic\n");
       printf("ires %d, imol %d\n",ires,imol);
#endif
       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
#ifdef DEBUG
       printf("from setup_nucleic\n");
       printf("iiatm %d\n",iiatm);
       printf("atm_label %s\n",atm_label);
       getchar();
#endif
            if (strcasecmp(atm_label,"O1P")==0) {ind_O1P = iiatm;}   
            if (strcasecmp(atm_label,"O2P")==0) {ind_O2P = iiatm;}   
            if (strcasecmp(atm_label,"P")==0)   {ind_P   = iiatm;}
            if (strcasecmp(atm_label,"O3'")==0) {ind_O3p = iiatm;}   
            if (strcasecmp(atm_label,"O4'")==0) {ind_O4p = iiatm;}
            if (strcasecmp(atm_label,"O5'")==0) {ind_O5p = iiatm;}
            if (strcasecmp(atm_label,"C1'")==0) {ind_C1p = iiatm;}
            if (strcasecmp(atm_label,"C2'")==0) {ind_C2p = iiatm;}
            if (strcasecmp(atm_label,"C3'")==0) {ind_C3p = iiatm;}
            if (strcasecmp(atm_label,"C4'")==0) {ind_C4p = iiatm;}
            if (strcasecmp(atm_label,"C5'")==0) {ind_C5p = iiatm;}
            if (strcasecmp(atm_label,"H1'")==0) {ind_H1p = iiatm;}
            if (strcasecmp(atm_label,"H2'")==0) {ind_H2p = iiatm;}
            if (strcasecmp(atm_label,"H3'")==0) {ind_H3p = iiatm;}
            if (strcasecmp(atm_label,"H4'")==0) {ind_H4p = iiatm;}
            if (strcasecmp(atm_label,"H5'")==0) {ind_H5p = iiatm;}
            if (strcasecmp(atm_label,"H2''")==0){ind_H2pp = iiatm;}
            if (strcasecmp(atm_label,"H5''")==0){ind_H5pp = iiatm;}
            if (strcasecmp(atm_label,"N9")==0)  {ind_N9   = iiatm;}
            if (strcasecmp(atm_label,"N1")==0)  {ind_N1   = iiatm;}
	    if (strcasecmp(atm_label,"O2'")==0) {ind_O2p = iiatm;}  /*RNA*/
       }/*endfor*/

       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_nucleic:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][4] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][3] = ind_O5p;  /*O5'*/
       ind_prim_tors[itrans][2] = ind_P;    /*P*/
       ind_prim_tors[itrans][1] = ind_O1P;  /*O1P*/

       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_nucleic:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 3;
       ind_trans[itrans][1] = ind_C4p;  /*C4'*/
       ind_trans[itrans][2] = ind_H5p;  /*H5'*/
       ind_trans[itrans][3] = ind_H5pp; /*H5''*/
       ind_prim_tors[itrans][4] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][3] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][2] = ind_O5p;  /*O5'*/
       ind_prim_tors[itrans][1] = ind_P;    /*P*/

       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_nucleic:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 12;
       ind_trans[itrans][1]  = ind_O4p;  /*O4'*/
       ind_trans[itrans][2]  = ind_C1p;  /*C1'*/
       ind_trans[itrans][3]  = ind_C2p;  /*C2'*/
       ind_trans[itrans][4]  = ind_C3p;  /*C3'*/
       ind_trans[itrans][5]  = ind_H1p;  /*H1'*/
       ind_trans[itrans][6]  = ind_H2pp; /*H2''*/
       ind_trans[itrans][7]  = ind_H3p;  /*H3'*/
       ind_trans[itrans][8]  = ind_O3p;  /*O3'*/
       ind_trans[itrans][9]  = ind_H4p;  /*H4'*/
       ind_trans[itrans][10] = ind_O2p;  /*O2'*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"ADE")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"GUA")==0) ) {
             ind_trans[itrans][11]  = ind_N9;  /*N9*/
       }/*endif*/
       if ( (strcasecmp(pos->residue[ires].residue_name,"URA")==0) ||
            (strcasecmp(pos->residue[ires].residue_name,"CYT")==0) ) {
             ind_trans[itrans][11]  = ind_N1;  /*N1*/
       }/*endif*/

       ind_prim_tors[itrans][4] = ind_O4p;  /*O4'*/
       ind_prim_tors[itrans][3] = ind_C4p;  /*C4'*/
       ind_prim_tors[itrans][2] = ind_C5p;  /*C5'*/
       ind_prim_tors[itrans][1] = ind_O5p;  /*O5'*/

       itrans++;
       if (itrans>num_trans) {
 	  sprintf(err_msg,"Tors_defs::setup_AA_RNA_strand:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/ 
       res_id[itrans] = ires;
       natm_trans[itrans] = 1;
       ind_trans[itrans][1] = ind_H2p;  /*H2'*/
       ind_prim_tors[itrans][4] = ind_H2p;  /*H2'*/
       ind_prim_tors[itrans][3] = ind_O2p;  /*O2'*/
       ind_prim_tors[itrans][2] = ind_C2p;  /*C2'*/
       ind_prim_tors[itrans][1] = ind_C1p;  /*C1'*/

      
    if (iresl==nres) {continue;}   /*~~~~~~~~~~~~~next residue~~~~~~~~~~~~*/
    ires = ind_res_mol[imol][iresl+1];
    for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
         iiatm     = pos->residue[ires].atom[iatm];
         atm_label = maps->upac_atm_label[iiatm];
         if (strcasecmp(atm_label,"O1P")==0) {ind_nextres_O1P = iiatm;}   
         if (strcasecmp(atm_label,"O2P")==0) {ind_nextres_O2P = iiatm;}   
         if (strcasecmp(atm_label,"P")==0)   {ind_nextres_P   = iiatm;}
         if (strcasecmp(atm_label,"O5'")==0) {ind_nextres_O5p = iiatm;}
    }/*endfor*/

    itrans++;
    if (itrans>num_trans) {
	  sprintf(err_msg,"Tors_defs::setup_AA_nucleic:itrans>num_trans");
       errorMessage(err_msg);
    }/*endif*/
    res_id[itrans] = ires;
    natm_trans[itrans] = 1;
    ind_trans[itrans][1] = ind_nextres_P; /*P*/
    ind_prim_tors[itrans][4] = ind_nextres_P; /*P*/
    ind_prim_tors[itrans][3] = ind_O3p; /*O3'*/
    ind_prim_tors[itrans][2] = ind_C3p; /*C3'*/
    ind_prim_tors[itrans][1] = ind_C4p; /*C4'*/

    itrans++;
    if (itrans>num_trans) {
	  sprintf(err_msg,"Tors_defs::setup_AA_nucleic:itrans>num_trans");
       errorMessage(err_msg);
    }/*endif*/
    res_id[itrans] = ires;
    natm_trans[itrans] = 3;
    ind_trans[itrans][1] = ind_nextres_O5p; /*O5'*/
    ind_trans[itrans][2] = ind_nextres_O1P; /*O1P*/
    ind_trans[itrans][3] = ind_nextres_O2P; /*O2P*/
    ind_prim_tors[itrans][4] = ind_nextres_O5p; /*O5'*/
    ind_prim_tors[itrans][3] = ind_nextres_P;  /*P*/
    ind_prim_tors[itrans][2] = ind_O3p; /*O3'*/
    ind_prim_tors[itrans][1] = ind_C3p; /*C3'*/

  }/*endfor-iresl*/

  num_trans_backbone = itrans - *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I.B) Define the groups of sidechain atoms                               */

  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[imol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     if (itrans>num_trans) {
        sprintf(err_msg,"Tors_defs::setup_AA_nucleic:itrans>num_trans");
        errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/
  num_trans_sidechain = itrans - *itrans_glob - num_trans_backbone;
  num_trans_mol       = num_trans_backbone + num_trans_sidechain;

/*-------------------------------------------------------------------------*/
/* II) Transformation types                                                */
/*     By default, everybody is transformed                                */

   for (itrans=1;itrans<=num_trans_mol;itrans++) {
        ityp_trans[itrans+(*itrans_glob)] = 1;
        off_trans[itrans+(*itrans_glob)]  = 0;
   }/*end for*/

/*-------------------------------------------------------------------------*/
/* III) If stride is defined in .pdb, only sample backbone                 */
/*       along 'C' secondary structure type residue                        */
/*       sidechain torsions are always transformed                         */

   if (general->stride_defined==1) {
     istride = ind_stride_mol[imol];
     if (istride==0) {
      sprintf(err_msg,"%d. molecule does not have a STRIDE",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (istride>nstride) {
      sprintf(err_msg,"%d. molecule istride > nstride",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[istride][0]))-1!=nres_mol[imol]) {
      sprintf(err_msg,"%d. STRIDE in PDB is corrupted",istride);
      errorMessage(err_msg);
     }/*endif*/
     for (itrans=1;itrans<=num_trans_backbone;itrans++) {
          ityp_trans[itrans+(*itrans_glob)] = 1;
          off_trans[itrans+(*itrans_glob)]  = 1;
          res_id_local = res_id[itrans+(*itrans_glob)]-ind_res_mol[imol][1]+1;
          if (stride[istride][res_id_local] == 'C') {
              off_trans[itrans+(*itrans_glob)] = 0;
          }/*endif*/
     }/*end for*/
   }/*end if*/

/*-------------------------------------------------------------------------*/
/* IV) Adjust itrans_glob                                                  */  
   *itrans_glob += num_trans_mol; 

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/
