#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_AA_protein(   const Maps *maps,
                                     const Pos *pos,
                                     const General *general,
                                     int  imol,
                                     int *itrans_glob        )
/*-------------------------------------------------------------------------*/
/*     sets up all the torsional angles for an all atom protein            */
/*     molecular index: imol                                               */
/*     transformation offset: itrans_glob                                  */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int ires, itrans;
  int iatm, iiatm, jatm, jjatm;
  int ichigrp;
  int istride;
  int transform;
  int num_trans_backbone, num_trans_sidechain, num_trans_mol;
  char   err_msg[LINE_NCHARMAX];

  int    res_id_local;
  int    nres, iresl;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;



/*-------------------------------------------------------------------------*/
/* 0) Incompatibilities                                                    */
/*-------------------------------------------------------------------------*/

    if (nres_mol[imol]==1) {
        sprintf(err_msg,"Tors_defs::setup_AA_protein:nres_mol[%d] = 1\n"
                        "Minimum two residues are required when using\n"
                        "regular torsional definitions for proteins!",imol);
        errorMessage(err_msg);
    }/*endif*/

/*-------------------------------------------------------------------------*/
/* I) Define the groups of atoms to be transformed for proteins            */
/*-------------------------------------------------------------------------*/
/* I.A) Define the groups of backbone atoms                                */

  itrans = *itrans_glob;
  nres = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
    if (iresl==1) {continue;}
     
     ires = ind_res_mol[imol][iresl];
     if (strcasecmp(pos->residue[ires].residue_name,"PRO")==0) {
       itrans++;
       if (itrans>num_trans) {
 	 sprintf(err_msg,"Tors_defs::setup_AA_protein:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 2;
       ind_trans[itrans][1] = pos->residue[ires  ].branch_root_f[0][0];  /* N*/
       ind_trans[itrans][2] = pos->residue[ires-1].branch_root_s[2][0];  /* O*/

       ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
       ind_prim_tors[itrans][3] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
       ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_f[1][0];/*CA*/
       ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_f[0][0];/*N*/

       itrans++;
       if (itrans>num_trans) {
           sprintf(err_msg,"Tors_defs::setup_AA_protein:itrans>num_trans");
           errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       natm_trans[itrans] = 5;
       ind_trans[itrans][1] = pos->residue[ires].branch_root_f[1][0];  /*CA*/
       ind_trans[itrans][2] = pos->residue[ires].branch_root_s[0][0]; /*C*/
       ind_trans[itrans][3] = pos->residue[ires].branch_root_f[1][2]; /*CB*/
       ind_trans[itrans][4] = pos->residue[ires].branch_root_f[2][0];  /*CD*/
       ind_trans[itrans][5] = pos->residue[ires].branch_root_f[2][1];  /*CG*/
 
       for (iatm=1;iatm<=(pos->residue[ires].natom);iatm++) {/* all H*/
            transform = 1;
            iiatm = pos->residue[ires].atom[iatm];
            for (jatm=1;jatm<=natm_trans[itrans];jatm++) {
	         jjatm = ind_trans[itrans][jatm];
	         if (iiatm==jjatm) {
		     transform = 0;
                 }/*endif*/
            }/*endfor*/

            if (iiatm==pos->residue[ires].branch_root_s[2][0]) {
		     transform = 0;
            }/*endif*/
            if (iiatm==pos->residue[ires].branch_root_f[0][0]) {
		     transform = 0;
            }/*endif*/
            if (pos->residue[ires].nbranch_s1>2) {/* terminal COO- */
                if (iiatm==pos->residue[ires].branch_root_s[3][0]) {
		         transform = 0;
                }/*endif*/
            }/*endif*/ 

            if ( transform == 1 ) {  
	         natm_trans[itrans]++;
                 ind_trans[itrans][natm_trans[itrans]] = iiatm;
            }/*endif*/                
       }/*endfor all H*/

       if (ires==pos->nres) {
           natm_trans[itrans]++;
           ind_trans[itrans][natm_trans[itrans]] 
                                = pos->residue[ires].branch_root_s[2][0];/*OC*/
           if (pos->residue[ires].nbranch_s1>2) {/* terminal COO- */
               natm_trans[itrans]++;
               ind_trans[itrans][natm_trans[itrans]] 
                                = pos->residue[ires].branch_root_s[3][0];/*OC*/
           }
       }/*end if*/


       ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[1][0];/*CA*/
       ind_prim_tors[itrans][3] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
       ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
       ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_s[1][0];/*CA*/
       continue;
     }/*PRO residue*/

     if (strcasecmp(pos->residue[ires].residue_name,"GLY")==0) {
         itrans++;
         if (itrans>num_trans) {
             sprintf(err_msg,"Tors_defs::setup_AA_protein:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         res_id[itrans] = ires;
         natm_trans[itrans] = 4;
         ind_trans[itrans][1] = pos->residue[ires  ].branch_root_f[0][0];  /* N*/
         ind_trans[itrans][2] = pos->residue[ires  ].branch_root_f[1][0];  /*CA*/
         ind_trans[itrans][3] = pos->residue[ires  ].branch_root_f[2][0];  /*H*/
         ind_trans[itrans][4] = pos->residue[ires-1].branch_root_s[2][0];  /*OC*/
         ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
         ind_prim_tors[itrans][3] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
         ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_f[1][0];/*CA*/
         ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_f[0][0];/*N*/
     
         itrans++;
         if (itrans>num_trans) {
             sprintf(err_msg,"Tors_defs::setup_AA_protein:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         res_id[itrans] = ires;
         natm_trans[itrans] = 3;
         ind_trans[itrans][1] = pos->residue[ires].branch_root_s[0][0]; /*C*/
         ind_trans[itrans][2] = pos->residue[ires].branch_root_s[1][2]; /*HA1*/
         ind_trans[itrans][3] = pos->residue[ires].branch_root_s[1][3]; /*HA2*/
         if (ires==pos->nres) {
             natm_trans[itrans] = 4;
             ind_trans[itrans][4] = pos->residue[ires].branch_root_s[2][0];/*OC*/
             if (pos->residue[ires].nbranch_s1>2) {/* terminal COO- */
              natm_trans[itrans] = 5;
              ind_trans[itrans][5] = pos->residue[ires].branch_root_s[3][0];/*OC*/
             }/*endif*/
         }/*end if*/
     
         ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_s[0][0];/*C*/
         ind_prim_tors[itrans][3] = pos->residue[ires  ].branch_root_s[1][0];/*CA*/         ind_prim_tors[itrans][2] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
         ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
         continue;
     }/*Gly residue*/

     itrans++;
     if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_AA_protein:itrans>num_trans");
         errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = 4;
     ind_trans[itrans][1] = pos->residue[ires  ].branch_root_f[0][0];  /* N*/
     ind_trans[itrans][2] = pos->residue[ires  ].branch_root_f[1][0];  /*CA*/
     ind_trans[itrans][3] = pos->residue[ires  ].branch_root_f[2][0];  /*H*/
     ind_trans[itrans][4] = pos->residue[ires-1].branch_root_s[2][0];  /*OC*/
     ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
     ind_prim_tors[itrans][3] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
     ind_prim_tors[itrans][2] = pos->residue[ires-1].branch_root_f[1][0];/*CA*/
     ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_f[0][0];/*N*/

     itrans++;
     if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_AA_protein:itrans>num_trans");
         errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = 3;
     ind_trans[itrans][1] = pos->residue[ires].branch_root_s[0][0]; /*C*/
     ind_trans[itrans][2] = pos->residue[ires].branch_root_s[1][1]; /*CB*/
     ind_trans[itrans][3] = pos->residue[ires].branch_root_s[1][3]; /*HA*/
     if (ires==pos->nres) {
         natm_trans[itrans] = 4;
         ind_trans[itrans][4] = pos->residue[ires].branch_root_s[2][0];/*OC*/
         if (pos->residue[ires].nbranch_s1>2) {/* terminal COO- */
             natm_trans[itrans] = 5;
             ind_trans[itrans][5] = pos->residue[ires].branch_root_s[3][0];/*OC*/
         }/*endif*/
     }/*end if*/

     ind_prim_tors[itrans][4] = pos->residue[ires  ].branch_root_s[0][0];/*C*/
     ind_prim_tors[itrans][3] = pos->residue[ires  ].branch_root_s[1][0];/*CA*/
     ind_prim_tors[itrans][2] = pos->residue[ires  ].branch_root_f[0][0];/*N*/
     ind_prim_tors[itrans][1] = pos->residue[ires-1].branch_root_s[0][0];/*C*/
  }/*end for*/
  num_trans_backbone = itrans - *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I.B) Define the groups of sidechain atoms                               */

  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[imol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     if (itrans>num_trans) {
        sprintf(err_msg,"Tors_defs::setup_cgresmodel_UA:itrans>num_trans");
        errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/
  num_trans_sidechain = itrans - *itrans_glob - num_trans_backbone;
  num_trans_mol       = num_trans_backbone + num_trans_sidechain;

/*-------------------------------------------------------------------------*/
/* II) Transformation types                                                */
/*     By default, everybody is transformed                                */

   for (itrans=1;itrans<=num_trans_mol;itrans++) {
        ityp_trans[itrans+(*itrans_glob)] = 1;
        off_trans[itrans+(*itrans_glob)]  = 0;
   }/*end for*/

/*-------------------------------------------------------------------------*/
/* III) If stride is defined in .pdb, only sample backbone                 */
/*       along 'C' secondary structure type residue                        */
/*       sidechain torsions are always transformed                         */

   if (general->stride_defined==1) {
     istride = ind_stride_mol[imol];
     if (istride==0) {
      sprintf(err_msg,"%d. molecule does not have a STRIDE",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (istride>nstride) {
      sprintf(err_msg,"%d. molecule istride > nstride",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[istride][0]))-1!=nres_mol[imol]) {
      sprintf(err_msg,"%d. STRIDE in PDB is corrupted",istride);
      errorMessage(err_msg);
     }/*endif*/
     for (itrans=1;itrans<=num_trans_backbone;itrans++) {
         ityp_trans[itrans+(*itrans_glob)] = 1;
         off_trans[itrans+(*itrans_glob)]  = 1;
         res_id_local = res_id[itrans+(*itrans_glob)]-ind_res_mol[imol][1] + 1;
         if (stride[istride][res_id_local] == 'C') {
             off_trans[itrans+(*itrans_glob)] = 0;
         }/*endif*/
     }/*end for*/
   }/*end if*/

/*-------------------------------------------------------------------------*/
/* IV) Adjust itrans_glob                                                  */  
   *itrans_glob += num_trans_mol; 

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/
