#include "../../include/standard.h"
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_AA_protein_D6(   const Maps *maps,
                                        const Pos *pos,
                                        const General *general,
                                        int  itorcompl,
                                        int *itrans_glob        )
/*-------------------------------------------------------------------------*/
/*   sets up all the torsional angles for an all atom protein              */
/*   and six dummy atoms responsible for the realtive orientaion of        */
/*   the central CA atom                                                   */
/*   locally these molecules are called:                                   */ 
/*                                       PROT, D1, D2, D3, D4, D5, D6      */
/*   torsional complex index: itorcompl                                    */
/*   transformation offset: itrans_glob                                    */
/*                                                                         */
/*   global indexing assumes and follows the order in TORCOMPL> definition */
/*   see. tors_defs_AA_control.cpp                                         */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int i;
  int ires, iresl, nres;
  int itrans, ichigrp, istride, transform;
  int res_id_local;

  int iatm, iiatm, imol, iimol;
  int num_trans_backbone, num_trans_sidechain, num_trans_mol;

  int ind_HN, ind_N, ind_CA, ind_HA, ind_CB, ind_C, ind_O;
  int ind_HA1, ind_HA2;         /*GLY*/
  int ind_CD, ind_HD1, ind_HD2; /*PRO*/ 
  int ind_CG, ind_HG1, ind_HG2; /*PRO*/
  int ind_HB1, ind_HB2;         /*PRO*/

  int ind_nextres_N, ind_nextres_HN, ind_nextres_CA, ind_nextres_C;
  int ind_prevres_C, ind_prevres_O,  ind_prevres_CA, ind_prevres_N;
  int ind_centres_C, ind_centres_O,  ind_centres_CA, ind_centres_N;

  int nextres_PRO, res_PRO, res_GLY;

  int ind_PROT;                                         /* molecular index */
  int ind_D1, ind_D2, ind_D3, ind_D4, ind_D5, ind_D6;   /*      atom index */

  int ind_rcent_PROT, rcent_PRO, rcent_GLY;
 
  char *atm_label;

  int    nmol        = maps->nmol;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;
  char **mol_label   = maps->mol_label;
  char **mol_chainid = maps->mol_chainid;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0.A) Get the global indices of molecule  PROT                           */ 
/*                               and atoms  D1,  D2,  D3                   */
/*                                          D4,  D5,  D6                   */
/*-------------------------------------------------------------------------*/
 
  ind_PROT = 0;
  for (imol=1;imol<=nmol_torcompl[itorcompl];imol++) {
       iimol = ind_mol_torcompl[itorcompl][imol];
       if ( (strcasecmp(mol_label[iimol],"protein")==0) ) {
	     ind_PROT = iimol;
       }/*endif*/
  }/*endfor*/

  if ( ind_PROT != ind_mol_torcompl[itorcompl][1] ) {
       sprintf(err_msg,"Tors_defs::setup_AA_protein_D6           \n"
                       "Protein is not the first molecule        \n"
                       "defined in TORCOMPL>                     \n");
       errorMessage(err_msg);
  }/*endif*/

  for (imol=2;imol<=nmol_torcompl[itorcompl];imol++) {
       iimol = ind_mol_torcompl[itorcompl][imol];
       ires  = ind_res_mol[iimol][1];
       switch (imol) {
         case 2:
            ind_D1 = pos->residue[ires].atom[1];
         case 3:
            ind_D2 = pos->residue[ires].atom[1];
         case 4:
            ind_D3 = pos->residue[ires].atom[1];
         case 5:
            ind_D4 = pos->residue[ires].atom[1];
         case 6:
            ind_D5 = pos->residue[ires].atom[1];
         case 7:
            ind_D6 = pos->residue[ires].atom[1];
       }/*end switch*/
  }/*endfor*/

/*-------------------------------------------------------------------------*/
/* 0.B) Get the index of the central residue in PROT                       */
/*-------------------------------------------------------------------------*/
  
   if ( (nres_mol[ind_PROT] % 2) == 0 ) {/*even*/
         ind_rcent_PROT = nres_mol[ind_PROT]/2;
   }/*endif*/

   if ( (nres_mol[ind_PROT] % 2) == 1 ) {/*odd*/
         ind_rcent_PROT = (nres_mol[ind_PROT]-1)/2 + 1;
   }/*endif*/

   ires = ind_res_mol[ind_PROT][ind_rcent_PROT];


   if ( nres_mol[ind_PROT] < 3 ) {
        sprintf(err_msg,"Tors_defs::setup_AA_protein_D6          \n"
                        "Due to consistent generalization of     \n"
                        "torsional definitions, protein molecule \n" 
                        "with chain %s has to have at least 3    \n"
                        "residues \n",mol_chainid[ind_PROT]);
        errorMessage(err_msg);
   }/*endif*/


#ifdef DEBUG
   printf("molecular indices\n");
   printf("ind_PROT %d\n",ind_PROT);
   printf("residue indices\n");
   printf("ind_rcent_PROT %d\n",ind_rcent_PROT);
   printf("atomic indices\n");
   printf("ind_D1 %d, ind_D2 %d\n",ind_D1, ind_D2);
   printf("ind_D3 %d, ind_D4 %d\n",ind_D3, ind_D4);
   printf("ind_D5 %d, ind_D6 %d\n",ind_D5, ind_D6);
#endif



/*-------------------------------------------------------------------------*/
/* 0.C) Set the number and indices of base atoms                           */
/*-------------------------------------------------------------------------*/

   natm_torcomplbase[itorcompl]       = 3;
   ind_atm_torcomplbase[itorcompl][1] = ind_D1;
   ind_atm_torcomplbase[itorcompl][2] = ind_D2;
   ind_atm_torcomplbase[itorcompl][3] = ind_D3;

/*-------------------------------------------------------------------------*/
/* I) Define the groups of backbone atoms to be transformed for protein_D6 */

  itrans = *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I.A) Define tors groups out of atoms DUM and atoms in ind_rcent_PROT    */
/*      Plus any atoms in overlaping groups with previous and next residue */
/*      Special cases if central residue is PRO or GLY                     */

  rcent_PRO = 0;
  rcent_GLY = 0;
  ires = ind_res_mol[ind_PROT][ind_rcent_PROT];
  
  if  ( strcasecmp(pos->residue[ires].residue_name,"PRO")==0 ) {rcent_PRO = 1;}
  if  ( strcasecmp(pos->residue[ires].residue_name,"GLY")==0 ) {rcent_GLY = 1;}

  for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
       iiatm     = pos->residue[ires].atom[iatm];
       atm_label = maps->upac_atm_label[iiatm];

       if (strcasecmp(atm_label,"HN")==0)  {ind_HN  = iiatm;}   
       if (strcasecmp(atm_label,"N")==0)   {ind_N   = iiatm;}   
       if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
       if (strcasecmp(atm_label,"HA")==0)  {ind_HA  = iiatm;}   
       if (strcasecmp(atm_label,"CB")==0)  {ind_CB  = iiatm;}
       if (strcasecmp(atm_label,"C")==0)   {ind_C   = iiatm;}
       if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
       if (strcasecmp(atm_label,"HA1")==0) {ind_HA1 = iiatm;}
       if (strcasecmp(atm_label,"HA2")==0) {ind_HA2 = iiatm;}
       if (strcasecmp(atm_label,"CD")==0)  {ind_CD  = iiatm;}
       if (strcasecmp(atm_label,"HD1")==0) {ind_HD1 = iiatm;}
       if (strcasecmp(atm_label,"HD2")==0) {ind_HD2 = iiatm;}
       if (strcasecmp(atm_label,"CG")==0)  {ind_CG  = iiatm;}
       if (strcasecmp(atm_label,"HG1")==0) {ind_HG1 = iiatm;}
       if (strcasecmp(atm_label,"HG2")==0) {ind_HG2 = iiatm;}
       if (strcasecmp(atm_label,"HB1")==0) {ind_HB1 = iiatm;}
       if (strcasecmp(atm_label,"HB2")==0) {ind_HB2 = iiatm;}

  }/*endfor*/

  ires = ind_res_mol[ind_PROT][ind_rcent_PROT-1];
  for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
       iiatm     = pos->residue[ires].atom[iatm];
       atm_label = maps->upac_atm_label[iiatm];
       if (strcasecmp(atm_label,"C")==0)   {ind_prevres_C  = iiatm;}   
       if (strcasecmp(atm_label,"O")==0)   {ind_prevres_O  = iiatm;}   
       if (strcasecmp(atm_label,"CA")==0)  {ind_prevres_CA = iiatm;}
  }/*endfor*/
  ires = ind_res_mol[ind_PROT][ind_rcent_PROT];

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   natm_trans[itrans]   = 1;
   ind_trans[itrans][1] = ind_D4;       /*D4*/
   ind_prim_tors[itrans][4] = ind_D4;   /*D4*/
   ind_prim_tors[itrans][3] = ind_D3;   /*D3*/
   ind_prim_tors[itrans][2] = ind_D2;   /*D2*/
   ind_prim_tors[itrans][1] = ind_D1;   /*D1*/

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   natm_trans[itrans]   = 3;
   ind_trans[itrans][1] = ind_CA;       /*CA*/
   ind_trans[itrans][2] = ind_D5;       /*D5*/
   ind_trans[itrans][3] = ind_D6;       /*D6*/
   ind_prim_tors[itrans][4] = ind_CA;   /*CA*/
   ind_prim_tors[itrans][3] = ind_D4;   /*D4*/
   ind_prim_tors[itrans][2] = ind_D3;   /*D3*/
   ind_prim_tors[itrans][1] = ind_D2;   /*D2*/

   itrans++;
   if (itrans>num_trans) {
      sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
      errorMessage(err_msg);
   }/*endif*/
   res_id[itrans] = ires;
   if ( rcent_PRO == 1 ) {
       natm_trans[itrans]    = 15;
       ind_trans[itrans][1]  = ind_C;          /*C  */
       ind_trans[itrans][2]  = ind_HA;         /*HA */
       ind_trans[itrans][3]  = ind_CB;         /*CB */
       ind_trans[itrans][4]  = ind_HB1;        /*HB1*/
       ind_trans[itrans][5]  = ind_HB2;        /*HB2*/
       ind_trans[itrans][6]  = ind_CG;         /*CG */
       ind_trans[itrans][7]  = ind_HG1;        /*HG1*/
       ind_trans[itrans][8]  = ind_HG2;        /*HG2*/
       ind_trans[itrans][9]  = ind_CD;         /*CD */
       ind_trans[itrans][10] = ind_HD1;        /*HD1*/
       ind_trans[itrans][11] = ind_HD2;        /*HD2*/
       ind_trans[itrans][12] = ind_N;          /*N  */
       ind_trans[itrans][13] = ind_prevres_C;  /*C  */
       ind_trans[itrans][14] = ind_prevres_O;  /*O  */
       ind_trans[itrans][15] = ind_prevres_CA; /*CA */
   }
   else if ( rcent_GLY == 1 ) {
       natm_trans[itrans]   = 4;
       ind_trans[itrans][1] = ind_C;           /*C  */
       ind_trans[itrans][2] = ind_HA1;         /*HA1*/
       ind_trans[itrans][3] = ind_HA2;         /*HA2*/
       ind_trans[itrans][4] = ind_N;           /*N  */
   } 
   else {
       natm_trans[itrans]   = 4;
       ind_trans[itrans][1] = ind_C;           /*C */
       ind_trans[itrans][2] = ind_HA;          /*HA*/
       ind_trans[itrans][3] = ind_CB;          /*CB*/
       ind_trans[itrans][4] = ind_N;           /*N */
   }/*endif*/
   ind_prim_tors[itrans][4] = ind_C;           /*C */
   ind_prim_tors[itrans][3] = ind_CA;          /*CA*/
   ind_prim_tors[itrans][2] = ind_D6;          /*D6*/
   ind_prim_tors[itrans][1] = ind_D5;          /*D5*/


#ifdef DEBUG
   printf("Check Point 0\n");
   num_trans_backbone = itrans - *itrans_glob;
   printf("num_trans_backbone %d\n",num_trans_backbone);
   getchar();
#endif


/*-------------------------------------------------------------------------*/
/* I.B) Define tors groups on residues backwards from ind_rcent_PROT-1 to  */
/*      the first residue                                                  */
  
  ind_centres_C  = ind_C;
  ind_centres_O  = ind_O;
  ind_centres_CA = ind_CA;
  ind_centres_N  = ind_N;

  ind_nextres_N   = ind_N;
  ind_nextres_HN  = ind_HN;
  ind_nextres_CA  = ind_CA;
  ind_nextres_C   = ind_C;
  nextres_PRO     = rcent_PRO;

  for (iresl=ind_rcent_PROT-1;iresl>=1;iresl--) {/*backwards*/

       ires = ind_res_mol[ind_PROT][iresl];
       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"HN")==0)  {ind_HN  = iiatm;}   
            if (strcasecmp(atm_label,"N")==0)   {ind_N   = iiatm;}   
            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"HA")==0)  {ind_HA  = iiatm;}   
            if (strcasecmp(atm_label,"CB")==0)  {ind_CB  = iiatm;}
            if (strcasecmp(atm_label,"C")==0)   {ind_C   = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"HA1")==0) {ind_HA1 = iiatm;}
            if (strcasecmp(atm_label,"HA2")==0) {ind_HA2 = iiatm;}
            if (strcasecmp(atm_label,"CD")==0)  {ind_CD  = iiatm;}
            if (strcasecmp(atm_label,"HD1")==0) {ind_HD1 = iiatm;}
            if (strcasecmp(atm_label,"HD2")==0) {ind_HD2 = iiatm;}
            if (strcasecmp(atm_label,"CG")==0)  {ind_CG  = iiatm;}
            if (strcasecmp(atm_label,"HG1")==0) {ind_HG1 = iiatm;}
            if (strcasecmp(atm_label,"HG2")==0) {ind_HG2 = iiatm;}
            if (strcasecmp(atm_label,"HB1")==0) {ind_HB1 = iiatm;}
            if (strcasecmp(atm_label,"HB2")==0) {ind_HB2 = iiatm;}
       }/*endfor*/
  
       if  ( strcasecmp(pos->residue[ires].residue_name,"PRO")==0 ) 
            {res_PRO = 1;}
       else {res_PRO = 0;}
       if  ( strcasecmp(pos->residue[ires].residue_name,"GLY")==0 ) 
            {res_GLY = 1;}
       else {res_GLY = 0;}

       if ( (res_PRO==1) && (iresl>1) ) {
          ires = ind_res_mol[ind_PROT][iresl-1];
          for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
               iiatm     = pos->residue[ires].atom[iatm];
               atm_label = maps->upac_atm_label[iiatm];
               if (strcasecmp(atm_label,"C")==0)   {ind_prevres_C  = iiatm;}   
               if (strcasecmp(atm_label,"O")==0)   {ind_prevres_O  = iiatm;}   
               if (strcasecmp(atm_label,"CA")==0)  {ind_prevres_CA = iiatm;}
          }/*endfor*/
          ires = ind_res_mol[ind_PROT][iresl];
       }/*endif*/
    
       if (nextres_PRO != 1) {
           itrans++;
           if (itrans>num_trans) {
             sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
             errorMessage(err_msg);
           }/*endif*/
           res_id[itrans] = ires; 
           natm_trans[itrans]   = 4;
           ind_trans[itrans][1] = ind_C;              /*C */
           ind_trans[itrans][2] = ind_O;              /*O */
           ind_trans[itrans][3] = ind_CA;             /*CA*/
           ind_trans[itrans][4] = ind_nextres_HN;     /*HN*/
           ind_prim_tors[itrans][4] = ind_C;          /*C */
           ind_prim_tors[itrans][3] = ind_nextres_N;  /*N */
           ind_prim_tors[itrans][2] = ind_nextres_CA; /*CA*/
           ind_prim_tors[itrans][1] = ind_nextres_C;  /*C */
       }/*endif*/

       itrans++;
       if (itrans>num_trans) {
         sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
         errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       if (res_PRO) {
         if (iresl!=1) {
             natm_trans[itrans]  = 14;
         } else {
             natm_trans[itrans]  = 11;
         }/*endif*/
         ind_trans[itrans][1]  = ind_N;                 /*N  */
         ind_trans[itrans][2]  = ind_CB;                /*CB */
         ind_trans[itrans][3]  = ind_HB1;               /*HB1*/
         ind_trans[itrans][4]  = ind_HB2;               /*HB2*/
         ind_trans[itrans][5]  = ind_CG;                /*CG */
         ind_trans[itrans][6]  = ind_HG1;               /*HG1*/
         ind_trans[itrans][7]  = ind_HG2;               /*HG2*/
         ind_trans[itrans][8]  = ind_CD;                /*CD */
         ind_trans[itrans][9]  = ind_HD1;               /*HD1*/
         ind_trans[itrans][10] = ind_HD2;               /*HD2*/
         ind_trans[itrans][11] = ind_HA;                /*HA */
         if (iresl!=1) {
             ind_trans[itrans][12] = ind_prevres_C;     /*C  */
             ind_trans[itrans][13] = ind_prevres_O;     /*O  */
             ind_trans[itrans][14] = ind_prevres_CA;    /*CA */
         }/*endif*/
       }
       else {/*regular residue or GLY*/
         natm_trans[itrans]   = 3;
         if (res_GLY) {
             ind_trans[itrans][1] = ind_N;              /*N  */
             ind_trans[itrans][2] = ind_HA1;            /*HA1*/
             ind_trans[itrans][3] = ind_HA2;            /*HA2*/
         } 
         else {
             ind_trans[itrans][1] = ind_N;              /*N  */
             ind_trans[itrans][2] = ind_HA;             /*HA */
             ind_trans[itrans][3] = ind_CB;             /*CB */
         }/*endif*/
       }/*endif*/
       ind_prim_tors[itrans][4] = ind_N;            /*N  */
       ind_prim_tors[itrans][3] = ind_CA;           /*CA */
       ind_prim_tors[itrans][2] = ind_C;            /*C  */
       ind_prim_tors[itrans][1] = ind_nextres_N;    /*N  */

       if (iresl==1) {       
          itrans++;
          if (itrans>num_trans) {
       	  sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
             errorMessage(err_msg);
          }/*endif*/
          res_id[itrans] = ires;
          natm_trans[itrans] = 1;
          ind_trans[itrans][1]     = ind_HN;          /*HN */
          ind_prim_tors[itrans][4] = ind_HN;          /*HN */
          ind_prim_tors[itrans][3] = ind_N;           /*N  */
          ind_prim_tors[itrans][2] = ind_CA;          /*CA */
          ind_prim_tors[itrans][1] = ind_C;           /*C  */
       }/*endif*/

       ind_nextres_N   = ind_N;
       ind_nextres_HN  = ind_HN;
       ind_nextres_CA  = ind_CA;
       ind_nextres_C   = ind_C;
       nextres_PRO     = res_PRO;
  }/*endfor-iresl*/



#ifdef DEBUG
   printf("Check Point I\n");
   num_trans_backbone = itrans - *itrans_glob;
   printf("num_trans_backbone %d\n",num_trans_backbone);
   getchar();
#endif


/*-------------------------------------------------------------------------*/
/* I.C) Define tors groups on residues forward from ind_rcent_PROT+1 to the*/
/*      last residue                                                       */

  ind_prevres_C  = ind_centres_C;   
  ind_prevres_O  = ind_centres_O;   
  ind_prevres_CA = ind_centres_CA;
  ind_prevres_N  = ind_centres_N;

  nres   = nres_mol[ind_PROT];
  for (iresl=ind_rcent_PROT+1;iresl<=nres;iresl++) {
       ires = ind_res_mol[ind_PROT][iresl];
       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"HN")==0)  {ind_HN  = iiatm;}   
            if (strcasecmp(atm_label,"N")==0)   {ind_N   = iiatm;}   
            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"HA")==0)  {ind_HA  = iiatm;}   
            if (strcasecmp(atm_label,"CB")==0)  {ind_CB  = iiatm;}
            if (strcasecmp(atm_label,"C")==0)   {ind_C   = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"HA1")==0) {ind_HA1 = iiatm;}
            if (strcasecmp(atm_label,"HA2")==0) {ind_HA2 = iiatm;}
            if (strcasecmp(atm_label,"CD")==0)  {ind_CD  = iiatm;}
            if (strcasecmp(atm_label,"HD1")==0) {ind_HD1 = iiatm;}
            if (strcasecmp(atm_label,"HD2")==0) {ind_HD2 = iiatm;}
            if (strcasecmp(atm_label,"CG")==0)  {ind_CG  = iiatm;}
            if (strcasecmp(atm_label,"HG1")==0) {ind_HG1 = iiatm;}
            if (strcasecmp(atm_label,"HG2")==0) {ind_HG2 = iiatm;}
            if (strcasecmp(atm_label,"HB1")==0) {ind_HB1 = iiatm;}
            if (strcasecmp(atm_label,"HB2")==0) {ind_HB2 = iiatm;}
       }/*endfor*/
  
       if  ( strcasecmp(pos->residue[ires].residue_name,"PRO")==0 ) 
            {res_PRO = 1;}
       else {res_PRO = 0;}
       if  ( strcasecmp(pos->residue[ires].residue_name,"GLY")==0 ) 
            {res_GLY = 1;}
       else {res_GLY = 0;}

       itrans++;
       if (itrans>num_trans) {
          sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
          errorMessage(err_msg);
       }/*endif*/
       res_id[itrans] = ires;
       if (res_PRO) {
             natm_trans[itrans]  = 14;
             ind_trans[itrans][1]  = ind_N;                 /*N  */
             ind_trans[itrans][2]  = ind_HA;                /*HA */
             ind_trans[itrans][3]  = ind_CB;                /*CB */
             ind_trans[itrans][4]  = ind_HB1;               /*HB1*/
             ind_trans[itrans][5]  = ind_HB2;               /*HB2*/
             ind_trans[itrans][6]  = ind_CG;                /*CG */
             ind_trans[itrans][7]  = ind_HG1;               /*HG1*/
             ind_trans[itrans][8]  = ind_HG2;               /*HG2*/
             ind_trans[itrans][9]  = ind_CD;                /*CD */
             ind_trans[itrans][10] = ind_HD1;               /*HD1*/
             ind_trans[itrans][11] = ind_HD2;               /*HD2*/
             ind_trans[itrans][12] = ind_CA;                /*CA */
             ind_trans[itrans][13] = ind_C;                 /*C  */
             ind_trans[itrans][14] = ind_prevres_O;         /*O  */
       }
       else {
             natm_trans[itrans]   = 4;
             ind_trans[itrans][1] = ind_N;                  /*N */
             ind_trans[itrans][2] = ind_HN;                 /*HN*/
             ind_trans[itrans][3] = ind_CA;                 /*CA*/
             ind_trans[itrans][4] = ind_prevres_O;          /*O */
       }/*endif*/
       ind_prim_tors[itrans][4] = ind_N;                    /*N */
       ind_prim_tors[itrans][3] = ind_prevres_C;            /*C */
       ind_prim_tors[itrans][2] = ind_prevres_CA;           /*CA*/
       ind_prim_tors[itrans][1] = ind_prevres_N;            /*N */


       if (res_PRO==0) {/*everything but PRO*/
         itrans++;
         if (itrans>num_trans) {
            sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
            errorMessage(err_msg);
         }/*endif*/
         res_id[itrans] = ires;
         natm_trans[itrans]   = 3;
         if (res_GLY==1) {
            ind_trans[itrans][1] = ind_C;               /*C  */
            ind_trans[itrans][2] = ind_HA1;             /*HA1*/
            ind_trans[itrans][3] = ind_HA2;             /*HA2*/
         }
         else {
            ind_trans[itrans][1] = ind_C;               /*C */
            ind_trans[itrans][2] = ind_CB;              /*CB*/
            ind_trans[itrans][3] = ind_HA;              /*HA*/
         }/*endif*/
         ind_prim_tors[itrans][4] = ind_C;              /*C */
         ind_prim_tors[itrans][3] = ind_CA;             /*CA*/
         ind_prim_tors[itrans][2] = ind_N;              /*N */
         ind_prim_tors[itrans][1] = ind_prevres_C;      /*C */
      }/*endif*/

       if (iresl==nres) {
         itrans++;
         if (itrans>num_trans) {
            sprintf(err_msg,"Tors_defs::setup_AA_protein_D6:itrans>num_trans");
            errorMessage(err_msg);
         }/*endif*/
         res_id[itrans] = ires;
         natm_trans[itrans]   = 1;
         ind_trans[itrans][1] = ind_O;                    /*O */
         ind_prim_tors[itrans][4] = ind_O;                /*O */
         ind_prim_tors[itrans][3] = ind_C;                /*C */
         ind_prim_tors[itrans][2] = ind_CA;               /*CA*/
         ind_prim_tors[itrans][1] = ind_N;                /*N */
       }/*endif*/
     
       ind_prevres_C  = ind_C;   
       ind_prevres_O  = ind_O;   
       ind_prevres_CA = ind_CA;
       ind_prevres_N  = ind_N;

  }/*endfor-iresl*/

 num_trans_backbone = itrans - *itrans_glob;

#ifdef DEBUG
   printf("Check Point II\n");
   printf("num_trans_backbone %d\n",num_trans_backbone);
   getchar();
#endif

/*-------------------------------------------------------------------------*/
/* II) Define the groups of sidechain atoms on the DNA strands             */

 for (imol=1;imol<=nmol_torcompl[itorcompl];imol++) {
  iimol = ind_mol_torcompl[itorcompl][imol];
  if  (strcasecmp(mol_label[iimol],"protein")!=0) {continue;}
  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[iimol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     if (itrans>num_trans) {
        sprintf(err_msg,"Tors_defs::setup_AA_DNA_duplex:itrans>num_trans");
        errorMessage(err_msg);
     }/*endif*/
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/
 }/*end for-imol*/

 num_trans_sidechain = itrans - *itrans_glob - num_trans_backbone;
 num_trans_mol       = num_trans_backbone + num_trans_sidechain;


#ifdef DEBUG
   printf("Check Point III\n");
   printf("num_trans_backbone %d\n",num_trans_backbone);
   printf("num_trans_sidechain %d\n",num_trans_sidechain);
   getchar();
#endif


/*-------------------------------------------------------------------------*/
/* II) Transformation types                                                */
/*     By default, everybody is transformed                                */

   for (itrans=1;itrans<=num_trans_mol;itrans++) {
        ityp_trans[itrans+(*itrans_glob)] = 1;
        off_trans[itrans+(*itrans_glob)]  = 0;
   }/*end for*/

/*-------------------------------------------------------------------------*/
/* III) If stride is defined in .pdb, only sample backbone                 */
/*       along 'C' secondary structure type residue                        */
/*       sidechain torsions are always transformed                         */ 
/*       Only consider STRIDE in case of mol_label == "DNA"                */

   if (general->stride_defined==1) {
    for (imol=1;imol<=nmol_torcompl[itorcompl];imol++) {
     iimol = ind_mol_torcompl[itorcompl][imol];
     if (strcasecmp(mol_label[iimol],"protein")!=0) {continue;}
     istride = ind_stride_mol[iimol];
     if (istride==0) {
      sprintf(err_msg,"%d. molecule does not have a STRIDE",iimol);
      errorMessage(err_msg);
     }/*endif*/
     if (istride>nstride) {
      sprintf(err_msg,"%d. molecule istride > nstride",iimol);
      errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[istride][0]))-1!=nres_mol[iimol]) {
      sprintf(err_msg,"%d. STRIDE in PDB is corrupted",istride);
      errorMessage(err_msg);
     }/*endif*/
     for (itrans=1;itrans<=num_trans_backbone;itrans++) {
          ityp_trans[itrans+(*itrans_glob)] = 1;
          off_trans[itrans+(*itrans_glob)]  = 1;
          res_id_local = res_id[itrans+(*itrans_glob)]-ind_res_mol[iimol][1]+1;
          if (stride[istride][res_id_local] == 'C') {
              off_trans[itrans+(*itrans_glob)] = 0;
          }/*endif*/
     }/*end for*/
    }/*end for-imol*/
   }/*end if*/

/*-------------------------------------------------------------------------*/
/* IV) Adjust itrans_glob                                                  */  
   *itrans_glob += num_trans_mol; 

/*-------------------------------------------------------------------------*/
}/*end routine*/  
/*=========================================================================*/
