#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
 void Tors_defs::setup_AA_protein_cblc(     const Maps *maps,
                                            const Pos *pos,
                                            const General *general,
                                            int  imol,
                                            int *itrans_glob,
                                            int *iclos_glob,
                                            int *iclbase_glob      )
/*-------------------------------------------------------------------------*/
/*     sets up all the torsional angles for an all atom protein molecule   */
/*     treated with the method of CBLC J. Comp. Chem. 27: 309-315 (2006)   */
/*     molecular index: imol                                               */
/*     transformation offset: itrans_glob                                  */
/*     closure offset: iclos_glob                                          */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int iatm, iiatm, jatm, jjatm;
  int ichigrp, istride, transform;
  int ires, iresl, nres, itrans, iclos, iclbase;
  int num_trans_backbone, num_trans_sidechain, num_trans_mol;
  int num_clos_mol, num_clbase_mol;

  int ind_HN, ind_N, ind_CA, ind_HA, ind_CB, ind_C, ind_O;
  int ind_HA1, ind_HA2;         /*GLY*/
  int ind_CD, ind_HD1, ind_HD2; /*PRO*/ 
  int ind_CG, ind_HG1, ind_HG2; /*PRO*/
  int ind_HB1, ind_HB2;         /*PRO*/

  int ind_nextres_N, ind_nextres_HN, ind_nextres_CA, ind_nextres_C;
  int ind_prevres_C, ind_prevres_O,  ind_prevres_CA, ind_prevres_N;

  int res_PRO, res_GLY;
 
  int    res_id_local;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;

  double dx, dy, dz;
  double *x = pos->x;
  double *y = pos->y;
  double *z = pos->z;

  char *atm_label;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Indicate the type of torsional setup                                 */
/*-------------------------------------------------------------------------*/

  printf("Constant Bond Length Double Closure (CBLC) for protein\n");

/*-------------------------------------------------------------------------*/
/* I) Define the groups of atoms to be transformed for protein             */
/*    >>ityp_trans<< needs to be defined everywhere explicitly!            */
/*-------------------------------------------------------------------------*/
/* I.A) Define the groups of backbone atoms                                */

  itrans  = *itrans_glob;
  iclos   = *iclos_glob;
  iclbase = *iclbase_glob;
  nres   = nres_mol[imol];

  for (iresl=1;iresl<=nres;iresl++) {
       res_PRO = 0;
       res_GLY = 0;
       ires = ind_res_mol[imol][iresl];
#ifdef DEBUG
       printf("from setup_AA_protein_cblc\n");
       printf("ires %d, imol %d\n",ires,imol);
#endif

      if ( strcasecmp(pos->residue[ires].residue_name,"PRO")==0 ){res_PRO=1;} 
      if ( strcasecmp(pos->residue[ires].residue_name,"GLY")==0 ){res_GLY=1;} 

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.1) Find atom indices                                                */

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
#ifdef DEBUG
       printf("from setup_AA_protein_cblc\n");
       printf("iiatm %d\n",iiatm);
       printf("atm_label %s\n",atm_label);
       getchar();
#endif
            if (strcasecmp(atm_label,"HN")==0)  {ind_HN  = iiatm;}
            if (strcasecmp(atm_label,"N")==0)   {ind_N   = iiatm;}   
            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"HA")==0)  {ind_HA  = iiatm;}   
            if (strcasecmp(atm_label,"CB")==0)  {ind_CB  = iiatm;}
            if (strcasecmp(atm_label,"C")==0)   {ind_C   = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"HA1")==0) {ind_HA1 = iiatm;}
            if (strcasecmp(atm_label,"HA2")==0) {ind_HA2 = iiatm;}
            if (strcasecmp(atm_label,"CD")==0)  {ind_CD  = iiatm;}
            if (strcasecmp(atm_label,"HD1")==0) {ind_HD1 = iiatm;}
            if (strcasecmp(atm_label,"HD2")==0) {ind_HD2 = iiatm;}
            if (strcasecmp(atm_label,"CG")==0)  {ind_CG  = iiatm;}
            if (strcasecmp(atm_label,"HG1")==0) {ind_HG1 = iiatm;}
            if (strcasecmp(atm_label,"HG2")==0) {ind_HG2 = iiatm;}
            if (strcasecmp(atm_label,"HB1")==0) {ind_HB1 = iiatm;}
            if (strcasecmp(atm_label,"HB2")==0) {ind_HB2 = iiatm;}
       }/*endfor*/

       if (iresl != nres) {
           ires = ind_res_mol[imol][iresl+1];
           for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
                iiatm     = pos->residue[ires].atom[iatm];
                atm_label = maps->upac_atm_label[iiatm];
                if (strcasecmp(atm_label,"N")==0)  {ind_nextres_N  = iiatm;}
                if (strcasecmp(atm_label,"CA")==0) {ind_nextres_CA = iiatm;}
           }/*endfor*/
           ires = ind_res_mol[imol][iresl];
       }/*endif*/

       if (iresl != 1) {
           ires = ind_res_mol[imol][iresl-1];
           for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
                iiatm     = pos->residue[ires].atom[iatm];
                atm_label = maps->upac_atm_label[iiatm];
                if (strcasecmp(atm_label,"C")==0)  {ind_prevres_C  = iiatm;}
                if (strcasecmp(atm_label,"CA")==0) {ind_prevres_CA = iiatm;}
           }/*endfor*/
           ires = ind_res_mol[imol][iresl];
       }/*endif*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.2) Define clbase                                                    */

       if (iresl != nres) {
           iclbase++;
           if (iclbase>num_clbase) {
    	     sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:iclbase>num_clbase");
             errorMessage(err_msg);
           }/*endif*/
           ind_clbase_mol[imol][iclbase-(*iclbase_glob)] = iclbase;
           natm_clbase[iclbase] = 3;
           ind_atm_clbase[iclbase][1] = ind_O;          /*O*/
           ind_atm_clbase[iclbase][2] = ind_C;          /*C*/
           ind_atm_clbase[iclbase][3] = ind_nextres_N;  /*N*/
       }/*endif*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.3) Define transformations in first residue                          */

      if (iresl == 1) {
         itrans++;
         if (itrans>num_trans) {
 	     sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.02; // 0.05 0.1 
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_CA;   /*CA*/
         ind_prim_tors[itrans][4] = ind_CA;         /*CA*/
         ind_prim_tors[itrans][3] = ind_C;          /*C*/
         ind_prim_tors[itrans][2] = ind_nextres_N;  /*N*/
         ind_prim_tors[itrans][1] = ind_nextres_CA; /*CA*/



         itrans++;
         if (itrans>num_trans) {
 	     sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.02; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 3;
         ind_trans[itrans][1] = ind_N;    /*N*/
         if (res_GLY==1) {
             ind_trans[itrans][2] = ind_HA1;   /*HA1*/
             ind_trans[itrans][3] = ind_HA2;   /*HA2*/
         } else {
             ind_trans[itrans][2] = ind_CB;   /*CB*/
             ind_trans[itrans][3] = ind_HA;   /*HA*/
         }/*endif*/
         ind_prim_tors[itrans][4] = ind_N;         /*N*/
         ind_prim_tors[itrans][3] = ind_CA;        /*CA*/
         ind_prim_tors[itrans][2] = ind_C;         /*C*/
         ind_prim_tors[itrans][1] = ind_nextres_N; /*N*/


         itrans++;
         if (itrans>num_trans) {
 	     sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.02; // 0.05 0.1 
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_HN;    /*HN*/
         ind_prim_tors[itrans][4] = ind_HN;  /*HN*/
         ind_prim_tors[itrans][3] = ind_N;   /*N*/
         ind_prim_tors[itrans][2] = ind_CA;  /*CA*/
         ind_prim_tors[itrans][1] = ind_C;   /*C*/
      }/*endif-iresl==1*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.4) Define transformations in bulk residue                           */

      if ( (iresl > 1) && (iresl < nres) ) {
         itrans++;
         if (itrans>num_trans) {
    	    sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
            errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 2;          /*needs closure*/
         prop_sig_scale[itrans] = 0.0;    /*peptide plane*/
         res_id[itrans] = ires;
         if (res_PRO==1) {
           natm_trans[itrans] = 1;
           ind_trans[itrans][1] = ind_CA;   /*CA*/
         } else {
           natm_trans[itrans] = 2;
           ind_trans[itrans][1] = ind_CA;   /*CA*/
           ind_trans[itrans][2] = ind_HN;   /*HN*/
         }/*endif*/
         ind_prim_tors[itrans][4] = ind_CA;         /*CA*/
         ind_prim_tors[itrans][3] = ind_N;          /*N*/
         ind_prim_tors[itrans][2] = ind_prevres_C;  /*C*/
         ind_prim_tors[itrans][1] = ind_prevres_CA; /*CA*/
         /*____________________/closure/____________________*/
         iclos++;
         if (iclos>num_clos) {
    	    sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:iclos>num_clos");
            errorMessage(err_msg);
         }/*endif*/
         ind_clos_trans[itrans] = iclos;
         ityp_clos[iclos]       = 0;        /*cblc*/
         natm_clos[iclos]       = 5;
         ind_atm_clos[iclos][5] = ind_C;          /*C*/
         ind_atm_clos[iclos][4] = ind_CA;         /*CA*/
         ind_atm_clos[iclos][3] = ind_N;          /*N*/
         ind_atm_clos[iclos][2] = ind_prevres_C;  /*C*/
         ind_atm_clos[iclos][1] = ind_prevres_CA; /*CA*/
         dx = x[ind_C] - x[ind_CA];
         dy = y[ind_C] - y[ind_CA];
         dz = z[ind_C] - z[ind_CA];
         const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
         dx = x[ind_N] - x[ind_CA];
         dy = y[ind_N] - y[ind_CA];
         dz = z[ind_N] - z[ind_CA];
         const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/


         itrans++;/*~~~~~~~~~~~~~~~~~~~secondary_only~~~~~~~~~~~~~~~~~~~*/
         if (itrans>num_trans) {
             sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 0;
         secondary_only[itrans] = itrans;
         res_id[itrans] = ires;
         if (res_GLY==1) {
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_HA1;     /*HA1*/
             ind_trans[itrans][2] = ind_HA2;     /*HA2*/
             ind_trans[itrans][3] = ind_C;       /*C*/

             ind_prim_tors[itrans][4] = ind_HA1; /*HA1*/
         } else if (res_PRO==1) {
             natm_trans[itrans] = 1; 
             ind_trans[itrans][1] = ind_HA;      /*HA*/
             ind_trans[itrans][2] = ind_C;       /*C*/

             ind_prim_tors[itrans][4] = ind_HA; /*HA*/
         } else {
             natm_trans[itrans] = 2;
             ind_trans[itrans][1]  = ind_CB;    /*CB*/
             ind_trans[itrans][2]  = ind_HA;    /*HA*/
             ind_trans[itrans][3]  = ind_C;     /*C*/

             ind_prim_tors[itrans][4] = ind_CB; /*CB*/
         }/*endif*/
         ind_prim_tors[itrans][3] = ind_CA;        /*CA*/
         ind_prim_tors[itrans][2] = ind_N;         /*N*/
         ind_prim_tors[itrans][1] = ind_prevres_C; /*C*/


         if (res_PRO==1) {
            itrans++;
            if (itrans>num_trans) {
        	sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
                errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 1;
            ind_trans[itrans][1] = ind_CD;               /*CD*/
            ind_prim_tors[itrans][4] = ind_CD;           /*CD*/
            ind_prim_tors[itrans][3] = ind_N;            /*N*/
            ind_prim_tors[itrans][2] = ind_prevres_C;    /*C*/
            ind_prim_tors[itrans][1] = ind_prevres_CA;   /*CA*/


            itrans++;
            if (itrans>num_trans) {
        	sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
                errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 1;
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 3;
            ind_trans[itrans][1] = ind_CG;               /*CG*/
            ind_trans[itrans][2] = ind_HD1;              /*HD1*/
            ind_trans[itrans][3] = ind_HD2;              /*HD2*/
            ind_prim_tors[itrans][4] = ind_CG;           /*CG*/
            ind_prim_tors[itrans][3] = ind_CD;           /*CD*/
            ind_prim_tors[itrans][2] = ind_N;            /*N*/
            ind_prim_tors[itrans][1] = ind_prevres_C;    /*C*/


            itrans++;
            if (itrans>num_trans) {
        	sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
                errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 2;          /*needs closure*/
            prop_sig_scale[itrans] = 0.0; // 0.05 0.1 
            res_id[itrans] = ires;
            natm_trans[itrans] = 3;
            ind_trans[itrans][1] = ind_CB;               /*CB*/
            ind_trans[itrans][2] = ind_HG1;              /*HG1*/
            ind_trans[itrans][3] = ind_HG2;              /*HG2*/
            ind_prim_tors[itrans][4] = ind_CB;           /*CB*/
            ind_prim_tors[itrans][3] = ind_CG;           /*CG*/
            ind_prim_tors[itrans][2] = ind_CD;           /*CD*/
            ind_prim_tors[itrans][1] = ind_N;            /*N*/
            /*____________________/closure/____________________*/
            iclos++;
            if (iclos>num_clos) {
       	    sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:iclos>num_clos");
               errorMessage(err_msg);
            }/*endif*/
            ind_clos_trans[itrans] = iclos;
            ityp_clos[iclos]       = 0;        /*cblc*/
            natm_clos[iclos]       = 5;
            ind_atm_clos[iclos][5] = ind_CA;         /*CA*/
            ind_atm_clos[iclos][4] = ind_CB;         /*CB*/
            ind_atm_clos[iclos][3] = ind_CG;         /*CG*/
            ind_atm_clos[iclos][2] = ind_CD;         /*CD*/
            ind_atm_clos[iclos][1] = ind_N;          /*N*/
            dx = x[ind_CA] - x[ind_CB];
            dy = y[ind_CA] - y[ind_CB];
            dz = z[ind_CA] - z[ind_CB];
            const_clos[iclos][1] = sqrt(dx*dx+dy*dy+dz*dz); /*r54*/
            dx = x[ind_CG] - x[ind_CB];
            dy = y[ind_CG] - y[ind_CB];
            dz = z[ind_CG] - z[ind_CB];
            const_clos[iclos][2] = sqrt(dx*dx+dy*dy+dz*dz); /*r34*/


            itrans++;/*~~~~~~~~~~~~~~~~~~~secondary_only~~~~~~~~~~~~~~~~~~~*/
            if (itrans>num_trans) {
                sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
                errorMessage(err_msg);
            }/*endif*/
            ityp_trans[itrans] = 0;
            secondary_only[itrans] = itrans;
            res_id[itrans] = ires;
            natm_trans[itrans] = 2;
            ind_trans[itrans][1] = ind_HB1;     /*HB1*/
            ind_trans[itrans][2] = ind_HB2;     /*HB2*/
            ind_trans[itrans][3] = ind_CA;      /*CA*/
      
            ind_prim_tors[itrans][4] = ind_HB1; /*HB1*/
            ind_prim_tors[itrans][3] = ind_CB;  /*CB*/
            ind_prim_tors[itrans][2] = ind_CG;  /*CG*/
            ind_prim_tors[itrans][1] = ind_CD;  /*CD*/
         }/*endif-res_PRO==1*/


      }/*endif--(iresl > 1 && iresl < nres)*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.5) Define transformations in last residue                           */

      if (iresl == nres) {
         itrans++;
         if (itrans>num_trans) {
    	  sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
            errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0;    /*peptide plane*/
         res_id[itrans] = ires;
         natm_trans[itrans] = 2;
         ind_trans[itrans][1] = ind_CA;   /*CA*/
         ind_trans[itrans][2] = ind_HN;   /*HN*/
         ind_prim_tors[itrans][4] = ind_CA;         /*CA*/
         ind_prim_tors[itrans][3] = ind_N;          /*N*/
         ind_prim_tors[itrans][2] = ind_prevres_C;  /*C*/
         ind_prim_tors[itrans][1] = ind_prevres_CA; /*CA*/


         itrans++;
         if (itrans>num_trans) {
    	  sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
            errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.0; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 3;
         ind_trans[itrans][1] = ind_C;    /*C*/
         if (res_GLY==1) {
             ind_trans[itrans][2] = ind_HA1;   /*HA1*/
             ind_trans[itrans][3] = ind_HA2;   /*HA2*/
         } else {
             ind_trans[itrans][2] = ind_CB;   /*CB*/
             ind_trans[itrans][3] = ind_HA;   /*HA*/
         }/*endif*/
         ind_prim_tors[itrans][4] = ind_C;          /*C*/
         ind_prim_tors[itrans][3] = ind_CA;         /*CA*/
         ind_prim_tors[itrans][2] = ind_N;          /*N*/
         ind_prim_tors[itrans][1] = ind_prevres_C;  /*C*/


         itrans++;
         if (itrans>num_trans) {
    	  sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
            errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         prop_sig_scale[itrans] = 0.02; // 0.05 0.1
         res_id[itrans] = ires;
         natm_trans[itrans] = 1;
         ind_trans[itrans][1] = ind_O;    /*O*/
         ind_prim_tors[itrans][4] = ind_O;  /*O*/
         ind_prim_tors[itrans][3] = ind_C;  /*C*/
         ind_prim_tors[itrans][2] = ind_CA; /*CA*/
         ind_prim_tors[itrans][1] = ind_N;  /*N*/
      }/*endif-iresl==nres*/
  }/*endfor-iresl*/

  num_trans_backbone = itrans - *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I.B) Define the groups of sidechain atoms                               */

  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[imol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
     itrans++;
     if (itrans>num_trans) {
        sprintf(err_msg,"Tors_defs::setup_AA_protein_cblc:itrans>num_trans");
        errorMessage(err_msg);
     }/*endif*/
     ityp_trans[itrans] = 1;
     prop_sig_scale[itrans] = 0.02; // 0.05 0.1
     res_id[itrans] = ires;
     natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
     for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
      ind_trans[itrans][iatm] = pos->residue[ires].chigrp_atom[ichigrp][iatm];
     }/*end for*/
     for (iatm=1;iatm<=4;iatm++) {
      ind_prim_tors[itrans][iatm]=pos->residue[ires].chiprim_atom[ichigrp][iatm];
     }/*end for*/
    }/*end for*/
  }/*end for*/
  num_trans_sidechain = itrans - *itrans_glob - num_trans_backbone;

  num_trans_mol       = num_trans_backbone + num_trans_sidechain;
  num_clos_mol        =  iclos   - *iclos_glob;
  num_clbase_mol      =  iclbase - *iclbase_glob;

/*-------------------------------------------------------------------------*/
/* II)  By default no transformation is turned off                         */

   for (itrans=1;itrans<=num_trans_mol;itrans++) {
        off_trans[itrans+(*itrans_glob)] = 0;
   }/*endfor*/

/*-------------------------------------------------------------------------*/
/* III)  If stride is defined in .pdb, only sample backbone                */
/*       along 'C' secondary structure type residue                        */
/*       sidechain torsions are always transformed                         */

   if (general->stride_defined==1) {
     istride = ind_stride_mol[imol];
     if (istride==0) {
      sprintf(err_msg,"%d. molecule does not have a STRIDE",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (istride>nstride) {
      sprintf(err_msg,"%d. molecule istride > nstride",imol);
      errorMessage(err_msg);
     }/*endif*/
     if (strlen(&(stride[istride][0]))-1!=nres_mol[imol]) {
      sprintf(err_msg,"%d. STRIDE in PDB is corrupted",istride);
      errorMessage(err_msg);
     }/*endif*/
     for (itrans=1;itrans<=num_trans_backbone;itrans++) {
          off_trans[itrans+(*itrans_glob)]  = 1;
          res_id_local = res_id[itrans+(*itrans_glob)]-ind_res_mol[imol][1]+1;
          if (stride[istride][res_id_local] == 'C') {
              off_trans[itrans+(*itrans_glob)] = 0;
          }/*endif*/
     }/*end for*/
   }/*end if*/

/*-------------------------------------------------------------------------*/
/* IV) Adjust itrans_glob, iclos_glob, iclbase_glob                        */  
   *itrans_glob  += num_trans_mol;
   *iclos_glob   += num_clos_mol;
   *iclbase_glob += num_clbase_mol;

/*-------------------------------------------------------------------------*/
}/*end routine*/
/*=========================================================================*/
