#include "../../include/standard.h" 
#include "../../include/typedefs.h"
#include "../../include/definitions.h"
#include "../../include/utillib.h"
#include "../../include/cmalloc.h"

/*-------------------------------------------------------------------------*/
/*                          stage_closed_transform                         */
/* type ==  1 : position forward                                           */
/* type == -1 : position backward                                          */
/* type ==  2 : position + force forward                                   */
/* type == -2 : position + force backward                                  */
/*-------------------------------------------------------------------------*/
/*ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
/*-------------------------------------------------------------------------*/

void stage_closed_transform(Pos *pos,Stage_defs *stage_defs,int i_trans,
                                                            int type)

/*-------------------------------------------------------------------------*/
   {/*begin*/ 
/*-------------------------------------------------------------------------*/

  int iatm, jatm, natm;
  int jatm1, jatmp1, jatmm1; 
  double rat, rat1, rat2;

  double *x  = pos->uxi; 
  double *y  = pos->uyi; 
  double *z  = pos->uzi; 
  double *fx = pos->fuxi;
  double *fy = pos->fuyi;
  double *fz = pos->fuzi;
 
  double *ux  = pos->ux; 
  double *uy  = pos->uy; 
  double *uz  = pos->uz; 
  double *fux = pos->fux;
  double *fuy = pos->fuy;
  double *fuz = pos->fuz;

  int **ind_group     = stage_defs->ind_trans;
  int *natm_group     = stage_defs->natm_trans;


/*-------------------------------------------------------------------------*/
/* I) Initialize local starting local x position                           */

  natm = pos->natom;

  for (iatm=1;iatm<=natm;iatm++) {
     x[iatm] = ux[iatm]; 
     y[iatm] = uy[iatm]; 
     z[iatm] = uz[iatm]; 
     if (type == 2) {
         fx[iatm] = fux[iatm];  
         fy[iatm] = fuy[iatm]; 
         fz[iatm] = fuz[iatm]; 
     }/*endif*/
  }/*endfor*/      

/*-------------------------------------------------------------------------*/
/* II) transformation                                                      */

   switch(type){
     
     case -2 :

        break;
     case -1 : 
        jatm = ind_group[i_trans][1];
        x[jatm] = ux[jatm];
        y[jatm] = uy[jatm];
        z[jatm] = uz[jatm];
        jatm1 = ind_group[i_trans][1];
        for (iatm=natm_group[i_trans];iatm>=2;iatm--) {/*group atoms*/ 
	    if (iatm==natm_group[i_trans]) {
                jatmp1 = ind_group[i_trans][1];
            } else {
                jatmp1 = ind_group[i_trans][iatm+1]; 
            }/*endif*/
            jatm   = ind_group[i_trans][iatm];
            rat1 = ((double)(iatm-1));
            rat2 = ((double)(iatm));
            x[jatm] = ux[jatm] + (rat1*x[jatmp1]+x[jatm1])/rat2;
            y[jatm] = uy[jatm] + (rat1*y[jatmp1]+y[jatm1])/rat2;
            z[jatm] = uz[jatm] + (rat1*z[jatmp1]+z[jatm1])/rat2;
        }/*group atoms*/
        break;
     case  1 : 
        jatm = ind_group[i_trans][1];
        ux[jatm] = x[jatm];
        uy[jatm] = y[jatm];
        uz[jatm] = z[jatm];
        jatm1 = ind_group[i_trans][1];
        for (iatm=2;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
	    if (iatm==natm_group[i_trans]) {
                jatmp1 = ind_group[i_trans][1];
            } else {
                jatmp1 = ind_group[i_trans][iatm+1]; 
            }/*endif*/
            jatm   = ind_group[i_trans][iatm];
            rat1 = ((double)(iatm-1));
            rat2 = ((double)(iatm));
            ux[jatm] = x[jatm] - (rat1*x[jatmp1]+x[jatm1])/rat2;
            uy[jatm] = y[jatm] - (rat1*y[jatmp1]+y[jatm1])/rat2;
            uz[jatm] = z[jatm] - (rat1*z[jatmp1]+z[jatm1])/rat2;
//            printf("cyclic x to u %d : %d %d %g\n",jatm,jatm1,jatmp1,ux[jatm]);
        }/*group atoms*/
        break;
     case  2 : 
        jatm = ind_group[i_trans][1];
        ux[jatm] = x[jatm];
        uy[jatm] = y[jatm];
        uz[jatm] = z[jatm];
        jatm1 = ind_group[i_trans][1];
        for (iatm=2;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
	    if (iatm==natm_group[i_trans]) {
                jatmp1 = ind_group[i_trans][1];
            } else {
                jatmp1 = ind_group[i_trans][iatm+1]; 
            }/*endif*/
            jatm   = ind_group[i_trans][iatm];
            rat1 = ((double)(iatm-1));
            rat2 = ((double)(iatm));
            ux[jatm] = x[jatm] - (rat1*x[jatmp1]+x[jatm1])/rat2;
            uy[jatm] = y[jatm] - (rat1*y[jatmp1]+y[jatm1])/rat2;
            uz[jatm] = z[jatm] - (rat1*z[jatmp1]+z[jatm1])/rat2;
        }/*group atoms*/


        jatm1 = ind_group[i_trans][1];
//        printf("cyclic force %d \n",jatm1);
        fux[jatm1] = 0.0; 
        fuy[jatm1] = 0.0;
        fuz[jatm1] = 0.0;
        for (iatm=1;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/ 
            jatm   = ind_group[i_trans][iatm];
//            printf("adding cyclic force of %d to %d : %g\n",jatm,jatm1,fx[jatm]);
            fux[jatm1] += fx[jatm];
            fuy[jatm1] += fy[jatm];
            fuz[jatm1] += fz[jatm];
        }/*group atoms*/
        for (iatm=2;iatm<=natm_group[i_trans];iatm++) {/*group atoms*/
            jatm   = ind_group[i_trans][iatm];
            jatmm1 = ind_group[i_trans][iatm-1]; 
//            printf("adding cyclic force of %d to %d : %g %g\n",jatmm1,jatm,
//            fux[jatmm1],fx[jatm]);
            rat = ((double)(iatm-2))/((double)(iatm-1));
            fux[jatm] = rat*fux[jatmm1] + fx[jatm];
            fuy[jatm] = rat*fuy[jatmm1] + fy[jatm];
            fuz[jatm] = rat*fuz[jatmm1] + fz[jatm];
        }/*group atoms*/
        break;
   }/* switch*/ 


/*-------------------------------------------------------------------------*/
/* III) redefine u, if u->x was  executed, type -1                         */

   if (type == -1) { 
      for (iatm=1;iatm<=natm;iatm++) {
           ux[iatm] = x[iatm]; 
           uy[iatm] = y[iatm]; 
           uz[iatm] = z[iatm]; 
      }/*endfor*/
   }/*endif*/      

/*-------------------------------------------------------------------------*/
  }/*end of translate*/
/*-------------------------------------------------------------------------*/
