#include "../../../include/standard.h" 
#include "../../../include/typedefs.h"
#include "../../../include/definitions.h"
#include "../../../include/utillib.h"
#include "../../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Stage_defs::setup_AA_DNA(   const Maps *maps,
                                 const Pos *pos,
                                 const General *general,
                                 int  imol,
                                 int *itrans_glob  )
/*-------------------------------------------------------------------------*/
/* sets up all the stage transformations for an all atom DNA strand        */
/* molecular index: imol                                                   */
/* transformation offset: itrans_glob                                      */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int iatm, iiatm, jatm;
  int ires, iresl, nres, itrans;

  int res_ADE, res_THY, res_CYT, res_GUA;

  int ind_O2, ind_O4, ind_O6;
  int ind_H61, ind_H62, ind_H21, ind_H22;
  int ind_H42, ind_H41, ind_H51, ind_H52, ind_H53;
  int ind_O1P, ind_O2P, ind_H5p, ind_H5pp, ind_H4p;
  int ind_H1, ind_H2, ind_H3, ind_H5, ind_H6, ind_H8;
  int ind_P, ind_O5p, ind_C5p, ind_C4p, ind_C3p, ind_O3p;
  int ind_H1p, ind_H2p, ind_H2pp, ind_H3p, ind_O4p, ind_C2p;
  int ind_N1, ind_N2, ind_N3, ind_N4, ind_N6, ind_N7, ind_N9;
  int ind_C1p, ind_C2, ind_C4, ind_C5, ind_C5M, ind_C6, ind_C8;

  int    res_id_local;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;

  char *atm_label;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Indicate the type of staging setup                                   */
/*-------------------------------------------------------------------------*/

  printf("Staging setup for all atom DNA strand\n");

  itrans  = *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I) Define the groups of atoms to be transformed for DNA                 */
/*    >>ityp_trans<< needs to be defined everywhere explicitly!            */
/*-------------------------------------------------------------------------*/
/* I.A) Define the group(s) of main chain atoms only                       */

  itrans++;
  if (itrans>num_trans) {
   sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
   errorMessage(err_msg);
  }/*endif*/
  ityp_trans[itrans] = 2;
  res_id[itrans] = 0;

  nres = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
       ires = ind_res_mol[imol][iresl];

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.0) Find atom indices                                                */

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"P")==0)   {ind_P   = iiatm;}   
            if (strcasecmp(atm_label,"O5'")==0) {ind_O5p = iiatm;}   
            if (strcasecmp(atm_label,"C5'")==0) {ind_C5p = iiatm;}
            if (strcasecmp(atm_label,"C4'")==0) {ind_C4p = iiatm;}   
            if (strcasecmp(atm_label,"C3'")==0) {ind_C3p = iiatm;}
            if (strcasecmp(atm_label,"O3'")==0) {ind_O3p = iiatm;}
       }/*endfor*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.1) Define atoms in group(s)                                         */

        natm_trans[itrans]++;
        ind_trans[itrans][natm_trans[itrans]] = ind_P;           /*P*/

        natm_trans[itrans]++;
        ind_trans[itrans][natm_trans[itrans]] = ind_O5p;         /*O5'*/

        natm_trans[itrans]++;
        ind_trans[itrans][natm_trans[itrans]] = ind_C5p;         /*C5'*/

        natm_trans[itrans]++;
        ind_trans[itrans][natm_trans[itrans]] = ind_C4p;         /*C4'*/

        /* skip C3' due to closed transformation on the ring */

        natm_trans[itrans]++;
        ind_trans[itrans][natm_trans[itrans]] = ind_O3p;         /*O3'*/

  }/*endfor-iresl*/


/*-------------------------------------------------------------------------*/
/* I.B) Define the groups of sidechain atoms                               */

  nres = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[imol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.B.0) Only Exceptional residues                                        */

    res_ADE = 0;
    res_THY = 0;
    res_CYT = 0;
    res_GUA = 0;
    if      (strcasecmp(pos->residue[ires].residue_name,"ADE")==0){res_ADE=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"THY")==0){res_THY=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"CYT")==0){res_CYT=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"GUA")==0){res_GUA=1;}

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];

            if (strcasecmp(atm_label,"H61")==0) {ind_H61 = iiatm;}   
            if (strcasecmp(atm_label,"H62")==0) {ind_H62 = iiatm;}   
            if (strcasecmp(atm_label,"H21")==0) {ind_H21 = iiatm;}
            if (strcasecmp(atm_label,"H22")==0) {ind_H22 = iiatm;}   
            if (strcasecmp(atm_label,"H42")==0) {ind_H42 = iiatm;}
            if (strcasecmp(atm_label,"H41")==0) {ind_H41 = iiatm;}
            if (strcasecmp(atm_label,"H51")==0) {ind_H51 = iiatm;}
            if (strcasecmp(atm_label,"H52")==0) {ind_H52 = iiatm;}
            if (strcasecmp(atm_label,"H53")==0) {ind_H53 = iiatm;}

            if (strcasecmp(atm_label,"N1")==0)  {ind_N1  = iiatm;}
            if (strcasecmp(atm_label,"N2")==0)  {ind_N2  = iiatm;}
            if (strcasecmp(atm_label,"N3")==0)  {ind_N3  = iiatm;}
            if (strcasecmp(atm_label,"N4")==0)  {ind_N4  = iiatm;}
            if (strcasecmp(atm_label,"N6")==0)  {ind_N6  = iiatm;}
            if (strcasecmp(atm_label,"N7")==0)  {ind_N7 = iiatm;}
            if (strcasecmp(atm_label,"N9")==0)  {ind_N9 = iiatm;}

            if (strcasecmp(atm_label,"C1'")==0) {ind_C1p= iiatm;}
            if (strcasecmp(atm_label,"C2")==0)  {ind_C2 = iiatm;}
            if (strcasecmp(atm_label,"C4")==0)  {ind_C4 = iiatm;}
            if (strcasecmp(atm_label,"C5")==0)  {ind_C5 = iiatm;}
            if (strcasecmp(atm_label,"C5M")==0) {ind_C5M= iiatm;}
            if (strcasecmp(atm_label,"C6")==0)  {ind_C6 = iiatm;}
            if (strcasecmp(atm_label,"C8")==0)  {ind_C8 = iiatm;}

            if (strcasecmp(atm_label,"O2")==0)  {ind_O2 = iiatm;}
            if (strcasecmp(atm_label,"O4")==0)  {ind_O4 = iiatm;}
            if (strcasecmp(atm_label,"O6")==0)  {ind_O6 = iiatm;}

            if (strcasecmp(atm_label,"H1")==0)  {ind_H1 = iiatm;}
            if (strcasecmp(atm_label,"H2")==0)  {ind_H2 = iiatm;}
            if (strcasecmp(atm_label,"H3")==0)  {ind_H3 = iiatm;}
            if (strcasecmp(atm_label,"H5")==0)  {ind_H5 = iiatm;}
            if (strcasecmp(atm_label,"H6")==0)  {ind_H6 = iiatm;}
            if (strcasecmp(atm_label,"H8")==0)  {ind_H8 = iiatm;}
       }/*endfor*/

       if (res_ADE == 1) {/*ADE*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C1p;   /*C1'*/
             ind_trans[itrans][2] = ind_N9;    /*N9*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 9;
             ind_trans[itrans][1] = ind_N9;    /*N9*/
             ind_trans[itrans][2] = ind_C8;    /*C8*/
             ind_trans[itrans][3] = ind_N7;    /*N7*/
             ind_trans[itrans][4] = ind_C5;    /*C5*/
             ind_trans[itrans][5] = ind_C6;    /*C6*/
             ind_trans[itrans][6] = ind_N1;    /*N1*/
             ind_trans[itrans][7] = ind_C2;    /*C2*/
             ind_trans[itrans][8] = ind_N3;    /*N3*/
             ind_trans[itrans][9] = ind_C4;    /*C4*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C8;   /*C8*/
             ind_trans[itrans][2] = ind_H8;   /*H8*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C2;   /*C2*/
             ind_trans[itrans][2] = ind_H2;   /*H2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C6;   /*C6*/
             ind_trans[itrans][2] = ind_N6;   /*N6*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_N6;    /*N6*/
             ind_trans[itrans][2] = ind_H61;   /*H61*/
             ind_trans[itrans][3] = ind_H62;   /*H62*/
       }/*ADE*/

       if (res_THY == 1) {/*THY*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C1p;   /*C1p*/
             ind_trans[itrans][2] = ind_N1;    /*N1*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 6;
             ind_trans[itrans][1] = ind_N1;    /*N1*/
             ind_trans[itrans][2] = ind_C2;    /*C2*/
             ind_trans[itrans][3] = ind_N3;    /*N3*/
             ind_trans[itrans][4] = ind_C4;    /*C4*/
             ind_trans[itrans][5] = ind_C5;    /*C5*/
             ind_trans[itrans][6] = ind_C6;    /*C6*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C2;   /*C2*/
             ind_trans[itrans][2] = ind_O2;   /*O2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_N3;   /*N3*/
             ind_trans[itrans][2] = ind_H3;   /*H3*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C4;   /*C4*/
             ind_trans[itrans][2] = ind_O4;   /*O4*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C5;    /*C5*/
             ind_trans[itrans][2] = ind_C5M;   /*C5M*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C6;    /*C6*/
             ind_trans[itrans][2] = ind_H6;    /*H6*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_C5M;   /*C5M*/
             ind_trans[itrans][2] = ind_H51;   /*H51*/
             ind_trans[itrans][3] = ind_H52;   /*H52*/
             ind_trans[itrans][4] = ind_H53;   /*H53*/
       }/*THY*/


       if (res_CYT == 1) {/*CYT*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C1p;   /*C1p*/
             ind_trans[itrans][2] = ind_N1;    /*N1*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 6;
             ind_trans[itrans][1] = ind_N1;    /*N1*/
             ind_trans[itrans][2] = ind_C2;    /*C2*/
             ind_trans[itrans][3] = ind_N3;    /*N3*/
             ind_trans[itrans][4] = ind_C4;    /*C4*/
             ind_trans[itrans][5] = ind_C5;    /*C5*/
             ind_trans[itrans][6] = ind_C6;    /*C6*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C2;   /*C2*/
             ind_trans[itrans][2] = ind_O2;   /*O2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C5;   /*C5*/
             ind_trans[itrans][2] = ind_H5;   /*H5*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C6;   /*C6*/
             ind_trans[itrans][2] = ind_H6;   /*H6*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C4;    /*C4*/
             ind_trans[itrans][2] = ind_N4;    /*N4*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_N4;    /*N4*/
             ind_trans[itrans][2] = ind_H42;   /*H42*/
             ind_trans[itrans][3] = ind_H41;   /*H41*/
       }/*CYT*/

       if (res_GUA == 1) {/*GUA*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C1p;   /*C1'*/
             ind_trans[itrans][2] = ind_N9;    /*N9*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 9;
             ind_trans[itrans][1] = ind_N9;    /*N9*/
             ind_trans[itrans][2] = ind_C8;    /*C8*/
             ind_trans[itrans][3] = ind_N7;    /*N7*/
             ind_trans[itrans][4] = ind_C5;    /*C5*/
             ind_trans[itrans][5] = ind_C6;    /*C6*/
             ind_trans[itrans][6] = ind_N1;    /*N1*/
             ind_trans[itrans][7] = ind_C2;    /*C2*/
             ind_trans[itrans][8] = ind_N3;    /*N3*/
             ind_trans[itrans][9] = ind_C4;    /*C4*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C8;   /*C8*/
             ind_trans[itrans][2] = ind_H8;   /*H8*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C6;   /*C6*/
             ind_trans[itrans][2] = ind_O6;   /*O6*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_N1;   /*N1*/
             ind_trans[itrans][2] = ind_H1;   /*H1*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C2;   /*C2*/
             ind_trans[itrans][2] = ind_N2;   /*N2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_N2;    /*N2*/
             ind_trans[itrans][2] = ind_H21;   /*H21*/
             ind_trans[itrans][3] = ind_H22;   /*H22*/
       }/*GUA*/

  }/*end for-iresl*/
  
/*-------------------------------------------------------------------------*/
/* I.C) Define the groups out of all atoms                                 */

  nres    = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
       ires = ind_res_mol[imol][iresl];
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.C.0) Find atom indices                                                */

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"P")==0)   {ind_P   = iiatm;}   
            if (strcasecmp(atm_label,"O1P")==0) {ind_O1P = iiatm;}   
            if (strcasecmp(atm_label,"O2P")==0) {ind_O2P = iiatm;}
            if (strcasecmp(atm_label,"C5'")==0) {ind_C5p = iiatm;}   
            if (strcasecmp(atm_label,"H5'")==0) {ind_H5p = iiatm;}
            if (strcasecmp(atm_label,"H5''")==0){ind_H5pp= iiatm;}
            if (strcasecmp(atm_label,"C4'")==0) {ind_C4p = iiatm;}
            if (strcasecmp(atm_label,"H4'")==0) {ind_H4p = iiatm;}
            if (strcasecmp(atm_label,"C3'")==0) {ind_C3p = iiatm;}
            if (strcasecmp(atm_label,"H3'")==0) {ind_H3p = iiatm;}
            if (strcasecmp(atm_label,"C2'")==0) {ind_C2p = iiatm;}
            if (strcasecmp(atm_label,"H2'")==0) {ind_H2p = iiatm;}
            if (strcasecmp(atm_label,"H2''")==0){ind_H2pp= iiatm;}
            if (strcasecmp(atm_label,"C1'")==0) {ind_C1p = iiatm;}
            if (strcasecmp(atm_label,"H1'")==0) {ind_H1p = iiatm;}
            if (strcasecmp(atm_label,"O4'")==0) {ind_O4p = iiatm;}
       }/*endfor*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.C.1) Define transformations in residue                                */
/*        participating atoms are either part of the backbone or           */
/*        directly connected to the backbone excluding any atoms           */ 
/*        where the main chain start branching                             */

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 5;
             ind_trans[itrans][1] = ind_C4p;    /*C4p*/
             ind_trans[itrans][2] = ind_C3p;    /*C3p*/
             ind_trans[itrans][3] = ind_C2p;    /*C2p*/
             ind_trans[itrans][4] = ind_C1p;    /*C1p*/
             ind_trans[itrans][5] = ind_O4p;    /*O4p*/
  
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_P;      /*P*/
             ind_trans[itrans][2] = ind_O1P;    /*O1P*/
             ind_trans[itrans][3] = ind_O2P;    /*O2P*/
     
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_C5p;     /*C5'*/
             ind_trans[itrans][2] = ind_H5p;     /*H5'*/
             ind_trans[itrans][3] = ind_H5pp;    /*H5''*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C4p;    /*C4p*/
             ind_trans[itrans][2] = ind_H4p;    /*H4p*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C1p;    /*C1p*/
             ind_trans[itrans][2] = ind_H1p;    /*H1p*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_C3p;    /*C3p*/
             ind_trans[itrans][2] = ind_H3p;    /*H3p*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_DNA:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_C2p;    /*C2'*/
             ind_trans[itrans][2] = ind_H2p;    /*H2'*/
             ind_trans[itrans][3] = ind_H2pp;   /*H2''*/

  }/*endfor-iresl*/

/*-------------------------------------------------------------------------*/
/* I.D) Adjust itrans_glob                                                 */  
   *itrans_glob  =  itrans;

/*-------------------------------------------------------------------------*/
}/*end routine*/
/*=========================================================================*/
