#include "../../../include/standard.h" 
#include "../../../include/typedefs.h"
#include "../../../include/definitions.h"
#include "../../../include/utillib.h"
#include "../../../include/cmalloc.h"

#define DEBUG
#undef DEBUG

/*=========================================================================*/
void Stage_defs::setup_AA_protein(    const Maps *maps,
                                      const Pos *pos,
                                      const General *general,
                                      int  imol,
                                      int *itrans_glob      )
/*-------------------------------------------------------------------------*/
/* sets up all the stage transformations for an all atom protein molecule  */
/* molecular index: imol                                                   */
/* transformation offset: itrans_glob                                      */
/*-------------------------------------------------------------------------*/
{/*begin routine*/
/*-------------------------------------------------------------------------*/

  int iatm, iiatm, jatm;
  int ichigrp;
  int ires, iresl, nres, itrans;

  int ind_HN, ind_N, ind_CA, ind_HA, ind_CB, ind_C, ind_O;
  int ind_HA1, ind_HA2;         /*GLY*/
  int ind_CD, ind_HD1, ind_HD2; /*PRO*/ 
  int ind_CG, ind_HG1, ind_HG2; /*PRO*/
  int ind_HB1, ind_HB2;         /*PRO*/


  int ind_HB, ind_HG;
  int ind_HD3, ind_HE21, ind_HE22;
  int ind_HD11, ind_HD12, ind_HD13;
  int ind_HD21, ind_HD22, ind_HD23;
  int ind_HG11, ind_HG12, ind_HG13;
  int ind_HG21, ind_HG22, ind_HG23;
  int ind_CD1, ind_CD2, ind_CG1, ind_CG2;
  int ind_OD1, ind_OG1, ind_OE1, ind_NE2, ind_ND2;

  int res_REG;
  int res_PRO, res_GLY;
  int res_LEU, res_ILE, res_THR, res_VAL, res_GLN, res_ASN;

  int    res_id_local;
  int   *nres_mol    = maps->nres_mol;
  int  **ind_res_mol = maps->ind_res_mol;

  char *atm_label;

  char   err_msg[LINE_NCHARMAX];

/*-------------------------------------------------------------------------*/
/* 0) Indicate the type of staging setup                                   */
/*-------------------------------------------------------------------------*/

  printf("Staging setup for all atom proteins\n");

  itrans  = *itrans_glob;

/*-------------------------------------------------------------------------*/
/* I) Define the groups of atoms to be transformed for protein             */
/*    >>ityp_trans<< needs to be defined everywhere explicitly!            */
/*-------------------------------------------------------------------------*/
/* I.A) Define the group(s) of main chain atoms only                       */

  itrans++;
  if (itrans>num_trans) {
   sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
   errorMessage(err_msg);
  }/*endif*/
  ityp_trans[itrans] = 2;
  res_id[itrans] = 0;

  nres = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
       ires = ind_res_mol[imol][iresl];
       if (strcasecmp(pos->residue[ires].residue_name,"PRO")==0)
	    {res_PRO = 1;}
       else {res_PRO = 0;}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.0) Find atom indices                                                */

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"HN")==0)  {ind_HN  = iiatm;}   
            if (strcasecmp(atm_label,"N")==0)   {ind_N   = iiatm;}   
            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"HA")==0)  {ind_HA  = iiatm;}   
            if (strcasecmp(atm_label,"CB")==0)  {ind_CB  = iiatm;}
            if (strcasecmp(atm_label,"C")==0)   {ind_C   = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"HA1")==0) {ind_HA1 = iiatm;}
            if (strcasecmp(atm_label,"HA2")==0) {ind_HA2 = iiatm;}
            if (strcasecmp(atm_label,"CD")==0)  {ind_CD  = iiatm;}
            if (strcasecmp(atm_label,"HD1")==0) {ind_HD1 = iiatm;}
            if (strcasecmp(atm_label,"HD2")==0) {ind_HD2 = iiatm;}
            if (strcasecmp(atm_label,"CG")==0)  {ind_CG  = iiatm;}
            if (strcasecmp(atm_label,"HG1")==0) {ind_HG1 = iiatm;}
            if (strcasecmp(atm_label,"HG2")==0) {ind_HG2 = iiatm;}
            if (strcasecmp(atm_label,"HB1")==0) {ind_HB1 = iiatm;}
            if (strcasecmp(atm_label,"HB2")==0) {ind_HB2 = iiatm;}
       }/*endfor*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.A.1) Define atomic groups                                             */

        natm_trans[itrans]++;
        ind_trans[itrans][natm_trans[itrans]] = ind_N;           /*N*/
        if (res_PRO != 1) {
            natm_trans[itrans]++;
            ind_trans[itrans][natm_trans[itrans]] = ind_CA;    /*CA*/
        }/*endif*/
        natm_trans[itrans]++;
        ind_trans[itrans][natm_trans[itrans]] = ind_C;         /*C*/

  }/*endfor-iresl*/


/*-------------------------------------------------------------------------*/
/* I.B) Define the groups of sidechain atoms                               */

  nres = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
    ires = ind_res_mol[imol][iresl];
    if ((pos->residue[ires].nchigrp)==0) {continue;}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.B.0) Exceptional residues                                             */

    res_LEU = 0;
    res_ILE = 0;
    res_THR = 0;
    res_VAL = 0;
    res_GLN = 0;
    res_ASN = 0;
    res_REG = 0;
    if      (strcasecmp(pos->residue[ires].residue_name,"LEU")==0){res_LEU=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"ILE")==0){res_ILE=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"THR")==0){res_THR=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"VAL")==0){res_VAL=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"GLN")==0){res_GLN=1;}
    else if (strcasecmp(pos->residue[ires].residue_name,"ASN")==0){res_ASN=1;}
    else                                                          {res_REG=1;}

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"HN")==0)  {ind_HN  = iiatm;}   
            if (strcasecmp(atm_label,"N")==0)   {ind_N   = iiatm;}   
            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"HA")==0)  {ind_HA  = iiatm;}   
            if (strcasecmp(atm_label,"CB")==0)  {ind_CB  = iiatm;}
            if (strcasecmp(atm_label,"C")==0)   {ind_C   = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"HA1")==0) {ind_HA1 = iiatm;}
            if (strcasecmp(atm_label,"HA2")==0) {ind_HA2 = iiatm;}
            if (strcasecmp(atm_label,"CD")==0)  {ind_CD  = iiatm;}
            if (strcasecmp(atm_label,"HD1")==0) {ind_HD1 = iiatm;}
            if (strcasecmp(atm_label,"HD2")==0) {ind_HD2 = iiatm;}
            if (strcasecmp(atm_label,"CG")==0)  {ind_CG  = iiatm;}
            if (strcasecmp(atm_label,"HG1")==0) {ind_HG1 = iiatm;}
            if (strcasecmp(atm_label,"HG2")==0) {ind_HG2 = iiatm;}
            if (strcasecmp(atm_label,"HB1")==0) {ind_HB1 = iiatm;}
            if (strcasecmp(atm_label,"HB2")==0) {ind_HB2 = iiatm;}

            if (strcasecmp(atm_label,"HB")==0) {ind_HB = iiatm;}
            if (strcasecmp(atm_label,"HG")==0) {ind_HG = iiatm;}

            if (strcasecmp(atm_label,"HD3")==0)  {ind_HD3 = iiatm;}
            if (strcasecmp(atm_label,"HE21")==0) {ind_HE21 = iiatm;}
            if (strcasecmp(atm_label,"HE22")==0) {ind_HE22 = iiatm;}

            if (strcasecmp(atm_label,"HD11")==0) {ind_HD11 = iiatm;}
            if (strcasecmp(atm_label,"HD12")==0) {ind_HD12 = iiatm;}
            if (strcasecmp(atm_label,"HD13")==0) {ind_HD13 = iiatm;}

            if (strcasecmp(atm_label,"HD21")==0) {ind_HD21 = iiatm;}
            if (strcasecmp(atm_label,"HD22")==0) {ind_HD22 = iiatm;}
            if (strcasecmp(atm_label,"HD23")==0) {ind_HD23 = iiatm;}

            if (strcasecmp(atm_label,"HG11")==0) {ind_HG11 = iiatm;}
            if (strcasecmp(atm_label,"HG12")==0) {ind_HG12 = iiatm;}
            if (strcasecmp(atm_label,"HG13")==0) {ind_HG13 = iiatm;}

            if (strcasecmp(atm_label,"HG21")==0) {ind_HG21 = iiatm;}
            if (strcasecmp(atm_label,"HG22")==0) {ind_HG22 = iiatm;}
            if (strcasecmp(atm_label,"HG23")==0) {ind_HG23 = iiatm;}

            if (strcasecmp(atm_label,"CD1")==0) {ind_CD1 = iiatm;}
            if (strcasecmp(atm_label,"CD2")==0) {ind_CD2 = iiatm;}
            if (strcasecmp(atm_label,"CG1")==0) {ind_CG1 = iiatm;}
            if (strcasecmp(atm_label,"CG2")==0) {ind_CG2 = iiatm;}

            if (strcasecmp(atm_label,"OD1")==0) {ind_OD1 = iiatm;}
            if (strcasecmp(atm_label,"OG1")==0) {ind_OG1 = iiatm;}
            if (strcasecmp(atm_label,"OE1")==0) {ind_OE1 = iiatm;}
            if (strcasecmp(atm_label,"NE2")==0) {ind_NE2 = iiatm;}
            if (strcasecmp(atm_label,"ND2")==0) {ind_ND2 = iiatm;}           
       }/*endfor*/

       if (res_LEU == 1) {/*LEU*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 2;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_CB;    /*CB*/
             ind_trans[itrans][3] = ind_CG;    /*CG*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CB;    /*CB*/
             ind_trans[itrans][2] = ind_HB1;   /*HB1*/
             ind_trans[itrans][3] = ind_HB2;   /*HB2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CG;    /*CG*/
             ind_trans[itrans][2] = ind_CD1;   /*CD1*/
             ind_trans[itrans][3] = ind_CD2;   /*CD2*/
             ind_trans[itrans][4] = ind_HG;    /*HG*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CD1;   /*CD1*/
             ind_trans[itrans][2] = ind_HD11;  /*HD11*/
             ind_trans[itrans][3] = ind_HD12;  /*HD12*/
             ind_trans[itrans][4] = ind_HD13;  /*HD13*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CD2;   /*CD2*/
             ind_trans[itrans][2] = ind_HD21;  /*HD21*/
             ind_trans[itrans][3] = ind_HD22;  /*HD22*/
             ind_trans[itrans][4] = ind_HD23;  /*HD23*/
       }/*LEU*/

       if (res_ILE == 1) {/*ILE*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_CB;    /*CB*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CB;    /*CB*/
             ind_trans[itrans][2] = ind_CG1;   /*CG1*/
             ind_trans[itrans][3] = ind_CG2;   /*CG2*/
             ind_trans[itrans][4] = ind_HB;    /*HB*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CG1;   /*CG1*/
             ind_trans[itrans][2] = ind_HG11;  /*HG11*/
             ind_trans[itrans][3] = ind_HG12;  /*HG12*/
             ind_trans[itrans][4] = ind_CD;    /*CD*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CD;    /*CD*/
             ind_trans[itrans][2] = ind_HD1;   /*HD1*/
             ind_trans[itrans][3] = ind_HD2;   /*HD2*/
             ind_trans[itrans][4] = ind_HD3;   /*HD3*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CG2;   /*CG2*/
             ind_trans[itrans][2] = ind_HG21;  /*HG21*/
             ind_trans[itrans][3] = ind_HG22;  /*HG22*/
             ind_trans[itrans][4] = ind_HG23;  /*HG23*/
       }/*ILE*/

       if (res_THR == 1) {/*THR*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_CB;    /*CB*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CB;    /*CB*/
             ind_trans[itrans][2] = ind_OG1;   /*OG1*/
             ind_trans[itrans][3] = ind_CG2;   /*CG2*/
             ind_trans[itrans][4] = ind_HB;    /*HB*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_OG1;   /*OG1*/
             ind_trans[itrans][2] = ind_HG1;   /*HG1*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CG2;   /*CG2*/
             ind_trans[itrans][2] = ind_HG21;  /*HG21*/
             ind_trans[itrans][3] = ind_HG22;  /*HG22*/
             ind_trans[itrans][4] = ind_HG23;  /*HG23*/
       }/*THR*/

       if (res_VAL == 1) {/*VAL*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_CB;    /*CB*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CB;    /*CB*/
             ind_trans[itrans][2] = ind_CG1;   /*CG1*/
             ind_trans[itrans][3] = ind_CG2;   /*CG2*/
             ind_trans[itrans][4] = ind_HB;    /*HB*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CG1;   /*CG1*/
             ind_trans[itrans][2] = ind_HG11;  /*HG11*/
             ind_trans[itrans][3] = ind_HG12;  /*HG12*/
             ind_trans[itrans][4] = ind_HG13;  /*HG13*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CG2;   /*CG2*/
             ind_trans[itrans][2] = ind_HG21;  /*HG21*/
             ind_trans[itrans][3] = ind_HG22;  /*HG22*/
             ind_trans[itrans][4] = ind_HG23;  /*HG23*/
       }/*VAL*/

       if (res_GLN == 1) {/*GLN*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 2;
             res_id[itrans] = ires;
             natm_trans[itrans] = 5;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_CB;    /*CB*/
             ind_trans[itrans][3] = ind_CG;    /*CG*/
             ind_trans[itrans][4] = ind_CD;    /*CD*/
             ind_trans[itrans][5] = ind_NE2;   /*NE2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CB;    /*CB*/
             ind_trans[itrans][2] = ind_HB1;   /*HB1*/
             ind_trans[itrans][3] = ind_HB2;   /*HB2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CG;    /*CG*/
             ind_trans[itrans][2] = ind_HG1;   /*HG1*/
             ind_trans[itrans][3] = ind_HG2;   /*HG2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_CD;    /*CD*/
             ind_trans[itrans][2] = ind_OE1;   /*OE1*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_NE2;   /*NE2*/
             ind_trans[itrans][2] = ind_HE21;  /*HE21*/
             ind_trans[itrans][3] = ind_HE22;  /*HE22*/
       }/*GLN*/

       if (res_ASN == 1) {/*ASN*/
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 2;
             res_id[itrans] = ires;
             natm_trans[itrans] = 4;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_CB;    /*CB*/
             ind_trans[itrans][3] = ind_CG;    /*CG*/
             ind_trans[itrans][4] = ind_ND2;   /*ND2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CB;    /*CB*/
             ind_trans[itrans][2] = ind_HB1;   /*HB1*/
             ind_trans[itrans][3] = ind_HB2;   /*HB2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_CG;    /*CG*/
             ind_trans[itrans][2] = ind_OD1;   /*OD1*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_ND2;   /*ND2*/
             ind_trans[itrans][2] = ind_HD21;  /*HD21*/
             ind_trans[itrans][3] = ind_HD22;  /*HD22*/
       }/*ASN*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.B.1) Regular residues                                                 */

    if (res_REG==0) {continue;}

    itrans++;
    if (itrans>num_trans) {
       sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
       errorMessage(err_msg);
    }/*endif*/
    if (pos->residue[ires].nchigrp==1) {
        ityp_trans[itrans] = 1;
    } else {
        ityp_trans[itrans] = 2;
    }/*endif*/
    res_id[itrans] = ires;
    natm_trans[itrans]   = pos->residue[ires].nchigrp + 1;
    ind_trans[itrans][1] = ind_CA;     /*CA*/ 
    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
         iatm = 1 + ichigrp;
         ind_trans[itrans][iatm] = pos->residue[ires].chiprim_atom[ichigrp][3];
    }/*endif*/

    for (ichigrp=1;ichigrp<=pos->residue[ires].nchigrp;ichigrp++) {
      itrans++;
      if (itrans>num_trans) {
         sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
         errorMessage(err_msg);
      }/*endif*/
      ityp_trans[itrans] = 1;
      res_id[itrans] = ires;
      if (ichigrp==pos->residue[ires].nchigrp) {
          natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp] + 1; 
          ind_trans[itrans][1] = pos->residue[ires].chiprim_atom[ichigrp][3];
          for (iatm=1;iatm<=natm_trans[itrans];iatm++) {
               ind_trans[itrans][iatm+1] = 
                           pos->residue[ires].chigrp_atom[ichigrp][iatm];
          }/*endfor*/
      } else {
          natm_trans[itrans] = pos->residue[ires].natm_chigrp[ichigrp]; 
          ind_trans[itrans][1] = pos->residue[ires].chiprim_atom[ichigrp][3];
          for (iatm=2;iatm<=natm_trans[itrans];iatm++) {
               ind_trans[itrans][iatm] = 
                           pos->residue[ires].chigrp_atom[ichigrp][iatm];
          }/*endfor*/
      }/*end for*/
    }/*end for-ichigrp*/
    

  }/*end for-iresl*/
  
/*-------------------------------------------------------------------------*/
/* I.C) Define the groups out of all atoms                                 */

  nres    = nres_mol[imol];
  for (iresl=1;iresl<=nres;iresl++) {
       ires = ind_res_mol[imol][iresl];
       if (strcasecmp(pos->residue[ires].residue_name,"PRO")==0)
	    {res_PRO = 1;}
       else {res_PRO = 0;}
       if (strcasecmp(pos->residue[ires].residue_name,"GLY")==0)
	    {res_GLY = 1;}
       else {res_GLY = 0;}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.C.0) Find atom indices                                                */

       for (iatm=1;iatm<=pos->residue[ires].natom;iatm++) {
            iiatm     = pos->residue[ires].atom[iatm];
            atm_label = maps->upac_atm_label[iiatm];
            if (strcasecmp(atm_label,"HN")==0)  {ind_HN  = iiatm;}   
            if (strcasecmp(atm_label,"N")==0)   {ind_N   = iiatm;}   
            if (strcasecmp(atm_label,"CA")==0)  {ind_CA  = iiatm;}
            if (strcasecmp(atm_label,"HA")==0)  {ind_HA  = iiatm;}   
            if (strcasecmp(atm_label,"CB")==0)  {ind_CB  = iiatm;}
            if (strcasecmp(atm_label,"C")==0)   {ind_C   = iiatm;}
            if (strcasecmp(atm_label,"O")==0)   {ind_O   = iiatm;}
            if (strcasecmp(atm_label,"HA1")==0) {ind_HA1 = iiatm;}
            if (strcasecmp(atm_label,"HA2")==0) {ind_HA2 = iiatm;}
            if (strcasecmp(atm_label,"CD")==0)  {ind_CD  = iiatm;}
            if (strcasecmp(atm_label,"HD1")==0) {ind_HD1 = iiatm;}
            if (strcasecmp(atm_label,"HD2")==0) {ind_HD2 = iiatm;}
            if (strcasecmp(atm_label,"CG")==0)  {ind_CG  = iiatm;}
            if (strcasecmp(atm_label,"HG1")==0) {ind_HG1 = iiatm;}
            if (strcasecmp(atm_label,"HG2")==0) {ind_HG2 = iiatm;}
            if (strcasecmp(atm_label,"HB1")==0) {ind_HB1 = iiatm;}
            if (strcasecmp(atm_label,"HB2")==0) {ind_HB2 = iiatm;}
       }/*endfor*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* I.C.1) Define transformations in residue                                */
/*        participating atoms are either part of the backbone or           */
/*        directly connected to the backbone excluding any atoms           */ 
/*        where the main chain start branching                             */

         if (res_PRO == 1) {
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 5;
             ind_trans[itrans][1] = ind_N;     /*N*/
             ind_trans[itrans][2] = ind_CD;    /*CD*/
             ind_trans[itrans][3] = ind_CG;    /*CG*/
             ind_trans[itrans][4] = ind_CB;    /*CB*/
             ind_trans[itrans][5] = ind_CA;    /*CA*/
     
             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CD;     /*CD*/
             ind_trans[itrans][2] = ind_HD1;    /*HD1*/
             ind_trans[itrans][3] = ind_HD2;    /*HD2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CG;     /*CG*/
             ind_trans[itrans][2] = ind_HG1;    /*HG1*/
             ind_trans[itrans][3] = ind_HG2;    /*HG2*/

             itrans++;
             if (itrans>num_trans) {
              sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
              errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CB;     /*CB*/
             ind_trans[itrans][2] = ind_HB1;    /*HB1*/
             ind_trans[itrans][3] = ind_HB2;    /*HB2*/
         } else {
             itrans++;
             if (itrans>num_trans) {
     	     sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
                 errorMessage(err_msg);
             }/*endif*/
             ityp_trans[itrans] = 1;
             res_id[itrans] = ires;
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_N;    /*N*/
             ind_trans[itrans][2] = ind_HN;   /*HN*/
         }/*endif*/

         itrans++;
         if (itrans>num_trans) {
 	     sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         res_id[itrans] = ires;
         if (res_GLY == 1) {
             natm_trans[itrans] = 3;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_HA1;   /*HA1*/
             ind_trans[itrans][3] = ind_HA2;   /*HA2*/
         } else {
             natm_trans[itrans] = 2;
             ind_trans[itrans][1] = ind_CA;    /*CA*/
             ind_trans[itrans][2] = ind_HA;    /*HA*/
         }/*endif*/

         itrans++;
         if (itrans>num_trans) {
 	     sprintf(err_msg,"Stage_defs::setup_AA_protein:itrans>num_trans");
             errorMessage(err_msg);
         }/*endif*/
         ityp_trans[itrans] = 1;
         res_id[itrans] = ires;
         natm_trans[itrans] = 2;
         ind_trans[itrans][1] = ind_C;    /*C*/
         ind_trans[itrans][2] = ind_O;    /*O*/

  }/*endfor-iresl*/

/*-------------------------------------------------------------------------*/
/* I.D) Adjust itrans_glob                                                 */  
   *itrans_glob  =  itrans;

/*-------------------------------------------------------------------------*/
}/*end routine*/
/*=========================================================================*/
